Package: tibble
Title: Simple Data Frames
Version: 3.3.1
Authors@R: c(
    person("Kirill", "Müller", , "kirill@cynkra.com", role = c("aut", "cre"),
           comment = c(ORCID = "0000-0002-1416-3412")),
    person("Hadley", "Wickham", , "hadley@rstudio.com", role = "aut"),
    person("Romain", "Francois", , "romain@r-enthusiasts.com", role = "ctb"),
    person("Jennifer", "Bryan", , "jenny@rstudio.com", role = "ctb"),
    person("Posit Software, PBC", role = c("cph", "fnd"),
           comment = c(ROR = "03wc8by49"))
  )
Description: Provides a 'tbl_df' class (the 'tibble') with stricter
    checking and better formatting than the traditional data frame.
License: MIT + file LICENSE
URL: https://tibble.tidyverse.org/, https://github.com/tidyverse/tibble
BugReports: https://github.com/tidyverse/tibble/issues
Depends: R (>= 3.4.0)
Imports: cli, lifecycle (>= 1.0.0), magrittr, methods, pillar (>=
        1.8.1), pkgconfig, rlang (>= 1.0.2), utils, vctrs (>= 0.5.0)
Suggests: bench, bit64, blob, brio, callr, DiagrammeR, dplyr, evaluate,
        formattable, ggplot2, here, hms, htmltools, knitr, lubridate,
        nycflights13, pkgload, purrr, rmarkdown, stringi, testthat (>=
        3.0.2), tidyr, withr
VignetteBuilder: knitr
Config/autostyle/rmd: false
Config/autostyle/scope: line_breaks
Config/autostyle/strict: true
Config/Needs/website: tidyverse/tidytemplate
Config/testthat/edition: 3
Config/testthat/parallel: true
Config/testthat/start-first: vignette-formats, as_tibble, add,
        invariants
Config/usethis/last-upkeep: 2025-06-07
Encoding: UTF-8
RoxygenNote: 7.3.3.9000
NeedsCompilation: yes
Packaged: 2026-01-10 18:28:47 UTC; kirill
Author: Kirill Müller [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-1416-3412>),
  Hadley Wickham [aut],
  Romain Francois [ctb],
  Jennifer Bryan [ctb],
  Posit Software, PBC [cph, fnd] (ROR: <https://ror.org/03wc8by49>)
Maintainer: Kirill Müller <kirill@cynkra.com>
Repository: CRAN
Date/Publication: 2026-01-11 10:10:02 UTC
Built: R 4.6.0; aarch64-apple-darwin23; 2026-04-26 05:14:40 UTC; unix
Archs: tibble.so.dSYM

Package: tibble
Title: Simple Data Frames
Version: 3.1.7
Authors@R: 
    c(person(given = "Kirill",
             family = "M\u00fcller",
             role = c("aut", "cre"),
             email = "krlmlr+r@mailbox.org"),
      person(given = "Hadley",
             family = "Wickham",
             role = "aut",
             email = "hadley@rstudio.com"),
      person(given = "Romain",
             family = "Francois",
             role = "ctb",
             email = "romain@r-enthusiasts.com"),
      person(given = "Jennifer",
             family = "Bryan",
             role = "ctb",
             email = "jenny@rstudio.com"),
      person(given = "RStudio",
             role = c("cph", "fnd")))
Description: Provides a 'tbl_df' class (the 'tibble') with stricter checking and better formatting than the traditional
    data frame.
License: MIT + file LICENSE
URL: https://tibble.tidyverse.org/, https://github.com/tidyverse/tibble
BugReports: https://github.com/tidyverse/tibble/issues
Depends: R (>= 3.1.0)
Imports: ellipsis (>= 0.3.2), fansi (>= 0.4.0), lifecycle (>= 1.0.0),
        magrittr, methods, pillar (>= 1.7.0), pkgconfig, rlang (>=
        1.0.1), utils, vctrs (>= 0.3.8)
Suggests: bench, bit64, blob, brio, callr, cli, covr, crayon (>=
        1.3.4), DiagrammeR, dplyr, evaluate, formattable, ggplot2, hms,
        htmltools, knitr, lubridate, mockr, nycflights13, pkgbuild,
        pkgload, purrr, rmarkdown, stringi, testthat (>= 3.0.2), tidyr,
        withr
VignetteBuilder: knitr
Encoding: UTF-8
RoxygenNote: 7.1.2
Config/testthat/edition: 3
Config/testthat/parallel: true
Config/testthat/start-first: subsetting, as_tibble, add, invariants
Config/autostyle/scope: line_breaks
Config/autostyle/strict: true
Config/Needs/website: tidyverse/tidytemplate
NeedsCompilation: yes
Packaged: 2022-04-26 14:15:54 UTC; kirill
Author: Kirill Müller [aut, cre],
  Hadley Wickham [aut],
  Romain Francois [ctb],
  Jennifer Bryan [ctb],
  RStudio [cph, fnd]
Maintainer: Kirill Müller <krlmlr+r@mailbox.org>
Repository: CRAN
Date/Publication: 2022-05-03 07:00:06 UTC

Package: tibble
Title: Simple Data Frames
Version: 3.3.1
Authors@R: c(
    person("Kirill", "Müller", , "kirill@cynkra.com", role = c("aut", "cre"),
           comment = c(ORCID = "0000-0002-1416-3412")),
    person("Hadley", "Wickham", , "hadley@rstudio.com", role = "aut"),
    person("Romain", "Francois", , "romain@r-enthusiasts.com", role = "ctb"),
    person("Jennifer", "Bryan", , "jenny@rstudio.com", role = "ctb"),
    person("Posit Software, PBC", role = c("cph", "fnd"),
           comment = c(ROR = "03wc8by49"))
  )
Description: Provides a 'tbl_df' class (the 'tibble') with stricter
    checking and better formatting than the traditional data frame.
License: MIT + file LICENSE
URL: https://tibble.tidyverse.org/, https://github.com/tidyverse/tibble
BugReports: https://github.com/tidyverse/tibble/issues
Depends: R (>= 3.4.0)
Imports: cli, lifecycle (>= 1.0.0), magrittr, methods, pillar (>=
        1.8.1), pkgconfig, rlang (>= 1.0.2), utils, vctrs (>= 0.5.0)
Suggests: bench, bit64, blob, brio, callr, DiagrammeR, dplyr, evaluate,
        formattable, ggplot2, here, hms, htmltools, knitr, lubridate,
        nycflights13, pkgload, purrr, rmarkdown, stringi, testthat (>=
        3.0.2), tidyr, withr
VignetteBuilder: knitr
Config/autostyle/rmd: false
Config/autostyle/scope: line_breaks
Config/autostyle/strict: true
Config/Needs/website: tidyverse/tidytemplate
Config/testthat/edition: 3
Config/testthat/parallel: true
Config/testthat/start-first: vignette-formats, as_tibble, add,
        invariants
Config/usethis/last-upkeep: 2025-06-07
Encoding: UTF-8
RoxygenNote: 7.3.3.9000
NeedsCompilation: yes
Packaged: 2026-01-10 18:28:47 UTC; kirill
Author: Kirill Müller [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-1416-3412>),
  Hadley Wickham [aut],
  Romain Francois [ctb],
  Jennifer Bryan [ctb],
  Posit Software, PBC [cph, fnd] (ROR: <https://ror.org/03wc8by49>)
Maintainer: Kirill Müller <kirill@cynkra.com>
Repository: CRAN
Date/Publication: 2026-01-11 10:10:02 UTC

# Generated by roxygen2: do not edit by hand

S3method("$",tbl_df)
S3method("$<-",tbl_df)
S3method("[",tbl_df)
S3method("[<-",tbl_df)
S3method("[[",tbl_df)
S3method("[[<-",tbl_df)
S3method("names<-",tbl_df)
S3method("row.names<-",tbl_df)
S3method(as.data.frame,tbl_df)
S3method(as_tibble,"NULL")
S3method(as_tibble,data.frame)
S3method(as_tibble,default)
S3method(as_tibble,list)
S3method(as_tibble,matrix)
S3method(as_tibble,poly)
S3method(as_tibble,table)
S3method(as_tibble,ts)
S3method(format,trunc_mat)
S3method(print,trunc_mat)
S3method(quote_n,character)
S3method(quote_n,default)
S3method(str,tbl_df)
S3method(tbl_sum,tbl_df)
S3method(vec_ptype_abbr,tbl_df)
export("%>%")
export(add_case)
export(add_column)
export(add_row)
export(as.tibble)
export(as_data_frame)
export(as_tibble)
export(as_tibble_col)
export(as_tibble_row)
export(char)
export(column_to_rownames)
export(data_frame)
export(data_frame_)
export(deframe)
export(enframe)
export(frame_data)
export(frame_matrix)
export(glimpse)
export(has_name)
export(has_rownames)
export(is.tibble)
export(is_tibble)
export(lst)
export(lst_)
export(new_tibble)
export(num)
export(obj_sum)
export(remove_rownames)
export(repair_names)
export(rowid_to_column)
export(rownames_to_column)
export(set_char_opts)
export(set_num_opts)
export(set_tidy_names)
export(size_sum)
export(tbl_sum)
export(tibble)
export(tibble_)
export(tibble_row)
export(tidy_names)
export(tribble)
export(trunc_mat)
export(type_sum)
export(validate_tibble)
export(view)
exportClasses(tbl_df)
import(rlang)
importFrom(lifecycle,deprecate_soft)
importFrom(lifecycle,deprecate_stop)
importFrom(lifecycle,deprecate_warn)
importFrom(lifecycle,expect_deprecated)
importFrom(magrittr,"%>%")
importFrom(methods,setOldClass)
importFrom(pillar,dim_desc)
importFrom(pillar,glimpse)
importFrom(pillar,obj_sum)
importFrom(pillar,size_sum)
importFrom(pillar,style_subtle)
importFrom(pillar,tbl_sum)
importFrom(pillar,type_sum)
importFrom(pkgconfig,set_config)
importFrom(utils,head)
importFrom(utils,tail)
importFrom(vctrs,new_data_frame)
importFrom(vctrs,new_rcrd)
importFrom(vctrs,num_as_location)
importFrom(vctrs,unspecified)
importFrom(vctrs,vec_as_location)
importFrom(vctrs,vec_as_location2)
importFrom(vctrs,vec_as_names)
importFrom(vctrs,vec_as_names_legacy)
importFrom(vctrs,vec_as_subscript)
importFrom(vctrs,vec_as_subscript2)
importFrom(vctrs,vec_assign)
importFrom(vctrs,vec_c)
importFrom(vctrs,vec_is)
importFrom(vctrs,vec_names)
importFrom(vctrs,vec_names2)
importFrom(vctrs,vec_ptype_abbr)
importFrom(vctrs,vec_rbind)
importFrom(vctrs,vec_recycle)
importFrom(vctrs,vec_set_names)
importFrom(vctrs,vec_size)
importFrom(vctrs,vec_slice)
useDynLib(tibble, .registration = TRUE)

<!-- README.md is generated from README.Rmd. Please edit that file -->

# tibble <img src="man/figures/logo.png" align="right" />

<!-- badges: start -->

[![R-CMD-check](https://github.com/tidyverse/tibble/workflows/R-CMD-check/badge.svg)](https://github.com/tidyverse/tibble/actions) [![Codecov test coverage](https://codecov.io/gh/tidyverse/tibble/branch/main/graph/badge.svg)](https://app.codecov.io/gh/tidyverse/tibble?branch=main) [![CRAN\_Status\_Badge](https://www.r-pkg.org/badges/version/tibble)](https://cran.r-project.org/package=tibble) [![Life cycle](https://img.shields.io/badge/lifecycle-stable-brightgreen.svg)](https://lifecycle.r-lib.org/articles/stages.html)

<!-- badges: end -->

## Overview

A **tibble**, or `tbl_df`, is a modern reimagining of the data.frame, keeping what time has proven to be effective, and throwing out what is not. Tibbles are data.frames that are lazy and surly: they do less (i.e. they don’t change variable names or types, and don’t do partial matching) and complain more (e.g. when a variable does not exist). This forces you to confront problems earlier, typically leading to cleaner, more expressive code. Tibbles also have an enhanced [`print()`](https://rdrr.io/r/base/print.html) method which makes them easier to use with large datasets containing complex objects.

If you are new to tibbles, the best place to start is the [tibbles chapter](https://r4ds.had.co.nz/tibbles.html) in *R for data science*.

## Installation

<pre class='chroma'>
<span class='c'># The easiest way to get tibble is to install the whole tidyverse:</span>
<span class='nf'><a href='https://rdrr.io/r/utils/install.packages.html'>install.packages</a></span><span class='o'>(</span><span class='s'>"tidyverse"</span><span class='o'>)</span>

<span class='c'># Alternatively, install just tibble:</span>
<span class='nf'><a href='https://rdrr.io/r/utils/install.packages.html'>install.packages</a></span><span class='o'>(</span><span class='s'>"tibble"</span><span class='o'>)</span>

<span class='c'># Or the the development version from GitHub:</span>
<span class='c'># install.packages("devtools")</span>
<span class='nf'>devtools</span><span class='nf'>::</span><span class='nf'><a href='https://devtools.r-lib.org/reference/remote-reexports.html'>install_github</a></span><span class='o'>(</span><span class='s'>"tidyverse/tibble"</span><span class='o'>)</span></pre>

## Usage

<pre class='chroma'>
<span class='kr'><a href='https://rdrr.io/r/base/library.html'>library</a></span><span class='o'>(</span><span class='nv'><a href='https://tibble.tidyverse.org/'>tibble</a></span><span class='o'>)</span></pre>

Create a tibble from an existing object with [`as_tibble()`](https://tibble.tidyverse.org/reference/as_tibble.html):

<pre class='chroma'>
<span class='nv'>data</span> <span class='o'>&lt;-</span> <span class='nf'><a href='https://rdrr.io/r/base/data.frame.html'>data.frame</a></span><span class='o'>(</span>a <span class='o'>=</span> <span class='m'>1</span><span class='o'>:</span><span class='m'>3</span>, b <span class='o'>=</span> <span class='nv'>letters</span><span class='o'>[</span><span class='m'>1</span><span class='o'>:</span><span class='m'>3</span><span class='o'>]</span>, c <span class='o'>=</span> <span class='nf'><a href='https://rdrr.io/r/base/Sys.time.html'>Sys.Date</a></span><span class='o'>(</span><span class='o'>)</span> <span class='o'>-</span> <span class='m'>1</span><span class='o'>:</span><span class='m'>3</span><span class='o'>)</span>
<span class='nv'>data</span>
<span class='c'>#&gt;   a b          c</span>
<span class='c'>#&gt; 1 1 a 2022-03-30</span>
<span class='c'>#&gt; 2 2 b 2022-03-29</span>
<span class='c'>#&gt; 3 3 c 2022-03-28</span>

<span class='nf'><a href='https://tibble.tidyverse.org/reference/as_tibble.html'>as_tibble</a></span><span class='o'>(</span><span class='nv'>data</span><span class='o'>)</span>
<span class='c'>#&gt; <span style='color: #555555;'># A tibble: 3 × 3</span></span>
<span class='c'>#&gt;       <span style='font-weight: bold;'>a</span> <span style='font-weight: bold;'>b</span>     <span style='font-weight: bold;'>c</span>         </span>
<span class='c'>#&gt;   <span style='color: #555555; font-style: italic;'>&lt;int&gt;</span> <span style='color: #555555; font-style: italic;'>&lt;chr&gt;</span> <span style='color: #555555; font-style: italic;'>&lt;date&gt;</span>    </span>
<span class='c'>#&gt; <span style='color: #555555;'>1</span>     1 a     2022-03-30</span>
<span class='c'>#&gt; <span style='color: #555555;'>2</span>     2 b     2022-03-29</span>
<span class='c'>#&gt; <span style='color: #555555;'>3</span>     3 c     2022-03-28</span></pre>

This will work for reasonable inputs that are already data.frames, lists, matrices, or tables.

You can also create a new tibble from column vectors with [`tibble()`](https://tibble.tidyverse.org/reference/tibble.html):

<pre class='chroma'>
<span class='nf'><a href='https://tibble.tidyverse.org/reference/tibble.html'>tibble</a></span><span class='o'>(</span>x <span class='o'>=</span> <span class='m'>1</span><span class='o'>:</span><span class='m'>5</span>, y <span class='o'>=</span> <span class='m'>1</span>, z <span class='o'>=</span> <span class='nv'>x</span> <span class='o'>^</span> <span class='m'>2</span> <span class='o'>+</span> <span class='nv'>y</span><span class='o'>)</span>
<span class='c'>#&gt; <span style='color: #555555;'># A tibble: 5 × 3</span></span>
<span class='c'>#&gt;       <span style='font-weight: bold;'>x</span>     <span style='font-weight: bold;'>y</span>     <span style='font-weight: bold;'>z</span></span>
<span class='c'>#&gt;   <span style='color: #555555; font-style: italic;'>&lt;int&gt;</span> <span style='color: #555555; font-style: italic;'>&lt;dbl&gt;</span> <span style='color: #555555; font-style: italic;'>&lt;dbl&gt;</span></span>
<span class='c'>#&gt; <span style='color: #555555;'>1</span>     1     1     2</span>
<span class='c'>#&gt; <span style='color: #555555;'>2</span>     2     1     5</span>
<span class='c'>#&gt; <span style='color: #555555;'>3</span>     3     1    10</span>
<span class='c'>#&gt; <span style='color: #555555;'>4</span>     4     1    17</span>
<span class='c'>#&gt; <span style='color: #555555;'>5</span>     5     1    26</span></pre>

[`tibble()`](https://tibble.tidyverse.org/reference/tibble.html) does much less than [`data.frame()`](https://rdrr.io/r/base/data.frame.html): it never changes the type of the inputs (e.g. it never converts strings to factors!), it never changes the names of variables, it only recycles inputs of length 1, and it never creates [`row.names()`](https://rdrr.io/r/base/row.names.html). You can read more about these features in [`vignette("tibble")`](https://tibble.tidyverse.org/articles/tibble.html).

You can define a tibble row-by-row with [`tribble()`](https://tibble.tidyverse.org/reference/tribble.html):

<pre class='chroma'>
<span class='nf'><a href='https://tibble.tidyverse.org/reference/tribble.html'>tribble</a></span><span class='o'>(</span>
  <span class='o'>~</span><span class='nv'>x</span>, <span class='o'>~</span><span class='nv'>y</span>,  <span class='o'>~</span><span class='nv'>z</span>,
  <span class='s'>"a"</span>, <span class='m'>2</span>,  <span class='m'>3.6</span>,
  <span class='s'>"b"</span>, <span class='m'>1</span>,  <span class='m'>8.5</span>
<span class='o'>)</span>
<span class='c'>#&gt; <span style='color: #555555;'># A tibble: 2 × 3</span></span>
<span class='c'>#&gt;   <span style='font-weight: bold;'>x</span>         <span style='font-weight: bold;'>y</span>     <span style='font-weight: bold;'>z</span></span>
<span class='c'>#&gt;   <span style='color: #555555; font-style: italic;'>&lt;chr&gt;</span> <span style='color: #555555; font-style: italic;'>&lt;dbl&gt;</span> <span style='color: #555555; font-style: italic;'>&lt;dbl&gt;</span></span>
<span class='c'>#&gt; <span style='color: #555555;'>1</span> a         2   3.6</span>
<span class='c'>#&gt; <span style='color: #555555;'>2</span> b         1   8.5</span></pre>

## Related work

The tibble print method draws inspiration from [data.table](https://rdatatable.gitlab.io/data.table), and [frame](https://github.com/patperry/r-frame). Like [`data.table::data.table()`](https://Rdatatable.gitlab.io/data.table/reference/data.table.html), [`tibble()`](https://tibble.tidyverse.org/reference/tibble.html) doesn’t coerce strings to factors by default, doesn’t change column names, and doesn’t use rownames.

------------------------------------------------------------------------

## Code of Conduct

Please note that the tibble project is released with a [Contributor Code of Conduct](https://tibble.tidyverse.org/CODE_OF_CONDUCT.html). By contributing to this project, you agree to abide by its terms.

<!-- README.md is generated from README.Rmd. Please edit that file -->

# tibble <img src="man/figures/logo.png" align="right" alt="Hexagonal logo for the R package ‘tibble’, styled with a sci-fi theme. The word ‘TIBBLE’ appears at the top in a futuristic font, and below it is a stylized table with colored bars resembling columns and rows, set against a starry space background." />

<!-- badges: start -->

[![R-CMD-check](https://github.com/tidyverse/tibble/workflows/rcc/badge.svg)](https://github.com/tidyverse/tibble/actions)
[![Codecov test
coverage](https://codecov.io/gh/tidyverse/tibble/branch/main/graph/badge.svg)](https://app.codecov.io/gh/tidyverse/tibble?branch=main)
[![CRAN_Status_Badge](https://www.r-pkg.org/badges/version/tibble)](https://cran.r-project.org/package=tibble)
[![Life
cycle](https://img.shields.io/badge/lifecycle-stable-brightgreen.svg)](https://lifecycle.r-lib.org/articles/stages.html)
<!-- badges: end -->

## Overview

A **tibble**, or `tbl_df`, is a modern reimagining of the data.frame,
keeping what time has proven to be effective, and throwing out what is
not. Tibbles are data.frames that are lazy and surly: they do less
(i.e. they don’t change variable names or types, and don’t do partial
matching) and complain more (e.g. when a variable does not exist). This
forces you to confront problems earlier, typically leading to cleaner,
more expressive code. Tibbles also have an enhanced `print()` method
which makes them easier to use with large datasets containing complex
objects.

If you are new to tibbles, the best place to start is the [tibbles
chapter](https://r4ds.had.co.nz/tibbles.html) in *R for data science*.

## Installation

``` r
# The easiest way to get tibble is to install the whole tidyverse:
install.packages("tidyverse")

# Alternatively, install just tibble:
install.packages("tibble")

# Or the the development version from GitHub:
# install.packages("pak")
pak::pak("tidyverse/tibble")
```

## Usage

``` r
library(tibble)
```

Create a tibble from an existing object with `as_tibble()`:

``` r
data <- data.frame(a = 1:3, b = letters[1:3], c = Sys.Date() - 1:3)
data
#>   a b          c
#> 1 1 a 2025-06-18
#> 2 2 b 2025-06-17
#> 3 3 c 2025-06-16

as_tibble(data)
#> # A tibble: 3 × 3
#>       a b     c         
#>   <int> <chr> <date>    
#> 1     1 a     2025-06-18
#> 2     2 b     2025-06-17
#> 3     3 c     2025-06-16
```

This will work for reasonable inputs that are already data.frames,
lists, matrices, or tables.

You can also create a new tibble from column vectors with `tibble()`:

``` r
tibble(x = 1:5, y = 1, z = x^2 + y)
#> # A tibble: 5 × 3
#>       x     y     z
#>   <int> <dbl> <dbl>
#> 1     1     1     2
#> 2     2     1     5
#> 3     3     1    10
#> 4     4     1    17
#> 5     5     1    26
```

`tibble()` does much less than `data.frame()`: it never changes the type
of the inputs (e.g. it keeps list columns as is), it never changes the
names of variables, it only recycles inputs of length 1, and it never
creates `row.names()`. You can read more about these features in
[`vignette("tibble")`](https://tibble.tidyverse.org/articles/tibble.html).

You can define a tibble row-by-row with `tribble()`:

``` r
tribble(
  ~x, ~y,  ~z,
  "a", 2,  3.6,
  "b", 1,  8.5
)
#> # A tibble: 2 × 3
#>   x         y     z
#>   <chr> <dbl> <dbl>
#> 1 a         2   3.6
#> 2 b         1   8.5
```

## Related work

The tibble print method draws inspiration from
[data.table](https://rdatatable.gitlab.io/data.table), and
[frame](https://github.com/patperry/r-frame). Like
`data.table::data.table()`, `tibble()` doesn’t change column names and
doesn’t use rownames.

------------------------------------------------------------------------

## Code of Conduct

Please note that the tibble project is released with a [Contributor Code
of Conduct](https://tibble.tidyverse.org/CODE_OF_CONDUCT.html). By
contributing to this project, you agree to abide by its terms.

<!-- README.md is generated from README.Rmd. Please edit that file -->

# tibble <img src="man/figures/logo.png" align="right" />

<!-- badges: start -->

[![R-CMD-check](https://github.com/tidyverse/tibble/workflows/R-CMD-check/badge.svg)](https://github.com/tidyverse/tibble/actions) [![Codecov test coverage](https://codecov.io/gh/tidyverse/tibble/branch/main/graph/badge.svg)](https://app.codecov.io/gh/tidyverse/tibble?branch=main) [![CRAN_Status_Badge](https://www.r-pkg.org/badges/version/tibble)](https://cran.r-project.org/package=tibble) [![Life cycle](https://img.shields.io/badge/lifecycle-stable-brightgreen.svg)](https://lifecycle.r-lib.org/articles/stages.html)

<!-- badges: end -->

## Overview

A **tibble**, or `tbl_df`, is a modern reimagining of the data.frame, keeping what time has proven to be effective, and throwing out what is not. Tibbles are data.frames that are lazy and surly: they do less (i.e. they don’t change variable names or types, and don’t do partial matching) and complain more (e.g. when a variable does not exist). This forces you to confront problems earlier, typically leading to cleaner, more expressive code. Tibbles also have an enhanced [`print()`](https://rdrr.io/r/base/print.html) method which makes them easier to use with large datasets containing complex objects.

If you are new to tibbles, the best place to start is the [tibbles chapter](https://r4ds.had.co.nz/tibbles.html) in *R for data science*.

## Installation

<pre class='chroma'>
<span><span class='c'># The easiest way to get tibble is to install the whole tidyverse:</span></span>
<span><span class='nf'><a href='https://rdrr.io/r/utils/install.packages.html'>install.packages</a></span><span class='o'>(</span><span class='s'>"tidyverse"</span><span class='o'>)</span></span>
<span></span>
<span><span class='c'># Alternatively, install just tibble:</span></span>
<span><span class='nf'><a href='https://rdrr.io/r/utils/install.packages.html'>install.packages</a></span><span class='o'>(</span><span class='s'>"tibble"</span><span class='o'>)</span></span>
<span></span>
<span><span class='c'># Or the the development version from GitHub:</span></span>
<span><span class='c'># install.packages("devtools")</span></span>
<span><span class='nf'>devtools</span><span class='nf'>::</span><span class='nf'><a href='https://remotes.r-lib.org/reference/install_github.html'>install_github</a></span><span class='o'>(</span><span class='s'>"tidyverse/tibble"</span><span class='o'>)</span></span></pre>

## Usage

<pre class='chroma'>
<span><span class='kr'><a href='https://rdrr.io/r/base/library.html'>library</a></span><span class='o'>(</span><span class='nv'><a href='https://tibble.tidyverse.org/'>tibble</a></span><span class='o'>)</span></span></pre>

Create a tibble from an existing object with [`as_tibble()`](https://tibble.tidyverse.org/reference/as_tibble.html):

<pre class='chroma'>
<span><span class='nv'>data</span> <span class='o'>&lt;-</span> <span class='nf'><a href='https://rdrr.io/r/base/data.frame.html'>data.frame</a></span><span class='o'>(</span>a <span class='o'>=</span> <span class='m'>1</span><span class='o'>:</span><span class='m'>3</span>, b <span class='o'>=</span> <span class='nv'>letters</span><span class='o'>[</span><span class='m'>1</span><span class='o'>:</span><span class='m'>3</span><span class='o'>]</span>, c <span class='o'>=</span> <span class='nf'><a href='https://rdrr.io/r/base/Sys.time.html'>Sys.Date</a></span><span class='o'>(</span><span class='o'>)</span> <span class='o'>-</span> <span class='m'>1</span><span class='o'>:</span><span class='m'>3</span><span class='o'>)</span></span>
<span><span class='nv'>data</span></span>
<span><span class='c'>#&gt;   a b          c</span></span>
<span><span class='c'>#&gt; 1 1 a 2023-02-21</span></span>
<span><span class='c'>#&gt; 2 2 b 2023-02-20</span></span>
<span><span class='c'>#&gt; 3 3 c 2023-02-19</span></span>
<span></span>
<span><span class='nf'><a href='https://tibble.tidyverse.org/reference/as_tibble.html'>as_tibble</a></span><span class='o'>(</span><span class='nv'>data</span><span class='o'>)</span></span>
<span><span class='c'>#&gt; <span style='color: #555555;'># A tibble: 3 × 3</span></span></span>
<span><span class='c'>#&gt;       <span style='font-weight: bold;'>a</span> <span style='font-weight: bold;'>b</span>     <span style='font-weight: bold;'>c</span>         </span></span>
<span><span class='c'>#&gt;   <span style='color: #555555; font-style: italic;'>&lt;int&gt;</span> <span style='color: #555555; font-style: italic;'>&lt;chr&gt;</span> <span style='color: #555555; font-style: italic;'>&lt;date&gt;</span>    </span></span>
<span><span class='c'>#&gt; <span style='color: #555555;'>1</span>     1 a     2023-02-21</span></span>
<span><span class='c'>#&gt; <span style='color: #555555;'>2</span>     2 b     2023-02-20</span></span>
<span><span class='c'>#&gt; <span style='color: #555555;'>3</span>     3 c     2023-02-19</span></span></pre>

This will work for reasonable inputs that are already data.frames, lists, matrices, or tables.

You can also create a new tibble from column vectors with [`tibble()`](https://tibble.tidyverse.org/reference/tibble.html):

<pre class='chroma'>
<span><span class='nf'><a href='https://tibble.tidyverse.org/reference/tibble.html'>tibble</a></span><span class='o'>(</span>x <span class='o'>=</span> <span class='m'>1</span><span class='o'>:</span><span class='m'>5</span>, y <span class='o'>=</span> <span class='m'>1</span>, z <span class='o'>=</span> <span class='nv'>x</span><span class='o'>^</span><span class='m'>2</span> <span class='o'>+</span> <span class='nv'>y</span><span class='o'>)</span></span>
<span><span class='c'>#&gt; <span style='color: #555555;'># A tibble: 5 × 3</span></span></span>
<span><span class='c'>#&gt;       <span style='font-weight: bold;'>x</span>     <span style='font-weight: bold;'>y</span>     <span style='font-weight: bold;'>z</span></span></span>
<span><span class='c'>#&gt;   <span style='color: #555555; font-style: italic;'>&lt;int&gt;</span> <span style='color: #555555; font-style: italic;'>&lt;dbl&gt;</span> <span style='color: #555555; font-style: italic;'>&lt;dbl&gt;</span></span></span>
<span><span class='c'>#&gt; <span style='color: #555555;'>1</span>     1     1     2</span></span>
<span><span class='c'>#&gt; <span style='color: #555555;'>2</span>     2     1     5</span></span>
<span><span class='c'>#&gt; <span style='color: #555555;'>3</span>     3     1    10</span></span>
<span><span class='c'>#&gt; <span style='color: #555555;'>4</span>     4     1    17</span></span>
<span><span class='c'>#&gt; <span style='color: #555555;'>5</span>     5     1    26</span></span></pre>

[`tibble()`](https://tibble.tidyverse.org/reference/tibble.html) does much less than [`data.frame()`](https://rdrr.io/r/base/data.frame.html): it never changes the type of the inputs (e.g. it keeps list columns unchanged, and never converts strings to factors!), it never changes the names of variables, it only recycles inputs of length 1, and it never creates [`row.names()`](https://rdrr.io/r/base/row.names.html). You can read more about these features in `vignette("tibble")`.

You can define a tibble row-by-row with [`tribble()`](https://tibble.tidyverse.org/reference/tribble.html):

<pre class='chroma'>
<span><span class='nf'><a href='https://tibble.tidyverse.org/reference/tribble.html'>tribble</a></span><span class='o'>(</span></span>
<span>  <span class='o'>~</span><span class='nv'>x</span>, <span class='o'>~</span><span class='nv'>y</span>,  <span class='o'>~</span><span class='nv'>z</span>,</span>
<span>  <span class='s'>"a"</span>, <span class='m'>2</span>,  <span class='m'>3.6</span>,</span>
<span>  <span class='s'>"b"</span>, <span class='m'>1</span>,  <span class='m'>8.5</span></span>
<span><span class='o'>)</span></span>
<span><span class='c'>#&gt; <span style='color: #555555;'># A tibble: 2 × 3</span></span></span>
<span><span class='c'>#&gt;   <span style='font-weight: bold;'>x</span>         <span style='font-weight: bold;'>y</span>     <span style='font-weight: bold;'>z</span></span></span>
<span><span class='c'>#&gt;   <span style='color: #555555; font-style: italic;'>&lt;chr&gt;</span> <span style='color: #555555; font-style: italic;'>&lt;dbl&gt;</span> <span style='color: #555555; font-style: italic;'>&lt;dbl&gt;</span></span></span>
<span><span class='c'>#&gt; <span style='color: #555555;'>1</span> a         2   3.6</span></span>
<span><span class='c'>#&gt; <span style='color: #555555;'>2</span> b         1   8.5</span></span></pre>

## Related work

The tibble print method draws inspiration from [data.table](https://rdatatable.gitlab.io/data.table), and [frame](https://github.com/patperry/r-frame). Like [`data.table::data.table()`](https://Rdatatable.gitlab.io/data.table/reference/data.table.html), [`tibble()`](https://tibble.tidyverse.org/reference/tibble.html) doesn’t change column names and doesn’t use rownames.

------------------------------------------------------------------------

## Code of Conduct

Please note that the tibble project is released with a [Contributor Code of Conduct](https://tibble.tidyverse.org/CODE_OF_CONDUCT.html). By contributing to this project, you agree to abide by its terms.

## -----------------------------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  eval = rlang::is_installed(c("dplyr", "tidyr", "purrr")),
  comment = "#>"
)

## -----------------------------------------------------------------------------
library(dplyr)
library(tidyr)
library(purrr)
requireNamespace("hms", quietly = TRUE)

## -----------------------------------------------------------------------------
library(tibble)

## -----------------------------------------------------------------------------
# library(tidyverse)
# library(tidymodels)
# library(vctrs)
# library(pillar)
# 
# all_methods <-
#   c("vec_ptype_abbr", "vec_ptype_full", "type_sum") %>%
#   map(methods) %>%
#   map(as.character) %>%
#   map(~ gsub("^.*[.]", "", .x)) %>%
#   unlist() %>%
#   unique()
# 
# set_names(rep_along(all_methods, list("")), all_methods) %>%
#   dput()

## -----------------------------------------------------------------------------
data <- compact(list(
  "Atomic" = rlang::quos(
    logical = TRUE,
    integer = 1L,
    double = 1.5,
    character = "A",
    complex = 1i,
    raw = as.raw(1),
    list = list(1),
    "named list" = list(a = 1)
  ),

  "Built-in objects" = rlang::quos(
    factor = factor("A"),
    ordered = ordered("a"),
    Date = Sys.Date(),
    POSIXt = Sys.time(),
    difftime = vctrs::new_duration(1)
  ),

  "Objects from other packages" = if (rlang::is_installed(c("bit64", "blob", "hms"))) rlang::quos(
    hms = hms::hms(1),
    integer64 = bit64::as.integer64(1e10),
    blob = blob::blob(raw(1))
  ),

  "Data frames" = rlang::quos(
    data.frame = data.frame(a = 1),
    tbl_df = tibble(a = 1)
  ),

  "Unchanged" = rlang::quos(
    AsIs = I(1L)
  ),

  "vctrs types" = rlang::quos(
    unspecified = vctrs::unspecified(1),

    vctrs_list_of = vctrs::list_of(c(1L)),
    vctrs_vctr = vctrs::new_vctr(1L)
  ),

  "Language objects" = rlang::quos(
    "function" = function() NULL,
    symbol = quote(a),
    expression = parse(text = "a <- 1\nb<- 2"),
    quosures = rlang::quos(a = 1)
  )
))

## -----------------------------------------------------------------------------
tbl <-
  data %>%
  map(unclass) %>%
  map(enframe, "Data type", "Expression") %>%
  enframe("Class", "data") %>%
  unnest(data) %>%
  mutate(Example = map_chr(Expression, rlang::as_label)) %>%
  mutate(Value = map(Expression, rlang::eval_tidy)) %>%
  select(-Expression) %>%
  mutate(Class = if_else(Class == lag(Class, default = ""), "", Class)) %>%
  mutate("Column header" = map_chr(Value, type_sum))

## -----------------------------------------------------------------------------
tbl %>%
  select(-Value) %>%
  mutate(Example = paste0("`", Example, "`")) %>%
  knitr::kable(escape = FALSE)

## -----------------------------------------------------------------------------
tbl %>%
  select(`Data type`, `Value`) %>%
  filter(map_lgl(Value, vctrs::vec_is)) %>%
  deframe() %>%
  as_tibble() %>%
  glimpse()

## -----------------------------------------------------------------------------
pillar:::type_sum.default

expect_tibble_error <- function(object, cnd, fixed = NULL) {
  cnd_actual <- expect_error(object, class = class(cnd)[[1]])
  expect_cnd_equivalent(cnd_actual, cnd)
  expect_s3_class(cnd_actual, class(cnd), exact = TRUE)
}

expect_cnd_equivalent <- function(actual, expected) {
  actual$trace <- NULL
  actual$parent <- NULL
  actual$body <- NULL
  expected$trace <- NULL
  expected$parent <- NULL
  expected$body <- NULL
  expect_equal(actual, expected)
}

expect_error_cnd <- function(object, class, message = NULL, ..., .fixed = TRUE) {
  cnd <- expect_error(object, regexp = message, class = class, fixed = .fixed)
  expect_true(inherits_all(cnd, class))

  exp_fields <- list2(...)
  if (has_length(exp_fields)) {
    expect_true(is_empty(setdiff(!!names(exp_fields), names(cnd))))
    expect_equal(cnd[names(exp_fields)], exp_fields)
  }
}

expect_snapshot_with_error <- function(code) {
  code <- rlang::enexpr(code)

  if (packageVersion("testthat") > "3.0.0") {
    rlang::eval_tidy(rlang::quo(expect_snapshot(!!code, error = TRUE)))
  } else {
    rlang::eval_tidy(rlang::quo(expect_snapshot(!!code)))
  }
}

expect_tibble_error <- function(object, cnd, fixed = NULL) {
  cnd_actual <- expect_error(object, class = class(cnd)[[1]])
  expect_cnd_equivalent(cnd_actual, cnd)
  expect_s3_class(cnd_actual, class(cnd), exact = TRUE)
}

expect_cnd_equivalent <- function(actual, expected) {
  actual$trace <- NULL
  actual$parent <- NULL
  actual$body <- NULL
  expected$trace <- NULL
  expected$parent <- NULL
  expected$body <- NULL
  expect_equal(actual, expected)
}

expect_error_cnd <- function(object, class, message = NULL, ..., .fixed = TRUE) {
  cnd <- expect_error(object, regexp = message, class = class, fixed = .fixed)
  expect_true(inherits_all(cnd, class))

  exp_fields <- list2(...)
  if (has_length(exp_fields)) {
    expect_true(is_empty(setdiff(!!names(exp_fields), names(cnd))))
    expect_equal(cnd[names(exp_fields)], exp_fields)
  }
}

expect_snapshot_with_error <- function(code) {
  code <- rlang::enexpr(code)

  if (packageVersion("testthat") >= "3.1.1") {
    rlang::eval_tidy(rlang::quo(expect_snapshot(variant = rlang_variant(), !!code, error = TRUE)))
  } else if (packageVersion("testthat") > "3.0.0") {
    rlang::eval_tidy(rlang::quo(expect_snapshot(!!code, error = TRUE)))
  } else {
    rlang::eval_tidy(rlang::quo(expect_snapshot(!!code)))
  }
}

rlang_variant <- function() {
  NULL
}

rlang_pillar_variant <- function() {
  NULL
}

# A data frame with all major types
df_all <- tibble(
  a = c(1, 2.5, NA),
  b = c(1:2, NA),
  c = c(T, F, NA),
  d = c("a", "b", NA),
  e = factor(c("a", "b", NA)),
  f = as.Date("2015-12-09") + c(1:2, NA),
  g = as.POSIXct("2015-12-09 10:51:34 UTC") + c(1:2, NA),
  h = as.list(c(1:2, NA)),
  i = list(list(1, 2:3), list(4:6), list(NA))
)

# An empty data frame with all major types
df_empty <- tibble(
  a = integer(0),
  b = double(0),
  c = logical(0),
  d = character(0),
  e = factor(integer(0)),
  f = as.Date(character(0)),
  g = as.POSIXct(character(0)),
  h = as.list(double(0)),
  # i = list(list(integer(0)), list(character(0))),
  to_be_added = double(0)
)

# I like the API file, but can't use pkgapi::api_roclet reliably:
# * pkgapi is not on CRAN, can't add to Suggests
# * RStudio default is to ignore the `Roxygen` field in `DESCRIPTION`

if (any(c("pkgload", "devtools") %in% loadedNamespaces())) {
  if (grepl("/tests/testthat$", getwd()) && file.exists("../../API")) {
    if (max(file.info(dir("../../R", full.names = TRUE))$mtime) > file.info("../../API")$mtime) {
      if (rlang::is_installed("pkgapi")) {
        rlang::inform("Updating API file")
        writeLines(format(pkgapi::extract_api()), "../../API")
      } else {
        rlang::warn("Install r-lib/pkgapi to update the API file")
      }
    }
  }
}

## ---- include = FALSE---------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
library(dplyr)
library(tidyr)
library(purrr)
requireNamespace("hms", quietly = TRUE)

## ----setup--------------------------------------------------------------------
library(tibble)

## ----howto, echo = FALSE, eval = FALSE----------------------------------------
#  library(tidyverse)
#  library(tidymodels)
#  library(vctrs)
#  library(pillar)
#  
#  all_methods <-
#    c("vec_ptype_abbr", "vec_ptype_full", "type_sum") %>%
#    map(methods) %>%
#    map(as.character) %>%
#    map(~ gsub("^.*[.]", "", .x)) %>%
#    unlist() %>%
#    unique()
#  
#  set_names(rep_along(all_methods, list("")), all_methods) %>%
#    dput()

## ----data, echo = FALSE-------------------------------------------------------
data <- list(
  "Atomic" = rlang::quos(
    logical = TRUE,
    integer = 1L,
    double = 1.5,
    character = "A",
    complex = 1i,
    raw = as.raw(1),
    list = list(1),
    "named list" = list(a = 1)
  ),

  "Built-in objects" = rlang::quos(
    factor = factor("A"),
    ordered = ordered("a"),
    Date = Sys.Date(),
    POSIXt = Sys.time(),
    difftime = vctrs::new_duration(1)
  ),

  "Objects from other packages" = rlang::quos(
    hms = hms::hms(1),
    integer64 = bit64::as.integer64(1e10),
    blob = blob::blob(raw(1))
  ),

  "Data frames" = rlang::quos(
    data.frame = data.frame(a = 1),
    tbl_df = tibble(a = 1)
  ),

  "Unchanged" = rlang::quos(
    AsIs = I(1L)
  ),

  "vctrs types" = rlang::quos(
    unspecified = vctrs::unspecified(1),

    vctrs_list_of = vctrs::list_of(c(1L)),
    vctrs_vctr = vctrs::new_vctr(1L),

    vctrs_partial_factor = vctrs::partial_factor(letters),
    vctrs_partial_frame = vctrs::partial_frame(a = 1)
  ),

  "Language objects" = rlang::quos(
    "function" = function() NULL,
    symbol = quote(a),
    expression = parse(text = "a <- 1\nb<- 2"),
    quosures = rlang::quos(a = 1)
  )
)

## ----table, echo = FALSE------------------------------------------------------
tbl <-
  data %>%
  map(unclass) %>%
  map(enframe, "Data type", "Expression") %>%
  enframe("Class", "data") %>%
  unnest(data) %>%
  mutate(Example = map_chr(Expression, rlang::as_label)) %>%
  mutate(Value = map(Expression, rlang::eval_tidy)) %>%
  select(-Expression) %>%
  mutate(Class = if_else(Class == lag(Class, default = ""), "", Class)) %>%
  mutate("Column header" = map_chr(Value, type_sum))

## ----kable, echo = FALSE------------------------------------------------------
tbl %>%
  select(-Value) %>%
  mutate(Example = paste0("`", Example, "`")) %>%
  knitr::kable(escape = FALSE)

## ----glimpse, echo = FALSE----------------------------------------------------
tbl %>%
  select(`Data type`, `Value`) %>%
  filter(map_lgl(Value, vctrs::vec_is)) %>%
  deframe() %>%
  as_tibble() %>%
  glimpse()

## ----type_sum_default, results = if (Sys.getenv("IN_GALLEY") != "") "hide" else "markup"----
pillar:::type_sum.default

as_unknown_rows <- function(x) {
  x <- as_tibble(x)
  class(x) <- c("unknown_rows", class(x))
  x
}

dim.unknown_rows <- function(x) {
  c(NA_integer_, length(x))
}

registerS3method("dim", "unknown_rows", dim.unknown_rows)

head.unknown_rows <- function(x, n) {
  head(as.data.frame(x), n)
}

registerS3method("head", "unknown_rows", head.unknown_rows)

test_that("print() returns output invisibly", {
  expect_output(ret <- withVisible(print(as_tibble(trees))))
  expect_false(ret$visible)
  expect_identical(ret$value, as_tibble(trees))
})

test_that("output test", {
  skip_if_not_installed("mockr")
  skip_if(getRversion() < "3.2")

  expect_snapshot({
    mtcars2 <- as_tibble(mtcars, rownames = NA)

    print_without_body(mtcars2, n = 8L, width = 30L)

    print_without_body(as_tibble(trees), n = 5L, width = 30L)

    print_without_body(as_tibble(trees), n = -1L, width = 30L)

    print_without_body(as_tibble(trees), n = Inf, width = 15L)

    print_without_body(as_tibble(trees), n = NULL, width = 70L)

    print_without_body(as_unknown_rows(trees), n = 10, width = 70L)

    print_without_body(as_unknown_rows(trees[1:9, ]), n = 10, width = 70L)

    print_without_body(as_unknown_rows(trees[1:10, ]), n = 10, width = 70L)

    print_without_body(as_unknown_rows(trees[1:11, ]), n = 10, width = 70L)

    print_without_body(df_all, n = NULL, width = 30L)

    print_without_body(df_all, n = NULL, width = 300L)

    print_without_body(tibble(a = seq.int(10000)), n = 5L, width = 30L)

    print_without_body(tibble(a = character(), b = logical()), width = 30L)

    print_without_body(as_tibble(trees)[character()], n = 5L, width = 30L)

    print_without_body(as_unknown_rows(trees[integer(), ]), n = 5L, width = 30L)

    print_without_body(as_unknown_rows(trees[, character()]), n = 5L, width = 30L)

    print_without_body(
      as_unknown_rows(tibble(a = seq.int(10000))),
      n = 5L,
      width = 30L
    )
  })
})

test_that("full output test", {
  skip_if(getRversion() < "3.2")
  skip_on_os("windows")

  expect_snapshot({
    df <- tibble(x = as.POSIXct("2016-01-01 12:34:56 GMT") + 1:12)
    df$y <- as.POSIXlt(df$x)

    print(df, n = 8L, width = 60L)

    x <- c("\u6210\u4ea4\u65e5\u671f", "\u5408\u540c\u5f55\u5165\u65e5\u671f")
    df <- setNames(tibble(1:3, 4:6), x)
    print(df, n = 8L, width = 60L)
  })
})

## ---- include = FALSE---------------------------------------------------------
# To suppress messages
library(tibble)
library(vctrs)

knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  error = TRUE
)
tibble:::set_dftbl_hooks()

options(
  lifecycle_verbosity = "warning",
  lifecycle_disable_warnings = FALSE,
  lifecycle_verbose_soft_deprecation = TRUE,
  lifecycle_repeat_warnings = TRUE
)

# Set to FALSE for production
eval_details <- (Sys.getenv("IN_GALLEY") != "")

## ----setup--------------------------------------------------------------------
library(tibble)
library(vctrs)

new_df <- function() {
  df <- data.frame(n = c(1L, NA, 3L, NA))
  df$c <- letters[5:8]
  df$li <- list(9, 10:11, 12:14, "text")
  df
}

new_tbl <- function() {
  as_tibble(new_df())
}

## ----show, dftbl = TRUE, dftbl_always = TRUE----------------------------------
new_df()
new_tbl()

## ----ro-----------------------------------------------------------------------
df <- new_df()
tbl <- new_tbl()

## ----setup2-------------------------------------------------------------------
new_tbl2 <- function() {
  tibble(
    tb = tbl,
    m = diag(4)
  )
}

new_df2 <- function() {
  df2 <- new_tbl2()
  class(df2) <- "data.frame"
  class(df2$tb) <- "data.frame"
  df2
}

df2 <- new_df2()
tbl2 <- new_tbl2()

## ----show-compare-2, dftbl = TRUE---------------------------------------------

new_tbl()

## ----with-def, include = FALSE------------------------------------------------
with_df <- function(code, verbose = FALSE) {
  code <- rlang::enexpr(code)

  full_code <- rlang::quo({
    df <- new_df()
    !!code
    df
  })
  if (verbose) rlang::expr_print(rlang::quo_get_expr(full_code))
  rlang::eval_tidy(full_code)
}

with_tbl <- function(code, verbose = FALSE) {
  code <- rlang::enexpr(code)

  full_code <- rlang::quo({
    tbl <- new_tbl()
    !!code
    tbl
  })
  if (verbose) rlang::expr_print(rlang::quo_get_expr(full_code))
  rlang::eval_tidy(full_code)
}

with_df2 <- function(code) {
  code <- rlang::enexpr(code)

  full_code <- rlang::quo({
    df2 <- new_df2()
    !!code
    df2
  })
  rlang::eval_tidy(full_code)
}

with_tbl2 <- function(code) {
  code <- rlang::enexpr(code)

  full_code <- rlang::quo({
    tbl2 <- new_tbl2()
    !!code
    tbl2
  })
  rlang::eval_tidy(full_code)
}

## ----with-demo, dftbl = TRUE--------------------------------------------------
with_df(df$n <- rev(df$n), verbose = TRUE)
with_tbl(tbl$n <- rev(tbl$n), verbose = TRUE)

## ----double-bracket-equivalent-to-subset2, dftbl = TRUE-----------------------

tbl[[1]]

.subset2(tbl, 1)

## ----double-bracket-equivalent-to-subset2-detail, dftbl = TRUE, include = eval_details, eval = eval_details----
#  identical(df[[3]], .subset2(df, 3))
#  identical(tbl[[3]], .subset2(tbl, 3))
#  identical(df2[["df"]], .subset2(df2, "df"))
#  identical(tbl2[["tbl"]], .subset2(tbl2, "tbl"))

## ----double-bracket-retains-size, dftbl = TRUE, include = eval_details, eval = eval_details----
#  vec_size(df[[1]])
#  vec_size(tbl[[1]])
#  vec_size(df[[3]])
#  vec_size(tbl[[3]])
#  vec_size(df2[[1]])
#  vec_size(tbl2[[1]])
#  vec_size(df2[[2]])
#  vec_size(tbl2[[2]])

## ----double-bracket-requires-scalar-j-index, dftbl = TRUE---------------------
df[[1:2]]
tbl[[1:2]]
df[[c("n", "c")]]
tbl[[c("n", "c")]]
df[[TRUE]]
tbl[[TRUE]]
df[[mean]]
tbl[[mean]]

## ----double-bracket-j-oob-numeric, dftbl = TRUE-------------------------------
df[[NA]]
tbl[[NA]]
df[[NA_character_]]
tbl[[NA_character_]]
df[[NA_integer_]]
tbl[[NA_integer_]]
df[[-1]]
tbl[[-1]]
df[[4]]
tbl[[4]]
df[[1.5]]
tbl[[1.5]]
df[[Inf]]
tbl[[Inf]]

## ----double-bracket-j-oob-character, dftbl = TRUE-----------------------------

tbl[["x"]]

## ----dollar-equivalent-to-subset, dftbl = TRUE--------------------------------

tbl$n

tbl$"n"

tbl[["n"]]

## ----dollar-equivalent-to-subset-detail, dftbl = TRUE, include = eval_details, eval = eval_details----
#  identical(df$li, df[["li"]])
#  identical(tbl$li, tbl[["li"]])
#  identical(df2$tb, df2[["tb"]])
#  identical(tbl2$tb, tbl2[["tb"]])
#  identical(df2$m, df2[["m"]])
#  identical(tbl2$m, tbl2[["m"]])

## ----dollar-equivalent-to-subset-pmatch, dftbl = TRUE-------------------------
df$l
tbl$l
df$not_present
tbl$not_present

## ----bracket-j-definition, dftbl = TRUE---------------------------------------

tbl[1:2]

## ----bracket-j-duplication, dftbl = TRUE--------------------------------------
df[c(1, 1)]
tbl[c(1, 1)]

## ----bracket-j-empty, dftbl = TRUE--------------------------------------------

tbl[integer()]

## ----bracket-j-logical-matrix, dftbl = TRUE-----------------------------------
df[is.na(df)]
tbl[is.na(tbl)]
df[!is.na(df)]
tbl[!is.na(tbl)]

## ----bracket-missing-i, dftbl = TRUE------------------------------------------
df[, 1]
tbl[, 1]

tbl[, 1:2]

## ----bracket-missing-i-detail, dftbl = TRUE, include = eval_details, eval = eval_details----
#  identical(df[, 2:3], df[2:3])
#  identical(tbl[, 2:3], tbl[2:3])
#  identical(df2[, 1:2], df2[1:2])
#  identical(tbl2[, 1:2], tbl2[1:2])

## ----bracket-always-returns-tibble-drop, dftbl = TRUE-------------------------

tbl[, 1, drop = TRUE]

## ----bracket-always-returns-tibble-drop-detail, dftbl = TRUE, include = eval_details, eval = eval_details----
#  identical(df[, 3, drop = TRUE], df[[3]])
#  identical(tbl[, 3, drop = TRUE], tbl[[3]])
#  identical(df2[, 1, drop = TRUE], df2[[1]])
#  identical(tbl2[, 1, drop = TRUE], tbl2[[1]])
#  identical(df2[, 2, drop = TRUE], df2[[2]])
#  identical(tbl2[, 2, drop = TRUE], tbl2[[2]])

## ----bracket-i, dftbl = TRUE--------------------------------------------------

tbl[3, ]

## ----bracket-i-wrong-type, dftbl = TRUE---------------------------------------
df[mean, ]
tbl[mean, ]
df[list(1), ]
tbl[list(1), ]

tbl["1", ]

## ----bracket-i-oob, dftbl = TRUE----------------------------------------------

tbl[10, ]

tbl["x", ]

## ----bracket-i-recycle, dftbl = TRUE------------------------------------------
df[c(TRUE, FALSE), ]
tbl[c(TRUE, FALSE), ]

## ----bracket-i-na, dftbl = TRUE-----------------------------------------------

tbl[NA, ]

tbl[NA_integer_, ]

## ----bracket-i-drop, dftbl = TRUE---------------------------------------------
df[1, , drop = TRUE]
tbl[1, , drop = TRUE]

## ----bracket-i-j-equivalent-to-i-subset-then-j, dftbl = TRUE, include = eval_details, eval = eval_details----
#  df[1, 1]
#  tbl[1, 1]
#  df[1, ][1]
#  tbl[1, ][1]
#  identical(df[1, 2:3], df[2:3][1, ])
#  identical(tbl[1, 2:3], tbl[2:3][1, ])
#  identical(df[2:3, 1], df[1][2:3, ])
#  identical(tbl[2:3, 1], tbl[1][2:3, ])
#  identical(df2[2:3, 1:2], df2[1:2][2:3, ])
#  identical(tbl2[2:3, 1:2], tbl2[1:2][2:3, ])

## ----bracket-bracket-i-j-equivalent-to-i-subset-then-j------------------------
df[[1, 1]]
df[[1, 3]]

## ----double-bracket-assign-definition, dftbl = TRUE, include = eval_details, eval = eval_details----
#  with_df(df[[1]] <- 0)
#  with_tbl(tbl[[1]] <- 0)
#  with_df(df[[3]] <- 4:1)
#  with_tbl(tbl[[3]] <- 4:1)
#  with_df2(df2[[1]] <- 0)
#  with_tbl2(tbl2[[1]] <- 0)
#  with_df2(df2[[2]] <- 4:1)
#  with_tbl2(tbl2[[2]] <- 4:1)

## ----double-bracket-assign-requires-scalar-j-index, dftbl = TRUE, include = eval_details, eval = eval_details----
#  with_df(df[[1]] <- 0)
#  with_tbl(tbl[[1]] <- 0)
#  with_df(df[["c"]] <- 0)
#  with_tbl(tbl[["c"]] <- 0)

## ----double-bracket-assign-requires-scalar-j-index-error, dftbl = TRUE, include = eval_details, eval = eval_details----
#  with_df(df[[TRUE]] <- 0)
#  with_tbl(tbl[[TRUE]] <- 0)
#  with_df(df[[1:3]] <- 0)
#  with_tbl(tbl[[1:3]] <- 0)
#  with_df(df[[c("n", "c")]] <- 0)
#  with_tbl(tbl[[c("n", "c")]] <- 0)
#  with_df(df[[FALSE]] <- 0)
#  with_tbl(tbl[[FALSE]] <- 0)
#  with_df(df[[1:2]] <- 0)
#  with_tbl(tbl[[1:2]] <- 0)
#  with_df(df[[NA_integer_]] <- 0)
#  with_tbl(tbl[[NA_integer_]] <- 0)
#  with_df(df[[NA]] <- 0)
#  with_tbl(tbl[[NA]] <- 0)
#  with_df(df[[NA_character_]] <- 0)
#  with_tbl(tbl[[NA_character_]] <- 0)

## ----double-bracket-assign-recycle, dftbl = TRUE------------------------------

with_tbl(tbl[["li"]] <- list(0))
with_df2(df2[["tb"]] <- df[1, ])
with_tbl2(tbl2[["tb"]] <- tbl[1, ])
with_df2(df2[["m"]] <- df2[["m"]][1, , drop = FALSE])
with_tbl2(tbl2[["m"]] <- tbl2[["m"]][1, , drop = FALSE])

## ----double-bracket-requires-size, dftbl = TRUE, include = eval_details, eval = eval_details----
#  with_df(df[[1]] <- 1)
#  with_tbl(tbl[[1]] <- 1)
#  with_df(df[[1]] <- 4:1)
#  with_tbl(tbl[[1]] <- 4:1)
#  with_df(df[[1]] <- 3:1)
#  with_tbl(tbl[[1]] <- 3:1)
#  with_df(df[[1]] <- 2:1)
#  with_tbl(tbl[[1]] <- 2:1)

## ----double-bracket-assign-supports-new, dftbl = TRUE-------------------------

with_tbl(tbl[["x"]] <- 0)
with_df(df[[4]] <- 0)
with_tbl(tbl[[4]] <- 0)
with_df(df[[5]] <- 0)
with_tbl(tbl[[5]] <- 0)

## ----double-bracket-assign-supports-type-change, dftbl = TRUE, include = eval_details, eval = eval_details----
#  with_df(df[[1]] <- df[[2]])
#  with_tbl(tbl[[1]] <- tbl[[2]])
#  with_df(df[[2]] <- df[[3]])
#  with_tbl(tbl[[2]] <- tbl[[3]])
#  with_df(df[[3]] <- df2[[1]])
#  with_tbl(tbl[[3]] <- tbl2[[1]])
#  with_df2(df2[[1]] <- df2[[2]])
#  with_tbl2(tbl2[[1]] <- tbl2[[2]])
#  with_df2(df2[[2]] <- df[[1]])
#  with_tbl2(tbl2[[2]] <- tbl[[1]])

## ----double-bracket-assign-supports-null, dftbl = TRUE, include = eval_details, eval = eval_details----
#  with_df(df[[1]] <- NULL)
#  with_tbl(tbl[[1]] <- NULL)
#  with_df2(df2[[2]] <- NULL)
#  with_tbl2(tbl2[[2]] <- NULL)

## ----double-bracket-assign-supports-null-unknown, dftbl = TRUE, include = eval_details, eval = eval_details----
#  with_df(df[["q"]] <- NULL)
#  with_tbl(tbl[["q"]] <- NULL)

## ----dollar-equivalent-to-subset-assign, dftbl = TRUE-------------------------

with_tbl(tbl$n <- 0)

with_tbl(tbl[["n"]] <- 0)

## ----dollar-equivalent-to-subset-assign-detail, dftbl = TRUE, include = eval_details, eval = eval_details----
#  with_df(df$"n" <- 0)
#  with_tbl(tbl$"n" <- 0)

## ----dollar-equivalent-to-subset-assign-pmatch, dftbl = TRUE------------------

with_tbl(tbl$l <- 0)

with_tbl(tbl[["l"]] <- 0)

## ----bracket-assign-def, dftbl = TRUE-----------------------------------------

with_tbl(tbl[1:2] <- list("x", 4:1))

with_tbl(tbl[c("li", "x", "c")] <- list("x", 4:1, NULL))

## ----bracket-assign-recycles, dftbl = TRUE------------------------------------

with_tbl(tbl[1:2] <- list(1))
with_df(df[1:2] <- list(0, 0, 0))
with_tbl(tbl[1:2] <- list(0, 0, 0))
with_df(df[1:3] <- list(0, 0))
with_tbl(tbl[1:3] <- list(0, 0))

## ---- bracket-assign-multiple, dftbl = TRUE-----------------------------------
with_df(df[c(1, 1)] <- list(1, 2))
with_tbl(tbl[c(1, 1)] <- list(1, 2))

## ----bracket-assign-remove, dftbl = TRUE--------------------------------------

with_tbl(tbl[1:2] <- list(NULL, 4:1))

## ----bracket-assign-na, dftbl = TRUE, include = eval_details, eval = eval_details----
#  with_df(df[NA] <- list("x"))
#  with_tbl(tbl[NA] <- list("x"))
#  with_df(df[NA_integer_] <- list("x"))
#  with_tbl(tbl[NA_integer_] <- list("x"))
#  with_df(df[NA_character_] <- list("x"))
#  with_tbl(tbl[NA_character_] <- list("x"))

## ----bracket-assign-supports-type-change, dftbl = TRUE, include = eval_details, eval = eval_details----
#  with_df(df[1] <- df[2])
#  with_tbl(tbl[1] <- tbl[2])
#  with_df(df[2] <- df[3])
#  with_tbl(tbl[2] <- tbl[3])
#  with_df(df[3] <- df2[1])
#  with_tbl(tbl[3] <- tbl2[1])
#  with_df2(df2[1] <- df2[2])
#  with_tbl2(tbl2[1] <- tbl2[2])
#  with_df2(df2[2] <- df[1])
#  with_tbl2(tbl2[2] <- tbl[1])

## ----bracket-assign-names, dftbl = TRUE---------------------------------------

with_tbl(tbl[c("x", "y")] <- tibble("x", x = 4:1))

with_tbl(tbl[3:4] <- list("x", x = 4:1))
with_df(df[4] <- list(4:1))
with_tbl(tbl[4] <- list(4:1))
with_df(df[5] <- list(4:1))
with_tbl(tbl[5] <- list(4:1))

## ----bracket-j-assign-logical-matrix, dftbl = TRUE----------------------------
with_df(df[is.na(df)] <- 4)
with_tbl(tbl[is.na(tbl)] <- 4)
with_df(df[is.na(df)] <- 1:2)
with_tbl(tbl[is.na(tbl)] <- 1:2)
with_df(df[matrix(c(rep(TRUE, 5), rep(FALSE, 7)), ncol = 3)] <- 4)
with_tbl(tbl[matrix(c(rep(TRUE, 5), rep(FALSE, 7)), ncol = 3)] <- 4)

## ----bracket-assign-array, dftbl = TRUE---------------------------------------

with_tbl(tbl[1:2] <- matrix(8:1, ncol = 2))
with_df(df[1:3, 1:2] <- matrix(6:1, ncol = 2))
with_tbl(tbl[1:3, 1:2] <- matrix(6:1, ncol = 2))

with_tbl(tbl[1:2] <- array(4:1, dim = c(4, 1, 1)))

with_tbl(tbl[1:2] <- array(8:1, dim = c(4, 2, 1)))
with_df(df[1:2] <- array(8:1, dim = c(2, 1, 4)))
with_tbl(tbl[1:2] <- array(8:1, dim = c(2, 1, 4)))
with_df(df[1:2] <- array(8:1, dim = c(4, 1, 2)))
with_tbl(tbl[1:2] <- array(8:1, dim = c(4, 1, 2)))

## ----bracket-assign-wraps, dftbl = TRUE---------------------------------------

with_tbl(tbl[1] <- 0)

with_tbl(tbl[1] <- list(0))

## ----bracket-assign-matrix, dftbl = TRUE--------------------------------------

with_tbl(tbl[1] <- list(matrix(1:8, ncol = 2)))



with_tbl(tbl[1:2] <- list(matrix(1:8, ncol = 2)))

## ----bracket-assign-null, dftbl = TRUE----------------------------------------

with_tbl(tbl[1] <- NULL)

with_tbl(tbl[, 2:3] <- NULL)
with_df(df[1, 2:3] <- NULL)
with_tbl(tbl[1, 2:3] <- NULL)

## ----bracket-assign-non-vector, dftbl = TRUE----------------------------------
with_df(df[1] <- mean)
with_tbl(tbl[1] <- mean)
with_df(df[1] <- lm(mpg ~ wt, data = mtcars))
with_tbl(tbl[1] <- lm(mpg ~ wt, data = mtcars))

## ----bracket-i-assign, dftbl = TRUE-------------------------------------------

with_tbl(tbl[2:3, ] <- tbl[1, ])

with_tbl(tbl[c(FALSE, TRUE, TRUE, FALSE), ] <- tbl[1, ])

## ----bracket-i-assign-detail, dftbl = TRUE, include = eval_details, eval = eval_details----
#  with_df(df[0:2, ] <- df[1, ])
#  with_tbl(tbl[0:2, ] <- tbl[1, ])
#  with_df(df[0, ] <- df[1, ])
#  with_tbl(tbl[0, ] <- tbl[1, ])
#  with_df(df[-2, ] <- df[1, ])
#  with_tbl(tbl[-2, ] <- tbl[1, ])
#  with_df(df[-1:2, ] <- df[1, ])
#  with_tbl(tbl[-1:2, ] <- tbl[1, ])
#  with_df(df[NA_integer_, ] <- df[1, ])
#  with_tbl(tbl[NA_integer_, ] <- tbl[1, ])
#  with_df2(df2[NA_integer_, ] <- df2[1, ])
#  with_tbl2(tbl2[NA_integer_, ] <- tbl2[1, ])
#  with_df(df[TRUE, ] <- df[1, ])
#  with_tbl(tbl[TRUE, ] <- tbl[1, ])
#  with_df(df[FALSE, ] <- df[1, ])
#  with_tbl(tbl[FALSE, ] <- tbl[1, ])
#  with_df(df[NA, ] <- df[1, ])
#  with_tbl(tbl[NA, ] <- tbl[1, ])

## ----bracket-i-recycle-assign, dftbl = TRUE-----------------------------------

with_tbl(tbl[2:3, ] <- tbl[1, ])

with_tbl(tbl[2:3, ] <- list(tbl$n[1], tbl$c[1:2], tbl$li[1]))
with_df(df[2:4, ] <- df[1:2, ])
with_tbl(tbl[2:4, ] <- tbl[1:2, ])

## ----bracket-i-recycle-assign-detail, dftbl = TRUE, include = eval_details, eval = eval_details----
#  with_df2(df2[2:4, ] <- df2[1, ])
#  with_tbl2(tbl2[2:4, ] <- tbl2[1, ])
#  with_df2(df2[2:4, ] <- df2[2:3, ])
#  with_tbl2(tbl2[2:4, ] <- tbl2[2:3, ])

## ----bracket-i-oob-num, dftbl = TRUE------------------------------------------

with_tbl(tbl[5, ] <- tbl[1, ])

with_tbl(tbl[5:7, ] <- tbl[1, ])
with_df(df[6, ] <- df[1, ])
with_tbl(tbl[6, ] <- tbl[1, ])
with_df(df[-5, ] <- df[1, ])
with_tbl(tbl[-5, ] <- tbl[1, ])
with_df(df[-(5:7), ] <- df[1, ])
with_tbl(tbl[-(5:7), ] <- tbl[1, ])
with_df(df[-6, ] <- df[1, ])
with_tbl(tbl[-6, ] <- tbl[1, ])

## ----bracket-i-character, dftbl = TRUE----------------------------------------

with_tbl(tbl[as.character(1:3), ] <- tbl[1, ])

## ----bracket-i-character-detail, dftbl = TRUE, include = eval_details, eval = eval_details----
#  with_df(df[as.character(-(1:3)), ] <- df[1, ])
#  with_tbl(tbl[as.character(-(1:3)), ] <- tbl[1, ])
#  with_df(df[as.character(3:5), ] <- df[1, ])
#  with_tbl(tbl[as.character(3:5), ] <- tbl[1, ])
#  with_df(df[as.character(-(3:5)), ] <- df[1, ])
#  with_tbl(tbl[as.character(-(3:5)), ] <- tbl[1, ])
#  with_df(df[NA_character_, ] <- df[1, ])
#  with_tbl(tbl[NA_character_, ] <- tbl[1, ])

## ----bracket-i-data-type, dftbl = TRUE----------------------------------------
with_df(df[2:3, 1] <- df[1:2, 2])
with_tbl(tbl[2:3, 1] <- tbl[1:2, 2])
with_df(df[2:3, 2] <- df[1:2, 3])
with_tbl(tbl[2:3, 2] <- tbl[1:2, 3])
with_df(df[2:3, 3] <- df2[1:2, 1])
with_tbl(tbl[2:3, 3] <- tbl2[1:2, 1])
with_df2(df2[2:3, 1] <- df2[1:2, 2])
with_tbl2(tbl2[2:3, 1] <- tbl2[1:2, 2])

with_tbl2(tbl2[2:3, 2] <- tbl[1:2, 1])

## ----bracket-i-j-na-init, dftbl = TRUE----------------------------------------

with_tbl({tbl$x <- NA; tbl[2:3, "x"] <- 3:2})
with_df({df[2:3, 2:3] <- NA})
with_tbl({tbl[2:3, 2:3] <- NA})

## ----bracket-i-j-typed-na-init, dftbl = TRUE----------------------------------

with_tbl({tbl$x <- NA_integer_; tbl[2:3, "x"] <- 3:2})

## ----subassign-ij-new-column, dftbl = TRUE------------------------------------
with_df(df[2:3, "n"] <- 1)
with_tbl(tbl[2:3, "n"] <- 1)

with_tbl(tbl[2:3, "x"] <- 1)
with_df(df[2:3, "n"] <- NULL)
with_tbl(tbl[2:3, "n"] <- NULL)

## ----append-rows-only-all-columns, dftbl = TRUE-------------------------------

with_tbl(tbl[5, "n"] <- list(0L))

## ----double-bracket-i-j-equivalent-to-row-subset-then-j, dftbl = TRUE, include = eval_details, eval = eval_details----
#  with_df(df[[1, 1]] <- 0)
#  with_tbl(tbl[[1, 1]] <- 0)
#  with_df(df[1, ][[1]] <- 0)
#  with_tbl(tbl[1, ][[1]] <- 0)
#  with_df(df[[1, 3]] <- list(NULL))
#  with_tbl(tbl[[1, 3]] <- list(NULL))
#  with_df(df[1, ][[3]] <- list(NULL))
#  with_tbl(tbl[1, ][[3]] <- list(NULL))
#  with_df2(df2[[1, 1]] <- df[1, ])
#  with_tbl2(tbl2[[1, 1]] <- tbl[1, ])
#  with_df2(df2[1, ][[1]] <- df[1, ])
#  with_tbl2(tbl2[1, ][[1]] <- tbl[1, ])
#  with_df2(df2[[1, 2]] <- t(1:4))
#  with_tbl2(tbl2[[1, 2]] <- t(1:4))
#  with_df2(df2[1, ][[2]] <- t(1:4))
#  with_tbl2(tbl2[1, ][[2]] <- t(1:4))
#  df[[1:2, 1]]
#  tbl[[1:2, 1]]
#  with_df(df[[1:2, 1]] <- 0)
#  with_tbl(tbl[[1:2, 1]] <- 0)

## ----check, dftbl = TRUE, include = FALSE-------------------------------------

stopifnot(identical(tbl, new_tbl()))

# nocov start
.onLoad <- function(libname, pkgname) {
  if (getRversion() == "3.3.0") {
    safe_match <<- safe_match_3_0
  } else {
    safe_match <<- safe_match_default
  }

  # Simplify once pillar 1.5.0 is on CRAN
  replace_if_pillar_has(glimpse)
  replace_if_pillar_has(tbl_sum)
  register_if_pillar_hasnt(c(
    "glimpse.default", "glimpse.data.frame", "glimpse.tbl",
    "tbl_sum.default", "tbl_sum.tbl", "format.tbl", "print.tbl"
  ))

  vctrs::s3_register("tibble::tbl_sum", "tbl_df")
}

safe_match_3_0 <- function(x, table) {
  match(x, table, incomparables = character())
}

safe_match_default <- function(x, table) {
  match(x, table)
}
# nocov end

#' Format a numeric vector
#'
#' @description
#' `r lifecycle::badge("experimental")`
#'
#' Constructs a numeric vector that can be formatted with predefined
#' significant digits, or with a maximum or fixed number of digits
#' after the decimal point.
#' Scaling is supported, as well as forcing a decimal, scientific
#' or engineering notation.
#' If a label is given, it is shown in the header of a column.
#'
#' The formatting is applied when the vector is printed or formatted,
#' and also in a tibble column.
#' The formatting annotation and the class survives most arithmetic transformations,
#' the most notable exceptions are [var()] and [sd()].
#'
#' @family vector classes
#' @inheritParams rlang::args_dots_empty
#' @param x A numeric vector.
#' @param sigfig Define the number of significant digits to show. Must be one or greater.
#'   The `"pillar.sigfig"` [option][pillar::pillar_options] is not consulted.
#'   Can't be combined with `digits`.
#' @param digits Number of digits after the decimal points to show.
#'   Positive numbers specify the exact number of digits to show.
#'   Negative numbers specify (after negation) the maximum number of digits to show.
#'   With `digits = 2`, the numbers 1.2 and 1.234 are printed as 1.20 and 1.23,
#'   with `digits = -2` as 1.2 and 1.23, respectively.
#'   Can't be combined with `sigfig`.
#' @param label A label to show instead of the type description.
#' @param scale Multiplier to apply to the data before showing.
#'   Useful for displaying e.g. percentages.
#'   Must be combined with `label`.
#' @param notation One of `"fit"`, `"dec"`, `"sci"`, `"eng"`, or `"si"`.
#'   - `"fit"`: Use decimal notation if it fits and if it consumes 13 digits or less,
#'     otherwise use scientific notation. (The default for numeric pillars.)
#'   - `"dec"`: Use decimal notation, regardless of width.
#'   - `"sci"`: Use scientific notation.
#'   - `"eng"`: Use engineering notation, i.e. scientific notation
#'       using exponents that are a multiple of three.
#'   - `"si"`: Use SI notation, prefixes between `1e-24` and `1e24` are supported.
#' @param fixed_exponent
#'   Use the same exponent for all numbers in scientific, engineering or SI notation.
#'   `-Inf` uses the smallest, `+Inf` the largest fixed_exponent present in the data.
#'   The default is to use varying exponents.
#' @param extra_sigfig
#'   If `TRUE`, increase the number of significant digits if the data consists of
#'   numbers of the same magnitude with subtle differences.
#' @export
#' @examples
#' # Display as a vector
#' num(9:11 * 100 + 0.5)
#' @examples
#'
#' # Significant figures
#' tibble(
#'   x3 = num(9:11 * 100 + 0.5, sigfig = 3),
#'   x4 = num(9:11 * 100 + 0.5, sigfig = 4),
#'   x5 = num(9:11 * 100 + 0.5, sigfig = 5),
#' )
#'
#' # Maximum digits after the decimal points
#' tibble(
#'   x0 = num(9:11 * 100 + 0.5, digits = 0),
#'   x1 = num(9:11 * 100 + 0.5, digits = -1),
#'   x2 = num(9:11 * 100 + 0.5, digits = -2),
#' )
#'
#' # Use fixed digits and a currency label
#' tibble(
#'   usd = num(9:11 * 100 + 0.5, digits = 2, label = "USD"),
#'   gbp = num(9:11 * 100 + 0.5, digits = 2, label = "£"),
#'   chf = num(9:11 * 100 + 0.5, digits = 2, label = "SFr")
#' )
#'
#' # Scale
#' tibble(
#'   small  = num(9:11 / 1000 + 0.00005, label = "%", scale = 100),
#'   medium = num(9:11 / 100 + 0.0005, label = "%", scale = 100),
#'   large  = num(9:11 / 10 + 0.005, label = "%", scale = 100)
#' )
#'
#' # Notation
#' tibble(
#'   sci = num(10^(-13:6), notation = "sci"),
#'   eng = num(10^(-13:6), notation = "eng"),
#'   si  = num(10^(-13:6), notation = "si"),
#'   dec = num(10^(-13:6), notation = "dec")
#' )
#'
#' # Fixed exponent
#' tibble(
#'   scimin = num(10^(-7:6) * 123, notation = "sci", fixed_exponent = -Inf),
#'   engmin = num(10^(-7:6) * 123, notation = "eng", fixed_exponent = -Inf),
#'   simin  = num(10^(-7:6) * 123, notation = "si", fixed_exponent = -Inf)
#' )
#'
#' tibble(
#'   scismall = num(10^(-7:6) * 123, notation = "sci", fixed_exponent = -3),
#'   scilarge = num(10^(-7:6) * 123, notation = "sci", fixed_exponent = 3),
#'   scimax   = num(10^(-7:6) * 123, notation = "sci", fixed_exponent = Inf)
#' )
#'
#' #' Extra significant digits
#' tibble(
#'   default = num(100 + 1:3 * 0.001),
#'   extra1 = num(100 + 1:3 * 0.001, extra_sigfig = TRUE),
#'   extra2 = num(100 + 1:3 * 0.0001, extra_sigfig = TRUE),
#'   extra3 = num(10000 + 1:3 * 0.00001, extra_sigfig = TRUE)
#' )
# Assigned in .onLoad()
num <- NULL

#' set_num_opts
#'
#' `set_num_opts()` adds formatting options to an arbitrary numeric vector,
#' useful for composing with other types.
#'
#' @export
#' @rdname num
# Assigned in .onLoad()
set_num_opts <- NULL

#' Format a character vector
#'
#' @description
#' `r lifecycle::badge("experimental")`
#'
#' Constructs a character vector that can be formatted with predefined minimum width
#' or without width restrictions, and where the abbreviation style can be configured.
#'
#' The formatting is applied when the vector is printed or formatted,
#' and also in a tibble column.
#'
#' @family vector classes
#' @inheritParams rlang::args_dots_empty
#' @param x A character vector.
#' @param min_chars The minimum width to allocate to this column, defaults to 15.
#'   The `"pillar.min_chars"` [option][pillar::pillar_options] is not consulted.
#' @param shorten How to abbreviate the data if necessary:
#' - `"back"` (default): add an ellipsis at the end
#' - `"front"`: add an ellipsis at the front
#' - `"mid"`: add an ellipsis in the middle
#' - `"abbreviate"`: use [abbreviate()]
#' @export
#' @examples
#' # Display as a vector:
#' char(letters[1:3])
#' @examplesIf { set.seed(20210331); rlang::is_installed("stringi") }
#' # Space constraints:
#' rand_strings <- stringi::stri_rand_strings(10, seq(40, 22, by = -2))
#'
#' # Plain character vectors get truncated if space is limited:
#' data_with_id <- function(id) {
#'   tibble(
#'     id,
#'     some_number_1 = 1, some_number_2 = 2, some_number_3 = 3,
#'     some_number_4 = 4, some_number_5 = 5, some_number_6 = 6,
#'     some_number_7 = 7, some_number_8 = 8, some_number_9 = 9
#'   )
#' }
#' data_with_id(rand_strings)
#'
#' # Use char() to avoid or control truncation
#' data_with_id(char(rand_strings, min_chars = 24))
#' data_with_id(char(rand_strings, min_chars = Inf))
#' data_with_id(char(rand_strings, min_chars = 24, shorten = "mid"))
#'
#' # Lorem Ipsum, one sentence per row.
#' lipsum <- unlist(strsplit(stringi::stri_rand_lipsum(1), "(?<=[.]) +", perl = TRUE))
#' tibble(
#'   back = char(lipsum, shorten = "back"),
#'   front = char(lipsum, shorten = "front"),
#'   mid = char(lipsum, shorten = "mid")
#' )
#' tibble(abbr = char(lipsum, shorten = "abbreviate"))
# Assigned in .onLoad()
char <- NULL

#' set_char_opts
#'
#' `set_char_opts()` adds formatting options to an arbitrary character vector,
#' useful for composing with other types.
#'
#' @export
#' @rdname char
# Assigned in .onLoad()
set_char_opts <- NULL

as_override_type_sum <- function(x) {
  structure(x, class = "override_type_sum")
}

type_sum.override_type_sum <- function(x, ...) {
  "SC"
}

registerS3method("type_sum", "override_type_sum", type_sum.override_type_sum, envir = asNamespace("pillar"))

`[.override_type_sum` <- function(x, ...) {
  as_override_type_sum(NextMethod())
}

registerS3method("[", "override_type_sum", `[.override_type_sum`, envir = asNamespace("tibble"))

as_override_tbl_sum <- function(x) {
  structure(x, class = c("override_tbl_sum", class(x)))
}

tbl_sum.override_tbl_sum <- function(x) {
  c(NextMethod(), "Overridden" = "tbl_sum")
}

registerS3method("tbl_sum", "override_tbl_sum", tbl_sum.override_tbl_sum, envir = asNamespace("tibble"))

#' Row-wise tibble creation
#'
#' @description
#' Create [tibble]s using an easier to read row-by-row layout.
#' This is useful for small tables of data where readability is
#' important.  Please see \link{tibble-package} for a general introduction.
#'
#' @param ... <[`dynamic-dots`][rlang::dyn-dots]>
#'   Arguments specifying the structure of a `tibble`.
#'   Variable names should be formulas, and may only appear before the data.
#'   These arguments are processed with [rlang::list2()]
#'   and support unquote via [`!!`] and unquote-splice via [`!!!`].
#' @return A [tibble].
#' @seealso
#'   See [quasiquotation] for more details on tidy dots semantics,
#'   i.e. exactly how  the `...` argument is processed.
#' @export
#' @examples
#' tribble(
#'   ~colA, ~colB,
#'   "a",   1,
#'   "b",   2,
#'   "c",   3
#' )
#'
#' # tribble will create a list column if the value in any cell is
#' # not a scalar
#' tribble(
#'   ~x,  ~y,
#'   "a", 1:3,
#'   "b", 4:6
#' )
#' @examplesIf rlang::is_installed("dplyr") && packageVersion("dplyr") >= "1.0.5"
#'
#' # Use dplyr::mutate(dplyr::across(...)) to assign an explicit type
#' tribble(
#'   ~a, ~b, ~c,
#'   1, "2000-01-01", "1.5"
#' ) %>%
#'   dplyr::mutate(
#'     dplyr::across(a, as.integer),
#'     dplyr::across(b, as.Date)
#'   )
tribble <- function(...) {
  data <- extract_frame_data_from_dots(...)
  turn_frame_data_into_tibble(data$frame_names, data$frame_rest)
}

#' Row-wise matrix creation
#'
#' @description
#' Create matrices laying out the data in rows, similar to
#' `matrix(..., byrow = TRUE)`, with a nicer-to-read syntax.
#' This is useful for small matrices, e.g. covariance matrices, where readability
#' is important. The syntax is inspired by [tribble()].
#'
#' @param ... <[`dynamic-dots`][rlang::dyn-dots]>
#'   Arguments specifying the structure of a `frame_matrix`.
#'   Column names should be formulas, and may only appear before the data.
#'   These arguments are processed with [rlang::list2()]
#'   and support unquote via [`!!`] and unquote-splice via [`!!!`].
#' @return A [matrix].
#' @seealso
#'   See [quasiquotation] for more details on tidy dots semantics,
#'   i.e. exactly how  the `...` argument is processed.
#' @export
#' @examples
#' frame_matrix(
#'   ~col1, ~col2,
#'   1,     3,
#'   5,     2
#' )
frame_matrix <- function(...) {
  data <- extract_frame_data_from_dots(...)
  turn_frame_data_into_frame_matrix(data$frame_names, data$frame_rest)
}

extract_frame_data_from_dots <- function(..., .call = caller_env()) {
  dots <- list2(...)

  # Extract the names.
  frame_names <- extract_frame_names_from_dots(dots, .call)

  # Extract the data
  if (length(frame_names) == 0 && length(dots) != 0) {
    abort_tribble_needs_columns(.call)
  }
  frame_rest <- dots[-seq_along(frame_names)]
  if (!is.null(names(frame_rest))) {
    abort_tribble_named_after_tilde(.call)
  }
  if (length(frame_rest) == 0L) {
    # Can't decide on type in absence of data -- use logical which is
    # coercible to all types
    frame_rest <- unspecified()
  }

  validate_rectangular_shape(frame_names, frame_rest, .call)

  list(frame_names = frame_names, frame_rest = frame_rest)
}

extract_frame_names_from_dots <- function(dots, call = caller_env()) {
  frame_names <- character()

  for (i in seq_along(dots)) {
    el <- dots[[i]]
    if (!is.call(el)) {
      break
    }

    if (!identical(el[[1]], as.name("~"))) {
      break
    }

    if (length(el) != 2) {
      abort_tribble_lhs_column_syntax(el[[2]], call)
    }

    candidate <- el[[2]]
    if (!(is.symbol(candidate) || is.character(candidate))) {
      abort_tribble_rhs_column_syntax(candidate)
    }

    frame_names <- c(frame_names, as.character(candidate))
  }

  frame_names
}

validate_rectangular_shape <- function(frame_names, frame_rest, call = caller_env()) {
  if (length(frame_names) == 0 && length(frame_rest) == 0) return()

  # Figure out the associated number of rows and number of columns,
  # and validate that the supplied formula produces a rectangular
  # structure.
  if (length(frame_rest) %% length(frame_names) != 0) {
    abort_tribble_non_rectangular(length(frame_names), length(frame_rest), call)
  }
}

turn_frame_data_into_tibble <- function(names, rest, call = caller_env()) {
  if (is_empty(names)) return(new_tibble(list(), nrow = 0))

  nrow <- length(rest) / length(names)
  dim(rest) <- c(length(names), nrow)
  dimnames(rest) <- list(names, NULL)

  frame_mat <- t(rest)
  frame_col <- turn_matrix_into_column_list(frame_mat, call)

  new_tibble(frame_col, nrow = nrow)
}

turn_matrix_into_column_list <- function(frame_mat, call) {
  frame_col <- vector("list", length = ncol(frame_mat))
  names(frame_col) <- colnames(frame_mat)

  # if a frame_mat's col is a list column, keep it unchanged (does not unlist)
  for (i in seq_len(ncol(frame_mat))) {
    col <- frame_mat[, i]

    if (inherits(col, "list") && !some(col, needs_list_col)) {
      subclass_tribble_c_errors(
        names(frame_col)[[i]],
        col <- vec_c(!!!unname(col)),
        call
      )
    }

    frame_col[[i]] <- unname(col)
  }
  return(frame_col)
}

turn_frame_data_into_frame_matrix <- function(names, rest, call = caller_env()) {
  list_cols <- which(map_lgl(rest, needs_list_col))
  if (has_length(list_cols)) {
    abort_frame_matrix_list(list_cols, call)
  }

  frame_ncol <- length(names)
  frame_mat <- matrix(unlist(rest), ncol = frame_ncol, byrow = TRUE)

  colnames(frame_mat) <- names
  frame_mat
}

subclass_tribble_c_errors <- function(name, code, call) {
  withCallingHandlers(
    code,
    vctrs_error = function(cnd) {
      abort_tribble_c(name, cnd, call)
    }
  )
}

# Errors ------------------------------------------------------------------

abort_tribble_needs_columns <- function(call = caller_env()) {
  tibble_abort(call = call, "Must specify at least one column using the `~name` syntax.")
}

abort_tribble_named_after_tilde <- function(call = caller_env()) {
  tibble_abort(call = call, "When using the `~name` syntax, subsequent values must not have names.")
}

abort_tribble_lhs_column_syntax <- function(lhs, call = caller_env()) {
  tibble_abort(call = call, problems(
    "All column specifications must use the `~name` syntax.",
    paste0("Found ", expr_label(lhs), " on the left-hand side of `~`.")
  ))
}

abort_tribble_rhs_column_syntax <- function(rhs, call = caller_env()) {
  tibble_abort(call = call, problems(
    'All column specifications must use the `~name` or `~"name"` syntax.',
    paste0("Found ", expr_label(rhs), " on the right-hand side of `~`.")
  ))
}

abort_tribble_non_rectangular <- function(cols, cells, call = caller_env()) {
  tibble_abort(call = call, bullets(
    "Data must be rectangular:",
    paste0("Found ", cols, " columns."),
    paste0("Found ", cells, " cells."),
    info = paste0(cells, " is not an integer multiple of ", cols, ".")
  ))
}

abort_frame_matrix_list <- function(pos, call = caller_env()) {
  tibble_abort(call = call, problems(
    "All values must be atomic:",
    pluralise_commas("Found list-valued element(s) at position(s) ", pos, ".")
  ))
}

abort_tribble_c <- function(name, cnd, call) {
  tibble_abort(
    paste0("Can't create column ", tick(name)),
    parent = cnd,
    call = call
  )
}

test_that("tribble() constructs 'tibble' as expected", {
  result <- tribble(
    ~colA, ~colB,
    "a", 1,
    "b", 2
  )

  compared <- tibble(colA = c("a", "b"), colB = c(1, 2))
  expect_equal(result, compared)

  ## wide
  wide <- tribble(
    ~colA, ~colB, ~colC, ~colD,
    1, 2, 3, 4,
    5, 6, 7, 8
  )

  wide_expectation <- tibble(
    colA = c(1, 5),
    colB = c(2, 6),
    colC = c(3, 7),
    colD = c(4, 8)
  )

  expect_equal(wide, wide_expectation)

  ## long
  long <- tribble(
    ~colA, ~colB,
    1, 6,
    2, 7,
    3, 8,
    4, 9,
    5, 10
  )

  long_expectation <- tibble(
    colA = as.numeric(1:5),
    colB = as.numeric(6:10)
  )

  expect_equal(long, long_expectation)
})

test_that("tribble() tolerates a trailing comma", {
  result <- tribble(
    ~colA, ~colB,
    "a", 1,
    "b", 2,
  )

  compared <- tibble(colA = c("a", "b"), colB = c(1, 2))
  expect_equal(result, compared)
})

test_that("tribble() handles columns with a class (#161)", {
  sys_date <- Sys.Date()
  sys_time <- Sys.time()
  date_time_col <- tribble(
    ~dt, ~dttm,
    sys_date, sys_time,
    as.Date("2003-01-02"), as.POSIXct("2004-04-05 13:45:17", tz = "UTC")
  )

  date_time_col_expectation <- tibble(
    dt = vec_c(sys_date, as.Date("2003-01-02")),
    dttm = vec_c(sys_time, as.POSIXct("2004-04-05 13:45:17", tz = "UTC"))
  )

  expect_equal(date_time_col, date_time_col_expectation)
})

test_that("tribble() creates lists for non-atomic inputs (#7)", {
  expect_identical(
    tribble(~a, ~b, NA, "A", letters, LETTERS[-1L]),
    tibble(a = list(NA, letters), b = list("A", LETTERS[-1L]))
  )

  expect_identical(
    tribble(~a, ~b, NA, NULL, 1, 2),
    tibble(a = c(NA, 1), b = list(NULL, 2))
  )
})

test_that("tribble() errs appropriately on bad calls", {
  # no colname
  expect_tibble_abort(
    tribble(1, 2, 3),
    abort_tribble_needs_columns()
  )

  # invalid colname syntax
  expect_tibble_abort(
    tribble(a ~ b),
    abort_tribble_lhs_column_syntax(quote(a))
  )

  # invalid colname syntax
  expect_tibble_abort(
    tribble(~ a + b),
    abort_tribble_rhs_column_syntax(quote(a + b))
  )

  # tribble() must be passed colnames
  expect_tibble_abort(
    tribble(
      "a", "b",
      1, 2
    ),
    abort_tribble_needs_columns()
  )

  # tribble() must produce rectangular structure (no filling)
  expect_tibble_abort(
    tribble(
      ~a, ~b, ~c,
      1, 2,
      3, 4, 5
    ),
    abort_tribble_non_rectangular(3, 5)
  )

  expect_tibble_abort(
    tribble(
      ~a, ~b, ~c, ~d,
      1, 2, 3, 4, 5,
      6, 7, 8, 9,
    ),
    abort_tribble_non_rectangular(4, 9)
  )
})

test_that("tribble can have list columns", {
  df <- tribble(
    ~x, ~y,
    1, list(a = 1),
    2, list(b = 2)
  )
  expect_equal(df$x, c(1, 2))
  expect_equal(df$y, list(list(a = 1), list(b = 2)))
})

test_that("tribble creates n-col empty data frame", {
  df <- tribble(~x, ~y)
  expect_equal(df, tibble(x = unspecified(), y = unspecified()))
})

test_that("tribble recognizes quoted non-formula call", {
  df <- tribble(
    ~x, ~y,
    quote(mean(1)), 1
  )
  expect_equal(df$x, list(quote(mean(1))))
  expect_equal(df$y, 1)
})

test_that("tribble returns 0x0 tibble when there's no argument", {
  df <- tribble()
  expect_equal(df, tibble())
})

test_that("names stripped at appropriate time (#775)", {
  expect_equal(
    tribble(~x, c(a = 1)),
    tibble(x = 1)
  )
})

test_that("lubridate::Period (#784)", {
  skip_if_not_installed("lubridate")
  expect_equal(
    tribble(~x, lubridate::days(1), lubridate::days(2)),
    tibble(x = lubridate::days(1:2))
  )
})

test_that("formattable (#785)", {
  skip_if_not_installed("formattable")
  expect_equal(
    tribble(~x, formattable::formattable(1.0, 1), formattable::formattable(2.0, 1)),
    tibble(x = formattable::formattable(1:2 + 0, 1))
  )
})

# ---- frame_matrix() ----

test_that("frame_matrix constructs a matrix as expected", {
  result <- frame_matrix(
    ~col1, ~col2,
    10, 3,
    5, 2
  )
  expected <- matrix(c(10, 5, 3, 2), ncol = 2)
  colnames(expected) <- c("col1", "col2")
  expect_equal(result, expected)
})

test_that("frame_matrix constructs empty matrix as expected", {
  result <- frame_matrix(
    ~col1, ~col2
  )
  expected <- matrix(logical(), ncol = 2)
  colnames(expected) <- c("col1", "col2")
  expect_equal(result, expected)
})

test_that("frame_matrix cannot have list columns", {
  expect_tibble_abort(
    frame_matrix(
      ~x,   ~y,
      "a", 1:3,
      "b", 4:6
    ),
    abort_frame_matrix_list(c(2, 4))
  )
})

test_that("tribble and frame_matrix cannot have named arguments", {
  expect_tibble_abort(
    extract_frame_data_from_dots(
      ~x, ~y,
      "a" = 1:3,
      "b" = 4:6
    ),
    abort_tribble_named_after_tilde()
  )
})

test_that("output test", {
  expect_snapshot(error = TRUE, {
    tribble(1)
    tribble(~a, ~b, 1)
    tribble(a ~ b, 1)
    tribble(a ~ b + c, 1)
    tribble(~b, 1, "a")

    frame_matrix(1)
    frame_matrix(~a, list(1))
  })
})

set_dftbl_hooks <- function() {
  width <- 36

  set_dftbl_opts_hook(width)
  set_dftbl_knit_hook(width)
  set_dftbl_source_hook()
  set_dftbl_chunk_hook()
}

# Defines a `dftbl` knitr option. If this chunk option is set, code is duplicated
# (running with tibbles or data frames, respectively): one line of df code
# followed by the same line of the corresponding tibble code.
# The code is also evaluated, if the results are identical (disregarding the
# class), only the tibble copy is retained.
#
set_dftbl_opts_hook <- function(width) {
  force(width)

  dftbl_opts_hook <- function(options) {
    df_code <- options$code
    tbl_code <- gsub("df", "tbl", df_code, fixed = TRUE)

    # FIXME: Evaluate, but surround in <details> element
    if (!isTRUE(options$dftbl_always) && isTRUE(options$eval)) {
      same <- map2_lgl(df_code, tbl_code, same_as_tbl_code)
      df_code[same] <- ""
    }

    new_code <- as.vector(t(matrix(c(df_code, tbl_code), ncol = 2)))
    options$code <- new_code
    options$width <- width - 4
    options
  }

  knitr::opts_hooks$set(dftbl = dftbl_opts_hook)
}

utils::globalVariables(c("new_df", "new_tbl"))

same_as_tbl_code <- function(df_code, tbl_code) {
  handler <- evaluate::new_output_handler(
    value = function(x, visible) {
      if (visible) x else NULL
    }
  )

  same_as_tbl(
    evaluate::evaluate(df_code, output_handler = handler),
    evaluate::evaluate(tbl_code, output_handler = handler)
  )
}

same_as_tbl <- function(df, tbl) {
  if (length(df) != length(tbl)) return(FALSE)
  if (length(df) < 2) return(FALSE)
  df <- df[-1]
  tbl <- tbl[-1]

  df_obj <- df[[length(df)]]
  tbl_obj <- tbl[[length(tbl)]]

  if (is.data.frame(df_obj) != is.data.frame(tbl_obj)) return(FALSE)

  if (is.data.frame(tbl_obj)) {
    df[[length(df)]] <- as_tibble_deep(df_obj)
  }

  identical(df, tbl)
}

as_tibble_deep <- function(x) {
  is_tibble <- which(map_lgl(x, is.data.frame))
  x[is_tibble] <- map(x[is_tibble], as_tibble)
  as_tibble(x)
}

# dftbl chunks have a reduced width
set_dftbl_knit_hook <- function(width) {
  force(width)

  # Need to use a closure here to keep state
  old_width <- NULL

  dftbl_knit_hook <- function(before, options, envir) {
    if (before) {
      old_width <<- options(width = width)
    } else {
      options(old_width)
      old_width <<- NULL
    }
  }

  knitr::knit_hooks$set(dftbl = dftbl_knit_hook)
}

# dftbl chunks are shown side by side, with the help of an HTML table.
# Each source chunk introduces a new table cell, even chunks also introduce
# a new table row.
# vertical-align: top keeps the table rows nicely aligned.
# This places some limitations on the chunk sources but works well so far.
set_dftbl_source_hook <- function() {
  # Need to use a closure here to daisy-chain hooks and to keep state

  old_source_hook <- knitr::knit_hooks$get("source")

  dftbl_source_even <- TRUE

  dftbl_source_hook_one <- function(x) {
    if (dftbl_source_even) {
      x <- paste0('</td></tr><tr style="vertical-align:top"><td>\n\n', x)
    } else {
      x <- paste0("</td><td>\n\n", x)
    }

    dftbl_source_even <<- !dftbl_source_even
    x
  }

  dftbl_source_hook <- function(x, options) {
    nonempty <- which(x != "")
    x[nonempty] <- vapply(x[nonempty], old_source_hook, options, FUN.VALUE = character(1))
    if (isTRUE(options$dftbl)) {
      x <- vapply(x, dftbl_source_hook_one, FUN.VALUE = character(1))
    }
    paste(x, collapse = "\n")
  }

  knitr::knit_hooks$set(source = dftbl_source_hook)
}

# The entire chunk needs to be surrounded by <table><tbody><tr><td>...</...> .
# We use the dftbl CSS class for the HTML table.
set_dftbl_chunk_hook <- function() {
  # Need to use a closure here to daisy-chain hooks

  old_chunk_hook <- knitr::knit_hooks$get("chunk")

  dftbl_chunk_hook <- function(x, options) {
    x <- old_chunk_hook(x, options)
    if (isTRUE(options$dftbl)) {
      x <- paste0('<table class="dftbl"><tbody><tr><td>\n\n', x, "\n\n</td></tr></tbody></table>")
      x <- gsub("<tr><td>\n\n</td></tr>", "", x, fixed = TRUE)
    }
    x
  }

  knitr::knit_hooks$set(chunk = dftbl_chunk_hook)
}

print_without_body <- function(x, ...) {
  class(x) <- c("tbl_df_without_body", class(x))
  print(x, ...)
}

tbl_format_body.tbl_df_without_body <- function(x, ...) {
  "<body created by pillar>"
}

# Need explicit method because can't be found in method env
vctrs::s3_register(
  "pillar::tbl_format_body", "tbl_df_without_body",
  tbl_format_body.tbl_df_without_body
)

if (getRversion() >= "3.6") {
  local_options(
    warnPartialMatchArgs = TRUE,
    warnPartialMatchAttr = TRUE,
    warnPartialMatchDollar = TRUE,
    .frame = testthat::teardown_env()
  )
} else {
  local_options(
    warnPartialMatchAttr = TRUE,
    warnPartialMatchDollar = TRUE,
    .frame = testthat::teardown_env()
  )
}

#' View an object
#'
#' @description
#' `r lifecycle::badge("experimental")`
#'
#' Calls [utils::View()] on the input and returns it, invisibly.
#' If the input is not a data frame, it is processed using a variant of
#' `as.data.frame(head(x, n))`.
#' A message is printed if the number of rows exceeds `n`.
#' This function has no effect in non[interactive] sessions.
#'
#' @details
#' The RStudio IDE overrides `utils::View()`, this is picked up
#' correctly.
#'
#' @param x The object to display.
#' @param title The title to use for the display, by default
#'   the deparsed expression is used.
#' @param ... Unused, must be empty.
#' @param n Maximum number of rows to display. Only used if `x` is not a
#'   data frame. Uses the `view_max` [option][tibble_options] by default.
#'
#' @export
view <- function(x, title = NULL, ..., n = NULL) {
  check_dots_empty()

  if (!is_interactive()) {
    return(invisible(x))
  }

  # The user's expression and the environment to re-evaluate it in
  quo <- enquo0(x)
  expr <- quo_get_expr(quo)
  env <- quo_get_env(quo)

  if (is.null(title)) {
    title <- as_label(expr)
  }

  # Retrieve the `View()` function, which includes the special
  # hooks created by RStudio or Positron
  fn <- get("View", envir = as.environment("package:utils"))

  if (!is.data.frame(x)) {
    return(view_with_coercion(x, n, title, fn))
  }

  # Make a `View()` call that we evaluate in the parent frame,
  # as if the user called `View()` directly rather than `view()`.
  # If `expr` directly references a data frame in the parent frame, this
  # allows RStudio and Positron to "track" that original object
  # for live updates in the data viewer.
  inject((!!fn)(!!expr, !!title), env = env)

  invisible(x)
}

view_with_coercion <- function(x, n, title, fn) {
  if (is.null(n)) {
    n <- get_tibble_option_view_max()
  }

  x <- head(x, n + 1)
  x <- as.data.frame(x)

  if (nrow(x) > n) {
    message("Showing the first ", n, " rows.")
    x <- vec_slice(x, seq_len(n))
  }

  # Since we just created `x`, there won't be anything for
  # RStudio or Positron to "track", so don't even make an effort
  # to try and evaluate in the parent frame with the original
  # expression
  fn(x, title)

  invisible(x)
}

#' Subsetting tibbles
#'
#' @description
#' Accessing columns, rows, or cells via `$`, `[[`, or `[` is mostly similar to
#' [regular data frames][base::Extract]. However, the
#' behavior is different for tibbles and data frames in some cases:
#' * `[` always returns a tibble by default, even if
#'   only one column is accessed.
#' * Partial matching of column names with `$` and `[[` is not supported, and
#'   `NULL` is returned.
#'   For `$`, a warning is given.
#' * Only scalars (vectors of length one) or vectors with the
#'   same length as the number of rows can be used for assignment.
#' * Rows outside of the tibble's boundaries cannot be accessed.
#' * When updating with `[[<-` and `[<-`, type changes of entire columns are
#'   supported, but updating a part of a column requires that the new value is
#'   coercible to the existing type.
#'   See [vec_slice()] for the underlying implementation.
#'
#' Unstable return type and implicit partial matching can lead to surprises and
#' bugs that are hard to catch. If you rely on code that requires the original
#' data frame behavior, coerce to a data frame via [as.data.frame()].
#'
#' @details
#' For better compatibility with older code written for regular data frames,
#' `[` supports a `drop` argument which defaults to `FALSE`.
#' New code should use `[[` to turn a column into a vector.
#'
#' @param x A tibble.
#' @param value A value to store in a row, column, range or cell.
#'   Tibbles are stricter than data frames in what is accepted here.
#'
#' @name subsetting
#' @examples
#' df <- data.frame(a = 1:3, bc = 4:6)
#' tbl <- tibble(a = 1:3, bc = 4:6)
#'
#' # Subsetting single columns:
#' df[, "a"]
#' tbl[, "a"]
#' tbl[, "a", drop = TRUE]
#' as.data.frame(tbl)[, "a"]
#'
#' # Subsetting single rows with the drop argument:
#' df[1, , drop = TRUE]
#' tbl[1, , drop = TRUE]
#' as.list(tbl[1, ])
#'
#' @examplesIf (Sys.getenv("NOT_CRAN") != "true" || Sys.getenv("IN_PKGDOWN") == "true")
#' # Accessing non-existent columns:
#' df$b
#' tbl$b
#'
#' df[["b", exact = FALSE]]
#' tbl[["b", exact = FALSE]]
#'
#' df$bd <- c("n", "e", "w")
#' tbl$bd <- c("n", "e", "w")
#' df$b
#' tbl$b
#' @examples
#'
#' df$b <- 7:9
#' tbl$b <- 7:9
#' df$b
#' tbl$b
#'
#' # Identical behavior:
#' tbl[1, ]
#' tbl[1, c("bc", "a")]
#' tbl[, c("bc", "a")]
#' tbl[c("bc", "a")]
#' tbl["a"]
#' tbl$a
#' tbl[["a"]]
NULL

#' @rdname subsetting
#' @param name A [name] or a string.
#' @export
`$.tbl_df` <- function(x, name) {
  out <- .subset2(x, name)
  if (is.null(out)) {
    warn(paste0("Unknown or uninitialised column: ", tick(name), "."))
  }
  out
}

#' @rdname subsetting
#' @param i,j Row and column indices. If `j` is omitted, `i` is used as column index.
#' @param ... Ignored.
#' @param exact Ignored, with a warning.
#' @export
`[[.tbl_df` <- function(x, i, j, ..., exact = TRUE) {
  if (!exact) {
    warn("`exact` ignored.")
  }

  n_dots <- dots_n(...)
  if (n_dots > 0) {
    warn("Extra arguments ignored.")
  }

  # Ignore exact as an argument for counting
  n_real_args <- nargs() - !missing(exact) - n_dots

  # Column subsetting if nargs() == 2L
  if (n_real_args <= 2L) {
    if (missing(i)) {
      abort_subset_columns_non_missing_only()
    }
    tbl_subset2(x, j = i, j_arg = substitute(i))
  } else if (missing(j) || missing(i)) {
    abort_subset_columns_non_missing_only()
  } else {
    i_arg <- substitute(i)
    i <- vectbl_as_row_location2(i, fast_nrow(x), i_arg)
    x <- tbl_subset2(x, j = j, j_arg = substitute(j))

    if (is.null(x)) {
      x
    } else {
      # Drop inner names with double subscript
      vectbl_set_names(vec_slice(x, i), NULL)
    }
  }
}

#' @rdname subsetting
#' @param drop Coerce to a vector if fetching one column via `tbl[, j]` .
#'   Default `FALSE`, ignored when accessing a column via `tbl[j]` .
#' @export
`[.tbl_df` <- function(x, i, j, drop = FALSE, ...) {
  i_arg <- substitute(i)
  j_arg <- substitute(j)

  if (missing(i)) {
    i <- NULL
    i_arg <- NULL
  } else if (is.null(i)) {
    i <- integer()
  }

  if (missing(j)) {
    j <- NULL
    j_arg <- NULL
  } else if (is.null(j)) {
    j <- integer()
  }

  # Ignore drop as an argument for counting
  n_real_args <- nargs() - !missing(drop)

  # Column or matrix subsetting if nargs() == 2L
  if (n_real_args <= 2L) {
    if (!missing(drop)) {
      warn("`drop` argument ignored for subsetting a tibble with `x[j]`, it has an effect only for `x[i, j]`.")
      drop <- FALSE
    }

    j <- i
    i <- NULL
    j_arg <- i_arg
    i_arg <- NULL

    # Special case, returns a vector:
    if (is.matrix(j)) {
      return(tbl_subset_matrix(x, j, j_arg))
    }
  }

  # From here on, i, j and drop contain correct values:
  if (is.null(j)) {
    xo <- x
  } else {
    j <- vectbl_as_col_location(j, length(x), names(x), j_arg = j_arg, assign = FALSE)

    xo <- .subset(x, j)

    if (anyDuplicated.default(j)) {
      xo <- set_repaired_names(xo, repair_hint = FALSE, .name_repair = "minimal")
    }
  }

  if (is.null(i)) {
    nrow <- fast_nrow(x)
  } else {
    i <- vectbl_as_row_index(i, x, i_arg)
    xo <- lapply(xo, vec_slice, i = i)
    nrow <- length(i)
  }

  if (drop && length(xo) == 1L) {
    tbl_subset2(xo, 1L, j_arg)
  } else {
    attr(xo, "row.names") <- .set_row_names(nrow)
    vectbl_restore(xo, x)
  }
}

tbl_subset2 <- function(x, j, j_arg, call = caller_env()) {
  if (is.matrix(j)) {
    deprecate_stop("3.0.0", "tibble::`[[.tbl_df`(j = 'can\\'t be a matrix')",
      details = "Recursive subsetting is deprecated for tibbles.",
      env = foreign_caller_env()
    )
  }

  if (is.object(j)) {
    j <- vectbl_as_col_subscript2(j, j_arg, call = call)
  }

  if (is.numeric(j)) {
    if (length(j) == 1L) {
      if (is.na(j) || j < 1 || j > length(x) || (is.double(j) && j != trunc(j))) {
        # Side effect: throw error for invalid j
        vectbl_as_col_location2(j, length(x), j_arg = j_arg, call = call)
      }
    } else if (length(j) == 2L) {
      deprecate_stop("3.0.0", "tibble::`[[.tbl_df`(j = 'can\\'t be a vector of length 2')",
        details = "Recursive subsetting is deprecated for tibbles.",
        env = foreign_caller_env()
      )
    } else {
      # Side effect: throw error for invalid j
      vectbl_as_col_location2(j, length(x), j_arg = j_arg, call = call)
    }
  } else if (is.symbol(j)) {
    # FIXME: Only relevant for R < 3.4
    j <- as.character(j)
  } else if (is.logical(j) || length(j) != 1L || !is_bare_atomic(j) || is.na(j)) {
    # Side effect: throw error for invalid j
    vectbl_as_col_location2(j, length(x), names(x), j_arg = j_arg, call = call)
  }

  .subset2(x, j)
}

vectbl_as_col_subscript2 <- function(j, j_arg, assign = FALSE, call = caller_env()) {
  subclass_col_index_errors(
    vec_as_subscript2(j, logical = "error", call = call),
    j_arg = j_arg,
    assign = assign
  )
}

vectbl_as_col_location2 <- function(j, n, names = NULL, j_arg, assign = FALSE, call = caller_env()) {
  subclass_col_index_errors(
    vec_as_location2(j, n, names, call = call),
    j_arg = j_arg,
    assign = assign
  )
}

vectbl_as_row_location2 <- function(i, n, i_arg, assign = FALSE, call = caller_env()) {
  subclass_row_index_errors(
    vec_as_location2(i, n, call = call),
    i_arg = i_arg,
    assign = assign
  )
}

vectbl_set_names <- function(x, names = NULL) {
  # Work around https://github.com/r-lib/vctrs/issues/1419
  if (inherits(x, "vctrs_rcrd")) {
    # A rcrd can't have names?
    return(x)
  }
  vec_set_names(x, names)
}

vectbl_as_col_location <- function(j,
                                   n,
                                   names = NULL,
                                   j_arg,
                                   assign = FALSE,
                                   call = caller_env()) {
  subclass_col_index_errors(
    vec_as_location(j, n, names, missing = "error", call = call),
    j_arg = j_arg,
    assign = assign
  )
}

vectbl_as_row_index <- function(i, x, i_arg, assign = FALSE, call = caller_env()) {
  stopifnot(!is.null(i))

  nr <- fast_nrow(x)

  if (is.character(i)) {
    is_na_orig <- is.na(i)

    if (has_rownames(x)) {
      i <- match(i, rownames(x))
    } else {
      i <- string_to_indices(i)
      i <- fix_oob(i, nr, warn = FALSE)
    }

    i <- fix_oob_invalid(i, is_na_orig)
    i
  } else if (is.numeric(i)) {
    i <- fix_oob(i, nr)
    vectbl_as_row_location(i, nr, i_arg, assign, call)
  } else {
    vectbl_as_row_location(i, nr, i_arg, assign, call)
  }
}

fix_oob <- function(i, n, warn = TRUE) {
  if (any(i > 0, na.rm = TRUE)) {
    fix_oob_positive(i, n, warn)
  } else if (any(i < 0, na.rm = TRUE)) {
    fix_oob_negative(i, n, warn)
  } else {
    # Will throw error in vec_as_location()
    i
  }
}

fix_oob_positive <- function(i, n, warn = TRUE) {
  oob <- which(i > n)
  if (warn && length(oob) > 0) {
    deprecate_soft("3.0.0", "tibble::`[.tbl_df`(i = 'must lie in [0, rows] if positive,')",
      details = "Use `NA_integer_` as row index to obtain a row full of `NA` values.",
      env = foreign_caller_env()
    )
  }

  i[oob] <- NA_integer_
  i
}

fix_oob_negative <- function(i, n, warn = TRUE) {
  oob <- (i < -n)
  if (warn && any(oob, na.rm = TRUE)) {
    deprecate_soft("3.0.0", "tibble::`[.tbl_df`(i = 'must lie in [-rows, 0] if negative,')",
      details = "Use `NA_integer_` as row index to obtain a row full of `NA` values.",
      env = foreign_caller_env()
    )
  }

  i <- i[!oob]
  if (is_empty(i)) i <- seq_len(n)
  i
}

fix_oob_invalid <- function(i, is_na_orig) {
  oob <- which(is.na(i) & !is_na_orig)

  if (length(oob) > 0) {
    deprecate_soft("3.0.0", "tibble::`[.tbl_df`(i = 'must use valid row names')",
      details = "Use `NA_integer_` as row index to obtain a row full of `NA` values.",
      env = foreign_caller_env()
    )

    i[oob] <- NA_integer_
  }
  i
}

fast_nrow <- function(x) {
  .row_names_info(x, 2L)
}

# External ----------------------------------------------------------------

vectbl_restore <- function(xo, x) {
  .Call(`tibble_restore_impl`, xo, x)
}

# Errors ------------------------------------------------------------------

abort_subset_columns_non_missing_only <- function(call = caller_env()) {
  tibble_abort(call = call, "Subscript can't be missing for tibbles in `[[`.")
}

# Subclassing errors ------------------------------------------------------

subclass_col_index_errors <- function(expr, j_arg, assign) {
  withCallingHandlers(
    expr,
    vctrs_error_subscript = function(cnd) {
      cnd$subscript_arg <- j_arg
      cnd$subscript_elt <- "column"
      if (isTRUE(assign) && !isTRUE(cnd$subscript_action %in% c("negate"))) {
        cnd$subscript_action <- "assign"
      }
      cnd_signal(cnd)
    }
  )
}

subclass_row_index_errors <- function(expr, i_arg, assign) {
  withCallingHandlers(
    expr,
    vctrs_error_subscript = function(cnd) {
      cnd$subscript_arg <- i_arg
      cnd$subscript_elt <- "row"
      if (isTRUE(assign) && !isTRUE(cnd$subscript_action %in% c("negate"))) {
        cnd$subscript_action <- "assign"
      }
      cnd_signal(cnd)
    }
  )
}

invalid_df <- function(problem, vars, extra = NULL, message = "Column(s)") {
  if (is.character(vars)) {
    vars <- tick(vars)
  }

  c(
    pluralise_commas(paste0(message, " "), vars, paste0(" ", problem, ".")),
    extra
  )
}

use_repair <- function(repair_hint) {
  if (repair_hint) "Use `.name_repair` to specify repair."
}

tibble_error_class <- function(class) {
  c(paste0("tibble_error_", class), "tibble_error")
}

# Errors get a class name derived from the name of the calling function
tibble_abort <- function(x, ..., call, parent = NULL) {
  abort_call <- sys.call(-1)
  fn_name <- as_name(abort_call[[1]])
  class <- tibble_error_class(gsub("^abort_", "", fn_name))

  abort(x, class, ..., call = call, parent = parent, use_cli_format = TRUE)
}

tibble_error <- function(x, ..., parent = NULL) {
  call <- sys.call(-1)
  fn_name <- as_name(call[[1]])
  class <- tibble_error_class(gsub("^error_", "", fn_name))
  error_cnd(class, ..., message = x, parent = parent, use_cli_format = TRUE)
}

test_that("tibble_error()", {
  # Must be called from a function whose name starts with `error_`
  error_foo <- function() {
    tibble_error("message", foo = 42, bar = 7)
  }
  expect_identical(
    error_foo(),
    error_cnd(
      .subclass = c("tibble_error_foo", "tibble_error"),
      message = "message",
      foo = 42,
      bar = 7
    )
  )
})

test_that("output test", {
  expect_snapshot({
    invalid_df("must be integer", "col", "\nFix this.")
    invalid_df("must be numeric", c("col1", "col2"))

    use_repair(TRUE)
    use_repair(FALSE)
  })
})

test_that("tibble_error()", {
  # Must be called from a function whose name starts with `error_`
  error_foo <- function() {
    tibble_error("message", foo = 42, bar = 7)
  }
  expect_identical(
    error_foo(),
    error_cnd(
      class = c("tibble_error_foo", "tibble_error"),
      message = "message",
      foo = 42,
      bar = 7
    )
  )
})

test_that("output test", {
  expect_snapshot({
    invalid_df("must be integer", "col", "\nFix this.")
    invalid_df("must be numeric", c("col1", "col2"))

    use_repair(TRUE)
    use_repair(FALSE)
  })
})

#' @rdname subsetting
#' @export
`$<-.tbl_df` <- function(x, name, value) {
  tbl_subassign(
    x,
    i = NULL, as_string(name), list(value),
    i_arg = NULL, j_arg = name, value_arg = substitute(value)
  )
}

#' @rdname subsetting
#' @export
`[[<-.tbl_df` <- function(x, i, j, ..., value) {
  i_arg <- substitute(i)
  j_arg <- substitute(j)
  value_arg <- substitute(value)

  if (missing(i)) {
    abort_assign_columns_non_missing_only()
  }

  if (missing(j)) {
    if (nargs() < 4) {
      j <- i
      i <- NULL
      j_arg <- i_arg
      i_arg <- NULL
    } else {
      abort_assign_columns_non_missing_only()
    }
  }

  if (!is.null(i)) {
    i <- vectbl_as_row_location2(i, fast_nrow(x), i_arg, assign = TRUE)
  }

  if (is.object(j)) {
    j <- vectbl_as_col_subscript2(j, j_arg, assign = TRUE)
  }

  # Side effect: check scalar
  if (is.symbol(j)) {
    # FIXME: as_utf8_character() needs rlang > 0.4.11
    j <- chr_unserialise_unicode(as.character(j))
  } else {
    if (!is.vector(j) || length(j) != 1L || is.na(j) || (is.numeric(j) && j < 0) || is.logical(j)) {
      vectbl_as_col_location2(j, length(x), j_arg = j_arg, assign = TRUE)
    }
  }

  j <- vectbl_as_new_col_index(j, x, j_arg)

  # New columns are added to the end, provide index to avoid matching column
  # names again
  value <- list(value)

  # j is already pretty
  tbl_subassign(x, i, j, value, i_arg = i_arg, j_arg = NULL, value_arg = value_arg)
}


#' @rdname subsetting
#' @export
`[<-.tbl_df` <- function(x, i, j, ..., value) {
  i_arg <- substitute(i)
  j_arg <- substitute(j)

  if (missing(i)) {
    i <- NULL
    i_arg <- NULL
  } else if (is.null(i)) {
    i <- integer()
  }

  if (missing(j)) {
    j <- NULL
    j_arg <- NULL
  } else if (is.null(j)) {
    j <- integer()
  }

  if (is.null(j) && nargs() < 4) {
    j <- i
    i <- NULL
    j_arg <- i_arg
    i_arg <- NULL

    # Special case:
    if (is.matrix(j)) {
      return(tbl_subassign_matrix(x, j, value, j_arg, substitute(value)))
    }
  }

  tbl_subassign(x, i, j, value, i_arg, j_arg, substitute(value))
}

vectbl_as_new_row_index <- function(i, x, i_arg, call) {
  if (is.null(i)) {
    i
  } else if (is_bare_numeric(i)) {
    if (anyDuplicated.default(i)) {
      abort_assign_duplicate_row_subscript(i, call)
    }

    nr <- fast_nrow(x)

    # Only update existing, caller knows how to deal with OOB
    numtbl_as_row_location_assign(i, nr, i_arg, call)
  } else if (is_logical(i)) {
    # Don't allow OOB logical
    vectbl_as_row_location(i, fast_nrow(x), i_arg, assign = TRUE, call = call)
  } else {
    i <- vectbl_as_row_index(i, x, i_arg, assign = TRUE, call = call)
    if (anyDuplicated.default(i, incomparables = NA)) {
      abort_assign_duplicate_row_subscript(i, call)
    }
    i
  }
}

vectbl_as_new_col_index <- function(j, x, j_arg, names = "", value_arg = NULL, call = caller_env()) {
  # Creates a named index vector
  # Values: index
  # Name: column name (for all columns)

  if (is.object(j)) {
    j <- vectbl_as_col_subscript(j, j_arg = j_arg, assign = TRUE, call = call)
  }

  if (is.character(j)) {
    if (anyNA(j)) {
      abort_assign_columns_non_na_only(call)
    }

    names <- j

    j <- match(names, names(x))
    new <- which(is.na(j))
    if (length(new) > 0) {
      # FIXME: Check consistency with assigning to the same existing column twice
      j[new] <- seq.int(length(x) + 1L, length.out = length(new))
    } else {
      new <- NULL
    }
  } else if (is.numeric(j)) {
    if (anyNA(j)) {
      abort_assign_columns_non_na_only(call)
    }

    j <- numtbl_as_col_location_assign(j, length(x), j_arg, call)

    old <- (j <= length(x))
    new <- which(!old)
    j_new <- j[new]

    # FIXME: Recycled names are not repaired
    # FIXME: Hard-coded name repair
    if (length(names) != 1L) {
      # Side effect: check compatibility
      vec_recycle(names, length(j), x_arg = as_label(value_arg), call = call)
    } else if (length(j) != 1L) {
      # length(names) == 1
      names <- vec_recycle(names, length(j), x_arg = as_label(value_arg))
    }

    if (length(new) > 0) {
      j[new] <- j_new
      names[new][names[new] == ""] <- paste0("...", j_new)
    } else {
      new <- NULL
    }

    names[old] <- names(x)[j[old]]
  } else {
    j <- vectbl_as_col_location(j, length(x), names(x), j_arg = j_arg, assign = TRUE, call = call)

    if (length(names) != 1L) {
      # Side effect: check compatibility
      vec_recycle(names, length(j), x_arg = as_label(value_arg), call = call)
    } else if (length(j) != 1L) {
      # length(names) == 1
      names <- vec_recycle(names, length(j), x_arg = as_label(value_arg))
    }

    old <- (j <= length(x))
    names[old] <- names(x)[j[old]]

    new <- NULL
  }

  if (anyDuplicated.default(j)) {
    abort_assign_duplicate_column_subscript(j, call)
  }

  names(j) <- names
  attr(j, "new") <- new
  j
}

vectbl_as_col_subscript <- function(j, j_arg, assign = FALSE, call = caller_env()) {
  subclass_col_index_errors(
    vec_as_subscript(j, call = call),
    j_arg = j_arg,
    assign = assign
  )
}

numtbl_as_row_location_assign <- function(i, n, i_arg, call) {
  subclass_row_index_errors(
    num_as_location(i, n, missing = "error", oob = "extend", zero = "error", call = call),
    i_arg = i_arg, assign = TRUE
  )
}

vectbl_as_row_location <- function(i, n, i_arg, assign = FALSE, call) {
  if (is_bare_atomic(i) && is.matrix(i) && ncol(i) == 1) {
    what <- paste0(
      "tibble::", if (assign) "`[<-`" else "`[`",
      "(i = 'can\\'t be a matrix')"
    )

    lifecycle::deprecate_soft("3.0.0", what,
      details = "Convert to a vector.",
      env = foreign_caller_env()
    )
    i <- i[, 1]
  }

  subclass_row_index_errors(
    vec_as_location(i, n, missing = if (assign) "error" else "propagate", call = call),
    i_arg = i_arg,
    assign = assign
  )
}

numtbl_as_col_location_assign <- function(j, n, j_arg, call) {
  subclass_col_index_errors(
    num_as_location(j, n, missing = "error", oob = "extend", zero = "error", call = call),
    j_arg = j_arg, assign = TRUE
  )
}

result_vectbl_wrap_rhs <- function(value) {
  if (!vec_is(value)) {
    NULL
  } else if (is.list(value)) {
    # Also covers the case of data frames
    unclass(value)
  } else if (is.array(value)) {
    if (any(dim(value)[-1:-2] != 1)) {
      return(NULL)
    }
    dim(value) <- head(dim(value), 2)
    as.list(as.data.frame(value, stringsAsFactors = FALSE))
  } else {
    list(value)
  }
}

# Errors ------------------------------------------------------------------

abort_need_rhs_vector <- function(value_arg, call = caller_env()) {
  tibble_abort(call = call, paste0(tick(as_label(value_arg)), " must be a vector, a bare list, a data frame or a matrix."))
}

abort_need_rhs_vector_or_null <- function(value_arg, call = caller_env()) {
  tibble_abort(call = call, paste0(tick(as_label(value_arg)), " must be a vector, a bare list, a data frame, a matrix, or NULL."))
}

abort_assign_columns_non_na_only <- function(call = caller_env()) {
  tibble_abort(call = call, "Can't use NA as column index in a tibble for assignment.")
}

abort_assign_columns_non_missing_only <- function(call = caller_env()) {
  tibble_abort(call = call, "Subscript can't be missing for tibbles in `[[<-`.")
}

abort_assign_duplicate_column_subscript <- function(j, call = caller_env()) {
  j <- unique(j[duplicated(j)])
  tibble_abort(call = call, pluralise_commas("Column index(es) ", j, " [is](are) used more than once for assignment."), j = j)
}

abort_assign_rows_non_na_only <- function(call = caller_env()) {
  tibble_abort(call = call, "Can't use NA as row index in a tibble for assignment.")
}

abort_assign_duplicate_row_subscript <- function(i, call = caller_env()) {
  i <- unique(i[duplicated(i)])
  tibble_abort(call = call, pluralise_commas("Row index(es) ", i, " [is](are) used more than once for assignment."), i = i)
}

abort_assign_incompatible_size <- function(nrow, value, j, i_arg, value_arg, parent = NULL, call = caller_env()) {
  if (is.null(i_arg)) {
    target <- "existing data"
    existing <- pluralise_count("Existing data has ", nrow, " row(s)")
  } else {
    target <- paste0("row subscript ", tick(as_label(i_arg)))
    existing <- pluralise_count("", nrow, " row(s) must be assigned")
  }

  new <- paste0(pluralise_count("has ", vec_size(value[[j]]), " row(s)"))
  if (length(value) != 1) {
    new <- paste0("Element ", j, " of assigned data ", new)
  } else {
    new <- paste0("Assigned data ", new)
  }

  tibble_abort(
    bullets(
      paste0("Assigned data ", tick(as_label(value_arg)), " must be compatible with ", target, ":"),
      x = existing,
      x = new,
      i = if (nrow != 1) "Only vectors of size 1 are recycled",
      i = if (nrow == 1 && vec_size(value[[j]]) != 1) "Row updates require a list value. Do you need `list()` or `as.list()`?"
    ),
    expected = nrow,
    actual = vec_size(value[[j]]),
    j = j,
    parent = parent,
    call = call
  )
}

abort_assign_incompatible_type <- function(x, value, j, value_arg, parent = NULL, call = caller_env()) {
  name <- names(x)[[j]]

  tibble_abort(
    bullets(
      paste0("Assigned data ", tick(as_label(value_arg)), " must be compatible with existing data:"),
      i = paste0("Error occurred for column ", tick(name))
    ),
    expected = x[[j]],
    actual = value[[j]],
    name = name,
    j = j,
    parent = parent,
    call = call
  )
}

abort_assign_vector <- function(value, j, value_arg, parent = NULL, call = caller_env()) {
  tibble_abort(
    paste0("Assigned data ", tick(as_label(value_arg)), " must be a vector."),
    actual = value[[j]],
    j = j,
    parent = parent,
    call = call
  )
}

#' Main function for subset assignment
#'
#' Powers $<-, [[<- and [<- for tibbles.
#'
#' @param x A tibble.
#' @param i An integer vector, or `NULL` to replace all rows.
#' @param j A named integer vector, or `NULL` to replace all columns.
#'   The names are required to define the names for new columns.
#'   In addition, if new columns are created, their names must be stored
#'   in the `"new"` attribute, for performance.
#' @param value A list of length one or of the same length as `j`.
#'   `NULL` elements indicate column removal.
#' @param i_arg,value_arg Argument names.
#' @noRd
tbl_subassign <- function(x, i, j, value, i_arg, j_arg, value_arg, call = caller_env()) {
  if (is.null(i)) {
    xo <- unclass(x)

    if (is.null(value)) {
      value <- list(value)
    } else {
      value <- result_vectbl_wrap_rhs(value)
      if (is.null(value)) {
        abort_need_rhs_vector_or_null(value_arg, call)
      }
    }

    if (is.null(j)) {
      j <- seq_along(xo)
      names(j) <- names2(j)
    } else if (!is.null(j_arg)) {
      j <- vectbl_as_new_col_index(j, xo, j_arg, names2(value), value_arg, call = call)
    }

    value <- vectbl_recycle_rhs_rows(value, fast_nrow(xo), i_arg = NULL, value_arg, call)
    value <- vectbl_recycle_rhs_cols(value, length(j), call)

    xo <- tbl_subassign_col(xo, j, value)
  } else if (is.null(i_arg)) {
    # x[NULL, ...] <- value
    return(x)
  } else {
    i <- vectbl_as_new_row_index(i, x, i_arg, call = call)

    # Fill up rows first if necessary
    x <- tbl_expand_to_nrow(x, i, call)
    value <- result_vectbl_wrap_rhs(value)
    if (is.null(value)) {
      abort_need_rhs_vector(value_arg, call)
    }

    # Only after tbl_expand_to_nrow() which needs data frame
    xo <- unclass(x)

    if (is.null(j)) {
      xo <- tbl_subassign_row(xo, i, value, i_arg, value_arg, call)
    } else {
      # Optimization: match only once
      # (Invariant: x[[j]] is equivalent to x[[vec_as_location(j)]],
      # allowed by corollary that only existing columns can be updated)
      if (!is.null(j_arg)) {
        j <- vectbl_as_new_col_index(j, xo, j_arg, names2(value), value_arg, call = call)
      }

      # Fill up columns if necessary
      new <- attr(j, "new")
      if (!is.null(new)) {
        init <- map(value[new], vec_slice, rep(NA_integer_, fast_nrow(xo)))
        xo <- tbl_subassign_col(xo, j[new], init)
      }

      xj <- .subset(xo, j)
      xj <- tbl_subassign_row(xj, i, value, i_arg, value_arg, call)
      xo <- tbl_subassign_col(xo, j, xj)
    }
  }

  vectbl_restore(xo, x)
}

vectbl_recycle_rhs_rows <- function(value, nrow, i_arg, value_arg, call) {
  if (length(value) > 0L) {
    withCallingHandlers(
      for (j in seq_along(value)) {
        if (!is.null(value[[j]])) {
          value[[j]] <- vec_recycle(value[[j]], nrow)
        }
      },
      vctrs_error_recycle_incompatible_size = function(cnd) {
        abort_assign_incompatible_size(nrow, value, j, i_arg, value_arg, cnd, call = call)
      },
      vctrs_error_scalar_type = function(cnd) {
        abort_assign_vector(value, j, value_arg, cnd, call = call)
      }
    )
  }

  value
}

vectbl_recycle_rhs_cols <- function(value, ncol, call) {
  if (length(value) != 1L || ncol != 1L) {
    # Errors have been caught beforehand in vectbl_as_new_col_index()
    value <- vec_recycle(value, ncol, call = call)
  }

  value
}

tbl_subassign_col <- function(x, j, value) {
  nrow <- fast_nrow(x)

  # Grow, assign new names
  new <- attr(j, "new")
  if (!is.null(new)) {
    length(x) <- max(j[new])
    names(x)[j[new]] <- names2(j)[new]
  }

  # Update
  to_remove <- integer()
  for (jj in seq_along(value)) {
    ji <- j[[jj]]
    value_jj <- value[[jj]]
    if (!is.null(value_jj)) {
      x[[ji]] <- value_jj
    } else {
      to_remove <- c(to_remove, ji)
    }
  }

  # Remove
  if (length(to_remove) > 0) {
    x <- x[-to_remove]
  }

  # Can be destroyed by setting length
  attr(x, "row.names") <- .set_row_names(nrow)
  x
}

tbl_expand_to_nrow <- function(x, i, call = caller_env()) {
  nrow <- fast_nrow(x)

  new_nrow <- max(i, nrow)

  if (is.na(new_nrow)) {
    abort_assign_rows_non_na_only(call)
  }

  if (new_nrow != nrow) {
    # FIXME: vec_expand()?
    i_expand <- c(seq_len(nrow), rep(NA_integer_, new_nrow - nrow))
    x <- vec_slice(x, i_expand)
  }

  x
}

tbl_subassign_row <- function(x, i, value, i_arg, value_arg, call) {
  recycled_value <- vectbl_recycle_rhs_cols(value, length(x), call)

  withCallingHandlers(
    for (j in seq_along(x)) {
      x[[j]] <- vectbl_assign(x[[j]], i, recycled_value[[j]])
    },
    vctrs_error = function(cnd) {
      # Side effect: check if `value` can be recycled
      vectbl_recycle_rhs_rows(value, length(i), i_arg, value_arg, call)

      abort_assign_incompatible_type(x, recycled_value, j, value_arg, cnd, call = call)
    }
  )

  x
}

vectbl_assign <- function(x, i, value) {
  if (is.logical(value)) {
    if (.Call(`tibble_need_coerce`, value)) {
      value <- vec_slice(x, NA_integer_)
    }
  } else {
    if (.Call(`tibble_need_coerce`, x)) {
      d <- dim(x)
      dn <- dimnames(x)
      x <- vec_slice(value, rep(NA_integer_, length(x)))
      dim(x) <- d
      dimnames(x) <- dn
    }
  }

  vec_assign(x, i, value)
}

## usethis namespace: start
#' @importFrom lifecycle deprecate_soft
#' @importFrom lifecycle deprecate_warn
#' @importFrom lifecycle deprecate_stop
#' @importFrom lifecycle expect_deprecated
#' @importFrom pkgconfig set_config
#' @importFrom utils head tail
#' @importFrom vctrs new_data_frame
#' @importFrom vctrs new_rcrd
#' @importFrom vctrs num_as_location
#' @importFrom vctrs unspecified
#' @importFrom vctrs vec_as_location
#' @importFrom vctrs vec_as_location2
#' @importFrom vctrs vec_as_names
#' @importFrom vctrs vec_as_names_legacy
#' @importFrom vctrs vec_as_subscript
#' @importFrom vctrs vec_as_subscript2
#' @importFrom vctrs vec_assign
#' @importFrom vctrs vec_c
#' @importFrom vctrs vec_is
#' @importFrom vctrs vec_names
#' @importFrom vctrs vec_names2
#' @importFrom vctrs vec_ptype_abbr
#' @importFrom vctrs vec_rbind
#' @importFrom vctrs vec_recycle
#' @importFrom vctrs vec_set_names
#' @importFrom vctrs vec_size
#' @importFrom vctrs vec_slice
## usethis namespace: end
NULL

#' @useDynLib tibble, .registration = TRUE
#' @import rlang
#' @aliases NULL tibble-package
#' @details
#' `r lifecycle::badge("stable")`
#'
#' The tibble package provides utilities for handling __tibbles__, where
#' "tibble" is a colloquial term for the S3 [`tbl_df`] class. The [`tbl_df`]
#' class is a special case of the base [`data.frame`][base::data.frame()]
#' class, developed in response to lessons learned over many years of data
#' analysis with data frames.
#'
#' Tibble is the central data structure for the set of packages known as the
#' [tidyverse](https://www.tidyverse.org/packages/), including
#' [dplyr](https://dplyr.tidyverse.org/),
#' [ggplot2](https://ggplot2.tidyverse.org/),
#' [tidyr](https://tidyr.tidyverse.org/), and
#' [readr](https://readr.tidyverse.org/).
#'
#' General resources:
#'   * Website for the tibble package: <https://tibble.tidyverse.org>
#'   * [Vectors chapter](https://adv-r.hadley.nz/vectors-chap.html) in *Advanced R*
#'     (2nd edition), specifically the
#'     [Data frames and tibbles section](https://adv-r.hadley.nz/vectors-chap.html#tibble)
#'
#' Resources on specific topics:
#'   * Create a tibble: [tibble()], [as_tibble()], [tribble()], [enframe()]
#'   * Inspect a tibble: [print.tbl()], [glimpse()]
#'   * Details on the S3 `tbl_df` class: [`tbl_df-class`]
#'   * Package options: [tibble_options]
"_PACKAGE"

#' @importFrom methods setOldClass
#' @exportClass tbl_df
setOldClass(c("tbl_df", "tbl", "data.frame"))

#' `tbl_df` class
#'
#' @description
#' The `tbl_df` class is a subclass of [`data.frame`][base::data.frame()],
#' created in order to have different default behaviour. The colloquial term
#' "tibble" refers to a data frame that has the `tbl_df` class. Tibble is the
#' central data structure for the set of packages known as the
#' [tidyverse](https://www.tidyverse.org/packages/), including
#' [dplyr](https://dplyr.tidyverse.org/),
#' [ggplot2](https://ggplot2.tidyverse.org/),
#' [tidyr](https://tidyr.tidyverse.org/), and
#' [readr](https://readr.tidyverse.org/).
#'
#' The general ethos is that tibbles are lazy and surly: they do less and
#' complain more than base [data.frame]s. This forces
#' problems to be tackled earlier and more explicitly, typically leading to code
#' that is more expressive and robust.
#'
#' @section Properties of `tbl_df`:
#'
#' Objects of class `tbl_df` have:
#' * A `class` attribute of `c("tbl_df", "tbl", "data.frame")`.
#' * A base type of `"list"`, where each element of the list has the same
#'   [vctrs::vec_size()].
#' * A `names` attribute that is a character vector the same length as the
#'   underlying list.
#' * A `row.names` attribute, included for compatibility with [data.frame].
#'   This attribute is only consulted to query the number of rows,
#'   any row names that might be stored there are ignored
#'   by most tibble methods.
#'
#' @section Behavior of `tbl_df`:
#'
#' How default behaviour of tibbles differs from that of
#' [data.frame]s, during creation and access:
#'
#' * Column data is not coerced. A character vector is not turned into a factor.
#'   List-columns are expressly anticipated and do not require special tricks.
#'   Internal names are never stripped from column data.
#'   Read more in [tibble()].
#' * Recycling only happens for a length 1 input.
#'   Read more in [vctrs::vec_recycle()].
#' * Column names are not munged, although missing names are auto-populated.
#'   Empty and duplicated column names are strongly discouraged, but the user
#'   must indicate how to resolve. Read more in [vctrs::vec_as_names()].
#' * Row names are not added and are strongly discouraged, in favor of storing
#'   that info as a column. Read about in [rownames].
#' * `df[, j]` returns a tibble; it does not automatically extract the column
#'   inside. `df[, j, drop = FALSE]` is the default. Read more in [subsetting].
#' * There is no partial matching when `$` is used to index by name. `df$name`
#'   for a nonexistent name generates a warning. Read more in [subsetting].
#'
#' See `vignette("invariants")` for a detailed description of the behavior.
#'
#' Furthermore, printing and inspection are a very high priority.
#' The goal is to convey as much information as possible, in a concise way,
#' even for large and complex tibbles. Read more in [formatting].
#'
#' @name tbl_df-class
#' @aliases tbl_df tbl_df-class
#' @seealso [tibble()], [as_tibble()], [tribble()], [print.tbl()],
#'   [glimpse()]
NULL

# Standard data frame methods --------------------------------------------------

#' @export
as.data.frame.tbl_df <- function(x, row.names = NULL, optional = FALSE, ...) {
  class(x) <- tibble_class
  unname <- which(!map_lgl(x, is_bare_list))
  x[unname] <- map(.subset(x, unname), vectbl_set_names, NULL)
  class(x) <- "data.frame"
  x
}

#' @export
`names<-.tbl_df` <- function(x, value) {
  # workaround for RStudio v1.1, which relies on the ability to set
  # data.frame names to NULL
  if (is.null(value) && is_rstudio()) {
    attr(x, "names") <- NULL
    return(x)
  }

  if (is.null(value)) {
    deprecate_soft("3.0.0", "tibble::`names<-`(value = 'can\\'t be NULL')")

    # FIXME: value <- rep("", length(x))
  }

  if (!has_length(value, length(x))) {
    deprecate_soft("3.0.0", "tibble::`names<-`(value = 'must have the same length as `x`')")

    # FIXME: Reset NA to "" in names

    if (length(value) < length(x)) {
      value <- c(value, rep(NA_character_, length(x) - length(value)))
    } else {
      length(value) <- length(x)
    }
  }

  if (anyNA(value)) {
    deprecate_soft("3.0.0", "tibble::`names<-`(value = 'can\\'t be empty')")

    # FIXME: Reset NA to "" in names
  }

  if (!is_character(value)) {
    deprecate_soft("3.0.0", "tibble::`names<-`(value = 'must be a character vector')")
    value <- as.character(value)
  }

  attr(x, "names") <- as.character(value)
  x
}

# Errors ------------------------------------------------------------------

msg_names_must_be_non_null <- function() {
  "`names` must not be `NULL`."
}

msg_names_must_have_length <- function(length, n) {
  paste0("`names` must have length ", n, ", not ", length, ".")
}

abort_names_must_be_non_null <- function(call = caller_env()) {
  tibble_abort(call = call, msg_names_must_be_non_null())
}

abort_names_must_have_length <- function(length, n, call = caller_env()) {
  tibble_abort(
    call = call,
    msg_names_must_have_length(length, n),
    expected = n,
    actual = length
  )
}

#' Printing tibbles
#'
#' @description
#' One of the main features of the `tbl_df` class is the printing:
#'
#' * Tibbles only print as many rows and columns as fit on one screen,
#'   supplemented by a summary of the remaining rows and columns.
#' * Tibble reveals the type of each column, which keeps the user informed about
#'   whether a variable is, e.g., `<chr>` or `<fct>` (character versus factor).
#'   See `vignette("types")` for an overview of common
#'   type abbreviations.
#'
#' Printing can be tweaked for a one-off call by calling `print()` explicitly
#' and setting arguments like `n` and `width`. More persistent control is
#' available by setting the options described in [pillar::pillar_options].
#' See also `vignette("digits")` for a comparison to base options,
#' and `vignette("numbers")` that showcases [num()] and [char()]
#' for creating columns with custom formatting options.
#'
#' As of tibble 3.1.0, printing is handled entirely by the \pkg{pillar} package.
#' If you implement a package that extends tibble,
#' the printed output can be customized in various ways.
#' See `vignette("extending", package = "pillar")` for details,
#' and [pillar::pillar_options] for options that control the display in the console.
#'
# Copied from pillar::format.tbl() to avoid roxygen2 warning
#' @param x Object to format or print.
#' @param ... Passed on to [tbl_format_setup()].
#' @param n Number of rows to show. If `NULL`, the default, will print all rows
#'   if less than the `print_max` [option][pillar::pillar_options].
#'   Otherwise, will print as many rows as specified by the
#'   `print_min` [option][pillar::pillar_options].
#' @param width Width of text output to generate. This defaults to `NULL`, which
#'   means use the `width` [option][pillar::pillar_options].
#' @param max_extra_cols Number of extra columns to print abbreviated information for,
#'   if the width is too small for the entire tibble. If `NULL`,
#'   the `max_extra_cols` [option][pillar::pillar_options] is used.
#'   The previously defined `n_extra` argument is soft-deprecated.
#' @param max_footer_lines Maximum number of footer lines. If `NULL`,
#'   the `max_footer_lines` [option][pillar::pillar_options] is used.
#'
#' @examples
#' print(as_tibble(mtcars))
#' print(as_tibble(mtcars), n = 1)
#' print(as_tibble(mtcars), n = 3)
#'
#' print(as_tibble(trees), n = 100)
#'
#' print(mtcars, width = 10)
#'
#' mtcars2 <- as_tibble(cbind(mtcars, mtcars), .name_repair = "unique")
#' print(mtcars2, n = 25, max_extra_cols = 3)
#'
#' @examplesIf requireNamespace("nycflights13", quietly = TRUE)
#' print(nycflights13::flights, max_footer_lines = 1)
#' print(nycflights13::flights, width = Inf)
#'
#' @name formatting
#' @aliases print.tbl format.tbl
NULL

# Only for documentation, doesn't do anything
#' @rdname formatting
print.tbl_df <- function(x, width = NULL, ..., n = NULL, max_extra_cols = NULL,
                         max_footer_lines = NULL) {
  NextMethod()
}

# Only for documentation, doesn't do anything
#' @rdname formatting
format.tbl_df <- function(x, width = NULL, ..., n = NULL, max_extra_cols = NULL,
                          max_footer_lines = NULL) {
  NextMethod()
}

#' Legacy printing
#'
#' @description
#' `r lifecycle::badge("deprecated")`
#' As of tibble 3.1.0, printing is handled entirely by the \pkg{pillar} package.
#' Do not use this function.
#' If you implement a package that extend tibble,
#' the printed output can be customized in various ways.
#' See `vignette("extending", package = "pillar")` for details.
#'
#' @param x Object to format or print.
#' @param n Number of rows to show. If `NULL`, the default, will print all rows
#'   if less than option `tibble.print_max`. Otherwise, will print
#'   `tibble.print_min` rows.
#' @param width Width of text output to generate. This defaults to `NULL`, which
#'   means use `getOption("tibble.width")` or (if also `NULL`)
#'   `getOption("width")`; the latter displays only the columns that fit on one
#'   screen. You can also set `options(tibble.width = Inf)` to override this
#'   default and always print all columns, this may be slow for very wide tibbles.
#' @param n_extra Number of extra columns to print abbreviated information for,
#'   if the width is too small for the entire tibble. If `NULL`, the default,
#'   will print information about at most `tibble.max_extra_cols` extra columns.
#'
#' @return An object with a `print()` method that will print the input
#'   similarly to a tibble.
#'   The internal data format is an implementation detail, do not rely on it.
#' @export
#' @keywords internal
trunc_mat <- function(x, n = NULL, width = NULL, n_extra = NULL) {
  deprecate_soft("3.1.0", "tibble::trunc_mat()",
    details = "Printing has moved to the pillar package."
  )

  if (!inherits(x, "tbl")) {
    class(x) <- c("tbl", class(x))
  }

  setup <- pillar::tbl_format_setup(x, width = width, n = n, max_extra_cols = n_extra)

  header <- pillar::tbl_format_header(x, setup)
  body <- pillar::tbl_format_body(x, setup)
  footer <- pillar::tbl_format_footer(x, setup)

  text <- c(header, body, footer)
  structure(list(text = text, summary = list(NULL)), class = "trunc_mat")
}

#' @importFrom pillar style_subtle
#' @export
format.trunc_mat <- function(x, width = NULL, ...) {
  unclass(x)[[1]]
}

#' @export
print.trunc_mat <- function(x, ...) {
  writeLines(format(x, ...))
  invisible(x)
}

print_without_body <- function(x, ...) {
  if (utils::packageVersion("pillar") >= "1.4.99") {
    class(x) <- c("tbl_df_without_body", class(x))
    print(x, ...)
  } else {
    print_with_mocked_format_body(x, ...)
  }
}

if (utils::packageVersion("pillar") >= "1.4.99") {
  tbl_format_body.tbl_df_without_body <- function(x, ...) {
    "<body created by pillar>"
  }

  # Need explicit method, because can't be found in method env
  vctrs::s3_register("pillar::tbl_format_body", "tbl_df_without_body",
    tbl_format_body.tbl_df_without_body
  )
}

# nocov start --- r-lib/rlang compat-friendly-type --- 2019-09-09 Mon 11:50

friendly_type_of <- function(x, length = FALSE) {
  if (is.object(x)) {
    return(sprintf("a `%s` object", paste(class(x), collapse = "/")))
  }

  friendly <- as_friendly_type(typeof(x))

  if (length && rlang::is_vector(x)) {
    friendly <- paste0(friendly, sprintf(" of length %s", length(x)))
  }

  friendly
}

as_friendly_type <- function(type) {
  switch(type,
    logical = "a logical vector",
    integer = "an integer vector",
    numeric = ,
    double = "a double vector",
    complex = "a complex vector",
    character = "a character vector",
    raw = "a raw vector",
    string = "a string",
    list = "a list",
    NULL = "NULL",
    environment = "an environment",
    externalptr = "a pointer",
    weakref = "a weak reference",
    S4 = "an S4 object",
    name = ,
    symbol = "a symbol",
    language = "a call",
    pairlist = "a pairlist node",
    expression = "an expression vector",
    quosure = "a quosure",
    formula = "a formula",
    char = "an internal string",
    promise = "an internal promise",
    ... = "an internal dots object",
    any = "an internal `any` object",
    bytecode = "an internal bytecode object",
    primitive = ,
    builtin = ,
    special = "a primitive function",
    closure = "a function",
    type
  )
}

# nocov end

#' Format a numeric vector
#'
#' @description
#' `r lifecycle::badge("experimental")`
#'
#' Constructs a numeric vector that can be formatted with predefined
#' significant digits, or with a maximum or fixed number of digits
#' after the decimal point.
#' Scaling is supported, as well as forcing a decimal, scientific
#' or engineering notation.
#' If a label is given, it is shown in the header of a column.
#'
#' The formatting is applied when the vector is printed or formatted,
#' and also in a tibble column.
#' The formatting annotation and the class survives most arithmetic transformations,
#' the most notable exceptions are [var()] and [sd()].
#'
#' @family vector classes
#' @inheritParams ellipsis::dots_empty
#' @param x A numeric vector.
#' @param sigfig Define the number of significant digits to show. Must be one or greater.
#'   The `"pillar.sigfig"` [option][pillar::pillar_options] is not consulted.
#'   Can't be combined with `digits`.
#' @param digits Number of digits after the decimal points to show.
#'   Positive numbers specify the exact number of digits to show.
#'   Negative numbers specify (after negation) the maximum number of digits to show.
#'   With `digits = 2`, the numbers 1.2 and 1.234 are printed as 1.20 and 1.23,
#'   with `digits = -2` as 1.2 and 1.23, respectively.
#'   Can't be combined with `sigfig`.
#' @param label A label to show instead of the type description.
#' @param scale Multiplier to apply to the data before showing.
#'   Useful for displaying e.g. percentages.
#'   Must be combined with `label`.
#' @param notation One of `"fit"`, `"dec"`, `"sci"`, `"eng"`, or `"si"`.
#'   - `"fit"`: Use decimal notation if it fits and if it consumes 13 digits or less,
#'     otherwise use scientific notation. (The default for numeric pillars.)
#'   - `"dec"`: Use decimal notation, regardless of width.
#'   - `"sci"`: Use scientific notation.
#'   - `"eng"`: Use engineering notation, i.e. scientific notation
#'       using exponents that are a multiple of three.
#'   - `"si"`: Use SI notation, prefixes between `1e-24` and `1e24` are supported.
#' @param fixed_exponent
#'   Use the same exponent for all numbers in scientific, engineering or SI notation.
#'   `-Inf` uses the smallest, `+Inf` the largest fixed_exponent present in the data.
#'   The default is to use varying exponents.
#' @param extra_sigfig
#'   If `TRUE`, increase the number of significant digits if the data consists of
#'   numbers of the same magnitude with subtle differences.
#' @export
#' @examples
#' # Display as a vector
#' num(9:11 * 100 + 0.5)
#' @examples
#'
#' # Significant figures
#' tibble(
#'   x3 = num(9:11 * 100 + 0.5, sigfig = 3),
#'   x4 = num(9:11 * 100 + 0.5, sigfig = 4),
#'   x5 = num(9:11 * 100 + 0.5, sigfig = 5),
#' )
#'
#' # Maximum digits after the decimal points
#' tibble(
#'   x0 = num(9:11 * 100 + 0.5, digits = 0),
#'   x1 = num(9:11 * 100 + 0.5, digits = -1),
#'   x2 = num(9:11 * 100 + 0.5, digits = -2),
#' )
#'
#' # Use fixed digits and a currency label
#' tibble(
#'   usd = num(9:11 * 100 + 0.5, digits = 2, label = "USD"),
#'   gbp = num(9:11 * 100 + 0.5, digits = 2, label = "£"),
#'   chf = num(9:11 * 100 + 0.5, digits = 2, label = "SFr")
#' )
#'
#' # Scale
#' tibble(
#'   small  = num(9:11 / 1000 + 0.00005, label = "%", scale = 100),
#'   medium = num(9:11 / 100 + 0.0005, label = "%", scale = 100),
#'   large  = num(9:11 / 10 + 0.005, label = "%", scale = 100)
#' )
#'
#' # Notation
#' tibble(
#'   sci = num(10^(-13:6), notation = "sci"),
#'   eng = num(10^(-13:6), notation = "eng"),
#'   si  = num(10^(-13:6), notation = "si"),
#'   dec = num(10^(-13:6), notation = "dec")
#' )
#'
#' # Fixed exponent
#' tibble(
#'   scimin = num(10^(-7:6) * 123, notation = "sci", fixed_exponent = -Inf),
#'   engmin = num(10^(-7:6) * 123, notation = "eng", fixed_exponent = -Inf),
#'   simin  = num(10^(-7:6) * 123, notation = "si", fixed_exponent = -Inf)
#' )
#'
#' tibble(
#'   scismall = num(10^(-7:6) * 123, notation = "sci", fixed_exponent = -3),
#'   scilarge = num(10^(-7:6) * 123, notation = "sci", fixed_exponent = 3),
#'   scimax   = num(10^(-7:6) * 123, notation = "sci", fixed_exponent = Inf)
#' )
#'
#' #' Extra significant digits
#' tibble(
#'   default = num(100 + 1:3 * 0.001),
#'   extra1 = num(100 + 1:3 * 0.001, extra_sigfig = TRUE),
#'   extra2 = num(100 + 1:3 * 0.0001, extra_sigfig = TRUE),
#'   extra3 = num(10000 + 1:3 * 0.00001, extra_sigfig = TRUE)
#' )
# Assigned in .onLoad()
num <- NULL

#' set_num_opts
#'
#' `set_num_opts()` adds formatting options to an arbitrary numeric vector,
#' useful for composing with other types.
#'
#' @export
#' @rdname num
# Assigned in .onLoad()
set_num_opts <- NULL

#' Format a character vector
#'
#' @description
#' `r lifecycle::badge("experimental")`
#'
#' Constructs a character vector that can be formatted with predefined minimum width
#' or without width restrictions, and where the abbreviation style can be configured.
#'
#' The formatting is applied when the vector is printed or formatted,
#' and also in a tibble column.
#'
#' @family vector classes
#' @inheritParams ellipsis::dots_empty
#' @param x A character vector.
#' @param min_chars The minimum width to allocate to this column, defaults to 15.
#'   The `"pillar.min_chars"` [option][pillar::pillar_options] is not consulted.
#' @param shorten How to abbreviate the data if necessary:
#' - `"back"` (default): add an ellipsis at the end
#' - `"front"`: add an ellipsis at the front
#' - `"mid"`: add an ellipsis in the middle
#' - `"abbreviate"`: use [abbreviate()]
#' @export
#' @examples
#' # Display as a vector:
#' char(letters[1:3])
#' @examplesIf { set.seed(20210331); rlang::is_installed("stringi") }
#' # Space constraints:
#' rand_strings <- stringi::stri_rand_strings(10, seq(40, 22, by = -2))
#'
#' # Plain character vectors get truncated if space is limited:
#' data_with_id <- function(id) {
#'   tibble(
#'     id,
#'     some_number_1 = 1, some_number_2 = 2, some_number_3 = 3,
#'     some_number_4 = 4, some_number_5 = 5, some_number_6 = 6,
#'     some_number_7 = 7, some_number_8 = 8, some_number_9 = 9
#'   )
#' }
#' data_with_id(rand_strings)
#'
#' # Use char() to avoid or control truncation
#' data_with_id(char(rand_strings, min_chars = 24))
#' data_with_id(char(rand_strings, min_chars = Inf))
#' data_with_id(char(rand_strings, min_chars = 24, shorten = "mid"))
#'
#' # Lorem Ipsum, one sentence per row.
#' lipsum <- unlist(strsplit(stringi::stri_rand_lipsum(1), "(?<=[.]) +", perl = TRUE))
#' tibble(
#'   back = char(lipsum, shorten = "back"),
#'   front = char(lipsum, shorten = "front"),
#'   mid = char(lipsum, shorten = "mid")
#' )
#' tibble(abbr = char(lipsum, shorten = "abbreviate"))
# Assigned in .onLoad()
char <- NULL

#' set_char_opts
#'
#' `set_char_opts()` adds formatting options to an arbitrary character vector,
#' useful for composing with other types.
#'
#' @export
#' @rdname char
# Assigned in .onLoad()
set_char_opts <- NULL

#' Superseded functions for name repair
#'
#' @description
#' `r lifecycle::badge("superseded")`
#'
#' @description
#' `tidy_names()`, `set_tidy_names()`, and `repair_names()` were early efforts
#' to facilitate *post hoc* name repair in tibble, given that [tibble()] and
#' [as_tibble()] did not do this.
#'
#' From tibble v2.0.0, the `.name_repair` argument gives direct access to three
#' specific levels of name repair: `minimal`, `unique`, and `universal`.
#' See [vctrs::vec_as_names()] for the implementation of the underlying logic.
#'
#' @section Life cycle:
#'
#' These functions are superseded. The `repair_names()` logic
#' will also remain available in [vctrs::vec_as_names_legacy()].
#'
#' ```
#' tibble(..., `.name_repair = "unique"`)
#' ## is preferred to
#' df <- tibble(...)
#' set_tidy_names(df, syntactic = FALSE)
#'
#' tibble(..., `.name_repair = "universal"`)
#' ## is preferred to
#' df <- tibble(...)
#' set_tidy_names(df, syntactic = TRUE)
#' ```
#'
#' @param x A vector.
#' @param name A `names` attribute, usually a character vector.
#' @param syntactic Should names be made syntactically valid? If `FALSE`, uses
#'   same logic as `.name_repair = "unique"`. If `TRUE`, uses same logic as
#'   `.name_repair = "universal"`.
#' @param quiet Whether to suppress messages about name repair.
#'
#' @return `x` with repaired names or a repaired version of `name`.
#'
#' @export
#' @name name-repair-superseded
#' @aliases name-repair-retired
#' @keywords internal
tidy_names <- function(name, syntactic = FALSE, quiet = FALSE) {
  # Local functions to preserve behavior in v1.4.2
  is_syntactic <- function(x) {
    ret <- make.names(x) == x
    ret[is.na(x)] <- FALSE
    ret
  }

  make_syntactic <- function(name, syntactic) {
    if (!syntactic) return(name)

    blank <- name == ""
    fix_syntactic <- (name != "") & !is_syntactic(name)
    name[fix_syntactic] <- make.names(name[fix_syntactic])
    name
  }

  append_pos <- function(name) {
    need_append_pos <- duplicated(name) | duplicated(name, fromLast = TRUE) | name == ""
    if (any(need_append_pos)) {
      rx <- "[.][.][1-9][0-9]*$"
      has_suffix <- grepl(rx, name)
      name[has_suffix] <- gsub(rx, "", name[has_suffix])
      need_append_pos <- need_append_pos | has_suffix
    }

    need_append_pos <- which(need_append_pos)
    name[need_append_pos] <- paste0(name[need_append_pos], "..", need_append_pos)
    name
  }

  describe_tidying <- function(orig_name, name, quiet) {
    stopifnot(length(orig_name) == length(name))
    if (quiet) return()
    new_names <- name != orig_name
    if (any(new_names)) {
      message(
        "New names:\n",
        paste0(orig_name[new_names], " -> ", name[new_names], collapse = "\n")
      )
    }
  }

  name[is.na(name)] <- ""
  orig_name <- name

  name <- make_syntactic(name, syntactic)
  name <- append_pos(name)

  describe_tidying(orig_name, name, quiet)
  name
}

#' @export
#' @rdname name-repair-superseded
set_tidy_names <- function(x, syntactic = FALSE, quiet = FALSE) {
  x <- set_repaired_names(x, repair_hint = FALSE, "minimal", quiet = TRUE)
  new_names <- tidy_names(names(x), syntactic = syntactic, quiet = quiet)
  set_names(x, new_names)
}

#' @param prefix A string, the prefix to use for new column names.
#' @param sep A string inserted between the column name and de-duplicating
#'   number.
#' @export
#' @rdname name-repair-superseded
repair_names <- function(x, prefix = "V", sep = "") {
  if (length(x) == 0) {
    names(x) <- character()
    return(x)
  }

  new_names <- vec_as_names_legacy(names2(x), prefix = prefix, sep = sep)
  set_names(x, new_names)
}

test_that("zero-length inputs given character names", {
  out <- set_tidy_names(character())
  expect_equal(names(out), character())
})

test_that("unnamed input gives uniquely named output", {
  expect_snapshot(
    out <- set_tidy_names(1:3)
  )
  expect_equal(names(out), c("..1", "..2", "..3"))
})

test_that("messages by default", {
  expect_snapshot(
    set_tidy_names(set_names(1, ""))
  )
})

test_that("quiet = TRUE", {
  expect_message(set_tidy_names(set_names(1, ""), quiet = TRUE), NA)
})

test_that("syntactic = TRUE", {
  out <- set_tidy_names(set_names(1, "a b"))
  expect_equal(names(out), tidy_names("a b"))
})

# tidy_names ---------------------------------------------------------------

test_that("zero-length input", {
  expect_equal(tidy_names(character()), character())
})

test_that("proper names", {
  expect_equal(tidy_names(letters), letters)
})

test_that("dupes", {
  expect_snapshot(
    names <- tidy_names(c("a", "b", "a", "c", "b"))
  )
  expect_equal(names, c("a..1", "b..2", "a..3", "c", "b..5"))
})

test_that("empty", {
  expect_snapshot(
    names <- tidy_names("")
  )
  expect_equal(names, "..1")
})

test_that("NA", {
  expect_snapshot(
    names <- tidy_names(NA_character_)
  )
  expect_equal(names, "..1")
})

test_that("corner case", {
  expect_snapshot({
    expect_equal(tidy_names(c("a..2", "a")), c("a..2", "a"))
    expect_equal(tidy_names(c("a..3", "a", "a")), c("a..1", "a..2", "a..3"))
    expect_equal(tidy_names(c("a..2", "a", "a")), c("a..1", "a..2", "a..3"))
    expect_equal(tidy_names(c("a..2", "a..2", "a..2")), c("a..1", "a..2", "a..3"))
  })
})

test_that("syntactic", {
  expect_snapshot(
    names <- tidy_names(c("a b"), syntactic = TRUE)
  )
  expect_equal(names, make.names("a b"))
})

test_that("some syntactic + message (#260)", {
  expect_snapshot(
    names <- tidy_names(c("a b", "c"), syntactic = TRUE)
  )
  expect_equal(names, c(make.names("a b"), "c"))
})

test_that("message", {
  expect_message(
    tidy_names(""),
    "New names:\n -> ..1\n",
    fixed = TRUE
  )
})

test_that("quiet", {
  expect_message(
    tidy_names("", quiet = TRUE),
    NA
  )
})

# make_syntactic -------------------------------------------------------------
expect_syntactic <- function(name, exp_syn_name) {
  expect_identical(
    syn_name <- make_syntactic(name),
    exp_syn_name
  )
  expect_identical(syn_name, make.names(syn_name))
}

test_that("make_syntactic(): empty or NA", {
  expect_syntactic(
      c("", NA_character_),
      c(".", ".")
  )
})

test_that("make_syntactic(): reserved words", {
  expect_syntactic(
    c("if", "TRUE", "Inf", "NA_real_", "normal"),
    c(".if", ".TRUE", ".Inf", ".NA_real_", "normal")
  )
})

test_that("make_syntactic(): underscore", {
  expect_syntactic(
    c( "_",  "_1",  "_a}"),
    c("._", "._1", "._a.")
  )
})

test_that("make_syntactic(): dots", {
  expect_syntactic(
    c(".", "..",  "...", "...."),
    c(".", "..", "....", "....")
  )
})

test_that("make_syntactic(): number", {
  expect_syntactic(
      c(   "0",    "1",    "22",    "333"),
      c("...0", "...1", "...22", "...333")
  )
})

test_that("make_syntactic(): number then character", {
  expect_syntactic(
    c(  "0a",   "1b",   "22c",   "333d"),
    c("..0a", "..1b", "..22c", "..333d")
  )
})

test_that("make_syntactic(): number then non-character", {
  expect_syntactic(
    c(  "0)",   "1&",   "22*",   "333@"),
    c("..0.", "..1.", "..22.", "..333.")
  )
})

test_that("make_syntactic(): dot then number", {
  expect_syntactic(
    c(  ".0",   ".1",   ".22",   ".333"),
    c("...0", "...1", "...22", "...333")
  )
})

test_that("make_syntactic(): dot then number then character", {
  expect_syntactic(
    c( ".0a",  ".1b",  ".22c",  ".333d"),
    c("..0a", "..1b", "..22c", "..333d")
  )
})

test_that("make_syntactic(): dot then number then non-character", {
  expect_syntactic(
    c( ".0)",  ".1&",  ".22*",  ".333@"),
    c("..0.", "..1.", "..22.", "..333.")
  )
})

test_that("make_syntactic(): dot dot then number", {
  expect_syntactic(
    c( "..0",  "..1",  "..22",  "..333"),
    c("...0", "...1", "...22", "...333")
  )
})

test_that("make_syntactic(): dot dot dot then number", {
  expect_syntactic(
    c("...0", "...1", "...22", "...333"),
    c("...0", "...1", "...22", "...333")
  )
})

test_that("make_syntactic(): dot dot dot dot then number", {
  expect_syntactic(
    c("....0", "....1", "....22", "....333"),
    c("....0", "....1", "....22", "....333")
  )
})

test_that("make_syntactic(): dot dot dot dot dot then number", {
  expect_syntactic(
    c(".....0", ".....1", ".....22", ".....333"),
    c(".....0", ".....1", ".....22", ".....333")
  )
})

test_that("make_syntactic(): dot dot then number then character", {
  expect_syntactic(
    c("..0a", "..1b", "..22c", "..333d"),
    c("..0a", "..1b", "..22c", "..333d")
  )
})

test_that("make_syntactic(): dot dot then number then non-character", {
  expect_syntactic(
    c("..0)", "..1&", "..22*", "..333@"),
    c("..0.", "..1.", "..22.", "..333.")
  )
})

# universal_names() -----------------------------------------------------------
test_that("zero-length input", {
  expect_equal(universal_names(character()), character())
})

test_that("universal names are not changed", {
  expect_equal(universal_names(letters), letters)
})

test_that("universal_names() pass checks for minimal, unique, and universal", {
  x <- c(NA, "", "x", "x", "a1:", "_x_y}")
  x_syn <- universal_names(x)
  expect_error(check_minimal(x_syn), NA)
  expect_error(check_unique(x_syn), NA)
  expect_true(all(is_syntactic(x_syn)))
  expect_identical(x_syn, c("...1", "...2", "x...3", "x...4", "a1.", "._x_y."))
})

test_that("universal_names() is idempotent", {
  x <- c(NA, "", "x", "x", "a1:", "_x_y}")
  expect_identical(universal_names(x), universal_names(universal_names(x)))
})

test_that("dupes get a suffix", {
  expect_equal(
    universal_names(c("a", "b", "a", "c", "b")),
    c("a...1", "b...2", "a...3", "c", "b...5")
  )
})

test_that("solo empty or NA gets suffix", {
  expect_identical(universal_names(""), "...1")
  expect_identical(universal_names(NA_character_), "...1")
})

test_that("ellipsis treated like empty string", {
  expect_identical(universal_names("..."), universal_names(""))
})

test_that("solo dot is unchanged", {
  expect_equal(universal_names("."), ".")
})

test_that("dot, dot gets suffix", {
  expect_equal(universal_names(c(".", ".")), c("....1", "....2"))
})

test_that("dot-dot, dot-dot gets suffix", {
  expect_equal(universal_names(c("..", "..")), c(".....1", ".....2"))
})

test_that("empty, dot becomes suffix, dot", {
  expect_equal(universal_names(c("", ".")), c("...1", "."))
})

test_that("empty, empty, dot becomes suffix, suffix, dot", {
  expect_equal(universal_names(c("", "", ".")), c("...1", "...2", "."))
})

test_that("dot, dot, empty becomes suffix, suffix, suffix", {
  expect_equal(universal_names(c(".", ".", "")), c("....1", "....2", "...3"))
})

test_that("dot, empty, dot becomes suffix, suffix, suffix", {
  expect_equal(universal_names(c(".", "", ".")), c("....1", "...2", "....3"))
})

test_that("empty, dot, empty becomes suffix, dot, suffix", {
  expect_equal(universal_names(c("", ".", "")), c("...1", ".", "...3"))
})

test_that("'...j' gets stripped then names are modified", {
  expect_equal(universal_names(c("...6", "...1...2")), c("...1", "...2"))
  expect_equal(universal_names("if...2"), ".if")
})

test_that("complicated inputs", {
  expect_equal(
    universal_names(c("", ".", NA, "if...4", "if", "if...8", "for", "if){]1")),
    c("...1", ".", "...3", ".if...4", ".if...5", ".if...6", ".for", "if...1")
  )
})

test_that("message", {
  skip_legacy()
  expect_message(
    universal_names(c("a b", "b c")),
    "New names:\n* `a b` -> a.b\n* `b c` -> b.c\n",
    fixed = TRUE
  )
})

test_that("quiet", {
  expect_message(
    universal_names("", quiet = TRUE),
    NA
  )
})

test_that("unique then universal is universal, with shuffling", {
  x <- c("", ".2", "..3", "...4", "....5", ".....6", "......7", "...")
  expect_identical(universal_names(unique_names(x)), universal_names(x))

  x2 <- x[c(7L, 4L, 3L, 6L, 5L, 1L, 2L, 8L)]
  expect_identical(universal_names(unique_names(x2)), universal_names(x2))

  x3 <- x[c(3L, 2L, 4L, 6L, 8L, 1L, 5L, 7L)]
  expect_identical(universal_names(unique_names(x3)), universal_names(x3))
})

# set_universal_names ---------------------------------------------------------
test_that("zero-length inputs given character names", {
  out <- set_universal_names(character())
  expect_equal(names(out), character())
})

test_that("unnamed input gives uniquely named output", {
  out <- set_universal_names(1:3)
  expect_equal(names(out), c("...1", "...2", "...3"))
})

test_that("messages by default", {
  skip_legacy()
  expect_message(
    set_universal_names(set_names(1, "a:b")),
    "New names:\n* `a:b` -> a.b\n",
    fixed = TRUE
  )
})

test_that("quiet = TRUE", {
  expect_message(set_universal_names(set_names(1, ""), quiet = TRUE), NA)
})

test_that("non-universal names", {
  out <- set_universal_names(set_names(1, "a b"))
  expect_equal(names(out), "a.b")

  expect_equal(universal_names("a b"), "a.b")
})

# check syntactic
test_that("check_syntactic() imposes check_minimal()", {
  expect_legacy_error(
    check_syntactic(NULL),
    error_names_must_be_non_null(repair_hint = FALSE),
    fixed = TRUE
  )
})

test_that("check_syntactic() imposes check_unique()", {
  expect_legacy_error(
    check_syntactic(c("x", "x", "y")),
    error_column_names_must_be_unique("x"),
    fixed = TRUE
  )
})

test_that("check_syntactic() errors for non-syntactic names", {
  expect_legacy_error(
    check_syntactic(c("x", "a:b", "break")),
    error_column_names_must_be_syntactic(c("a:b", "break"), repair_hint = FALSE),
    fixed = TRUE
  )
})

# make_syntactic -------------------------------------------------------------
expect_syntactic <- function(name, exp_syn_name) {
  expect_identical(
    syn_name <- make_syntactic(name),
    exp_syn_name
  )
  expect_identical(syn_name, make.names(syn_name))
}

test_that("make_syntactic(): empty or NA", {
  expect_syntactic(
    c("", NA_character_),
    c(".", ".")
  )
})

test_that("make_syntactic(): reserved words", {
  expect_syntactic(
    c("if", "TRUE", "Inf", "NA_real_", "normal"),
    c(".if", ".TRUE", ".Inf", ".NA_real_", "normal")
  )
})

test_that("make_syntactic(): underscore", {
  expect_syntactic(
    c("_", "_1", "_a}"),
    c("._", "._1", "._a.")
  )
})

test_that("make_syntactic(): dots", {
  expect_syntactic(
    c(".", "..", "...", "...."),
    c(".", "..", "....", "....")
  )
})

test_that("make_syntactic(): number", {
  expect_syntactic(
    c("0", "1", "22", "333"),
    c("...0", "...1", "...22", "...333")
  )
})

test_that("make_syntactic(): number then character", {
  expect_syntactic(
    c("0a", "1b", "22c", "333d"),
    c("..0a", "..1b", "..22c", "..333d")
  )
})

test_that("make_syntactic(): number then non-character", {
  expect_syntactic(
    c("0)", "1&", "22*", "333@"),
    c("..0.", "..1.", "..22.", "..333.")
  )
})

test_that("make_syntactic(): dot then number", {
  expect_syntactic(
    c(".0", ".1", ".22", ".333"),
    c("...0", "...1", "...22", "...333")
  )
})

test_that("make_syntactic(): dot then number then character", {
  expect_syntactic(
    c(".0a", ".1b", ".22c", ".333d"),
    c("..0a", "..1b", "..22c", "..333d")
  )
})

test_that("make_syntactic(): dot then number then non-character", {
  expect_syntactic(
    c(".0)", ".1&", ".22*", ".333@"),
    c("..0.", "..1.", "..22.", "..333.")
  )
})

test_that("make_syntactic(): dot dot then number", {
  expect_syntactic(
    c("..0", "..1", "..22", "..333"),
    c("...0", "...1", "...22", "...333")
  )
})

test_that("make_syntactic(): dot dot dot then number", {
  expect_syntactic(
    c("...0", "...1", "...22", "...333"),
    c("...0", "...1", "...22", "...333")
  )
})

test_that("make_syntactic(): dot dot dot dot then number", {
  expect_syntactic(
    c("....0", "....1", "....22", "....333"),
    c("....0", "....1", "....22", "....333")
  )
})

test_that("make_syntactic(): dot dot dot dot dot then number", {
  expect_syntactic(
    c(".....0", ".....1", ".....22", ".....333"),
    c(".....0", ".....1", ".....22", ".....333")
  )
})

test_that("make_syntactic(): dot dot then number then character", {
  expect_syntactic(
    c("..0a", "..1b", "..22c", "..333d"),
    c("..0a", "..1b", "..22c", "..333d")
  )
})

test_that("make_syntactic(): dot dot then number then non-character", {
  expect_syntactic(
    c("..0)", "..1&", "..22*", "..333@"),
    c("..0.", "..1.", "..22.", "..333.")
  )
})

# universal_names() -----------------------------------------------------------
test_that("zero-length input", {
  expect_equal(universal_names(character()), character())
})

test_that("universal names are not changed", {
  expect_equal(universal_names(letters), letters)
})

test_that("universal_names() pass checks for minimal, unique, and universal", {
  x <- c(NA, "", "x", "x", "a1:", "_x_y}")
  x_syn <- universal_names(x)
  expect_error(check_minimal(x_syn), NA)
  expect_error(check_unique(x_syn), NA)
  expect_true(all(is_syntactic(x_syn)))
  expect_identical(x_syn, c("...1", "...2", "x...3", "x...4", "a1.", "._x_y."))
})

test_that("universal_names() is idempotent", {
  x <- c(NA, "", "x", "x", "a1:", "_x_y}")
  expect_identical(universal_names(x), universal_names(universal_names(x)))
})

test_that("dupes get a suffix", {
  expect_equal(
    universal_names(c("a", "b", "a", "c", "b")),
    c("a...1", "b...2", "a...3", "c", "b...5")
  )
})

test_that("solo empty or NA gets suffix", {
  expect_identical(universal_names(""), "...1")
  expect_identical(universal_names(NA_character_), "...1")
})

test_that("ellipsis treated like empty string", {
  expect_identical(universal_names("..."), universal_names(""))
})

test_that("solo dot is unchanged", {
  expect_equal(universal_names("."), ".")
})

test_that("dot, dot gets suffix", {
  expect_equal(universal_names(c(".", ".")), c("....1", "....2"))
})

test_that("dot-dot, dot-dot gets suffix", {
  expect_equal(universal_names(c("..", "..")), c(".....1", ".....2"))
})

test_that("empty, dot becomes suffix, dot", {
  expect_equal(universal_names(c("", ".")), c("...1", "."))
})

test_that("empty, empty, dot becomes suffix, suffix, dot", {
  expect_equal(universal_names(c("", "", ".")), c("...1", "...2", "."))
})

test_that("dot, dot, empty becomes suffix, suffix, suffix", {
  expect_equal(universal_names(c(".", ".", "")), c("....1", "....2", "...3"))
})

test_that("dot, empty, dot becomes suffix, suffix, suffix", {
  expect_equal(universal_names(c(".", "", ".")), c("....1", "...2", "....3"))
})

test_that("empty, dot, empty becomes suffix, dot, suffix", {
  expect_equal(universal_names(c("", ".", "")), c("...1", ".", "...3"))
})

test_that("'...j' gets stripped then names are modified", {
  expect_equal(universal_names(c("...6", "...1...2")), c("...1", "...2"))
  expect_equal(universal_names("if...2"), ".if")
})

test_that("complicated inputs", {
  expect_equal(
    universal_names(c("", ".", NA, "if...4", "if", "if...8", "for", "if){]1")),
    c("...1", ".", "...3", ".if...4", ".if...5", ".if...6", ".for", "if...1")
  )
})

test_that("message", {
  skip_legacy()
  expect_message(
    universal_names(c("a b", "b c")),
    "New names:\n* `a b` -> a.b\n* `b c` -> b.c\n",
    fixed = TRUE
  )
})

test_that("quiet", {
  expect_message(
    universal_names("", quiet = TRUE),
    NA
  )
})

test_that("unique then universal is universal, with shuffling", {
  x <- c("", ".2", "..3", "...4", "....5", ".....6", "......7", "...")
  expect_identical(universal_names(unique_names(x)), universal_names(x))

  x2 <- x[c(7L, 4L, 3L, 6L, 5L, 1L, 2L, 8L)]
  expect_identical(universal_names(unique_names(x2)), universal_names(x2))

  x3 <- x[c(3L, 2L, 4L, 6L, 8L, 1L, 5L, 7L)]
  expect_identical(universal_names(unique_names(x3)), universal_names(x3))
})

# set_universal_names ---------------------------------------------------------
test_that("zero-length inputs given character names", {
  out <- set_universal_names(character())
  expect_equal(names(out), character())
})

test_that("unnamed input gives uniquely named output", {
  out <- set_universal_names(1:3)
  expect_equal(names(out), c("...1", "...2", "...3"))
})

test_that("messages by default", {
  skip_legacy()
  expect_message(
    set_universal_names(set_names(1, "a:b")),
    "New names:\n* `a:b` -> a.b\n",
    fixed = TRUE
  )
})

test_that("quiet = TRUE", {
  expect_message(set_universal_names(set_names(1, ""), quiet = TRUE), NA)
})

test_that("non-universal names", {
  out <- set_universal_names(set_names(1, "a b"))
  expect_equal(names(out), "a.b")

  expect_equal(universal_names("a b"), "a.b")
})

# check syntactic
test_that("check_syntactic() imposes check_minimal()", {
  expect_legacy_error(
    check_syntactic(NULL),
    error_names_must_be_non_null(repair_hint = FALSE),
    fixed = TRUE
  )
})

test_that("check_syntactic() imposes check_unique()", {
  expect_legacy_error(
    check_syntactic(c("x", "x", "y")),
    error_column_names_must_be_unique("x"),
    fixed = TRUE
  )
})

test_that("check_syntactic() errors for non-syntactic names", {
  expect_legacy_error(
    check_syntactic(c("x", "a:b", "break")),
    error_column_names_must_be_syntactic(c("a:b", "break"), repair_hint = FALSE),
    fixed = TRUE
  )
})

#' View an object
#'
#' @description
#' `r lifecycle::badge("experimental")`
#'
#' Calls [utils::View()] on the input and returns it, invisibly.
#' If the input is not a data frame, it is processed using a variant of
#' `as.data.frame(head(x, n))`.
#' A message is printed if the number of rows exceeds `n`.
#' This function has no effect in non[interactive] sessions.
#'
#' @details
#' The RStudio IDE overrides `utils::View()`, this is picked up
#' correctly.
#'
#' @param x The object to display.
#' @param title The title to use for the display, by default
#'   the deparsed expression is used.
#' @param ... Unused, must be empty.
#' @param n Maximum number of rows to display. Only used if `x` is not a
#'   data frame. Uses the `view_max` [option][tibble_options] by default.
#'
#' @export
view <- function(x, title = NULL, ..., n = NULL) {
  check_dots_empty()

  if (!interactive()) return(invisible(x))

  if (is.null(title)) {
    title <- expr_deparse(substitute(x), width = Inf)
  }

  if (!is.data.frame(x)) {
    if (is.null(n)) {
      n <- get_tibble_option_view_max()
    }
    x <- head(x, n + 1)
    x <- as.data.frame(x)
    if (nrow(x) > n) {
      message("Showing the first ", n, " rows.")
      x <- head(x, n)
    }
  }

  view_fun <- get("View", envir = as.environment("package:utils"))
  # Construct call with defused arguments, some versions of RStudio evaluate
  # in the global environment (?).
  eval_tidy(quo(view_fun(!!x, !!title)))

  invisible(x)
}

#' Add rows to a data frame
#'
#' @description
#' This is a convenient way to add one or more rows of data to an existing data
#' frame. See [tribble()] for an easy way to create an complete
#' data frame row-by-row. Use [tibble_row()] to ensure that the new data
#' has only one row.
#'
#' `add_case()` is an alias of `add_row()`.
#'
#' @param .data Data frame to append to.
#' @param ... <[`dynamic-dots`][rlang::dyn-dots]>
#'   Name-value pairs, passed on to [tibble()]. Values can be defined
#'   only for columns that already exist in `.data` and unset columns will get an
#'   `NA` value.
#' @param .before,.after One-based row index where to add the new rows,
#'   default: after last row.
#' @family addition
#' @examples
#' # add_row ---------------------------------
#' df <- tibble(x = 1:3, y = 3:1)
#'
#' df %>% add_row(x = 4, y = 0)
#'
#' # You can specify where to add the new rows
#' df %>% add_row(x = 4, y = 0, .before = 2)
#'
#' # You can supply vectors, to add multiple rows (this isn't
#' # recommended because it's a bit hard to read)
#' df %>% add_row(x = 4:5, y = 0:-1)
#'
#' # Use tibble_row() to add one row only
#' df %>% add_row(tibble_row(x = 4, y = 0))
#' try(df %>% add_row(tibble_row(x = 4:5, y = 0:-1)))
#'
#' # Absent variables get missing values
#' df %>% add_row(x = 4)
#'
#' # You can't create new variables
#' try(df %>% add_row(z = 10))
#' @export
add_row <- function(.data, ..., .before = NULL, .after = NULL) {
  if (inherits(.data, "grouped_df")) {
    abort_add_rows_to_grouped_df()
  }

  if (!is.data.frame(.data)) {
    deprecate_stop("2.1.1", "add_row(.data = 'must be a data frame')")
  }

  if (dots_n(...) == 0L) {
    # A single row of missing values is added if no input is supplied
    df <- new_tibble(list(), nrow = 1L)
  } else {
    df <- tibble(...)
  }

  extra_vars <- setdiff(names(df), names(.data))
  if (has_length(extra_vars)) {
    abort_incompatible_new_rows(extra_vars)
  }

  pos <- pos_from_before_after(.before, .after, nrow(.data))
  out <- rbind_at(.data, df, pos)

  vectbl_restore(out, .data)
}

#' @export
#' @rdname add_row
#' @usage NULL
add_case <- add_row

na_value <- function(boilerplate) {
  if (is.list(boilerplate)) {
    list(NULL)
  } else {
    NA
  }
}

rbind_at <- function(old, new, pos) {
  out <- vec_rbind(old, new)

  # Append at end: Nothing more to do.
  if (pos >= nrow(old)) {
    return(out)
  }

  # Splice: Construct index vector
  pos <- max(pos, 0L)
  idx <- c(
    seq2(1L, pos),
    seq2(nrow(old) + 1L, nrow(old) + nrow(new)),
    seq2(pos + 1L, nrow(old))
  )
  vec_slice(out, idx)
}

#' Add columns to a data frame
#'
#' This is a convenient way to add one or more columns to an existing data
#' frame.
#'
#' @param .data Data frame to append to.
#' @param ... <[`dynamic-dots`][rlang::dyn-dots]>
#'   Name-value pairs, passed on to [tibble()]. All values must have
#'   the same size of `.data` or size 1.
#' @param .before,.after One-based column index or column name where to add the
#'   new columns, default: after last column.
#' @inheritParams tibble
#' @family addition
#' @examples
#' # add_column ---------------------------------
#' df <- tibble(x = 1:3, y = 3:1)
#'
#' df %>% add_column(z = -1:1, w = 0)
#' df %>% add_column(z = -1:1, .before = "y")
#'
#' # You can't overwrite existing columns
#' try(df %>% add_column(x = 4:6))
#'
#' # You can't create new observations
#' try(df %>% add_column(z = 1:5))
#'
#' @export
add_column <- function(.data, ..., .before = NULL, .after = NULL,
                       .name_repair = c("check_unique", "unique", "universal", "minimal")) {
  if (!is.data.frame(.data)) {
    deprecate_stop("2.1.1", "add_column(.data = 'must be a data frame')")
  }

  if (has_length(.data) && (!is_named(.data) || anyDuplicated(names2(.data))) && missing(.name_repair)) {
    deprecate_stop("3.0.0", "add_column(.data = 'must have unique names')",
      details = 'Use `.name_repair = "minimal"`.'
    )
  }

  df <- tibble(..., .name_repair = .name_repair)

  if (ncol(df) == 0L) {
    return(.data)
  }

  if (nrow(df) != nrow(.data)) {
    if (nrow(df) == 1) {
      df <- df[rep(1L, nrow(.data)), ]
    } else {
      abort_incompatible_new_cols(nrow(.data), df)
    }
  }

  pos <- pos_from_before_after_names(.before, .after, colnames(.data))

  end_pos <- ncol(.data) + seq_len(ncol(df))

  indexes_before <- rlang::seq2(1L, pos)
  indexes_after <- rlang::seq2(pos + 1L, ncol(.data))
  indexes <- c(indexes_before, end_pos, indexes_after)

  new_data <- .data
  new_data[end_pos] <- df

  out <- new_data[indexes]

  out <- set_repaired_names(out, repair_hint = TRUE, .name_repair)
  vectbl_restore(out, .data)
}


# helpers -----------------------------------------------------------------

pos_from_before_after_names <- function(before, after, names, call = caller_env()) {
  before <- check_names_before_after(before, names)
  after <- check_names_before_after(after, names)

  pos_from_before_after(before, after, length(names), call)
}

pos_from_before_after <- function(before, after, len, call = caller_env()) {
  if (is.null(before)) {
    if (is.null(after)) {
      len
    } else {
      limit_pos_range(after, len)
    }
  } else {
    if (is.null(after)) {
      limit_pos_range(before - 1L, len)
    } else {
      abort_both_before_after(call)
    }
  }
}

limit_pos_range <- function(pos, len) {
  max(0L, min(len, pos))
}

# check_names_before_after ------------------------------------------------

check_names_before_after <- function(j, x) {
  if (!is_bare_character(j)) {
    return(j)
  }

  check_needs_no_dim(j)
  check_names_before_after_character(j, x)
}

check_needs_no_dim <- function(j) {
  if (needs_dim(j)) {
    abort_dim_column_index(j)
  }
}

check_names_before_after_character <- function(j, names) {
  pos <- safe_match(j, names)
  if (anyNA(pos)) {
    unknown_names <- j[is.na(pos)]
    abort_unknown_column_names(unknown_names)
  }
  pos
}

# Errors ------------------------------------------------------------------

msg_unknown_column_names <- function(names) {
  pluralise_commas("Can't find column(s) ", tick(names), " in `.data`.")
}

abort_add_rows_to_grouped_df <- function(call = caller_env()) {
  tibble_abort(call = call, "Can't add rows to grouped data frames.")
}

abort_incompatible_new_rows <- function(names, call = caller_env()) {
  tibble_abort(
    call = call,
    problems(
      "New rows can't add columns:",
      msg_unknown_column_names(names)
    ),
    names = names
  )
}

abort_both_before_after <- function(call = caller_env()) {
  tibble_abort(call = call, "Can't specify both `.before` and `.after`.")
}

abort_unknown_column_names <- function(j, parent = NULL, call = caller_env()) {
  tibble_abort(call = call, pluralise_commas("Can't find column(s) ", tick(j), " in `.data`."), j = j, parent = parent)
}

abort_incompatible_new_cols <- function(n, df, call = caller_env()) {
  tibble_abort(
    call = call,
    bullets(
      "New columns must be compatible with `.data`:",
      x = paste0(
        pluralise_n("New column(s) ha[s](ve)", ncol(df)), " ",
        nrow(df), " rows"
      ),
      i = pluralise_count("`.data` has ", n, " row(s)")
    ),
    expected = n,
    actual = nrow(df)
  )
}

abort_dim_column_index <- function(j, call = caller_env()) {
  # friendly_type_of() doesn't distinguish between matrices and arrays
  tibble_abort(call = call, paste0("Must use a vector in `[`, not an object of class ", class(j)[[1]], "."))
}

#' Get a glimpse of your data
#'
#' @description
#' `r lifecycle::badge("maturing")`
#'
#' `glimpse()` is like a transposed version of `print()`:
#' columns run down the page, and data runs across.
#' This makes it possible to see every column in a data frame.
#' It's a little like [str()] applied to a data frame
#' but it tries to show you as much data as possible.
#' (And it always shows the underlying data, even when applied
#' to a remote data source.)
#'
#' This generic will be moved to \pkg{pillar}, and reexported from there
#' as soon as it becomes available.
#'
#' @section S3 methods:
#' `glimpse` is an S3 generic with a customised method for `tbl`s and
#' `data.frames`, and a default method that calls [str()].
#'
#' @param x An object to glimpse at.
#' @param width Width of output: defaults to the setting of the option
#'   `tibble.width` (if finite) or the width of the console.
#' @param ... Unused, for extensibility.
#' @return x original x is (invisibly) returned, allowing `glimpse()` to be
#'   used within a data pipe line.
#' @export
#' @examples
#' glimpse(mtcars)
#'
#' @examplesIf requireNamespace("nycflights13", quietly = TRUE)
#' glimpse(nycflights13::flights)
#'
# Can be overridden in .onLoad()
glimpse <- function(x, width = NULL, ...) {
  UseMethod("glimpse")
}

#' @importFrom pillar new_pillar_title
#' @importFrom pillar new_pillar_type
# If needed, registered in .onLoad() via replace_if_pillar_has()
glimpse.tbl <- function(x, width = NULL, ...) {
  width <- tibble_glimpse_width(width)
  if (!is.finite(width)) {
    cnd_signal(error_glimpse_infinite_width())
  }

  cli::cat_line("Rows: ", big_mark(nrow(x)))

  # this is an overestimate, but shouldn't be too expensive.
  # every type needs at least three characters: "x, "
  rows <- as.integer(width / 3)
  df <- as.data.frame(head(x, rows))
  cli::cat_line("Columns: ", big_mark(ncol(df)))

  summary <- tbl_sum(x)
  brief_summary <- summary[-1]

  if (has_length(brief_summary)) {
    cli::cat_line(names(brief_summary), ": ", brief_summary)
  }

  if (ncol(df) == 0) return(invisible(x))

  var_types <- map_chr(map(df, new_pillar_type), format)
  ticked_names <- format(new_pillar_title(tick_if_needed(names(df))))
  var_names <- paste0("$ ", justify(ticked_names, right = FALSE), " ", var_types, " ")

  data_width <- width - crayon::col_nchar(var_names) - 2
  formatted <- map_chr(df, function(x) collapse(format_v(x)))
  truncated <- str_trunc(formatted, data_width)

  cli::cat_line(var_names, truncated)
  invisible(x)
}

# If needed, registered in .onLoad() via replace_if_pillar_has()
glimpse.data.frame <- glimpse.tbl

#' @importFrom utils str
# If needed, registered in .onLoad() via replace_if_pillar_has()
glimpse.default <- function(x, width = NULL, max.level = 3, ...) {
  str(x, width = tibble_width(width), max.level = max.level, ...)
  invisible(x)
}

str_trunc <- function(x, max_width) {
  width <- nchar(x)

  nchar_ellipsis <- nchar_width(cli::symbol$ellipsis)

  for (i in seq_along(x)) {
    if (width[i] <= max_width[i]) next

    x[i] <- paste0(substr(x[i], 1, max_width[i] - nchar_ellipsis), cli::symbol$ellipsis)
  }

  x
}

format_v <- function(x) UseMethod("format_v")

#' @export
format_v.default <- function(x) {
  dims <- dim(x)

  if (!is.null(dims)){
    dims_out <- paste0(dims, collapse = " x ")
    out <- paste0("<", class(x)[[1]], "[", dims_out, "]>")
    out
  } else {
    format(x, trim = TRUE, justify = "none")
  }
}

#' @export
format_v.list <- function(x) {
  out <- map(x, format_v)
  atomic <- (map_int(out, length) == 1L)
  out <- map_chr(out, collapse)
  out[!atomic] <- paste0("<", out[!atomic], ">")
  paste0("[", collapse(out), "]")
}

#' @export
format_v.character <- function(x) encodeString(x, quote = '"')

#' @export
format_v.factor <- function(x) {
  if (any(grepl(",", x, fixed = TRUE))) {
    encodeString(as.character(x), quote = '"')
  } else {
    format(x, trim = TRUE, justify = "none")
  }
}

# Errors ------------------------------------------------------------------

error_glimpse_infinite_width <- function() {
  tibble_error("`width` must be finite.")
}

expect_legacy_error <- function(code, ...) {
  expect_error(code)
}

expect_legacy_warning <- function(code, ...) {
  suppressWarnings(expect_warning(code))
}

skip_legacy <- function() {
  skip_if(packageVersion("tibble") >= "2.99.99")
}

skip_brk_add_row_vctrs <- function() {
  # BRK: add_row() uses vctrs coercion rules
  skip_legacy()
}

skip_brk_auto_splice_anonymous <- function() {
  # ENH: tibble() auto-splices anonymous tibble arguments
  skip_legacy()
}

skip_brk_inner_data_frames <- function() {
  # BRK: data frame columns are no longer coerced to tibbles
  skip_legacy()
}

skip_brk_inner_names_not_stripped <- function() {
  # BRK: inner names not stripped from coluns
  skip_legacy()
}

skip_brk_inner_dim_not_stripped <- function() {
  # BRK: inner dim not stripped from coluns
  skip_legacy()
}

skip_brk_no_recursive_indexing <- function() {
  # BRK: no recursive indexing in [[
  skip_legacy()
}

skip_brk_logical_subsetting_no_base_recycling <- function() {
  # BRK: logical indexes must be length one or match the length
  skip_legacy()
}

skip_brk_character_subsetting_no_negative <- function() {
  # BRK: character subsetting for rows no longer supports negative numbers
  skip_legacy()
}

skip_dep_oob_subsetting_warning <- function() {
  # DEP: numeric indexes give warning when indexing OOB
  skip_legacy()
}

skip_dep_rowname_subsetting_warning <- function() {
  # DEP: character indexes give warning with OOB matching
  skip_legacy()
}

skip_dep_new_tibble_subclass <- function() {
  # DEP: new_tibble() warns with subclass argument
  skip_legacy()
}

skip_dep_glimpse <- function() {
  # DEP: glimpse() and format_v() now in pillar
  skip_legacy()
}

skip_enh_posixlt_supported <- function() {
  # ENH: POSIXlt supported
  skip_legacy()
}

skip_enh_as_tibble_rownames <- function() {
  # ENH: rownames argument to as_tibble() works if data frame or matrix doesn't have row names
  skip_legacy()
}

skip_enh_tibble_null <- function() {
  # ENH: NULL arguments to tibble() are silently removed
  skip_legacy()
}

skip_enh_new_tibble_nrow_null <- function() {
  # ENH: new_tibble(nrow = NULL), #781
  skip_legacy()
}

skip_enh_empty_tribble_unspecified <- function() {
  # ENH: zero-row tribbles create unspecified columns
  skip_legacy()
}

skip_enh_as_tibble_retired <- function() {
  # ENH: retiring as_tibble() for vectors and lists, #447
  skip_legacy()
}

skip_enh_bullets_format <- function() {
  # ENH: new bullets format
  skip_legacy()
}

skip_enh_enframe_vector <- function() {
  # ENH: enframe() supports all vectors (#730)
  skip_legacy()
}

skip_enh_print_tbl_args <- function() {
  # ENH: print() and format() support more arguments
  skip_legacy()
}

skip_int_error_unknown_names <- function() {
  # INT: error_unknown_names() no longer implemented
  skip_legacy()
}

skip_int_error_names_must_be_null <- function() {
  # INT: error_names_must_be_null() no longer implemented
  skip_legacy()
}

skip_int_data_frame_tibble_diff <- function() {
  # INT: changed data_frame_() implementation to support custom deprecation warning
  skip_legacy()
}

skip_int_lifecycle <- function() {
  # INT: lifecycle changes classes for deprecation messages in R < 3.6
  skip_legacy()
}

universal_names <- function(...) {
  # INT: universal_names() no longer implemented
  skip_legacy()
}

set_universal_names <- function(...) {
  # INT: set_universal_names() no longer implemented
  skip_legacy()
}

unique_names <- function(...) {
  # INT: unique_names() no longer implemented
  skip_legacy()
}

minimal_names <- function(...) {
  # INT: minimal_names() no longer implemented
  skip_legacy()
}

set_minimal_names <- function(...) {
  # INT: set_minimal_names() no longer implemented
  skip_legacy()
}

make_syntactic <- function(...) {
  # INT: make_syntactic() no longer implemented
  skip_legacy()
}

make_unique <- function(...) {
  # INT: make_unique() no longer implemented
  skip_legacy()
}

two_to_three_dots <- function(...) {
  # INT: two_to_three_dots() no longer implemented
  skip_legacy()
}

expect_output <- function(...) {
  suppressWarnings(testthat::expect_output(...))
}

test_that("format_v for values", {
  skip_dep_glimpse()

  expect_equal(format_v(1), "1")
  expect_equal(format_v(1:3), c("1", "2", "3"))
  expect_equal(format_v(NA), "NA")
  expect_equal(format_v(TRUE), "TRUE")
  expect_equal(format_v(logical()), character())
})

test_that("format_v for character", {
  skip_dep_glimpse()

  expect_equal(format_v("1"), paste0('"', "1", '"'))
  expect_equal(format_v(letters), paste0('"', letters, '"'))
  expect_equal(format_v(NA_character_), "NA")
  expect_equal(format_v(character()), character())
})

test_that("format_v for factor", {
  skip_dep_glimpse()

  expect_equal(format_v(factor(c("1", "a"))), c("1", "a"))
  expect_equal(format_v(factor(c("foo", '"bar"'))), c("foo", "\"bar\""))
  expect_equal(format_v(factor()), character())
  # Add quotes around factor levels with comma
  # so they don't appear as if they were two observations (GH 384)
  expect_equal(
    format_v(factor(c("foo, bar", "foo", '"bar"'))),
    paste0('"', c("foo, bar", "foo", "\\\"bar\\\""), '"')
  )
})

test_that("format_v for list", {
  skip_dep_glimpse()

  expect_equal(format_v(list(1:3)), "[<1, 2, 3>]")
  expect_equal(format_v(as.list(1:3)), "[1, 2, 3]")
  expect_equal(format_v(list(1:3, 4)), "[<1, 2, 3>, 4]")
  expect_equal(format_v(list(1:3, 4:5)), "[<1, 2, 3>, <4, 5>]")
  expect_equal(format_v(list()), "[]")

  expect_equal(format_v(list(list())), "[[]]")
  expect_equal(format_v(list(character())), "[<>]")
  expect_equal(format_v(list(1:3, list(4))), "[<1, 2, 3>, [4]]")
  expect_equal(format_v(list(1:3, list(4:5))), "[<1, 2, 3>, [<4, 5>]]")
})

test_that("glimpse output matches known output", {
  skip_dep_glimpse()

  skip_on_non_utf8_locale()
  skip_if(packageVersion("pillar") >= "1.4.99")

  expect_output_file_rel(
    glimpse(as_tibble(mtcars), width = 70L),
    "glimpse/mtcars-70.txt"
  )

  expect_output_file_rel(
    glimpse(as_tibble(iris), width = 70L),
    "glimpse/iris-70.txt"
  )

  expect_output_file_rel(
    glimpse(as_tibble(iris[integer()]), width = 70L),
    "glimpse/iris-empty-70.txt"
  )

  expect_output_file_rel(
    glimpse(tibble("mean(x)" = 5, "var(x)" = 3), width = 28),
    "glimpse/non-syntactic.txt"
  )

  expect_output_file_rel(
    glimpse(as_tibble(df_all), width = 70L),
    "glimpse/all-70.txt"
  )

  withr::with_options(
    list(tibble.width = 50),
    expect_output_file_rel(
      glimpse(as_tibble(df_all)),
      "glimpse/all-50.txt"
    )
  )

  withr::with_options(
    list(tibble.width = 35),
    expect_output_file_rel(
      glimpse(as_tibble(df_all)),
      "glimpse/all-35.txt"
    )
  )

  expect_output_file_rel(
    glimpse(5),
    "glimpse/5.txt"
  )
})

test_that("glimpse(width = Inf) raises legible error", {
  expect_legacy_error(
    glimpse(mtcars, width = Inf),
    error_glimpse_infinite_width(),
    fixed = TRUE
  )
})

test_that("glimpse works for structures with unknown rows", {
  skip_on_non_utf8_locale() # capture_output_lines() forces native encoding
  skip_if(packageVersion("pillar") >= "1.4.99")
  iris2 <- as_unknown_rows(iris)

  expect_output_file_rel(
    glimpse(iris2, width = 70L),
    "glimpse/iris-70-na-nrow.txt"
  )
})

test_that("glimpse calls tbl_sum() (#550)", {
  skip_on_non_utf8_locale() # capture_output_lines() forces native encoding
  skip_if(packageVersion("pillar") >= "1.4.99")
  iris2 <- as_override_tbl_sum(iris)

  expect_output(
    glimpse(iris2),
    "Overridden: tbl_sum",
    fixed = TRUE
  )
})

test_that("glimpse works on nested data (#486)", {
  skip_on_non_utf8_locale()
  skip_if(packageVersion("pillar") >= "1.4.99")

  nested_iris_df <- tibble(
    Species = unique(iris$Species),
    data = unname(split(iris, iris$Species))
  )

  nested_iris_tbl <- tibble(
    Species = unique(iris$Species),
    data = map(unname(split(iris, iris$Species)), as_tibble)
  )

  expect_output_file_rel(
    glimpse(nested_iris_df, width = 70L),
    "glimpse/iris-nested-df-70.txt"
  )

  expect_output_file_rel(
    glimpse(nested_iris_tbl, width = 70L),
    "glimpse/iris-nested-tbl-70.txt"
  )
})

#' Row-wise tibble creation
#'
#' @description
#' Create [tibble]s using an easier to read row-by-row layout.
#' This is useful for small tables of data where readability is
#' important.  Please see \link{tibble-package} for a general introduction.
#'
#' @param ... <[`dynamic-dots`][rlang::dyn-dots]>
#'   Arguments specifying the structure of a `tibble`.
#'   Variable names should be formulas, and may only appear before the data.
#'   These arguments are processed with [rlang::list2()]
#'   and support unquote via [`!!`] and unquote-splice via [`!!!`].
#' @return A [tibble].
#' @seealso
#'   See [quasiquotation] for more details on tidy dots semantics,
#'   i.e. exactly how  the `...` argument is processed.
#' @export
#' @examples
#' tribble(
#'   ~colA, ~colB,
#'   "a",   1,
#'   "b",   2,
#'   "c",   3
#' )
#'
#' # tribble will create a list column if the value in any cell is
#' # not a scalar
#' tribble(
#'   ~x,  ~y,
#'   "a", 1:3,
#'   "b", 4:6
#' )
#' @examplesIf rlang::is_installed("dplyr") && packageVersion("dplyr") >= "1.0.5"
#'
#' # Use dplyr::mutate(dplyr::across(...)) to assign an explicit type
#' tribble(
#'   ~a, ~b, ~c,
#'   1, "2000-01-01", "1.5"
#' ) %>%
#'   dplyr::mutate(
#'     dplyr::across(a, as.integer),
#'     dplyr::across(b, as.Date)
#'   )
tribble <- function(...) {
  data <- extract_frame_data_from_dots(...)
  turn_frame_data_into_tibble(data$frame_names, data$frame_rest)
}

#' Row-wise matrix creation
#'
#' @description
#' Create matrices laying out the data in rows, similar to
#' `matrix(..., byrow = TRUE)`, with a nicer-to-read syntax.
#' This is useful for small matrices, e.g. covariance matrices, where readability
#' is important. The syntax is inspired by [tribble()].
#'
#' @param ... <[`dynamic-dots`][rlang::dyn-dots]>
#'   Arguments specifying the structure of a `frame_matrix`.
#'   Column names should be formulas, and may only appear before the data.
#'   These arguments are processed with [rlang::list2()]
#'   and support unquote via [`!!`] and unquote-splice via [`!!!`].
#' @return A [matrix].
#' @seealso
#'   See [quasiquotation] for more details on tidy dots semantics,
#'   i.e. exactly how  the `...` argument is processed.
#' @export
#' @examples
#' frame_matrix(
#'   ~col1, ~col2,
#'   1,     3,
#'   5,     2
#' )
frame_matrix <- function(...) {
  data <- extract_frame_data_from_dots(...)
  turn_frame_data_into_frame_matrix(data$frame_names, data$frame_rest)
}

extract_frame_data_from_dots <- function(...) {
  dots <- list2(...)

  # Extract the names.
  frame_names <- extract_frame_names_from_dots(dots)

  # Extract the data
  if (length(frame_names) == 0 && length(dots) != 0) {
    cnd_signal(error_tribble_needs_columns())
  }
  frame_rest <- dots[-seq_along(frame_names)]
  if (!is.null(names(frame_rest))) {
    cnd_signal(error_tribble_named_after_tilde())
  }
  if (length(frame_rest) == 0L) {
    # Can't decide on type in absence of data -- use logical which is
    # coercible to all types
    frame_rest <- unspecified()
  }

  validate_rectangular_shape(frame_names, frame_rest)

  list(frame_names = frame_names, frame_rest = frame_rest)
}

extract_frame_names_from_dots <- function(dots) {
  frame_names <- character()

  for (i in seq_along(dots)) {
    el <- dots[[i]]
    if (!is.call(el)) {
      break
    }

    if (!identical(el[[1]], as.name("~"))) {
      break
    }

    if (length(el) != 2) {
      cnd_signal(error_tribble_lhs_column_syntax(el[[2]]))
    }

    candidate <- el[[2]]
    if (!(is.symbol(candidate) || is.character(candidate))) {
      cnd_signal(error_tribble_rhs_column_syntax(candidate))
    }

    frame_names <- c(frame_names, as.character(candidate))
  }

  frame_names
}

validate_rectangular_shape <- function(frame_names, frame_rest) {
  if (length(frame_names) == 0 && length(frame_rest) == 0) return()

  # Figure out the associated number of rows and number of columns,
  # and validate that the supplied formula produces a rectangular
  # structure.
  if (length(frame_rest) %% length(frame_names) != 0) {
    cnd_signal(error_tribble_non_rectangular(
      length(frame_names),
      length(frame_rest)
    ))
  }
}

turn_frame_data_into_tibble <- function(names, rest) {
  if (is_empty(names)) return(new_tibble(list(), nrow = 0))

  nrow <- length(rest) / length(names)
  dim(rest) <- c(length(names), nrow)
  dimnames(rest) <- list(names, NULL)

  frame_mat <- t(rest)
  frame_col <- turn_matrix_into_column_list(frame_mat)

  new_tibble(frame_col, nrow = nrow)
}

turn_matrix_into_column_list <- function(frame_mat) {
  frame_col <- vector("list", length = ncol(frame_mat))
  names(frame_col) <- colnames(frame_mat)

  # if a frame_mat's col is a list column, keep it unchanged (does not unlist)
  for (i in seq_len(ncol(frame_mat))) {
    col <- frame_mat[, i]

    if (inherits(col, "list") && !some(col, needs_list_col)) {
      subclass_tribble_c_errors(
        names(frame_col)[[i]],
        col <- vec_c(!!!unname(col))
      )
    }

    frame_col[[i]] <- unname(col)
  }
  return(frame_col)
}

turn_frame_data_into_frame_matrix <- function(names, rest) {
  list_cols <- which(map_lgl(rest, needs_list_col))
  if (has_length(list_cols)) {
    cnd_signal(error_frame_matrix_list(list_cols))
  }

  frame_ncol <- length(names)
  frame_mat <- matrix(unlist(rest), ncol = frame_ncol, byrow = TRUE)

  colnames(frame_mat) <- names
  frame_mat
}

subclass_tribble_c_errors <- function(name, code) {
  withCallingHandlers(
    code,
    vctrs_error = function(cnd) {
      cnd_signal(error_tribble_c(name, cnd))
    }
  )
}

# Errors ------------------------------------------------------------------

error_tribble_needs_columns <- function() {
  tibble_error("Must specify at least one column using the `~name` syntax.")
}

error_tribble_named_after_tilde <- function() {
  tibble_error("When using the `~name` syntax, subsequent values must not have names.")
}

error_tribble_lhs_column_syntax <- function(lhs) {
  tibble_error(problems(
    "All column specifications must use the `~name` syntax.",
    paste0("Found ", expr_label(lhs), " on the left-hand side of `~`.")
  ))
}

error_tribble_rhs_column_syntax <- function(rhs) {
  tibble_error(problems(
    'All column specifications must use the `~name` or `~"name"` syntax.',
    paste0("Found ", expr_label(rhs), " on the right-hand side of `~`.")
  ))
}

error_tribble_non_rectangular <- function(cols, cells) {
  tibble_error(bullets(
    "Data must be rectangular:",
    paste0("Found ", cols, " columns."),
    paste0("Found ", cells, " cells."),
    info = paste0(cells, " is not an integer multiple of ", cols, ".")
  ))
}

error_frame_matrix_list <- function(pos) {
  tibble_error(problems(
    "All values must be atomic:",
    pluralise_commas("Found list-valued element(s) at position(s) ", pos, ".")
  ))
}

error_tribble_c <- function(name, cnd) {
  cnd$message <- paste0("Can't create column ", tick(name), ": ", cnd_header(cnd))
  cnd$class <- c(tibble_error_class("tribble_c"), class(cnd))
  cnd
}

skip("All of them")

test_that("error_enframe_value_null()", {
  expect_equal(
    error_enframe_value_null(),
    "The `value` argument to `enframe()` cannot be NULL."
  )
})

test_that("error_enframe_has_dim()", {
  expect_equal(
    error_enframe_has_dim(Titanic),
    "`x` must not have more than one dimension. `length(dim(x))` must be zero or one, not 4."
  )
})

test_that("error_1d_array_column()", {
  expect_equal(
    error_1d_array_column(),
    "1d arrays are not supported in a tibble column."
  )
})

test_that("error_unsupported_index()", {
  expect_equal(
    error_unsupported_index(raw()),
    "Can't subset with `[` using an object of class raw."
  )
})

test_that("error_na_column_index()", {
  expect_equal(
    error_na_column_index(),
    "Can't use numeric NA as column index with `[`."
  )
})

test_that("error_nonint_column_index()", {
  expect_equal(
    error_nonint_column_index(2:3, 3:4 + 0.5),
    bullets(
      "Must use integers to index columns with `[`:",
      paste0("Position ", 2:3, " equals ", 3:4 + 0.5)
    )
  )
})

test_that("error_small_column_index()", {
  expect_equal(
    error_small_column_index(3, 2, -4),
    bullets(
      "Negative column indexes in `[` must match number of columns:",
      "`.data` has 3 columns",
      "Position 2 equals -4"
    )
  )
})

test_that("error_large_column_index()", {
  expect_equal(
    error_large_column_index(3, 2, 5),
    bullets(
      "Positive column indexes in `[` must match number of columns:",
      "`.data` has 3 columns",
      "Position 2 equals 5"
    )
  )
})

test_that("error_mismatch_column_flag()", {
  expect_equal(
    error_mismatch_column_flag(5, 3),
    bullets(
      "Length of logical index vector for `[` must equal number of columns (or 1):",
      "`.data` has 5 columns",
      "Index vector has length 3"
    )
  )
})

test_that("error_na_column_flag()", {
  expect_equal(
    error_na_column_flag(),
    "Can't use logical NA when selecting columns with `[`."
  )
})

test_that("error_unknown_names()", {
  expect_equal(
    error_unknown_names("a"),
    "Can't find column `a` in `.data`."
  )
  expect_equal(
    error_unknown_names(c("b", "c")),
    "Can't find columns `b`, `c` in `.data`."
  )
  expect_equal(
    unell(error_unknown_names(LETTERS)),
    "Can't find columns `A`, `B`, `C`, `D`, `E`, ... (and 21 more) in `.data`."
  )
})

test_that("error_existing_names()", {
  expect_equal(
    error_existing_names("a"),
    "Column `a` already exists in `.data`."
  )
  expect_equal(
    error_existing_names(c("b", "c")),
    "Columns `b`, `c` already exist in `.data`."
  )
  expect_equal(
    unell(error_existing_names(LETTERS)),
    "Columns `A`, `B`, `C`, `D`, `E`, ... (and 21 more) already exist in `.data`."
  )
})

test_that("error_add_rows_to_grouped_df()", {
  expect_equal(
    error_add_rows_to_grouped_df(),
    "Can't add rows to grouped data frames"
  )
})

test_that("error_inconsistent_new_rows()", {
  expect_equal(
    error_inconsistent_new_rows("a"),
    bullets(
      "New rows in `add_row()` must use columns that already exist:",
      "Can't find column `a` in `.data`."
    )
  )

  expect_equal(
    error_inconsistent_new_rows(letters[2:3]),
    bullets(
      "New rows in `add_row()` must use columns that already exist:",
      "Can't find columns `b`, `c` in `.data`."
    )
  )

  expect_equal(
    unell(error_inconsistent_new_rows(LETTERS)),
    bullets(
      "New rows in `add_row()` must use columns that already exist:",
      "Can't find columns `A`, `B`, `C`, `D`, `E`, ... (and 21 more) in `.data`."
    )
  )
})

test_that("error_names_must_not_be_null()", {
  expect_equal(
    error_names_must_be_non_null(repair_hint = TRUE),
    "The `names` must not be `NULL`.\nUse .name_repair to specify repair."
  )
  expect_equal(
    error_names_must_be_non_null(repair_hint = FALSE),
    "The `names` must not be `NULL`."
  )
})

test_that("error_names_must_have_length()", {
  expect_equal(
    error_names_must_have_length(length = 5, n = 3),
    "The `names` must have length 3, not 5."
  )
})

test_that("error_column_must_be_named()", {
  expect_equal(
    error_column_must_be_named(1, repair_hint = TRUE),
    "Column 1 must be named.\nUse .name_repair to specify repair."
  )
  expect_equal(
    error_column_must_be_named(2:3, repair_hint = TRUE),
    "Columns 2, 3 must be named.\nUse .name_repair to specify repair."
  )
  expect_equal(
    unell(error_column_must_be_named(seq_along(letters), repair_hint = TRUE)),
    "Columns 1, 2, 3, 4, 5, ... (and 21 more) must be named.\nUse .name_repair to specify repair."
  )
  expect_equal(
    error_column_must_be_named(4:6, repair_hint = FALSE),
    "Columns 4, 5, 6 must be named."
  )
})

test_that("error_column_names_must_be_unique()", {
  expect_equal(
    error_column_names_must_be_unique("a", repair_hint = FALSE),
    "Column name `a` must not be duplicated."
  )
  expect_equal(
    error_column_names_must_be_unique(letters[2:3], repair_hint = TRUE),
    "Column names `b`, `c` must not be duplicated.\nUse .name_repair to specify repair."
  )
  expect_equal(
    unell(error_column_names_must_be_unique(LETTERS, repair_hint = TRUE)),
    "Column names `A`, `B`, `C`, `D`, `E`, ... (and 21 more) must not be duplicated.\nUse .name_repair to specify repair."
  )
})

test_that("error_column_names_must_be_syntactic()", {
  expect_equal(
    error_column_names_must_be_syntactic("a", repair_hint = FALSE),
    "Column name `a` must be syntactic."
  )
  expect_equal(
    error_column_names_must_be_syntactic(letters[2:3], repair_hint = TRUE),
    "Column names `b`, `c` must be syntactic.\nUse .name_repair to specify repair."
  )
  expect_equal(
    unell(error_column_names_must_be_syntactic(LETTERS, repair_hint = TRUE)),
    "Column names `A`, `B`, `C`, `D`, `E`, ... (and 21 more) must be syntactic.\nUse .name_repair to specify repair."
  )
})

test_that("error_column_scalar_type()", {
  expect_equal(
    error_column_scalar_type("a", "environment"),
    bullets(
      "All columns in a tibble must be 1d or 2d objects:",
      "Column `a` is environment"
    )
  )

  expect_equal(
    error_column_scalar_type(letters[2:3], c("name", "NULL")),
    bullets(
      "All columns in a tibble must be 1d or 2d objects:",
      "Column `b` is name",
      "Column `c` is NULL"
    )
  )

  expect_equal(
    error_column_scalar_type(LETTERS, letters),
    bullets(
      "All columns in a tibble must be 1d or 2d objects:",
      paste0("Column `", LETTERS, "` is ", letters)
    )
  )
})

test_that("error_time_column_must_be_posixct()", {
  expect_equal(
    error_time_column_must_be_posixct("a"),
    "Column `a` is a date/time and must be stored as POSIXct, not POSIXlt."
  )
  expect_equal(
    error_time_column_must_be_posixct(letters[2:3]),
    "Columns `b`, `c` are dates/times and must be stored as POSIXct, not POSIXlt."
  )
  expect_equal(
    unell(error_time_column_must_be_posixct(LETTERS)),
    "Columns `A`, `B`, `C`, `D`, `E`, ... (and 21 more) are dates/times and must be stored as POSIXct, not POSIXlt."
  )
})

test_that("error_inconsistent_cols()", {
  expect_equal(
    error_inconsistent_cols(
      10,
      letters[1:3],
      c(4, 4, 3),
      "`uvw` argument"
    ),
    bullets(
      "Tibble columns must have consistent lengths, only values of length one are recycled:",
      "Length 10: Requested with `uvw` argument",
      "Length 3: Column `c`",
      "Length 4: Columns `a`, `b`"
    )
  )

  expect_equal(
    error_inconsistent_cols(
      10,
      letters[1:3],
      c(2, 2, 3),
      "`xyz` argument"
    ),
    bullets(
      "Tibble columns must have consistent lengths, only values of length one are recycled:",
      "Length 10: Requested with `xyz` argument",
      "Length 2: Columns `a`, `b`",
      "Length 3: Column `c`"
    )
  )

  expect_equal(
    error_inconsistent_cols(
      NULL,
      letters[1:3],
      c(2, 2, 3),
      "`xyz` argument"
    ),
    bullets(
      "Tibble columns must have consistent lengths, only values of length one are recycled:",
      "Length 2: Columns `a`, `b`",
      "Length 3: Column `c`"
    )
  )
})

test_that("error_inconsistent_new_cols()", {
  expect_equal(
    error_inconsistent_new_cols(10, data.frame(a = 1:2)),
    bullets(
      "New columns in `add_column()` must be consistent with `.data`:",
      "`.data` has 10 rows",
      "New column contributes 2 rows"
    )
  )

  expect_equal(
    error_inconsistent_new_cols(1, data.frame(a = 1:3, b = 2:4)),
    bullets(
      "New columns in `add_column()` must be consistent with `.data`:",
      "`.data` has 1 row",
      "New columns contribute 3 rows"
    )
  )
})

test_that("error_duplicate_new_cols()", {
  expect_equal(
    error_duplicate_new_cols("a"),
    bullets(
      "Can't add duplicate columns with `add_column()`:",
      "Column `a` already exists in `.data`."
    )
  )

  expect_equal(
    error_duplicate_new_cols(letters[2:3]),
    bullets(
      "Can't add duplicate columns with `add_column()`:",
      "Columns `b`, `c` already exist in `.data`."
    )
  )

  expect_equal(
    unell(error_duplicate_new_cols(LETTERS)),
    bullets(
      "Can't add duplicate columns with `add_column()`:",
      "Columns `A`, `B`, `C`, `D`, `E`, ... (and 21 more) already exist in `.data`."
    )
  )
})

test_that("error_both_before_after()", {
  expect_equal(
    error_both_before_after(),
    "Can't specify both `.before` and `.after`."
  )
})

test_that("error_already_has_rownames()", {
  expect_equal(
    error_already_has_rownames(),
    "`df` must be a data frame without row names in `column_to_rownames()`."
  )
})

test_that("error_already_has_rownames()", {
  expect_equal(
    error_as_tibble_needs_rownames(),
    "Object passed to `as_tibble()` must have row names if the `rownames` argument is set."
  )
})

test_that("error_glimpse_infinite_width()", {
  expect_equal(
    error_glimpse_infinite_width(),
    "`glimpse()` requires a finite value for the `width` argument."
  )
})

test_that("error_tribble_needs_columns()", {
  expect_equal(
    error_tribble_needs_columns(),
    "`tribble()` needs to specify at least one column using the `~name` syntax."
  )
})

test_that("error_tribble_lhs_column_syntax()", {
  expect_equal(
    error_tribble_lhs_column_syntax(quote(lhs)),
    bullets(
      "All column specifications in `tribble()` must use the `~name` syntax.",
      "Found `lhs` on the left-hand side of `~`."
    )
  )
})

test_that("error_tribble_rhs_column_syntax()", {
  expect_equal(
    error_tribble_rhs_column_syntax(quote(a + b)),
    bullets(
      'All column specifications in `tribble()` must use the `~name` or `~"name"` syntax.',
      "Found `a + b` on the right-hand side of `~`."
    )
  )
})

test_that("error_tribble_non_rectangular()", {
  expect_equal(
    error_tribble_non_rectangular(5, 17),
    bullets(
      "`tribble()` must be used with rectangular data:",
      "Found 5 columns.",
      "Found 17 cells.",
      "17 is not an integer multiple of 5."
    )
  )
})

test_that("error_frame_matrix_list()", {
  expect_equal(
    error_frame_matrix_list(2:4),
    bullets(
      "All values in `frame_matrix()` must be atomic:",
      "Found list-valued elements at positions 2, 3, 4."
    )
  )
})

test_that("error_name_repair_arg()", {
  expect_equal(
    error_name_repair_arg(),
    "The `.name_repair` argument must be a string or a function that specifies the name repair strategy."
  )
})

test_that("error_new_tibble_must_be_list()", {
  expect_equal(
    error_new_tibble_must_be_list(),
    "Must pass a list as `x` argument to `new_tibble()`."
  )
})

test_that("error_new_tibble_needs_nrow()", {
  expect_equal(
    error_new_tibble_needs_nrow(),
    "Must pass a scalar integer as `nrow` argument to `new_tibble()`."
  )
})

set_dftbl_hooks <- function() {
  width <- 40

  set_dftbl_opts_hook(width)
  set_dftbl_knit_hook(width)
  set_dftbl_source_hook()
  set_dftbl_chunk_hook()
  set_dftbl_error_hook()
  set_dftbl_warning_hook()
}

# Defines a `dftbl` knitr option. If this chunk option is set, code is duplicated
# (running with tibbles or data frames, respectively): one line of df code
# followed by the same line of the corresponding tibble code.
# The code is also evaluated, if the results are identical (disregarding the
# class), only the tibble copy is retained.
#
set_dftbl_opts_hook <- function(width) {
  force(width)

  dftbl_opts_hook <- function(options) {
    df_code <- options$code
    tbl_code <- gsub("df", "tbl", df_code, fixed = TRUE)

    # FIXME: Evaluate, but surround in <details> element
    if (!isTRUE(options$dftbl_always) && isTRUE(options$eval)) {
      same <- map2_lgl(df_code, tbl_code, same_as_tbl_code)
      df_code[same] <- ""
    }

    new_code <- as.vector(t(matrix(c(df_code, tbl_code), ncol = 2)))
    options$code <- new_code
    options$width <- width - 4
    options
  }

  knitr::opts_hooks$set(dftbl = dftbl_opts_hook)
}

utils::globalVariables(c("new_df", "new_tbl"))

same_as_tbl_code <- function(df_code, tbl_code) {
  handler <- evaluate::new_output_handler(
    value = function(x, visible) {
      if (visible) x else NULL
    }
  )

  same_as_tbl(
    evaluate::evaluate(df_code, output_handler = handler),
    evaluate::evaluate(tbl_code, output_handler = handler)
  )
}

same_as_tbl <- function(df, tbl) {
  if (length(df) != length(tbl)) return(FALSE)
  if (length(df) < 2) return(FALSE)
  df <- df[-1]
  tbl <- tbl[-1]

  df_obj <- df[[length(df)]]
  tbl_obj <- tbl[[length(tbl)]]

  if (is.data.frame(df_obj) != is.data.frame(tbl_obj)) return(FALSE)

  if (is.data.frame(tbl_obj)) {
    df[[length(df)]] <- as_tibble_deep(df_obj)
  }

  identical(df, tbl)
}

as_tibble_deep <- function(x) {
  is_tibble <- which(map_lgl(x, is.data.frame))
  x[is_tibble] <- map(x[is_tibble], as_tibble)
  as_tibble(x)
}

# dftbl chunks have a reduced width
set_dftbl_knit_hook <- function(width) {
  force(width)

  # Need to use a closure here to keep state
  old_width <- NULL

  dftbl_knit_hook <- function(before, options, envir) {
    if (before) {
      old_width <<- options(width = width)
    } else {
      options(old_width)
      old_width <<- NULL
    }
  }

  knitr::knit_hooks$set(dftbl = dftbl_knit_hook)
}

# dftbl chunks are shown side by side, with the help of an HTML table.
# Each source chunk introduces a new table cell, even chunks also introduce
# a new table row.
# vertical-align: top keeps the table rows nicely aligned.
# This places some limitations on the chunk sources but works well so far.
set_dftbl_source_hook <- function() {
  # Need to use a closure here to daisy-chain hooks and to keep state

  old_source_hook <- knitr::knit_hooks$get("source")

  dftbl_source_even <- TRUE

  dftbl_source_hook_one <- function(x) {
    if (dftbl_source_even) {
      x <- paste0('</td></tr><tr style="vertical-align:top"><td>\n\n', x)
    } else {
      x <- paste0("</td><td>\n\n", x)
    }

    dftbl_source_even <<- !dftbl_source_even
    x
  }

  dftbl_source_hook <- function(x, options) {
    nonempty <- which(x != "")
    x[nonempty] <- vapply(x[nonempty], old_source_hook, options, FUN.VALUE = character(1))
    if (isTRUE(options$dftbl)) {
      x <- vapply(x, dftbl_source_hook_one, FUN.VALUE = character(1))
    }
    paste(x, collapse = "\n")
  }

  knitr::knit_hooks$set(source = dftbl_source_hook)
}

# The entire chunk needs to be surrounded by <table><tbody><tr><td>...</...> .
# We use the dftbl CSS class for the HTML table.
set_dftbl_chunk_hook <- function() {
  # Need to use a closure here to daisy-chain hooks

  old_chunk_hook <- knitr::knit_hooks$get("chunk")

  dftbl_chunk_hook <- function(x, options) {
    x <- old_chunk_hook(x, options)
    if (isTRUE(options$dftbl)) {
      x <- paste0('<table class="dftbl"><tbody><tr><td>\n\n', x, "\n\n</td></tr></tbody></table>")
      x <- gsub("<tr><td>\n\n</td></tr>", "", x, fixed = TRUE)
    }
    x
  }

  knitr::knit_hooks$set(chunk = dftbl_chunk_hook)
}

# I don't understand why knitr doesn't wrap error output.
set_dftbl_error_hook <- function() {
  # Need to use a closure here to daisy-chain hooks

  old_error_hook <- knitr::knit_hooks$get("error")

  dftbl_error_hook <- function(x, options) {
    if (isTRUE(options$dftbl)) {
      x <- strsplit(x, "\n", fixed = TRUE)[[1]]
      x <- unlist(map(x, fansi::strwrap_sgr, getOption("width") + 4, prefix = "#> ", initial = ""))
      x <- paste(paste0(x, "\n"), collapse = "")
    }
    x <- old_error_hook(x, options)
    paste0('<div class="error">', x, '</div>')
  }

  knitr::knit_hooks$set(error = dftbl_error_hook)
}

# I don't understand why knitr doesn't wrap warning output.
set_dftbl_warning_hook <- function() {
  # Need to use a closure here to daisy-chain hooks

  old_warning_hook <- knitr::knit_hooks$get("warning")

  dftbl_warning_hook <- function(x, options) {
    if (isTRUE(options$dftbl)) {
      x <- strsplit(x, "\n", fixed = TRUE)[[1]]
      x <- unlist(map(x, fansi::strwrap_sgr, getOption("width") + 4, prefix = "#> ", initial = ""))
      x <- paste(paste0(x, "\n"), collapse = "")
    }
    x <- old_warning_hook(x, options)
    paste0('<div class="warning">', x, '</div>')
  }

  knitr::knit_hooks$set(warning = dftbl_warning_hook)
}

#' Subsetting tibbles
#'
#' @description
#' Accessing columns, rows, or cells via `$`, `[[`, or `[` is mostly similar to
#' [regular data frames][base::Extract]. However, the
#' behavior is different for tibbles and data frames in some cases:
#' * `[` always returns a tibble by default, even if
#'   only one column is accessed.
#' * Partial matching of column names with `$` and `[[` is not supported, and
#'   `NULL` is returned.
#'   For `$`, a warning is given.
#' * Only scalars (vectors of length one) or vectors with the
#'   same length as the number of rows can be used for assignment.
#' * Rows outside of the tibble's boundaries cannot be accessed.
#' * When updating with `[[<-` and `[<-`, type changes of entire columns are
#'   supported, but updating a part of a column requires that the new value is
#'   coercible to the existing type.
#'   See [vec_slice()] for the underlying implementation.
#'
#' Unstable return type and implicit partial matching can lead to surprises and
#' bugs that are hard to catch. If you rely on code that requires the original
#' data frame behavior, coerce to a data frame via [as.data.frame()].
#'
#' @details
#' For better compatibility with older code written for regular data frames,
#' `[` supports a `drop` argument which defaults to `FALSE`.
#' New code should use `[[` to turn a column into a vector.
#'
#' @param x A tibble.
#' @param value A value to store in a row, column, range or cell.
#'   Tibbles are stricter than data frames in what is accepted here.
#'
#' @name subsetting
#' @examples
#' df <- data.frame(a = 1:3, bc = 4:6)
#' tbl <- tibble(a = 1:3, bc = 4:6)
#'
#' # Subsetting single columns:
#' df[, "a"]
#' tbl[, "a"]
#' tbl[, "a", drop = TRUE]
#' as.data.frame(tbl)[, "a"]
#'
#' # Subsetting single rows with the drop argument:
#' df[1, , drop = TRUE]
#' tbl[1, , drop = TRUE]
#' as.list(tbl[1, ])
#'
#' @examplesIf (Sys.getenv("NOT_CRAN") != "true" || Sys.getenv("IN_PKGDOWN") == "true")
#' # Accessing non-existent columns:
#' df$b
#' tbl$b
#'
#' df[["b", exact = FALSE]]
#' tbl[["b", exact = FALSE]]
#'
#' df$bd <- c("n", "e", "w")
#' tbl$bd <- c("n", "e", "w")
#' df$b
#' tbl$b
#' @examples
#'
#' df$b <- 7:9
#' tbl$b <- 7:9
#' df$b
#' tbl$b
#'
#' # Identical behavior:
#' tbl[1, ]
#' tbl[1, c("bc", "a")]
#' tbl[, c("bc", "a")]
#' tbl[c("bc", "a")]
#' tbl["a"]
#' tbl$a
#' tbl[["a"]]
NULL

#' @rdname subsetting
#' @param name A [name] or a string.
#' @export
`$.tbl_df` <- function(x, name) {
  out <- .subset2(x, name)
  if (is.null(out)) {
    warn(paste0("Unknown or uninitialised column: ", tick(name), "."))
  }
  out
}


#' @rdname subsetting
#' @export
`$<-.tbl_df` <- function(x, name, value) {
  tbl_subassign(
    x, i = NULL, as_string(name), list(value),
    i_arg = NULL, j_arg = name, value_arg = substitute(value)
  )
}

#' @rdname subsetting
#' @param i,j Row and column indices. If `j` is omitted, `i` is used as column index.
#' @param ... Ignored.
#' @param exact Ignored, with a warning.
#' @export
`[[.tbl_df` <- function(x, i, j, ..., exact = TRUE) {
  if (!exact) {
    warn("`exact` ignored.")
  }

  n_dots <- dots_n(...)
  if (n_dots > 0) {
    warn("Extra arguments ignored.")
  }

  # Ignore exact as an argument for counting
  n_real_args <- nargs() - !missing(exact) - n_dots

  # Column subsetting if nargs() == 2L
  if (n_real_args <= 2L) {
    if (missing(i)) {
      cnd_signal(error_subset_columns_non_missing_only())
    }
    tbl_subset2(x, j = i, j_arg = substitute(i))
  } else if (missing(j) || missing(i)) {
    cnd_signal(error_subset_columns_non_missing_only())
  } else {
    i_arg <- substitute(i)
    i <- vectbl_as_row_location2(i, fast_nrow(x), i_arg)
    x <- tbl_subset2(x, j = j, j_arg = substitute(j))

    if (is.null(x)) {
      x
    } else {
      # Drop inner names with double subscript
      vectbl_set_names(vec_slice(x, i), NULL)
    }
  }
}

#' @rdname subsetting
#' @export
`[[<-.tbl_df` <- function(x, i, j, ..., value) {
  i_arg <- substitute(i)
  j_arg <- substitute(j)
  value_arg <- substitute(value)

  if (missing(i)) {
    cnd_signal(error_assign_columns_non_missing_only())
  }

  if (missing(j)) {
    if (nargs() < 4) {
      j <- i
      i <- NULL
      j_arg <- i_arg
      i_arg <- NULL
    } else {
      cnd_signal(error_assign_columns_non_missing_only())
    }
  }

  if (!is.null(i)) {
    i <- vectbl_as_row_location2(i, fast_nrow(x), i_arg, assign = TRUE)
  }

  if (is.object(j)) {
    j <- vectbl_as_col_subscript2(j, j_arg, assign = TRUE)
  }

  # Side effect: check scalar
  if (is.symbol(j)) {
    # FIXME: as_utf8_character() needs rlang > 0.4.11
    j <- chr_unserialise_unicode(as.character(j))
  } else {
    if (!is.vector(j) || length(j) != 1L || is.na(j) || (is.numeric(j) && j < 0) || is.logical(j)) {
      vectbl_as_col_location2(j, length(x), j_arg = j_arg, assign = TRUE)
    }
  }

  j <- vectbl_as_new_col_index(j, x, j_arg)

  # New columns are added to the end, provide index to avoid matching column
  # names again
  value <- list(value)

  # j is already pretty
  tbl_subassign(x, i, j, value, i_arg = i_arg, j_arg = NULL, value_arg = value_arg)
}


#' @rdname subsetting
#' @param drop Coerce to a vector if fetching one column via `tbl[, j]` .
#'   Default `FALSE`, ignored when accessing a column via `tbl[j]` .
#' @export
`[.tbl_df` <- function(x, i, j, drop = FALSE, ...) {
  i_arg <- substitute(i)
  j_arg <- substitute(j)

  if (missing(i)) {
    i <- NULL
    i_arg <- NULL
  } else if (is.null(i)) {
    i <- integer()
  }

  if (missing(j)) {
    j <- NULL
    j_arg <- NULL
  } else if (is.null(j)) {
    j <- integer()
  }

  # Ignore drop as an argument for counting
  n_real_args <- nargs() - !missing(drop)

  # Column or matrix subsetting if nargs() == 2L
  if (n_real_args <= 2L) {
    if (!missing(drop)) {
      warn("`drop` argument ignored for subsetting a tibble with `x[j]`, it has an effect only for `x[i, j]`.")
      drop <- FALSE
    }

    j <- i
    i <- NULL
    j_arg <- i_arg
    i_arg <- NULL

    # Special case, returns a vector:
    if (is.matrix(j)) {
      return(tbl_subset_matrix(x, j, j_arg))
    }
  }

  # From here on, i, j and drop contain correct values:
  if (is.null(j)) {
    xo <- x
  } else {
    j <- vectbl_as_col_location(j, length(x), names(x), j_arg = j_arg, assign = FALSE)

    if (anyNA(j)) {
      cnd_signal(error_na_column_index(which(is.na(j))))
    }

    xo <- .subset(x, j)

    if (anyDuplicated(j)) {
      xo <- set_repaired_names(xo, repair_hint = FALSE, .name_repair = "minimal")
    }

    xo <- set_tibble_class(xo, nrow = fast_nrow(x))
  }

  if (!is.null(i)) {
    xo <- tbl_subset_row(xo, i = i, i_arg)
  }

  if (drop && length(xo) == 1L) {
    tbl_subset2(xo, 1L, j_arg)
  } else {
    vectbl_restore(xo, x)
  }
}

#' @rdname subsetting
#' @export
`[<-.tbl_df` <- function(x, i, j, ..., value) {
  i_arg <- substitute(i)
  j_arg <- substitute(j)

  if (missing(i)) {
    i <- NULL
    i_arg <- NULL
  } else if (is.null(i)) {
    i <- integer()
  }

  if (missing(j)) {
    j <- NULL
    j_arg <- NULL
  } else if (is.null(j)) {
    j <- integer()
  }

  if (is.null(j) && nargs() < 4) {
    j <- i
    i <- NULL
    j_arg <- i_arg
    i_arg <- NULL

    # Special case:
    if (is.matrix(j)) {
      return(tbl_subassign_matrix(x, j, value, j_arg, substitute(value)))
    }
  }

  tbl_subassign(x, i, j, value, i_arg, j_arg, substitute(value))
}

vectbl_as_row_index <- function(i, x, i_arg, assign = FALSE) {
  stopifnot(!is.null(i))

  nr <- fast_nrow(x)

  if (is.character(i)) {
    is_na_orig <- is.na(i)

    if (has_rownames(x)) {
      i <- match(i, rownames(x))
    } else {
      i <- string_to_indices(i)
      i <- fix_oob(i, nr, warn = FALSE)
    }

    i <- fix_oob_invalid(i, is_na_orig)
    i
  } else if (is.numeric(i)) {
    i <- fix_oob(i, nr)
    vectbl_as_row_location(i, nr, i_arg, assign)
  } else {
    vectbl_as_row_location(i, nr, i_arg, assign)
  }
}

fix_oob <- function(i, n, warn = TRUE) {
  if (any(i > 0, na.rm = TRUE)) {
    fix_oob_positive(i, n, warn)
  } else if (any(i < 0, na.rm = TRUE)) {
    fix_oob_negative(i, n, warn)
  } else {
    # Will throw error in vec_as_location()
    i
  }
}

fix_oob_positive <- function(i, n, warn = TRUE) {
  oob <- which(i > n)
  if (warn && length(oob) > 0) {
    deprecate_soft("3.0.0", "tibble::`[.tbl_df`(i = 'must lie in [0, rows] if positive,')",
                   details = "Use `NA_integer_` as row index to obtain a row full of `NA` values.",
                   env = foreign_caller_env())
  }

  i[oob] <- NA_integer_
  i
}

fix_oob_negative <- function(i, n, warn = TRUE) {
  oob <- (i < -n)
  if (warn && any(oob, na.rm = TRUE)) {
    deprecate_soft("3.0.0", "tibble::`[.tbl_df`(i = 'must lie in [-rows, 0] if negative,')",
                   details = "Use `NA_integer_` as row index to obtain a row full of `NA` values.",
                   env = foreign_caller_env())
  }

  i <- i[!oob]
  if (is_empty(i)) i <- seq_len(n)
  i
}

fix_oob_invalid <- function(i, is_na_orig) {
  oob <- which(is.na(i) & !is_na_orig)

  if (has_length(oob)) {
    deprecate_soft("3.0.0", "tibble::`[.tbl_df`(i = 'must use valid row names')",
      details = "Use `NA_integer_` as row index to obtain a row full of `NA` values.",
      env = foreign_caller_env())

    i[oob] <- NA_integer_
  }
  i
}

tbl_subset2 <- function(x, j, j_arg) {
  if (is.matrix(j)) {
    deprecate_soft("3.0.0", "tibble::`[[.tbl_df`(j = 'can\\'t be a matrix",
      details = "Recursive subsetting is deprecated for tibbles.",
      env = foreign_caller_env())

    return(as.matrix(x)[[j]])
  }

  if (is.object(j)) {
    j <- vectbl_as_col_subscript2(j, j_arg)
  }

  if (is.numeric(j)) {
    if (length(j) == 1L) {
      if (is.na(j) || j < 1 || j > length(x) || (is.double(j) && j != trunc(j))) {
        # Side effect: throw error for invalid j
        vectbl_as_col_location2(j, length(x), j_arg = j_arg)
      }
    } else if (length(j) == 2L) {
      deprecate_soft("3.0.0", "tibble::`[[.tbl_df`(j = 'can\\'t be a vector of length 2')",
        details = "Recursive subsetting is deprecated for tibbles.",
        env = foreign_caller_env())
    } else {
      # Side effect: throw error for invalid j
      vectbl_as_col_location2(j, length(x), j_arg = j_arg)
    }
  } else if (is.symbol(j)) {
    # FIXME: Only relevant for R < 3.4
    j <- as.character(j)
  } else if (is.logical(j) || length(j) != 1L || !is_bare_atomic(j) || is.na(j)) {
    # Side effect: throw error for invalid j
    vectbl_as_col_location2(j, length(x), names(x), j_arg = j_arg)
  }

  .subset2(x, j)
}

tbl_subset_row <- function(x, i, i_arg) {
  if (is.null(i)) return(x)
  i <- vectbl_as_row_index(i, x, i_arg)
  xo <- lapply(unclass(x), vec_slice, i = i)
  set_tibble_class(xo, nrow = length(i))
}

tbl_subassign <- function(x, i, j, value, i_arg, j_arg, value_arg) {
  if (is.null(i)) {
    value <- vectbl_wrap_rhs_col(value, value_arg)

    if (is.null(j)) {
      j <- seq_along(x)
      names(j) <- names2(j)
    } else if (!is.null(j_arg)) {
      j <- vectbl_as_new_col_index(j, x, j_arg, names2(value), value_arg)
    }

    value <- vectbl_recycle_rhs_rows(value, fast_nrow(x), i_arg = NULL, value_arg)
    value <- vectbl_recycle_rhs_cols(value, length(j))

    xo <- tbl_subassign_col(x, j, value)
  } else if (is.null(i_arg)) {
    # x[NULL, ...] <- value
    return(x)
  } else {
    i <- vectbl_as_new_row_index(i, x, i_arg)

    # Fill up rows first if necessary
    x <- tbl_expand_to_nrow(x, i)
    value <- vectbl_wrap_rhs_row(value, value_arg)

    if (is.null(j)) {
      xo <- tbl_subassign_row(x, i, value, i_arg, value_arg)
    } else {
      # Optimization: match only once
      # (Invariant: x[[j]] is equivalent to x[[vec_as_location(j)]],
      # allowed by corollary that only existing columns can be updated)
      if (!is.null(j_arg)) {
        j <- vectbl_as_new_col_index(j, x, j_arg, names2(value), value_arg)
      }
      new <- which(j > length(x))

      # Fill up columns if necessary
      if (has_length(new)) {
        init <- map(value[new], vec_slice, rep(NA_integer_, fast_nrow(x)))
        x <- tbl_subassign_col(x, j[new], init)
      }

      xj <- .subset(x, j)
      xj <- tbl_subassign_row(xj, i, value, i_arg, value_arg)
      xo <- tbl_subassign_col(x, j, unclass(xj))
    }
  }

  vectbl_restore(xo, x)
}

vectbl_as_new_row_index <- function(i, x, i_arg) {
  if (is.null(i)) {
    i
  } else if (is_bare_numeric(i)) {
    if (anyDuplicated(i)) {
      cnd_signal(error_duplicate_row_subscript_for_assignment(i))
    }

    nr <- fast_nrow(x)

    # Only update existing, caller knows how to deal with OOB
    numtbl_as_row_location_assign(i, nr, i_arg)
  } else if (is_logical(i)) {
    # Don't allow OOB logical
    vectbl_as_row_location(i, fast_nrow(x), i_arg, assign = TRUE)
  } else {
    i <- vectbl_as_row_index(i, x, i_arg, assign = TRUE)
    if (anyDuplicated(i, incomparables = NA)) {
      cnd_signal(error_duplicate_row_subscript_for_assignment(i))
    }
    i
  }
}

vectbl_as_new_col_index <- function(j, x, j_arg, names = "", value_arg = NULL) {
  # Creates a named index vector
  # Values: index
  # Name: column name (for all columns)

  if (is_bare_character(j)) {
    if (anyNA(j)) {
      cnd_signal(error_assign_columns_non_na_only())
    }

    names <- j

    j <- match(names, names(x))
    new <- which(is.na(j))
    if (has_length(new)) {
      j[new] <- seq.int(length(x) + 1L, length.out = length(new))
    }
  } else if (is_bare_numeric(j)) {
    if (anyNA(j)) {
      cnd_signal(error_assign_columns_non_na_only())
    }

    j <- numtbl_as_col_location_assign(j, length(x), j_arg = j_arg)

    old <- (j <= length(x))
    new <- which(!old)
    j_new <- j[new]

    # FIXME: Recycled names are not repaired
    # FIXME: Hard-coded name repair
    if (length(names) != 1L) {
      # Side effect: check compatibility
      vec_recycle(names, length(j), x_arg = as_label(value_arg))
    } else if (length(j) != 1L) {
      # length(names) == 1
      names <- vec_recycle(names, length(j), x_arg = as_label(value_arg))
    }

    if (length(new) > 0) {
      j[new] <- j_new
      names[new][names[new] == ""] <- paste0("...", j_new)
    }

    names[old] <- names(x)[ j[old] ]
  } else {
    j <- vectbl_as_col_location(j, length(x), names(x), j_arg = j_arg, assign = TRUE)

    if (anyNA(j)) {
      cnd_signal(error_na_column_index(which(is.na(j))))
    }

    if (length(names) != 1L) {
      # Side effect: check compatibility
      vec_recycle(names, length(j), x_arg = as_label(value_arg))
    } else if (length(j) != 1L) {
      # length(names) == 1
      names <- vec_recycle(names, length(j), x_arg = as_label(value_arg))
    }

    old <- (j <= length(x))
    names[old] <- names(x)[ j[old] ]
  }

  if (anyDuplicated(j)) {
    cnd_signal(error_duplicate_column_subscript_for_assignment(j))
  }

  set_names(j, names)
}

numtbl_as_row_location_assign <- function(i, n, i_arg) {
  subclass_row_index_errors(
    num_as_location(i, n, missing = "error", oob = "extend", zero = "error"),
    i_arg = i_arg, assign = TRUE
  )
}

vectbl_as_row_location <- function(i, n, i_arg, assign = FALSE) {
  if (is_bare_atomic(i) && is.matrix(i) && ncol(i) == 1) {
    what <- paste0(
      "tibble::", if (assign) "`[<-`" else "`[`",
      "(i = 'can\\'t be a matrix')"
    )

    lifecycle::deprecate_soft("3.0.0", what,
      details = "Convert to a vector.",
      env = foreign_caller_env()
    )
    i <- i[, 1]
  }

  subclass_row_index_errors(vec_as_location(i, n), i_arg = i_arg, assign = assign)
}

vectbl_as_row_location2 <- function(i, n, i_arg, assign = FALSE) {
  subclass_row_index_errors(vec_as_location2(i, n), i_arg = i_arg, assign = assign)
}

numtbl_as_col_location_assign <- function(j, n, j_arg) {
  subclass_col_index_errors(
    num_as_location(j, n, missing = "error", oob = "extend", zero = "error"),
    j_arg = j_arg, assign = TRUE
  )
}

vectbl_as_col_location <- function(j, n, names = NULL, j_arg, assign = FALSE) {
  subclass_col_index_errors(vec_as_location(j, n, names), j_arg = j_arg, assign = assign)
}

vectbl_as_col_location2 <- function(j, n, names = NULL, j_arg, assign = FALSE) {
  subclass_col_index_errors(vec_as_location2(j, n, names), j_arg = j_arg, assign = assign)
}

vectbl_as_col_subscript2 <- function(j, j_arg, assign = FALSE) {
  subclass_col_index_errors(vec_as_subscript2(j, logical = "error"), j_arg = j_arg, assign = assign)
}

is_tight_sequence_at_end <- function(i_new, n) {
  all(sort(i_new) == seq2(n + 1, n + length(i_new)))
}

tbl_subassign_col <- function(x, j, value) {
  is_data <- !vapply(value, is.null, NA)
  nrow <- fast_nrow(x)

  x <- unclass(x)

  # Grow, assign new names
  new <- which(j > length(x))
  if (has_length(new)) {
    length(x) <- max(j[new])
    names(x)[ j[new] ] <- names2(j)[new]
  }

  # Update
  for (jj in which(is_data)) {
    ji <- j[[jj]]
    x[[ji]] <- value[[jj]]
  }

  # Remove
  j_remove <- j[!is_data & !is.na(j)]
  if (has_length(j_remove)) x <- x[-j_remove]

  # Restore
  set_tibble_class(x, nrow)
}

tbl_expand_to_nrow <- function(x, i) {
  nrow <- fast_nrow(x)

  new_nrow <- max(i, nrow)

  if (is.na(new_nrow)) {
    cnd_signal(error_assign_rows_non_na_only())
  }

  if (new_nrow != nrow) {
    # FIXME: vec_expand()?
    i_expand <- c(seq_len(nrow), rep(NA_integer_, new_nrow - nrow))
    x <- vec_slice(x, i_expand)
  }

  x
}

tbl_subassign_row <- function(x, i, value, i_arg, value_arg) {
  nrow <- fast_nrow(x)
  x <- unclass(x)
  recycled_value <- vectbl_recycle_rhs_cols(value, length(x))

  withCallingHandlers(
    for (j in seq_along(x)) {
      x[[j]] <- vectbl_assign(x[[j]], i, recycled_value[[j]])
    },

    vctrs_error = function(cnd) {
      # Side effect: check if `value` can be recycled
      vectbl_recycle_rhs_rows(value, length(i), i_arg, value_arg)

      cnd_signal(error_assign_incompatible_type(x, recycled_value, j, value_arg, cnd_message(cnd)))
    }
  )

  set_tibble_class(x, nrow)
}

fast_nrow <- function(x) {
  .row_names_info(x, 2L)
}

vectbl_assign <- function(x, i, value) {
  if (is.logical(value)) {
    if (.Call("tibble_need_coerce", value)) {
      value <- vec_slice(x, NA_integer_)
    }
  } else {
    if (.Call("tibble_need_coerce", x)) {
      d <- dim(x)
      dn <- dimnames(x)
      x <- vec_slice(value, rep(NA_integer_, length(x)))
      dim(x) <- d
      dimnames(x) <- dn
    }
  }

  vec_assign(x, i, value)
}

vectbl_wrap_rhs_col <- function(value, value_arg) {
  if (is.null(value)) {
    return(list(value))
  }

  value <- result_vectbl_wrap_rhs(value)
  if (is.null(value)) {
    cnd_signal(error_need_rhs_vector_or_null(value_arg))
  }

  value
}

vectbl_wrap_rhs_row <- function(value, value_arg) {
  value <- result_vectbl_wrap_rhs(value)
  if (is.null(value)) {
    cnd_signal(error_need_rhs_vector(value_arg))
  }

  value
}

result_vectbl_wrap_rhs <- function(value) {
  if (!vec_is(value)) {
    NULL
  } else if (is.list(value)) {
    # Also covers the case of data frames
    unclass(value)
  } else if (is.array(value)) {
    if (any(dim(value)[-1:-2] != 1)) {
      return(NULL)
    }
    dim(value) <- head(dim(value), 2)
    as.list(as.data.frame(value, stringsAsFactors = FALSE))
  } else {
    list(value)
  }
}

vectbl_recycle_rhs <- function(value, nrow, ncol, i_arg, value_arg) {
  value <- vectbl_recycle_rhs_rows(value, nrow, i_arg, value_arg)
  vectbl_recycle_rhs_cols(value, ncol)
}

vectbl_recycle_rhs_rows <- function(value, nrow, i_arg, value_arg) {
  if (length(value) > 0L) {
    withCallingHandlers(
      for (j in seq_along(value)) {
        if (!is.null(value[[j]])) {
          value[[j]] <- vec_recycle(value[[j]], nrow)
        }
      },

      vctrs_error_recycle_incompatible_size = function(cnd) {
        cnd_signal(error_assign_incompatible_size(nrow, value, j, i_arg, value_arg))
      }
    )
  }

  value
}

vectbl_recycle_rhs_cols <- function(value, ncol) {
  if (length(value) != 1L || ncol != 1L) {
    # Errors have been caught beforehand in vectbl_as_new_col_index()
    value <- vec_recycle(value, ncol)
  }

  value
}

# Dedicated functions for faster subsetting
set_tibble_class <- function(x, nrow) {
  attr(x, "row.names") <- .set_row_names(nrow)
  class(x) <- tibble_class
  x
}

# External ----------------------------------------------------------------

vectbl_restore <- function(xo, x) {
  .Call(`tibble_restore_impl`, xo, x)
}

# Errors ------------------------------------------------------------------

error_need_rhs_vector <- function(value_arg) {
  tibble_error(paste0(tick(as_label(value_arg)), " must be a vector, a bare list, a data frame or a matrix."))
}

error_need_rhs_vector_or_null <- function(value_arg) {
  tibble_error(paste0(tick(as_label(value_arg)), " must be a vector, a bare list, a data frame, a matrix, or NULL."))
}

error_na_column_index <- function(j) {
  tibble_error(pluralise_commas("Can't use NA as column index with `[` at position(s) ", j, "."), j = j)
}

error_dim_column_index <- function(j) {
  # friendly_type_of() doesn't distinguish between matrices and arrays
  tibble_error(paste0("Must use a vector in `[`, not an object of class ", class(j)[[1]], "."))
}

error_assign_columns_non_na_only <- function() {
  tibble_error("Can't use NA as column index in a tibble for assignment.")
}

error_subset_columns_non_missing_only <- function() {
  tibble_error("Subscript can't be missing for tibbles in `[[`.")
}

error_assign_columns_non_missing_only <- function() {
  tibble_error("Subscript can't be missing for tibbles in `[[<-`.")
}

error_duplicate_column_subscript_for_assignment <- function(j) {
  j <- unique(j[duplicated(j)])
  tibble_error(pluralise_commas("Column index(es) ", j, " [is](are) used more than once for assignment."), j = j)
}

error_assign_rows_non_na_only <- function() {
  tibble_error("Can't use NA as row index in a tibble for assignment.")
}

error_duplicate_row_subscript_for_assignment <- function(i) {
  i <- unique(i[duplicated(i)])
  tibble_error(pluralise_commas("Row index(es) ", i, " [is](are) used more than once for assignment."), i = i)
}

error_assign_incompatible_size <- function(nrow, value, j, i_arg, value_arg) {
  if (is.null(i_arg)) {
    target <- "existing data"
    existing <- pluralise_count("Existing data has ", nrow, " row(s)")
  } else {
    target <- paste0("row subscript ", tick(as_label(i_arg)))
    existing <- pluralise_count("", nrow, " row(s) must be assigned")
  }

  new <- paste0(pluralise_count("has ", vec_size(value[[j]]), " row(s)"))
  if (length(value) != 1) {
    new <- paste0("Element ", j, " of assigned data ", new)
  } else {
    new <- paste0("Assigned data ", new)
  }

  tibble_error(
    bullets(
      paste0("Assigned data ", tick(as_label(value_arg)), " must be compatible with ", target, ":"),
      x = existing,
      x = new,
      i = if (nrow != 1) "Only vectors of size 1 are recycled",
      i = if (nrow == 1 && vec_size(value[[j]]) != 1) "Row updates require a list value. Do you need `list()` or `as.list()`?"
    ),
    expected = nrow,
    actual = vec_size(value[[j]]),
    j = j
  )
}

error_assign_incompatible_type <- function(x, value, j, value_arg, message) {
  name <- names(x)[[j]]

  tibble_error(
    bullets(
      paste0("Assigned data ", tick(as_label(value_arg)), " must be compatible with existing data:"),
      i = paste0("Error occurred for column ", tick(name)),
      x = message
    ),
    expected = x[[j]],
    actual = value[[j]],
    name = name,
    j = j
  )
}

# Subclassing errors ------------------------------------------------------

subclass_col_index_errors <- function(expr, j_arg, assign) {
  withCallingHandlers(
    expr,
    vctrs_error_subscript = function(cnd) {
      cnd$subscript_arg <- j_arg
      cnd$subscript_elt <- "column"
      if (isTRUE(assign) && !isTRUE(cnd$subscript_action %in% c("negate"))) {
        cnd$subscript_action <- "assign"
      }
      cnd_signal(cnd)
    }
  )
}

subclass_row_index_errors <- function(expr, i_arg, assign) {
  withCallingHandlers(
    expr,
    vctrs_error_subscript = function(cnd) {
      cnd$subscript_arg <- i_arg
      cnd$subscript_elt <- "row"
      if (isTRUE(assign) && !isTRUE(cnd$subscript_action %in% c("negate"))) {
        cnd$subscript_action <- "assign"
      }
      cnd_signal(cnd)
    }
  )
}

test_that("error class", {
  expect_equal(tibble_error_class("boo"), c("tibble_error_boo", "tibble_error"))
})

test_that("aborting with class", {
  expect_error(cnd_signal(error_enframe_value_null()), class = tibble_error_class("enframe_value_null")[[1]])
})

test_that("output test", {
  skip_if_not_installed("rlang", "0.99.0.9000")

  expect_snapshot({
    "# add"
    error_add_rows_to_grouped_df()

    error_incompatible_new_rows("a")
    error_incompatible_new_rows(letters[2:3])
    error_incompatible_new_rows(LETTERS)

    error_both_before_after()

    error_unknown_column_names("a")
    error_unknown_column_names(c("b", "c"))
    error_unknown_column_names(LETTERS)

    error_incompatible_new_cols(10, data.frame(a = 1:2))
    error_incompatible_new_cols(1, data.frame(a = 1:3, b = 2:4))

    "# as_tibble"
    error_column_scalar_type("a", 3, "environment")
    error_column_scalar_type("", 3, "environment")
    error_column_scalar_type(letters[2:3], 3:4, c("name", "NULL"))
    error_column_scalar_type(c("", "", LETTERS), 1:28, c("QQ", "VV", letters))

    error_as_tibble_row_vector(new_environment())
    error_as_tibble_row_size_one(3, "foo", 7)

    "# class-tbl_df"
    error_names_must_be_non_null()

    error_names_must_have_length(length = 5, n = 3)

    "#enframe"
    error_enframe_value_null()
    error_enframe_must_be_vector(lm(speed ~ ., cars))

    "# names"
    error_column_names_cannot_be_empty(1, repair_hint = TRUE)
    error_column_names_cannot_be_empty(2:3, repair_hint = TRUE)
    error_column_names_cannot_be_empty(seq_along(letters), repair_hint = TRUE)
    error_column_names_cannot_be_empty(4:6, repair_hint = FALSE)

    error_column_names_cannot_be_dot_dot(1, repair_hint = FALSE)
    error_column_names_cannot_be_dot_dot(2:3, repair_hint = TRUE)
    error_column_names_cannot_be_dot_dot(1:26, repair_hint = TRUE)

    error_column_names_must_be_unique("a", repair_hint = FALSE)
    error_column_names_must_be_unique(letters[2:3], repair_hint = TRUE)
    error_column_names_must_be_unique(LETTERS, repair_hint = TRUE)

    "# new"
    error_new_tibble_must_be_list()
    error_new_tibble_nrow_must_be_nonnegative()

    "# rownames"
    error_already_has_rownames()

    "# subsetting"
    error_need_rhs_vector(quote(RHS))
    error_need_rhs_vector_or_null(quote(RHS))

    error_na_column_index(1:3)
    error_dim_column_index(as.matrix("x"))

    error_assign_columns_non_na_only()
    error_subset_columns_non_missing_only()
    error_assign_columns_non_missing_only()

    error_duplicate_column_subscript_for_assignment(c(1, 1))
    error_duplicate_column_subscript_for_assignment(c(1, 1, 2, 2))

    error_assign_rows_non_na_only()

    error_duplicate_row_subscript_for_assignment(c(1, 1))
    error_duplicate_row_subscript_for_assignment(c(1, 1, 2, 2))

    error_assign_incompatible_size(3, list(1:2), 1, NULL, quote(rhs))
    error_assign_incompatible_size(4, list(1:4, 3:4), 2, quote(4:1), quote(rhs))

    error_assign_incompatible_type(tibble(a = 1), list("c"), 1, quote(rhs), "Can't frobnicate.")

    "# subsetting-matrix"
    error_subset_matrix_must_be_logical(quote(is.na(x) + 1))
    error_subset_matrix_must_have_same_dimensions(quote(t(is.na(x))))
    error_subset_matrix_scalar_type(quote(is.na(x)), quote(new_environment()))
    error_subset_matrix_must_be_scalar(quote(is.na(x)), quote(1:3))

    "# tibble"
    error_tibble_row_size_one(3, "foo", 7)

    error_incompatible_size(
      10,
      letters[1:3],
      c(4, 4, 3),
      "Requested with `uvw` argument"
    )
    error_incompatible_size(
      10,
      letters[1:3],
      c(2, 2, 3),
      "Requested with `xyz` argument"
    )
    error_incompatible_size(
      NULL,
      letters[1:3],
      c(2, 2, 3),
      "Requested with `xyz` argument"
    )
    error_incompatible_size(
      10,
      1:3,
      c(4, 4, 3),
      "Requested with `uvw` argument"
    )
    error_incompatible_size(
      10,
      1:3,
      c(2, 2, 3),
      "Requested with `xyz` argument"
    )
    error_incompatible_size(
      NULL,
      1:3,
      c(2, 2, 3),
      "Requested with `xyz` argument"
    )

    "# tribble"
    error_tribble_needs_columns()

    error_tribble_lhs_column_syntax(quote(lhs))

    error_tribble_rhs_column_syntax(quote(a + b))

    error_tribble_non_rectangular(5, 17)

    error_frame_matrix_list(2:4)
  })
})

#' Tibble constructor and validator
#'
#' @description
#' Creates or validates a subclass of a tibble.
#' These function is mostly useful for package authors that implement subclasses
#' of a tibble, like \pkg{sf} or \pkg{tsibble}.
#'
#' `new_tibble()` creates a new object as a subclass of `tbl_df`, `tbl` and `data.frame`.
#' This function is optimized for performance, checks are reduced to a minimum.
#' See [vctrs::new_data_frame()] for details.
#'
#' @param x A tibble-like object.
#' @param ... Name-value pairs of additional attributes.
#' @param nrow The number of rows, inferred from `x` if omitted.
#' @param class Subclasses to assign to the new object, default: none.
#' @param subclass Deprecated, retained for compatibility. Please use the `class` argument.
#'
#' @seealso
#' [tibble()] and [as_tibble()] for ways to construct a tibble
#' with recycling of scalars and automatic name repair,
#' and [vctrs::df_list()] and [vctrs::new_data_frame()]
#' for lower-level implementations.
#'
#' @export
#' @examples
#' # The nrow argument can be omitted:
#' new_tibble(list(a = 1:3, b = 4:6))
#'
#' # Existing row.names attributes are ignored:
#' try(validate_tibble(new_tibble(trees, nrow = 3)))
#'
#' # The length of all columns must be compatible with the nrow argument:
#' try(validate_tibble(new_tibble(list(a = 1:3, b = 4:6), nrow = 2)))
new_tibble <- function(x, ..., nrow = NULL, class = NULL, subclass = NULL) {
  # For compatibility with tibble < 2.0.0
  if (is.null(class) && !is.null(subclass)) {
    deprecate_stop("2.0.0", "tibble::new_tibble(subclass = )", "new_tibble(class = )")
  }

  #' @section Construction:
  #'
  #' For `new_tibble()`, `x` must be a list.
  x <- unclass(x)

  if (!is.list(x)) {
    abort_new_tibble_must_be_list()
  }

  #' The `nrow` argument may be omitted as of tibble 3.1.4.
  #' If present, every element of the list `x` should have [vctrs::vec_size()]
  #' equal to this value.
  #' (But this is not checked by the constructor).
  #' This takes the place of the "row.names" attribute in a data frame.
  if (!is.null(nrow)) {
    if (!is.numeric(nrow) || length(nrow) != 1 || nrow < 0 || !is_integerish(nrow, 1) || nrow >= 2147483648) {
      abort_new_tibble_nrow_must_be_nonnegative()
    }
    nrow <- as.integer(nrow)
  }

  args <- attributes(x)

  if (is.null(args)) {
    args <- list()
  }

  new_attrs <- pairlist2(...)
  nms <- names(new_attrs)

  for (i in seq_along(nms)) {
    nm <- nms[[i]]

    if (nm == "") {
      next
    }

    args[[nm]] <- new_attrs[[i]]
  }

  #' `x` must have names (or be empty),
  #' but the names are not checked for correctness.
  if (length(x) == 0) {
    # Leaving this because creating a named list of length zero seems difficult
    args[["names"]] <- character()
  } else if (is.null(args[["names"]])) {
    abort_names_must_be_non_null()
  }

  if (is.null(class)) {
    class <- tibble_class_no_data_frame
  } else {
    class <- c(class[!class %in% tibble_class], tibble_class_no_data_frame)
  }

  slots <- c("x", "n", "class")
  args[slots] <- list(x, nrow, class)

  # `new_data_frame()` restores compact row names
  # Can't add to the assignment above, a literal NULL would be inserted otherwise
  args[["row.names"]] <- NULL

  # need exec() to avoid evaluating language attributes (e.g. rsample)
  exec(new_data_frame, !!!args)
}

#' @description
#' `validate_tibble()` checks a tibble for internal consistency.
#' Correct behavior can be guaranteed only if this function
#' runs without raising an error.
#'
#' @rdname new_tibble
#' @export
validate_tibble <- function(x) {
  #' @section Validation:
  #' `validate_tibble()` checks for "minimal" names
  check_minimal_names(x)

  #' and that all columns are vectors, data frames or matrices.
  check_valid_cols(unclass(x))

  #' It also makes sure that all columns have the same length,
  #' and that [vctrs::vec_size()] is consistent with the data.
  validate_nrow(names(x), col_lengths(x), vec_size(x))

  x
}

check_minimal_names <- function(x) {
  names <- names(x)

  if (is.null(names)) {
    abort_names_must_be_non_null()
  }

  if (anyNA(names)) {
    abort_column_names_cannot_be_empty(which(is.na(names)), repair_hint = FALSE)
  }

  invisible(x)
}

col_lengths <- function(x) {
  map_int(x, vec_size)
}

validate_nrow <- function(names, lengths, nrow) {
  # Validate column lengths, don't recycle
  bad_len <- which(lengths != nrow)
  if (has_length(bad_len)) {
    abort_incompatible_size(nrow, names, lengths, "Requested with `nrow` argument")
  }
}

tibble_class <- c("tbl_df", "tbl", "data.frame")
tibble_class_no_data_frame <- c("tbl_df", "tbl")

# Errors ------------------------------------------------------------------

abort_new_tibble_must_be_list <- function(call = caller_env()) {
  tibble_abort(call = call, "`x` must be a list.")
}

abort_new_tibble_nrow_must_be_nonnegative <- function(call = caller_env()) {
  tibble_abort(call = call, "`nrow` must be a nonnegative whole number smaller than 2^31.")
}

test_that("new_tibble() with new class argument", {
  tbl <- new_tibble(
    data.frame(a = 1:3),
    names = "b",
    attr1 = "value1",
    attr2 = 2,
    nrow = 3,
    class = c("nt", "data.frame")
  )

  # Can't compare directly due to dplyr:::all.equal.tbl_df()
  expect_identical(class(tbl), c("nt", "tbl_df", "tbl", "data.frame"))
  expect_equal(
    unclass(tbl),
    structure(
      list(b = 1:3),
      attr1 = "value1",
      attr2 = 2,
      .Names = "b",
      row.names = .set_row_names(3L)
    )
  )
})

test_that("new_tibble() with additional attributes", {
  df <- data.frame(a = 1:3)
  foo <- df
  attr(foo, "foo") <- "bar"

  tbl_df <- new_tibble(df, nrow = 3, foo = "baz")
  tbl_foo <- new_tibble(foo, nrow = 3, foo = "baz")
  expect_identical(tbl_df, tbl_foo)
})

test_that("new_tibble() can add attributes on zero column tibbles with no attributes", {
  expect_identical(
    attr(new_tibble(list(), nrow = 0L, foo = 10), "foo"),
    10
  )
})

test_that("new_tibble() ignores unnamed additional attributes", {
  expect_identical(
    new_tibble(list(x = 1), "foo", nrow = 1),
    new_tibble(list(x = 1), nrow = 1)
  )

  expect_identical(
    new_tibble(list(x = 1), "foo", bar = "bar", nrow = 1),
    new_tibble(list(x = 1), bar = "bar", nrow = 1)
  )
})

test_that("new_tibble() allows setting names through `...`", {
  expect_identical(
    new_tibble(list(1), names = "x", nrow = 1),
    new_tibble(list(x = 1), nrow = 1)
  )
})

test_that("new_tibble() supports missing `nrow` (#781)", {
  expect_identical(new_tibble(list()), tibble())
  expect_identical(new_tibble(list(a = 1:3)), tibble(a = 1:3))
})

test_that("new_tibble() supports language objects", {
  expect_identical(
    new_tibble(list(), foo = quote(bar())),
    structure(new_tibble(list()), foo = quote(bar()))
  )
})

test_that("new_tibble checks", {
  scoped_lifecycle_errors()

  expect_identical(new_tibble(list(), nrow = 0), tibble())
  expect_identical(new_tibble(list(), nrow = 5), tibble(.rows = 5))
  expect_identical(new_tibble(list(a = 1:3, b = 4:6), nrow = 3), tibble(a = 1:3, b = 4:6))
  expect_tibble_abort(
    new_tibble(1:3, nrow = 1),
    abort_new_tibble_must_be_list()
  )
  expect_tibble_abort(
    new_tibble(list(a = 1), nrow = -1),
    abort_new_tibble_nrow_must_be_nonnegative()
  )
  expect_tibble_abort(
    new_tibble(list(a = 1), nrow = "a"),
    abort_new_tibble_nrow_must_be_nonnegative()
  )
  expect_tibble_abort(
    new_tibble(list(a = 1), nrow = 1:2),
    abort_new_tibble_nrow_must_be_nonnegative()
  )
  expect_tibble_abort(
    new_tibble(list(a = 1), nrow = 2147483648),
    abort_new_tibble_nrow_must_be_nonnegative()
  )
  expect_tibble_abort(
    new_tibble(list(1), nrow = 1),
    abort_names_must_be_non_null()
  )
  expect_error(
    new_tibble(set_names(list(1), NA_character_), nrow = 1),
    NA
  )
  expect_error(
    new_tibble(set_names(list(1), ""), nrow = 1),
    NA
  )
  expect_error(
    new_tibble(list(a = 1, b = 2:3), nrow = 1),
    NA
  )
  expect_error(
    new_tibble(
      structure(list(a = 1, b = 2), row.names = .set_row_names(2)),
      nrow = 1
    ),
    NA
  )
})

test_that("validate_tibble() checks", {
  expect_tibble_abort(
    validate_tibble(new_tibble(list(a = 1, b = 2:3), nrow = 1)),
    abort_incompatible_size(1, c("a", "b"), 1:2, "Requested with `nrow` argument")
  )
})

test_that("output test", {
  expect_snapshot(error = TRUE, {
    new_tibble(1:3, nrow = 1)
    new_tibble(as.list(1:3))
  })
})

# nocov start
replace_if_pillar_has <- function(fun) {
  pillar_ns <- asNamespace("pillar")

  fun <- as_string(ensym(fun))
  value <- mget(fun, pillar_ns, mode = "function", ifnotfound = list(NULL))[[1]]
  if (!is.null(value)) {
    assign(fun, value, inherits = TRUE)
  }
}

register_if_pillar_hasnt <- function(names) {
  pillar_ns <- asNamespace("pillar")

  # Register our method implementations only if pillar doesn't provide them
  methods <- mget(names, asNamespace("tibble"), mode = "function")
  pillar_methods <- mget(names, pillar_ns, mode = "function", ifnotfound = list(NULL))

  pillar_has <- !map_lgl(pillar_methods, is.null)

  imap(methods[!pillar_has], register_method)

  # Get our implementations out of sight to avoid confusing load_all()
  #map2(names(methods)[pillar_has], pillar_methods[pillar_has], assign, inherits = TRUE)
  rm(list = names[pillar_has], inherits = TRUE)
}

register_method <- function(method, name) {
  class <- sub("^[^.]*.", "", name)
  fun <- sub("[.].*$", "", name)
  generic <- paste0("tibble::", fun)

  vctrs::s3_register(generic, class, method)
  invisible()
}
# nocov end

#' Package options
#'
#' Options that affect interactive display.
#' See [pillar::pillar_options] for options that affect display on the console,
#' and [cli::num_ansi_colors()] for enabling and disabling colored output
#' via ANSI sequences like `[3m[38;5;246m[39m[23m`.
#'
#' These options can be set via [options()] and queried via [getOption()].
#' For this, add a `tibble.` prefix (the package name and a dot) to the option name.
#' Example: for an option `foo`, use `options(tibble.foo = value)` to set it
#' and `getOption("tibble.foo")` to retrieve the current value.
#' An option value of `NULL` means that the default is used.
#'
#' @format NULL
#'
#' @examples
#' # Default setting:
#' getOption("tibble.view_max")
#'
#' # Change for the duration of the session:
#' old <- options(tibble.view_max = 100)
#'
#' # view() would show only 100 rows e.g. for a lazy data frame
#'
#' # Change back to the original value:
#' options(old)
#'
#' # Local scope:
#' local({
#'   rlang::local_options(tibble.view_max = 100)
#'   # view() would show only 100 rows e.g. for a lazy data frame
#' })
#' # view() would show the default 1000 rows e.g. for a lazy data frame
#' @section Options for the tibble package:
tibble_options <- list2(
  #' - `view_max`: Maximum number of rows shown by [view()]
  #'   if the input is not a data frame, passed on to [head()]. Default: `1000`.
  view_max = make_option_impl(
    getOption("tibble.view_max", default = 1000L)
  ),
)

#' Build a data frame
#'
#' @description
#'
#' `tibble()` constructs a data frame. It is used like [base::data.frame()], but
#' with a couple notable differences:
#'
#'   * The returned data frame has the class [`tbl_df`][tbl_df-class], in
#'     addition to `data.frame`. This allows so-called "tibbles" to exhibit some
#'     special behaviour, such as [enhanced printing][formatting]. Tibbles are
#'     fully described in [`tbl_df`][tbl_df-class].
#'   * `tibble()` is much lazier than [base::data.frame()] in terms of
#'     transforming the user's input.
#'
#'       - Character vectors are not coerced to factor.
#'       - List-columns are expressly anticipated and do not require special tricks.
#'       - Column names are not modified.
#'       - Inner names in columns are left unchanged.
#'
#'   * `tibble()` builds columns sequentially. When defining a column, you can
#'     refer to columns created earlier in the call. Only columns of length one
#'     are recycled.
#'   * If a column evaluates to a data frame or tibble, it is nested or spliced.
#'     If it evaluates to a matrix or a array, it remains a matrix or array,
#'     respectively.
#'     See examples.
#'
#' @param ... <[`dynamic-dots`][rlang::dyn-dots]>
#'   A set of name-value pairs. These arguments are
#'   processed with [rlang::quos()] and support unquote via [`!!`] and
#'   unquote-splice via [`!!!`]. Use `:=` to create columns that start with a dot.
#'
#'   Arguments are evaluated sequentially.
#'   You can refer to previously created elements directly or using the [.data]
#'   pronoun.
#'   To refer explicitly to objects in the calling environment, use [`!!`] or
#'   [.env], e.g. `!!.data` or `.env$.data` for the special case of an object
#'   named `.data`.
#' @param .rows The number of rows, useful to create a 0-column tibble or
#'   just as an additional check.
#' @param .name_repair Treatment of problematic column names:
#'   * `"minimal"`: No name repair or checks, beyond basic existence,
#'   * `"unique"`: Make sure names are unique and not empty,
#'   * `"check_unique"`: (default value), no name repair, but check they are
#'     `unique`,
#'   * `"universal"`: Make the names `unique` and syntactic
#'   * a function: apply custom name repair (e.g., `.name_repair = make.names`
#'     for names in the style of base R).
#'   * A purrr-style anonymous function, see [rlang::as_function()]
#'
#'   This argument is passed on as `repair` to [vctrs::vec_as_names()].
#'   See there for more details on these terms and the strategies used
#'   to enforce them.
#'
#' @return A tibble, which is a colloquial term for an object of class
#'   [`tbl_df`][tbl_df-class]. A [`tbl_df`][tbl_df-class] object is also a data
#'   frame, i.e. it has class `data.frame`.
#' @seealso Use [as_tibble()] to turn an existing object into a tibble. Use
#'   `enframe()` to convert a named vector into a tibble. Name repair is
#'   detailed in [vctrs::vec_as_names()].
#'   See [quasiquotation] for more details on tidy dots semantics,
#'   i.e. exactly how  the `...` argument is processed.
#' @export
#' @examples
#' # Unnamed arguments are named with their expression:
#' a <- 1:5
#' tibble(a, a * 2)
#'
#' # Scalars (vectors of length one) are recycled:
#' tibble(a, b = a * 2, c = 1)
#'
#' # Columns are available in subsequent expressions:
#' tibble(x = runif(10), y = x * 2)
#'
#' # tibble() never coerces its inputs,
#' str(tibble(letters))
#' str(tibble(x = list(diag(1), diag(2))))
#'
#' # or munges column names (unless requested),
#' tibble(`a + b` = 1:5)
#'
#' # but it forces you to take charge of names, if they need repair:
#' try(tibble(x = 1, x = 2))
#' tibble(x = 1, x = 2, .name_repair = "unique")
#' tibble(x = 1, x = 2, .name_repair = "minimal")
#'
#' ## By default, non-syntactic names are allowed,
#' df <- tibble(`a 1` = 1, `a 2` = 2)
#' ## because you can still index by name:
#' df[["a 1"]]
#' df$`a 1`
#' with(df, `a 1`)
#'
#' ## Syntactic names are easier to work with, though, and you can request them:
#' df <- tibble(`a 1` = 1, `a 2` = 2, .name_repair = "universal")
#' df$a.1
#'
#' ## You can specify your own name repair function:
#' tibble(x = 1, x = 2, .name_repair = make.unique)
#'
#' fix_names <- function(x) gsub("\\s+", "_", x)
#' tibble(`year 1` = 1, `year 2` = 2, .name_repair = fix_names)
#'
#' ## purrr-style anonymous functions and constants
#' ## are also supported
#' tibble(x = 1, x = 2, .name_repair = ~ make.names(., unique = TRUE))
#'
#' tibble(x = 1, x = 2, .name_repair = ~ c("a", "b"))
#'
#' # Tibbles can contain columns that are tibbles or matrices
#' # if the number of rows is compatible. Unnamed tibbled are
#' # spliced, i.e. the inner columns are inserted into the
#' # tibble under construction.
#' tibble(
#'   a = 1:3,
#'   tibble(
#'     b = 4:6,
#'     c = 7:9
#'   ),
#'   d = tibble(
#'     e = tibble(
#'       f = b
#'     )
#'   )
#' )
#' tibble(
#'   a = 1:3,
#'   b = diag(3),
#'   c = cor(trees),
#'   d = Titanic[1:3, , , ]
#' )
#'
#' # Data can not contain tibbles or matrices with incompatible number of rows:
#' try(tibble(a = 1:3, b = tibble(c = 4:7)))
#'
#' # Use := to create columns with names that start with a dot:
#' tibble(.dotted := 3)
#'
#' # This also works, but might break in the future:
#' tibble(.dotted = 3)
#'
#' # You can unquote an expression:
#' x <- 3
#' tibble(x = 1, y = x)
#' tibble(x = 1, y = !!x)
#'
#' # You can splice-unquote a list of quosures and expressions:
#' tibble(!!!list(x = rlang::quo(1:10), y = quote(x * 2)))
#'
#' # Use .data, .env and !! to refer explicitly to columns or outside objects
#' a <- 1
#' tibble(a = 2, b = a)
#' tibble(a = 2, b = .data$a)
#' tibble(a = 2, b = .env$a)
#' tibble(a = 2, b = !!a)
#' try(tibble(a = 2, b = .env$bogus))
#' try(tibble(a = 2, b = !!bogus))
tibble <- function(...,
                   .rows = NULL,
                   .name_repair = c("check_unique", "unique", "universal", "minimal")) {
  xs <- quos(...)

  tibble_quos(xs, .rows, .name_repair)
}

#' tibble_row()
#'
#' @description
#' `tibble_row()` constructs a data frame that is guaranteed to occupy one row.
#' Vector columns are required to have size one, non-vector columns are wrapped
#' in a list.
#'
#' @rdname tibble
#' @export
#' @examples
#'
#' # Use tibble_row() to construct a one-row tibble:
#' tibble_row(a = 1, lm = lm(Height ~ Girth + Volume, data = trees))
tibble_row <- function(...,
                       .name_repair = c("check_unique", "unique", "universal", "minimal")) {
  xs <- enquos(...)

  tibble_quos(xs, .rows = 1, .name_repair = .name_repair, single_row = TRUE)
}

#' Test if the object is a tibble
#'
#' This function returns `TRUE` for tibbles or subclasses thereof,
#' and `FALSE` for all other objects, including regular data frames.
#'
#' @param x An object
#' @return `TRUE` if the object inherits from the `tbl_df` class.
#' @export
is_tibble <- function(x) {
  inherits(x, "tbl_df")
}

#' Deprecated test for tibble-ness
#'
#' @description
#' `r lifecycle::badge("soft-deprecated")`
#'
#' Please use [is_tibble()] instead.
#'
#' @inheritParams is_tibble
#' @export
#' @keywords internal
is.tibble <- function(x) {
  deprecate_warn("2.0.0", "is.tibble()", "is_tibble()")

  is_tibble(x)
}

tibble_quos <- function(xs, .rows, .name_repair, single_row = FALSE, call = caller_env()) {
  # Evaluate each column in turn
  col_names <- given_col_names <- names2(xs)
  empty_col_names <- which(col_names == "")
  col_names[empty_col_names] <- names(quos_auto_name(xs[empty_col_names]))
  lengths <- rep_along(xs, 0L)

  output <- rep_along(xs, list(NULL))

  env <- new_environment()
  mask <- new_data_mask_with_data(env)

  first_size <- .rows

  for (j in seq_along(xs)) {
    res <- eval_tidy(xs[[j]], mask)

    if (!is.null(res)) {
      # Single-row mode: Vectors must be length one, non-vectors are wrapped
      # in a list (which is length one by definition)
      if (single_row) {
        if (vec_is(res)) {
          if (vec_size(res) != 1) {
            abort_tibble_row_size_one(j, given_col_names[[j]], vec_size(res))
          }
        } else {
          res <- list(res)
        }
      } else {
        # 657
        res <- check_valid_col(res, col_names[[j]], j, call)

        lengths[[j]] <- current_size <- vec_size(res)
        if (is.null(first_size)) {
          first_size <- current_size
        } else if (first_size == 1L && current_size != 1L) {
          idx_to_fix <- seq2(1L, j - 1L)
          output[idx_to_fix] <- fixed_output <-
            map(output[idx_to_fix], vec_recycle, current_size)

          # Refill entire data mask
          map2(output[idx_to_fix], col_names[idx_to_fix], add_to_env2, env = env)

          first_size <- current_size
        } else {
          res <- vectbl_recycle_rows(res, first_size, j, given_col_names[[j]], call)
        }
      }

      output[[j]] <- res
      col_names[[j]] <- add_to_env2(res, given_col_names[[j]], col_names[[j]], env)
    }
  }

  names(output) <- col_names

  is_null <- map_lgl(output, is.null)
  output <- output[!is_null]

  output <- splice_dfs(output)
  output <- set_repaired_names(output, repair_hint = TRUE, .name_repair = .name_repair, call = call)

  new_tibble(output, nrow = first_size %||% 0L)
}

check_valid_col <- function(x, name, pos, call) {
  if (name == "") {
    ret <- check_valid_cols(list(x), pos = pos, call = call)
  } else {
    ret <- check_valid_cols(set_names(list(x), name), call = call)
  }
  invisible(ret[[1]])
}

new_data_mask_with_data <- function(env) {
  mask <- new_data_mask(env)
  mask$.data <- as_data_pronoun(env)
  mask
}

add_to_env2 <- function(x, given_name, name = given_name, env) {
  if (is.data.frame(x) && given_name == "") {
    imap(x, add_to_env, env)
    ""
  } else {
    add_to_env(x, name, env)
    name
  }
}

add_to_env <- function(x, name, env) {
  env[[name]] <- x
  invisible()
}

splice_dfs <- function(x) {
  # Avoiding .ptype argument to vec_c()
  if (is_empty(x)) {
    return(list())
  }

  x <- imap(x, function(.x, .y) {
    if (.y == "") unclass(.x) else list2(!!.y := .x)
  })
  vec_c(!!!x, .name_spec = "{inner}")
}

vectbl_recycle_rows <- function(x, n, j, name, call = caller_env()) {
  size <- vec_size(x)
  if (size == n) return(x)
  if (size == 1) return(vec_recycle(x, n))

  if (name == "") {
    name <- j
  }

  abort_incompatible_size(n, name, size, "Existing data", call)
}

# Errors ------------------------------------------------------------------

abort_tibble_row_size_one <- function(j, name, size, call = caller_env()) {
  if (name != "") {
    desc <- tick(name)
  } else {
    desc <- paste0("at position ", j)
  }

  tibble_abort(call = call, problems(
    "All vectors must be size one, use `list()` to wrap.",
    paste0("Column ", desc, " is of size ", size, ".")
  ))
}

abort_incompatible_size <- function(.rows, vars, vars_len, rows_source, call = caller_env()) {
  vars_split <- split(vars, vars_len)

  vars_split[["1"]] <- NULL
  if (!is.null(.rows)) {
    vars_split[[as.character(.rows)]] <- NULL
  }

  problems <- map2_chr(names(vars_split), vars_split, function(x, y) {
    if (is.numeric(y)) {
      text <- "Column(s) at position(s) "
    } else {
      text <- "Column(s) "
      y <- tick(y)
    }

    paste0("Size ", x, ": ", pluralise_commas(text, y))
  })

  tibble_abort(call = call, bullets(
    "Tibble columns must have compatible sizes:",
    if (!is.null(.rows)) paste0("Size ", .rows, ": ", rows_source),
    problems,
    info = "Only values of size one are recycled."
  ))
}

test_that("[.tbl_df supports subsetting with a logical matrix (#649)", {
  foo <- tibble(x = 1:10, y = 1:10, z = letters[1:10])

  m <- matrix(c(rep(FALSE, 8), rep(TRUE, 3), rep(FALSE, 19)), ncol = 3)
  expect_equal(foo[m], c(9, 10, 1))

  m <- matrix(c(rep(FALSE, 18), rep(TRUE, 3), rep(FALSE, 9)), ncol = 3)
  expect_error(foo[m], class = "vctrs_error_incompatible_type")
})

test_that("[<-.tbl_df supports subsetting with a logical matrix (#649)", {
  foo <- tibble(x = 1:10, y = 1:10, z = letters[1:10])

  m <- matrix(c(rep(FALSE, 8), rep(TRUE, 3), rep(FALSE, 19)), ncol = 3)
  foo[m] <- 1
  expect_equal(foo[m], c(1, 1, 1))

  expect_error(foo[m] <- 1:3)

  m <- matrix(c(rep(FALSE, 18), rep(TRUE, 3), rep(FALSE, 9)), ncol = 3)
  expect_snapshot(error = TRUE, {
    foo[m] <- 1
  })
})

local_options(lifecycle_verbosity = "quiet")

# [ -----------------------------------------------------------------------

test_that("[ never drops", {
  mtcars2 <- as_tibble(mtcars)

  expect_s3_class(mtcars2[, 1], "data.frame")
  expect_s3_class(mtcars2[, 1], "tbl_df")
  expect_equal(mtcars2[, 1], mtcars2[1])
})

test_that("[ retains class", {
  mtcars2 <- as_tibble(mtcars)

  expect_identical(class(mtcars2), class(mtcars2[1:5, ]))
  expect_identical(class(mtcars2), class(mtcars2[, 1:5]))
  expect_identical(class(mtcars2), class(mtcars2[1:5, 1:5]))
})

test_that("[ and as_tibble commute", {
  mtcars2 <- as_tibble(mtcars)
  expect_identical(mtcars2, as_tibble(mtcars))
  expect_identical(mtcars2[], remove_rownames(as_tibble(mtcars[])))
  expect_identical(mtcars2[1:5, ], remove_rownames(as_tibble(mtcars[1:5, ])))
  expect_identical(mtcars2[, 1:5], remove_rownames(as_tibble(mtcars[, 1:5])))
  expect_identical(mtcars2[1:5, 1:5], remove_rownames(as_tibble(mtcars[1:5, 1:5])))
  expect_identical(mtcars2[1:5], remove_rownames(as_tibble(mtcars[1:5])))
})

test_that("[ with 0 cols creates correct row names (#656)", {
  zero_row <- as_tibble(iris)[, 0]
  expect_s3_class(zero_row, "tbl_df")
  expect_equal(nrow(zero_row), 150)
  expect_equal(ncol(zero_row), 0)

  expect_identical(zero_row, as_tibble(iris)[0])
})

test_that("[ with 0 cols returns correct number of rows", {
  iris_tbl <- as_tibble(iris)
  nrow_iris <- nrow(iris_tbl)

  expect_equal(nrow(iris_tbl[0]), nrow_iris)
  expect_equal(nrow(iris_tbl[, 0]), nrow_iris)

  expect_equal(nrow(iris_tbl[, 0][1:10, ]), 10)
  expect_equal(nrow(iris_tbl[0][1:10, ]), 10)
  expect_equal(nrow(iris_tbl[1:10, ][, 0]), 10)
  expect_equal(nrow(iris_tbl[1:10, ][0]), 10)
  expect_equal(nrow(iris_tbl[1:10, 0]), 10)

  expect_equal(nrow(iris_tbl[, 0][-(1:10), ]), nrow_iris - 10)
  expect_equal(nrow(iris_tbl[0][-(1:10), ]), nrow_iris - 10)
  expect_equal(nrow(iris_tbl[-(1:10), ][, 0]), nrow_iris - 10)
  expect_equal(nrow(iris_tbl[-(1:10), ][0]), nrow_iris - 10)
  expect_equal(nrow(iris_tbl[-(1:10), 0]), nrow_iris - 10)
})

test_that("[.tbl_df is careful about names (#1245)", {
  foo <- tibble(x = 1:10, y = 1:10)
  expect_legacy_error(
    foo["z"],
    error_unknown_names("z"),
    fixed = TRUE
  )
  expect_legacy_error(
    foo[c("x", "y", "z")],
    error_unknown_names("z"),
    fixed = TRUE
  )

  expect_legacy_error(
    foo[, "z"],
    error_unknown_names("z"),
    fixed = TRUE
  )
  expect_legacy_error(
    foo[, c("x", "y", "z")],
    error_unknown_names("z"),
    fixed = TRUE
  )

  expect_legacy_error(
    foo[as.matrix("x")],
    error_dim_column_index(as.matrix("x")),
    fixed = TRUE
  )
  expect_legacy_error(
    foo[array("x", dim = c(1, 1, 1))],
    error_dim_column_index(array("x", dim = c(1, 1, 1))),
    fixed = TRUE
  )
})

test_that("[.tbl_df is careful about column indexes (#83)", {
  foo <- tibble(x = 1:10, y = 1:10, z = 1:10)
  expect_identical(foo[1:3], foo)

  expect_legacy_error(
    foo[0.5],
    error_nonint_column_index(1, 0.5),
    fixed = TRUE
  )
  expect_legacy_error(
    foo[1:5],
    error_large_column_index(3, 4:5, 4:5),
    fixed = TRUE
  )

  # Message from base R
  expect_error(foo[-1:1])
  expect_error(foo[c(-1, 1)])

  expect_legacy_error(
    foo[-4],
    error_small_column_index(3, 1, -4),
    fixed = TRUE
  )
  expect_legacy_error(
    foo[c(1:3, NA)],
    error_na_column_index(),
    fixed = TRUE
  )

  expect_legacy_error(
    foo[as.matrix(1)],
    error_dim_column_index(as.matrix("x")),
    fixed = TRUE
  )
  expect_legacy_error(
    foo[array(1, dim = c(1, 1, 1))],
    error_dim_column_index(array("x", dim = c(1, 1, 1))),
    fixed = TRUE
  )
})

test_that("[.tbl_df is careful about column flags (#83)", {
  foo <- tibble(x = 1:10, y = 1:10, z = 1:10)
  expect_identical(foo[TRUE], foo)
  expect_identical(foo[c(TRUE, TRUE, TRUE)], foo)
  expect_identical(foo[FALSE], foo[integer()])
  expect_identical(foo[c(FALSE, TRUE, FALSE)], foo[2])

  expect_legacy_error(
    foo[c(TRUE, TRUE)],
    error_mismatch_column_flag(3, 2),
    fixed = TRUE
  )
  expect_legacy_error(
    foo[c(TRUE, TRUE, FALSE, FALSE)],
    error_mismatch_column_flag(3, 4),
    fixed = TRUE
  )
  expect_legacy_error(
    foo[c(TRUE, TRUE, NA)],
    error_na_column_flag(),
    fixed = TRUE
  )

  expect_legacy_error(
    foo[as.matrix(TRUE)],
    error_dim_column_index(as.matrix("x")),
    fixed = TRUE
  )
  expect_legacy_error(
    foo[array(TRUE, dim = c(1, 1, 1))],
    error_dim_column_index(array("x", dim = c(1, 1, 1))),
    fixed = TRUE
  )
})

test_that("[.tbl_df rejects unknown column indexes (#83)", {
  foo <- tibble(x = 1:10, y = 1:10, z = 1:10)
  expect_legacy_error(
    foo[list(1:3)],
    error_unsupported_index(list(1:3)),
    fixed = TRUE
  )
  expect_legacy_error(
    foo[as.list(1:3)],
    error_unsupported_index(as.list(1:3)),
    fixed = TRUE
  )
  expect_legacy_error(
    foo[factor(1:3)],
    error_unsupported_index(factor(1:3)),
    fixed = TRUE
  )
  expect_legacy_error(
    foo[Sys.Date()],
    error_unsupported_index(Sys.Date()),
    fixed = TRUE
  )
  expect_legacy_error(
    foo[Sys.time()],
    error_unsupported_index(Sys.time()),
    fixed = TRUE
  )
})

test_that("[.tbl_df supports character subsetting (#312)", {
  foo <- tibble(x = 1:10, y = 1:10, z = 1:10)
  expect_identical(foo[as.character(2:4), ], foo[2:4, ])
  skip_brk_character_subsetting_no_negative()
  expect_identical(foo[as.character(-3:-5), ], foo[-3:-5, ])
})

test_that("[.tbl_df supports character subsetting (#312)", {
  foo <- tibble(x = 1:10, y = 1:10, z = 1:10)
  skip_dep_rowname_subsetting_warning()
  expect_identical(foo[as.character(9:12), ], foo[9:12, ])
})

test_that("[.tbl_df supports character subsetting (#312)", {
  foo <- tibble(x = 1:10, y = 1:10, z = 1:10)
  skip_dep_rowname_subsetting_warning()
  expect_identical(foo[letters, ], foo[rlang::rep_along(letters, NA_integer_), ])
  expect_identical(foo["9a", ], foo[NA_integer_, ])
})

test_that("[.tbl_df supports character subsetting if row names are present (#312)", {
  foo <- as_tibble(mtcars, rownames = NA)
  idx <- function(x) rownames(mtcars)[x]
  expect_identical(foo[idx(2:4), ], foo[2:4, ])
  expect_identical(foo[idx(-3:-5), ], foo[-3:-5, ])
  skip_dep_oob_subsetting_warning()
  expect_identical(foo[idx(29:34), ], foo[29:34, ])
})

test_that("[.tbl_df supports character subsetting if row names are present (#312)", {
  foo <- as_tibble(mtcars, rownames = NA)
  skip_dep_rowname_subsetting_warning()
  expect_identical(foo[letters, ], foo[rlang::rep_along(letters, NA_integer_), ])
  expect_identical(foo["9a", ], foo[NA_integer_, ])
})

test_that("[.tbl_df supports logical subsetting (#318)", {
  foo <- tibble(x = 1:10, y = 1:10, z = 1:10)
  expect_identical(foo[c(FALSE, rep(TRUE, 3), rep(F, 6)), ], foo[2:4, ])
  expect_identical(foo[TRUE, ], foo)
  expect_identical(foo[FALSE, ], foo[0L, ])
  skip_brk_logical_subsetting_no_base_recycling()
  expect_warning(
    foo[c(TRUE, FALSE), ],
    "Length of logical index must be 1 or 10, not 2",
    fixed = TRUE
  )
})

test_that("[.tbl_df is no-op if args missing", {
  expect_identical(df_all[], df_all)
})

test_that("[.tbl_df supports drop argument (#311)", {
  expect_identical(df_all[1, 2, drop = TRUE], df_all[[2]][1])
  expect_identical(df_all[1, , drop = TRUE], df_all[1, , ])
})

test_that("[.tbl_df ignores drop argument (with warning) without j argument (#307)", {
  expect_warning(expect_identical(df_all[1, drop = TRUE], df_all[1]))
})


test_that("[.tbl_df is careful about attributes (#155)", {
  df <- tibble(x = 1:2, y = x)
  attr(df, "along for the ride") <- "still here"

  expect_identical(attr(df[names(df)], "along for the ride"), "still here")
  expect_identical(attr(df["x"], "along for the ride"), "still here")
  expect_identical(attr(df[1:2], "along for the ride"), "still here")
  expect_identical(attr(df[2], "along for the ride"), "still here")
  expect_identical(attr(df[c(TRUE, FALSE)], "along for the ride"), "still here")
  expect_identical(attr(df[, names(df)], "along for the ride"), "still here")
  expect_identical(attr(df[, "x"], "along for the ride"), "still here")
  expect_identical(attr(df[, 1:2], "along for the ride"), "still here")
  expect_identical(attr(df[, 2], "along for the ride"), "still here")
  expect_identical(attr(df[, c(TRUE, FALSE)], "along for the ride"), "still here")
  expect_identical(attr(df[1, names(df)], "along for the ride"), "still here")
  expect_identical(attr(df[1, "x"], "along for the ride"), "still here")
  expect_identical(attr(df[1, 1:2], "along for the ride"), "still here")
  expect_identical(attr(df[1, 2], "along for the ride"), "still here")
  expect_identical(attr(df[1, c(TRUE, FALSE)], "along for the ride"), "still here")

  expect_identical(attr(df[1:2, ], "along for the ride"), "still here")
  expect_identical(attr(df[-1, ], "along for the ride"), "still here")

  expect_identical(attr(df[, ], "along for the ride"), "still here")
  expect_identical(attr(df[], "along for the ride"), "still here")
})

# [[ ----------------------------------------------------------------------

test_that("[[.tbl_df ignores exact argument", {
  foo <- tibble(x = 1:10, y = 1:10)
  expect_warning(foo[["x"]], NA)
  expect_warning(foo[["x", exact = FALSE]], "ignored")
  expect_identical(getElement(foo, "y"), 1:10)
})

test_that("can use recursive indexing with [[", {
  foo <- tibble(x = list(y = 1:3, z = 4:5))
  skip_brk_no_recursive_indexing()
  expect_equal(foo[[c(1, 1)]], 1:3)
  expect_equal(foo[[c("x", "y")]], 1:3)
})

test_that("[[ returns NULL if name doesn't exist", {
  df <- tibble(x = 1)
  expect_null(df[["y"]])
  expect_null(df[[1, "y"]])
})

test_that("can use two-dimensional indexing with [[", {
  iris2 <- as_tibble(iris)
  expect_equal(iris2[[1, 2]], iris[[1, 2]])
  expect_equal(iris2[[2, 3]], iris[[2, 3]])
})

# $ -----------------------------------------------------------------------

test_that("$ throws warning if name doesn't exist", {
  df <- tibble(x = 1)
  expect_legacy_warning(
    expect_null(df$y),
    "Unknown or uninitialised column: 'y'"
  )
})

test_that("$ throws different warning if attempting a partial initialization (#199)", {
  df <- tibble(x = 1)
  expect_legacy_warning(df$y[1] <- 2, "Unknown or uninitialised column: 'y'")
})

test_that("$ doesn't do partial matching", {
  df <- tibble(partial = 1)
  expect_legacy_warning(
    expect_null(df$p),
    "Unknown or uninitialised column: 'p'"
  )
  expect_legacy_warning(
    expect_null(df$part),
    "Unknown or uninitialised column: 'part'"
  )
  expect_error(df$partial, NA)
})

# is_tibble ---------------------------------------------------------------

test_that("is_tibble", {
  expect_false(is_tibble(iris))
  expect_true(is_tibble(as_tibble(iris)))
  expect_false(is_tibble(NULL))
  expect_false(is_tibble(0))
})

test_that("is_tibble", {
  scoped_lifecycle_silence()
  expect_identical(is.tibble(iris), is_tibble(iris))
})

# new_tibble --------------------------------------------------------------

test_that("new_tibble() with deprecated subclass argument", {
  skip_dep_new_tibble_subclass()

  tbl <- new_tibble(
    data.frame(a = 1:3),
    names = "b",
    attr1 = "value1",
    attr2 = 2,
    nrow = 3,
    subclass = "nt"
  )

  # Can't compare directly due to dplyr:::all.equal.tbl_df()
  expect_identical(class(tbl), c("nt", "tbl_df", "tbl", "data.frame"))
  expect_equal(
    unclass(tbl),
    structure(
      list(b = 1:3),
      attr1 = "value1",
      attr2 = 2,
      .Names = "b",
      row.names = .set_row_names(3L)
    )
  )
})

test_that("new_tibble() with new class argument", {
  tbl <- new_tibble(
    data.frame(a = 1:3),
    names = "b",
    attr1 = "value1",
    attr2 = 2,
    nrow = 3,
    class = "nt"
  )

  # Can't compare directly due to dplyr:::all.equal.tbl_df()
  expect_identical(class(tbl), c("nt", "tbl_df", "tbl", "data.frame"))
  expect_equal(
    unclass(tbl),
    structure(
      list(b = 1:3),
      attr1 = "value1",
      attr2 = 2,
      .Names = "b",
      row.names = .set_row_names(3L)
    )
  )
})

test_that("new_tibble checks", {
  scoped_lifecycle_errors()

  expect_identical(new_tibble(list(), nrow = 0), tibble())
  expect_identical(new_tibble(list(), nrow = 5), tibble(.rows = 5))
  expect_identical(new_tibble(list(a = 1:3, b = 4:6), nrow = 3), tibble(a = 1:3, b = 4:6))
  expect_legacy_error(
    new_tibble(1:3, nrow = 1),
    error_new_tibble_must_be_list(),
    fixed = TRUE
  )
})

test_that("new_tibble checks (2)", {
  skip_enh_new_tibble_nrow_null()

  scoped_lifecycle_errors()

  expect_legacy_error(
    new_tibble(list(a = 1)),
    class = get_defunct_error_class(),
    error_new_tibble_needs_nrow(),
    fixed = TRUE
  )
  expect_legacy_error(
    new_tibble(list(1), nrow = NULL),
    error_new_tibble_needs_nrow(),
    fixed = TRUE
  )
})

test_that("new_tibble checks (3)", {
  scoped_lifecycle_errors()

  expect_legacy_error(
    new_tibble(list(1), nrow = 1),
    error_names_must_be_non_null(repair_hint = FALSE),
    fixed = TRUE
  )
  expect_error(
    new_tibble(set_names(list(1), NA_character_), nrow = 1),
    NA
  )
  expect_error(
    new_tibble(set_names(list(1), ""), nrow = 1),
    NA
  )
  expect_error(
    new_tibble(list(a = 1, b = 2:3), nrow = 1),
    NA
  )
  expect_error(
    new_tibble(
      structure(list(a = 1, b = 2), row.names = .set_row_names(2)),
      nrow = 1
    ),
    NA
  )
})


test_that("validate_tibble() checks", {
  expect_legacy_error(
    validate_tibble(new_tibble(list(a = 1, b = 2:3), nrow = 1)),
    error_inconsistent_cols(1, c("a", "b"), 1:2, "`nrow` argument"),
    fixed = TRUE
  )

  skip_brk_inner_dim_not_stripped()
  expect_error(
    validate_tibble(new_tibble(list(a = array(1:3)), nrow = 3)),
    error_1d_array_column(),
    fixed = TRUE
  )
})

invalid_df <- function(problem, vars, ...) {
  if (is.character(vars)) {
    vars <- tick(vars)
  }

  pluralise_commas(
    "Column(s) ",
    vars,
    paste0(" ", problem, ".", ...)
  )
}

use_repair <- function(repair_hint) {
  if (repair_hint) "\nUse .name_repair to specify repair."
}

tibble_error_class <- function(class) {
  c(paste0("tibble_error_", class), "tibble_error")
}

# Errors get a class name derived from the name of the calling function
tibble_error <- function(x, ..., parent = NULL) {
  call <- sys.call(-1)
  fn_name <- as_name(call[[1]])
  class <- tibble_error_class(gsub("^error_", "", fn_name))
  error_cnd(class, ..., message = x, parent = parent)
}

as_override_type_sum <- function(x) {
  structure(x, class = "override_type_sum")
}

type_sum.override_type_sum <- function(x, ...) {
  "SC"
}

registerS3method(
  "type_sum",
  "override_type_sum",
  type_sum.override_type_sum,
  envir = asNamespace("pillar")
)

`[.override_type_sum` <- function(x, ...) {
  as_override_type_sum(NextMethod())
}

registerS3method(
  "[",
  "override_type_sum",
  `[.override_type_sum`,
  envir = asNamespace("tibble")
)

as_override_tbl_sum <- function(x) {
  structure(x, class = c("override_tbl_sum", class(x)))
}

tbl_sum.override_tbl_sum <- function(x) {
  c(NextMethod(), "Overridden" = "tbl_sum")
}

registerS3method(
  "tbl_sum",
  "override_tbl_sum",
  tbl_sum.override_tbl_sum,
  envir = asNamespace("tibble")
)

# nocov start
.onLoad <- function(libname, pkgname) {
  if (getRversion() == "3.3.0") {
    safe_match <<- safe_match_3_0
  } else {
    safe_match <<- safe_match_default
  }

  num <<- pillar::num
  set_num_opts <<- pillar::set_num_opts
  char <<- pillar::char
  set_char_opts <<- pillar::set_char_opts
}

safe_match_3_0 <- function(x, table) {
  match(x, table, incomparables = character())
}

safe_match_default <- function(x, table) {
  match(x, table)
}
# nocov end

#' @useDynLib tibble, .registration = TRUE
#' @importFrom utils head tail
#' @importFrom pkgconfig set_config
#' @import rlang
#' @import ellipsis
#' @importFrom lifecycle deprecate_soft deprecate_warn expect_deprecated
#' @importFrom vctrs vec_as_location vec_as_location2 vec_as_names vec_as_names_legacy vec_c
#' @importFrom vctrs vec_is vec_rbind vec_recycle vec_size vec_slice vec_assign
#' @importFrom vctrs unspecified vec_as_subscript2 num_as_location vec_ptype_abbr
#' @importFrom vctrs vec_names vec_names2 vec_set_names
#' @importFrom vctrs new_rcrd
#' @importFrom vctrs new_data_frame
#' @aliases NULL tibble-package
#' @details
#' `r lifecycle::badge("stable")`
#'
#' The tibble package provides utilities for handling __tibbles__, where
#' "tibble" is a colloquial term for the S3 [`tbl_df`] class. The [`tbl_df`]
#' class is a special case of the base [`data.frame`][base::data.frame()].
#' class, developed in response to lessons learned over many years of data
#' analysis with data frames.
#'
#' Tibble is the central data structure for the set of packages known as the
#' [tidyverse](https://www.tidyverse.org/packages/), including
#' [dplyr](https://dplyr.tidyverse.org/),
#' [ggplot2](https://ggplot2.tidyverse.org/),
#' [tidyr](https://tidyr.tidyverse.org/), and
#' [readr](https://readr.tidyverse.org/).
#'
#' General resources:
#'   * Website for the tibble package: <https://tibble.tidyverse.org>
#'   * [Tibbles chapter](https://r4ds.had.co.nz/tibbles.html) in *R for Data
#'     Science*
#'
#' Resources on specific topics:
#'   * Create a tibble: [tibble()], [as_tibble()], [tribble()], [enframe()]
#'   * Inspect a tibble: [print.tbl()], [glimpse()]
#'   * Details on the S3 `tbl_df` class: [`tbl_df-class`]
#'   * Package options: [tibble_options]
"_PACKAGE"

#' Tools for working with row names
#'
#' @description
#' While a tibble can have row names (e.g., when converting from a regular data
#' frame), they are removed when subsetting with the `[` operator.
#' A warning will be raised when attempting to assign non-`NULL` row names
#' to a tibble.
#' Generally, it is best to avoid row names, because they are basically a
#' character column with different semantics than every other column.
#'
#' These functions allow to you detect if a data frame has row names
#' (`has_rownames()`), remove them (`remove_rownames()`), or convert
#' them back-and-forth between an explicit column (`rownames_to_column()`
#' and `column_to_rownames()`).
#' Also included is `rowid_to_column()`, which adds a column at the start of the
#' dataframe of ascending sequential row ids starting at 1. Note that this will
#' remove any existing row names.
#'
#' @return `column_to_rownames()` always returns a data frame.
#'   `has_rownames()` returns a scalar logical.
#'   All other functions return an object of the same class as the input.
#'
#' @param .data A data frame.
#' @param var Name of column to use for rownames.
#' @examples
#' # Detect row names ----------------------------------------------------
#' has_rownames(mtcars)
#' has_rownames(trees)
#'
#' # Remove row names ----------------------------------------------------
#' remove_rownames(mtcars) %>% has_rownames()
#'
#' # Convert between row names and column --------------------------------
#' mtcars_tbl <- rownames_to_column(mtcars, var = "car") %>% as_tibble()
#' mtcars_tbl
#' column_to_rownames(mtcars_tbl, var = "car") %>% head()
#'
#' # Adding rowid as a column --------------------------------------------
#' rowid_to_column(trees) %>% head()
#'
#' @name rownames
NULL


#' @export
#' @rdname rownames
has_rownames <- function(.data) {
  .row_names_info(.data) > 0L && !is.na(.row_names_info(.data, 0L)[[1L]])
}

#' @export
#' @rdname rownames
remove_rownames <- function(.data) {
  stopifnot(is.data.frame(.data))
  rownames(.data) <- NULL
  .data
}

#' @export
#' @rdname rownames
rownames_to_column <- function(.data, var = "rowname") {
  # rename, because .data has special semantics in tidy evaluation
  df <- .data

  stopifnot(is.data.frame(df))

  # Side effect: check unique names
  repaired_names(c(unique(names2(df)), var), repair_hint = FALSE)

  new_df <- add_column(df, !!var := rownames(df), .before = 1)
  remove_rownames(new_df)
}

#' @export
#' @rdname rownames
rowid_to_column <- function(.data, var = "rowid") {
  # rename, because .data has special semantics in tidy evaluation
  df <- .data

  stopifnot(is.data.frame(df))

  # Side effect: check unique names
  repaired_names(c(unique(names2(df)), var), repair_hint = FALSE)

  new_df <- add_column(df, !!var := seq_len(nrow(df)), .before = 1)
  remove_rownames(new_df)
}

#' @rdname rownames
#' @export
column_to_rownames <- function(.data, var = "rowname") {
  stopifnot(is.data.frame(.data))

  if (has_rownames(.data)) {
    abort_already_has_rownames()
  }

  if (!has_name(.data, var)) {
    abort_unknown_column_names(var)
  }

  .data <- as.data.frame(.data)
  rownames(.data) <- .data[[var]]
  .data[[var]] <- NULL
  .data
}

#' @export
`row.names<-.tbl_df` <- function(x, value) {
  if (!is.null(value)) {
    warn("Setting row names on a tibble is deprecated.")
  }
  NextMethod()
}

raw_rownames <- function(x) {
  .row_names_info(x, 0L) %||% .set_row_names(.row_names_info(x, 2L))
}

# Errors ------------------------------------------------------------------

abort_already_has_rownames <- function(call = caller_env()) {
  tibble_abort(call = call, "`.data` must be a data frame without row names.")
}

test_that("correct rows and cols", {
  x <- matrix(1:6, nrow = 2)
  out <- as_tibble(x, .name_repair = "minimal")

  expect_equal(dim(out), c(2, 3))
})

test_that("correct rows and cols for 0 cols", {
  x <- matrix(integer(), nrow = 2)
  out <- as_tibble(x, .name_repair = "minimal")

  expect_equal(dim(out), c(2, 0))
})

test_that("correct rows and cols for 0 cols and legacy naming", {
  scoped_lifecycle_silence()

  x <- matrix(integer(), nrow = 2)
  out <- as_tibble(x)

  expect_equal(dim(out), c(2, 0))
})

test_that("correct rows and cols for 0 rows", {
  x <- matrix(integer(), ncol = 3)
  out <- as_tibble(x, .name_repair = "minimal")

  expect_equal(dim(out), c(0, 3))
})

test_that("preserves col names", {
  x <- matrix(1:4, nrow = 2)
  colnames(x) <- c("a", "b")

  out <- as_tibble(x)
  expect_equal(names(out), c("a", "b"))
})

test_that("supports compat col names", {
  scoped_lifecycle_silence()

  x <- matrix(1:4, nrow = 2)

  out <- as_tibble(x)
  expect_equal(names(out), c("V1", "V2"))
})

test_that("creates col names with name repair", {
  x <- matrix(1:4, nrow = 2)

  suppressMessages(out <- as_tibble(x, .name_repair = "unique"))
  expect_equal(names(out), c("...1", "...2"))

  suppressMessages(out <- as_tibble(x, .name_repair = "universal"))
  expect_equal(names(out), c("...1", "...2"))
})

test_that("preserves attributes except dim and names", {
  date <- Sys.Date() + 0:3
  dim(date) <- c(2, 2)
  colnames(date) <- c("a", "b")
  attr(date, "special") <- 42

  out <- as_tibble.matrix(date)
  expect_null(attributes(out[[1]])$names)
  expect_equal(attributes(out[[1]])$class, "Date")
  expect_equal(attributes(out[[2]])$special, 42)
})

test_that("properly handles poly class (#110)", {
  p <- poly(1:6, 3)
  p_df <- as_tibble(p)

  expect_equal(names(p_df), colnames(p))
  expect_equal(class(p_df[[1L]]), class(p[, 1]))
})

test_that("handles atomic vectors", {
  x <- matrix(TRUE, nrow = 2)
  out <- as_tibble(x, .name_repair = "minimal")
  expect_equal(out[[1]], c(TRUE, TRUE))

  x <- matrix(1L, nrow = 2)
  out <- as_tibble(x, .name_repair = "minimal")
  expect_equal(out[[1]], c(1L, 1L))

  x <- matrix(1.5, nrow = 2)
  out <- as_tibble(x, .name_repair = "minimal")
  expect_equal(out[[1]], c(1.5, 1.5))

  x <- matrix("a", nrow = 2)
  out <- as_tibble(x, .name_repair = "minimal")
  expect_equal(out[[1]], c("a", "a"))

  x <- matrix(complex(real = 1, imaginary = 2), nrow = 2)
  out <- as_tibble(x, .name_repair = "minimal")
  expect_equal(out[[1]], as.vector(x))
})

test_that("forwarding to as.data.frame() for ts objects (#184)", {
  mts <- cbind(
    A = ts(c(1, 1, 2, 2),     start = 2016, frequency = 4),
    B = ts(c(11, 11, 12, 13), start = 2016, frequency = 4)
  )
  expect_identical(as_tibble(mts), as_tibble(as.data.frame(mts)))
})


test_that("converting from matrix removes row names by default", {
  x <- matrix(1:30, 6, 5, dimnames = list(letters[1:6], LETTERS[1:5]))
  df <- data.frame(A = 1:6, B = 7:12, C = 13:18, D = 19:24, E = 25:30)
  out <- as_tibble(x)
  expect_false(has_rownames(out))
  expect_identical(out, as_tibble(df))
})

test_that("converting from matrix keeps row names if argument has them, with rownames = NA", {
  x <- matrix(1:30, 6, 5, dimnames = list(letters[1:6], LETTERS[1:5]))
  df <- data.frame(
    A = 1:6, B = 7:12, C = 13:18, D = 19:24, E = 25:30,
    row.names = letters[1:6]
  )

  out <- as_tibble(x, rownames = NA)
  expect_identical(rownames(out), rownames(x))
  expect_identical(remove_rownames(out), as_tibble(df))
})

test_that("converting from matrix supports storing row names in a column", {
  x <- matrix(1:30, 6, 5, dimnames = list(letters[1:6], LETTERS[1:5]))
  df <- tibble(id = letters[1:6], A = 1:6, B = 7:12, C = 13:18, D = 19:24, E = 25:30)
  out <- as_tibble(x, rownames = "id")
  expect_identical(out, df)
})

test_that("converting from matrix throws an error if user turns missing row names into column", {
  skip_enh_as_tibble_rownames()

  x <- matrix(1:30, 6, 5)
  expect_error(
    as_tibble(x, rownames = "id", .name_repair = "minimal"),
    error_as_tibble_needs_rownames(),
    fixed = TRUE
  )
})

test_that("correct rows and cols", {
  x <- matrix(1:6, nrow = 2)
  out <- as_tibble(x, .name_repair = "minimal")

  expect_equal(dim(out), c(2, 3))
})

test_that("correct rows and cols for 0 cols", {
  x <- matrix(integer(), nrow = 2)
  out <- as_tibble(x, .name_repair = "minimal")

  expect_equal(dim(out), c(2, 0))
})

test_that("correct rows and cols for 0 cols and legacy naming", {
  scoped_lifecycle_silence()

  x <- matrix(integer(), nrow = 2)
  out <- as_tibble(x)

  expect_equal(dim(out), c(2, 0))
})

test_that("correct rows and cols for 0 rows", {
  x <- matrix(integer(), ncol = 3)
  out <- as_tibble(x, .name_repair = "minimal")

  expect_equal(dim(out), c(0, 3))
})

test_that("preserves col names", {
  x <- matrix(1:4, nrow = 2)
  colnames(x) <- c("a", "b")

  out <- as_tibble(x)
  expect_equal(names(out), c("a", "b"))
})

test_that("supports compat col names", {
  scoped_lifecycle_silence()

  x <- matrix(1:4, nrow = 2)

  out <- as_tibble(x)
  expect_equal(names(out), c("V1", "V2"))
})

test_that("creates col names with name repair", {
  x <- matrix(1:4, nrow = 2)

  suppressMessages(out <- as_tibble(x, .name_repair = "unique"))
  expect_equal(names(out), c("...1", "...2"))

  suppressMessages(out <- as_tibble(x, .name_repair = "universal"))
  expect_equal(names(out), c("...1", "...2"))
})

test_that("preserves attributes except dim and names", {
  date <- Sys.Date() + 0:3
  dim(date) <- c(2, 2)
  colnames(date) <- c("a", "b")
  attr(date, "special") <- 42

  out <- as_tibble.matrix(date)
  expect_null(attributes(out[[1]])$names)
  expect_equal(attributes(out[[1]])$class, "Date")
  expect_equal(attributes(out[[2]])$special, 42)
})

test_that("properly handles poly class (#110)", {
  p <- poly(1:6, 3)
  p_df <- as_tibble(p)

  expect_equal(names(p_df), colnames(p))
  expect_equal(class(p_df[[1L]]), class(p[, 1]))
})

test_that("handles atomic vectors", {
  x <- matrix(TRUE, nrow = 2)
  out <- as_tibble(x, .name_repair = "minimal")
  expect_equal(out[[1]], c(TRUE, TRUE))

  x <- matrix(1L, nrow = 2)
  out <- as_tibble(x, .name_repair = "minimal")
  expect_equal(out[[1]], c(1L, 1L))

  x <- matrix(1.5, nrow = 2)
  out <- as_tibble(x, .name_repair = "minimal")
  expect_equal(out[[1]], c(1.5, 1.5))

  x <- matrix("a", nrow = 2)
  out <- as_tibble(x, .name_repair = "minimal")
  expect_equal(out[[1]], c("a", "a"))

  x <- matrix(complex(real = 1, imaginary = 2), nrow = 2)
  out <- as_tibble(x, .name_repair = "minimal")
  expect_equal(out[[1]], as.vector(x))
})

test_that("forwarding to as.data.frame() for ts objects (#184)", {
  mts <- cbind(
    A = ts(c(1, 1, 2, 2), start = 2016, frequency = 4),
    B = ts(c(11, 11, 12, 13), start = 2016, frequency = 4)
  )
  expect_identical(as_tibble(mts), as_tibble(as.data.frame(mts)))
})


test_that("converting from matrix removes row names by default", {
  x <- matrix(1:30, 6, 5, dimnames = list(letters[1:6], LETTERS[1:5]))
  df <- data.frame(A = 1:6, B = 7:12, C = 13:18, D = 19:24, E = 25:30)
  out <- as_tibble(x)
  expect_false(has_rownames(out))
  expect_identical(out, as_tibble(df))
})

test_that("converting from matrix keeps row names if argument has them, with rownames = NA", {
  x <- matrix(1:30, 6, 5, dimnames = list(letters[1:6], LETTERS[1:5]))
  df <- data.frame(
    A = 1:6, B = 7:12, C = 13:18, D = 19:24, E = 25:30,
    row.names = letters[1:6]
  )

  out <- as_tibble(x, rownames = NA)
  expect_identical(rownames(out), rownames(x))
  expect_identical(remove_rownames(out), as_tibble(df))
})

test_that("converting from matrix supports storing row names in a column", {
  x <- matrix(1:30, 6, 5, dimnames = list(letters[1:6], LETTERS[1:5]))
  df <- tibble(id = letters[1:6], A = 1:6, B = 7:12, C = 13:18, D = 19:24, E = 25:30)
  out <- as_tibble(x, rownames = "id")
  expect_identical(out, df)
})

test_that("converting from matrix throws an error if user turns missing row names into column", {
  skip_enh_as_tibble_rownames()

  x <- matrix(1:30, 6, 5)
  expect_error(
    as_tibble(x, rownames = "id", .name_repair = "minimal"),
    error_as_tibble_needs_rownames(),
    fixed = TRUE
  )
})

#' @importFrom methods setOldClass
#' @exportClass tbl_df
setOldClass(c("tbl_df", "tbl", "data.frame"))

#' `tbl_df` class
#'
#' @description
#' The `tbl_df` class is a subclass of [`data.frame`][base::data.frame()],
#' created in order to have different default behaviour. The colloquial term
#' "tibble" refers to a data frame that has the `tbl_df` class. Tibble is the
#' central data structure for the set of packages known as the
#' [tidyverse](https://www.tidyverse.org/packages/), including
#' [dplyr](https://dplyr.tidyverse.org/),
#' [ggplot2](https://ggplot2.tidyverse.org/),
#' [tidyr](https://tidyr.tidyverse.org/), and
#' [readr](https://readr.tidyverse.org/).
#'
#' The general ethos is that tibbles are lazy and surly: they do less and
#' complain more than base [data.frame]s. This forces
#' problems to be tackled earlier and more explicitly, typically leading to code
#' that is more expressive and robust.
#'
#' @section Properties of `tbl_df`:
#'
#' Objects of class `tbl_df` have:
#' * A `class` attribute of `c("tbl_df", "tbl", "data.frame")`.
#' * A base type of `"list"`, where each element of the list has the same
#'   [vctrs::vec_size()].
#' * A `names` attribute that is a character vector the same length as the
#'   underlying list.
#' * A `row.names` attribute, included for compatibility with [data.frame].
#'   This attribute is only consulted to query the number of rows,
#'   any row names that might be stored there are ignored
#'   by most tibble methods.
#'
#' @section Behavior of `tbl_df`:
#'
#' How default behaviour of tibbles differs from that of
#' [data.frame]s, during creation and access:
#'
#' * Column data is not coerced. A character vector is not turned into a factor.
#'   List-columns are expressly anticipated and do not require special tricks.
#'   Internal names are never stripped from column data.
#'   Read more in [tibble()].
#' * Recycling only happens for a length 1 input.
#'   Read more in [vctrs::vec_recycle()].
#' * Column names are not munged, although missing names are auto-populated.
#'   Empty and duplicated column names are strongly discouraged, but the user
#'   must indicate how to resolve. Read more in [vctrs::vec_as_names()].
#' * Row names are not added and are strongly discouraged, in favor of storing
#'   that info as a column. Read about in [rownames].
#' * `df[, j]` returns a tibble; it does not automatically extract the column
#'   inside. `df[, j, drop = FALSE]` is the default. Read more in [subsetting].
#' * There is no partial matching when `$` is used to index by name. `df$name`
#'   for a nonexistent name generates a warning. Read more in [subsetting].
#'
#' See `vignette("invariants")` for a detailed description of the behavior.
#'
#' Furthermore, printing and inspection are a very high priority.
#' The goal is to convey as much information as possible, in a concise way,
#' even for large and complex tibbles. Read more in [formatting].
#'
#' @name tbl_df-class
#' @aliases tbl_df tbl_df-class
#' @seealso [tibble()], [as_tibble()], [tribble()], [print.tbl()],
#'   [glimpse()]
NULL

# Standard data frame methods --------------------------------------------------

#' @export
as.data.frame.tbl_df <- function(x, row.names = NULL, optional = FALSE, ...) {
  class(x) <- tibble_class
  unname <- which(!map_lgl(x, is_bare_list))
  x[unname] <- map(.subset(x, unname), vectbl_set_names, NULL)
  class(x) <- "data.frame"
  x
}

#' @export
`names<-.tbl_df` <- function(x, value) {

  # workaround for RStudio v1.1, which relies on the ability to set
  # data.frame names to NULL
  if (is.null(value) && is_rstudio()) {
    attr(x, "names") <- NULL
    return(x)
  }

  cnd <- cnd_names_non_null(value)
  if (!is.null(cnd)) {
    deprecate_soft("3.0.0", "tibble::`names<-`(value = 'can\\'t be NULL')",
      details = cnd$message
    )

    # FIXME: value <- rep("", length(x))
  }

  if (!has_length(value, length(x))) {
    deprecate_soft("3.0.0", "tibble::`names<-`(value = 'must have the same length as `x`')",
      details = error_names_must_have_length(length(value), length(x))$message
    )

    # FIXME: Reset NA to "" in names

    if (length(value) < length(x)) {
      value <- c(value, rep(NA_character_, length(x) - length(value)))
    } else {
      length(value) <- length(x)
    }
  }

  cnd <- cnd_names_non_na(value)
  if (!is.null(cnd)) {
    deprecate_soft("3.0.0", "tibble::`names<-`(value = 'can\\'t be empty')",
      details = cnd$message
    )

    # FIXME: Reset NA to "" in names
  }

  if (!is_character(value)) {
    deprecate_soft("3.0.0", "tibble::`names<-`(value = 'must be a character vector')")
    value <- as.character(value)
  }

  attr(x, "names") <- as.character(value)
  x
}

cnd_names_non_null <- function(name) {
  if (is.null(name)) {
    error_names_must_be_non_null()
  } else {
    invisible()
  }
}

cnd_names_non_na <- function(name) {
  bad_name <- which(is.na(name))
  if (has_length(bad_name)) {
    error_column_names_cannot_be_empty(bad_name, repair_hint = FALSE)
  } else {
    invisible()
  }
}

# Errors ------------------------------------------------------------------

error_names_must_be_non_null <- function() {
  tibble_error("`names` must not be `NULL`.")
}

error_names_must_have_length <- function(length, n) {
  tibble_error(
    paste0("`names` must have length ", n, ", not ", length, "."),
    expected = n,
    actual = length
  )
}

# minimal names -------------------------------------------------------------
test_that("minimal names are made from `n` when `name = NULL`", {
  expect_identical(minimal_names(NULL, 2), c("", ""))
  expect_error(
    minimal_names(NULL),
    error_name_length_required(),
    fixed = TRUE
  )
})

test_that("minimal names have '' instead of NAs", {
  expect_identical(minimal_names(c("", NA, "", NA)), c("", "", "", ""))
})

test_that("set_minimal_names() copes with NULL input names", {
  x <- 1:3
  x_named <- set_minimal_names(x)
  expect_equal(names(x_named), rep("", 3))
})

test_that("check_minimal() errors when names aren't minimal", {
  expect_legacy_error(
    check_minimal(NULL),
    error_names_must_be_non_null(repair_hint = FALSE),
    fixed = TRUE
  )
  expect_legacy_error(
    check_minimal(c("a", NA)),
    error_column_must_be_named(2, repair_hint = FALSE),
    fixed = TRUE
  )
})

test_that("minimal_names() is idempotent", {
  x <- c("", "", NA)
  expect_identical(minimal_names(x), minimal_names(minimal_names(x)))
})

# unique names -------------------------------------------------------------
test_that("unique_names() eliminates emptiness and duplication", {
  x <- c("", "x", "y", "x")
  expect_identical(unique_names(x), c("...1", "x...2", "y", "x...4"))
})

test_that("solo empty or NA gets suffix", {
  expect_identical(unique_names(""), "...1")
  expect_identical(unique_names(NA_character_), "...1")
})

test_that("ellipsis treated like empty string", {
  expect_identical(unique_names("..."), unique_names(""))
})

test_that("two_three_dots() does its job and no more", {
  x <- c(".", ".1", "...1", "..1a")
  expect_identical(two_to_three_dots(x), x)

  expect_identical(two_to_three_dots(c("..1", "..22")), c("...1", "...22"))
})

test_that("two dots then number treated like three dots then number", {
  expect_identical(unique_names("..2"), unique_names("...5"))
})

test_that("unique_names() strips positional suffixes, re-applies as needed", {
  x <- c("...20", "a...1", "b", "", "a...2...34")
  expect_identical(unique_names(x), c("...1", "a...2", "b", "...4", "a...5"))

  expect_identical(unique_names("a...1"), "a")
  expect_identical(unique_names(c("a...2", "a")), c("a...1", "a...2"))
  expect_identical(unique_names(c("a...3", "a", "a")), c("a...1", "a...2", "a...3"))
  expect_identical(unique_names(c("a...2", "a", "a")), c("a...1", "a...2", "a...3"))
  expect_identical(unique_names(c("a...2", "a...2", "a...2")), c("a...1", "a...2", "a...3"))
})

test_that("check_unique() imposes check_minimal()", {
  expect_legacy_error(
    check_unique(NULL),
    error_names_must_be_non_null(repair_hint = FALSE),
    fixed = TRUE
  )

  expect_legacy_error(
    check_unique(c("x", NA)),
    error_column_must_be_named(2, repair_hint = FALSE),
    fixed = TRUE
  )
})

test_that("check_unique() errors for empty or duplicated names", {
  expect_legacy_error(
    check_unique(c("x", "")),
    error_column_must_be_named(2, repair_hint = FALSE),
    fixed = TRUE
  )
  expect_legacy_error(
    check_unique(c("", "x", "")),
    error_column_must_be_named(c(1, 3), repair_hint = FALSE),
    fixed = TRUE
  )
  expect_legacy_error(
    check_unique(c("x", "x", "y")),
    error_column_names_must_be_unique("x", repair_hint = FALSE),
    fixed = TRUE
  )
  expect_legacy_error(
    check_unique(c("x", "y", "x", "y")),
    error_column_names_must_be_unique(c("x", "y"), repair_hint = FALSE),
    fixed = TRUE
  )
})

test_that("unique_names() is idempotent", {
  x <- c("...20", "a...1", "b", "", "a...2")
  expect_identical(unique_names(!!x), unique_names(unique_names(!!x)))
})

test_that("unique-ification has an 'algebraic'-y property", {
  ## inspired by, but different from, this guarantee about base::make.unique()
  ## make.unique(c(A, B)) == make.unique(c(make.unique(A), B))
  ## If A is already unique, then make.unique(c(A, B)) preserves A.

  ## I haven't formulated what we guarantee very well yet, but it's probably
  ## implicit in this test (?)

  x <- c("...20", "a...1", "b", "", "a...2", "d")
  y <- c("", "a...3", "b", "...3", "e")

  ## fix names on each, catenate, fix the whole
  z1 <- unique_names(
    c(
      unique_names(x), unique_names(y)
    )
  )

  ## fix names on x, catenate, fix the whole
  z2 <- unique_names(
    c(
      unique_names(x), y
    )
  )

  ## fix names on y, catenate, fix the whole
  z3 <- unique_names(
    c(
      x, unique_names(y)
    )
  )

  ## catenate, fix the whole
  z4 <- unique_names(
    c(
      x, y
    )
  )

  expect_identical(z1, z2)
  expect_identical(z1, z3)
  expect_identical(z1, z4)
})

# names<-() -------------------------------------------------------------------
test_that("names<-() and set_names() reject non-minimal names", {
  df <- tibble(a = 1:3, b = 4:6, c = 7:9)

  scoped_lifecycle_warnings()

  skip_int_error_names_must_be_null()

  # https://github.com/tidyverse/tibble/issues/562
  expect_warning(
    set_names(df, NULL),
    if (is_rstudio()) NA else error_names_must_be_non_null(),
    fixed = TRUE
  )
})

test_that("names<-() and set_names() reject non-minimal names", {
  df <- tibble(a = 1:3, b = 4:6, c = 7:9)

  scoped_lifecycle_warnings()

  expect_legacy_warning(
    `names<-`(df, NA),
    error_names_must_have_length(1, 3),
    fixed = TRUE
  )

  expect_legacy_warning(
    `names<-`(df, ""),
    error_names_must_have_length(1, 3),
    fixed = TRUE
  )

  expect_legacy_warning(
    `names<-`(df, 1),
    error_names_must_have_length(1, 3),
    fixed = TRUE
  )

  expect_legacy_warning(
    `names<-`(df, 1:2),
    error_names_must_have_length(2, 3),
    fixed = TRUE
  )

  expect_identical(
    set_names(df, letters[1:3]),
    tibble(a = 1:3, b = 4:6, c = 7:9)
  )
})

# repair_names (deprecated) ---------------------------------------------------
test_that("zero-length inputs given character names", {
  out <- repair_names(character())
  expect_equal(names(out), character())
})

test_that("unnamed input gives uniquely named output", {
  out <- repair_names(1:3)
  expect_equal(names(out), c("V1", "V2", "V3"))
})

# make_unique (deprecated) ----------------------------------------------------
test_that("duplicates are de-deduped", {
  expect_equal(make_unique(c("x", "x")), c("x", "x1"))
})

test_that("blanks get prefix + numeric id", {
  expect_equal(make_unique(c("", "")), c("V1", "V2"))
})

test_that("blanks skip existing names", {
  expect_equal(make_unique(c("", "V1")), c("V2", "V1"))
})

test_that("blanks skip names created when de-duping", {
  expect_equal(make_unique(c("", "V", "V")), c("V2", "V", "V1"))
})

#' Converting vectors to data frames, and vice versa
#'
#' @description
#' `enframe()` converts named atomic vectors or lists to one- or two-column
#' data frames.
#' For a list, the result will be a nested tibble with a column of type `list`.
#' For unnamed vectors, the natural sequence is used as name column.
#'
#' @param x A vector (for `enframe()`) or a data frame with one or two columns
#'   (for `deframe()`).
#' @param name,value Names of the columns that store the names and values.
#'   If `name` is `NULL`, a one-column tibble is returned; `value` cannot be `NULL`.
#'
#' @return For `enframe()`, a [tibble] with two columns (if `name` is not `NULL`, the default)
#'   or one column (otherwise).
#' @export
#'
#' @examples
#' enframe(1:3)
#' enframe(c(a = 5, b = 7))
#' enframe(list(one = 1, two = 2:3, three = 4:6))
enframe <- function(x, name = "name", value = "value") {
  if (is.null(value)) {
    abort_enframe_value_null()
  }

  if (is.null(x)) {
    x <- logical()
  }

  # FIXME: Enable again for data frames, add test
  if (!vec_is(x) || is.data.frame(x)) {
    abort_enframe_must_be_vector(x)
  }

  if (is.null(name)) {
    df <- list(vectbl_set_names(x))
  } else if (is.null(vec_names(x))) {
    df <- list(seq_len(vec_size(x)), x)
  } else {
    df <- list(vec_names2(x), vectbl_set_names(x))
  }

  names(df) <- c(name, value)
  new_tibble(df, nrow = vec_size(x))
}

#' @rdname enframe
#' @description
#' `deframe()` converts two-column data frames to a named vector or list,
#' using the first column as name and the second column as value.
#' If the input has only one column, an unnamed vector is returned.
#' @return For `deframe()`, a vector (named or unnamed).
#' @export
#' @examples
#' deframe(enframe(3:1))
#' deframe(tibble(a = 1:3))
#' deframe(tibble(a = as.list(1:3)))
deframe <- function(x) {
  if (length(x) == 1) {
    return(x[[1]])
  } else if (length(x) != 2) {
    warn("`x` must be a one- or two-column data frame in `deframe()`.")
  }

  value <- x[[2L]]
  name <- x[[1L]]
  vectbl_set_names(value, as.character(name))
}

abort_enframe_value_null <- function(call = caller_env()) {
  tibble_abort(call = call, "`value` can't be NULL.")
}

abort_enframe_must_be_vector <- function(x, call = caller_env()) {
  tibble_abort(call = call, paste0(
    "The `x` argument to `enframe()` must be a vector, not ", class(x)[[1]], "."
  ))
}

# nocov start - compat-lazyeval (last updated: rlang 0.3.0)

# This file serves as a reference for compatibility functions for lazyeval.
# Please find the most recent version in rlang's repository.


warn_underscored <- function() {
  return(NULL)
  warn(paste(
    "The underscored versions are deprecated in favour of",
    "tidy evaluation idioms. Please see the documentation",
    "for `quo()` in rlang"
  ))
}
warn_text_se <- function() {
  return(NULL)
  warn("Text parsing is deprecated, please supply an expression or formula")
}

compat_lazy <- function(lazy, env = caller_env(), warn = TRUE) {
  if (warn) warn_underscored()

  if (missing(lazy)) {
    return(quo())
  }
  if (is_quosure(lazy)) {
    return(lazy)
  }
  if (is_formula(lazy)) {
    return(as_quosure(lazy, env))
  }

  out <- switch(typeof(lazy),
    symbol = ,
    language = new_quosure(lazy, env),
    character = {
      if (warn) warn_text_se()
      parse_quo(lazy[[1]], env)
    },
    logical = ,
    integer = ,
    double = {
      if (length(lazy) > 1) {
        warn("Truncating vector to length 1")
        lazy <- lazy[[1]]
      }
      new_quosure(lazy, env)
    },
    list =
      if (inherits(lazy, "lazy")) {
        lazy = new_quosure(lazy$expr, lazy$env)
      }
  )

  if (is.null(out)) {
    abort_compat_lazy(lazy)
  } else {
    out
  }
}

abort_compat_lazy <- function(lazy, call = caller_env()) {
  tibble_abort(call = call, sprintf("Can't convert a %s to a quosure", typeof(lazy)))
}

compat_lazy_dots <- function(dots, env, ..., .named = FALSE) {
  if (missing(dots)) {
    dots <- list()
  }
  if (inherits(dots, c("lazy", "formula"))) {
    dots <- list(dots)
  } else {
    dots <- unclass(dots)
  }
  dots <- c(dots, list(...))

  warn <- TRUE
  for (i in seq_along(dots)) {
    dots[[i]] <- compat_lazy(dots[[i]], env, warn)
    warn <- FALSE
  }

  named <- have_name(dots)
  if (.named && any(!named)) {
    nms <- vapply(dots[!named], function(x) expr_text(get_expr(x)), character(1))
    names(dots)[!named] <- nms
  }

  names(dots) <- names2(dots)
  dots
}

compat_as_lazy <- function(quo) {
  structure(class = "lazy", list(
    expr = get_expr(quo),
    env = get_env(quo)
  ))
}
compat_as_lazy_dots <- function(...) {
  structure(class = "lazy_dots", lapply(quos(...), compat_as_lazy))
}


# nocov end

#' Deprecated functions
#'
#' @description
#' `r lifecycle::badge("deprecated")`
#'
#' Use [tibble()] instead of `data_frame()`.
#'
#' @export
#' @keywords internal
#' @name deprecated
data_frame <- function(...) {
  deprecate_warn("1.1.0", "data_frame()", "tibble()")

  # Unquote-splice to avoid argument matching
  tibble(!!!quos(...))
}

#' @description
#' Use [quasiquotation] instead of `tibble_()`, `data_frame_()`, and `lst_()`.
#'
#' @export
#' @keywords internal
#' @rdname deprecated
tibble_ <- function(xs) {
  deprecate_soft("2.0.0", "tibble_()", "tibble()",
    details = '`tibble()` supports dynamic dots, see `?"dyn-dots"`.'
  )

  xs <- compat_lazy_dots(xs, caller_env())
  tibble(!!!xs)
}

#' @export
#' @rdname deprecated
data_frame_ <- function(xs) {
  deprecate_stop("2.0.0", "data_frame_()", "tibble()",
    details = '`tibble()` supports dynamic dots, see `?"dyn-dots"`.'
  )
}

#' @export
#' @rdname deprecated
lst_ <- function(xs) {
  deprecate_stop("2.0.0", "lst_()", "lst()",
    details = '`lst()` supports dynamic dots, see `?"dyn-dots"`.'
  )
}

#' @description
#' Use [as_tibble()] instead of `as_data_frame()` or `as.tibble()`, but mind the
#' new signature and semantics.
#'
#' @export
#' @rdname deprecated
as_data_frame <- function(x, ...) {
  deprecate_warn("2.0.0", "as_data_frame()",
    details = "Please use `as_tibble()` (with slightly different semantics) to convert to a tibble, or `as.data.frame()` to convert to a data frame."
  )

  as_tibble(x, ...)
}

#' @export
#' @rdname deprecated
as.tibble <- function(x, ...) {
  deprecate_warn("2.0.0", "as.tibble()", "as_tibble()",
    details = "The signature and semantics have changed, see `?as_tibble`."
  )

  as_tibble(x, ...)
}

#' @description
#' Use [tribble()] instead of `frame_data()`.
#' @export
#' @rdname deprecated
frame_data <- function(...) {
  deprecate_stop("2.0.0", "frame_data()", "tribble()")
}

#' Name repair
#'
#' Please review [vctrs::vec_as_names()].
#'
#' @name name-repair
#' @keywords internal
NULL

#' Printing tibbles
#'
#' @description
#' One of the main features of the `tbl_df` class is the printing:
#'
#' * Tibbles only print as many rows and columns as fit on one screen,
#'   supplemented by a summary of the remaining rows and columns.
#' * Tibble reveals the type of each column, which keeps the user informed about
#'   whether a variable is, e.g., `<chr>` or `<fct>` (character versus factor).
#'   See `vignette("types")` for an overview of common
#'   type abbreviations.
#'
#' Printing can be tweaked for a one-off call by calling `print()` explicitly
#' and setting arguments like `n` and `width`. More persistent control is
#' available by setting the options described in [pillar::pillar_options].
#' See also `vignette("digits")` for a comparison to base options,
#' and `vignette("numbers")` that showcases [num()] and [char()]
#' for creating columns with custom formatting options.
#'
#' As of tibble 3.1.0, printing is handled entirely by the \pkg{pillar} package.
#' If you implement a package that extends tibble,
#' the printed output can be customized in various ways.
#' See `vignette("extending", package = "pillar")` for details,
#' and [pillar::pillar_options] for options that control the display in the console.
#'
# Copied from pillar::format.tbl() to avoid roxygen2 warning
#' @param x Object to format or print.
#' @param ... Passed on to [tbl_format_setup()].
#' @param n Number of rows to show. If `NULL`, the default, will print all rows
#'   if less than the `print_max` [option][pillar::pillar_options].
#'   Otherwise, will print as many rows as specified by the
#'   `print_min` [option][pillar::pillar_options].
#' @param width Width of text output to generate. This defaults to `NULL`, which
#'   means use the `width` [option][pillar::pillar_options].
#' @param max_extra_cols Number of extra columns to print abbreviated information for,
#'   if the width is too small for the entire tibble. If `NULL`,
#'   the `max_extra_cols` [option][pillar::pillar_options] is used.
#'   The previously defined `n_extra` argument is soft-deprecated.
#' @param max_footer_lines Maximum number of footer lines. If `NULL`,
#'   the `max_footer_lines` [option][pillar::pillar_options] is used.
#'
#' @examples
#' print(as_tibble(mtcars))
#' print(as_tibble(mtcars), n = 1)
#' print(as_tibble(mtcars), n = 3)
#'
#' print(as_tibble(trees), n = 100)
#'
#' print(mtcars, width = 10)
#'
#' mtcars2 <- as_tibble(cbind(mtcars, mtcars), .name_repair = "unique")
#' print(mtcars2, n = 25, max_extra_cols = 3)
#'
#' @examplesIf requireNamespace("nycflights13", quietly = TRUE)
#' print(nycflights13::flights, max_footer_lines = 1)
#' print(nycflights13::flights, width = Inf)
#'
#' @name formatting
#' @aliases print.tbl format.tbl
NULL

# Only for documentation, doesn't do anything
#' @rdname formatting
print.tbl_df <- function(x, width = NULL, ..., n = NULL, max_extra_cols = NULL,
                         max_footer_lines = NULL) {
  NextMethod()
}

# Only for documentation, doesn't do anything
#' @rdname formatting
format.tbl_df <- function(x, width = NULL, ..., n = NULL, max_extra_cols = NULL,
                          max_footer_lines = NULL) {
  NextMethod()
}

#' Legacy printing
#'
#' @description
#' `r lifecycle::badge("deprecated")`
#' As of tibble 3.1.0, printing is handled entirely by the \pkg{pillar} package.
#' Do not use this function.
#' If you implement a package that extend tibble,
#' the printed output can be customized in various ways.
#' See `vignette("extending", package = "pillar")` for details.
#'
#' @param x Object to format or print.
#' @param n Number of rows to show. If `NULL`, the default, will print all rows
#'   if less than option `tibble.print_max`. Otherwise, will print
#'   `tibble.print_min` rows.
#' @param width Width of text output to generate. This defaults to `NULL`, which
#'   means use `getOption("tibble.width")` or (if also `NULL`)
#'   `getOption("width")`; the latter displays only the columns that fit on one
#'   screen. You can also set `options(tibble.width = Inf)` to override this
#'   default and always print all columns, this may be slow for very wide tibbles.
#' @param n_extra Number of extra columns to print abbreviated information for,
#'   if the width is too small for the entire tibble. If `NULL`, the default,
#'   will print information about at most `tibble.max_extra_cols` extra columns.
#'
#' @return An object with a `print()` method that will print the input
#'   similarly to a tibble.
#'   The internal data format is an implementation detail, do not rely on it.
#' @export
#' @keywords internal
trunc_mat <- function(x, n = NULL, width = NULL, n_extra = NULL) {
  deprecate_soft("3.1.0", "tibble::trunc_mat()",
    details = "Printing has moved to the pillar package."
  )

  rows <- nrow(x)

  if (is.null(n) || n < 0) {
    if (is.na(rows) || rows > tibble_opt("print_max")) {
      n <- tibble_opt("print_min")
    } else {
      n <- rows
    }
  }
  n_extra <- n_extra %||% tibble_opt("max_extra_cols")

  if (is.na(rows)) {
    df <- as.data.frame(head(x, n + 1))
    if (nrow(df) <= n) {
      rows <- nrow(df)
    } else {
      df <- df[seq_len(n), , drop = FALSE]
    }
  } else {
    df <- as.data.frame(head(x, n))
  }

  shrunk <- shrink_mat(df, rows, n, star = has_rownames(x))
  trunc_info <- list(
    width = width, rows_total = rows, rows_min = nrow(df),
    n_extra = n_extra, summary = tbl_sum(x)
  )

  structure(
    c(shrunk, trunc_info),
    class = c(paste0("trunc_mat_", class(x)), "trunc_mat")
  )
}

shrink_mat <- function(df, rows, n, star) {
  df <- remove_rownames(df)
  if (is.na(rows)) {
    needs_dots <- (nrow(df) >= n)
  } else {
    needs_dots <- (rows > n)
  }

  if (needs_dots) {
    rows_missing <- rows - n
  } else {
    rows_missing <- 0L
  }

  mcf <- pillar::colonnade(
    df,
    has_row_id = if (star) "*" else TRUE
  )

  list(.do_not_rely_on_this_mcf = mcf, .do_not_rely_on_this_rows_missing = rows_missing)
}

#' @importFrom pillar style_subtle
#' @export
format.trunc_mat <- function(x, width = NULL, ...) {
  if (is.null(width)) {
    width <- x$width
  }

  width <- tibble_width(width)

  named_header <- format_header(x)
  if (all(names2(named_header) == "")) {
    header <- named_header
  } else {
    header <- paste0(
      justify(
        paste0(names2(named_header), ":"),
        right = FALSE, space = NBSP
      ),
      # We add a space after the NBSP inserted by justify()
      # so that wrapping occurs at the right location for very narrow outputs
      " ",
      named_header
    )
  }

  comment <- format_comment(header, width = width)
  squeezed <- pillar::squeeze(x$.do_not_rely_on_this_mcf, width = width)
  mcf <- format_body(squeezed)

  # Splitting lines is important, otherwise subtle style may be lost
  # if column names contain spaces.
  footer <- pre_dots(format_footer(x, squeezed))
  footer_comment <- split_lines(format_comment(footer, width = width))

  c(style_subtle(comment), mcf, style_subtle(footer_comment))
}

# Needs to be defined in package code: r-lib/pkgload#85
print_with_mocked_format_body <- function(x, ...) {
  scoped_lifecycle_silence()

  mockr::with_mock(
    format_body = function(x, ...) {
      paste0("<body created by pillar>")
    },
    {
      print(x, ...)
    }
  )
}

#' @export
print.trunc_mat <- function(x, ...) {
  cli::cat_line(format(x, ...))
  invisible(x)
}

format_header <- function(x) {
  x$summary
}

format_body <- function(x) {
  format(x)
}

format_footer <- function(x, squeezed_colonnade) {
  extra_rows <- format_footer_rows(x)
  extra_cols <- format_footer_cols(x, pillar::extra_cols(squeezed_colonnade, n = x$n_extra))

  extra <- c(extra_rows, extra_cols)
  if (length(extra) >= 1) {
    extra[[1]] <- paste0("with ", extra[[1]])
    extra[-1] <- map_chr(extra[-1], function(ex) paste0("and ", ex))
    collapse(extra)
  } else {
    character()
  }
}

format_footer_rows <- function(x) {
  if (length(x$.do_not_rely_on_this_mcf) != 0) {
    if (is.na(x$.do_not_rely_on_this_rows_missing)) {
      "more rows"
    } else if (x$.do_not_rely_on_this_rows_missing > 0) {
      paste0(big_mark(x$.do_not_rely_on_this_rows_missing), pluralise_n(" more row(s)", x$.do_not_rely_on_this_rows_missing))
    }
  } else if (is.na(x$rows_total) && x$rows_min > 0) {
    paste0("at least ", big_mark(x$rows_min), pluralise_n(" row(s) total", x$rows_min))
  }
}

format_footer_cols <- function(x, extra_cols) {
  if (length(extra_cols) == 0) return(NULL)

  vars <- format_extra_vars(extra_cols)
  paste0(
    big_mark(length(extra_cols)), " ",
    if (!identical(x$rows_total, 0L) && x$rows_min > 0) "more ",
    pluralise("variable(s)", extra_cols), vars
  )
}

format_extra_vars <- function(extra_cols) {
  # Also covers empty extra_cols vector!
  if (is.na(extra_cols[1])) return("")

  if (anyNA(extra_cols)) {
    extra_cols <- c(extra_cols[!is.na(extra_cols)], cli::symbol$ellipsis)
  }

  paste0(": ", collapse(extra_cols))
}

format_comment <- function(x, width) {
  if (length(x) == 0L) return(character())
  map_chr(x, wrap, prefix = "# ", width = min(width, getOption("width")))
}

pre_dots <- function(x) {
  if (length(x) > 0) {
    paste0(cli::symbol$ellipsis, " ", x)
  } else {
    character()
  }
}

justify <- function(x, right = TRUE, space = " ") {
  if (length(x) == 0L) return(character())
  width <- nchar_width(x)
  max_width <- max(width)
  spaces_template <- paste(rep(space, max_width), collapse = "")
  spaces <- map_chr(max_width - width, substr, x = spaces_template, start = 1L)
  if (right) {
    paste0(spaces, x)
  } else {
    paste0(x, spaces)
  }
}

split_lines <- function(x) {
  # Avoid .ptype argument to vec_c()
  if (is_empty(x)) return(character())

  unlist(strsplit(x, "\n", fixed = TRUE))
}


#' knit_print method for trunc mat
#' @keywords internal
#' @export
knit_print.trunc_mat <- function(x, options) {
  header <- format_header(x)
  if (length(header) > 0L) {
    header[names2(header) != ""] <- paste0(names2(header), ": ", header)
    summary <- header
  } else {
    summary <- character()
  }

  squeezed <- pillar::squeeze(x$.do_not_rely_on_this_mcf, x$width)

  kable <- format_knitr_body(squeezed)
  extra <- format_footer(x, squeezed)

  if (length(extra) > 0) {
    extra <- wrap("(", collapse(extra), ")", width = x$width)
  } else {
    extra <- "\n"
  }

  res <- paste(c("", "", summary, "", kable, "", extra), collapse = "\n")
  knitr::asis_output(fansi::strip_sgr(res), cacheable = TRUE)
}

format_knitr_body <- function(x) {
  knitr::knit_print(x)
}

big_mark <- function(x, ...) {
  # The thousand separator,
  # "," unless it's used for the decimal point, in which case "."
  mark <- if (identical(getOption("OutDec"), ",")) "." else ","
  ret <- formatC(x, big.mark = mark, format = "d", ...)
  ret[is.na(x)] <- "??"
  ret
}

mult_sign <- function() {
  "x"
}

spaces_around <- function(x) {
  paste0(" ", x, " ")
}

format_n <- function(x) collapse(quote_n(x))

quote_n <- function(x) UseMethod("quote_n")
#' @export
quote_n.default <- function(x) as.character(x)
#' @export
quote_n.character <- function(x) tick(x)

collapse <- function(x) paste(x, collapse = ", ")

# wrap --------------------------------------------------------------------

NBSP <- "\U00A0"

wrap <- function(..., indent = 0, prefix = "", width) {
  x <- paste0(..., collapse = "")
  wrapped <- strwrap2(x, width - nchar_width(prefix), indent)
  wrapped <- paste0(prefix, wrapped)
  wrapped <- gsub(NBSP, " ", wrapped)

  paste0(wrapped, collapse = "\n")
}

strwrap2 <- function(x, width, indent) {
  fansi::strwrap_ctl(x, width = max(width, 0), indent = indent, exdent = indent + 2)
}

scoped_lifecycle_silence()

test_that("interface of print() identical to trunc_mat()", {
  skip_enh_print_tbl_args()

  if (utils::packageVersion("pillar") >= "1.4.99") {
    print_arg_names <- names(formals(pillar:::print.tbl))
  } else {
    print_arg_names <- names(formals(print.tbl))
  }

  print_arg_names_without <- setdiff(print_arg_names, c("...", "width"))

  trunc_mat_arg_names <- names(formals(trunc_mat))
  trunc_mat_arg_names_without <- setdiff(trunc_mat_arg_names, "width")

  expect_equal(print_arg_names_without, trunc_mat_arg_names_without)
})

test_that("print() returns output invisibly", {
  expect_output(ret <- withVisible(print(as_tibble(iris))))
  expect_false(ret$visible)
  expect_identical(ret$value, as_tibble(iris))
})

test_that("trunc_mat output matches known output", {
  skip_on_non_utf8_locale()
  skip_if_not_installed("mockr")
  testthat::skip_if(getRversion() < "3.2")

  mtcars2 <- as_tibble(mtcars, rownames = NA)

  expect_output_file_rel(
    print_without_body(mtcars2, n = 8L, width = 30L),
    "trunc_mat/mtcars-8-30.txt"
  )

  expect_output_file_rel(
    print_without_body(as_tibble(iris), n = 5L, width = 30L),
    "trunc_mat/iris-5-30.txt"
  )

  expect_output_file_rel(
    print_without_body(as_tibble(iris), n = -1L, width = 30L),
    "trunc_mat/iris-neg-30.txt"
  )

  expect_output_file_rel(
    print_without_body(as_tibble(iris), n = Inf, width = 30L),
    "trunc_mat/iris-inf-30.txt"
  )

  expect_output_file_rel(
    print_without_body(as_tibble(iris), n = NULL, width = 70L),
    "trunc_mat/iris--70.txt"
  )

  expect_output_file_rel(
    print_without_body(as_unknown_rows(iris), n = 10, width = 70L),
    "trunc_mat/iris_unk-10-70.txt"
  )

  expect_output_file_rel(
    print_without_body(as_unknown_rows(iris[1:9, ]), n = 10, width = 70L),
    "trunc_mat/iris_9_unk-10-70.txt"
  )

  expect_output_file_rel(
    print_without_body(as_unknown_rows(iris[1:10, ]), n = 10, width = 70L),
    "trunc_mat/iris_10_unk-10-70.txt"
  )

  expect_output_file_rel(
    print_without_body(as_unknown_rows(iris[1:11, ]), n = 10, width = 70L),
    "trunc_mat/iris_11_unk-10-70.txt"
  )

  expect_output_file_rel(
    print_without_body(df_all, n = NULL, width = 30L),
    "trunc_mat/all--30.txt"
  )

  expect_output_file_rel(
    print_without_body(df_all, n = NULL, width = 300L),
    "trunc_mat/all--300.txt"
  )

  expect_output_file_rel(
    print_without_body(tibble(a = seq.int(10000)), n = 5L, width = 30L),
    "trunc_mat/long-5-30.txt"
  )

  expect_output_file_rel(
    print_without_body(tibble(a = character(), b = logical()), width = 30L),
    "trunc_mat/zero_rows--30.txt"
  )

  expect_output_file_rel(
    print_without_body(as_tibble(iris)[character()], n = 5L, width = 30L),
    "trunc_mat/zero_cols-5-30.txt"
  )

  expect_output_file_rel(
    print_without_body(as_unknown_rows(iris[integer(), ]), n = 5L, width = 30L),
    "trunc_mat/zero-rows_unk-5-30.txt"
  )

  expect_output_file_rel(
    print_without_body(as_unknown_rows(iris[, character()]), n = 5L, width = 30L),
    "trunc_mat/zero-cols_unk-5-30.txt"
  )

  expect_output_file_rel(
    print_without_body(
      as_unknown_rows(tibble(a = seq.int(10000))),
      n = 5L,
      width = 30L
    ),
    "trunc_mat/long_unk-5-30.txt"
  )

  expect_output_file_rel(
    print_with_mocked_format_body(trunc_mat(df_all, n = 1L, n_extra = 2L, width = 30L)),
    "trunc_mat/all-1-30-2.txt"
  )

  expect_output_file_rel(
    print_with_mocked_format_body(trunc_mat(df_all, n = 1L, n_extra = 0L, width = 30L)),
    "trunc_mat/all-1-30-0.txt"
  )

  expect_output_file_rel(
    print_with_mocked_format_body(trunc_mat(tibble("mean(x)" = 5, "var(x)" = 3), width = 28)),
    "trunc_mat/non-syntactic.txt"
  )
})

test_that("trunc_mat for POSIXlt columns (#86)", {
  skip_on_non_utf8_locale()

  df <- tibble(x = as.POSIXct("2016-01-01 12:34:56 GMT") + 1:12)
  df$y <- as.POSIXlt(df$x)

  expect_output_file_rel(
    print(df, n = 8L, width = 60L),
    "trunc_mat/POSIXlt-8-60.txt"
  )
})

test_that("trunc_mat for wide-character columns (#100)", {
  skip_on_non_utf8_locale() # capture_output_lines() forces native encoding

  x <- c("\u6210\u4ea4\u65e5\u671f", "\u5408\u540c\u5f55\u5165\u65e5\u671f")
  df <- setNames(tibble(1:3, 4:6), x)

  expect_output_file_rel(
    print(df, n = 8L, width = 60L),
    "trunc_mat/wide-8-60.txt"
  )
})

test_that("trunc_mat for wide-character columns in non-UTF-8 locale", {
  skip_on_non_utf8_locale() # capture_output_lines() forces native encoding

  with_non_utf8_locale({
    x <- c("\u6210\u4ea4\u65e5\u671f", "\u5408\u540c\u5f55\u5165\u65e5\u671f")
    df <- setNames(tibble(1:3, 4:6), x)

    expect_output_file_rel(
      print(df, n = 8L, width = 60L),
      "trunc_mat/wide-8-60.txt"
    )
  })
})

print_without_body <- function(x, ...) {
  if (utils::packageVersion("pillar") >= "1.4.99") {
    class(x) <- c("tbl_df_without_body", class(x))
    print(x, ...)
  } else {
    print_with_mocked_format_body(x, ...)
  }
}

if (utils::packageVersion("pillar") >= "1.4.99") {
  tbl_format_body.tbl_df_without_body <- function(x, ...) {
    "<body created by pillar>"
  }

  # Need explicit method, because can't be found in method env
  vctrs::s3_register(
    "pillar::tbl_format_body", "tbl_df_without_body",
    tbl_format_body.tbl_df_without_body
  )
}

#' Build a list
#'
#' @description
#' `lst()` constructs a list, similar to [base::list()], but with some of the
#' same features as [tibble()]. `lst()` builds components sequentially. When
#' defining a component, you can refer to components created earlier in the
#' call. `lst()` also generates missing names automatically.
#'
#' See [rlang::list2()] for a simpler and faster alternative without tibble's
#' evaluation and auto-name semantics.
#'
#' @inheritParams tibble
#' @return A named list.
#' @export
#' @examples
#' # the value of n can be used immediately in the definition of x
#' lst(n = 5, x = runif(n))
#'
#' # missing names are constructed from user's input
#' lst(1:3, z = letters[4:6], runif(3))
#'
#' a <- 1:3
#' b <- letters[4:6]
#' lst(a, b)
#'
#' # pre-formed quoted expressions can be used with lst() and then
#' # unquoted (with !!) or unquoted and spliced (with !!!)
#' n1 <- 2
#' n2 <- 3
#' n_stuff <- quote(n1 + n2)
#' x_stuff <- quote(seq_len(n))
#' lst(!!!list(n = n_stuff, x = x_stuff))
#' lst(n = !!n_stuff, x = !!x_stuff)
#' lst(n = 4, x = !!x_stuff)
#' lst(!!!list(n = 2, x = x_stuff))
lst <- function(...) {
  xs <- quos(..., .named = TRUE)
  lst_quos(xs)
}

lst_quos <- function(xs) {
  # TODO:
  # - soft-deprecate lst()

  # Evaluate each column in turn
  col_names <- names2(xs)
  lengths <- rep_along(xs, 0L)

  output <- rep_named(rep_along(xs, ""), list(NULL))

  for (i in seq_along(xs)) {
    unique_output <- output[!duplicated(names(output)[seq_len(i)], fromLast = TRUE)]
    res <- eval_tidy(xs[[i]], unique_output)
    if (!is.null(res)) {
      lengths[[i]] <- NROW(res)
      output[[i]] <- res
    }
    names(output)[[i]] <- col_names[[i]]
  }

  output
}

pluralise_msg <- function(message, objects) {
  paste0(
    pluralise(message, objects),
    format_n(objects)
  )
}

format_n <- function(x) collapse(quote_n(x))

quote_n <- function(x) UseMethod("quote_n")
#' @export
quote_n.default <- function(x) as.character(x)
#' @export
quote_n.character <- function(x) tick(x)

collapse <- function(x) paste(x, collapse = ", ")

pluralise_commas <- function(message, objects, ...) {
  paste0(
    pluralise_n(message, length(objects)),
    commas(objects),
    pluralise_n(paste0(...), length(objects))
  )
}

pluralise_count <- function(message, count, ...) {
  paste0(
    pluralise_n(message, count),
    count,
    pluralise_n(paste0(...), count)
  )
}

pluralise <- function(message, objects) {
  pluralise_n(message, length(objects))
}

pluralise_n <- function(message, n) {
  stopifnot(n >= 0)


  # Don't strip parens if they have a space in between
  # (but not if the space comes before the closing paren)

  if (n == 1) {
    # strip [
    message <- gsub("\\[([^\\] ]* *)\\]", "\\1", message, perl = TRUE)
    # remove ( and its content
    message <- gsub("\\([^\\) ]* *\\)", "", message, perl = TRUE)
  } else {
    # strip (
    message <- gsub("\\(([^\\) ]* *)\\)", "\\1", message, perl = TRUE)
    # remove [ and its content
    message <- gsub("\\[[^\\] ]* *\\]", "", message, perl = TRUE)
  }

  message
}

bullets <- function(header, ..., info = NULL) {
  # FIXME: Avoid ensure_full_stop()
  bullets <- vec_c(..., .name_spec = "{outer}")
  bullets <- set_default_name(bullets, "*")

  vec_c(
    ensure_full_stop(vec_c(header, bullets, .name_spec = "{outer}")),
    i = info,
    .name_spec = "{outer}"
  )
}

problems <- function(header, ..., .problem = " problem(s)") {
  problems <- vec_c(..., .name_spec = "{outer}")
  MAX_BULLETS <- 6L
  if (length(problems) >= MAX_BULLETS) {
    n_more <- length(problems) - MAX_BULLETS + 1L
    problems[[MAX_BULLETS]] <-
      pluralise_n(paste0(pre_dots("and "), n_more, " more", .problem), n_more)
    length(problems) <- MAX_BULLETS
  }

  problems <- set_default_name(problems, "x")
  bullets(header, problems)
}

pre_dots <- function(x) {
  if (length(x) > 0) {
    paste0(cli::symbol$ellipsis, " ", x)
  } else {
    character()
  }
}

commas <- function(problems) {
  MAX_BULLETS <- 6L

  n <- length(problems)
  if (n <= 1) {
    return(problems)
  } else if (n == 2) {
    return(paste(problems, collapse = " and "))
  }

  if (n >= MAX_BULLETS) {
    n_more <- length(problems) - MAX_BULLETS + 1L
    problems[[MAX_BULLETS]] <- paste0(n_more, " more")
    length(problems) <- MAX_BULLETS
    n <- MAX_BULLETS
  }

  problems[[n]] <- paste0("and ", problems[[n]])

  paste(problems, collapse = ", ")
}

ensure_full_stop <- function(x) {
  set_names(gsub("(?::|([^.?]))$", "\\1.", x), names(x))
}

set_default_name <- function(x, name) {
  if (is.null(names(x))) {
    names(x) <- rep_along(x, name)
  } else {
    names(x)[names(x) == ""] <- name
  }

  x
}

test_that("bullets", {
  skip_enh_bullets_format()

  expect_equal(
    unell_bullets("header", c("item 1", "item 2")),
    "header\n* item 1\n* item 2"
  )
  expect_equal(
    unell_bullets("header", LETTERS),
    "header\n* A\n* B\n* C\n* D\n* E\n* ... and 21 more problems"
  )
  expect_equal(
    unell_bullets("header", 1:6),
    "header\n* 1\n* 2\n* 3\n* 4\n* 5\n* ... and 1 more problem"
  )
})

test_that("commas", {
  skip_enh_bullets_format()

  expect_equal(
    commas("1"),
    "1"
  )
  expect_equal(
    commas(letters[2:4]),
    "b, c, d"
  )
  expect_equal(
    unell_commas(LETTERS),
    "A, B, C, D, E, ... (and 21 more)"
  )
})


test_that("pluralise works correctly", {
  expect_identical(pluralise("[an ]index(es)", c("x")), "an index")
  expect_identical(pluralise("[an ]index(es)", c("x", "y")), "indexes")
})

test_that("pluralise leaves alone parentheses / square brackets that have spaces inside", {
  expect_identical(
    pluralise("[an ]invalid index(es) (be careful) [for real]", c("x")),
    "an invalid index (be careful) [for real]"
  )
  expect_identical(
    pluralise("[an ]invalid index(es) (be careful) [for real]", c("x", "y")),
    "invalid indexes (be careful) [for real]"
  )
})

test_that("pluralise_msg works correctly", {
  expect_identical(pluralise_msg("[an ]index(es) ", c("x")), "an index `x`")
  expect_identical(pluralise_msg("[an ]index(es) ", c("x", "y")), "indexes `x`, `y`")
  expect_identical(pluralise_msg("[an ]index(es) ", c(-4, -5)), "indexes -4, -5")
})

## Legacy, for trunc_mat() and friends
op.tibble <- list(
  tibble.print_min = 10L,
  tibble.print_max = 20L,
  tibble.width = NULL,
  tibble.max_extra_cols = 100L
)

tibble_opt <- function(x, dplyr = TRUE) {
  x_tibble <- paste0("tibble.", x)
  res <- getOption(x_tibble)
  if (!is.null(res)) {
    return(res)
  }

  if (dplyr) {
    x_dplyr <- paste0("dplyr.", x)
    res <- getOption(x_dplyr)
    if (!is.null(res)) {
      return(res)
    }
  }

  op.tibble[[x_tibble]]
}

tibble_width <- function(width) {
  if (!is.null(width)) {
    return(width)
  }

  width <- tibble_opt("width")
  if (!is.null(width)) {
    return(width)
  }

  getOption("width")
}

tibble_glimpse_width <- function(width) {
  if (!is.null(width)) {
    return(width)
  }

  width <- tibble_opt("width")
  if (!is.null(width) && is.finite(width)) {
    return(width)
  }

  getOption("width")
}

name_or_pos <- function(names, positions) {
  empty <- (names == "")
  names[!empty] <- tick(names[!empty])
  names[empty] <- positions[empty]
  names
}

# FIXME: Also exists in pillar, do we need to export?
tick <- function(x) {
  ifelse(is.na(x), "NA", encodeString(x, quote = "`"))
}

tick_if_needed <- function(x) {
  needs_ticks <- !is_syntactic(x)
  x[needs_ticks] <- tick(x[needs_ticks])
  x
}

is_syntactic <- function(x) {
  ret <- rep_along(x, FALSE)
  valid <- which(!is.na(x))
  ret[valid] <- is_syntactic_impl(x[valid])
  ret
}

is_syntactic_impl <- function(x) {
  unchanged_after_repair <- (x == make.names(x))
  # https://r.789695.n4.nabble.com/Dots-are-not-fixed-by-make-names-td4752920.html
  dot_dot_dot_or_numbers <- grepl("^(?:(?:[.][.][.])|(?:[.][.][0-9]+))$", x)

  unchanged_after_repair & !dot_dot_dot_or_numbers
}

#' Subsetting tibbles
#'
#' @description
#' Accessing columns, rows, or cells via `$`, `[[`, or `[` is mostly similar to
#' [regular data frames][base::Extract]. However, the
#' behavior is different for tibbles and data frames in some cases:
#' * `[` always returns a tibble by default, even if
#'   only one column is accessed.
#' * Partial matching of column names with `$` and `[[` is not supported, and
#'   `NULL` is returned.
#'   For `$`, a warning is given.
#' * Only scalars (vectors of length one) or vectors with the
#'   same length as the number of rows can be used for assignment.
#' * Rows outside of the tibble's boundaries cannot be accessed.
#' * When updating with `[[<-` and `[<-`, type changes of entire columns are
#'   supported, but updating a part of a column requires that the new value is
#'   coercible to the existing type.
#'   See [vec_slice()] for the underlying implementation.
#'
#' Unstable return type and implicit partial matching can lead to surprises and
#' bugs that are hard to catch. If you rely on code that requires the original
#' data frame behavior, coerce to a data frame via [as.data.frame()].
#'
#' @details
#' For better compatibility with older code written for regular data frames,
#' `[` supports a `drop` argument which defaults to `FALSE`.
#' New code should use `[[` to turn a column into a vector.
#'
#' @param x A tibble.
#' @param value A value to store in a row, column, range or cell.
#'   Tibbles are stricter than data frames in what is accepted here.
#'
#' @name subsetting
#' @examples
#' df <- data.frame(a = 1:3, bc = 4:6)
#' tbl <- tibble(a = 1:3, bc = 4:6)
#'
#' # Subsetting single columns:
#' df[, "a"]
#' tbl[, "a"]
#' tbl[, "a", drop = TRUE]
#' as.data.frame(tbl)[, "a"]
#'
#' # Subsetting single rows with the drop argument:
#' df[1, , drop = TRUE]
#' tbl[1, , drop = TRUE]
#' as.list(tbl[1, ])
#'
#' @examplesIf (Sys.getenv("NOT_CRAN") != "true" || Sys.getenv("IN_PKGDOWN") == "true")
#' # Accessing non-existent columns:
#' df$b
#' tbl$b
#'
#' df[["b", exact = FALSE]]
#' tbl[["b", exact = FALSE]]
#'
#' df$bd <- c("n", "e", "w")
#' tbl$bd <- c("n", "e", "w")
#' df$b
#' tbl$b
#' @examples
#'
#' df$b <- 7:9
#' tbl$b <- 7:9
#' df$b
#' tbl$b
#'
#' # Identical behavior:
#' tbl[1, ]
#' tbl[1, c("bc", "a")]
#' tbl[, c("bc", "a")]
#' tbl[c("bc", "a")]
#' tbl["a"]
#' tbl$a
#' tbl[["a"]]
NULL

#' @rdname subsetting
#' @param name A [name] or a string.
#' @export
`$.tbl_df` <- function(x, name) {
  out <- .subset2(x, name)
  if (is.null(out)) {
    warn(paste0("Unknown or uninitialised column: ", tick(name), "."))
  }
  out
}


#' @rdname subsetting
#' @export
`$<-.tbl_df` <- function(x, name, value) {
  tbl_subassign(
    x,
    i = NULL, as_string(name), list(value),
    i_arg = NULL, j_arg = name, value_arg = substitute(value)
  )
}

#' @rdname subsetting
#' @param i,j Row and column indices. If `j` is omitted, `i` is used as column index.
#' @param ... Ignored.
#' @param exact Ignored, with a warning.
#' @export
`[[.tbl_df` <- function(x, i, j, ..., exact = TRUE) {
  if (!exact) {
    warn("`exact` ignored.")
  }

  n_dots <- dots_n(...)
  if (n_dots > 0) {
    warn("Extra arguments ignored.")
  }

  # Ignore exact as an argument for counting
  n_real_args <- nargs() - !missing(exact) - n_dots

  # Column subsetting if nargs() == 2L
  if (n_real_args <= 2L) {
    if (missing(i)) {
      cnd_signal(error_subset_columns_non_missing_only())
    }
    tbl_subset2(x, j = i, j_arg = substitute(i))
  } else if (missing(j) || missing(i)) {
    cnd_signal(error_subset_columns_non_missing_only())
  } else {
    i_arg <- substitute(i)
    i <- vectbl_as_row_location2(i, fast_nrow(x), i_arg)
    x <- tbl_subset2(x, j = j, j_arg = substitute(j))

    if (is.null(x)) {
      x
    } else {
      # Drop inner names with double subscript
      vectbl_set_names(vec_slice(x, i), NULL)
    }
  }
}

#' @rdname subsetting
#' @export
`[[<-.tbl_df` <- function(x, i, j, ..., value) {
  i_arg <- substitute(i)
  j_arg <- substitute(j)
  value_arg <- substitute(value)

  if (missing(i)) {
    cnd_signal(error_assign_columns_non_missing_only())
  }

  if (missing(j)) {
    if (nargs() < 4) {
      j <- i
      i <- NULL
      j_arg <- i_arg
      i_arg <- NULL
    } else {
      cnd_signal(error_assign_columns_non_missing_only())
    }
  }

  if (!is.null(i)) {
    i <- vectbl_as_row_location2(i, fast_nrow(x), i_arg, assign = TRUE)
  }

  if (is.object(j)) {
    j <- vectbl_as_col_subscript2(j, j_arg, assign = TRUE)
  }

  # Side effect: check scalar
  if (is.symbol(j)) {
    # FIXME: as_utf8_character() needs rlang > 0.4.11
    j <- chr_unserialise_unicode(as.character(j))
  } else {
    if (!is.vector(j) || length(j) != 1L || is.na(j) || (is.numeric(j) && j < 0) || is.logical(j)) {
      vectbl_as_col_location2(j, length(x), j_arg = j_arg, assign = TRUE)
    }
  }

  j <- vectbl_as_new_col_index(j, x, j_arg)

  # New columns are added to the end, provide index to avoid matching column
  # names again
  value <- list(value)

  # j is already pretty
  tbl_subassign(x, i, j, value, i_arg = i_arg, j_arg = NULL, value_arg = value_arg)
}


#' @rdname subsetting
#' @param drop Coerce to a vector if fetching one column via `tbl[, j]` .
#'   Default `FALSE`, ignored when accessing a column via `tbl[j]` .
#' @export
`[.tbl_df` <- function(x, i, j, drop = FALSE, ...) {
  i_arg <- substitute(i)
  j_arg <- substitute(j)

  if (missing(i)) {
    i <- NULL
    i_arg <- NULL
  } else if (is.null(i)) {
    i <- integer()
  }

  if (missing(j)) {
    j <- NULL
    j_arg <- NULL
  } else if (is.null(j)) {
    j <- integer()
  }

  # Ignore drop as an argument for counting
  n_real_args <- nargs() - !missing(drop)

  # Column or matrix subsetting if nargs() == 2L
  if (n_real_args <= 2L) {
    if (!missing(drop)) {
      warn("`drop` argument ignored for subsetting a tibble with `x[j]`, it has an effect only for `x[i, j]`.")
      drop <- FALSE
    }

    j <- i
    i <- NULL
    j_arg <- i_arg
    i_arg <- NULL

    # Special case, returns a vector:
    if (is.matrix(j)) {
      return(tbl_subset_matrix(x, j, j_arg))
    }
  }

  # From here on, i, j and drop contain correct values:
  if (is.null(j)) {
    xo <- x
  } else {
    j <- vectbl_as_col_location(j, length(x), names(x), j_arg = j_arg, assign = FALSE)

    if (anyNA(j)) {
      cnd_signal(error_na_column_index(which(is.na(j))))
    }

    xo <- .subset(x, j)

    if (anyDuplicated(j)) {
      xo <- set_repaired_names(xo, repair_hint = FALSE, .name_repair = "minimal")
    }

    xo <- set_tibble_class(xo, nrow = fast_nrow(x))
  }

  if (!is.null(i)) {
    xo <- tbl_subset_row(xo, i = i, i_arg)
  }

  if (drop && length(xo) == 1L) {
    tbl_subset2(xo, 1L, j_arg)
  } else {
    vectbl_restore(xo, x)
  }
}

#' @rdname subsetting
#' @export
`[<-.tbl_df` <- function(x, i, j, ..., value) {
  i_arg <- substitute(i)
  j_arg <- substitute(j)

  if (missing(i)) {
    i <- NULL
    i_arg <- NULL
  } else if (is.null(i)) {
    i <- integer()
  }

  if (missing(j)) {
    j <- NULL
    j_arg <- NULL
  } else if (is.null(j)) {
    j <- integer()
  }

  if (is.null(j) && nargs() < 4) {
    j <- i
    i <- NULL
    j_arg <- i_arg
    i_arg <- NULL

    # Special case:
    if (is.matrix(j)) {
      return(tbl_subassign_matrix(x, j, value, j_arg, substitute(value)))
    }
  }

  tbl_subassign(x, i, j, value, i_arg, j_arg, substitute(value))
}

vectbl_as_row_index <- function(i, x, i_arg, assign = FALSE) {
  stopifnot(!is.null(i))

  nr <- fast_nrow(x)

  if (is.character(i)) {
    is_na_orig <- is.na(i)

    if (has_rownames(x)) {
      i <- match(i, rownames(x))
    } else {
      i <- string_to_indices(i)
      i <- fix_oob(i, nr, warn = FALSE)
    }

    i <- fix_oob_invalid(i, is_na_orig)
    i
  } else if (is.numeric(i)) {
    i <- fix_oob(i, nr)
    vectbl_as_row_location(i, nr, i_arg, assign)
  } else {
    vectbl_as_row_location(i, nr, i_arg, assign)
  }
}

fix_oob <- function(i, n, warn = TRUE) {
  if (any(i > 0, na.rm = TRUE)) {
    fix_oob_positive(i, n, warn)
  } else if (any(i < 0, na.rm = TRUE)) {
    fix_oob_negative(i, n, warn)
  } else {
    # Will throw error in vec_as_location()
    i
  }
}

fix_oob_positive <- function(i, n, warn = TRUE) {
  oob <- which(i > n)
  if (warn && length(oob) > 0) {
    deprecate_soft("3.0.0", "tibble::`[.tbl_df`(i = 'must lie in [0, rows] if positive,')",
      details = "Use `NA_integer_` as row index to obtain a row full of `NA` values.",
      env = foreign_caller_env()
    )
  }

  i[oob] <- NA_integer_
  i
}

fix_oob_negative <- function(i, n, warn = TRUE) {
  oob <- (i < -n)
  if (warn && any(oob, na.rm = TRUE)) {
    deprecate_soft("3.0.0", "tibble::`[.tbl_df`(i = 'must lie in [-rows, 0] if negative,')",
      details = "Use `NA_integer_` as row index to obtain a row full of `NA` values.",
      env = foreign_caller_env()
    )
  }

  i <- i[!oob]
  if (is_empty(i)) i <- seq_len(n)
  i
}

fix_oob_invalid <- function(i, is_na_orig) {
  oob <- which(is.na(i) & !is_na_orig)

  if (has_length(oob)) {
    deprecate_soft("3.0.0", "tibble::`[.tbl_df`(i = 'must use valid row names')",
      details = "Use `NA_integer_` as row index to obtain a row full of `NA` values.",
      env = foreign_caller_env()
    )

    i[oob] <- NA_integer_
  }
  i
}

tbl_subset2 <- function(x, j, j_arg) {
  if (is.matrix(j)) {
    deprecate_soft("3.0.0", "tibble::`[[.tbl_df`(j = 'can\\'t be a matrix",
      details = "Recursive subsetting is deprecated for tibbles.",
      env = foreign_caller_env()
    )

    return(as.matrix(x)[[j]])
  }

  if (is.object(j)) {
    j <- vectbl_as_col_subscript2(j, j_arg)
  }

  if (is.numeric(j)) {
    if (length(j) == 1L) {
      if (is.na(j) || j < 1 || j > length(x) || (is.double(j) && j != trunc(j))) {
        # Side effect: throw error for invalid j
        vectbl_as_col_location2(j, length(x), j_arg = j_arg)
      }
    } else if (length(j) == 2L) {
      deprecate_soft("3.0.0", "tibble::`[[.tbl_df`(j = 'can\\'t be a vector of length 2')",
        details = "Recursive subsetting is deprecated for tibbles.",
        env = foreign_caller_env()
      )
    } else {
      # Side effect: throw error for invalid j
      vectbl_as_col_location2(j, length(x), j_arg = j_arg)
    }
  } else if (is.symbol(j)) {
    # FIXME: Only relevant for R < 3.4
    j <- as.character(j)
  } else if (is.logical(j) || length(j) != 1L || !is_bare_atomic(j) || is.na(j)) {
    # Side effect: throw error for invalid j
    vectbl_as_col_location2(j, length(x), names(x), j_arg = j_arg)
  }

  .subset2(x, j)
}

tbl_subset_row <- function(x, i, i_arg) {
  if (is.null(i)) return(x)
  i <- vectbl_as_row_index(i, x, i_arg)
  xo <- lapply(unclass(x), vec_slice, i = i)
  set_tibble_class(xo, nrow = length(i))
}

tbl_subassign <- function(x, i, j, value, i_arg, j_arg, value_arg) {
  if (is.null(i)) {
    value <- vectbl_wrap_rhs_col(value, value_arg)

    if (is.null(j)) {
      j <- seq_along(x)
      names(j) <- names2(j)
    } else if (!is.null(j_arg)) {
      j <- vectbl_as_new_col_index(j, x, j_arg, names2(value), value_arg)
    }

    value <- vectbl_recycle_rhs_rows(value, fast_nrow(x), i_arg = NULL, value_arg)
    value <- vectbl_recycle_rhs_cols(value, length(j))

    xo <- tbl_subassign_col(x, j, value)
  } else if (is.null(i_arg)) {
    # x[NULL, ...] <- value
    return(x)
  } else {
    i <- vectbl_as_new_row_index(i, x, i_arg)

    # Fill up rows first if necessary
    x <- tbl_expand_to_nrow(x, i)
    value <- vectbl_wrap_rhs_row(value, value_arg)

    if (is.null(j)) {
      xo <- tbl_subassign_row(x, i, value, i_arg, value_arg)
    } else {
      # Optimization: match only once
      # (Invariant: x[[j]] is equivalent to x[[vec_as_location(j)]],
      # allowed by corollary that only existing columns can be updated)
      if (!is.null(j_arg)) {
        j <- vectbl_as_new_col_index(j, x, j_arg, names2(value), value_arg)
      }
      new <- which(j > length(x))

      # Fill up columns if necessary
      if (has_length(new)) {
        init <- map(value[new], vec_slice, rep(NA_integer_, fast_nrow(x)))
        x <- tbl_subassign_col(x, j[new], init)
      }

      xj <- .subset(x, j)
      xj <- tbl_subassign_row(xj, i, value, i_arg, value_arg)
      xo <- tbl_subassign_col(x, j, unclass(xj))
    }
  }

  vectbl_restore(xo, x)
}

vectbl_as_new_row_index <- function(i, x, i_arg) {
  if (is.null(i)) {
    i
  } else if (is_bare_numeric(i)) {
    if (anyDuplicated(i)) {
      cnd_signal(error_duplicate_row_subscript_for_assignment(i))
    }

    nr <- fast_nrow(x)

    # Only update existing, caller knows how to deal with OOB
    numtbl_as_row_location_assign(i, nr, i_arg)
  } else if (is_logical(i)) {
    # Don't allow OOB logical
    vectbl_as_row_location(i, fast_nrow(x), i_arg, assign = TRUE)
  } else {
    i <- vectbl_as_row_index(i, x, i_arg, assign = TRUE)
    if (anyDuplicated(i, incomparables = NA)) {
      cnd_signal(error_duplicate_row_subscript_for_assignment(i))
    }
    i
  }
}

vectbl_as_new_col_index <- function(j, x, j_arg, names = "", value_arg = NULL) {
  # Creates a named index vector
  # Values: index
  # Name: column name (for all columns)

  if (is_bare_character(j)) {
    if (anyNA(j)) {
      cnd_signal(error_assign_columns_non_na_only())
    }

    names <- j

    j <- match(names, names(x))
    new <- which(is.na(j))
    if (has_length(new)) {
      j[new] <- seq.int(length(x) + 1L, length.out = length(new))
    }
  } else if (is_bare_numeric(j)) {
    if (anyNA(j)) {
      cnd_signal(error_assign_columns_non_na_only())
    }

    j <- numtbl_as_col_location_assign(j, length(x), j_arg = j_arg)

    old <- (j <= length(x))
    new <- which(!old)
    j_new <- j[new]

    # FIXME: Recycled names are not repaired
    # FIXME: Hard-coded name repair
    if (length(names) != 1L) {
      # Side effect: check compatibility
      vec_recycle(names, length(j), x_arg = as_label(value_arg))
    } else if (length(j) != 1L) {
      # length(names) == 1
      names <- vec_recycle(names, length(j), x_arg = as_label(value_arg))
    }

    if (length(new) > 0) {
      j[new] <- j_new
      names[new][names[new] == ""] <- paste0("...", j_new)
    }

    names[old] <- names(x)[j[old]]
  } else {
    j <- vectbl_as_col_location(j, length(x), names(x), j_arg = j_arg, assign = TRUE)

    if (anyNA(j)) {
      cnd_signal(error_na_column_index(which(is.na(j))))
    }

    if (length(names) != 1L) {
      # Side effect: check compatibility
      vec_recycle(names, length(j), x_arg = as_label(value_arg))
    } else if (length(j) != 1L) {
      # length(names) == 1
      names <- vec_recycle(names, length(j), x_arg = as_label(value_arg))
    }

    old <- (j <= length(x))
    names[old] <- names(x)[j[old]]
  }

  if (anyDuplicated(j)) {
    cnd_signal(error_duplicate_column_subscript_for_assignment(j))
  }

  set_names(j, names)
}

numtbl_as_row_location_assign <- function(i, n, i_arg) {
  subclass_row_index_errors(
    num_as_location(i, n, missing = "error", oob = "extend", zero = "error"),
    i_arg = i_arg, assign = TRUE
  )
}

vectbl_as_row_location <- function(i, n, i_arg, assign = FALSE) {
  if (is_bare_atomic(i) && is.matrix(i) && ncol(i) == 1) {
    what <- paste0(
      "tibble::", if (assign) "`[<-`" else "`[`",
      "(i = 'can\\'t be a matrix')"
    )

    lifecycle::deprecate_soft("3.0.0", what,
      details = "Convert to a vector.",
      env = foreign_caller_env()
    )
    i <- i[, 1]
  }

  subclass_row_index_errors(vec_as_location(i, n), i_arg = i_arg, assign = assign)
}

vectbl_as_row_location2 <- function(i, n, i_arg, assign = FALSE) {
  subclass_row_index_errors(vec_as_location2(i, n), i_arg = i_arg, assign = assign)
}

numtbl_as_col_location_assign <- function(j, n, j_arg) {
  subclass_col_index_errors(
    num_as_location(j, n, missing = "error", oob = "extend", zero = "error"),
    j_arg = j_arg, assign = TRUE
  )
}

vectbl_as_col_location <- function(j, n, names = NULL, j_arg, assign = FALSE) {
  subclass_col_index_errors(vec_as_location(j, n, names), j_arg = j_arg, assign = assign)
}

vectbl_as_col_location2 <- function(j, n, names = NULL, j_arg, assign = FALSE) {
  subclass_col_index_errors(vec_as_location2(j, n, names), j_arg = j_arg, assign = assign)
}

vectbl_as_col_subscript2 <- function(j, j_arg, assign = FALSE) {
  subclass_col_index_errors(vec_as_subscript2(j, logical = "error"), j_arg = j_arg, assign = assign)
}

is_tight_sequence_at_end <- function(i_new, n) {
  all(sort(i_new) == seq2(n + 1, n + length(i_new)))
}

tbl_subassign_col <- function(x, j, value) {
  is_data <- !vapply(value, is.null, NA)
  nrow <- fast_nrow(x)

  x <- unclass(x)

  # Grow, assign new names
  new <- which(j > length(x))
  if (has_length(new)) {
    length(x) <- max(j[new])
    names(x)[j[new]] <- names2(j)[new]
  }

  # Update
  for (jj in which(is_data)) {
    ji <- j[[jj]]
    x[[ji]] <- value[[jj]]
  }

  # Remove
  j_remove <- j[!is_data & !is.na(j)]
  if (has_length(j_remove)) x <- x[-j_remove]

  # Restore
  set_tibble_class(x, nrow)
}

tbl_expand_to_nrow <- function(x, i) {
  nrow <- fast_nrow(x)

  new_nrow <- max(i, nrow)

  if (is.na(new_nrow)) {
    cnd_signal(error_assign_rows_non_na_only())
  }

  if (new_nrow != nrow) {
    # FIXME: vec_expand()?
    i_expand <- c(seq_len(nrow), rep(NA_integer_, new_nrow - nrow))
    x <- vec_slice(x, i_expand)
  }

  x
}

tbl_subassign_row <- function(x, i, value, i_arg, value_arg) {
  nrow <- fast_nrow(x)
  x <- unclass(x)
  recycled_value <- vectbl_recycle_rhs_cols(value, length(x))

  withCallingHandlers(
    for (j in seq_along(x)) {
      x[[j]] <- vectbl_assign(x[[j]], i, recycled_value[[j]])
    },
    vctrs_error = function(cnd) {
      # Side effect: check if `value` can be recycled
      vectbl_recycle_rhs_rows(value, length(i), i_arg, value_arg)

      cnd_signal(error_assign_incompatible_type(x, recycled_value, j, value_arg, cnd_message(cnd)))
    }
  )

  set_tibble_class(x, nrow)
}

fast_nrow <- function(x) {
  .row_names_info(x, 2L)
}

vectbl_assign <- function(x, i, value) {
  if (is.logical(value)) {
    if (.Call("tibble_need_coerce", value)) {
      value <- vec_slice(x, NA_integer_)
    }
  } else {
    if (.Call("tibble_need_coerce", x)) {
      d <- dim(x)
      dn <- dimnames(x)
      x <- vec_slice(value, rep(NA_integer_, length(x)))
      dim(x) <- d
      dimnames(x) <- dn
    }
  }

  vec_assign(x, i, value)
}

vectbl_wrap_rhs_col <- function(value, value_arg) {
  if (is.null(value)) {
    return(list(value))
  }

  value <- result_vectbl_wrap_rhs(value)
  if (is.null(value)) {
    cnd_signal(error_need_rhs_vector_or_null(value_arg))
  }

  value
}

vectbl_wrap_rhs_row <- function(value, value_arg) {
  value <- result_vectbl_wrap_rhs(value)
  if (is.null(value)) {
    cnd_signal(error_need_rhs_vector(value_arg))
  }

  value
}

result_vectbl_wrap_rhs <- function(value) {
  if (!vec_is(value)) {
    NULL
  } else if (is.list(value)) {
    # Also covers the case of data frames
    unclass(value)
  } else if (is.array(value)) {
    if (any(dim(value)[-1:-2] != 1)) {
      return(NULL)
    }
    dim(value) <- head(dim(value), 2)
    as.list(as.data.frame(value, stringsAsFactors = FALSE))
  } else {
    list(value)
  }
}

vectbl_recycle_rhs <- function(value, nrow, ncol, i_arg, value_arg) {
  value <- vectbl_recycle_rhs_rows(value, nrow, i_arg, value_arg)
  vectbl_recycle_rhs_cols(value, ncol)
}

vectbl_recycle_rhs_rows <- function(value, nrow, i_arg, value_arg) {
  if (length(value) > 0L) {
    withCallingHandlers(
      for (j in seq_along(value)) {
        if (!is.null(value[[j]])) {
          value[[j]] <- vec_recycle(value[[j]], nrow)
        }
      },
      vctrs_error_recycle_incompatible_size = function(cnd) {
        cnd_signal(error_assign_incompatible_size(nrow, value, j, i_arg, value_arg))
      }
    )
  }

  value
}

vectbl_recycle_rhs_cols <- function(value, ncol) {
  if (length(value) != 1L || ncol != 1L) {
    # Errors have been caught beforehand in vectbl_as_new_col_index()
    value <- vec_recycle(value, ncol)
  }

  value
}

# Dedicated functions for faster subsetting
set_tibble_class <- function(x, nrow) {
  attr(x, "row.names") <- .set_row_names(nrow)
  class(x) <- tibble_class
  x
}

# External ----------------------------------------------------------------

vectbl_restore <- function(xo, x) {
  .Call(`tibble_restore_impl`, xo, x)
}

# Errors ------------------------------------------------------------------

error_need_rhs_vector <- function(value_arg) {
  tibble_error(paste0(tick(as_label(value_arg)), " must be a vector, a bare list, a data frame or a matrix."))
}

error_need_rhs_vector_or_null <- function(value_arg) {
  tibble_error(paste0(tick(as_label(value_arg)), " must be a vector, a bare list, a data frame, a matrix, or NULL."))
}

error_na_column_index <- function(j) {
  tibble_error(pluralise_commas("Can't use NA as column index with `[` at position(s) ", j, "."), j = j)
}

error_dim_column_index <- function(j) {
  # friendly_type_of() doesn't distinguish between matrices and arrays
  tibble_error(paste0("Must use a vector in `[`, not an object of class ", class(j)[[1]], "."))
}

error_assign_columns_non_na_only <- function() {
  tibble_error("Can't use NA as column index in a tibble for assignment.")
}

error_subset_columns_non_missing_only <- function() {
  tibble_error("Subscript can't be missing for tibbles in `[[`.")
}

error_assign_columns_non_missing_only <- function() {
  tibble_error("Subscript can't be missing for tibbles in `[[<-`.")
}

error_duplicate_column_subscript_for_assignment <- function(j) {
  j <- unique(j[duplicated(j)])
  tibble_error(pluralise_commas("Column index(es) ", j, " [is](are) used more than once for assignment."), j = j)
}

error_assign_rows_non_na_only <- function() {
  tibble_error("Can't use NA as row index in a tibble for assignment.")
}

error_duplicate_row_subscript_for_assignment <- function(i) {
  i <- unique(i[duplicated(i)])
  tibble_error(pluralise_commas("Row index(es) ", i, " [is](are) used more than once for assignment."), i = i)
}

error_assign_incompatible_size <- function(nrow, value, j, i_arg, value_arg) {
  if (is.null(i_arg)) {
    target <- "existing data"
    existing <- pluralise_count("Existing data has ", nrow, " row(s)")
  } else {
    target <- paste0("row subscript ", tick(as_label(i_arg)))
    existing <- pluralise_count("", nrow, " row(s) must be assigned")
  }

  new <- paste0(pluralise_count("has ", vec_size(value[[j]]), " row(s)"))
  if (length(value) != 1) {
    new <- paste0("Element ", j, " of assigned data ", new)
  } else {
    new <- paste0("Assigned data ", new)
  }

  tibble_error(
    bullets(
      paste0("Assigned data ", tick(as_label(value_arg)), " must be compatible with ", target, ":"),
      x = existing,
      x = new,
      i = if (nrow != 1) "Only vectors of size 1 are recycled",
      i = if (nrow == 1 && vec_size(value[[j]]) != 1) "Row updates require a list value. Do you need `list()` or `as.list()`?"
    ),
    expected = nrow,
    actual = vec_size(value[[j]]),
    j = j
  )
}

error_assign_incompatible_type <- function(x, value, j, value_arg, message) {
  name <- names(x)[[j]]

  tibble_error(
    bullets(
      paste0("Assigned data ", tick(as_label(value_arg)), " must be compatible with existing data:"),
      i = paste0("Error occurred for column ", tick(name)),
      x = message
    ),
    expected = x[[j]],
    actual = value[[j]],
    name = name,
    j = j
  )
}

# Subclassing errors ------------------------------------------------------

subclass_col_index_errors <- function(expr, j_arg, assign) {
  withCallingHandlers(
    expr,
    vctrs_error_subscript = function(cnd) {
      cnd$subscript_arg <- j_arg
      cnd$subscript_elt <- "column"
      if (isTRUE(assign) && !isTRUE(cnd$subscript_action %in% c("negate"))) {
        cnd$subscript_action <- "assign"
      }
      cnd_signal(cnd)
    }
  )
}

subclass_row_index_errors <- function(expr, i_arg, assign) {
  withCallingHandlers(
    expr,
    vctrs_error_subscript = function(cnd) {
      cnd$subscript_arg <- i_arg
      cnd$subscript_elt <- "row"
      if (isTRUE(assign) && !isTRUE(cnd$subscript_action %in% c("negate"))) {
        cnd$subscript_action <- "assign"
      }
      cnd_signal(cnd)
    }
  )
}

tbl_subset_matrix <- function(x, j, j_arg, call = caller_env()) {
  cells <- matrix_to_cells(j, x, j_arg, call)
  col_idx <- cells_to_col_idx(cells)

  if (is_empty(col_idx)) {
    return(unspecified())
  }

  values <- map2(x[col_idx], cells[col_idx], vec_slice)

  # Also checks conformity of vectors:
  unname(vec_c(!!!values, .name_spec = ~.x))
}

tbl_subassign_matrix <- function(x, j, value, j_arg, value_arg, call = caller_env()) {
  # FIXME: use size argument in vctrs >= 0.3.0

  if (!vec_is(value)) {
    abort_subset_matrix_scalar_type(j_arg, value_arg, call)
  }

  if (vec_size(value) != 1) {
    abort_subset_matrix_must_be_scalar(j_arg, value_arg, call)
  }

  cells <- matrix_to_cells(j, x, j_arg, call)
  col_idx <- cells_to_col_idx(cells)

  withCallingHandlers(
    for (j in col_idx) {
      x[[j]] <- vectbl_assign(x[[j]], cells[[j]], value)
    },
    vctrs_error_incompatible_type = function(cnd) {
      abort_assign_incompatible_type(x, rep(list(value), j), j, value_arg, cnd, call = call)
    }
  )

  x
}

matrix_to_cells <- function(j, x, j_arg, call = caller_env()) {
  if (!is_bare_logical(j)) {
    abort_subset_matrix_must_be_logical(j_arg, call)
  }
  if (!identical(dim(j), dim(x))) {
    abort_subset_matrix_must_have_same_dimensions(j_arg, call)
  }

  # Need unlist(list(...)) because apply() isn't type stable if the return
  # has the same length everywhere
  # FIXME: Faster with a C implementation?
  cells <- unlist(apply(j, 2, function(x) list(which(x))), recursive = FALSE)
  cells
}

cells_to_col_idx <- function(cells) {
  sizes <- map_int(cells, vec_size)
  col_idx <- which(sizes > 0)

  col_idx
}

# Errors ------------------------------------------------------------------

abort_subset_matrix_must_be_logical <- function(j_arg, call = caller_env()) {
  tibble_abort(call = call, paste0(
    "Subscript ", tick(as_label(j_arg)),
    " is a matrix, it must be of type logical."
  ))
}

abort_subset_matrix_must_have_same_dimensions <- function(j_arg, call = caller_env()) {
  tibble_abort(call = call, paste0(
    "Subscript ", tick(as_label(j_arg)),
    " is a matrix, it must have the same dimensions as the input."
  ))
}

abort_subset_matrix_scalar_type <- function(j_arg, value_arg, call = caller_env()) {
  tibble_abort(call = call, paste0(
    "Subscript ", tick(as_label(j_arg)),
    " is a matrix, the data ", tick(as_label(value_arg)),
    " must be a vector of size 1."
  ))
}

abort_subset_matrix_must_be_scalar <- function(j_arg, value_arg, call = caller_env()) {
  tibble_abort(call = call, paste0(
    "Subscript ", tick(as_label(j_arg)),
    " is a matrix, the data ", tick(as_label(value_arg)),
    " must have size 1."
  ))
}

#' Add rows to a data frame
#'
#' @description
#' This is a convenient way to add one or more rows of data to an existing data
#' frame. See [tribble()] for an easy way to create an complete
#' data frame row-by-row. Use [tibble_row()] to ensure that the new data
#' has only one row.
#'
#' `add_case()` is an alias of `add_row()`.
#'
#' @param .data Data frame to append to.
#' @param ... <[`dynamic-dots`][rlang::dyn-dots]>
#'   Name-value pairs, passed on to [tibble()]. Values can be defined
#'   only for columns that already exist in `.data` and unset columns will get an
#'   `NA` value.
#' @param .before,.after One-based row index where to add the new rows,
#'   default: after last row.
#' @family addition
#' @examples
#' # add_row ---------------------------------
#' df <- tibble(x = 1:3, y = 3:1)
#'
#' df %>% add_row(x = 4, y = 0)
#'
#' # You can specify where to add the new rows
#' df %>% add_row(x = 4, y = 0, .before = 2)
#'
#' # You can supply vectors, to add multiple rows (this isn't
#' # recommended because it's a bit hard to read)
#' df %>% add_row(x = 4:5, y = 0:-1)
#'
#' # Use tibble_row() to add one row only
#' df %>% add_row(tibble_row(x = 4, y = 0))
#' try(df %>% add_row(tibble_row(x = 4:5, y = 0:-1)))
#'
#' # Absent variables get missing values
#' df %>% add_row(x = 4)
#'
#' # You can't create new variables
#' try(df %>% add_row(z = 10))
#' @export
add_row <- function(.data, ..., .before = NULL, .after = NULL) {
  if (inherits(.data, "grouped_df")) {
    cnd_signal(error_add_rows_to_grouped_df())
  }

  if (!is.data.frame(.data)) {
    deprecate_warn("2.1.1", "add_row(.data = 'must be a data frame')")
  }

  if (dots_n(...) == 0L) {
    # A single row of missing values is added if no input is supplied
    df <- new_tibble(list(), nrow = 1L)
  } else {
    df <- tibble(...)
  }

  extra_vars <- setdiff(names(df), names(.data))
  if (has_length(extra_vars)) {
    cnd_signal(error_incompatible_new_rows(extra_vars))
  }

  pos <- pos_from_before_after(.before, .after, nrow(.data))
  out <- rbind_at(.data, df, pos)

  vectbl_restore(out, .data)
}

#' @export
#' @rdname add_row
#' @usage NULL
add_case <- add_row

na_value <- function(boilerplate) {
  if (is.list(boilerplate)) {
    list(NULL)
  } else {
    NA
  }
}

rbind_at <- function(old, new, pos) {
  out <- vec_rbind(old, new)

  # Append at end: Nothing more to do.
  if (pos >= nrow(old)) {
    return(out)
  }

  # Splice: Construct index vector
  pos <- max(pos, 0L)
  idx <- c(
    seq2(1L, pos),
    seq2(nrow(old) + 1L, nrow(old) + nrow(new)),
    seq2(pos + 1L, nrow(old))
  )
  vec_slice(out, idx)
}

#' Add columns to a data frame
#'
#' This is a convenient way to add one or more columns to an existing data
#' frame.
#'
#' @param .data Data frame to append to.
#' @param ... <[`dynamic-dots`][rlang::dyn-dots]>
#'   Name-value pairs, passed on to [tibble()]. All values must have
#'   the same size of `.data` or size 1.
#' @param .before,.after One-based column index or column name where to add the
#'   new columns, default: after last column.
#' @inheritParams tibble
#' @family addition
#' @examples
#' # add_column ---------------------------------
#' df <- tibble(x = 1:3, y = 3:1)
#'
#' df %>% add_column(z = -1:1, w = 0)
#' df %>% add_column(z = -1:1, .before = "y")
#'
#' # You can't overwrite existing columns
#' try(df %>% add_column(x = 4:6))
#'
#' # You can't create new observations
#' try(df %>% add_column(z = 1:5))
#'
#' @export
add_column <- function(.data, ..., .before = NULL, .after = NULL,
                       .name_repair = c("check_unique", "unique", "universal", "minimal")) {
  if (!is.data.frame(.data)) {
    deprecate_warn("2.1.1", "add_column(.data = 'must be a data frame')")
  }

  if (has_length(.data) && (!is_named(.data) || anyDuplicated(names2(.data))) && missing(.name_repair)) {
    deprecate_warn("3.0.0", "add_column(.data = 'must have unique names')",
      details = 'Use `.name_repair = "minimal"`.'
    )
    .name_repair <- "minimal"
  }

  df <- tibble(..., .name_repair = .name_repair)

  if (ncol(df) == 0L) {
    return(.data)
  }

  if (nrow(df) != nrow(.data)) {
    if (nrow(df) == 1) {
      df <- df[rep(1L, nrow(.data)), ]
    } else {
      cnd_signal(error_incompatible_new_cols(nrow(.data), df))
    }
  }

  pos <- pos_from_before_after_names(.before, .after, colnames(.data))

  end_pos <- ncol(.data) + seq_len(ncol(df))

  indexes_before <- rlang::seq2(1L, pos)
  indexes_after <- rlang::seq2(pos + 1L, ncol(.data))
  indexes <- c(indexes_before, end_pos, indexes_after)

  new_data <- .data
  new_data[end_pos] <- df

  out <- new_data[indexes]

  out <- set_repaired_names(out, repair_hint = TRUE, .name_repair)
  vectbl_restore(out, .data)
}


# helpers -----------------------------------------------------------------

pos_from_before_after_names <- function(before, after, names) {
  before <- check_names_before_after(before, names)
  after <- check_names_before_after(after, names)

  pos_from_before_after(before, after, length(names))
}

pos_from_before_after <- function(before, after, len) {
  if (is.null(before)) {
    if (is.null(after)) {
      len
    } else {
      limit_pos_range(after, len)
    }
  } else {
    if (is.null(after)) {
      limit_pos_range(before - 1L, len)
    } else {
      cnd_signal(error_both_before_after())
    }
  }
}

limit_pos_range <- function(pos, len) {
  max(0L, min(len, pos))
}

# check_names_before_after ------------------------------------------------

check_names_before_after <- function(j, x) {
  if (!is_bare_character(j)) {
    return(j)
  }

  check_needs_no_dim(j)
  check_names_before_after_character(j, x)
}

check_needs_no_dim <- function(j) {
  if (needs_dim(j)) {
    cnd_signal(error_dim_column_index(j))
  }
}

check_names_before_after_character <- function(j, names) {
  pos <- safe_match(j, names)
  if (anyNA(pos)) {
    unknown_names <- j[is.na(pos)]
    cnd_signal(error_unknown_column_names(unknown_names))
  }
  pos
}

# Errors ------------------------------------------------------------------

error_add_rows_to_grouped_df <- function() {
  tibble_error("Can't add rows to grouped data frames.")
}

error_incompatible_new_rows <- function(names) {
  tibble_error(
    problems(
      "New rows can't add columns:",
      cnd_message(error_unknown_column_names(names))
    ),
    names = names
  )
}

error_both_before_after <- function() {
  tibble_error("Can't specify both `.before` and `.after`.")
}

error_unknown_column_names <- function(j, parent = NULL) {
  tibble_error(pluralise_commas("Can't find column(s) ", tick(j), " in `.data`."), j = j, parent = parent)
}

error_incompatible_new_cols <- function(n, df) {
  tibble_error(
    bullets(
      "New columns must be compatible with `.data`:",
      x = paste0(
        pluralise_n("New column(s) ha[s](ve)", ncol(df)), " ",
        nrow(df), " rows"
      ),
      i = pluralise_count("`.data` has ", n, " row(s)")
    ),
    expected = n,
    actual = nrow(df)
  )
}

# add_row ---------------------------------------------------------------

test_that("can add new row", {
  df_all_new <- add_row(df_all, a = 4, b = 3L)
  expect_identical(colnames(df_all_new), colnames(df_all))
  expect_identical(nrow(df_all_new), nrow(df_all) + 1L)
  expect_identical(df_all_new$a, c(df_all$a, 4))
  expect_identical(df_all_new$b, c(df_all$b, 3L))
  expect_identical(df_all_new$c, c(df_all$c, NA))
})

test_that("add_row() keeps class of object", {
  iris_new <- add_row(iris, Species = "unknown")
  expect_equal(class(iris), class(iris_new))

  iris_new <- add_row(as_tibble(iris), Species = "unknown")
  expect_equal(class(as_tibble(iris)), class(iris_new))
})

test_that("add_row() keeps class of object when adding in the middle", {
  iris_new <- add_row(iris, Species = "unknown", .after = 10)
  expect_equal(class(iris), class(iris_new))

  iris_new <- add_row(as_tibble(iris), Species = "unknown")
  expect_equal(class(as_tibble(iris)), class(iris_new))
})

test_that("add_row() keeps class of object when adding in the beginning", {
  iris_new <- add_row(iris, Species = "unknown", .after = 0)
  expect_equal(class(iris), class(iris_new))

  iris_new <- add_row(as_tibble(iris), Species = "unknown")
  expect_equal(class(as_tibble(iris)), class(iris_new))
})

test_that("adds empty row if no arguments", {
  new_iris_row <- add_row(iris)[nrow(iris) + 1, , drop = TRUE]
  expect_true(all(is.na(new_iris_row)))
})

test_that("error if adding row with unknown variables", {
  expect_legacy_error(
    add_row(tibble(a = 3), xxyzy = "err"),
    error_inconsistent_new_rows("xxyzy"),
    fixed = TRUE
  )

  expect_legacy_error(
    add_row(tibble(a = 3), b = "err", c = "oops"),
    error_inconsistent_new_rows(c("b", "c")),
    fixed = TRUE
  )
})

test_that("deprecated adding rows to non-data-frames", {
  scoped_lifecycle_errors()

  expect_error(add_row(as.matrix(mtcars), mpg = 4))
})

test_that("can add multiple rows", {
  df <- tibble(a = 3L)
  df_new <- add_row(df, a = 4:5)
  expect_identical(df_new, tibble(a = 3:5))
})

test_that("can recycle when adding rows", {
  iris_new <- add_row(iris, Sepal.Length = -1:-2, Species = "unknown")
  expect_identical(nrow(iris_new), nrow(iris) + 2L)
  expect_identical(iris_new$Sepal.Length, c(iris$Sepal.Length, -1:-2))
  expect_identical(
    as.character(iris_new$Species),
    c(as.character(iris$Species), "unknown", "unknown")
  )
})

test_that("can add as first row via .before = 1", {
  df <- tibble(a = 3L)
  df_new <- add_row(df, a = 2L, .before = 1)
  expect_identical(df_new, tibble(a = 2:3))
})

test_that("can add as first row via .after = 0", {
  df <- tibble(a = 3L)
  df_new <- add_row(df, a = 2L, .after = 0)
  expect_identical(df_new, tibble(a = 2:3))
})

test_that("can add row inbetween", {
  df <- tibble(a = 1:3)
  df_new <- add_row(df, a = 4:5, .after = 2)
  expect_identical(df_new, tibble(a = c(1:2, 4:5, 3L)))
})

test_that("can safely add to factor columns everywhere (#296)", {
  df <- tibble(a = factor(letters[1:3]))
  expect_identical(add_row(df), tibble(a = factor(c(letters[1:3], NA))))
  expect_identical(add_row(df, .before = 1), tibble(a = factor(c(NA, letters[1:3]))))
  expect_identical(add_row(df, .before = 2), tibble(a = factor(c("a", NA, letters[2:3]))))
  skip_brk_add_row_vctrs()
  expect_identical(add_row(df, a = "d"), tibble(a = factor(c(letters[1:4]))))
  expect_identical(add_row(df, a = "d", .before = 1), tibble(a = factor(c("d", letters[1:3]))))
  expect_identical(add_row(df, a = "d", .before = 2), tibble(a = factor(c("a", "d", letters[2:3]))))
})

test_that("error if both .before and .after are given", {
  df <- tibble(a = 1:3)
  expect_legacy_error(
    add_row(df, a = 4:5, .after = 2, .before = 3),
    error_both_before_after(),
    fixed = TRUE
  )
})

test_that("missing row names stay missing when adding row", {
  expect_false(has_rownames(iris))
  expect_false(has_rownames(add_row(iris, Species = "unknown", .after = 0)))
  expect_false(has_rownames(add_row(iris, Species = "unknown", .after = nrow(iris))))
  expect_false(has_rownames(add_row(iris, Species = "unknown", .before = 10)))
})

test_that("adding to a list column adds a NULL value (#148)", {
  expect_null(add_row(tibble(a = as.list(1:3)))$a[[4]])
  expect_null(add_row(tibble(a = as.list(1:3)), .before = 1)$a[[1]])
  expect_null(add_row(tibble(a = as.list(1:3)), .after = 1)$a[[2]])
  expect_null(add_row(tibble(a = as.list(1:3), b = 1:3), b = 4:6)$a[[5]])
})

test_that("add_row() keeps the class of empty columns", {
  new_tibble <- add_row(df_empty, to_be_added = 5)
  expect_equal(sapply(df_empty, class), sapply(new_tibble, class))
})

test_that("add_row() fails nicely for grouped data frames (#179)", {
  skip_if_not_installed("dplyr")
  expect_legacy_error(
    add_row(dplyr::group_by(iris, Species), Petal.Width = 3),
    error_add_rows_to_grouped_df(),
    fixed = TRUE
  )
})

# add_column ------------------------------------------------------------

test_that("can add new column", {
  df_all_new <- add_column(df_all, j = 1:3, k = 3:1)
  expect_identical(nrow(df_all_new), nrow(df_all))
  expect_identical(df_all_new[seq_along(df_all)], df_all)
  expect_identical(df_all_new$j, 1:3)
  expect_identical(df_all_new$k, 3:1)
})

test_that("add_column() keeps class of object", {
  iris_new <- add_column(iris, x = 1:150)
  expect_equal(class(iris), class(iris_new))

  iris_new <- add_column(as_tibble(iris), x = 1:150)
  expect_equal(class(as_tibble(iris)), class(iris_new))
})

test_that("add_column() keeps class of object when adding in the middle", {
  iris_new <- add_column(iris, x = 1:150, .after = 3)
  expect_equal(class(iris), class(iris_new))

  iris_new <- add_column(as_tibble(iris), x = 1:150)
  expect_equal(class(as_tibble(iris)), class(iris_new))
})

test_that("add_column() keeps class of object when adding in the beginning", {
  iris_new <- add_column(iris, x = 1:150, .after = 0)
  expect_equal(class(iris), class(iris_new))

  iris_new <- add_column(as_tibble(iris), x = 1:150)
  expect_equal(class(as_tibble(iris)), class(iris_new))
})

test_that("add_column() keeps unchanged if no arguments", {
  expect_identical(iris, add_column(iris))
})

test_that("add_column() can add to empty tibble or data frame", {
  expect_identical(add_column(tibble(.rows = 3), a = 1:3), tibble(a = 1:3))
  expect_identical(add_column(as.data.frame(tibble(.rows = 3)), a = 1:3), data.frame(a = 1:3))
})

test_that("error if adding existing columns", {
  expect_legacy_error(
    add_column(tibble(a = 3), a = 5),
    error_duplicate_new_cols("a"),
    fixed = TRUE
  )
})

test_that("error if adding wrong number of rows with add_column()", {
  expect_legacy_error(
    add_column(tibble(a = 3), b = 4:5),
    error_inconsistent_new_cols(1, data.frame(b = 4:5)),
    fixed = TRUE
  )
})

test_that("can add multiple columns", {
  df <- tibble(a = 1:3)
  df_new <- add_column(df, b = 4:6, c = 3:1)
  expect_identical(df_new, tibble(a = 1:3, b = 4:6, c = 3:1))
})

test_that("can recycle when adding columns", {
  df <- tibble(a = 1:3)
  df_new <- add_column(df, b = 4, c = 3:1)
  expect_identical(df_new, tibble(a = 1:3, b = rep(4, 3), c = 3:1))
})

test_that("can recycle when adding a column of length 1", {
  df <- tibble(a = 1:3)
  df_new <- add_column(df, b = 4)
  expect_identical(df_new, tibble(a = 1:3, b = rep(4, 3)))
})

test_that("can recyle when adding multiple columns of length 1", {
  df <- tibble(a = 1:3)
  df_new <- add_column(df, b = 4, c = 5)
  expect_identical(df_new, tibble(a = 1:3, b = rep(4, 3), c = rep(5, 3)))
})

test_that("can recyle for zero-row data frame (#167)", {
  df <- tibble(a = 1:3)[0, ]
  df_new <- add_column(df, b = 4, c = character())
  expect_identical(df_new, tibble(a = integer(), b = numeric(), c = character()))
})

test_that("can add as first column via .before = 1", {
  df <- tibble(a = 3L)
  df_new <- add_column(df, b = 2L, .before = 1)
  expect_identical(df_new, tibble(b = 2L, a = 3L))
})

test_that("can add as first column via .after = 0", {
  df <- tibble(a = 3L)
  df_new <- add_column(df, b = 2L, .after = 0)
  expect_identical(df_new, tibble(b = 2L, a = 3L))
})

test_that("can add column inbetween", {
  df <- tibble(a = 1:3, c = 4:6)
  df_new <- add_column(df, b = -1:1, .after = 1)
  expect_identical(df_new, tibble(a = 1:3, b = -1:1, c = 4:6))
})

test_that("can add column relative to named column", {
  df <- tibble(a = 1:3, c = 4:6)
  df_new <- add_column(df, b = -1:1, .before = "c")
  expect_identical(df_new, tibble(a = 1:3, b = -1:1, c = 4:6))
})

test_that("error if both .before and .after are given", {
  df <- tibble(a = 1:3)
  expect_legacy_error(
    add_column(df, b = 4:6, .after = 2, .before = 3),
    error_both_before_after(),
    fixed = TRUE
  )
})

test_that("error if column named by .before or .after not found", {
  skip_int_error_unknown_names()

  df <- tibble(a = 1:3)
  expect_error(
    add_column(df, b = 4:6, .after = "x"),
    error_unknown_names("x"),
    fixed = TRUE
  )
  expect_error(
    add_column(df, b = 4:6, .before = "x"),
    error_unknown_names("x"),
    fixed = TRUE
  )
})

test_that("deprecated adding rows to non-data-frames", {
  scoped_lifecycle_errors()

  expect_error(add_column(as.matrix(mtcars), x = 1))
})

test_that("missing row names stay missing when adding column", {
  expect_false(has_rownames(iris))
  expect_false(has_rownames(add_column(iris, x = 1:150, .after = 0)))
  expect_false(has_rownames(add_column(iris, x = 1:150, .after = ncol(iris))))
  expect_false(has_rownames(add_column(iris, x = 1:150, .before = 2)))
})

#' Tibble constructor and validator
#'
#' @description
#' Creates or validates a subclass of a tibble.
#' These function is mostly useful for package authors that implement subclasses
#' of a tibble, like \pkg{sf} or \pkg{tsibble}.
#'
#' `new_tibble()` creates a new object as a subclass of `tbl_df`, `tbl` and `data.frame`.
#' This function is optimized for performance, checks are reduced to a minimum.
#' See [vctrs::new_data_frame()] for details.
#'
#' @param x A tibble-like object.
#' @param ... Name-value pairs of additional attributes.
#' @param nrow The number of rows, inferred from `x` if omitted.
#' @param class Subclasses to assign to the new object, default: none.
#' @param subclass Deprecated, retained for compatibility. Please use the `class` argument.
#'
#' @seealso
#' [tibble()] and [as_tibble()] for ways to construct a tibble
#' with recycling of scalars and automatic name repair,
#' and [vctrs::df_list()] and [vctrs::new_data_frame()]
#' for lower-level implementations.
#'
#' @export
#' @examples
#' # The nrow argument is always required:
#' new_tibble(list(a = 1:3, b = 4:6), nrow = 3)
#'
#' # Existing row.names attributes are ignored:
#' try(validate_tibble(new_tibble(trees, nrow = 3)))
#'
#' # The length of all columns must be compatible with the nrow argument:
#' try(validate_tibble(new_tibble(list(a = 1:3, b = 4:6), nrow = 2)))
new_tibble <- function(x, ..., nrow = NULL, class = NULL, subclass = NULL) {
  # For compatibility with tibble < 2.0.0
  if (is.null(class) && !is.null(subclass)) {
    deprecate_soft("2.0.0", "tibble::new_tibble(subclass = )", "new_tibble(class = )")
    class <- subclass
  }

  #' @section Construction:
  #'
  #' For `new_tibble()`, `x` must be a list.
  x <- unclass(x)

  if (!is.list(x)) {
    cnd_signal(error_new_tibble_must_be_list())
  }

  #' The `nrow` argument may be omitted as of tibble 3.1.4.
  #' If present, every element of the list `x` should have [vctrs::vec_size()]
  #' equal to this value.
  #' (But this is not checked by the constructor).
  #' This takes the place of the "row.names" attribute in a data frame.
  if (!is.null(nrow)) {
    if (!is.numeric(nrow) || length(nrow) != 1 || nrow < 0 || !is_integerish(nrow, 1) || nrow >= 2147483648) {
      cnd_signal(error_new_tibble_nrow_must_be_nonnegative())
    }
    nrow <- as.integer(nrow)
  }

  args <- attributes(x)

  if (is.null(args)) {
    args <- list()
  }

  new_attrs <- pairlist2(...)
  nms <- names(new_attrs)

  for (i in seq_along(nms)) {
    nm <- nms[[i]]

    if (nm == "") {
      next
    }

    args[[nm]] <- new_attrs[[i]]
  }

  #' `x` must have names (or be empty),
  #' but the names are not checked for correctness.
  if (length(x) == 0) {
    # Leaving this because creating a named list of length zero seems difficult
    args[["names"]] <- character()
  } else if (is.null(args[["names"]])) {
    cnd_signal(error_names_must_be_non_null())
  }

  if (is.null(class)) {
    class <- tibble_class_no_data_frame
  } else {
    class <- c(class[!class %in% tibble_class], tibble_class_no_data_frame)
  }

  slots <- c("x", "n", "class")
  args[slots] <- list(x, nrow, class)

  # `new_data_frame()` restores compact row names
  # Can't add to the assignment above, a literal NULL would be inserted otherwise
  args[["row.names"]] <- NULL

  # need exec() to avoid evaluating language attributes (e.g. rsample)
  exec(new_data_frame, !!!args)
}

#' @description
#' `validate_tibble()` checks a tibble for internal consistency.
#' Correct behavior can be guaranteed only if this function
#' runs without raising an error.
#'
#' @rdname new_tibble
#' @export
validate_tibble <- function(x) {
  #' @section Validation:
  #' `validate_tibble()` checks for "minimal" names
  check_minimal_names(x)

  #' and that all columns are vectors, data frames or matrices.
  check_valid_cols(unclass(x))

  #' It also makes sure that all columns have the same length,
  #' and that [vctrs::vec_size()] is consistent with the data.
  validate_nrow(names(x), col_lengths(x), vec_size(x))

  x
}

cnd_signal_if <- function(x) {
  if (!is.null(x)) {
    cnd_signal(x)
  }
}

check_minimal <- function(name) {
  cnd_signal_if(cnd_names_non_null(name))
  cnd_signal_if(cnd_names_non_na(name))
}

check_minimal_names <- function(x) {
  check_minimal(names(x))
  invisible(x)
}

col_lengths <- function(x) {
  map_int(x, vec_size)
}

validate_nrow <- function(names, lengths, nrow) {
  # Validate column lengths, don't recycle
  bad_len <- which(lengths != nrow)
  if (has_length(bad_len)) {
    cnd_signal(error_incompatible_size(nrow, names, lengths, "Requested with `nrow` argument"))
  }
}

tibble_class <- c("tbl_df", "tbl", "data.frame")
tibble_class_no_data_frame <- c("tbl_df", "tbl")

# Errors ------------------------------------------------------------------

error_new_tibble_must_be_list <- function() {
  tibble_error("`x` must be a list.")
}

error_new_tibble_nrow_must_be_nonnegative <- function() {
  tibble_error("`nrow` must be a nonnegative whole number smaller than 2^31.")
}

#' Build a data frame
#'
#' @description
#'
#' `tibble()` constructs a data frame. It is used like [base::data.frame()], but
#' with a couple notable differences:
#'
#'   * The returned data frame has the class [`tbl_df`][tbl_df-class], in
#'     addition to `data.frame`. This allows so-called "tibbles" to exhibit some
#'     special behaviour, such as [enhanced printing][formatting]. Tibbles are
#'     fully described in [`tbl_df`][tbl_df-class].
#'   * `tibble()` is much lazier than [base::data.frame()] in terms of
#'     transforming the user's input.
#'
#'       - Character vectors are not coerced to factor.
#'       - List-columns are expressly anticipated and do not require special tricks.
#'       - Column names are not modified.
#'       - Inner names in columns are left unchanged.
#'
#'   * `tibble()` builds columns sequentially. When defining a column, you can
#'     refer to columns created earlier in the call. Only columns of length one
#'     are recycled.
#'   * If a column evaluates to a data frame or tibble, it is nested or spliced.
#'     If it evaluates to a matrix or a array, it remains a matrix or array,
#'     respectively.
#'     See examples.
#'
#' @param ... <[`dynamic-dots`][rlang::dyn-dots]>
#'   A set of name-value pairs. These arguments are
#'   processed with [rlang::quos()] and support unquote via [`!!`] and
#'   unquote-splice via [`!!!`]. Use `:=` to create columns that start with a dot.
#'
#'   Arguments are evaluated sequentially.
#'   You can refer to previously created elements directly or using the [.data]
#'   pronoun.
#'   To refer explicitly to objects in the calling environment, use [`!!`] or
#'   [.env], e.g. `!!.data` or `.env$.data` for the special case of an object
#'   named `.data`.
#' @param .rows The number of rows, useful to create a 0-column tibble or
#'   just as an additional check.
#' @param .name_repair Treatment of problematic column names:
#'   * `"minimal"`: No name repair or checks, beyond basic existence,
#'   * `"unique"`: Make sure names are unique and not empty,
#'   * `"check_unique"`: (default value), no name repair, but check they are
#'     `unique`,
#'   * `"universal"`: Make the names `unique` and syntactic
#'   * a function: apply custom name repair (e.g., `.name_repair = make.names`
#'     for names in the style of base R).
#'   * A purrr-style anonymous function, see [rlang::as_function()]
#'
#'   This argument is passed on as `repair` to [vctrs::vec_as_names()].
#'   See there for more details on these terms and the strategies used
#'   to enforce them.
#'
#' @return A tibble, which is a colloquial term for an object of class
#'   [`tbl_df`][tbl_df-class]. A [`tbl_df`][tbl_df-class] object is also a data
#'   frame, i.e. it has class `data.frame`.
#' @seealso Use [as_tibble()] to turn an existing object into a tibble. Use
#'   `enframe()` to convert a named vector into a tibble. Name repair is
#'   detailed in [vctrs::vec_as_names()].
#'   See [quasiquotation] for more details on tidy dots semantics,
#'   i.e. exactly how  the `...` argument is processed.
#' @export
#' @examples
#' # Unnamed arguments are named with their expression:
#' a <- 1:5
#' tibble(a, a * 2)
#'
#' # Scalars (vectors of length one) are recycled:
#' tibble(a, b = a * 2, c = 1)
#'
#' # Columns are available in subsequent expressions:
#' tibble(x = runif(10), y = x * 2)
#'
#' # tibble() never coerces its inputs,
#' str(tibble(letters))
#' str(tibble(x = list(diag(1), diag(2))))
#'
#' # or munges column names (unless requested),
#' tibble(`a + b` = 1:5)
#'
#' # but it forces you to take charge of names, if they need repair:
#' try(tibble(x = 1, x = 2))
#' tibble(x = 1, x = 2, .name_repair = "unique")
#' tibble(x = 1, x = 2, .name_repair = "minimal")
#'
#' ## By default, non-syntactic names are allowed,
#' df <- tibble(`a 1` = 1, `a 2` = 2)
#' ## because you can still index by name:
#' df[["a 1"]]
#' df$`a 1`
#' with(df, `a 1`)
#'
#' ## Syntactic names are easier to work with, though, and you can request them:
#' df <- tibble(`a 1` = 1, `a 2` = 2, .name_repair = "universal")
#' df$a.1
#'
#' ## You can specify your own name repair function:
#' tibble(x = 1, x = 2, .name_repair = make.unique)
#'
#' fix_names <- function(x) gsub("\\s+", "_", x)
#' tibble(`year 1` = 1, `year 2` = 2, .name_repair = fix_names)
#'
#' ## purrr-style anonymous functions and constants
#' ## are also supported
#' tibble(x = 1, x = 2, .name_repair = ~ make.names(., unique = TRUE))
#'
#' tibble(x = 1, x = 2, .name_repair = ~ c("a", "b"))
#'
#' # Tibbles can contain columns that are tibbles or matrices
#' # if the number of rows is compatible. Unnamed tibbled are
#' # spliced, i.e. the inner columns are inserted into the
#' # tibble under construction.
#' tibble(
#'   a = 1:3,
#'   tibble(
#'     b = 4:6,
#'     c = 7:9
#'   ),
#'   d = tibble(
#'     e = tibble(
#'       f = b
#'     )
#'   )
#' )
#' tibble(
#'   a = 1:3,
#'   b = diag(3),
#'   c = cor(trees),
#'   d = Titanic[1:3, , , ]
#' )
#'
#' # Data can not contain tibbles or matrices with incompatible number of rows:
#' try(tibble(a = 1:3, b = tibble(c = 4:7)))
#'
#' # Use := to create columns with names that start with a dot:
#' tibble(.dotted := 3)
#'
#' # This also works, but might break in the future:
#' tibble(.dotted = 3)
#'
#' # You can unquote an expression:
#' x <- 3
#' tibble(x = 1, y = x)
#' tibble(x = 1, y = !!x)
#'
#' # You can splice-unquote a list of quosures and expressions:
#' tibble(!!!list(x = rlang::quo(1:10), y = quote(x * 2)))
#'
#' # Use .data, .env and !! to refer explicitly to columns or outside objects
#' a <- 1
#' tibble(a = 2, b = a)
#' tibble(a = 2, b = .data$a)
#' tibble(a = 2, b = .env$a)
#' tibble(a = 2, b = !!a)
#' try(tibble(a = 2, b = .env$bogus))
#' try(tibble(a = 2, b = !!bogus))
tibble <- function(...,
                   .rows = NULL,
                   .name_repair = c("check_unique", "unique", "universal", "minimal")) {
  xs <- quos(...)

  tibble_quos(xs, .rows, .name_repair)
}

#' tibble_row()
#'
#' @description
#' `tibble_row()` constructs a data frame that is guaranteed to occupy one row.
#' Vector columns are required to have size one, non-vector columns are wrapped
#' in a list.
#'
#' @rdname tibble
#' @export
#' @examples
#'
#' # Use tibble_row() to construct a one-row tibble:
#' tibble_row(a = 1, lm = lm(Height ~ Girth + Volume, data = trees))
tibble_row <- function(...,
                       .name_repair = c("check_unique", "unique", "universal", "minimal")) {
  xs <- enquos(...)

  tibble_quos(xs, .rows = 1, .name_repair = .name_repair, single_row = TRUE)
}

#' Test if the object is a tibble
#'
#' This function returns `TRUE` for tibbles or subclasses thereof,
#' and `FALSE` for all other objects, including regular data frames.
#'
#' @param x An object
#' @return `TRUE` if the object inherits from the `tbl_df` class.
#' @export
is_tibble <- function(x) {
  inherits(x, "tbl_df")
}

#' Deprecated test for tibble-ness
#'
#' @description
#' `r lifecycle::badge("soft-deprecated")`
#'
#' Please use [is_tibble()] instead.
#'
#' @inheritParams is_tibble
#' @export
#' @keywords internal
is.tibble <- function(x) {
  deprecate_warn("2.0.0", "is.tibble()", "is_tibble()")

  is_tibble(x)
}

tibble_quos <- function(xs, .rows, .name_repair, single_row = FALSE) {
  # Evaluate each column in turn
  col_names <- given_col_names <- names2(xs)
  empty_col_names <- which(col_names == "")
  col_names[empty_col_names] <- names(quos_auto_name(xs[empty_col_names]))
  lengths <- rep_along(xs, 0L)

  output <- rep_along(xs, list(NULL))

  env <- new_environment()
  mask <- new_data_mask_with_data(env)

  first_size <- .rows

  for (j in seq_along(xs)) {
    res <- eval_tidy(xs[[j]], mask)

    if (!is.null(res)) {
      # Single-row mode: Vectors must be length one, non-vectors are wrapped
      # in a list (which is length one by definition)
      if (single_row) {
        if (vec_is(res)) {
          if (vec_size(res) != 1) {
            cnd_signal(error_tibble_row_size_one(j, given_col_names[[j]], vec_size(res)))
          }
        } else {
          res <- list(res)
        }
      } else {
        # 657
        res <- check_valid_col(res, col_names[[j]], j)

        lengths[[j]] <- current_size <- vec_size(res)
        if (is.null(first_size)) {
          first_size <- current_size
        } else if (first_size == 1L && current_size != 1L) {
          idx_to_fix <- seq2(1L, j - 1L)
          output[idx_to_fix] <- fixed_output <-
            map(output[idx_to_fix], vec_recycle, current_size)

          # Refill entire data mask
          map2(output[idx_to_fix], col_names[idx_to_fix], add_to_env2, env = env)

          first_size <- current_size
        } else {
          res <- vectbl_recycle_rows(res, first_size, j, given_col_names[[j]])
        }
      }

      output[[j]] <- res
      col_names[[j]] <- add_to_env2(res, given_col_names[[j]], col_names[[j]], env)
    }
  }

  names(output) <- col_names

  is_null <- map_lgl(output, is.null)
  output <- output[!is_null]

  output <- splice_dfs(output)
  output <- set_repaired_names(output, repair_hint = TRUE, .name_repair = .name_repair)

  new_tibble(output, nrow = first_size %||% 0L)
}

check_valid_col <- function(x, name, pos) {
  if (name == "") {
    ret <- check_valid_cols(list(x), pos = pos)
  } else {
    ret <- check_valid_cols(set_names(list(x), name))
  }
  invisible(ret[[1]])
}

new_data_mask_with_data <- function(env) {
  mask <- new_data_mask(env)
  mask$.data <- as_data_pronoun(env)
  mask
}

add_to_env2 <- function(x, given_name, name = given_name, env) {
  if (is.data.frame(x) && given_name == "") {
    imap(x, add_to_env, env)
    ""
  } else {
    add_to_env(x, name, env)
    name
  }
}

add_to_env <- function(x, name, env) {
  env[[name]] <- x
  invisible()
}

splice_dfs <- function(x) {
  # Avoiding .ptype argument to vec_c()
  if (is_empty(x)) {
    return(list())
  }

  x <- imap(x, function(.x, .y) {
    if (.y == "") unclass(.x) else list2(!!.y := .x)
  })
  vec_c(!!!x, .name_spec = "{inner}")
}

vectbl_recycle_rows <- function(x, n, j, name) {
  size <- vec_size(x)
  if (size == n) return(x)
  if (size == 1) return(vec_recycle(x, n))

  if (name == "") {
    name <- j
  }

  cnd_signal(error_incompatible_size(n, name, size, "Existing data"))
}

# Errors ------------------------------------------------------------------

error_tibble_row_size_one <- function(j, name, size) {
  if (name != "") {
    desc <- tick(name)
  } else {
    desc <- paste0("at position ", j)
  }

  tibble_error(problems(
    "All vectors must be size one, use `list()` to wrap.",
    paste0("Column ", desc, " is of size ", size, ".")
  ))
}

error_incompatible_size <- function(.rows, vars, vars_len, rows_source) {
  vars_split <- split(vars, vars_len)

  vars_split[["1"]] <- NULL
  if (!is.null(.rows)) {
    vars_split[[as.character(.rows)]] <- NULL
  }

  problems <- map2_chr(names(vars_split), vars_split, function(x, y) {
    if (is.numeric(y)) {
      text <- "Column(s) at position(s) "
    } else {
      text <- "Column(s) "
      y <- tick(y)
    }

    paste0("Size ", x, ": ", pluralise_commas(text, y))
  })

  tibble_error(bullets(
    "Tibble columns must have compatible sizes:",
    if (!is.null(.rows)) paste0("Size ", .rows, ": ", rows_source),
    problems,
    info = "Only values of size one are recycled."
  ))
}

#' @importFrom pillar tbl_sum
#' @export
pillar::tbl_sum

#' @export
#' @importFrom pillar dim_desc
tbl_sum.tbl_df <- function(x, ...) {
  c("A tibble" = dim_desc(x))
}

#' @importFrom pillar obj_sum
#' @export
pillar::obj_sum

#' @importFrom pillar type_sum
#' @export
pillar::type_sum

#' @importFrom pillar size_sum
#' @export
pillar::size_sum

#' @export
vec_ptype_abbr.tbl_df <- function(x, ...) {
  abbr <- class(x)[[1]]
  if (abbr == "tbl_df") {
    abbr <- "tibble"
  }
  abbr
}

render_galley_ext <- function(input_path, pkg, installed, output_dir, output_file) {
  # stopifnot(!installed)
  if (installed) {
    library(pkg, character.only = TRUE)
  } else {
    pkgload::load_all()
  }

  testthat::local_reproducible_output()

  Sys.time <- function() {
    structure(1627618285.45488, class = c("POSIXct", "POSIXt"), tzone = "UTC")
  }
  Sys.Date <- function() {
    structure(18838, class = "Date")
  }
  set.seed(20210730)
  Sys.setenv("IN_PKGDOWN" = "true", "IN_GALLEY" = "true")

  rmarkdown::render(
    input_path,
    output_dir = output_dir,
    output_file = output_file,
    run_pandoc = FALSE,
    output_format = rmarkdown::md_document(preserve_yaml = TRUE)
  )
}

galley_use_installed <- function() {
  grepl("[.]Rcheck$", basename(normalizePath("../..")))
}

render_galley <- function(name, md_name) {
  pkg <- utils::packageName()
  # FIXME: Hack!
  installed <- galley_use_installed()
  # stopifnot(!installed)

  if (installed) {
    input_path <- system.file("doc", name, package = pkg)
  } else {
    input_path <- system.file("vignettes", name, package = pkg)
  }

  # Need fixed file name for stability
  output_dir <- tempdir()
  output_file <- md_name

  out_text <- character()

  knit_path <- tryCatch(
    callr::r(
      render_galley_ext,
      args = list(
        input_path = input_path, pkg = pkg, installed = installed,
        output_dir = output_dir, output_file = output_file
      ),
      callback = function(x) {
        out_text <<- c(out_text, x)
      }
    ),
    error = function(e) {
      writeLines(c("", out_text, ""))
      stop(e)
      # rlang::abort(paste0("Error rendering ", name))
    }
  )

  path <- file.path(output_dir, output_file)
  full_knit_path <- file.path(dirname(input_path), knit_path)
  scrub_file(path, full_knit_path)
  unlink(full_knit_path)

  path
}

scrub_tempdir <- function(x) {
  stable_tmpdir <- "${TEMP}"

  tmpdir_rx <- utils::glob2rx(paste0("*", dirname(tempdir()), "*"), trim.head = TRUE)
  gsub(paste0("(/private)?", tmpdir_rx, "[/\\\\]+Rtmp[0-9a-zA-Z]+"), stable_tmpdir, x)
}

scrub <- function(x) {
  x <- gsub("[<]bytecode: 0x.*[>]", "<bytecode: 0x1ee4c0de>", x)
  x <- gsub("[<]environment: 0x.*[>]", "<environment: 0xdeadbeef>", x)

  x <- scrub_tempdir(x)

  paste0(x, "\n", collapse = "")
}

scrub_file <- function(path, in_path = path) {
  text <- brio::read_lines(in_path)
  brio::write_file(scrub(text), path)
}

test_galley <- function(name, variant = NULL) {
  testthat::skip_on_cran()
  testthat::skip_if("covr" %in% loadedNamespaces())

  rmd_name <- paste0(name, ".Rmd")
  md_name <- paste0(name, ".md")

  path <- render_galley(rmd_name, md_name)

  if (!is.null(variant)) {
    testthat::skip_if_not_installed("testthat", "3.1.1")
    testthat::expect_snapshot_file(
      path,
      name = md_name, compare = testthat::compare_file_text,
      variant = variant
    )
  } else {
    testthat::expect_snapshot_file(
      path,
      name = md_name, compare = testthat::compare_file_text
    )
  }

  # FIXME: Test generated files
}

needs_dim <- function(x) {
  length(dim(x)) > 1L
}

has_null_names <- function(x) {
  is.null(names(x))
}

needs_list_col <- function(x) {
  is_list(x) || !vec_is(x) || vec_size(x) != 1L
}

# Work around bug in R 3.3.0
# Can be ressigned during loading (#544)
safe_match <- match


nchar_width <- function(x) {
  nchar(x, type = "width")
}

is_rstudio <- function() {
  !is.na(Sys.getenv("RSTUDIO", unset = NA))
}

#' Coerce lists, matrices, and more to data frames
#'
#' @description
#' `as_tibble()` turns an existing object, such as a data frame or
#' matrix, into a so-called tibble, a data frame with class [`tbl_df`]. This is
#' in contrast with [tibble()], which builds a tibble from individual columns.
#' `as_tibble()` is to [`tibble()`] as [base::as.data.frame()] is to
#' [base::data.frame()].
#'
#' `as_tibble()` is an S3 generic, with methods for:
#' * [`data.frame`][base::data.frame()]: Thin wrapper around the `list` method
#'   that implements tibble's treatment of [rownames].
#' * [`matrix`][base::matrix()], [`poly`][stats::poly()],
#'   [`ts`][stats::ts()], [`table`][base::table()]
#' * Default: Other inputs are first coerced with [base::as.data.frame()].
#'
#' @section Row names:
#' The default behavior is to silently remove row names.
#'
#' New code should explicitly convert row names to a new column using the
#' `rownames` argument.
#'
#' For existing code that relies on the retention of row names, call
#' `pkgconfig::set_config("tibble::rownames" = NA)` in your script or in your
#' package's [.onLoad()]  function.
#'
#' @section Life cycle:
#' Using `as_tibble()` for vectors is superseded as of version 3.0.0,
#' prefer the more expressive `as_tibble_row()` and
#' `as_tibble_col()` variants for new code.
#'
#' @seealso [tibble()] constructs a tibble from individual columns. [enframe()]
#'   converts a named vector to a tibble with a column of names and column of
#'   values. Name repair is implemented using [vctrs::vec_as_names()].
#'
#' @param x A data frame, list, matrix, or other object that could reasonably be
#'   coerced to a tibble.
#' @param ... Unused, for extensibility.
#' @inheritParams tibble
#' @param rownames How to treat existing row names of a data frame or matrix:
#'   * `NULL`: remove row names. This is the default.
#'   * `NA`: keep row names.
#'   * A string: the name of a new column. Existing rownames are transferred
#'     into this column and the `row.names` attribute is deleted.
#'     No name repair is applied to the new column name, even if `x` already contains
#'     a column of that name.
#'     Use `as_tibble(rownames_to_column(...))` to safeguard against this case.
#'
#'  Read more in [rownames].

#' @param _n,validate
#'   `r lifecycle::badge("soft-deprecated")`
#'
#'   For compatibility only, do not use for new code.
#' @export
#' @examples
#' m <- matrix(rnorm(50), ncol = 5)
#' colnames(m) <- c("a", "b", "c", "d", "e")
#' df <- as_tibble(m)
as_tibble <- function(x, ...,
                      .rows = NULL,
                      .name_repair = c("check_unique", "unique", "universal", "minimal"),
                      rownames = pkgconfig::get_config("tibble::rownames", NULL)) {
  UseMethod("as_tibble")
}

#' @export
#' @rdname as_tibble
as_tibble.data.frame <- function(x, validate = NULL, ...,
                                 .rows = NULL,
                                 .name_repair = c("check_unique", "unique", "universal", "minimal"),
                                 rownames = pkgconfig::get_config("tibble::rownames", NULL)) {
  if (!is.null(validate)) {
    deprecate_stop("2.0.0", "tibble::as_tibble(validate = )", "as_tibble(.name_repair =)")
  }

  old_rownames <- raw_rownames(x)
  if (is.null(.rows)) {
    .rows <- nrow(x)
  }

  result <- lst_to_tibble(unclass(x), .rows, .name_repair)

  if (is.null(rownames)) {
    result
  } else if (is.na(rownames)) {
    attr(result, "row.names") <- old_rownames
    result
  } else {
    if (length(old_rownames) > 0 && is.na(old_rownames[1L])) {
      # if implicit rownames
      old_rownames <- seq_len(abs(old_rownames[2L]))
    }
    old_rownames <- as.character(old_rownames)
    add_column(result, !!rownames := old_rownames, .before = 1L, .name_repair = "minimal")
  }
}

#' @export
#' @rdname as_tibble
as_tibble.list <- function(x, validate = NULL, ..., .rows = NULL,
                           .name_repair = c("check_unique", "unique", "universal", "minimal")) {
  if (!is.null(validate)) {
    deprecate_stop("2.0.0", "tibble::as_tibble(validate = )", "as_tibble(.name_repair =)")
  }

  lst_to_tibble(x, .rows, .name_repair, col_lengths(x))
}

lst_to_tibble <- function(x, .rows, .name_repair, lengths = NULL, call = caller_env()) {
  x <- unclass(x)
  x <- set_repaired_names(x, repair_hint = TRUE, .name_repair, call = call)
  x <- check_valid_cols(x, call = call)
  recycle_columns(x, .rows, lengths)
}

check_valid_cols <- function(x, pos = NULL, call = caller_env()) {
  names_x <- names2(x)

  is_xd <- which(!map_lgl(x, is_valid_col))
  if (has_length(is_xd)) {
    classes <- map_chr(x[is_xd], friendly_type_of)
    abort_column_scalar_type(names_x[is_xd], pos[is_xd], classes, call)
  }

  # 657
  x[] <- map(x, make_valid_col)
  invisible(x)
}

make_valid_col <- function(x) {
  if (is.expression(x)) {
    x <- as.list(x)
  }
  x
}

is_valid_col <- function(x) {
  # 657
  vec_is(x) || is.expression(x)
}

recycle_columns <- function(x, .rows, lengths) {
  nrow <- guess_nrow(lengths, .rows)

  # Shortcut if all columns have the requested or implied length
  different_len <- which(lengths != nrow)
  if (is_empty(different_len)) return(new_tibble(x, nrow = nrow, subclass = NULL))

  if (any(lengths[different_len] != 1)) {
    abort_incompatible_size(.rows, names(x), lengths, "Requested with `.rows` argument")
  }

  if (nrow != 1L) {
    short <- which(lengths == 1L)
    if (has_length(short)) {
      x[short] <- map(x[short], vec_recycle, nrow)
    }
  }

  new_tibble(x, nrow = nrow, subclass = NULL)
}

guess_nrow <- function(lengths, .rows) {
  if (!is.null(.rows)) {
    return(.rows)
  }
  if (is_empty(lengths)) {
    return(0)
  }

  nontrivial_lengths <- lengths[lengths != 1L]
  if (is_empty(nontrivial_lengths)) {
    return(1)
  }

  max(nontrivial_lengths)
}

#' @export
#' @rdname as_tibble
as_tibble.matrix <- function(x, ..., validate = NULL, .name_repair = NULL) {
  m <- matrixToDataFrame(x)
  names <- colnames(x)
  if (is.null(.name_repair)) {
    if ((is.null(names) || any(bad_names <- duplicated(names) | names == "")) && has_length(x)) {
      deprecate_warn("2.0.0", "as_tibble.matrix(x = 'must have unique column names if `.name_repair` is omitted')",
        details = "Using compatibility `.name_repair`."
      )
      compat_names <- paste0("V", seq_along(m))
      if (is.null(names)) {
        names <- compat_names
      } else {
        names[bad_names] <- compat_names[bad_names]
      }
      .name_repair <- function(x) names
    } else {
      .name_repair <- "check_unique"
    }
    validate <- NULL
  }

  colnames(m) <- names
  as_tibble(m, ..., validate = validate, .name_repair = .name_repair)
}

#' @export
as_tibble.poly <- function(x, ...) {
  m <- matrixToDataFrame(unclass(x))
  colnames(m) <- colnames(x)
  as_tibble(m, ...)
}

#' @export
as_tibble.ts <- function(x, ..., .name_repair = "minimal") {
  df <- as.data.frame(x)
  if (length(dim(x)) == 2) {
    colnames(df) <- colnames(x)
  }
  as_tibble(df, ..., .name_repair = .name_repair)
}

#' @export
#' @param n Name for count column, default: `"n"`.
#' @rdname as_tibble
as_tibble.table <- function(x, `_n` = "n", ..., n = `_n`, .name_repair = "check_unique") {
  if (!missing(`_n`)) {
    warn("Please pass `n` as a named argument to `as_tibble.table()`.")
  }

  df <- as.data.frame(x, stringsAsFactors = FALSE)

  names(df) <- repaired_names(
    c(names2(dimnames(x)), n),
    repair_hint = TRUE, .name_repair = .name_repair
  )

  # Names already repaired:
  as_tibble(df, ..., .name_repair = "minimal")
}

#' @export
#' @rdname as_tibble
#' @usage \method{as_tibble}{NULL}(x, ...)
as_tibble.NULL <- function(x, ...) {
  if (missing(x)) {
    deprecate_stop("3.0.0", "as_tibble(x = 'can\\'t be missing')")
  }

  new_tibble(list(), nrow = 0)
}

#' @export
#' @rdname as_tibble
as_tibble.default <- function(x, ...) {
  value <- x
  if (is_atomic(value)) {
    signal_superseded(
      "3.0.0", "as_tibble(x = 'can\\'t be an atomic vector')",
      "as_tibble_col()"
    )
  }
  as_tibble(as.data.frame(value, stringsAsFactors = FALSE), ...)
}

#' @description
#' `as_tibble_row()` converts a vector to a tibble with one row.
#' If the input is a list, all elements must have size one.
#'
#' @rdname as_tibble
#' @export
#' @examples
#'
#' as_tibble_row(c(a = 1, b = 2))
#' as_tibble_row(list(c = "three", d = list(4:5)))
#' as_tibble_row(1:3, .name_repair = "unique")
as_tibble_row <- function(x,
                          .name_repair = c("check_unique", "unique", "universal", "minimal")) {
  if (!vec_is(x)) {
    abort_as_tibble_row_vector(x)
  }

  names <- vectbl_names2(x, .name_repair = .name_repair)

  # FIXME: Use vec_chop2() when https://github.com/r-lib/vctrs/pull/1226 is in
  if (is_bare_list(x)) {
    slices <- x
  } else {
    x <- vectbl_set_names(x, NULL)
    slices <- lapply(seq_len(vec_size(x)), vec_slice, x = x)
    names(slices) <- names
  }

  check_all_lengths_one(slices)

  new_tibble(slices, nrow = 1)
}

check_all_lengths_one <- function(x, call = caller_env()) {
  sizes <- col_lengths(x)

  bad_lengths <- which(sizes != 1)
  if (!is_empty(bad_lengths)) {
    abort_as_tibble_row_size_one(
      seq_along(x)[bad_lengths],
      names2(x)[bad_lengths],
      sizes[bad_lengths],
      call
    )
  }
}


#' @description
#' `as_tibble_col()` converts a vector to a tibble with one column.
#'
#' @param column_name Name of the column.
#'
#' @rdname as_tibble
#' @export
#' @examples
#'
#' as_tibble_col(1:3)
#' as_tibble_col(
#'   list(c = "three", d = list(4:5)),
#'   column_name = "data"
#' )
as_tibble_col <- function(x, column_name = "value") {
  # Side effect: checking that x is a vector
  tibble(!!column_name := x)
}

# External ----------------------------------------------------------------

matrixToDataFrame <- function(x) {
  .Call(`tibble_matrixToDataFrame`, x)
}

# Errors ------------------------------------------------------------------

abort_column_scalar_type <- function(names, positions, classes, call = caller_env()) {
  tibble_abort(
    call = call,
    problems(
      "All columns in a tibble must be vectors:",
      x = paste0("Column ", name_or_pos(names, positions), " is ", classes)
    ),
    names = names
  )
}

abort_as_tibble_row_vector <- function(x, call = caller_env()) {
  tibble_abort(call = call, paste0(
    "`x` must be a vector in `as_tibble_row()`, not ", class(x)[[1]], "."
  ))
}

abort_as_tibble_row_size_one <- function(j, name, size, call = caller_env()) {
  desc <- tick(name)
  desc[name == ""] <- paste0("at position ", j[name == ""])

  tibble_abort(call = call, problems(
    "All elements must be size one, use `list()` to wrap.",
    paste0("Element ", desc, " is of size ", size, ".")
  ))
}

# as_tibble -----------------------------------------------------------

test_that("columns are recycled to common length", {
  expect_identical(
    as_tibble(list(x = 1, y = 1:3)),
    tibble(x = rep(1, 3), y = 1:3)
  )
  expect_identical(
    as_tibble(list(x = 1:3, y = 1)),
    tibble(x = 1:3, y = rep(1, 3))
  )
  expect_identical(
    as_tibble(list(x = character(), y = 1)),
    tibble(x = character(), y = numeric())
  )
})

test_that("columns must be same length", {
  expect_tibble_abort(
    as_tibble(list(x = 1:2, y = 1:3)),
    abort_incompatible_size(NULL, c("x", "y"), 2:3, NA)
  )
  expect_tibble_abort(
    as_tibble(list(x = 1:2, y = 1:3, z = 1:4)),
    abort_incompatible_size(
      NULL,
      c("x", "y", "z"),
      2:4,
      NA
    )
  )
  expect_tibble_abort(
    as_tibble(list(x = 1:4, y = 1:2, z = 1:2)),
    abort_incompatible_size(
      NULL,
      c("x", "y", "z"),
      c(4, 2, 2),
      NA
    )
  )
  expect_tibble_abort(
    as_tibble(list(x = 1, y = 1:4, z = 1:2)),
    abort_incompatible_size(
      NULL,
      c("y", "z"),
      c(4, 2),
      NA
    )
  )
  expect_tibble_abort(
    as_tibble(list(x = 1:2, y = 1:4, z = 1)),
    abort_incompatible_size(
      NULL,
      c("x", "y"),
      c(2, 4),
      NA
    )
  )
})

test_that("empty list() makes 0 x 0 tbl_df", {
  zero <- as_tibble(list())
  expect_s3_class(zero, "tbl_df")
  expect_equal(dim(zero), c(0L, 0L))
})


test_that("NULL makes 0 x 0 tbl_df", {
  nnnull <- as_tibble(NULL)
  expect_s3_class(nnnull, "tbl_df")
  expect_equal(dim(nnnull), c(0L, 0L))
})


test_that("as_tibble() without arguments raises a lifecycle warning", {
  scoped_lifecycle_errors()

  expect_error(as_tibble())
})


test_that("as_tibble.tbl_df() leaves classes unchanged (#60)", {
  df <- tibble()
  expect_equal(
    class(df),
    c("tbl_df", "tbl", "data.frame")
  )
  expect_equal(
    class(structure(df, class = c("my_df", class(df)))),
    c("my_df", "tbl_df", "tbl", "data.frame")
  )
})


test_that("Can convert tables to data frame", {
  mtcars_table <- xtabs(mtcars, formula = ~ vs + am + cyl)

  mtcars2 <- as_tibble(mtcars_table)
  expect_equal(names(mtcars2), c(names(dimnames(mtcars_table)), "n"))

  expect_warning(
    mtcars2 <- as_tibble(mtcars_table, "Freq"),
    "named argument",
    fixed = TRUE
  )
  expect_equal(names(mtcars2), c(names(dimnames(mtcars_table)), "Freq"))

  mtcars2 <- as_tibble(mtcars_table, n = "Freq")
  expect_equal(names(mtcars2), c(names(dimnames(mtcars_table)), "Freq"))
})


test_that("Superseded: Can convert unnamed atomic vectors to tibble by default", {
  expect_equal(as_tibble(1:3), tibble(value = 1:3))
  expect_equal(as_tibble(c(TRUE, FALSE, NA)), tibble(value = c(TRUE, FALSE, NA)))
  expect_equal(as_tibble(1.5:3.5), tibble(value = 1.5:3.5))
  expect_equal(as_tibble(letters), tibble(value = letters))
})


test_that("as_tibble() checks for `unique` names by default (#278)", {
  l1 <- list(1:10)
  expect_tibble_abort(
    as_tibble(l1),
    abort_column_names_cannot_be_empty(1, repair_hint = TRUE)
  )

  l2 <- list(x = 1, 2)
  expect_tibble_abort(
    as_tibble(l2),
    abort_column_names_cannot_be_empty(2, repair_hint = TRUE)
  )

  l3 <- list(x = 1, ... = 2)
  expect_tibble_abort(
    as_tibble(l3),
    abort_column_names_cannot_be_dot_dot(2, repair_hint = TRUE)
  )

  l4 <- list(x = 1, ..1 = 2)
  expect_tibble_abort(
    as_tibble(l4),
    abort_column_names_cannot_be_dot_dot(2, repair_hint = TRUE)
  )

  df <- list(a = 1, b = 2)
  names(df) <- c("", NA)
  df <- new_tibble(df, nrow = 1)
  expect_tibble_abort(
    as_tibble(df),
    abort_column_names_cannot_be_empty(1:2, repair_hint = TRUE)
  )
})


test_that("as_tibble() makes names `minimal`, even if not fixing names", {
  invalid_df <- as_tibble(list(3, 4, 5), .name_repair = "minimal")
  expect_equal(length(invalid_df), 3)
  expect_equal(nrow(invalid_df), 1)
  expect_equal(names(invalid_df), rep("", 3))
})

test_that("as_tibble() implements unique names", {
  skip_if_not_installed("vctrs", "0.3.8.9001")

  expect_snapshot({
    invalid_df <- as_tibble(list(3, 4, 5), .name_repair = "unique")
  })
  expect_equal(length(invalid_df), 3)
  expect_equal(nrow(invalid_df), 1)
  expect_equal(
    names(invalid_df),
    vec_as_names(rep("", 3), repair = "unique", quiet = TRUE)
  )
})

test_that("as_tibble() implements universal names", {
  skip_if_not_installed("vctrs", "0.3.8.9001")

  expect_snapshot({
    invalid_df <- as_tibble(list(3, 4, 5), .name_repair = "universal")
  })
  expect_equal(length(invalid_df), 3)
  expect_equal(nrow(invalid_df), 1)
  expect_equal(
    names(invalid_df),
    vec_as_names(rep("", 3), repair = "universal", quiet = TRUE)
  )
})


test_that("as_tibble() implements custom name repair", {
  expect_silent(
    invalid_df <- as_tibble(
      list(3, 4, 5),
      .name_repair = function(x) make.names(x, unique = TRUE)
    )
  )
  expect_equal(length(invalid_df), 3)
  expect_equal(nrow(invalid_df), 1)
  expect_equal(names(invalid_df), make.names(rep("", 3), unique = TRUE))

  invalid_df_purrr <- as_tibble(
    list(3, 4, 5),
    .name_repair = ~ make.names(., unique = TRUE)
  )
  expect_identical(invalid_df_purrr, invalid_df)
})

test_that("as_tibble.matrix() supports validate (with warning) (#558)", {
  expect_warning(
    expect_identical(
      as_tibble(diag(3), validate = TRUE),
      tibble(
        V1 = c(1, 0, 0),
        V2 = c(0, 1, 0),
        V3 = c(0, 0, 1)
      )
    )
  )
})

test_that("as_tibble.matrix() supports .name_repair", {
  skip_if_not_installed("vctrs", "0.3.8.9001")

  scoped_lifecycle_warnings()

  x <- matrix(1:6, nrow = 3)

  expect_warning(as_tibble(x))

  minimal <- as_tibble(x, .name_repair = "minimal")
  expect_identical(names(minimal), rep("", 2))

  expect_snapshot(
    universal <- as_tibble(x, .name_repair = "universal")
  )
  expect_identical(names(universal), paste0("...", 1:2))

  x <- matrix(1:6, nrow = 3, dimnames = list(x = LETTERS[1:3], y = c("if", "when")))

  expect_identical(
    names(as_tibble(x)),
    c("if", "when")
  )
  expect_identical(
    names(as_tibble(x, .name_repair = "minimal")),
    c("if", "when")
  )
  expect_snapshot(
    universal <- as_tibble(x, .name_repair = "universal")
  )
  expect_identical(names(universal), c(".if", "when"))
})

test_that("as_tibble.poly() supports .name_repair", {
  skip_if_not_installed("vctrs", "0.3.8.9001")

  x <- poly(1:6, 3)

  expect_identical(
    names(as_tibble(x)),
    as.character(1:3)
  )
  expect_identical(
    names(as_tibble(x, .name_repair = "minimal")),
    as.character(1:3)
  )
  expect_snapshot(
    universal <- as_tibble(x, .name_repair = "universal")
  )
  expect_identical(names(universal), paste0("...", 1:3))
})

test_that("as_tibble.table() supports .name_repair", {
  skip_if_not_installed("vctrs", "0.3.8.9001")

  expect_snapshot(error = TRUE, {
    as_tibble(table(a = c(1, 1, 1, 2, 2, 2), a = c(3, 4, 5, 3, 4, 5)))
    as_tibble(table(c(1, 1, 1, 2, 2, 2), c(3, 4, 5, 3, 4, 5)))
  })

  x <- table(a = c(1, 1, 1, 2, 2, 2), a = c(3, 4, 5, 3, 4, 5))
  expect_identical(
    names(as_tibble(x, .name_repair = "minimal")),
    c("a", "a", "n")
  )
  expect_snapshot(
    universal <- as_tibble(x, .name_repair = "universal")
  )
  expect_identical(names(universal), c("a...1", "a...2", "n"))

  x <- table("if" = c(1, 1, 1, 2, 2, 2), "when" = c(3, 4, 5, 3, 4, 5))

  expect_identical(
    names(as_tibble(x)),
    c("if", "when", "n")
  )
  expect_identical(
    names(as_tibble(x, .name_repair = "minimal")),
    c("if", "when", "n")
  )
  expect_snapshot(
    universal <- as_tibble(x, .name_repair = "universal")
  )
  expect_identical(names(universal), c(".if", "when", "n"))

  x <- table("m" = c(1, 1, 1, 2, 2, 2), "n" = c(3, 4, 5, 3, 4, 5))

  expect_identical(
    names(as_tibble(x, .name_repair = "minimal")),
    c("m", "n", "n")
  )
  expect_snapshot(
    universal <- as_tibble(x, .name_repair = "universal")
  )
  expect_identical(names(universal), c("m", "n...2", "n...3"))
})

test_that("as_tibble.ts() supports .name_repair, minimal by default (#537)", {
  skip_if_not_installed("vctrs", "0.3.8.9001")

  x <- ts(matrix(rnorm(6), nrow = 3), start = c(1961, 1), frequency = 12, names = NULL)

  expect_identical(
    names(as_tibble(x)),
    rep("", 2)
  )
  expect_identical(
    names(as_tibble(x, .name_repair = "minimal")),
    rep("", 2)
  )
  expect_snapshot(
    universal <- as_tibble(x, .name_repair = "universal")
  )
  expect_identical(names(universal), paste0("...", 1:2))

  x <- ts(matrix(rnorm(6), nrow = 3), start = c(1961, 1), frequency = 12, names = c("if", "when"))

  expect_identical(
    names(as_tibble(x)),
    c("if", "when")
  )
  expect_identical(
    names(as_tibble(x, .name_repair = "minimal")),
    c("if", "when")
  )
  expect_snapshot(
    universal <- as_tibble(x, .name_repair = "universal")
  )
  expect_identical(names(universal), c(".if", "when"))
})

test_that("as_tibble() can convert row names", {
  df <- data.frame(a = 1:3, b = 2:4, row.names = letters[5:7])

  expect_identical(
    as_tibble(df, rownames = NULL),
    tibble(a = 1:3, b = 2:4)
  )
  expect_identical(
    as_tibble(df, rownames = "id"),
    tibble(id = letters[5:7], a = 1:3, b = 2:4)
  )
  tbl_df <- as_tibble(df, rownames = NA)
  expect_identical(rownames(tbl_df), rownames(df))
  expect_identical(unclass(tbl_df), unclass(df))
})

test_that("as_tibble() can convert row names for zero-row tibbles", {
  df <- data.frame(a = 1:3, b = 2:4, row.names = letters[5:7])[0, ]

  expect_identical(
    as_tibble(df, rownames = NULL),
    tibble(a = integer(), b = integer())
  )
  expect_identical(
    as_tibble(df, rownames = "id"),
    tibble(id = character(), a = integer(), b = integer())
  )
  tbl_df <- as_tibble(df, rownames = NA)
  expect_identical(rownames(tbl_df), rownames(df))
  expect_identical(unclass(tbl_df), unclass(df))
})

test_that("as_tibble() converts implicit row names when `rownames =` is passed", {
  df <- data.frame(a = 1:3, b = 2:4)
  expect_equal(
    as_tibble(df, rownames = "id"),
    tibble(id = as.character(1:3), a = 1:3, b = 2:4)
  )
  expect_equal(
    as_tibble(df[0, ], rownames = "id"),
    tibble(id = character(0), a = integer(0), b = integer(0))
  )
})

test_that("as_data_frame() is an alias of as_tibble()", {
  scoped_lifecycle_silence()
  expect_identical(as_data_frame(NULL), as_tibble(NULL))
})

test_that("as.tibble() is an alias of as_tibble()", {
  scoped_lifecycle_silence()
  expect_identical(as.tibble(NULL), as_tibble(NULL))
})


# as_tibble_row -----------------------------------------------------------

test_that("as_tibble_row() can convert named bare vectors to data frame", {
  expect_identical(as_tibble_row(setNames(nm = 1:3)), tibble(`1` = 1L, `2` = 2L, `3` = 3L))
  expect_identical(as_tibble_row(setNames(nm = c(TRUE, FALSE))), tibble(`TRUE` = TRUE, `FALSE` = FALSE))
  expect_identical(as_tibble_row(setNames(nm = 1.5:3.5)), tibble(`1.5` = 1.5, `2.5` = 2.5, `3.5` = 3.5))
  expect_identical(as_tibble_row(setNames(nm = letters)), tibble(!!!setNames(nm = letters)))
  expect_identical(
    as_tibble_row(list(a = 1, b = list(2:3))),
    tibble(a = 1, b = list(2:3))
  )

  expect_tibble_abort(
    as_tibble_row(list(a = 1, b = 2:3)),
    abort_as_tibble_row_size_one(2, "b", 2)
  )
  expect_tibble_abort(
    as_tibble_row(setNames(nm = c(TRUE, FALSE, NA))),
    abort_column_names_cannot_be_empty(3, repair_hint = TRUE)
  )
})

test_that("as_tibble_row() works with non-bare vectors (#797)", {
  expect_tibble_abort(
    as_tibble_row(new_environment()),
    abort_as_tibble_row_vector(new_environment())
  )

  time <- vec_slice(Sys.time(), 1)
  expect_identical(
    as_tibble_row(time, .name_repair = "unique"),
    tibble(...1 = time)
  )
  expect_identical(
    as_tibble_row(trees[1:3, ], .name_repair = "unique"),
    tibble(
      ...1 = remove_rownames(trees[1, ]),
      ...2 = remove_rownames(trees[2, ]),
      ...3 = remove_rownames(trees[3, ])
    )
  )

  remove_first_dimname <- function(x) {
    dn <- dimnames(x)
    dn[1] <- list(NULL)
    dimnames(x) <- dn
    x
  }

  expect_identical(
    as_tibble_row(Titanic),
    tibble(
      "1st" = remove_first_dimname(Titanic[1, , , , drop = FALSE]),
      "2nd" = remove_first_dimname(Titanic[2, , , , drop = FALSE]),
      "3rd" = remove_first_dimname(Titanic[3, , , , drop = FALSE]),
      Crew  = remove_first_dimname(Titanic[4, , , , drop = FALSE])
    )
  )
})


# as_tibble_col -----------------------------------------------------------

test_that("as_tibble_col() can convert atomic vectors to data frame", {
  expect_identical(as_tibble_col(1:3), tibble(value = 1:3))
  expect_identical(as_tibble_col(list(4, 5:6), column_name = "data"), tibble(data = list(4, 5:6)))

  expect_tibble_abort(
    as_tibble_col(lm(y ~ x, data.frame(x = 1:3, y = 2:4))),
    abort_column_scalar_type("value", 1, "a `lm` object")
  )
})

# Validation --------------------------------------------------------------

test_that("`validate` triggers deprecation message, but then works", {
  scoped_lifecycle_warnings()

  expect_error(
    as_tibble(list(a = 1, "hi"), validate = TRUE)
  )

  expect_error(
    as_tibble(list(a = 1, "hi", a = 2), validate = FALSE),
    "deprecated",
    fixed = TRUE
  )

  df <- data.frame(a = 1, "hi", a = 2)
  names(df) <- c("a", "", "a")
  expect_error(
    as_tibble(df, validate = FALSE)
  )

  df <- data.frame(a = 1, "hi")
  names(df) <- c("a", "")
  expect_error(
    as_tibble(df, validate = TRUE)
  )
})

test_that("`validate` always raises lifecycle warning.", {
  expect_error(
    as_tibble(list(a = 1, "hi"), validate = TRUE, .name_repair = "check_unique")
  )

  expect_error(
    as_tibble(list(a = 1, "hi", a = 2), validate = FALSE, .name_repair = "minimal")
  )

  df <- data.frame(a = 1, "hi", a = 2)
  names(df) <- c("a", "", "a")
  expect_error(
    as_tibble(df, validate = FALSE, .name_repair = "minimal")
  )

  df <- data.frame(a = 1, "hi")
  names(df) <- c("a", "")
  expect_error(
    as_tibble(df, validate = TRUE, .name_repair = "check_unique")
  )
})

test_that("Inconsistent `validate` and `.name_repair` used together raise a warning.", {
  expect_error(
    as_tibble(list(a = 1, "hi"), validate = FALSE, .name_repair = "check_unique")
  )

  expect_error(
    as_tibble(list(a = 1, "hi", a = 2), validate = TRUE, .name_repair = "minimal")
  )

  df <- data.frame(a = 1, "hi", a = 2)
  names(df) <- c("a", "", "a")
  expect_error(
    as_tibble(df, validate = TRUE, .name_repair = "minimal")
  )

  df <- data.frame(a = 1, "hi")
  names(df) <- c("a", "")
  expect_error(
    as_tibble(df, validate = FALSE, .name_repair = "check_unique")
  )
})

test_that("correct rows and cols", {
  x <- matrix(1:6, nrow = 2)
  out <- as_tibble(x, .name_repair = "minimal")

  expect_equal(dim(out), c(2, 3))
})

test_that("correct rows and cols for 0 cols", {
  x <- matrix(integer(), nrow = 2)
  out <- as_tibble(x, .name_repair = "minimal")

  expect_equal(dim(out), c(2, 0))
})

test_that("correct rows and cols for 0 cols and legacy naming", {
  scoped_lifecycle_silence()

  x <- matrix(integer(), nrow = 2)
  out <- as_tibble(x)

  expect_equal(dim(out), c(2, 0))
})

test_that("correct rows and cols for 0 rows", {
  x <- matrix(integer(), ncol = 3)
  out <- as_tibble(x, .name_repair = "minimal")

  expect_equal(dim(out), c(0, 3))
})

test_that("preserves col names", {
  x <- matrix(1:4, nrow = 2)
  colnames(x) <- c("a", "b")

  out <- as_tibble(x)
  expect_equal(names(out), c("a", "b"))
})

test_that("supports compat col names", {
  scoped_lifecycle_warnings()

  x <- matrix(1:4, nrow = 2)

  expect_warning(out <- as_tibble(x))
  expect_equal(names(out), c("V1", "V2"))
})

test_that("creates col names with name repair", {
  skip_if_not_installed("vctrs", "0.3.8.9001")

  x <- matrix(1:4, nrow = 2)

  expect_snapshot(
    out <- as_tibble(x, .name_repair = "unique")
  )
  expect_equal(names(out), c("...1", "...2"))

  expect_snapshot(
    out <- as_tibble(x, .name_repair = "universal")
  )
  expect_equal(names(out), c("...1", "...2"))
})

test_that("preserves attributes except dim and names", {
  date <- Sys.Date() + 0:3
  dim(date) <- c(2, 2)
  colnames(date) <- c("a", "b")
  attr(date, "special") <- 42

  out <- as_tibble.matrix(date)
  expect_null(attributes(out[[1]])$names)
  expect_equal(attributes(out[[1]])$class, "Date")
  expect_equal(attributes(out[[2]])$special, 42)
})

test_that("properly handles poly class (#110)", {
  p <- poly(1:6, 3)
  p_df <- as_tibble(p)

  expect_equal(names(p_df), colnames(p))
  expect_equal(class(p_df[[1L]]), class(p[, 1]))
})

test_that("handles atomic vectors", {
  x <- matrix(TRUE, nrow = 2)
  out <- as_tibble(x, .name_repair = "minimal")
  expect_equal(out[[1]], c(TRUE, TRUE))

  x <- matrix(1L, nrow = 2)
  out <- as_tibble(x, .name_repair = "minimal")
  expect_equal(out[[1]], c(1L, 1L))

  x <- matrix(1.5, nrow = 2)
  out <- as_tibble(x, .name_repair = "minimal")
  expect_equal(out[[1]], c(1.5, 1.5))

  x <- matrix("a", nrow = 2)
  out <- as_tibble(x, .name_repair = "minimal")
  expect_equal(out[[1]], c("a", "a"))

  x <- matrix(complex(real = 1, imaginary = 2), nrow = 2)
  out <- as_tibble(x, .name_repair = "minimal")
  expect_equal(out[[1]], as.vector(x))
})

test_that("forwarding to as.data.frame() for ts objects (#184)", {
  mts <- cbind(
    A = ts(c(1, 1, 2, 2), start = 2016, frequency = 4),
    B = ts(c(11, 11, 12, 13), start = 2016, frequency = 4)
  )
  expect_identical(as_tibble(mts), as_tibble(as.data.frame(mts)))
})


test_that("converting from matrix removes row names by default", {
  x <- matrix(1:30, 6, 5, dimnames = list(letters[1:6], LETTERS[1:5]))
  df <- data.frame(A = 1:6, B = 7:12, C = 13:18, D = 19:24, E = 25:30)
  out <- as_tibble(x)
  expect_false(has_rownames(out))
  expect_identical(out, as_tibble(df))
})

test_that("converting from matrix keeps row names if argument has them, with rownames = NA", {
  x <- matrix(1:30, 6, 5, dimnames = list(letters[1:6], LETTERS[1:5]))
  df <- data.frame(
    A = 1:6, B = 7:12, C = 13:18, D = 19:24, E = 25:30,
    row.names = letters[1:6]
  )

  out <- as_tibble(x, rownames = NA)
  expect_identical(rownames(out), rownames(x))
  expect_identical(remove_rownames(out), as_tibble(df))
})

test_that("converting from matrix supports storing row names in a column", {
  x <- matrix(1:30, 6, 5, dimnames = list(letters[1:6], LETTERS[1:5]))
  df <- tibble(id = letters[1:6], A = 1:6, B = 7:12, C = 13:18, D = 19:24, E = 25:30)
  out <- as_tibble(x, rownames = "id")
  expect_identical(out, df)
})

test_that("converting from matrix uses implicit row names when `rownames =` is passed", {
  x <- matrix(1:30, 6, 5)
  y <- as_tibble(x, rownames = "id", .name_repair = "minimal")
  z <- new_tibble(
    list(
      id = c("1", "2", "3", "4", "5", "6"),
      c(1L, 2L, 3L, 4L, 5L, 6L),
      c(7L, 8L, 9L, 10L, 11L, 12L),
      c(13L, 14L, 15L, 16L, 17L, 18L),
      c(19L, 20L, 21L, 22L, 23L, 24L),
      c(25L, 26L, 27L, 28L, 29L, 30L)
    ),
    nrow = 6
  )
  expect_equal(y, z)
})

test_that("output test", {
  expect_snapshot(error = TRUE, {
    as_tibble(list(1))
    as_tibble(list(1, 2))
    as_tibble(list(a = 1, 2))
    as_tibble(as.list(1:26))
    as_tibble(set_names(list(1), "..1"))
    as_tibble(set_names(as.list(1:26), paste0("..", 1:26)))
    as_tibble(list(a = 1, a = 1))
    as_tibble(list(a = 1, a = 1, b = 1, b = 1))
    as_tibble(list(a = new_environment()))

    as_tibble_row(list(1))
    as_tibble_row(list(1, 2))
    as_tibble_row(list(a = 1, 2))
    as_tibble_row(as.list(1:26))
    as_tibble_row(set_names(list(1), "..1"))
    as_tibble_row(set_names(as.list(1:26), paste0("..", 1:26)))
    as_tibble_row(list(a = 1, a = 1))
    as_tibble_row(list(a = 1, a = 1, b = 1, b = 1))
    as_tibble_row(list(a = new_environment()))

    as_tibble_row(list(a = 1:3))
    as_tibble_row(list(a = 1:3, b = 1:3))
  })
})

#' Tools for working with row names
#'
#' @description
#' While a tibble can have row names (e.g., when converting from a regular data
#' frame), they are removed when subsetting with the `[` operator.
#' A warning will be raised when attempting to assign non-`NULL` row names
#' to a tibble.
#' Generally, it is best to avoid row names, because they are basically a
#' character column with different semantics than every other column.
#'
#' These functions allow to you detect if a data frame has row names
#' (`has_rownames()`), remove them (`remove_rownames()`), or convert
#' them back-and-forth between an explicit column (`rownames_to_column()`
#' and `column_to_rownames()`).
#' Also included is `rowid_to_column()`, which adds a column at the start of the
#' dataframe of ascending sequential row ids starting at 1. Note that this will
#' remove any existing row names.
#'
#' @return `column_to_rownames()` always returns a data frame.
#'   `has_rownames()` returns a scalar logical.
#'   All other functions return an object of the same class as the input.
#'
#' @param .data A data frame.
#' @param var Name of column to use for rownames.
#' @examples
#' # Detect row names ----------------------------------------------------
#' has_rownames(mtcars)
#' has_rownames(trees)
#'
#' # Remove row names ----------------------------------------------------
#' remove_rownames(mtcars) %>% has_rownames()
#'
#' # Convert between row names and column --------------------------------
#' mtcars_tbl <- rownames_to_column(mtcars, var = "car") %>% as_tibble()
#' mtcars_tbl
#' column_to_rownames(mtcars_tbl, var = "car") %>% head()
#'
#' # Adding rowid as a column --------------------------------------------
#' rowid_to_column(trees) %>% head()
#'
#' @name rownames
NULL


#' @export
#' @rdname rownames
has_rownames <- function(.data) {
  .row_names_info(.data) > 0L && !is.na(.row_names_info(.data, 0L)[[1L]])
}

#' @export
#' @rdname rownames
remove_rownames <- function(.data) {
  stopifnot(is.data.frame(.data))
  rownames(.data) <- NULL
  .data
}

#' @export
#' @rdname rownames
rownames_to_column <- function(.data, var = "rowname") {
  # rename, because .data has special semantics in tidy evaluation
  df <- .data

  stopifnot(is.data.frame(df))

  # Side effect: check unique names
  repaired_names(c(unique(names2(df)), var), repair_hint = FALSE)

  new_df <- add_column(df, !!var := rownames(df), .before = 1)
  remove_rownames(new_df)
}

#' @export
#' @rdname rownames
rowid_to_column <- function(.data, var = "rowid") {
  # rename, because .data has special semantics in tidy evaluation
  df <- .data

  stopifnot(is.data.frame(df))

  # Side effect: check unique names
  repaired_names(c(unique(names2(df)), var), repair_hint = FALSE)

  new_df <- add_column(df, !!var := seq_len(nrow(df)), .before = 1)
  remove_rownames(new_df)
}

#' @rdname rownames
#' @export
column_to_rownames <- function(.data, var = "rowname") {
  stopifnot(is.data.frame(.data))

  if (has_rownames(.data)) {
    cnd_signal(error_already_has_rownames())
  }

  if (!has_name(.data, var)) {
    cnd_signal(error_unknown_column_names(var))
  }

  .data <- as.data.frame(.data)
  rownames(.data) <- .data[[var]]
  .data[[var]] <- NULL
  .data
}

#' @export
`row.names<-.tbl_df` <- function(x, value) {
  if (!is.null(value)) {
    warn("Setting row names on a tibble is deprecated.")
  }
  NextMethod()
}

raw_rownames <- function(x) {
  .row_names_info(x, 0L) %||% .set_row_names(.row_names_info(x, 2L))
}

# Errors ------------------------------------------------------------------

error_already_has_rownames <- function() {
  tibble_error("`.data` must be a data frame without row names.")
}

vectbl_names2 <- function(
  x,
  .name_repair = c(
    "check_unique",
    "unique",
    "universal",
    "minimal",
    "unique_quiet",
    "universal_quiet"
  ),
  quiet = FALSE,
  call = caller_env()
) {
  name <- vec_names2(x, repair = "minimal", quiet = quiet)
  repaired_names(
    name,
    repair_hint = TRUE,
    .name_repair = .name_repair,
    quiet = quiet,
    call = call
  )
}

set_repaired_names <- function(
  x,
  repair_hint,
  .name_repair = c(
    "check_unique",
    "unique",
    "universal",
    "minimal",
    "unique_quiet",
    "universal_quiet"
  ),
  quiet = FALSE,
  call = caller_env()
) {
  names <- repaired_names(
    names2(x),
    repair_hint,
    .name_repair = .name_repair,
    quiet = quiet,
    call = call
  )
  set_names(x, names)
}

repaired_names <- function(
  name,
  repair_hint,
  .name_repair = c(
    "check_unique",
    "unique",
    "universal",
    "minimal",
    "unique_quiet",
    "universal_quiet"
  ),
  quiet = FALSE,
  details = NULL,
  call = caller_env()
) {
  subclass_name_repair_errors(
    name = name,
    details = details,
    repair_hint = repair_hint,
    vec_as_names(
      name,
      repair = .name_repair,
      quiet = quiet || !is_character(.name_repair)
    ),
    call = call
  )
}

# Errors ------------------------------------------------------------------

abort_column_names_cannot_be_empty <- function(
  names,
  repair_hint,
  details = NULL,
  parent = NULL,
  call = caller_env()
) {
  tibble_abort(
    invalid_df("must be named", names, use_repair(repair_hint)),
    names = names,
    parent = parent,
    call = call
  )
}

abort_column_names_cannot_be_dot_dot <- function(
  names,
  repair_hint,
  parent = NULL,
  call = caller_env()
) {
  tibble_abort(
    invalid_df(
      "must not have names of the form ... or ..j",
      names,
      use_repair(repair_hint)
    ),
    names = names,
    parent = parent,
    call = call
  )
}

abort_column_names_must_be_unique <- function(
  names,
  repair_hint,
  parent = NULL,
  call = caller_env()
) {
  tibble_abort(
    invalid_df(
      "must not be duplicated",
      names,
      use_repair(repair_hint),
      message = "Column name(s)"
    ),
    names = names,
    parent = parent,
    call = call
  )
}

# Subclassing errors ------------------------------------------------------

subclass_name_repair_errors <- function(
  expr,
  name,
  details = NULL,
  repair_hint = FALSE,
  call
) {
  withCallingHandlers(
    expr,

    # FIXME: use cnd$names with vctrs >= 0.3.0
    vctrs_error_names_cannot_be_empty = function(cnd) {
      abort_column_names_cannot_be_empty(
        detect_empty_names(name),
        details = details,
        parent = cnd,
        repair_hint = repair_hint,
        call = call
      )
    },
    vctrs_error_names_cannot_be_dot_dot = function(cnd) {
      abort_column_names_cannot_be_dot_dot(
        detect_dot_dot(name),
        parent = cnd,
        repair_hint = repair_hint,
        call = call
      )
    },
    vctrs_error_names_must_be_unique = function(cnd) {
      abort_column_names_must_be_unique(
        detect_duplicates(name),
        parent = cnd,
        repair_hint = repair_hint,
        call = call
      )
    }
  )
}

# Anticipate vctrs 0.3.0 release: locations replaced by names
detect_empty_names <- function(names) {
  which(names == "")
}
detect_dot_dot <- function(names) {
  grep("^[.][.](?:[.]|[1-9][0-9]*)$", names)
}
detect_duplicates <- function(names) {
  names[which(duplicated(names))]
}

test_that("set_repaired_names()", {
  x <- set_names(1:3, letters[1:3])
  expect_equal(set_repaired_names(x), x)
  expect_tibble_abort(set_repaired_names(1, repair_hint = FALSE), abort_column_names_cannot_be_empty(1, repair_hint = FALSE))
})

test_that("repaired_names()", {
  expect_equal(repaired_names(letters[1:3], repair_hint = FALSE), letters[1:3])
  expect_tibble_abort(repaired_names(c(""), repair_hint = FALSE), abort_column_names_cannot_be_empty(1, repair_hint = FALSE))
  expect_tibble_abort(repaired_names(c("..1"), repair_hint = FALSE), abort_column_names_cannot_be_dot_dot(1, repair_hint = FALSE))
  expect_tibble_abort(repaired_names(c("a", "a"), repair_hint = FALSE), abort_column_names_must_be_unique("a", repair_hint = FALSE))
  expect_equal(repaired_names(c("a", "a"), .name_repair = "minimal"), c("a", "a"))
})

test_that("output test", {
  skip_if_not_installed("vctrs", "0.3.8.9001")

  expect_snapshot(error = TRUE, {
    repaired_names(letters[1:3], repair_hint = FALSE)
    repaired_names("", repair_hint = FALSE)
    repaired_names("", repair_hint = TRUE)
    repaired_names(c("a", "a"), repair_hint = FALSE)
    repaired_names("..1", repair_hint = FALSE)
    repaired_names(c("a", "a"), repair_hint = FALSE, .name_repair = "universal")
    repaired_names(c("a", "a"), repair_hint = FALSE, .name_repair = "universal", quiet = TRUE)
    repaired_names(c("if"), repair_hint = FALSE, .name_repair = "universal")
  })
})

#' Converting vectors to data frames, and vice versa
#'
#' @description
#' `enframe()` converts named atomic vectors or lists to one- or two-column
#' data frames.
#' For a list, the result will be a nested tibble with a column of type `list`.
#' For unnamed vectors, the natural sequence is used as name column.
#'
#' @param x A vector (for `enframe()`) or a data frame with one or two columns
#'   (for `deframe()`).
#' @param name,value Names of the columns that store the names and values.
#'   If `name` is `NULL`, a one-column tibble is returned; `value` cannot be `NULL`.
#'
#' @return For `enframe()`, a [tibble] with two columns (if `name` is not `NULL`, the default)
#'   or one column (otherwise).
#' @export
#'
#' @examples
#' enframe(1:3)
#' enframe(c(a = 5, b = 7))
#' enframe(list(one = 1, two = 2:3, three = 4:6))
enframe <- function(x, name = "name", value = "value") {
  if (is.null(value)) {
    cnd_signal(error_enframe_value_null())
  }

  if (is.null(x)) {
    x <- logical()
  }

  # FIXME: Enable again for data frames, add test
  if (!vec_is(x) || is.data.frame(x)) {
    cnd_signal(error_enframe_must_be_vector(x))
  }

  if (is.null(name)) {
    df <- list(vectbl_set_names(x))
  } else if (is.null(vec_names(x))) {
    df <- list(seq_len(vec_size(x)), x)
  } else {
    df <- list(vec_names2(x), vectbl_set_names(x))
  }

  names(df) <- c(name, value)
  new_tibble(df, nrow = vec_size(x))
}

vectbl_set_names <- function(x, names = NULL) {
  # Work around https://github.com/r-lib/vctrs/issues/1419
  if (inherits(x, "vctrs_rcrd")) {
    # A rcrd can't have names?
    return(x)
  }
  vec_set_names(x, names)
}

#' @rdname enframe
#' @description
#' `deframe()` converts two-column data frames to a named vector or list,
#' using the first column as name and the second column as value.
#' If the input has only one column, an unnamed vector is returned.
#' @return For `deframe()`, a vector (named or unnamed).
#' @export
#' @examples
#' deframe(enframe(3:1))
#' deframe(tibble(a = 1:3))
#' deframe(tibble(a = as.list(1:3)))
deframe <- function(x) {
  if (length(x) == 1) {
    return(x[[1]])
  } else if (length(x) != 2) {
    warn("`x` must be a one- or two-column data frame in `deframe()`.")
  }

  value <- x[[2L]]
  name <- x[[1L]]
  vectbl_set_names(value, as.character(name))
}

error_enframe_value_null <- function() {
  tibble_error("`value` can't be NULL.")
}

error_enframe_must_be_vector <- function(x) {
  tibble_error(paste0(
    "The `x` argument to `enframe()` must be a vector, not ", class(x)[[1]], "."
  ))
}

# nocov start - compat-lazyeval (last updated: rlang 0.3.0)

# This file serves as a reference for compatibility functions for lazyeval.
# Please find the most recent version in rlang's repository.


warn_underscored <- function() {
  return(NULL)
  warn(paste(
    "The underscored versions are deprecated in favour of",
    "tidy evaluation idioms. Please see the documentation",
    "for `quo()` in rlang"
  ))
}
warn_text_se <- function() {
  return(NULL)
  warn("Text parsing is deprecated, please supply an expression or formula")
}

compat_lazy <- function(lazy, env = caller_env(), warn = TRUE) {
  if (warn) warn_underscored()

  if (missing(lazy)) {
    return(quo())
  }
  if (is_quosure(lazy)) {
    return(lazy)
  }
  if (is_formula(lazy)) {
    return(as_quosure(lazy, env))
  }

  out <- switch(typeof(lazy),
    symbol = ,
    language = new_quosure(lazy, env),
    character = {
      if (warn) warn_text_se()
      parse_quo(lazy[[1]], env)
    },
    logical = ,
    integer = ,
    double = {
      if (length(lazy) > 1) {
        warn("Truncating vector to length 1")
        lazy <- lazy[[1]]
      }
      new_quosure(lazy, env)
    },
    list =
      if (inherits(lazy, "lazy")) {
        lazy = new_quosure(lazy$expr, lazy$env)
      }
  )

  if (is.null(out)) {
    cnd_signal(sprintf("Can't convert a %s to a quosure", typeof(lazy)))
  } else {
    out
  }
}

compat_lazy_dots <- function(dots, env, ..., .named = FALSE) {
  if (missing(dots)) {
    dots <- list()
  }
  if (inherits(dots, c("lazy", "formula"))) {
    dots <- list(dots)
  } else {
    dots <- unclass(dots)
  }
  dots <- c(dots, list(...))

  warn <- TRUE
  for (i in seq_along(dots)) {
    dots[[i]] <- compat_lazy(dots[[i]], env, warn)
    warn <- FALSE
  }

  named <- have_name(dots)
  if (.named && any(!named)) {
    nms <- vapply(dots[!named], function(x) expr_text(get_expr(x)), character(1))
    names(dots)[!named] <- nms
  }

  names(dots) <- names2(dots)
  dots
}

compat_as_lazy <- function(quo) {
  structure(class = "lazy", list(
    expr = get_expr(quo),
    env = get_env(quo)
  ))
}
compat_as_lazy_dots <- function(...) {
  structure(class = "lazy_dots", lapply(quos(...), compat_as_lazy))
}


# nocov end

#' Deprecated functions
#'
#' @description
#' `r lifecycle::badge("deprecated")`
#'
#' Use [tibble()] instead of `data_frame()`.
#'
#' @export
#' @keywords internal
#' @name deprecated
data_frame <- function(...) {
  deprecate_warn("1.1.0", "data_frame()", "tibble()")

  # Unquote-splice to avoid argument matching
  tibble(!!!quos(...))
}

#' @description
#' Use [quasiquotation] instead of `tibble_()`, `data_frame_()`, and `lst_()`.
#'
#' @export
#' @keywords internal
#' @rdname deprecated
tibble_ <- function(xs) {
  deprecate_soft("2.0.0", "tibble_()", "tibble()",
    details = '`tibble()` supports dynamic dots, see `?"dyn-dots"`.'
  )

  xs <- compat_lazy_dots(xs, caller_env())
  tibble(!!!xs)
}

#' @export
#' @rdname deprecated
data_frame_ <- function(xs) {
  deprecate_soft("2.0.0", "data_frame_()", "tibble()",
    details = '`tibble()` supports dynamic dots, see `?"dyn-dots"`.'
  )

  xs <- compat_lazy_dots(xs, caller_env())
  tibble(!!!xs)
}

#' @export
#' @rdname deprecated
lst_ <- function(xs) {
  deprecate_soft("2.0.0", "lst_()", "lst()",
    details = '`lst()` supports dynamic dots, see `?"dyn-dots"`.'
  )

  xs <- compat_lazy_dots(xs, caller_env())
  lst(!!!xs)
}

#' @description
#' Use [as_tibble()] instead of `as_data_frame()` or `as.tibble()`, but mind the
#' new signature and semantics.
#'
#' @export
#' @rdname deprecated
as_data_frame <- function(x, ...) {
  deprecate_warn("2.0.0", "as_data_frame()", "as_tibble()",
    details = "The signature and semantics have changed, see `?as_tibble`."
  )

  as_tibble(x, ...)
}

#' @export
#' @rdname deprecated
as.tibble <- function(x, ...) {
  deprecate_warn("2.0.0", "as.tibble()", "as_tibble()",
    details = "The signature and semantics have changed, see `?as_tibble`."
  )

  as_tibble(x, ...)
}

#' @description
#' Use [tribble()] instead of `frame_data()`.
#' @export
#' @rdname deprecated
frame_data <- function(...) {
  deprecate_soft("2.0.0", "frame_data()", "tribble()")

  tribble(...)
}

#' Name repair
#'
#' Please review [vctrs::vec_as_names()].
#'
#' @name name-repair
#' @keywords internal
NULL

# For RSDA package
tbl_subset_col <- function(x, j, j_arg) {
  if (is.null(j)) return(x)

  j <- vectbl_as_col_location(j, length(x), names(x), j_arg = j_arg, assign = FALSE)

  if (anyNA(j)) {
    cnd_signal(error_na_column_index(which(is.na(j))))
  }

  xo <- .subset(x, j)
  if (anyDuplicated(j)) {
    xo <- set_repaired_names(xo, repair_hint = FALSE, .name_repair = "minimal")
  }
  set_tibble_class(xo, nrow = fast_nrow(x))
}

pluralise_msg <- function(message, objects) {
  paste0(
    pluralise(message, objects),
    format_n(objects)
  )
}

pluralise_commas <- function(message, objects, ...) {
  paste0(
    pluralise_n(message, length(objects)),
    commas(objects),
    pluralise_n(paste0(...), length(objects))
  )
}

pluralise_count <- function(message, count, ...) {
  paste0(
    pluralise_n(message, count),
    count,
    pluralise_n(paste0(...), count)
  )
}

pluralise <- function(message, objects) {
  pluralise_n(message, length(objects))
}

pluralise_n <- function(message, n) {
  stopifnot(n >= 0)


  # Don't strip parens if they have a space in between
  # (but not if the space comes before the closing paren)

  if (n == 1) {
    # strip [
    message <- gsub("\\[([^\\] ]* *)\\]", "\\1", message, perl = TRUE)
    # remove ( and its content
    message <- gsub("\\([^\\) ]* *\\)", "", message, perl = TRUE)
  } else {
    # strip (
    message <- gsub("\\(([^\\) ]* *)\\)", "\\1", message, perl = TRUE)
    # remove [ and its content
    message <- gsub("\\[[^\\] ]* *\\]", "", message, perl = TRUE)
  }

  message
}

bullets <- function(header, ..., info = NULL) {
  # FIXME: Convert info to i with rlang >= 0.4.12, use set_default_name(bullets, "*")
  # FIXME: Avoid ensure_full_stop()
  bullets <- vec_c(..., .name_spec = "{outer}")

  paste0(
    ensure_full_stop(header), "\n",
    format_error_bullets(ensure_full_stop(bullets)),
    if (!is.null(info)) paste0("\n", format_error_bullets(c(i = info)))
  )
}

problems <- function(header, ..., .problem = " problem(s)") {
  problems <- vec_c(..., .name_spec = "{outer}")
  problems <- set_default_name(problems, "x")
  MAX_BULLETS <- 6L
  if (length(problems) >= MAX_BULLETS) {
    n_more <- length(problems) - MAX_BULLETS + 1L
    problems[[MAX_BULLETS]] <-
      pluralise_n(paste0(pre_dots("and "), n_more, " more", .problem), n_more)
    length(problems) <- MAX_BULLETS
  }

  bullets(header, problems)
}

commas <- function(problems) {
  MAX_BULLETS <- 6L

  n <- length(problems)
  if (n <= 1) {
    return(problems)
  } else if (n == 2) {
    return(paste(problems, collapse = " and "))
  }

  if (n >= MAX_BULLETS) {
    n_more <- length(problems) - MAX_BULLETS + 1L
    problems[[MAX_BULLETS]] <- paste0(n_more, " more")
    length(problems) <- MAX_BULLETS
    n <- MAX_BULLETS
  }

  problems[[n]] <- paste0("and ", problems[[n]])

  paste(problems, collapse = ", ")
}

ensure_full_stop <- function(x) {
  set_names(gsub("(?::|([^.?]))$", "\\1.", x), names(x))
}

set_default_name <- function(x, name) {
  if (is.null(names(x))) {
    names(x) <- rep_along(x, name)
  } else {
    names(x)[names(x) == ""] <- name
  }

  x
}

#' @export
str.tbl_df <- function(object, ..., indent.str = " ", nest.lev = 0) {
  if (nest.lev != 0L) cat(" ")
  cat(
    tibble::obj_sum(object),
    " (S3: ", paste0(class(object), collapse = "/"), ")",
    "\n",
    sep = ""
  )

  utils::str(
    as.list(object),
    no.list = TRUE,
    ...,
    nest.lev = nest.lev + 1L,
    indent.str = indent.str
  )
}

tbl_subset_matrix <- function(x, j, j_arg) {
  cells <- matrix_to_cells(j, x, j_arg)
  col_idx <- cells_to_col_idx(cells)

  if (is_empty(col_idx)) {
    return(unspecified())
  }

  values <- map2(x[col_idx], cells[col_idx], vec_slice)

  # Also checks conformity of vectors:
  unname(vec_c(!!!values, .name_spec = ~.x))
}

tbl_subassign_matrix <- function(x, j, value, j_arg, value_arg) {
  # FIXME: use size argument in vctrs >= 0.3.0

  if (!vec_is(value)) {
    cnd_signal(error_subset_matrix_scalar_type(j_arg, value_arg))
  }

  if (vec_size(value) != 1) {
    cnd_signal(error_subset_matrix_must_be_scalar(j_arg, value_arg))
  }

  cells <- matrix_to_cells(j, x, j_arg)
  col_idx <- cells_to_col_idx(cells)

  withCallingHandlers(
    for (j in col_idx) {
      x[[j]] <- vectbl_assign(x[[j]], cells[[j]], value)
    },
    vctrs_error_incompatible_type = function(cnd) {
      cnd_signal(error_assign_incompatible_type(x, rep(list(value), j), j, value_arg, cnd_message(cnd)))
    }
  )

  x
}

matrix_to_cells <- function(j, x, j_arg) {
  if (!is_bare_logical(j)) {
    cnd_signal(error_subset_matrix_must_be_logical(j_arg))
  }
  if (!identical(dim(j), dim(x))) {
    cnd_signal(error_subset_matrix_must_have_same_dimensions(j_arg))
  }

  # Need unlist(list(...)) because apply() isn't type stable if the return
  # has the same length everywhere
  # FIXME: Faster with a C implementation?
  cells <- unlist(apply(j, 2, function(x) list(which(x))), recursive = FALSE)
  cells
}

cells_to_col_idx <- function(cells) {
  sizes <- map_int(cells, vec_size)
  col_idx <- which(sizes > 0)

  col_idx
}

# Errors ------------------------------------------------------------------

error_subset_matrix_must_be_logical <- function(j_arg) {
  tibble_error(paste0(
    "Subscript ", tick(as_label(j_arg)),
    " is a matrix, it must be of type logical."
  ))
}

error_subset_matrix_must_have_same_dimensions <- function(j_arg) {
  tibble_error(paste0(
    "Subscript ", tick(as_label(j_arg)),
    " is a matrix, it must have the same dimensions as the input."
  ))
}

error_subset_matrix_scalar_type <- function(j_arg, value_arg) {
  tibble_error(paste0(
    "Subscript ", tick(as_label(j_arg)),
    " is a matrix, the data ", tick(as_label(value_arg)),
    " must be a vector of size 1."
  ))
}

error_subset_matrix_must_be_scalar <- function(j_arg, value_arg) {
  tibble_error(paste0(
    "Subscript ", tick(as_label(j_arg)),
    " is a matrix, the data ", tick(as_label(value_arg)),
    " must have size 1."
  ))
}

# nocov start - https://github.com/tidyverse/tibble/blob/main/R/fansi.R
set_fansi_hooks <- function() {
  knitr::opts_chunk$set(collapse = TRUE)

  if (Sys.getenv("IN_GALLEY") != "") {
    knitr::opts_chunk$set(comment = "#>")
    return()
  }

  knitr::opts_chunk$set(comment = pillar::style_subtle("#>"))

  options(cli.num_colors = 256)
  options(pillar.bold = TRUE)

  knitr::knit_hooks$set(
    output = colourise_chunk("output"),
    message = colourise_chunk("message"),
    warning = colourise_chunk("warning"),
    error = colourise_chunk("error")
  )

  invisible()
}

colourise_chunk <- function(type) {
  # Fallback if fansi is missing
  if (is_installed("fansi")) {
    sgr_to_html <- fansi::sgr_to_html
  } else {
    sgr_to_html <- identity
  }

  function(x, options) {
    # lines <- strsplit(x, "\\n")[[1]]
    lines <- x
    if (type != "output") {
      lines <- crayon::red(lines)
    }
    paste0(
      '<div class="sourceCode"><pre class="sourceCode"><code class="sourceCode">',
      paste0(
        sgr_to_html(htmltools::htmlEscape(lines)),
        collapse = "\n"
      ),
      "</code></pre></div>"
    )
  }
}
# nocov end

vectbl_names2 <- function(x,
                          .name_repair = c("check_unique", "unique", "universal", "minimal"),
                          quiet = FALSE) {
  name <- vec_names2(x, repair = "minimal", quiet = quiet)
  repaired_names(name, repair_hint = TRUE, .name_repair = .name_repair, quiet = quiet)
}

set_repaired_names <- function(x,
                               repair_hint,
                               .name_repair = c("check_unique", "unique", "universal", "minimal"),
                               quiet = FALSE) {
  set_names(x, repaired_names(names2(x), repair_hint, .name_repair = .name_repair, quiet = quiet))
}

repaired_names <- function(name,
                           repair_hint,
                           .name_repair = c("check_unique", "unique", "universal", "minimal"),
                           quiet = FALSE,
                           details = NULL) {
  subclass_name_repair_errors(
    name = name, details = details, repair_hint = repair_hint,
    vec_as_names(name, repair = .name_repair, quiet = quiet || !is_character(.name_repair))
  )
}

# Errors ------------------------------------------------------------------

error_column_names_cannot_be_empty <- function(names, repair_hint, parent = NULL) {
  tibble_error(invalid_df("must be named", names, use_repair(repair_hint)), names = names, parent = parent)
}

error_column_names_cannot_be_dot_dot <- function(names, repair_hint, parent = NULL) {
  tibble_error(invalid_df("must not have names of the form ... or ..j", names, use_repair(repair_hint)), names = names, parent = parent)
}

error_column_names_must_be_unique <- function(names, repair_hint, parent = NULL) {
  tibble_error(pluralise_commas("Column name(s) ", tick(names), " must not be duplicated.", use_repair(repair_hint)), names = names, parent = parent)
}

# Subclassing errors ------------------------------------------------------

subclass_name_repair_errors <- function(expr, name, details = NULL, repair_hint = FALSE) {
  withCallingHandlers(
    expr,

    # FIXME: use cnd$names with vctrs >= 0.3.0
    vctrs_error_names_cannot_be_empty = function(cnd) {
      cnd <- error_column_names_cannot_be_empty(detect_empty_names(name), parent = cnd, repair_hint = repair_hint)
      cnd$body <- details

      cnd_signal(cnd)
    },
    vctrs_error_names_cannot_be_dot_dot = function(cnd) {
      cnd <- error_column_names_cannot_be_dot_dot(detect_dot_dot(name), parent = cnd, repair_hint = repair_hint)
      cnd_signal(cnd)
    },
    vctrs_error_names_must_be_unique = function(cnd) {
      cnd <- error_column_names_must_be_unique(detect_duplicates(name), parent = cnd, repair_hint = repair_hint)
      cnd_signal(cnd)
    }
  )
}

# Anticipate vctrs 0.3.0 release: locations replaced by names
detect_empty_names <- function(names) {
  which(names == "")
}
detect_dot_dot <- function(names) {
  grep("^[.][.](?:[.]|[1-9][0-9]*)$", names)
}
detect_duplicates <- function(names) {
  names[which(duplicated(names))]
}

test_that("set_repaired_names()", {
  x <- set_names(1:3, letters[1:3])
  expect_equal(set_repaired_names(x), x)
  expect_tibble_error(set_repaired_names(1), error_column_names_cannot_be_empty(1))
})

test_that("repaired_names()", {
  expect_equal(repaired_names(letters[1:3]), letters[1:3])
  expect_tibble_error(repaired_names(c("")), error_column_names_cannot_be_empty(1))
  expect_tibble_error(repaired_names(c("..1")), error_column_names_cannot_be_dot_dot(1))
  expect_tibble_error(repaired_names(c("a", "a")), error_column_names_must_be_unique("a"))
  expect_equal(repaired_names(c("a", "a"), .name_repair = "minimal"), c("a", "a"))
})

test_that("output test", {
  expect_snapshot_with_error({
    repaired_names(letters[1:3])
    repaired_names("")
    repaired_names(c("a", "a"))
    repaired_names("..1")
    repaired_names(c("a", "a"), .name_repair = "universal")
    repaired_names(c("a", "a"), .name_repair = "universal", quiet = TRUE)
    repaired_names(c("if"), .name_repair = "universal")
  })
})

#' Coerce lists, matrices, and more to data frames
#'
#' @description
#' `as_tibble()` turns an existing object, such as a data frame or
#' matrix, into a so-called tibble, a data frame with class [`tbl_df`]. This is
#' in contrast with [tibble()], which builds a tibble from individual columns.
#' `as_tibble()` is to [`tibble()`] as [base::as.data.frame()] is to
#' [base::data.frame()].
#'
#' `as_tibble()` is an S3 generic, with methods for:
#' * [`data.frame`][base::data.frame()]: Thin wrapper around the `list` method
#'   that implements tibble's treatment of [rownames].
#' * [`matrix`][base::matrix()], [`poly`][stats::poly()],
#'   [`ts`][stats::ts()], [`table`][base::table()]
#' * Default: Other inputs are first coerced with [base::as.data.frame()].
#'
#' @section Row names:
#' The default behavior is to silently remove row names.
#'
#' New code should explicitly convert row names to a new column using the
#' `rownames` argument.
#'
#' For existing code that relies on the retention of row names, call
#' `pkgconfig::set_config("tibble::rownames" = NA)` in your script or in your
#' package's [.onLoad()]  function.
#'
#' @section Life cycle:
#' Using `as_tibble()` for vectors is superseded as of version 3.0.0,
#' prefer the more expressive `as_tibble_row()` and
#' `as_tibble_col()` variants for new code.
#'
#' @seealso [tibble()] constructs a tibble from individual columns. [enframe()]
#'   converts a named vector to a tibble with a column of names and column of
#'   values. Name repair is implemented using [vctrs::vec_as_names()].
#'
#' @param x A data frame, list, matrix, or other object that could reasonably be
#'   coerced to a tibble.
#' @param ... Unused, for extensibility.
#' @inheritParams tibble
#' @param rownames How to treat existing row names of a data frame or matrix:
#'   * `NULL`: remove row names. This is the default.
#'   * `NA`: keep row names.
#'   * A string: the name of a new column. Existing rownames are transferred
#'     into this column and the `row.names` attribute is deleted.
#'  Read more in [rownames].

#' @param _n,validate
#'   `r lifecycle::badge("soft-deprecated")`
#'
#'   For compatibility only, do not use for new code.
#' @export
#' @examples
#' m <- matrix(rnorm(50), ncol = 5)
#' colnames(m) <- c("a", "b", "c", "d", "e")
#' df <- as_tibble(m)
as_tibble <- function(x, ...,
                      .rows = NULL,
                      .name_repair = c("check_unique", "unique", "universal", "minimal"),
                      rownames = pkgconfig::get_config("tibble::rownames", NULL)) {
  UseMethod("as_tibble")
}

#' @export
#' @rdname as_tibble
as_tibble.data.frame <- function(x, validate = NULL, ...,
                                 .rows = NULL,
                                 .name_repair = c("check_unique", "unique", "universal", "minimal"),
                                 rownames = pkgconfig::get_config("tibble::rownames", NULL)) {
  .name_repair <- compat_name_repair(.name_repair, validate, missing(.name_repair))

  old_rownames <- raw_rownames(x)
  if (is.null(.rows)) {
    .rows <- nrow(x)
  }

  result <- lst_to_tibble(unclass(x), .rows, .name_repair)

  if (is.null(rownames)) {
    result
  } else if (is.na(rownames)) {
    attr(result, "row.names") <- old_rownames
    result
  } else {
    if (length(old_rownames) > 0 && is.na(old_rownames[1L])) { # if implicit rownames
      old_rownames <- seq_len(abs(old_rownames[2L]))
    }
    old_rownames <- as.character(old_rownames)
    add_column(result, !!rownames := old_rownames, .before = 1L, .name_repair = "minimal")
  }
}

#' @export
#' @rdname as_tibble
as_tibble.list <- function(x, validate = NULL, ..., .rows = NULL,
                           .name_repair = c("check_unique", "unique", "universal", "minimal")) {
  .name_repair <- compat_name_repair(.name_repair, validate, missing(.name_repair))

  lst_to_tibble(x, .rows, .name_repair, col_lengths(x))
}

lst_to_tibble <- function(x, .rows, .name_repair, lengths = NULL) {
  x <- unclass(x)
  x <- set_repaired_names(x, repair_hint = TRUE, .name_repair)
  x <- check_valid_cols(x)
  recycle_columns(x, .rows, lengths)
}

compat_name_repair <- function(.name_repair, validate, .missing_name_repair) {
  if (is.null(validate)) return(.name_repair)


  if (!.missing_name_repair) {
    name_repair <- .name_repair
  } else if (isTRUE(validate)) {
    name_repair <- "check_unique"
  } else {
    name_repair <- "minimal"
  }

  deprecate_soft("2.0.0", "tibble::as_tibble(validate = )", "as_tibble(.name_repair =)",
    env = foreign_caller_env()
  )

  name_repair
}

check_valid_cols <- function(x, pos = NULL) {
  names_x <- names2(x)

  is_xd <- which(!map_lgl(x, is_valid_col))
  if (has_length(is_xd)) {
    classes <- map_chr(x[is_xd], friendly_type_of)
    cnd_signal(error_column_scalar_type(names_x[is_xd], pos[is_xd], classes))
  }

  # 657
  x[] <- map(x, make_valid_col)
  invisible(x)
}

make_valid_col <- function(x) {
  if (is.expression(x)) {
    x <- as.list(x)
  }
  x
}

is_valid_col <- function(x) {
  # 657
  vec_is(x) || is.expression(x)
}

recycle_columns <- function(x, .rows, lengths) {
  nrow <- guess_nrow(lengths, .rows)

  # Shortcut if all columns have the requested or implied length
  different_len <- which(lengths != nrow)
  if (is_empty(different_len)) return(new_tibble(x, nrow = nrow, subclass = NULL))

  if (any(lengths[different_len] != 1)) {
    cnd_signal(error_incompatible_size(.rows, names(x), lengths, "Requested with `.rows` argument"))
  }

  if (nrow != 1L) {
    short <- which(lengths == 1L)
    if (has_length(short)) {
      x[short] <- map(x[short], vec_recycle, nrow)
    }
  }

  new_tibble(x, nrow = nrow, subclass = NULL)
}

guess_nrow <- function(lengths, .rows) {
  if (!is.null(.rows)) {
    return(.rows)
  }
  if (is_empty(lengths)) {
    return(0)
  }

  nontrivial_lengths <- lengths[lengths != 1L]
  if (is_empty(nontrivial_lengths)) {
    return(1)
  }

  max(nontrivial_lengths)
}

#' @export
#' @rdname as_tibble
as_tibble.matrix <- function(x, ..., validate = NULL, .name_repair = NULL) {
  m <- matrixToDataFrame(x)
  names <- colnames(x)
  if (is.null(.name_repair)) {
    if ((is.null(names) || any(bad_names <- duplicated(names) | names == "")) && has_length(x)) {
      deprecate_warn("2.0.0", "as_tibble.matrix(x = 'must have unique column names if `.name_repair` is omitted')",
        details = "Using compatibility `.name_repair`."
      )
      compat_names <- paste0("V", seq_along(m))
      if (is.null(names)) {
        names <- compat_names
      } else {
        names[bad_names] <- compat_names[bad_names]
      }
      .name_repair <- function(x) names
    } else {
      .name_repair <- "check_unique"
    }
    validate <- NULL
  }

  colnames(m) <- names
  as_tibble(m, ..., validate = validate, .name_repair = .name_repair)
}

#' @export
as_tibble.poly <- function(x, ...) {
  m <- matrixToDataFrame(unclass(x))
  colnames(m) <- colnames(x)
  as_tibble(m, ...)
}

#' @export
as_tibble.ts <- function(x, ..., .name_repair = "minimal") {
  df <- as.data.frame(x)
  if (length(dim(x)) == 2) {
    colnames(df) <- colnames(x)
  }
  as_tibble(df, ..., .name_repair = .name_repair)
}

#' @export
#' @param n Name for count column, default: `"n"`.
#' @rdname as_tibble
as_tibble.table <- function(x, `_n` = "n", ..., n = `_n`, .name_repair = "check_unique") {
  if (!missing(`_n`)) {
    warn("Please pass `n` as a named argument to `as_tibble.table()`.")
  }

  df <- as.data.frame(x, stringsAsFactors = FALSE)

  names(df) <- repaired_names(
    c(names2(dimnames(x)), n),
    repair_hint = TRUE, .name_repair = .name_repair
  )

  # Names already repaired:
  as_tibble(df, ..., .name_repair = "minimal")
}

#' @export
#' @rdname as_tibble
as_tibble.NULL <- function(x, ...) {
  if (missing(x)) {
    deprecate_soft("3.0.0", "as_tibble(x = 'can\\'t be missing')")
  }

  new_tibble(list(), nrow = 0)
}

#' @export
#' @rdname as_tibble
as_tibble.default <- function(x, ...) {
  value <- x
  if (is_atomic(value)) {
    signal_superseded(
      "3.0.0", "as_tibble(x = 'can\\'t be an atomic vector')",
      "as_tibble_col()"
    )
  }
  as_tibble(as.data.frame(value, stringsAsFactors = FALSE), ...)
}

#' @description
#' `as_tibble_row()` converts a vector to a tibble with one row.
#' If the input is a list, all elements must have size one.
#'
#' @rdname as_tibble
#' @export
#' @examples
#'
#' as_tibble_row(c(a = 1, b = 2))
#' as_tibble_row(list(c = "three", d = list(4:5)))
#' as_tibble_row(1:3, .name_repair = "unique")
as_tibble_row <- function(x,
                          .name_repair = c("check_unique", "unique", "universal", "minimal")) {
  if (!vec_is(x)) {
    cnd_signal(error_as_tibble_row_vector(x))
  }

  names <- vectbl_names2(x, .name_repair = .name_repair)

  # FIXME: Use vec_chop2() when https://github.com/r-lib/vctrs/pull/1226 is in
  if (is_bare_list(x)) {
    slices <- x
  } else {
    x <- vectbl_set_names(x, NULL)
    slices <- lapply(seq_len(vec_size(x)), vec_slice, x = x)
    names(slices) <- names
  }

  check_all_lengths_one(slices)

  new_tibble(slices, nrow = 1)
}

check_all_lengths_one <- function(x) {
  sizes <- col_lengths(x)

  bad_lengths <- which(sizes != 1)
  if (!is_empty(bad_lengths)) {
    cnd_signal(error_as_tibble_row_size_one(
      seq_along(x)[bad_lengths],
      names2(x)[bad_lengths],
      sizes[bad_lengths]
    ))
  }
}


#' @description
#' `as_tibble_col()` converts a vector to a tibble with one column.
#'
#' @param column_name Name of the column.
#'
#' @rdname as_tibble
#' @export
#' @examples
#'
#' as_tibble_col(1:3)
#' as_tibble_col(
#'   list(c = "three", d = list(4:5)),
#'   column_name = "data"
#' )
as_tibble_col <- function(x, column_name = "value") {
  # Side effect: checking that x is a vector
  tibble(!!column_name := x)
}

# External ----------------------------------------------------------------

matrixToDataFrame <- function(x) {
  .Call(`tibble_matrixToDataFrame`, x)
}

# Errors ------------------------------------------------------------------

error_column_scalar_type <- function(names, positions, classes) {
  tibble_error(
    problems(
      "All columns in a tibble must be vectors:",
      x = paste0("Column ", name_or_pos(names, positions), " is ", classes)
    ),
    names = names
  )
}

error_as_tibble_row_vector <- function(x) {
  tibble_error(paste0(
    "`x` must be a vector in `as_tibble_row()`, not ", class(x)[[1]], "."
  ))
}

error_as_tibble_row_size_one <- function(j, name, size) {
  desc <- tick(name)
  desc[name == ""] <- paste0("at position ", j[name == ""])

  tibble_error(problems(
    "All elements must be size one, use `list()` to wrap.",
    paste0("Element ", desc, " is of size ", size, ".")
  ))
}

test_that("tibble returns correct number of rows with all combinatinos", {
  expect_equal(nrow(tibble(value = 1:10)), 10L)
  expect_equal(nrow(tibble(value = 1:10, name = "recycle_me")), 10L)
  expect_equal(nrow(tibble(name = "recycle_me", value = 1:10)), 10L)
  expect_equal(nrow(tibble(name = "recycle_me", value = 1:10, value2 = 11:20)), 10L)
  expect_equal(nrow(tibble(value = 1:10, name = "recycle_me", value2 = 11:20)), 10L)
})

test_that("dim attribute is stripped of 1D array (#84)", {
  skip_brk_inner_dim_not_stripped()
  expect_null(dim(tibble(x = array(1:3))$x))
})

test_that("bogus columns raise an error", {
  skip_enh_tibble_null()
  expect_error(
    tibble(a = NULL),
    error_column_scalar_type("a", "NULL"),
    fixed = TRUE
  )
})

test_that("bogus columns raise an error", {
  expect_legacy_error(
    tibble(a = new.env()),
    error_column_scalar_type("a", "environment"),
    fixed = TRUE
  )
  expect_legacy_error(
    tibble(a = quote(a)),
    error_column_scalar_type("a", "name"),
    fixed = TRUE
  )
})

test_that("length 1 vectors are recycled", {
  expect_equal(nrow(tibble(x = 1:10)), 10)
  expect_equal(nrow(tibble(x = 1:10, y = 1)), 10)
  expect_legacy_error(
    tibble(x = 1:10, y = 1:2),
    error_inconsistent_cols(NULL, c("x", "y"), c(10, 2), NA),
    fixed = TRUE
  )
})

test_that("length 1 vectors in hierarchical data frames are recycled (#502)", {
  expect_identical(
    tibble(x = 1:10, y = tibble(z = 1)),
    tibble(x = 1:10, y = tibble(z = rep(1, 10)))
  )
  expect_identical(
    tibble(y = tibble(z = 1), x = 1:10),
    tibble(y = tibble(z = rep(1, 10)), x = 1:10)
  )
  expect_identical(
    tibble(x = 1, y = tibble(z = 1:10)),
    tibble(x = rep(1, 10), y = tibble(z = 1:10))
  )
  expect_identical(
    tibble(y = tibble(z = 1:10), x = 1),
    tibble(y = tibble(z = 1:10), x = rep(1, 10))
  )
})

test_that("missing names are imputed from call", {
  x <- 1:10
  df <- tibble(x, y = x)
  expect_equal(names(df), c("x", "y"))
})

test_that("empty input makes 0 x 0 tbl_df", {
  zero <- tibble()
  expect_s3_class(zero, "tbl_df")
  expect_equal(dim(zero), c(0L, 0L))
  expect_identical(attr(zero, "names"), character(0L))
})

test_that("SE version", {
  scoped_lifecycle_silence()
  expect_identical(tibble_(list(a = ~1:10)), tibble(a = 1:10))
})

test_that("names are stripped from vectors", {
  skip_brk_inner_names_not_stripped()
  foo <- tibble(x = c(y = 1, z = 2))
  expect_equal(names(foo), "x")
  expect_null(names(foo$x))
})

test_that("names in list columns are preserved", {
  foo <- tibble(x = list(y = 1:3, z = 4:5))
  expect_equal(names(foo), "x")
  expect_equal(names(foo$x), c("y", "z"))
})

test_that("attributes are preserved", {
  df <- structure(
    data.frame(x = 1:10, g1 = rep(1:2, each = 5), g2 = rep(1:5, 2)),
    meta = "this is important"
  )
  res <- as_tibble(df)

  expect_identical(attr(res, "meta"), attr(df, "meta"))
})

test_that("tibble aliases", {
  scoped_lifecycle_silence()
  expect_identical(data_frame(a = 1), tibble(a = 1))

  skip_int_data_frame_tibble_diff()
  expect_identical(data_frame_, tibble_)
})


# as_tibble -----------------------------------------------------------

test_that("columns are recycled to common length", {
  expect_identical(
    as_tibble(list(x = 1, y = 1:3)),
    tibble(x = rep(1, 3), y = 1:3)
  )
  expect_identical(
    as_tibble(list(x = 1:3, y = 1)),
    tibble(x = 1:3, y = rep(1, 3))
  )
  expect_identical(
    as_tibble(list(x = character(), y = 1)),
    tibble(x = character(), y = numeric())
  )
})

test_that("columns must be same length", {
  expect_legacy_error(
    as_tibble(list(x = 1:2, y = 1:3)),
    error_inconsistent_cols(NULL,  c("x", "y"), 2:3, NA),
    fixed = TRUE
  )
  expect_legacy_error(
    as_tibble(list(x = 1:2, y = 1:3, z = 1:4)),
    error_inconsistent_cols(
      NULL,
      c("x", "y", "z"),
      2:4,
      NA
    ),
    fixed = TRUE
  )
  expect_legacy_error(
    as_tibble(list(x = 1:4, y = 1:2, z = 1:2)),
    error_inconsistent_cols(
      NULL,
      c("x", "y", "z"),
      c(4, 2, 2),
      NA
    ),
    fixed = TRUE
  )
  expect_legacy_error(
    as_tibble(list(x = 1, y = 1:4, z = 1:2)),
    error_inconsistent_cols(
      NULL,
      c("y", "z"),
      c(4, 2),
      NA
    ),
    fixed = TRUE
  )
  expect_legacy_error(
    as_tibble(list(x = 1:2, y = 1:4, z = 1)),
    error_inconsistent_cols(
      NULL,
      c("x", "y"),
      c(2, 4),
      NA
    ),
    fixed = TRUE
  )
})

test_that("empty list() makes 0 x 0 tbl_df", {
  zero <- as_tibble(list())
  expect_s3_class(zero, "tbl_df")
  expect_equal(dim(zero), c(0L, 0L))
})


test_that("NULL makes 0 x 0 tbl_df", {
  nnnull <- as_tibble(NULL)
  expect_s3_class(nnnull, "tbl_df")
  expect_equal(dim(nnnull), c(0L, 0L))
})


test_that("as_tibble.tbl_df() leaves classes unchanged (#60)", {
  df <- tibble()
  expect_equal(
    class(df),
    c("tbl_df", "tbl", "data.frame")
  )
  expect_equal(
    class(structure(df, class = c("my_df", class(df)))),
    c("my_df", "tbl_df", "tbl", "data.frame")
  )
})


test_that("Can convert tables to data frame", {
  mtcars_table <- xtabs(mtcars, formula = ~vs + am + cyl)

  mtcars2 <- as_tibble(mtcars_table)
  expect_equal(names(mtcars2), c(names(dimnames(mtcars_table)), "n"))

  expect_warning(
    mtcars2 <- as_tibble(mtcars_table, "Freq"),
    "named argument",
    fixed = TRUE
  )
  expect_equal(names(mtcars2), c(names(dimnames(mtcars_table)), "Freq"))

  mtcars2 <- as_tibble(mtcars_table, n = "Freq")
  expect_equal(names(mtcars2), c(names(dimnames(mtcars_table)), "Freq"))
})


test_that("Can convert unnamed atomic vectors to tibble by default", {
  skip_enh_as_tibble_retired()

  scoped_lifecycle_warnings()
  expect_warning(
    expect_equal(as_tibble(1:3), tibble(value = 1:3)),
    "discouraged",
    fixed = TRUE
  )

  expect_warning(
    expect_equal(as_tibble(c(TRUE, FALSE, NA)), tibble(value = c(TRUE, FALSE, NA))),
    "discouraged",
    fixed = TRUE
  )

  expect_warning(
    expect_equal(as_tibble(1.5:3.5), tibble(value = 1.5:3.5)),
    "discouraged",
    fixed = TRUE
  )

  expect_warning(
    expect_equal(as_tibble(letters), tibble(value = letters)),
    "discouraged",
    fixed = TRUE
  )
})


test_that("Can convert named atomic vectors to data frame", {
  skip("Do we want an .as_row argument?")
  expect_equal(as_tibble(setNames(nm = 1:3)), tibble(`1` = 1L, `2` = 2L, `3` = 3L))
  expect_equal(as_tibble(setNames(nm = c(TRUE, FALSE))), tibble(`TRUE` = TRUE, `FALSE` = FALSE))
  expect_equal(as_tibble(setNames(nm = 1.5:3.5)), tibble(`1.5` = 1.5, `2.5` = 2.5, `3.5` = 3.5))
  expect_equal(as_tibble(setNames(nm = letters)), tibble(!!!setNames(nm = letters)))

  expect_error(
    as_tibble(setNames(nm = c(TRUE, FALSE, NA))),
    invalid_df("must be named", 3),
    fixed = TRUE
  )
})

test_that("as_tibble() checks for `unique` names by default (#278)", {
  l1 <- list(1:10)
  expect_legacy_error(
    as_tibble(l1),
    error_column_must_be_named(1, repair_hint = TRUE),
    fixed = TRUE
  )

  l2 <- list(x = 1, 2)
  expect_legacy_error(
    as_tibble(l2),
    error_column_must_be_named(2, repair_hint = TRUE),
    fixed = TRUE
  )

  l3 <- list(x = 1, ... = 2)
  expect_legacy_error(
    as_tibble(l3),
    error_column_must_not_be_dot_dot(2, repair_hint = TRUE),
    fixed = TRUE
  )

  l4 <- list(x = 1, ..1 = 2)
  expect_legacy_error(
    as_tibble(l4),
    error_column_must_not_be_dot_dot(2, repair_hint = TRUE),
    fixed = TRUE
  )

  df <- list(a = 1, b = 2)
  names(df) <- c("", NA)
  df <- new_tibble(df, nrow = 1)
  expect_legacy_error(
    as_tibble(df),
    error_column_must_be_named(1:2, repair_hint = TRUE),
    fixed = TRUE
  )
})

test_that("as_tibble() makes names `minimal`, even if not fixing names", {
  invalid_df <- as_tibble(list(3, 4, 5), .name_repair = "minimal")
  expect_equal(length(invalid_df), 3)
  expect_equal(nrow(invalid_df), 1)
  expect_equal(names(invalid_df), rep("", 3))
})

test_that("as_tibble() implements unique names", {
  suppressMessages(
    invalid_df <- as_tibble(list(3, 4, 5), .name_repair = "unique")
  )
  expect_equal(length(invalid_df), 3)
  expect_equal(nrow(invalid_df), 1)
  expect_equal(names(invalid_df), unique_names(rep("", 3)))
})

test_that("as_tibble() implements universal names", {
  suppressMessages(
    invalid_df <- as_tibble(list(3, 4, 5), .name_repair = "universal")
  )
  expect_equal(length(invalid_df), 3)
  expect_equal(nrow(invalid_df), 1)
  expect_equal(names(invalid_df), universal_names(rep("", 3)))
})


test_that("as_tibble() implements custom name repair", {
  invalid_df <- as_tibble(
    list(3, 4, 5),
    .name_repair = function(x) make.names(x, unique = TRUE)
  )
  expect_equal(length(invalid_df), 3)
  expect_equal(nrow(invalid_df), 1)
  expect_equal(names(invalid_df), make.names(rep("", 3), unique = TRUE))

  invalid_df_purrr <- as_tibble(
    list(3, 4, 5),
    .name_repair = ~make.names(., unique = TRUE)
  )
  expect_identical(invalid_df_purrr, invalid_df)
})

test_that("as_tibble.matrix() supports validate (with warning) (#558)", {
  scoped_lifecycle_silence()

  expect_identical(
    as_tibble(diag(3), validate = TRUE),
    tibble(
      V1 = c(1, 0, 0),
      V2 = c(0, 1, 0),
      V3 = c(0, 0, 1)
    )
  )
})

test_that("as_tibble.matrix() supports .name_repair", {
  scoped_lifecycle_errors() # When removing this, double-check error messages below.

  x <- matrix(1:6, nrow = 3)

  skip_int_lifecycle()

  expect_error(
    as_tibble(x),
    class = get_defunct_error_class(),
    "name",
    fixed = TRUE
  )
})

test_that("as_tibble.matrix() supports .name_repair", {
  scoped_lifecycle_errors() # When removing this, double-check error messages below.

  x <- matrix(1:6, nrow = 3)

  expect_identical(
    names(as_tibble(x, .name_repair = "minimal")),
    rep("", 2)
  )
  expect_identical(
    names(suppressMessages(as_tibble(x, .name_repair = "universal"))),
    paste0("...", 1:2)
  )

  x <- matrix(1:6, nrow = 3, dimnames = list(x = LETTERS[1:3], y = c("if", "when")))

  expect_identical(
    names(as_tibble(x)),
    c("if", "when")
  )
  expect_identical(
    names(as_tibble(x, .name_repair = "minimal")),
    c("if", "when")
  )
  expect_identical(
    names(suppressMessages(as_tibble(x, .name_repair = "universal"))),
    c(".if", "when")
  )
})

test_that("as_tibble.poly() supports .name_repair", {
  x <- poly(1:6, 3)

  expect_identical(
    names(as_tibble(x)),
    as.character(1:3)
  )
  expect_identical(
    names(as_tibble(x, .name_repair = "minimal")),
    as.character(1:3)
  )
  expect_identical(
    names(suppressMessages(as_tibble(x, .name_repair = "universal"))),
    paste0("...", 1:3)
  )
})

test_that("as_tibble.table() supports .name_repair", {
  x <- table(a = c(1, 1, 1, 2, 2, 2), a = c(3, 4, 5, 3, 4, 5))

  expect_legacy_error(
    as_tibble(x),
    error_column_names_must_be_unique("a")
  )
  expect_identical(
    names(as_tibble(x, .name_repair = "minimal")),
    c("a", "a", "n")
  )
  expect_identical(
    names(suppressMessages(as_tibble(x, .name_repair = "universal"))),
    c("a...1", "a...2", "n")
  )

  x <- table("if" = c(1, 1, 1, 2, 2, 2), "when" = c(3, 4, 5, 3, 4, 5))

  expect_identical(
    names(as_tibble(x)),
    c("if", "when", "n")
  )
  expect_identical(
    names(as_tibble(x, .name_repair = "minimal")),
    c("if", "when", "n")
  )
  expect_identical(
    names(suppressMessages(as_tibble(x, .name_repair = "universal"))),
    c(".if", "when", "n")
  )

  x <- table("m" = c(1, 1, 1, 2, 2, 2), "n" = c(3, 4, 5, 3, 4, 5))

  expect_identical(
    names(as_tibble(x, .name_repair = "minimal")),
    c("m", "n", "n")
  )
  expect_identical(
    names(suppressMessages(as_tibble(x, .name_repair = "universal"))),
    c("m", "n...2", "n...3")
  )
})

test_that("as_tibble.ts() supports .name_repair, minimal by default (#537)", {
  x <- ts(matrix(rnorm(6), nrow = 3), start = c(1961, 1), frequency = 12, names = NULL)

  expect_identical(
    names(as_tibble(x)),
    rep("", 2)
  )
  expect_identical(
    names(as_tibble(x, .name_repair = "minimal")),
    rep("", 2)
  )
  expect_identical(
    names(suppressMessages(as_tibble(x, .name_repair = "universal"))),
    paste0("...", 1:2)
  )

  x <- ts(matrix(rnorm(6), nrow = 3), start = c(1961, 1), frequency = 12, names = c("if", "when"))

  expect_identical(
    names(as_tibble(x)),
    c("if", "when")
  )
  expect_identical(
    names(as_tibble(x, .name_repair = "minimal")),
    c("if", "when")
  )
  expect_identical(
    names(suppressMessages(as_tibble(x, .name_repair = "universal"))),
    c(".if", "when")
  )
})

test_that("as_tibble() can convert row names", {
  df <- data.frame(a = 1:3, b = 2:4, row.names = letters[5:7])

  expect_identical(
    as_tibble(df, rownames = NULL),
    tibble(a = 1:3, b = 2:4)
  )
  expect_identical(
    as_tibble(df, rownames = "id"),
    tibble(id = letters[5:7], a = 1:3, b = 2:4)
  )
  tbl_df <- as_tibble(df, rownames = NA)
  expect_identical(rownames(tbl_df), rownames(df))
  expect_identical(unclass(tbl_df), unclass(df))
})

test_that("as_tibble() can convert row names for zero-row tibbles", {
  df <- data.frame(a = 1:3, b = 2:4, row.names = letters[5:7])[0, ]

  expect_identical(
    as_tibble(df, rownames = NULL),
    tibble(a = integer(), b = integer())
  )
  expect_identical(
    as_tibble(df, rownames = "id"),
    tibble(id = character(), a = integer(), b = integer())
  )
  tbl_df <- as_tibble(df, rownames = NA)
  expect_identical(rownames(tbl_df), rownames(df))
  expect_identical(unclass(tbl_df), unclass(df))
})

test_that("as_tibble() throws an error when user turns missing row names into column", {
  skip_enh_as_tibble_rownames()

  df <- data.frame(a = 1:3, b = 2:4)
  expect_legacy_error(
    as_tibble(df, rownames = "id"),
    error_as_tibble_needs_rownames(),
    fixed = TRUE
  )
  expect_legacy_error(
    as_tibble(df[0, ], rownames = "id"),
    error_as_tibble_needs_rownames(),
    fixed = TRUE
  )
})

test_that("as_data_frame is an alias of as_tibble", {
  scoped_lifecycle_silence()
  expect_identical(as_data_frame(NULL), as_tibble(NULL))
})

test_that("as.tibble is an alias of as_tibble", {
  scoped_lifecycle_silence()
  expect_identical(as.tibble(NULL), as_tibble(NULL))
})

# Validation --------------------------------------------------------------

test_that("POSIXlt isn't a valid column", {
  skip_enh_posixlt_supported()
  expect_legacy_error(
    check_valid_cols(list(x = as.POSIXlt(Sys.time()))),
    error_time_column_must_be_posixct("x"),
    fixed = TRUE
  )
})

test_that("NULL isn't a valid column", {
  expect_legacy_error(
    check_valid_cols(list(a = NULL)),
    error_column_scalar_type("a", "NULL"),
    fixed = TRUE
  )
})

test_that("mutate() semantics for tibble() (#213)", {
  expect_equal(
    tibble(a = 1:2, b = 1, c = b / sum(b)),
    tibble(a = 1:2, b = c(1, 1), c = c(0.5, 0.5))
  )

  expect_equal(
    tibble(b = 1, a = 1:2, c = b / sum(b)),
    tibble(b = c(1, 1), a = 1:2, c = c(0.5, 0.5))
  )

  expect_equal(
    tibble(b = 1, c = b / sum(b), a = 1:2),
    tibble(b = c(1, 1), c = c(1, 1), a = 1:2)
  )
})

test_that("types preserved when recycling in tibble() (#284)", {
  expect_equal(
    tibble(a = 1:2, b = as.difftime(1, units = "hours")),
    tibble(a = 1:2, b = as.difftime(c(1, 1), units = "hours"))
  )

  expect_equal(
    tibble(b = as.difftime(1, units = "hours"), a = 1:2),
    tibble(b = as.difftime(c(1, 1), units = "hours"), a = 1:2)
  )
})

test_that("`validate` triggers deprecation message, but then works", {
  scoped_lifecycle_warnings()

  expect_legacy_error(
    expect_warning(
      as_tibble(list(a = 1, "hi"), validate = TRUE),
      "deprecated",
      fixed = TRUE
    ),
    error_column_must_be_named(2, repair_hint = TRUE)
  )

  expect_warning(
    df <- as_tibble(list(a = 1, "hi", a = 2), validate = FALSE),
    "deprecated",
    fixed = TRUE
  )
  expect_identical(names(df), c("a", "", "a"))

  df <- data.frame(a = 1, "hi", a = 2)
  names(df) <- c("a", "", "a")
  expect_warning(
    df <- as_tibble(df, validate = FALSE),
    "deprecated",
    fixed = TRUE
  )
  expect_identical(names(df), c("a", "", "a"))

  df <- data.frame(a = 1, "hi")
  names(df) <- c("a", "")
  expect_legacy_error(
    expect_warning(
      as_tibble(df, validate = TRUE),
      "deprecated",
      fixed = TRUE
    ),
    error_column_must_be_named(2, repair_hint = TRUE)
  )
})

test_that("Consistent `validate` and `.name_repair` used together keep silent.", {
  skip_legacy()

  scoped_lifecycle_warnings()

  expect_legacy_error(
    expect_warning(
      as_tibble(list(a = 1, "hi"), validate = TRUE, .name_repair = "check_unique"),
      NA
    ),
    error_column_must_be_named(2, repair_hint = TRUE)
  )

  expect_warning(
    df <- as_tibble(list(a = 1, "hi", a = 2), validate = FALSE, .name_repair = "minimal"),
    NA
  )
  expect_identical(names(df), c("a", "", "a"))

  df <- data.frame(a = 1, "hi", a = 2)
  names(df) <- c("a", "", "a")
  expect_warning(
    df <- as_tibble(df, validate = FALSE, .name_repair = "minimal"),
    NA
  )
  expect_identical(names(df), c("a", "", "a"))

  df <- data.frame(a = 1, "hi")
  names(df) <- c("a", "")
  expect_legacy_error(
    expect_warning(
      as_tibble(df, validate = TRUE, .name_repair = "check_unique"),
      NA
    ),
    error_column_must_be_named(2, repair_hint = TRUE)
  )
})

test_that("Inconsistent `validate` and `.name_repair` used together raise a warning.", {
  skip_legacy()

  expect_legacy_error(
    expect_warning(
      as_tibble(list(a = 1, "hi"), validate = FALSE, .name_repair = "check_unique"),
      "precedence"
    ),
    error_column_must_be_named(2, repair_hint = TRUE)
  )

  expect_warning(
    df <- as_tibble(list(a = 1, "hi", a = 2), validate = TRUE, .name_repair = "minimal"),
    "precedence"
  )
  expect_identical(names(df), c("a", "", "a"))

  df <- data.frame(a = 1, "hi", a = 2)
  names(df) <- c("a", "", "a")
  expect_warning(
    df <- as_tibble(df, validate = TRUE, .name_repair = "minimal"),
    "precedence"
  )
  expect_identical(names(df), c("a", "", "a"))

  df <- data.frame(a = 1, "hi")
  names(df) <- c("a", "")
  expect_legacy_error(
    expect_warning(
      as_tibble(df, validate = FALSE, .name_repair = "check_unique"),
      "precedence"
    ),
    error_column_must_be_named(2, repair_hint = TRUE)
  )
})


# Data frame and matrix columns -------------------------------------------

test_that("can make tibble containing data.frame or array (#416)", {
  skip_brk_auto_splice_anonymous()
  expect_identical(
    tibble(mtcars),
    new_tibble(list(mtcars = remove_rownames(mtcars)), nrow = nrow(mtcars))
  )
})

test_that("can make tibble containing data.frame or array (#416)", {
  expect_identical(
    tibble(diag(5)),
    new_tibble(list(`diag(5)` = diag(5)), nrow = 5)
  )
})

test_that("can coerce list data.frame or array (#416)", {
  skip_brk_inner_data_frames()
  expect_identical(
    as_tibble(list(x = mtcars)),
    new_tibble(list(x = remove_rownames(mtcars)), nrow = nrow(mtcars))
  )
})

test_that("can coerce list data.frame or array (#416)", {
  expect_identical(
    as_tibble(list(x = diag(5))),
    new_tibble(list(x = diag(5)), nrow = 5)
  )
})

test_that("susbsetting returns the correct number of rows", {
  expect_identical(
    tibble(x = mtcars)[1:3, ],
    tibble(x = mtcars[1:3, ])
  )
  expect_identical(
    tibble(y = diag(5))[1:3, ],
    tibble(y = diag(5)[1:3, ])
  )
})

test_that("subsetting one row retains columns", {
  expect_identical(
    tibble(y = diag(5))[1, ],
    tibble(y = diag(5)[1, , drop = FALSE])
  )
})

test_that("tibble returns correct number of rows with all combinatinos", {
  expect_equal(nrow(tibble(value = 1:10)), 10L)
  expect_equal(nrow(tibble(value = 1:10, name = "recycle_me")), 10L)
  expect_equal(nrow(tibble(name = "recycle_me", value = 1:10)), 10L)
  expect_equal(nrow(tibble(name = "recycle_me", value = 1:10, value2 = 11:20)), 10L)
  expect_equal(nrow(tibble(value = 1:10, name = "recycle_me", value2 = 11:20)), 10L)
})

test_that("dim attribute is stripped of 1D array (#84)", {
  skip_brk_inner_dim_not_stripped()
  expect_null(dim(tibble(x = array(1:3))$x))
})

test_that("bogus columns raise an error", {
  skip_enh_tibble_null()
  expect_error(
    tibble(a = NULL),
    error_column_scalar_type("a", "NULL"),
    fixed = TRUE
  )
})

test_that("bogus columns raise an error", {
  expect_legacy_error(
    tibble(a = new.env()),
    error_column_scalar_type("a", "environment"),
    fixed = TRUE
  )
  expect_legacy_error(
    tibble(a = quote(a)),
    error_column_scalar_type("a", "name"),
    fixed = TRUE
  )
})

test_that("length 1 vectors are recycled", {
  expect_equal(nrow(tibble(x = 1:10)), 10)
  expect_equal(nrow(tibble(x = 1:10, y = 1)), 10)
  expect_legacy_error(
    tibble(x = 1:10, y = 1:2),
    error_inconsistent_cols(NULL, c("x", "y"), c(10, 2), NA),
    fixed = TRUE
  )
})

test_that("length 1 vectors in hierarchical data frames are recycled (#502)", {
  expect_identical(
    tibble(x = 1:10, y = tibble(z = 1)),
    tibble(x = 1:10, y = tibble(z = rep(1, 10)))
  )
  expect_identical(
    tibble(y = tibble(z = 1), x = 1:10),
    tibble(y = tibble(z = rep(1, 10)), x = 1:10)
  )
  expect_identical(
    tibble(x = 1, y = tibble(z = 1:10)),
    tibble(x = rep(1, 10), y = tibble(z = 1:10))
  )
  expect_identical(
    tibble(y = tibble(z = 1:10), x = 1),
    tibble(y = tibble(z = 1:10), x = rep(1, 10))
  )
})

test_that("missing names are imputed from call", {
  x <- 1:10
  df <- tibble(x, y = x)
  expect_equal(names(df), c("x", "y"))
})

test_that("empty input makes 0 x 0 tbl_df", {
  zero <- tibble()
  expect_s3_class(zero, "tbl_df")
  expect_equal(dim(zero), c(0L, 0L))
  expect_identical(attr(zero, "names"), character(0L))
})

test_that("SE version", {
  scoped_lifecycle_silence()
  expect_identical(tibble_(list(a = ~ 1:10)), tibble(a = 1:10))
})

test_that("names are stripped from vectors", {
  skip_brk_inner_names_not_stripped()
  foo <- tibble(x = c(y = 1, z = 2))
  expect_equal(names(foo), "x")
  expect_null(names(foo$x))
})

test_that("names in list columns are preserved", {
  foo <- tibble(x = list(y = 1:3, z = 4:5))
  expect_equal(names(foo), "x")
  expect_equal(names(foo$x), c("y", "z"))
})

test_that("attributes are preserved", {
  df <- structure(
    data.frame(x = 1:10, g1 = rep(1:2, each = 5), g2 = rep(1:5, 2)),
    meta = "this is important"
  )
  res <- as_tibble(df)

  expect_identical(attr(res, "meta"), attr(df, "meta"))
})

test_that("tibble aliases", {
  scoped_lifecycle_silence()
  expect_identical(data_frame(a = 1), tibble(a = 1))

  skip_int_data_frame_tibble_diff()
  expect_identical(data_frame_, tibble_)
})


# as_tibble -----------------------------------------------------------

test_that("columns are recycled to common length", {
  expect_identical(
    as_tibble(list(x = 1, y = 1:3)),
    tibble(x = rep(1, 3), y = 1:3)
  )
  expect_identical(
    as_tibble(list(x = 1:3, y = 1)),
    tibble(x = 1:3, y = rep(1, 3))
  )
  expect_identical(
    as_tibble(list(x = character(), y = 1)),
    tibble(x = character(), y = numeric())
  )
})

test_that("columns must be same length", {
  expect_legacy_error(
    as_tibble(list(x = 1:2, y = 1:3)),
    error_inconsistent_cols(NULL, c("x", "y"), 2:3, NA),
    fixed = TRUE
  )
  expect_legacy_error(
    as_tibble(list(x = 1:2, y = 1:3, z = 1:4)),
    error_inconsistent_cols(
      NULL,
      c("x", "y", "z"),
      2:4,
      NA
    ),
    fixed = TRUE
  )
  expect_legacy_error(
    as_tibble(list(x = 1:4, y = 1:2, z = 1:2)),
    error_inconsistent_cols(
      NULL,
      c("x", "y", "z"),
      c(4, 2, 2),
      NA
    ),
    fixed = TRUE
  )
  expect_legacy_error(
    as_tibble(list(x = 1, y = 1:4, z = 1:2)),
    error_inconsistent_cols(
      NULL,
      c("y", "z"),
      c(4, 2),
      NA
    ),
    fixed = TRUE
  )
  expect_legacy_error(
    as_tibble(list(x = 1:2, y = 1:4, z = 1)),
    error_inconsistent_cols(
      NULL,
      c("x", "y"),
      c(2, 4),
      NA
    ),
    fixed = TRUE
  )
})

test_that("empty list() makes 0 x 0 tbl_df", {
  zero <- as_tibble(list())
  expect_s3_class(zero, "tbl_df")
  expect_equal(dim(zero), c(0L, 0L))
})


test_that("NULL makes 0 x 0 tbl_df", {
  nnnull <- as_tibble(NULL)
  expect_s3_class(nnnull, "tbl_df")
  expect_equal(dim(nnnull), c(0L, 0L))
})


test_that("as_tibble.tbl_df() leaves classes unchanged (#60)", {
  df <- tibble()
  expect_equal(
    class(df),
    c("tbl_df", "tbl", "data.frame")
  )
  expect_equal(
    class(structure(df, class = c("my_df", class(df)))),
    c("my_df", "tbl_df", "tbl", "data.frame")
  )
})


test_that("Can convert tables to data frame", {
  mtcars_table <- xtabs(mtcars, formula = ~ vs + am + cyl)

  mtcars2 <- as_tibble(mtcars_table)
  expect_equal(names(mtcars2), c(names(dimnames(mtcars_table)), "n"))

  expect_warning(
    mtcars2 <- as_tibble(mtcars_table, "Freq"),
    "named argument",
    fixed = TRUE
  )
  expect_equal(names(mtcars2), c(names(dimnames(mtcars_table)), "Freq"))

  mtcars2 <- as_tibble(mtcars_table, n = "Freq")
  expect_equal(names(mtcars2), c(names(dimnames(mtcars_table)), "Freq"))
})


test_that("Can convert unnamed atomic vectors to tibble by default", {
  skip_enh_as_tibble_retired()

  scoped_lifecycle_warnings()
  expect_warning(
    expect_equal(as_tibble(1:3), tibble(value = 1:3)),
    "discouraged",
    fixed = TRUE
  )

  expect_warning(
    expect_equal(as_tibble(c(TRUE, FALSE, NA)), tibble(value = c(TRUE, FALSE, NA))),
    "discouraged",
    fixed = TRUE
  )

  expect_warning(
    expect_equal(as_tibble(1.5:3.5), tibble(value = 1.5:3.5)),
    "discouraged",
    fixed = TRUE
  )

  expect_warning(
    expect_equal(as_tibble(letters), tibble(value = letters)),
    "discouraged",
    fixed = TRUE
  )
})


test_that("Can convert named atomic vectors to data frame", {
  skip("Do we want an .as_row argument?")
  expect_equal(as_tibble(setNames(nm = 1:3)), tibble(`1` = 1L, `2` = 2L, `3` = 3L))
  expect_equal(as_tibble(setNames(nm = c(TRUE, FALSE))), tibble(`TRUE` = TRUE, `FALSE` = FALSE))
  expect_equal(as_tibble(setNames(nm = 1.5:3.5)), tibble(`1.5` = 1.5, `2.5` = 2.5, `3.5` = 3.5))
  expect_equal(as_tibble(setNames(nm = letters)), tibble(!!!setNames(nm = letters)))

  expect_error(
    as_tibble(setNames(nm = c(TRUE, FALSE, NA))),
    invalid_df("must be named", 3),
    fixed = TRUE
  )
})

test_that("as_tibble() checks for `unique` names by default (#278)", {
  l1 <- list(1:10)
  expect_legacy_error(
    as_tibble(l1),
    error_column_must_be_named(1, repair_hint = TRUE),
    fixed = TRUE
  )

  l2 <- list(x = 1, 2)
  expect_legacy_error(
    as_tibble(l2),
    error_column_must_be_named(2, repair_hint = TRUE),
    fixed = TRUE
  )

  l3 <- list(x = 1, ... = 2)
  expect_legacy_error(
    as_tibble(l3),
    error_column_must_not_be_dot_dot(2, repair_hint = TRUE),
    fixed = TRUE
  )

  l4 <- list(x = 1, ..1 = 2)
  expect_legacy_error(
    as_tibble(l4),
    error_column_must_not_be_dot_dot(2, repair_hint = TRUE),
    fixed = TRUE
  )

  df <- list(a = 1, b = 2)
  names(df) <- c("", NA)
  df <- new_tibble(df, nrow = 1)
  expect_legacy_error(
    as_tibble(df),
    error_column_must_be_named(1:2, repair_hint = TRUE),
    fixed = TRUE
  )
})

test_that("as_tibble() makes names `minimal`, even if not fixing names", {
  invalid_df <- as_tibble(list(3, 4, 5), .name_repair = "minimal")
  expect_equal(length(invalid_df), 3)
  expect_equal(nrow(invalid_df), 1)
  expect_equal(names(invalid_df), rep("", 3))
})

test_that("as_tibble() implements unique names", {
  suppressMessages(
    invalid_df <- as_tibble(list(3, 4, 5), .name_repair = "unique")
  )
  expect_equal(length(invalid_df), 3)
  expect_equal(nrow(invalid_df), 1)
  expect_equal(names(invalid_df), unique_names(rep("", 3)))
})

test_that("as_tibble() implements universal names", {
  suppressMessages(
    invalid_df <- as_tibble(list(3, 4, 5), .name_repair = "universal")
  )
  expect_equal(length(invalid_df), 3)
  expect_equal(nrow(invalid_df), 1)
  expect_equal(names(invalid_df), universal_names(rep("", 3)))
})


test_that("as_tibble() implements custom name repair", {
  invalid_df <- as_tibble(
    list(3, 4, 5),
    .name_repair = function(x) make.names(x, unique = TRUE)
  )
  expect_equal(length(invalid_df), 3)
  expect_equal(nrow(invalid_df), 1)
  expect_equal(names(invalid_df), make.names(rep("", 3), unique = TRUE))

  invalid_df_purrr <- as_tibble(
    list(3, 4, 5),
    .name_repair = ~ make.names(., unique = TRUE)
  )
  expect_identical(invalid_df_purrr, invalid_df)
})

test_that("as_tibble.matrix() supports validate (with warning) (#558)", {
  scoped_lifecycle_silence()

  expect_identical(
    as_tibble(diag(3), validate = TRUE),
    tibble(
      V1 = c(1, 0, 0),
      V2 = c(0, 1, 0),
      V3 = c(0, 0, 1)
    )
  )
})

test_that("as_tibble.matrix() supports .name_repair", {
  scoped_lifecycle_errors() # When removing this, double-check error messages below.

  x <- matrix(1:6, nrow = 3)

  skip_int_lifecycle()

  expect_error(
    as_tibble(x),
    class = get_defunct_error_class(),
    "name",
    fixed = TRUE
  )
})

test_that("as_tibble.matrix() supports .name_repair", {
  scoped_lifecycle_errors() # When removing this, double-check error messages below.

  x <- matrix(1:6, nrow = 3)

  expect_identical(
    names(as_tibble(x, .name_repair = "minimal")),
    rep("", 2)
  )
  expect_identical(
    names(suppressMessages(as_tibble(x, .name_repair = "universal"))),
    paste0("...", 1:2)
  )

  x <- matrix(1:6, nrow = 3, dimnames = list(x = LETTERS[1:3], y = c("if", "when")))

  expect_identical(
    names(as_tibble(x)),
    c("if", "when")
  )
  expect_identical(
    names(as_tibble(x, .name_repair = "minimal")),
    c("if", "when")
  )
  expect_identical(
    names(suppressMessages(as_tibble(x, .name_repair = "universal"))),
    c(".if", "when")
  )
})

test_that("as_tibble.poly() supports .name_repair", {
  x <- poly(1:6, 3)

  expect_identical(
    names(as_tibble(x)),
    as.character(1:3)
  )
  expect_identical(
    names(as_tibble(x, .name_repair = "minimal")),
    as.character(1:3)
  )
  expect_identical(
    names(suppressMessages(as_tibble(x, .name_repair = "universal"))),
    paste0("...", 1:3)
  )
})

test_that("as_tibble.table() supports .name_repair", {
  x <- table(a = c(1, 1, 1, 2, 2, 2), a = c(3, 4, 5, 3, 4, 5))

  expect_legacy_error(
    as_tibble(x),
    error_column_names_must_be_unique("a")
  )
  expect_identical(
    names(as_tibble(x, .name_repair = "minimal")),
    c("a", "a", "n")
  )
  expect_identical(
    names(suppressMessages(as_tibble(x, .name_repair = "universal"))),
    c("a...1", "a...2", "n")
  )

  x <- table("if" = c(1, 1, 1, 2, 2, 2), "when" = c(3, 4, 5, 3, 4, 5))

  expect_identical(
    names(as_tibble(x)),
    c("if", "when", "n")
  )
  expect_identical(
    names(as_tibble(x, .name_repair = "minimal")),
    c("if", "when", "n")
  )
  expect_identical(
    names(suppressMessages(as_tibble(x, .name_repair = "universal"))),
    c(".if", "when", "n")
  )

  x <- table("m" = c(1, 1, 1, 2, 2, 2), "n" = c(3, 4, 5, 3, 4, 5))

  expect_identical(
    names(as_tibble(x, .name_repair = "minimal")),
    c("m", "n", "n")
  )
  expect_identical(
    names(suppressMessages(as_tibble(x, .name_repair = "universal"))),
    c("m", "n...2", "n...3")
  )
})

test_that("as_tibble.ts() supports .name_repair, minimal by default (#537)", {
  x <- ts(matrix(rnorm(6), nrow = 3), start = c(1961, 1), frequency = 12, names = NULL)

  expect_identical(
    names(as_tibble(x)),
    rep("", 2)
  )
  expect_identical(
    names(as_tibble(x, .name_repair = "minimal")),
    rep("", 2)
  )
  expect_identical(
    names(suppressMessages(as_tibble(x, .name_repair = "universal"))),
    paste0("...", 1:2)
  )

  x <- ts(matrix(rnorm(6), nrow = 3), start = c(1961, 1), frequency = 12, names = c("if", "when"))

  expect_identical(
    names(as_tibble(x)),
    c("if", "when")
  )
  expect_identical(
    names(as_tibble(x, .name_repair = "minimal")),
    c("if", "when")
  )
  expect_identical(
    names(suppressMessages(as_tibble(x, .name_repair = "universal"))),
    c(".if", "when")
  )
})

test_that("as_tibble() can convert row names", {
  df <- data.frame(a = 1:3, b = 2:4, row.names = letters[5:7])

  expect_identical(
    as_tibble(df, rownames = NULL),
    tibble(a = 1:3, b = 2:4)
  )
  expect_identical(
    as_tibble(df, rownames = "id"),
    tibble(id = letters[5:7], a = 1:3, b = 2:4)
  )
  tbl_df <- as_tibble(df, rownames = NA)
  expect_identical(rownames(tbl_df), rownames(df))
  expect_identical(unclass(tbl_df), unclass(df))
})

test_that("as_tibble() can convert row names for zero-row tibbles", {
  df <- data.frame(a = 1:3, b = 2:4, row.names = letters[5:7])[0, ]

  expect_identical(
    as_tibble(df, rownames = NULL),
    tibble(a = integer(), b = integer())
  )
  expect_identical(
    as_tibble(df, rownames = "id"),
    tibble(id = character(), a = integer(), b = integer())
  )
  tbl_df <- as_tibble(df, rownames = NA)
  expect_identical(rownames(tbl_df), rownames(df))
  expect_identical(unclass(tbl_df), unclass(df))
})

test_that("as_tibble() throws an error when user turns missing row names into column", {
  skip_enh_as_tibble_rownames()

  df <- data.frame(a = 1:3, b = 2:4)
  expect_legacy_error(
    as_tibble(df, rownames = "id"),
    error_as_tibble_needs_rownames(),
    fixed = TRUE
  )
  expect_legacy_error(
    as_tibble(df[0, ], rownames = "id"),
    error_as_tibble_needs_rownames(),
    fixed = TRUE
  )
})

test_that("as_data_frame is an alias of as_tibble", {
  scoped_lifecycle_silence()
  expect_identical(as_data_frame(NULL), as_tibble(NULL))
})

test_that("as.tibble is an alias of as_tibble", {
  scoped_lifecycle_silence()
  expect_identical(as.tibble(NULL), as_tibble(NULL))
})

# Validation --------------------------------------------------------------

test_that("POSIXlt isn't a valid column", {
  skip_enh_posixlt_supported()
  expect_legacy_error(
    check_valid_cols(list(x = as.POSIXlt(Sys.time()))),
    error_time_column_must_be_posixct("x"),
    fixed = TRUE
  )
})

test_that("NULL isn't a valid column", {
  expect_legacy_error(
    check_valid_cols(list(a = NULL)),
    error_column_scalar_type("a", "NULL"),
    fixed = TRUE
  )
})

test_that("mutate() semantics for tibble() (#213)", {
  expect_equal(
    tibble(a = 1:2, b = 1, c = b / sum(b)),
    tibble(a = 1:2, b = c(1, 1), c = c(0.5, 0.5))
  )

  expect_equal(
    tibble(b = 1, a = 1:2, c = b / sum(b)),
    tibble(b = c(1, 1), a = 1:2, c = c(0.5, 0.5))
  )

  expect_equal(
    tibble(b = 1, c = b / sum(b), a = 1:2),
    tibble(b = c(1, 1), c = c(1, 1), a = 1:2)
  )
})

test_that("types preserved when recycling in tibble() (#284)", {
  expect_equal(
    tibble(a = 1:2, b = as.difftime(1, units = "hours")),
    tibble(a = 1:2, b = as.difftime(c(1, 1), units = "hours"))
  )

  expect_equal(
    tibble(b = as.difftime(1, units = "hours"), a = 1:2),
    tibble(b = as.difftime(c(1, 1), units = "hours"), a = 1:2)
  )
})

test_that("`validate` triggers deprecation message, but then works", {
  scoped_lifecycle_warnings()

  expect_legacy_error(
    expect_warning(
      as_tibble(list(a = 1, "hi"), validate = TRUE),
      "deprecated",
      fixed = TRUE
    ),
    error_column_must_be_named(2, repair_hint = TRUE)
  )

  expect_warning(
    df <- as_tibble(list(a = 1, "hi", a = 2), validate = FALSE),
    "deprecated",
    fixed = TRUE
  )
  expect_identical(names(df), c("a", "", "a"))

  df <- data.frame(a = 1, "hi", a = 2)
  names(df) <- c("a", "", "a")
  expect_warning(
    df <- as_tibble(df, validate = FALSE),
    "deprecated",
    fixed = TRUE
  )
  expect_identical(names(df), c("a", "", "a"))

  df <- data.frame(a = 1, "hi")
  names(df) <- c("a", "")
  expect_legacy_error(
    expect_warning(
      as_tibble(df, validate = TRUE),
      "deprecated",
      fixed = TRUE
    ),
    error_column_must_be_named(2, repair_hint = TRUE)
  )
})

test_that("Consistent `validate` and `.name_repair` used together keep silent.", {
  skip_legacy()

  scoped_lifecycle_warnings()

  expect_legacy_error(
    expect_warning(
      as_tibble(list(a = 1, "hi"), validate = TRUE, .name_repair = "check_unique"),
      NA
    ),
    error_column_must_be_named(2, repair_hint = TRUE)
  )

  expect_warning(
    df <- as_tibble(list(a = 1, "hi", a = 2), validate = FALSE, .name_repair = "minimal"),
    NA
  )
  expect_identical(names(df), c("a", "", "a"))

  df <- data.frame(a = 1, "hi", a = 2)
  names(df) <- c("a", "", "a")
  expect_warning(
    df <- as_tibble(df, validate = FALSE, .name_repair = "minimal"),
    NA
  )
  expect_identical(names(df), c("a", "", "a"))

  df <- data.frame(a = 1, "hi")
  names(df) <- c("a", "")
  expect_legacy_error(
    expect_warning(
      as_tibble(df, validate = TRUE, .name_repair = "check_unique"),
      NA
    ),
    error_column_must_be_named(2, repair_hint = TRUE)
  )
})

test_that("Inconsistent `validate` and `.name_repair` used together raise a warning.", {
  skip_legacy()

  expect_legacy_error(
    expect_warning(
      as_tibble(list(a = 1, "hi"), validate = FALSE, .name_repair = "check_unique"),
      "precedence"
    ),
    error_column_must_be_named(2, repair_hint = TRUE)
  )

  expect_warning(
    df <- as_tibble(list(a = 1, "hi", a = 2), validate = TRUE, .name_repair = "minimal"),
    "precedence"
  )
  expect_identical(names(df), c("a", "", "a"))

  df <- data.frame(a = 1, "hi", a = 2)
  names(df) <- c("a", "", "a")
  expect_warning(
    df <- as_tibble(df, validate = TRUE, .name_repair = "minimal"),
    "precedence"
  )
  expect_identical(names(df), c("a", "", "a"))

  df <- data.frame(a = 1, "hi")
  names(df) <- c("a", "")
  expect_legacy_error(
    expect_warning(
      as_tibble(df, validate = FALSE, .name_repair = "check_unique"),
      "precedence"
    ),
    error_column_must_be_named(2, repair_hint = TRUE)
  )
})


# Data frame and matrix columns -------------------------------------------

test_that("can make tibble containing data.frame or array (#416)", {
  skip_brk_auto_splice_anonymous()
  expect_identical(
    tibble(mtcars),
    new_tibble(list(mtcars = remove_rownames(mtcars)), nrow = nrow(mtcars))
  )
})

test_that("can make tibble containing data.frame or array (#416)", {
  expect_identical(
    tibble(diag(5)),
    new_tibble(list(`diag(5)` = diag(5)), nrow = 5)
  )
})

test_that("can coerce list data.frame or array (#416)", {
  skip_brk_inner_data_frames()
  expect_identical(
    as_tibble(list(x = mtcars)),
    new_tibble(list(x = remove_rownames(mtcars)), nrow = nrow(mtcars))
  )
})

test_that("can coerce list data.frame or array (#416)", {
  expect_identical(
    as_tibble(list(x = diag(5))),
    new_tibble(list(x = diag(5)), nrow = 5)
  )
})

test_that("susbsetting returns the correct number of rows", {
  expect_identical(
    tibble(x = mtcars)[1:3, ],
    tibble(x = mtcars[1:3, ])
  )
  expect_identical(
    tibble(y = diag(5))[1:3, ],
    tibble(y = diag(5)[1:3, ])
  )
})

test_that("subsetting one row retains columns", {
  expect_identical(
    tibble(y = diag(5))[1, ],
    tibble(y = diag(5)[1, , drop = FALSE])
  )
})

test_that("basic", {
  expect_true(has_name(trees, "Volume"))
  expect_false(has_name(mtcars, "gears"))
})

test_that("other types", {
  expect_true(has_name(list(a = 1), "a"))
  expect_true(has_name(c(a = 1), "a"))
})

test_that("vectorized", {
  expect_equal(has_name(list(a = 1), letters), c(TRUE, rep(FALSE, 25)))
})

test_that("NA", {
  expect_false(has_name(list(a = 1), NA))
})

test_that("unnamed", {
  expect_false(has_name(1, "a"))
  expect_true(has_name(1, ""))
})

test_that("works as expected", {
  expect_identical(string_to_indices(as.character(1:3)), 1:3)
  expect_identical(string_to_indices(letters[1:3]), rep(NA_integer_, 3))
  expect_identical(string_to_indices(as.character(1:3 + 1e10)), 1:3 + 1e10)
  expect_identical(string_to_indices(c(as.character(1:3 + 1e10), "x")), c(1:3 + 1e10, NA))
  expect_identical(string_to_indices(as.character(c(1:3, 1:3 + 1e10))), c(1:3, 1:3 + 1e10))
})

---
title: "Invariants: Comparing behavior with data frames"
#output: rmarkdown::word_document
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Invariants: Comparing behavior with data frames}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

<style type="text/css">
.dftbl {
    width: 100%;
    table-layout: fixed;
    display: inline-table;
}

.error pre code {
    color: red;
}

.warning pre code {
    color: violet;
}
</style>

```{r, include = FALSE}
# To suppress messages
library(tibble)
library(vctrs)

knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  error = TRUE
)
tibble:::set_dftbl_hooks()

options(
  lifecycle_verbosity = "warning",
  lifecycle_disable_warnings = FALSE,
  lifecycle_verbose_soft_deprecation = TRUE,
  lifecycle_repeat_warnings = TRUE
)

# Set to FALSE for production
eval_details <- (Sys.getenv("IN_GALLEY") != "")
```

This vignette defines invariants for subsetting and subset-assignment for tibbles, and illustrates where their behaviour differs from data frames.
The goal is to define a small set of invariants that consistently define how behaviors interact.
Some behaviors are defined using functions of the vctrs package, e.g. `vec_slice()`, `vec_recycle()` and `vec_as_index()`.
Refer to their documentation for more details about the invariants that they follow.

The subsetting and subassignment operators for data frames and tibbles are particularly tricky, because they support both row and column indexes, both of which are optionally missing.
We resolve this by first defining column access with `[[` and `$`, then column-wise subsetting with `[`, then row-wise subsetting, then the composition of both.

## Conventions

In this article, all behaviors are demonstrated using one example data frame and its tibble equivalent:

```{r setup}
library(tibble)
library(vctrs)

new_df <- function() {
  df <- data.frame(n = c(1L, NA, 3L, NA))
  df$c <- letters[5:8]
  df$li <- list(9, 10:11, 12:14, "text")
  df
}

new_tbl <- function() {
  as_tibble(new_df())
}
```

Results of the same code for data frames and tibbles are presented side by side:

```{r show, dftbl = TRUE, dftbl_always = TRUE}
new_df()
```

If the results are identical (after converting to a data frame if necessary), only the tibble result is shown.

Subsetting operations are read-only.
The same objects are reused in all examples:

```{r ro}
df <- new_df()
tbl <- new_tbl()
```

Where needed, we also show examples with hierarchical columns containing a data frame or a matrix:

```{r setup2}
new_tbl2 <- function() {
  tibble(
    tb = tbl,
    m = diag(4)
  )
}

new_df2 <- function() {
  df2 <- new_tbl2()
  class(df2) <- "data.frame"
  class(df2$tb) <- "data.frame"
  df2
}

df2 <- new_df2()
tbl2 <- new_tbl2()
```

```{r show-compare-2, dftbl = TRUE}
new_df()
```

For subset assignment (subassignment, for short), we need a fresh copy of the data for each test.
The `with_*()` functions (omitted here for brevity) allow for a more concise notation.
These functions take an assignment expression, execute it on a fresh copy of the data, and return the data for printing.
The first example prints what's really executed, further examples omit this output.

```{r with-def, include = FALSE}
with_df <- function(code, verbose = FALSE) {
  code <- rlang::enexpr(code)

  full_code <- rlang::quo({
    df <- new_df()
    !!code
    df
  })
  if (verbose) rlang::expr_print(rlang::quo_get_expr(full_code))
  rlang::eval_tidy(full_code)
}

with_tbl <- function(code, verbose = FALSE) {
  code <- rlang::enexpr(code)

  full_code <- rlang::quo({
    tbl <- new_tbl()
    !!code
    tbl
  })
  if (verbose) rlang::expr_print(rlang::quo_get_expr(full_code))
  rlang::eval_tidy(full_code)
}

with_df2 <- function(code) {
  code <- rlang::enexpr(code)

  full_code <- rlang::quo({
    df2 <- new_df2()
    !!code
    df2
  })
  rlang::eval_tidy(full_code)
}

with_tbl2 <- function(code) {
  code <- rlang::enexpr(code)

  full_code <- rlang::quo({
    tbl2 <- new_tbl2()
    !!code
    tbl2
  })
  rlang::eval_tidy(full_code)
}
```

```{r with-demo, dftbl = TRUE}
with_df(df$n <- rev(df$n), verbose = TRUE)
```

## Column extraction

### Definition of `x[[j]]`

`x[[j]]` is equal to `.subset2(x, j)`.

```{r double-bracket-equivalent-to-subset2, dftbl = TRUE}
df[[1]]
.subset2(df, 1)
```


```{r double-bracket-equivalent-to-subset2-detail, dftbl = TRUE, include = eval_details, eval = eval_details}
identical(df[[3]], .subset2(df, 3))
identical(df2[["df"]], .subset2(df2, "df"))
```

NB: `x[[j]]` always returns an object of size `nrow(x)` if the column exists.

```{r double-bracket-retains-size, dftbl = TRUE, include = eval_details, eval = eval_details}
vec_size(df[[1]])
vec_size(df[[3]])
vec_size(df2[[1]])
vec_size(df2[[2]])
```

`j` must be a single number or a string, as enforced by `.subset2(x, j)`.

```{r double-bracket-requires-scalar-j-index, dftbl = TRUE}
df[[1:2]]
df[[c("n", "c")]]
df[[TRUE]]
df[[mean]]
```

`NA` indexes, numeric out-of-bounds (OOB) values, and non-integers throw an error:

```{r double-bracket-j-oob-numeric, dftbl = TRUE}
df[[NA]]
df[[NA_character_]]
df[[NA_integer_]]
df[[-1]]
df[[4]]
df[[1.5]]
df[[Inf]]
```

Character OOB access is silent because a common package idiom is to check for the absence of a column with `is.null(df[[var]])`.

```{r double-bracket-j-oob-character, dftbl = TRUE}
df[["x"]]
```

### Definition of `x$name`

`x$name` and `x$"name"` are equal to `x[["name"]]`.

```{r dollar-equivalent-to-subset, dftbl = TRUE}
df$n
df$"n"
df[["n"]]
```

```{r dollar-equivalent-to-subset-detail, dftbl = TRUE, include = eval_details, eval = eval_details}
identical(df$li, df[["li"]])
identical(df2$tb, df2[["tb"]])
identical(df2$m, df2[["m"]])
```

Unlike data frames, tibbles do not partially match names.
Because `df$x` is rarely used in packages, it can raise a warning:

```{r dollar-equivalent-to-subset-pmatch, dftbl = TRUE}
df$l
df$not_present
```

## Column subsetting

### Definition of `x[j]`

`j` is converted to an integer vector by `vec_as_index(j, ncol(x), names = names(x))`.
Then `x[c(j_1, j_2, ..., j_n)]` is equivalent to `tibble(x[[j_1]], x[[j_2]], ..., x[[j_n]])`, keeping the corresponding column names.
This implies that `j` must be a numeric or character vector, or a logical vector with length 1 or `ncol(x)`.[^subset-extract-commute]

[^subset-extract-commute]: `x[j][[jj]]` is equal to `x[[ j[[jj]] ]]`, in particular `x[j][[1]]` is equal to `x[[j]]` for scalar numeric or integer `j`.


```{r bracket-j-definition, dftbl = TRUE}
df[1:2]
```

When subsetting repeated indexes, the resulting column names are undefined, do not rely on them.

```{r bracket-j-duplication, dftbl = TRUE}
df[c(1, 1)]
```

For tibbles with repeated column names, subsetting by name uses the first matching column.

`nrow(df[j])` equals `nrow(df)`.

```{r bracket-j-empty, dftbl = TRUE}
df[integer()]
```

Tibbles support indexing by a logical matrix, but only if all values in the returned vector are compatible.

```{r bracket-j-logical-matrix, dftbl = TRUE}
df[is.na(df)]
df[!is.na(df)]
```

### Definition of `x[, j]`

`x[, j]` is equal to `x[j]`.
Tibbles do not perform column extraction if `x[j]` would yield a single column.

```{r bracket-missing-i, dftbl = TRUE}
df[, 1]
df[, 1:2]
```

```{r bracket-missing-i-detail, dftbl = TRUE, include = eval_details, eval = eval_details}
identical(df[, 2:3], df[2:3])
identical(df2[, 1:2], df2[1:2])
```

### Definition of `x[, j, drop = TRUE]`

For backward compatiblity, `x[, j, drop = TRUE]` performs column __extraction__, returning `x[j][[1]]` when `ncol(x[j])` is 1.

```{r bracket-always-returns-tibble-drop, dftbl = TRUE}
df[, 1, drop = TRUE]
```

```{r bracket-always-returns-tibble-drop-detail, dftbl = TRUE, include = eval_details, eval = eval_details}
identical(df[, 3, drop = TRUE], df[[3]])
identical(df2[, 1, drop = TRUE], df2[[1]])
identical(df2[, 2, drop = TRUE], df2[[2]])
```

## Row subsetting

### Definition of `x[i, ]`

`x[i, ]` is equal to `tibble(vec_slice(x[[1]], i), vec_slice(x[[2]], i), ...)`.[^row-subset-efficiency]

[^row-subset-efficiency]: Row subsetting `x[i, ]` is not defined in terms of `x[[j]][i]` because that definition does not generalise to matrix and data frame columns.
For efficiency and backward compatibility, `i` is converted to an integer vector by `vec_as_index(i, nrow(x))` first.

```{r bracket-i, dftbl = TRUE}
df[3, ]
```

This means that `i` must be a numeric vector, or a logical vector of length `nrow(x)` or 1.
For compatibility, `i` can also be a character vector containing positive numbers.

```{r bracket-i-wrong-type, dftbl = TRUE}
df[mean, ]
df[list(1), ]
df["1", ]
```

Exception: OOB values generate warnings instead of errors:

```{r bracket-i-oob, dftbl = TRUE}
df[10, ]
df["x", ]
```


Unlike data frames, only logical vectors of length 1 are recycled.
<!-- TODO: should this be an error? #648 -->

```{r bracket-i-recycle, dftbl = TRUE}
df[c(TRUE, FALSE), ]
```

NB: scalar logicals are recycled, but scalar numerics are not.
That makes the `x[NA, ]` and `x[NA_integer_, ]` return different results.

```{r bracket-i-na, dftbl = TRUE}
df[NA, ]
df[NA_integer_, ]
```

### Definition of `x[i, , drop = TRUE]`

`drop = TRUE` has no effect when not selecting a single row:

```{r bracket-i-drop, dftbl = TRUE}
df[1, , drop = TRUE]
```

<!-- TODO: soft-deprecate -->

## Row and column subsetting

### Definition of `x[]` and `x[,]`

`x[]` and `x[,]` are equivalent to `x`.[^bracket-comma]

[^bracket-comma]: `x[,]` is equivalent to `x[]` because `x[, j]` is equivalent to `x[j]`.

### Definition of `x[i, j]`

`x[i, j]` is equal to `x[i, ][j]`.[^bracket-flip]

[^bracket-flip]: A more efficient implementation of `x[i, j]` would forward to `x[j][i, ]`.

```{r bracket-i-j-equivalent-to-i-subset-then-j, dftbl = TRUE, include = eval_details, eval = eval_details}
df[1, 1]
df[1, ][1]
identical(df[1, 2:3], df[2:3][1, ])
identical(df[2:3, 1], df[1][2:3, ])
identical(df2[2:3, 1:2], df2[1:2][2:3, ])
```


### Definition of `x[[i, j]]`

`i` must be a numeric vector of length 1.
`x[[i, j]]` is equal to `x[i, ][[j]]`, or `vctrs::vec_slice(x[[j]], i)`.[^bracket2-flip]

[^bracket2-flip]: Cell subsetting `x[[i, j]]` is not defined in terms of `x[[j]][[i]]` because that definition does not generalise to list, matrix and data frame columns.
A more efficient implementation of `x[[i, j]]` would check that `j` is a scalar and forward to `x[i, j][[1]]`.

```{r bracket-bracket-i-j-equivalent-to-i-subset-then-j}
df[[1, 1]]
df[[1, 3]]
```

This implies that `j` must be a numeric or character vector of length 1.

NB: `vec_size(x[[i, j]])` always equals 1.
Unlike `x[i, ]`, `x[[i, ]]` is not valid.

## Column update

### Definition of `x[[j]] <- a`

If `a` is a vector then `x[[j]] <- a` replaces the `j`th column with value `a`.

```{r double-bracket-assign-definition, dftbl = TRUE, include = eval_details, eval = eval_details}
with_df(df[[1]] <- 0)
with_df(df[[3]] <- 4:1)
with_df2(df2[[1]] <- 0)
with_df2(df2[[2]] <- 4:1)
```

```{r double-bracket-assign-requires-scalar-j-index, dftbl = TRUE, include = eval_details, eval = eval_details}
with_df(df[[1]] <- 0)
with_df(df[["c"]] <- 0)
```

```{r double-bracket-assign-requires-scalar-j-index-error, dftbl = TRUE, include = eval_details, eval = eval_details}
with_df(df[[TRUE]] <- 0)
with_df(df[[1:3]] <- 0)
with_df(df[[c("n", "c")]] <- 0)
with_df(df[[FALSE]] <- 0)
with_df(df[[1:2]] <- 0)
with_df(df[[NA_integer_]] <- 0)
with_df(df[[NA]] <- 0)
with_df(df[[NA_character_]] <- 0)
```

`a` is recycled to the same size as `x` so must have size `nrow(x)` or 1.
(The only exception is when `a` is `NULL`, as described below.)
Recycling also works for list, data frame, and matrix columns.

```{r double-bracket-assign-recycle, dftbl = TRUE}
with_df(df[["li"]] <- list(0))
with_df2(df2[["tb"]] <- df[1, ])
with_df2(df2[["m"]] <- df2[["m"]][1, , drop = FALSE])
```

```{r double-bracket-requires-size, dftbl = TRUE, include = eval_details, eval = eval_details}
with_df(df[[1]] <- 1)
with_df(df[[1]] <- 4:1)
with_df(df[[1]] <- 3:1)
with_df(df[[1]] <- 2:1)
```

`j` must be a scalar numeric or a string, and cannot be `NA`.
If `j` is OOB, a new column is added on the right hand side, with name repair if needed.

```{r double-bracket-assign-supports-new, dftbl = TRUE}
with_df(df[["x"]] <- 0)
with_df(df[[4]] <- 0)
with_df(df[[5]] <- 0)
```

<!-- HW: should we permitted oob assignment with numeric j? It's a bit weird to create a column with unknonw column -->

`df[[j]] <- a` replaces the complete column so can change the type.

```{r double-bracket-assign-supports-type-change, dftbl = TRUE, include = eval_details, eval = eval_details}
with_df(df[[1]] <- df[[2]])
with_df(df[[2]] <- df[[3]])
with_df(df[[3]] <- df2[[1]])
with_df2(df2[[1]] <- df2[[2]])
with_df2(df2[[2]] <- df[[1]])
```

`[[<-` supports removing a column by assigning `NULL` to it.

```{r double-bracket-assign-supports-null, dftbl = TRUE, include = eval_details, eval = eval_details}
with_df(df[[1]] <- NULL)
with_df2(df2[[2]] <- NULL)
```

Removing a nonexistent column is a no-op.

```{r double-bracket-assign-supports-null-unknown, dftbl = TRUE, include = eval_details, eval = eval_details}
with_df(df[["q"]] <- NULL)
```

### Definition of `x$name <- a`

`x$name <- a` and `x$"name" <- a` are equivalent to `x[["name"]] <- a`.[^column-assign-symmetry]

[^column-assign-symmetry]: `$` behaves almost completely symmetrically to `[[` when comparing subsetting and subassignment.

```{r dollar-equivalent-to-subset-assign, dftbl = TRUE}
with_df(df$n <- 0)
with_df(df[["n"]] <- 0)
```

```{r dollar-equivalent-to-subset-assign-detail, dftbl = TRUE, include = eval_details, eval = eval_details}
with_df(df$"n" <- 0)
```

`$<-` does not perform partial matching.

```{r dollar-equivalent-to-subset-assign-pmatch, dftbl = TRUE}
with_df(df$l <- 0)
with_df(df[["l"]] <- 0)
```

## Column subassignment: `x[j] <- a`

* If `j` is missing, it's replaced with `seq_along(x)`
* If `j` is logical vector, it's converted to numeric with `seq_along(x)[j]`.

### `a` is a list or data frame

If `inherits(a, "list")` or `inherits(a, "data.frame")` is `TRUE`, then `x[j] <- a` is equivalent to `x[[j[[1]]] <- a[[1]]`, `x[[j[[2]]]] <- a[[2]]`, ...

```{r bracket-assign-def, dftbl = TRUE}
with_df(df[1:2] <- list("x", 4:1))
with_df(df[c("li", "x", "c")] <- list("x", 4:1, NULL))
```

If `length(a)` equals 1, then it is recycled to the same length as `j`.

```{r bracket-assign-recycles, dftbl = TRUE}
with_df(df[1:2] <- list(1))
with_df(df[1:2] <- list(0, 0, 0))
with_df(df[1:3] <- list(0, 0))
```

An attempt to update the same column twice gives an error.

```{r, bracket-assign-multiple, dftbl = TRUE}
with_df(df[c(1, 1)] <- list(1, 2))
```

If `a` contains `NULL` values, the corresponding columns are removed *after* updating (i.e. position indexes refer to columns before any modifications).

```{r bracket-assign-remove, dftbl = TRUE}
with_df(df[1:2] <- list(NULL, 4:1))
```

`NA` indexes are not supported.

```{r bracket-assign-na, dftbl = TRUE, include = eval_details, eval = eval_details}
with_df(df[NA] <- list("x"))
with_df(df[NA_integer_] <- list("x"))
with_df(df[NA_character_] <- list("x"))
```

Just like column updates, `[<-` supports changing the type of an existing column.

```{r bracket-assign-supports-type-change, dftbl = TRUE, include = eval_details, eval = eval_details}
with_df(df[1] <- df[2])
with_df(df[2] <- df[3])
with_df(df[3] <- df2[1])
with_df2(df2[1] <- df2[2])
with_df2(df2[2] <- df[1])
```

Appending columns at the end (without gaps) is supported.
The name of new columns is determined by the LHS, the RHS, or by name repair (in that order of precedence).

```{r bracket-assign-names, dftbl = TRUE}
with_df(df[c("x", "y")] <- tibble("x", x = 4:1))
with_df(df[3:4] <- list("x", x = 4:1))
with_df(df[4] <- list(4:1))
with_df(df[5] <- list(4:1))
```

Tibbles support indexing by a logical matrix, but only for a scalar RHS, and if all columns updated are compatible with the value assigned.

```{r bracket-j-assign-logical-matrix, dftbl = TRUE}
with_df(df[is.na(df)] <- 4)
with_df(df[is.na(df)] <- 1:2)
with_df(df[matrix(c(rep(TRUE, 5), rep(FALSE, 7)), ncol = 3)] <- 4)
```

### `a` is a matrix or array

If `is.matrix(a)`, then `a` is coerced to a data frame with `as.data.frame()` before assigning.
If rows are assigned, the matrix type must be compatible with all columns.
If `is.array(a)` and `any(dim(a)[-1:-2] != 1)`, an error is thrown.

```{r bracket-assign-array, dftbl = TRUE}
with_df(df[1:2] <- matrix(8:1, ncol = 2))
with_df(df[1:3, 1:2] <- matrix(6:1, ncol = 2))
with_df(df[1:2] <- array(4:1, dim = c(4, 1, 1)))
with_df(df[1:2] <- array(8:1, dim = c(4, 2, 1)))
with_df(df[1:2] <- array(8:1, dim = c(2, 1, 4)))
with_df(df[1:2] <- array(8:1, dim = c(4, 1, 2)))
```

### `a` is another type of vector

If `vec_is(a)`, then `x[j] <- a` is equivalent to `x[j] <- list(a)`.
This is primarily provided for backward compatibility.

```{r bracket-assign-wraps, dftbl = TRUE}
with_df(df[1] <- 0)
with_df(df[1] <- list(0))
```

Matrices must be wrapped in `list()` before assignment to create a matrix column.

```{r bracket-assign-matrix, dftbl = TRUE}
with_df(df[1] <- list(matrix(1:8, ncol = 2)))

with_df(df[1:2] <- list(matrix(1:8, ncol = 2)))
```

### `a` is `NULL`

Entire columns can be removed.
Specifying `i` is an error.

```{r bracket-assign-null, dftbl = TRUE}
with_df(df[1] <- NULL)
with_df(df[, 2:3] <- NULL)
with_df(df[1, 2:3] <- NULL)
```

### `a` is not a vector

Any other type for `a` is an error.
Note that if `is.list(a)` is `TRUE`, but `inherits(a, "list")` is `FALSE`, then `a` is considered to be a scalar.
See `?vec_is` and `?vec_proxy` for details.

```{r bracket-assign-non-vector, dftbl = TRUE}
with_df(df[1] <- mean)
with_df(df[1] <- lm(mpg ~ wt, data = mtcars))
```

<!-- HW: we need better error messages for these cases -->

## Row subassignment: `x[i, ] <- list(...)`

`x[i, ] <- a` is the same as `vec_slice(x[[j_1]], i) <- a[[1]]`,  `vec_slice(x[[j_2]], i) <- a[[2]]`, ... .[^row-assign-symmetry]

[^row-assign-symmetry]: `x[i, ]` is symmetrical for subset and subassignment.

```{r bracket-i-assign, dftbl = TRUE}
with_df(df[2:3, ] <- df[1, ])
with_df(df[c(FALSE, TRUE, TRUE, FALSE), ] <- df[1, ])
```

```{r bracket-i-assign-detail, dftbl = TRUE, include = eval_details, eval = eval_details}
with_df(df[0:2, ] <- df[1, ])
with_df(df[0, ] <- df[1, ])
with_df(df[-2, ] <- df[1, ])
with_df(df[-1:2, ] <- df[1, ])
with_df(df[NA_integer_, ] <- df[1, ])
with_df2(df2[NA_integer_, ] <- df2[1, ])
with_df(df[TRUE, ] <- df[1, ])
with_df(df[FALSE, ] <- df[1, ])
with_df(df[NA, ] <- df[1, ])
```

Only values of size one can be recycled.

```{r bracket-i-recycle-assign, dftbl = TRUE}
with_df(df[2:3, ] <- df[1, ])
with_df(df[2:3, ] <- list(df$n[1], df$c[1:2], df$li[1]))
with_df(df[2:4, ] <- df[1:2, ])
```

```{r bracket-i-recycle-assign-detail, dftbl = TRUE, include = eval_details, eval = eval_details}
with_df2(df2[2:4, ] <- df2[1, ])
with_df2(df2[2:4, ] <- df2[2:3, ])
```

For compatibility, only a warning is issued for indexing beyond the number of rows.
Appending rows right at the end of the existing data is supported, without warning.

```{r bracket-i-oob-num, dftbl = TRUE}
with_df(df[5, ] <- df[1, ])
with_df(df[5:7, ] <- df[1, ])
with_df(df[6, ] <- df[1, ])
with_df(df[-5, ] <- df[1, ])
with_df(df[-(5:7), ] <- df[1, ])
with_df(df[-6, ] <- df[1, ])
```

For compatibility, `i` can also be a character vector containing positive numbers.

```{r bracket-i-character, dftbl = TRUE}
with_df(df[as.character(1:3), ] <- df[1, ])
```

```{r bracket-i-character-detail, dftbl = TRUE, include = eval_details, eval = eval_details}
with_df(df[as.character(-(1:3)), ] <- df[1, ])
with_df(df[as.character(3:5), ] <- df[1, ])
with_df(df[as.character(-(3:5)), ] <- df[1, ])
with_df(df[NA_character_, ] <- df[1, ])
```

## Row and column subassignment

### Definition of `x[i, j] <- a`

`x[i, j] <- a` is equivalent to `x[i, ][j] <- a`.[^bracket-assign-flip]

[^bracket-assign-flip]: `x[i, j]` is symmetrical for subsetting and subassignment.
A more efficient implementation of `x[i, j] <- a` would forward to `x[j][i, ] <- a`.

Subassignment to `x[i, j]` is stricter for tibbles than for data frames.
`x[i, j] <- a` can't change the data type of existing columns.

```{r bracket-i-data-type, dftbl = TRUE}
with_df(df[2:3, 1] <- df[1:2, 2])
with_df(df[2:3, 2] <- df[1:2, 3])
with_df(df[2:3, 3] <- df2[1:2, 1])
with_df2(df2[2:3, 1] <- df2[1:2, 2])
with_df2(df2[2:3, 2] <- df[1:2, 1])
```

A notable exception is the population of a column full of `NA` (which is of type `logical`), or the use of `NA` on the right-hand side of the assignment.

```{r bracket-i-j-na-init, dftbl = TRUE}
with_df({df$x <- NA; df[2:3, "x"] <- 3:2})
with_df({df[2:3, 2:3] <- NA})
```

For programming, it is always safer (and faster) to use the correct type of `NA` to initialize columns.

```{r bracket-i-j-typed-na-init, dftbl = TRUE}
with_df({df$x <- NA_integer_; df[2:3, "x"] <- 3:2})
```


For new columns, `x[i, j] <- a` fills the unassigned rows with `NA`.

```{r subassign-ij-new-column, dftbl = TRUE}
with_df(df[2:3, "n"] <- 1)
with_df(df[2:3, "x"] <- 1)
with_df(df[2:3, "n"] <- NULL)
```

Likewise, for new rows, `x[i, j] <- a` fills the unassigned columns with `NA`.

```{r append-rows-only-all-columns, dftbl = TRUE}
with_df(df[5, "n"] <- list(0L))
```


### Definition of `x[[i, j]] <- a`

`i` must be a numeric vector of length 1.
`x[[i, j]] <- a` is equivalent to `x[i, ][[j]] <- a`.[^double-bracket-ij-symmetry]

[^double-bracket-ij-symmetry]: `x[[i, j]]` is symmetrical for subsetting and subassignment.
An efficient implementation would check that `i` and `j` are scalar and forward to `x[i, j][[1]] <- a`.


```{r double-bracket-i-j-equivalent-to-row-subset-then-j, dftbl = TRUE, include = eval_details, eval = eval_details}
with_df(df[[1, 1]] <- 0)
with_df(df[1, ][[1]] <- 0)
with_df(df[[1, 3]] <- list(NULL))
with_df(df[1, ][[3]] <- list(NULL))
with_df2(df2[[1, 1]] <- df[1, ])
with_df2(df2[1, ][[1]] <- df[1, ])
with_df2(df2[[1, 2]] <- t(1:4))
with_df2(df2[1, ][[2]] <- t(1:4))
df[[1:2, 1]]
with_df(df[[1:2, 1]] <- 0)
```

NB: `vec_size(a)` must equal 1.
Unlike `x[i, ] <-`, `x[[i, ]] <-` is not valid.

```{r check, dftbl = TRUE, include = FALSE}
stopifnot(identical(df, new_df()))
```

---
title: "Controlling display of numbers"
output: html_vignette
vignette: >
  %\VignetteIndexEntry{Controlling display of numbers}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include = FALSE}
library(tibble)
set.seed(1014)
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

Tibbles print numbers with three significant digits by default, switching to scientific notation if the available space is too small.
Underlines are used to highlight groups of three digits.
The display differs from the default display for data frames, see `vignette("digits")` for an overview over the differences.
This display works for many, but not for all use cases.

```{r numbers-2}
library(tibble)
```

## Options

The easiest way to customize the display of numbers and other data in a tibble is to define options.
See `?pillar::pillar_options` for a comprehensive overview.

```{r numbers-3}
tibble(x = 123.4567)
old <- options(pillar.sigfig = 7)
tibble(x = 123.4567)
# Restore old options, see also rlang::local_options() for a more elegant way
options(old)
```

This changes the display of all columns.
Read on to see how too specify display options on a column-by-column basis.


## Per-column number formatting

The new `num()` constructor allows creating vectors that behave like numbers but allow customizing their display.
Below a few examples are shown, see `?num` for a comprehensive overview.
Similarly, `char()` allows customizing the display of character columns.

```{r numbers-5}
num(-1:3, notation = "sci")
tibble(
  x4 = num(8:12 * 100 + 0.5, digits = 4),
  x1 = num(8:12 * 100 + 0.5, digits = -1),
  usd = num(8:12 * 100 + 0.5, digits = 2, label = "USD"),
  percent = num(8:12 / 100 + 0.0005, label = "%", scale = 100),
  eng = num(10^(-3:1), notation = "eng", fixed_exponent = -Inf),
  si = num(10^(-3:1) * 123, notation = "si"),
  char = char(paste(LETTERS, collapse = " "), shorten = "mid")
)
```

The pillar package that is responsible for the display of tibbles tries hard to get the number display right, however it is impossible to accommodate all use cases.
Whenever the default formatting does not suit the application, `num()` or `char()` allow redefining the formatting for individual columns.
The formatting survives most data transformations.

## Rule-based number formatting

Currently, formatting must be applied manually for each column.
The following pattern may help doing this consistently for all columns in a tibble, or for some columns based on their name.

```{r}
library(dplyr, warn.conflicts = FALSE)

markets <-
  as_tibble(EuStockMarkets) %>%
  mutate(time = time(EuStockMarkets), .before = 1)

markets
markets %>%
  mutate(across(-time, ~ num(.x, digits = 3)))
```

## Computing on `num`

Formatting numbers is useful for presentation of results.
If defined early on in the analysis, the formatting options survive most operations.
It is worth defining output options that suit your data once early on in the process, to benefit from the formatting throughout the analysis.
We are working on seamlessly applying this formatting to the final presentation (plots, tables, ...).


### Arithmetics

When applying arithmetic operations on numbers created by `num()`, the result inherits the formatting of the first `num` object.

```{r numbers-13}
num(1) + 2
1 + num(2)
1L + num(2)
num(3.23456, sigfig = 4) - num(2)
num(4, sigfig = 2) * num(3, digits = 2)
num(3, digits = 2) * num(4, sigfig = 2)
-num(2)
```

### Mathematics

Similarly, for mathematical operations, the formatting is inherited.

```{r numbers-14}
min(num(1:3, label = "$"))
mean(num(1:3, notation = "eng"))
sin(num(1:3, label = "%", scale = 100))
```


### Override

In some cases, the ideal formatting changes after a transformation.
`num()` can be applied repeatedly, the last setting wins.

```{r numbers-15}
num(1:3 + 0.125, digits = 4)
transf <- 10^num(1:3 + 0.125, digits = 4)
transf
num(transf, sigfig = 3)
```

### Recovery

The `var()` function is one of the examples where the formatting is lost:

```{r numbers-16}
x <- num(c(1, 2, 4), notation = "eng")
var(x)
```

The `median()` function is worse, it breaks for `num()` objects:

```{r numbers-16-a, error = TRUE}
median(x)
```

One way to recover is to apply `num()` to the result:

```{r numbers-16a}
num(var(x), notation = "eng")
num(median(as.numeric(x)), notation = "eng")
```

For automatic recovery, we can also define our version of `var()`, or even overwrite the base implementation.
Note that this pattern is still experimental and may be subject to change:

```{r numbers-16b}
var_ <- function(x, ...) {
  out <- var(vctrs::vec_proxy(x), ...)
  vctrs::vec_restore(out, x)
}
var_(x)
```

This pattern can be applied to all functions that lose the formatting.
The `make_restore()` function defined below is a function factory that consumes a function and returns a derived function:

```{r numbers-16c}
make_restore <- function(fun) {
  force(fun)
  function(x, ...) {
    out <- fun(vctrs::vec_proxy(x), ...)
    vctrs::vec_restore(out, x)
  }
}

var_ <- make_restore(var)
sd_ <- make_restore(sd)

var_(x)
sd_(x)
```

---
title: "Column types"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Column types}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
library(dplyr)
library(tidyr)
library(purrr)
requireNamespace("hms", quietly = TRUE)
```

```{r setup}
library(tibble)
```

## Overview

This vignette shows an overview of known data types and their abbreviations, and their origin.
For example, `<int>` in the header of a column indicates an integer column, and `<chr>` denotes a character column.

```{r howto, echo = FALSE, eval = FALSE}
library(tidyverse)
library(tidymodels)
library(vctrs)
library(pillar)

all_methods <-
  c("vec_ptype_abbr", "vec_ptype_full", "type_sum") %>%
  map(methods) %>%
  map(as.character) %>%
  map(~ gsub("^.*[.]", "", .x)) %>%
  unlist() %>%
  unique()

set_names(rep_along(all_methods, list("")), all_methods) %>%
  dput()
```



```{r data, echo = FALSE}
data <- list(
  "Atomic" = rlang::quos(
    logical = TRUE,
    integer = 1L,
    double = 1.5,
    character = "A",
    complex = 1i,
    raw = as.raw(1),
    list = list(1),
    "named list" = list(a = 1)
  ),

  "Built-in objects" = rlang::quos(
    factor = factor("A"),
    ordered = ordered("a"),
    Date = Sys.Date(),
    POSIXt = Sys.time(),
    difftime = vctrs::new_duration(1)
  ),

  "Objects from other packages" = rlang::quos(
    hms = hms::hms(1),
    integer64 = bit64::as.integer64(1e10),
    blob = blob::blob(raw(1))
  ),

  "Data frames" = rlang::quos(
    data.frame = data.frame(a = 1),
    tbl_df = tibble(a = 1)
  ),

  "Unchanged" = rlang::quos(
    AsIs = I(1L)
  ),

  "vctrs types" = rlang::quos(
    unspecified = vctrs::unspecified(1),

    vctrs_list_of = vctrs::list_of(c(1L)),
    vctrs_vctr = vctrs::new_vctr(1L),

    vctrs_partial_factor = vctrs::partial_factor(letters),
    vctrs_partial_frame = vctrs::partial_frame(a = 1)
  ),

  "Language objects" = rlang::quos(
    "function" = function() NULL,
    symbol = quote(a),
    expression = parse(text = "a <- 1\nb<- 2"),
    quosures = rlang::quos(a = 1)
  )
)
```

```{r table, echo = FALSE}
tbl <-
  data %>%
  map(unclass) %>%
  map(enframe, "Data type", "Expression") %>%
  enframe("Class", "data") %>%
  unnest(data) %>%
  mutate(Example = map_chr(Expression, rlang::as_label)) %>%
  mutate(Value = map(Expression, rlang::eval_tidy)) %>%
  select(-Expression) %>%
  mutate(Class = if_else(Class == lag(Class, default = ""), "", Class)) %>%
  mutate("Column header" = map_chr(Value, type_sum))
```

```{r kable, echo = FALSE}
tbl %>%
  select(-Value) %>%
  mutate(Example = paste0("`", Example, "`")) %>%
  knitr::kable(escape = FALSE)
```

## Example values

The following overview contains example values for each type:

```{r glimpse, echo = FALSE}
tbl %>%
  select(`Data type`, `Value`) %>%
  filter(map_lgl(Value, vctrs::vec_is)) %>%
  deframe() %>%
  as_tibble() %>%
  glimpse()
```


## Implementation

The column header is obtained by calling `pillar::type_sum()` on the column.
This is an S3 method that can be overridden,
but most of the time it is more useful to override `vctrs::vec_ptype_abbr()`:

```{r type_sum_default, results = if (Sys.getenv("IN_GALLEY") != "") "hide" else "markup"}
pillar:::type_sum.default
```

---
title: "Comparing display with data frames"
output: html_vignette
vignette: >
  %\VignetteIndexEntry{Comparing display with data frames}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include = FALSE}
library(tibble)
set.seed(1014)
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

```{r}
library(tibble)
```

Base R offers the `"digits"` and `"scipen"` options to control the number of significant digits and the switch to scientific notation.
For tibble, the options `"pillar.sigfig"` and `"pillar.max_dec_width"` fulfill a similar purpose.
This vignette showcases similarities and differences.
See `?pillar::pillar_options` and `?tibble_options` for an overview over all options.

## Digits

### Basic differences

The default for `getOption("digits")` is 7, whereas the `"pillar.sigfig"` option defaults to 3.
In the default setting, pillar prints the first three digits only (i.e. the digits that represent > 99.9% of the value of the number).
Another difference is that pillar will show at most the specified number of significant digits, even if space is available.
The rationale is to allow a quick glance over the most significant digits of a number, without spending too much horizontal space and without distraction from insignificant digits.

```{r}
options(digits = 3)
c(1.2345, 12.345, 123.45, 1234.5, 12345)
tibble(x = c(1.2345, 12.345, 123.45, 1234.5, 12345))
```

### Terminal zeros

Terminal zeros are only shown in pillar if there is a nonzero value past the significant digits shown.
This is in contrast to base R where terminal zeros are always shown if there is space, but hidden if the value is too insignificant:

```{r}
c(1, 1.00001)
tibble(x = c(1, 1.00001))
```

### Trailing dot

A trailing decimal separator is shown if there is a fractional part but the integer part already exceeds the significant digits.
The presence of the decimal separator does **not** indicate that the number is larger, only that there exists a nonzero fractional part:

```{r}
c(123, 123.45, 567.89)
tibble(x = c(123, 123.45, 567.89))
```

### Showing more digits

To show more significant digits, set the `"pillar.sigfig"` option to a larger value:

```{r}
options(digits = 7)
options(pillar.sigfig = 7)
c(1.2345, 12.345, 123.45, 1234.5, 12345)
tibble(x = c(1.2345, 12.345, 123.45, 1234.5, 12345))
```

Setting `"pillar.sigfig"` to a larger value will not enhance the display with digits deemed insignificant:

```{r}
options(digits = 7)
options(pillar.sigfig = 7)
c(1.2345, 12.3456, 123.4567, 1234.5678, 12345.6789)
tibble(x = c(1.2345, 12.3456, 123.4567, 1234.5678, 12345.6789))
```

### Fixed number of digits

To show a fixed number of decimal digits, use `num()` with a `digits` argument:

```{r}
num(c(1.2345, 12.345, 123.45, 1234.5, 12345), digits = 2)
```

See `vignette("numbers")` for details.

## Scientific notation

### When is it used?

Both base R and pillar switch to scientific notation when the decimal representation becomes too wide.
The larger `getOption("scipen")`, the stronger the resistance to switching to scientific notation.
The default `0` seems to be anchored at 13 digits for the integer part.

```{r}
123456789012
123456789012.3
1234567890123
1234567890123.4
options(scipen = 1)
1234567890123
12345678901234
12345678901234.5
```

The `"pillar.max_dec_width"` option is similar, it indicates the width that must be exceeded for a switch to scientific notation to happen.
This width includes the decimal separator.

```{r}
tibble(x = 123456789012)
tibble(x = 123456789012.3)
tibble(x = 1234567890123)
tibble(x = 1234567890123.4)
options(pillar.max_dec_width = 14)
tibble(x = 1234567890123)
tibble(x = 12345678901234)
tibble(x = 12345678901234.5)
```

### Enforce notation

To avoid switching to scientific notation, set the `"pillar.max_dec_width"` option to a large value.
Note that if the required width is not available to show the column, it will not be shown at all in this case.
The `notation` argument to `num()` offers more options:

```{r}
num(12345678901234567, notation = "dec")
num(12345678901234567, notation = "sci")
num(12345678901234567, notation = "eng")
num(12345678901234567, notation = "si")
```

---
title: "Extending tibble"
author: "Kirill Müller, Hadley Wickham"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Extending tibble}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

Tibbles are S3 objects of class `c("tbl_df", "tbl", "data.frame")`.
This means that they inherit their behavior from the `"data.frame"` class and add two subclasses `"tbl"` and `"tbl_df"`.
The pillar and tibble packages provide methods for `"tbl"` and `"tbl_df"`, respectively.
Package authors and programmers can implement subclasses that extend either `"tbl_df"` (and its subclasses) or only `"tbl"` to provide custom behavior for tibble-like objects.
In addition, vectors classes can be implemented from scratch or on top of existing classes, to be used as columns.
This article provides links to the various customization points that help you avoiding reimplement existing behavior, and describes the difference between `"tbl"` and `"tbl_df"`.

Read more in [the second edition of Advanced R](https://adv-r.hadley.nz/index.html):

- about the internal representation of data frames and tibbles in the [Lists](https://adv-r.hadley.nz/vectors-chap.html#lists) and [Data frames and tibbles](https://adv-r.hadley.nz/vectors-chap.html#tibble) sections of the [Vectors chapter](https://adv-r.hadley.nz/vectors-chap.html),
- about R's S3 object-oriented system in the [S3 chapter](https://adv-r.hadley.nz/s3.html).

## Topics documented elsewhere

- Change or tweak the way a tibble prints: `vignette("extending", package = "pillar")`

- Change or tweak the way a column type is printed in a tibble: `vignette("pillar", package = "vctrs")`

- Implement a new column data type: `vignette("s3-vector", package = "vctrs")`

- Making your tibble subclass work well with dplyr: `?dplyr::dplyr_extending`

## Data frame subclasses

```{r}
library(tibble)
```

For tibble >= 3.0.0, the `"tbl"` class is responsible for printing, while the `"tbl_df"` class adds tibble's sturdy subsetting and subset assignment behavior (see `vignette("invariants")` for details).
This means that a data frame class that would like to (mostly) print like a tibble but keep the behavior from base R's data frames can inherit from `c("tbl", "data.frame")` or just from `"tbl"`.
An example is the `"tbl_sql"` class in the dbplyr package that is used to implement lazy database tables.

### Tibble example

```{r}
my_tbl_df <- new_tibble(
  list(a = 1:3, b = 2:4),
  class = "my_tbl_df"
)

tbl_sum.my_tbl_df <- function(x, ...) {
  c(
    "My custom tibble" = "Some info about it",
    NextMethod()
  )
}

my_tbl_df
my_tbl_df[, "a"]
```

### Data frame example

```{r}
my_tbl <- vctrs::new_data_frame(
  list(a = 1:3, b = 2:4),
  class = c("my_tbl", "tbl")
)

tbl_sum.my_tbl <- function(x, ...) {
  c(
    "My custom data frame" = "Some info about it",
    NextMethod()
  )
}

my_tbl
my_tbl[, "a"]
```

---
title: "Extending tibble"
author: "Kirill Müller, Hadley Wickham"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Extending tibble}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

This material has moved to <https://vctrs.r-lib.org/articles/pillar.html>.

---
title: "Subassignment"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{subassign}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

<style type="text/css">
.dftbl {
    width: 100%;
    table-layout: fixed;
    display: inline-table;
}

.error pre code {
    color: red;
}

.warning pre code {
    color: violet;
}
</style>

```{r, include = FALSE}
knitr::opts_chunk$set(
  error = TRUE,
  collapse = TRUE,
  comment = "#>"
)

tibble:::set_dftbl_hooks()

options(
  lifecycle_disable_warnings = FALSE,
  lifecycle_verbose_soft_deprecation = TRUE,
  lifecycle_repeat_warnings = TRUE
)
```

This vignette is an attempt to provide a comprehensive overview over all subassignment operations, highlighting where the tibble implementation differs from the data frame implementation.

```{r setup}
library(tibble)

new_df <- function() {
  df <- data.frame(a = 1:4)
  df$b <- letters[5:8]
  df$cd <- list(9, 10:11, 12:14, "text")
  df
}

new_tbl <- function() {
  as_tibble(new_df())
}
```

Results of the same code for data frames and tibbles are presented side by side:

```{r show, dftbl = TRUE, dftbl_always = TRUE}
new_df()
```

In the following, if the results are identical (after converting to a data frame if necessary), only the tibble result is shown, as in the example below.
This allows to spot differences easier.

```{r show-compare, dftbl = TRUE}
new_df()
```

For subassignment, we need a fresh copy of the data for each test.
The `with_*()` functions allow for a more concise notation
(`with_tbl()` omitted here for brevity):

```{r with-df-def}
with_df <- function(code, verbose = FALSE) {
  code <- rlang::enexpr(code)

  full_code <- rlang::quo({
    df <- new_df()
    !!code
    df
  })
  if (verbose) rlang::expr_print(rlang::quo_get_expr(full_code))
  rlang::eval_tidy(full_code)
}
```


```{r with-tbl-def, include = FALSE}
with_tbl <- function(code, verbose = FALSE) {
  code <- rlang::enexpr(code)

  full_code <- rlang::quo({
    tbl <- new_tbl()
    !!code
    tbl
  })
  if (verbose) rlang::expr_print(rlang::quo_get_expr(full_code))
  rlang::eval_tidy(full_code)
}
```

This function takes an assignment expression and executes it on a fresh copy of the data.
The first example prints what's really executed, further examples omit this output.

```{r with-demo, dftbl = TRUE}
with_df(df$a <- rev(df$a), verbose = TRUE)
```


## $<-

### Scalars and full length

Assigning a scalar or a full-length vector to a column consistently overwrites existing data or appends a new column at the end. Partial matching doesn't happen:

```{r dollar-assign-scalar, dftbl = TRUE}
with_df(df$a <- 1)
with_df(df$b <- 1)
with_df(df$c <- 1)
with_df(df$cd <- 1)
```

```{r dollar-assign-full, dftbl = TRUE}
with_df(df$a <- 4:1)
with_df(df$b <- 4:1)
with_df(df$c <- 4:1)
with_df(df$cd <- 4:1)
```

### Recycling

Tibbles allow recycling only for vectors of length 1 or of the same length as the data.

```{r dollar-assign-recycle, dftbl = TRUE}
with_df(df$a <- 1:2)
with_df(df$a <- 1:3)
with_df(df$a <- 1:5)
with_df(df$c <- 1:2)
```



### Subset assignment

Updating parts of a column extracted by `$` is the responsibility of the column vector.
Tibbles can't control what happens after `$` has returned.

```{r dollar-assign-subset, dftbl = TRUE}
with_df(df$a[1:2] <- 4:3)
with_df(df$b[1:2] <- 4:3)
with_df(df$c[1:2] <- 4:3)
with_df(df$cd[1:2] <- 4:3)
with_df(df$a[1:3] <- 4:3)
with_df(df$a[1:4] <- 4:3)
```

For columns of the stricter `"vctrs_vctr"` class, this class implements the check, which then works identically for data frames and tibbles:

```{r vctrs}
with_df({ df$v = vctrs::new_vctr(1:4); df$v[1:2] <- vctrs::new_vctr(4:3)})
with_df({ df$v = vctrs::new_vctr(1:4); df$v[1:2] <- vctrs::new_vctr(letters[4:3])})
```

## [[<-

### Scalars and full length

As with `$` subsetting, columns are consistently overwritten, and partial matching doesn't occur.
Numeric indexing is supported, but tibbles don't support creation of new numbered columns for a good reason.

```{r double-bracket-assign-col-scalar, dftbl = TRUE}
with_df(df[["a"]] <- 1)
with_df(df[["a"]] <- "x")
with_df(df[["b"]] <- "x")
with_df(df[["c"]] <- "x")
with_df(df[["cd"]] <- "x")
with_df(df[[1]] <- "x")
with_df(df[[2]] <- "x")
with_df(df[[4]] <- "x")
with_df(df[[5]] <- "x")
```

```{r double-bracket-assign-col-full, dftbl = TRUE}
with_df(df[["a"]] <- 4:1)
with_df(df[["a"]] <- letters[4:1])
with_df(df[["b"]] <- letters[4:1])
with_df(df[["c"]] <- letters[4:1])
with_df(df[["cd"]] <- letters[4:1])
```

### Cells

Tibbles are stricter when updating single cells, the value must be coercible to the existing contents.
Updating a list column requires the contents to be wrapped in a list, consistently with `[[` subsetting which returns a list if a cell in a list column is accessed:

```{r double-bracket-assign-cell, dftbl = TRUE}
with_df(df[[2, "a"]] <- 1)
with_df(df[[2, "a"]] <- 1.5)
with_df(df[[2, "a"]] <- "x")
with_df(df[[2, "b"]] <- "x")
with_df(df[[2, 1]] <- "x")
with_df(df[[2, 2]] <- "x")
with_df(df[[2, "cd"]] <- "x")
with_df(df[[2, "cd"]] <- list("x"))
with_df(df[[2, "c"]] <- "x")
with_df(df[[1:2, "cd"]] <- "x")
with_df(df[[1:2, "c"]] <- "x")
with_df(df[[2, c("cd", "d")]] <- "x")
```

## [<-

### Scalars

```{r bracket-assign-scalar-col, dftbl = TRUE}
with_df(df[2, "a"] <- 1)
with_df(df[2, "a"] <- 1.5)
with_df(df[2, "a"] <- "x")
with_df(df[2, "b"] <- "x")
with_df(df[2, "cd"] <- "x")
with_df(df[2, "cd"] <- list("x"))
with_df(df[2, "c"] <- "x")
with_df(df[2, 1] <- "x")
with_df(df[2, 2] <- "x")
with_df(df[2, 3] <- "x")
with_df(df[2, 4] <- "x")
```

### Full length columns

```{r bracket-assign-full-col, dftbl = TRUE}
with_df(df[, "a"] <- 4:1)
with_df(df[, "b"] <- 4:1)
with_df(df[, "c"] <- 4:1)
with_df(df[, "cd"] <- 4:1)
with_df(df[, 1] <- 4:1)
with_df(df[, 2] <- 4:1)
with_df(df[, 3] <- 4:1)
with_df(df[, 4] <- 4:1)
with_df(df[, "a"] <- 1)
with_df(df[, "b"] <- 1)
with_df(df[, "c"] <- 1)
with_df(df[, "cd"] <- 1)
with_df(df[, 1] <- 1)
with_df(df[, 2] <- 1)
with_df(df[, 3] <- 1)
with_df(df[, 4] <- 1)
```

### Multiple full length columns

```{r bracket-assign-full-multicol, dftbl = TRUE}
with_df(df[, c("a", "b")] <- 4:1)
with_df(df[, c("a", "b")] <- 1)
with_df(df[, c("a", "b")] <- data.frame(a = 4:1, b = letters[4:1]))
with_df(df[, c("a", "b")] <- data.frame(b = 4:1, a = letters[4:1]))
with_df(df[, c("a", "b")] <- data.frame(c = 4:1, d = letters[4:1]))
with_df(df[, c("a", "b")] <- data.frame(a = 4:1))
with_df(df[, c("a", "b")] <- data.frame(a = 4:1, b = letters[4:1], c = 1:4))
with_df(df[, c("a", "b")] <- data.frame(4:1, 1))
with_df(df[, c("a", "b", "c")] <- data.frame(4:1, letters[4:1]))
with_df(df[, c("a", "b", "cd")] <- data.frame(4:1, letters[4:1]))
```

### Full length rows

```{r bracket-assign-full-row, dftbl = TRUE}
with_df(df[2, ] <- 1)
with_df(df[2, ] <- "x")
with_df(df[2, ] <- tibble(a = 1, b = "x"))
with_df(df[2, ] <- tibble(a = 1, b = "x", c = list("y")))
with_df(df[2, ] <- tibble(a = 1, b = "x", c = list("y"), d = "z"))
with_df(df[0, ] <- tibble(a = 1, b = "x", c = list("y")))
with_df(df[5, ] <- tibble(a = 1, b = "x", c = list("y")))
```

### Multiple full length rows

```{r bracket-assign-full-multirow, dftbl = TRUE}
with_df(df[2:3, ] <- 1)
with_df(df[2:3, ] <- 1:2)
with_df(df[2:3, ] <- c("x", "y"))
with_df(df[2:3, ] <- tibble(a = 1:2, b = c("x", "y")))
with_df(df[2:3, ] <- tibble(a = 1, b = "x", c = list("y")))
with_df(df[2:3, ] <- tibble(a = 1:2, b = "x", c = list("y")))
with_df(df[2:3, ] <- tibble(a = 1, b = "x", c = list("y"), d = "z"))
with_df(df[-(1:2), ] <- tibble(a = 1:2, b = "x", c = list("y")))
with_df(df[0:1, ] <- tibble(a = 1:2, b = "x", c = list("y")))
with_df(df[4:5, ] <- tibble(a = 1:2, b = "x", c = list("y")))
```

### Unspecified

```{r bracket-assign-unspecified, dftbl = TRUE}
with_df(df[] <- 1)
with_df(df[] <- 4:1)
with_df(df[] <- 3:1)
with_df(df[] <- 5:1)
with_df(df[] <- data.frame(1, "x"))
with_df(df[] <- data.frame(4:1, "x", 2))
with_df(df[] <- data.frame(1, "x", 2))
with_df(df[] <- data.frame(1, "x", 2, 3))
with_df(df[] <- df)
with_df(df[,] <- 1)
with_df(df[,] <- 4:1)
with_df(df[,] <- 3:1)
with_df(df[,] <- 5:1)
with_df(df[,] <- data.frame(1, "x"))
with_df(df[,] <- data.frame(4:1, "x", 2))
with_df(df[,] <- data.frame(1, "x", 2))
with_df(df[,] <- data.frame(1, "x", 2, 3))
with_df(df[,] <- df)
```

### Subset assignment

Due to tibble's default of `drop = FALSE`, updating a portion of a `[` subset is still safe, because tibble is still in control.
Only one example is given here.

```{r bracket-assign-subset, dftbl = TRUE}
with_df(df["a"][1, ] <- "b")
```

---
title: "Subassignment"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{subassign}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

<style type="text/css">
.dftbl {
    width: 100%;
    table-layout: fixed;
    display: inline-table;
}

.error pre code {
    color: red;
}

.warning pre code {
    color: violet;
}
</style>

```{r, include = FALSE}
knitr::opts_chunk$set(
  error = TRUE,
  collapse = TRUE,
  comment = "#>"
)

tibble:::set_dftbl_hooks()

options(
  lifecycle_disable_warnings = FALSE,
  lifecycle_verbose_soft_deprecation = TRUE,
  lifecycle_repeat_warnings = TRUE
)
```

This vignette is an attempt to provide a comprehensive overview over all subassignment operations, highlighting where the tibble implementation differs from the data frame implementation.

```{r setup}
library(tibble)

new_df <- function() {
  df <- data.frame(a = 1:4)
  df$b <- letters[5:8]
  df$cd <- list(9, 10:11, 12:14, "text")
  df
}

new_tbl <- function() {
  as_tibble(new_df())
}
```

Results of the same code for data frames and tibbles are presented side by side:

```{r show, dftbl = TRUE, dftbl_always = TRUE}
new_df()
```

In the following, if the results are identical (after converting to a data frame if necessary), only the tibble result is shown, as in the example below.
This allows to spot differences easier.

```{r show-compare, dftbl = TRUE}
new_df()
```

For subassignment, we need a fresh copy of the data for each test.
The `with_*()` functions allow for a more concise notation
(`with_tbl()` omitted here for brevity):

```{r with-df-def}
with_df <- function(code, verbose = FALSE) {
  code <- rlang::enexpr(code)
  
  full_code <- rlang::quo({
    df <- new_df()
    !!code
    df
  })
  if (verbose) rlang::expr_print(rlang::quo_get_expr(full_code))
  rlang::eval_tidy(full_code)
}
```


```{r with-tbl-def, include = FALSE}
with_tbl <- function(code, verbose = FALSE) {
  code <- rlang::enexpr(code)
  
  full_code <- rlang::quo({
    tbl <- new_tbl()
    !!code
    tbl
  })
  if (verbose) rlang::expr_print(rlang::quo_get_expr(full_code))
  rlang::eval_tidy(full_code)
}
```

This function takes an assignment expression and executes it on a fresh copy of the data.
The first example prints what's really executed, further examples omit this output.

```{r with-demo, dftbl = TRUE}
with_df(df$a <- rev(df$a), verbose = TRUE)
```


## $<-

### Scalars and full length

Assigning a scalar or a full-length vector to a column consistently overwrites existing data or appends a new column at the end. Partial matching doesn't happen:

```{r dollar-assign-scalar, dftbl = TRUE}
with_df(df$a <- 1)
with_df(df$b <- 1)
with_df(df$c <- 1)
with_df(df$cd <- 1)
```

```{r dollar-assign-full, dftbl = TRUE}
with_df(df$a <- 4:1)
with_df(df$b <- 4:1)
with_df(df$c <- 4:1)
with_df(df$cd <- 4:1)
```

### Recycling

Tibbles allow recycling only for vectors of length 1 or of the same length as the data.

```{r dollar-assign-recycle, dftbl = TRUE}
with_df(df$a <- 1:2)
with_df(df$a <- 1:3)
with_df(df$a <- 1:5)
with_df(df$c <- 1:2)
```



### Subset assignment

Updating parts of a column extracted by `$` is the responsibility of the column vector.
Tibbles can't control what happens after `$` has returned.

```{r dollar-assign-subset, dftbl = TRUE}
with_df(df$a[1:2] <- 4:3)
with_df(df$b[1:2] <- 4:3)
with_df(df$c[1:2] <- 4:3)
with_df(df$cd[1:2] <- 4:3)
with_df(df$a[1:3] <- 4:3)
with_df(df$a[1:4] <- 4:3)
```

For columns of the stricter `"vctrs_vctr"` class, this class implements the check, which then works identically for data frames and tibbles:

```{r vctrs}
with_df({ df$v = vctrs::new_vctr(1:4); df$v[1:2] <- vctrs::new_vctr(4:3)})
with_df({ df$v = vctrs::new_vctr(1:4); df$v[1:2] <- vctrs::new_vctr(letters[4:3])})
```

## [[<-

### Scalars and full length

As with `$` subsetting, columns are consistently overwritten, and partial matching doesn't occur.
Numeric indexing is supported, but tibbles don't support creation of new numbered columns for a good reason.

```{r double-bracket-assign-col-scalar, dftbl = TRUE}
with_df(df[["a"]] <- 1)
with_df(df[["a"]] <- "x")
with_df(df[["b"]] <- "x")
with_df(df[["c"]] <- "x")
with_df(df[["cd"]] <- "x")
with_df(df[[1]] <- "x")
with_df(df[[2]] <- "x")
with_df(df[[4]] <- "x")
with_df(df[[5]] <- "x")
```

```{r double-bracket-assign-col-full, dftbl = TRUE}
with_df(df[["a"]] <- 4:1)
with_df(df[["a"]] <- letters[4:1])
with_df(df[["b"]] <- letters[4:1])
with_df(df[["c"]] <- letters[4:1])
with_df(df[["cd"]] <- letters[4:1])
```

### Cells

Tibbles are stricter when updating single cells, the value must be coercible to the existing contents.
Updating a list column requires the contents to be wrapped in a list, consistently with `[[` subsetting which returns a list if a cell in a list column is accessed:

```{r double-bracket-assign-cell, dftbl = TRUE}
with_df(df[[2, "a"]] <- 1)
with_df(df[[2, "a"]] <- 1.5)
with_df(df[[2, "a"]] <- "x")
with_df(df[[2, "b"]] <- "x")
with_df(df[[2, 1]] <- "x")
with_df(df[[2, 2]] <- "x")
with_df(df[[2, "cd"]] <- "x")
with_df(df[[2, "cd"]] <- list("x"))
with_df(df[[2, "c"]] <- "x")
with_df(df[[1:2, "cd"]] <- "x")
with_df(df[[1:2, "c"]] <- "x")
with_df(df[[2, c("cd", "d")]] <- "x")
```

## [<-

### Scalars

```{r bracket-assign-scalar-col, dftbl = TRUE}
with_df(df[2, "a"] <- 1)
with_df(df[2, "a"] <- 1.5)
with_df(df[2, "a"] <- "x")
with_df(df[2, "b"] <- "x")
with_df(df[2, "cd"] <- "x")
with_df(df[2, "cd"] <- list("x"))
with_df(df[2, "c"] <- "x")
with_df(df[2, 1] <- "x")
with_df(df[2, 2] <- "x")
with_df(df[2, 3] <- "x")
with_df(df[2, 4] <- "x")
```

### Full length columns

```{r bracket-assign-full-col, dftbl = TRUE}
with_df(df[, "a"] <- 4:1)
with_df(df[, "b"] <- 4:1)
with_df(df[, "c"] <- 4:1)
with_df(df[, "cd"] <- 4:1)
with_df(df[, 1] <- 4:1)
with_df(df[, 2] <- 4:1)
with_df(df[, 3] <- 4:1)
with_df(df[, 4] <- 4:1)
with_df(df[, "a"] <- 1)
with_df(df[, "b"] <- 1)
with_df(df[, "c"] <- 1)
with_df(df[, "cd"] <- 1)
with_df(df[, 1] <- 1)
with_df(df[, 2] <- 1)
with_df(df[, 3] <- 1)
with_df(df[, 4] <- 1)
```

### Multiple full length columns

```{r bracket-assign-full-multicol, dftbl = TRUE}
with_df(df[, c("a", "b")] <- 4:1)
with_df(df[, c("a", "b")] <- 1)
with_df(df[, c("a", "b")] <- data.frame(a = 4:1, b = letters[4:1]))
with_df(df[, c("a", "b")] <- data.frame(b = 4:1, a = letters[4:1]))
with_df(df[, c("a", "b")] <- data.frame(c = 4:1, d = letters[4:1]))
with_df(df[, c("a", "b")] <- data.frame(a = 4:1))
with_df(df[, c("a", "b")] <- data.frame(a = 4:1, b = letters[4:1], c = 1:4))
with_df(df[, c("a", "b")] <- data.frame(4:1, 1))
with_df(df[, c("a", "b", "c")] <- data.frame(4:1, letters[4:1]))
with_df(df[, c("a", "b", "cd")] <- data.frame(4:1, letters[4:1]))
```

### Full length rows

```{r bracket-assign-full-row, dftbl = TRUE}
with_df(df[2, ] <- 1)
with_df(df[2, ] <- "x")
with_df(df[2, ] <- tibble(a = 1, b = "x"))
with_df(df[2, ] <- tibble(a = 1, b = "x", c = list("y")))
with_df(df[2, ] <- tibble(a = 1, b = "x", c = list("y"), d = "z"))
with_df(df[0, ] <- tibble(a = 1, b = "x", c = list("y")))
with_df(df[5, ] <- tibble(a = 1, b = "x", c = list("y")))
```

### Multiple full length rows

```{r bracket-assign-full-multirow, dftbl = TRUE}
with_df(df[2:3, ] <- 1)
with_df(df[2:3, ] <- 1:2)
with_df(df[2:3, ] <- c("x", "y"))
with_df(df[2:3, ] <- tibble(a = 1:2, b = c("x", "y")))
with_df(df[2:3, ] <- tibble(a = 1, b = "x", c = list("y")))
with_df(df[2:3, ] <- tibble(a = 1:2, b = "x", c = list("y")))
with_df(df[2:3, ] <- tibble(a = 1, b = "x", c = list("y"), d = "z"))
with_df(df[-(1:2), ] <- tibble(a = 1:2, b = "x", c = list("y")))
with_df(df[0:1, ] <- tibble(a = 1:2, b = "x", c = list("y")))
with_df(df[4:5, ] <- tibble(a = 1:2, b = "x", c = list("y")))
```

### Unspecified

```{r bracket-assign-unspecified, dftbl = TRUE}
with_df(df[] <- 1)
with_df(df[] <- 4:1)
with_df(df[] <- 3:1)
with_df(df[] <- 5:1)
with_df(df[] <- data.frame(1, "x"))
with_df(df[] <- data.frame(4:1, "x", 2))
with_df(df[] <- data.frame(1, "x", 2))
with_df(df[] <- data.frame(1, "x", 2, 3))
with_df(df[] <- df)
with_df(df[,] <- 1)
with_df(df[,] <- 4:1)
with_df(df[,] <- 3:1)
with_df(df[,] <- 5:1)
with_df(df[,] <- data.frame(1, "x"))
with_df(df[,] <- data.frame(4:1, "x", 2))
with_df(df[,] <- data.frame(1, "x", 2))
with_df(df[,] <- data.frame(1, "x", 2, 3))
with_df(df[,] <- df)
```

### Subset assignment

Due to tibble's default of `drop = FALSE`, updating a portion of a `[` subset is still safe, because tibble is still in control.
Only one example is given here.

```{r bracket-assign-subset, dftbl = TRUE}
with_df(df["a"][1, ] <- "b")
```

# output test

    Code
      # # add
      print_error(abort_add_rows_to_grouped_df())
    Output
      <error/tibble_error_add_rows_to_grouped_df>
      Error:
      ! Can't add rows to grouped data frames.
    Code
      print_error(abort_incompatible_new_rows("a"))
    Output
      <error/tibble_error_incompatible_new_rows>
      Error:
      ! New rows can't add columns.
      x Can't find column `a` in `.data`.
    Code
      print_error(abort_incompatible_new_rows(letters[2:3]))
    Output
      <error/tibble_error_incompatible_new_rows>
      Error:
      ! New rows can't add columns.
      x Can't find columns `b` and `c` in `.data`.
    Code
      print_error(abort_incompatible_new_rows(LETTERS))
    Output
      <error/tibble_error_incompatible_new_rows>
      Error:
      ! New rows can't add columns.
      x Can't find columns `A`, `B`, `C`, `D`, `E`, and 21 more in `.data`.
    Code
      print_error(abort_both_before_after())
    Output
      <error/tibble_error_both_before_after>
      Error:
      ! Can't specify both `.before` and `.after`.
    Code
      print_error(abort_unknown_column_names("a"))
    Output
      <error/tibble_error_unknown_column_names>
      Error:
      ! Can't find column `a` in `.data`.
    Code
      print_error(abort_unknown_column_names(c("b", "c")))
    Output
      <error/tibble_error_unknown_column_names>
      Error:
      ! Can't find columns `b` and `c` in `.data`.
    Code
      print_error(abort_unknown_column_names(LETTERS))
    Output
      <error/tibble_error_unknown_column_names>
      Error:
      ! Can't find columns `A`, `B`, `C`, `D`, `E`, and 21 more in `.data`.
    Code
      print_error(abort_incompatible_new_cols(10, data.frame(a = 1:2)))
    Output
      <error/tibble_error_incompatible_new_cols>
      Error:
      ! New columns must be compatible with `.data`.
      x New column has 2 rows.
      i `.data` has 10 rows.
    Code
      print_error(abort_incompatible_new_cols(1, data.frame(a = 1:3, b = 2:4)))
    Output
      <error/tibble_error_incompatible_new_cols>
      Error:
      ! New columns must be compatible with `.data`.
      x New columns have 3 rows.
      i `.data` has 1 row.
    Code
      # # as_tibble
      print_error(abort_column_scalar_type("a", 3, "environment"))
    Output
      <error/tibble_error_column_scalar_type>
      Error:
      ! All columns in a tibble must be vectors.
      x Column `a` is environment.
    Code
      print_error(abort_column_scalar_type("", 3, "environment"))
    Output
      <error/tibble_error_column_scalar_type>
      Error:
      ! All columns in a tibble must be vectors.
      x Column 3 is environment.
    Code
      print_error(abort_column_scalar_type(letters[2:3], 3:4, c("name", "NULL")))
    Output
      <error/tibble_error_column_scalar_type>
      Error:
      ! All columns in a tibble must be vectors.
      x Column `b` is name.
      x Column `c` is NULL.
    Code
      print_error(abort_column_scalar_type(c("", "", LETTERS), 1:28, c("QQ", "VV",
        letters)))
    Output
      <error/tibble_error_column_scalar_type>
      Error:
      ! All columns in a tibble must be vectors.
      x Column 1 is QQ.
      x Column 2 is VV.
      x Column `A` is a.
      x Column `B` is b.
      x Column `C` is c.
      x ... and 23 more problems.
    Code
      print_error(abort_as_tibble_row_vector(new_environment()))
    Output
      <error/tibble_error_as_tibble_row_vector>
      Error:
      ! `x` must be a vector in `as_tibble_row()`, not environment.
    Code
      print_error(abort_as_tibble_row_size_one(3, "foo", 7))
    Output
      <error/tibble_error_as_tibble_row_size_one>
      Error:
      ! All elements must be size one, use `list()` to wrap.
      x Element `foo` is of size 7.
    Code
      # # class-tbl_df
      print_error(abort_names_must_be_non_null())
    Output
      <error/tibble_error_names_must_be_non_null>
      Error:
      ! `names` must not be `NULL`.
    Code
      print_error(abort_names_must_have_length(length = 5, n = 3))
    Output
      <error/tibble_error_names_must_have_length>
      Error:
      ! `names` must have length 3, not 5.
    Code
      # #enframe
      print_error(abort_enframe_value_null())
    Output
      <error/tibble_error_enframe_value_null>
      Error:
      ! `value` can't be NULL.
    Code
      print_error(abort_enframe_must_be_vector(lm(speed ~ ., cars)))
    Output
      <error/tibble_error_enframe_must_be_vector>
      Error:
      ! The `x` argument to `enframe()` must be a vector, not lm.
    Code
      # # names
      print_error(abort_column_names_cannot_be_empty(1, repair_hint = TRUE))
    Output
      <error/tibble_error_column_names_cannot_be_empty>
      Error:
      ! Column 1 must be named.
      Use `.name_repair` to specify repair.
    Code
      print_error(abort_column_names_cannot_be_empty(2:3, repair_hint = TRUE))
    Output
      <error/tibble_error_column_names_cannot_be_empty>
      Error:
      ! Columns 2 and 3 must be named.
      Use `.name_repair` to specify repair.
    Code
      print_error(abort_column_names_cannot_be_empty(seq_along(letters), repair_hint = TRUE))
    Output
      <error/tibble_error_column_names_cannot_be_empty>
      Error:
      ! Columns 1, 2, 3, 4, 5, and 21 more must be named.
      Use `.name_repair` to specify repair.
    Code
      print_error(abort_column_names_cannot_be_empty(4:6, repair_hint = FALSE))
    Output
      <error/tibble_error_column_names_cannot_be_empty>
      Error:
      ! Columns 4, 5, and 6 must be named.
    Code
      print_error(abort_column_names_cannot_be_dot_dot(1, repair_hint = FALSE))
    Output
      <error/tibble_error_column_names_cannot_be_dot_dot>
      Error:
      ! Column 1 must not have names of the form ... or ..j.
    Code
      print_error(abort_column_names_cannot_be_dot_dot(2:3, repair_hint = TRUE))
    Output
      <error/tibble_error_column_names_cannot_be_dot_dot>
      Error:
      ! Columns 2 and 3 must not have names of the form ... or ..j.
      Use `.name_repair` to specify repair.
    Code
      print_error(abort_column_names_cannot_be_dot_dot(1:26, repair_hint = TRUE))
    Output
      <error/tibble_error_column_names_cannot_be_dot_dot>
      Error:
      ! Columns 1, 2, 3, 4, 5, and 21 more must not have names of the form ... or ..j.
      Use `.name_repair` to specify repair.
    Code
      print_error(abort_column_names_must_be_unique("a", repair_hint = FALSE))
    Output
      <error/tibble_error_column_names_must_be_unique>
      Error:
      ! Column name `a` must not be duplicated.
    Code
      print_error(abort_column_names_must_be_unique(letters[2:3], repair_hint = TRUE))
    Output
      <error/tibble_error_column_names_must_be_unique>
      Error:
      ! Column names `b` and `c` must not be duplicated.
      Use `.name_repair` to specify repair.
    Code
      print_error(abort_column_names_must_be_unique(LETTERS, repair_hint = TRUE))
    Output
      <error/tibble_error_column_names_must_be_unique>
      Error:
      ! Column names `A`, `B`, `C`, `D`, `E`, and 21 more must not be duplicated.
      Use `.name_repair` to specify repair.
    Code
      # # new
      print_error(abort_new_tibble_must_be_list())
    Output
      <error/tibble_error_new_tibble_must_be_list>
      Error:
      ! `x` must be a list.
    Code
      print_error(abort_new_tibble_nrow_must_be_nonnegative())
    Output
      <error/tibble_error_new_tibble_nrow_must_be_nonnegative>
      Error:
      ! `nrow` must be a nonnegative whole number smaller than 2^31.
    Code
      # # rownames
      print_error(abort_already_has_rownames())
    Output
      <error/tibble_error_already_has_rownames>
      Error:
      ! `.data` must be a data frame without row names.
    Code
      # # subsetting
      print_error(abort_need_rhs_vector(quote(RHS)))
    Output
      <error/tibble_error_need_rhs_vector>
      Error:
      ! `RHS` must be a vector, a bare list, a data frame or a matrix.
    Code
      print_error(abort_need_rhs_vector_or_null(quote(RHS)))
    Output
      <error/tibble_error_need_rhs_vector_or_null>
      Error:
      ! `RHS` must be a vector, a bare list, a data frame, a matrix, or NULL.
    Code
      print_error(abort_dim_column_index(as.matrix("x")))
    Output
      <error/tibble_error_dim_column_index>
      Error:
      ! Must use a vector in `[`, not an object of class matrix.
    Code
      print_error(abort_assign_columns_non_na_only())
    Output
      <error/tibble_error_assign_columns_non_na_only>
      Error:
      ! Can't use NA as column index in a tibble for assignment.
    Code
      print_error(abort_subset_columns_non_missing_only())
    Output
      <error/tibble_error_subset_columns_non_missing_only>
      Error:
      ! Subscript can't be missing for tibbles in `[[`.
    Code
      print_error(abort_assign_columns_non_missing_only())
    Output
      <error/tibble_error_assign_columns_non_missing_only>
      Error:
      ! Subscript can't be missing for tibbles in `[[<-`.
    Code
      print_error(abort_assign_duplicate_column_subscript(c(1, 1)))
    Output
      <error/tibble_error_assign_duplicate_column_subscript>
      Error:
      ! Column index 1 is used more than once for assignment.
    Code
      print_error(abort_assign_duplicate_column_subscript(c(1, 1, 2, 2)))
    Output
      <error/tibble_error_assign_duplicate_column_subscript>
      Error:
      ! Column indexes 1 and 2 are used more than once for assignment.
    Code
      print_error(abort_assign_rows_non_na_only())
    Output
      <error/tibble_error_assign_rows_non_na_only>
      Error:
      ! Can't use NA as row index in a tibble for assignment.
    Code
      print_error(abort_assign_duplicate_row_subscript(c(1, 1)))
    Output
      <error/tibble_error_assign_duplicate_row_subscript>
      Error:
      ! Row index 1 is used more than once for assignment.
    Code
      print_error(abort_assign_duplicate_row_subscript(c(1, 1, 2, 2)))
    Output
      <error/tibble_error_assign_duplicate_row_subscript>
      Error:
      ! Row indexes 1 and 2 are used more than once for assignment.
    Code
      print_error(abort_assign_incompatible_size(3, list(1:2), 1, NULL, quote(rhs)))
    Output
      <error/tibble_error_assign_incompatible_size>
      Error:
      ! Assigned data `rhs` must be compatible with existing data.
      x Existing data has 3 rows.
      x Assigned data has 2 rows.
      i Only vectors of size 1 are recycled.
    Code
      print_error(abort_assign_incompatible_size(4, list(1:4, 3:4), 2, quote(4:1),
      quote(rhs)))
    Output
      <error/tibble_error_assign_incompatible_size>
      Error:
      ! Assigned data `rhs` must be compatible with row subscript `4:1`.
      x 4 rows must be assigned.
      x Element 2 of assigned data has 2 rows.
      i Only vectors of size 1 are recycled.
    Code
      print_error(abort_assign_incompatible_type(tibble(a = 1), list("c"), 1, quote(
        rhs)))
    Output
      <error/tibble_error_assign_incompatible_type>
      Error:
      ! Assigned data `rhs` must be compatible with existing data.
      i Error occurred for column `a`.
    Code
      print_error(abort_assign_vector(list("c"), 1, quote(rhs)))
    Output
      <error/tibble_error_assign_vector>
      Error:
      ! Assigned data `rhs` must be a vector.
    Code
      # # subsetting-matrix
      print_error(abort_subset_matrix_must_be_logical(quote(is.na(x) + 1)))
    Output
      <error/tibble_error_subset_matrix_must_be_logical>
      Error:
      ! Subscript `is.na(x) + 1` is a matrix, it must be of type logical.
    Code
      print_error(abort_subset_matrix_must_have_same_dimensions(quote(t(is.na(x)))))
    Output
      <error/tibble_error_subset_matrix_must_have_same_dimensions>
      Error:
      ! Subscript `t(is.na(x))` is a matrix, it must have the same dimensions as the input.
    Code
      print_error(abort_subset_matrix_scalar_type(quote(is.na(x)), quote(
        new_environment())))
    Output
      <error/tibble_error_subset_matrix_scalar_type>
      Error:
      ! Subscript `is.na(x)` is a matrix, the data `new_environment()` must be a vector of size 1.
    Code
      print_error(abort_subset_matrix_must_be_scalar(quote(is.na(x)), quote(1:3)))
    Output
      <error/tibble_error_subset_matrix_must_be_scalar>
      Error:
      ! Subscript `is.na(x)` is a matrix, the data `1:3` must have size 1.
    Code
      # # tibble
      print_error(abort_tibble_row_size_one(3, "foo", 7))
    Output
      <error/tibble_error_tibble_row_size_one>
      Error:
      ! All vectors must be size one, use `list()` to wrap.
      x Column `foo` is of size 7.
    Code
      print_error(abort_incompatible_size(10, letters[1:3], c(4, 4, 3),
      "Requested with `uvw` argument"))
    Output
      <error/tibble_error_incompatible_size>
      Error:
      ! Tibble columns must have compatible sizes.
      * Size 10: Requested with `uvw` argument.
      * Size 3: Column `c`.
      * Size 4: Columns `a` and `b`.
      i Only values of size one are recycled.
    Code
      print_error(abort_incompatible_size(10, letters[1:3], c(2, 2, 3),
      "Requested with `xyz` argument"))
    Output
      <error/tibble_error_incompatible_size>
      Error:
      ! Tibble columns must have compatible sizes.
      * Size 10: Requested with `xyz` argument.
      * Size 2: Columns `a` and `b`.
      * Size 3: Column `c`.
      i Only values of size one are recycled.
    Code
      print_error(abort_incompatible_size(NULL, letters[1:3], c(2, 2, 3),
      "Requested with `xyz` argument"))
    Output
      <error/tibble_error_incompatible_size>
      Error:
      ! Tibble columns must have compatible sizes.
      * Size 2: Columns `a` and `b`.
      * Size 3: Column `c`.
      i Only values of size one are recycled.
    Code
      print_error(abort_incompatible_size(10, 1:3, c(4, 4, 3),
      "Requested with `uvw` argument"))
    Output
      <error/tibble_error_incompatible_size>
      Error:
      ! Tibble columns must have compatible sizes.
      * Size 10: Requested with `uvw` argument.
      * Size 3: Column at position 3.
      * Size 4: Columns at positions 1 and 2.
      i Only values of size one are recycled.
    Code
      print_error(abort_incompatible_size(10, 1:3, c(2, 2, 3),
      "Requested with `xyz` argument"))
    Output
      <error/tibble_error_incompatible_size>
      Error:
      ! Tibble columns must have compatible sizes.
      * Size 10: Requested with `xyz` argument.
      * Size 2: Columns at positions 1 and 2.
      * Size 3: Column at position 3.
      i Only values of size one are recycled.
    Code
      print_error(abort_incompatible_size(NULL, 1:3, c(2, 2, 3),
      "Requested with `xyz` argument"))
    Output
      <error/tibble_error_incompatible_size>
      Error:
      ! Tibble columns must have compatible sizes.
      * Size 2: Columns at positions 1 and 2.
      * Size 3: Column at position 3.
      i Only values of size one are recycled.
    Code
      # # tribble
      print_error(abort_tribble_needs_columns())
    Output
      <error/tibble_error_tribble_needs_columns>
      Error:
      ! Must specify at least one column using the `~name` syntax.
    Code
      print_error(abort_tribble_lhs_column_syntax(quote(lhs)))
    Output
      <error/tibble_error_tribble_lhs_column_syntax>
      Error:
      ! All column specifications must use the `~name` syntax.
      x Found `lhs` on the left-hand side of `~`.
    Code
      print_error(abort_tribble_rhs_column_syntax(quote(a + b)))
    Output
      <error/tibble_error_tribble_rhs_column_syntax>
      Error:
      ! All column specifications must use the `~name` or `~"name"` syntax.
      x Found `a + b` on the right-hand side of `~`.
    Code
      print_error(abort_tribble_non_rectangular(5, 17))
    Output
      <error/tibble_error_tribble_non_rectangular>
      Error:
      ! Data must be rectangular.
      * Found 5 columns.
      * Found 17 cells.
      i 17 is not an integer multiple of 5.
    Code
      print_error(abort_frame_matrix_list(2:4))
    Output
      <error/tibble_error_frame_matrix_list>
      Error:
      ! All values must be atomic.
      x Found list-valued elements at positions 2, 3, and 4.

---
title: "Controlling display of numbers"
output: html_vignette
vignette: >
  %\VignetteIndexEntry{Controlling display of numbers}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---



Tibbles print numbers with three significant digits by default, switching to scientific notation if the available space is too small.
Underlines are used to highlight groups of three digits.
The display differs from the default display for data frames, see `vignette("digits")` for an overview over the differences.
This display works for many, but not for all use cases.


```r
library(tibble)
```

## Options

The easiest way to customize the display of numbers and other data in a tibble is to define options.
See `?pillar::pillar_options` for a comprehensive overview.


```r
tibble(x = 123.4567)
#> # A tibble: 1 x 1
#>       x
#>   <dbl>
#> 1  123.
old <- options(pillar.sigfig = 7)
tibble(x = 123.4567)
#> # A tibble: 1 x 1
#>          x
#>      <dbl>
#> 1 123.4567
# Restore old options, see also rlang::local_options() for a more elegant way
options(old)
```

This changes the display of all columns.
Read on to see how too specify display options on a column-by-column basis.


## Per-column number formatting

The new `num()` constructor allows creating vectors that behave like numbers but allow customizing their display.
Below a few examples are shown, see `?num` for a comprehensive overview.
Similarly, `char()` allows customizing the display of character columns.


```r
num(-1:3, notation = "sci")
#> <pillar_num(sci)[5]>
#> [1] -1e0  0    1e0  2e0  3e0
tibble(
  x4 = num(8:12 * 100 + 0.5, digits = 4),
  x1 = num(8:12 * 100 + 0.5, digits = -1),
  usd = num(8:12 * 100 + 0.5, digits = 2, label = "USD"),
  percent = num(8:12 / 100 + 0.0005, label = "%", scale = 100),
  eng = num(10^(-3:1), notation = "eng", fixed_exponent = -Inf),
  si = num(10^(-3:1) * 123, notation = "si"),
  char = char(paste(LETTERS, collapse = " "), shorten = "mid")
)
#> # A tibble: 5 x 7
#>          x4       x1     usd percent   eng      si char                         
#>   <num:.4!> <num:.1>     USD       % [e-3]    <si> <char>                       
#> 1  800.5000    800.5  800.50    8.05     1 123   m A B C D E F G ~ T U V W X Y Z
#> 2  900.5000    900.5  900.50    9.05    10   1.23  A B C D E F G ~ T U V W X Y Z
#> 3 1000.5000   1000.5 1000.50   10.0    100  12.3   A B C D E F G ~ T U V W X Y Z
#> 4 1100.5000   1100.5 1100.50   11.0   1000 123     A B C D E F G ~ T U V W X Y Z
#> 5 1200.5000   1200.5 1200.50   12.0  10000   1.23k A B C D E F G ~ T U V W X Y Z
```

The pillar package that is responsible for the display of tibbles tries hard to get the number display right, however it is impossible to accommodate all use cases.
Whenever the default formatting does not suit the application, `num()` or `char()` allow redefining the formatting for individual columns.
The formatting survives most data transformations.

## Rule-based number formatting

Currently, formatting must be applied manually for each column.
The following pattern may help doing this consistently for all columns in a tibble, or for some columns based on their name.


```r
library(dplyr, warn.conflicts = FALSE)

markets <-
  as_tibble(EuStockMarkets) %>%
  mutate(time = time(EuStockMarkets), .before = 1)

markets
#> # A tibble: 1,860 x 5
#>     time   DAX   SMI   CAC  FTSE
#>    <dbl> <dbl> <dbl> <dbl> <dbl>
#>  1 1991. 1629. 1678. 1773. 2444.
#>  2 1992. 1614. 1688. 1750. 2460.
#>  3 1992. 1607. 1679. 1718  2448.
#>  4 1992. 1621. 1684. 1708. 2470.
#>  5 1992. 1618. 1687. 1723. 2485.
#>  6 1992. 1611. 1672. 1714. 2467.
#>  7 1992. 1631. 1683. 1734. 2488.
#>  8 1992. 1640. 1704. 1757. 2508.
#>  9 1992. 1635. 1698. 1754  2510.
#> 10 1992. 1646. 1716. 1754. 2497.
#> # ... with 1,850 more rows
markets %>%
  mutate(across(-time, ~ num(.x, digits = 3)))
#> # A tibble: 1,860 x 5
#>     time       DAX       SMI       CAC      FTSE
#>    <dbl> <num:.3!> <num:.3!> <num:.3!> <num:.3!>
#>  1 1991.  1628.750  1678.100  1772.800  2443.600
#>  2 1992.  1613.630  1688.500  1750.500  2460.200
#>  3 1992.  1606.510  1678.600  1718.000  2448.200
#>  4 1992.  1621.040  1684.100  1708.100  2470.400
#>  5 1992.  1618.160  1686.600  1723.100  2484.700
#>  6 1992.  1610.610  1671.600  1714.300  2466.800
#>  7 1992.  1630.750  1682.900  1734.500  2487.900
#>  8 1992.  1640.170  1703.600  1757.400  2508.400
#>  9 1992.  1635.470  1697.500  1754.000  2510.500
#> 10 1992.  1645.890  1716.300  1754.300  2497.400
#> # ... with 1,850 more rows
```

## Computing on `num`

Formatting numbers is useful for presentation of results.
If defined early on in the analysis, the formatting options survive most operations.
It is worth defining output options that suit your data once early on in the process, to benefit from the formatting throughout the analysis.
We are working on seamlessly applying this formatting to the final presentation (plots, tables, ...).


### Arithmetics

When applying arithmetic operations on numbers created by `num()`, the result inherits the formatting of the first `num` object.


```r
num(1) + 2
#> <pillar_num[1]>
#> [1] 3
1 + num(2)
#> <pillar_num[1]>
#> [1] 3
1L + num(2)
#> <pillar_num[1]>
#> [1] 3
num(3.23456, sigfig = 4) - num(2)
#> <pillar_num:4[1]>
#> [1] 1.235
num(4, sigfig = 2) * num(3, digits = 2)
#> <pillar_num:2[1]>
#> [1] 12
num(3, digits = 2) * num(4, sigfig = 2)
#> <pillar_num:.2![1]>
#> [1] 12.00
-num(2)
#> <pillar_num[1]>
#> [1] -2
```

### Mathematics

Similarly, for mathematical operations, the formatting is inherited.


```r
min(num(1:3, label = "$"))
#> <pillar_num{$}[1]>
#> [1] 1
mean(num(1:3, notation = "eng"))
#> <pillar_num(eng)[1]>
#> [1] 2e0
sin(num(1:3, label = "%", scale = 100))
#> <pillar_num{%}*100[3]>
#> [1] 84.1 90.9 14.1
```


### Override

In some cases, the ideal formatting changes after a transformation.
`num()` can be applied repeatedly, the last setting wins.


```r
num(1:3 + 0.125, digits = 4)
#> <pillar_num:.4![3]>
#> [1] 1.1250 2.1250 3.1250
transf <- 10^num(1:3 + 0.125, digits = 4)
transf
#> <pillar_num:.4![3]>
#> [1]   13.3352  133.3521 1333.5214
num(transf, sigfig = 3)
#> <pillar_num:3[3]>
#> [1]   13.3  133.  1334.
```

### Recovery

The `var()` function is one of the examples where the formatting is lost:


```r
x <- num(c(1, 2, 4), notation = "eng")
var(x)
#> [1] 2.333333
```

The `median()` function is worse, it breaks for `num()` objects:


```r
median(x)
#> Error in `median()`:
#> ! `median.pillar_num()` not implemented.
```

One way to recover is to apply `num()` to the result:


```r
num(var(x), notation = "eng")
#> <pillar_num(eng)[1]>
#> [1] 2.33e0
num(median(as.numeric(x)), notation = "eng")
#> <pillar_num(eng)[1]>
#> [1] 2e0
```

For automatic recovery, we can also define our version of `var()`, or even overwrite the base implementation.
Note that this pattern is still experimental and may be subject to change:


```r
var_ <- function(x, ...) {
  out <- var(vctrs::vec_proxy(x), ...)
  vctrs::vec_restore(out, x)
}
var_(x)
#> <pillar_num(eng)[1]>
#> [1] 2.33e0
```

This pattern can be applied to all functions that lose the formatting.
The `make_restore()` function defined below is a function factory that consumes a function and returns a derived function:


```r
make_restore <- function(fun) {
  force(fun)
  function(x, ...) {
    out <- fun(vctrs::vec_proxy(x), ...)
    vctrs::vec_restore(out, x)
  }
}

var_ <- make_restore(var)
sd_ <- make_restore(sd)

var_(x)
#> <pillar_num(eng)[1]>
#> [1] 2.33e0
sd_(x)
#> <pillar_num(eng)[1]>
#> [1] 1.53e0
```

---
title: "Comparing display with data frames"
output: html_vignette
vignette: >
  %\VignetteIndexEntry{Comparing display with data frames}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---




```r
library(tibble)
```

Base R offers the `"digits"` and `"scipen"` options to control the number of significant digits and the switch to scientific notation.
For tibble, the options `"pillar.sigfig"` and `"pillar.max_dec_width"` fulfill a similar purpose.
This vignette showcases similarities and differences.
See `?pillar::pillar_options` and `?tibble_options` for an overview over all options.

## Digits

### Basic differences

The default for `getOption("digits")` is 7, whereas the `"pillar.sigfig"` option defaults to 3.
In the default setting, pillar prints the first three digits only (i.e. the digits that represent > 99.9% of the value of the number).
Another difference is that pillar will show at most the specified number of significant digits, even if space is available.
The rationale is to allow a quick glance over the most significant digits of a number, without spending too much horizontal space and without distraction from insignificant digits.


```r
options(digits = 3)
c(1.2345, 12.345, 123.45, 1234.5, 12345)
#> [1]     1.23    12.35   123.45  1234.50 12345.00
tibble(x = c(1.2345, 12.345, 123.45, 1234.5, 12345))
#> # A tibble: 5 x 1
#>          x
#>      <dbl>
#> 1     1.23
#> 2    12.3 
#> 3   123.  
#> 4  1234.  
#> 5 12345
```

### Terminal zeros

Terminal zeros are only shown in pillar if there is a nonzero value past the significant digits shown.
This is in contrast to base R where terminal zeros are always shown if there is space, but hidden if the value is too insignificant:


```r
c(1, 1.00001)
#> [1] 1 1
tibble(x = c(1, 1.00001))
#> # A tibble: 2 x 1
#>       x
#>   <dbl>
#> 1  1   
#> 2  1.00
```

### Trailing dot

A trailing decimal separator is shown if there is a fractional part but the integer part already exceeds the significant digits.
The presence of the decimal separator does **not** indicate that the number is larger, only that there exists a nonzero fractional part:


```r
c(123, 123.45, 567.89)
#> [1] 123 123 568
tibble(x = c(123, 123.45, 567.89))
#> # A tibble: 3 x 1
#>       x
#>   <dbl>
#> 1  123 
#> 2  123.
#> 3  568.
```

### Showing more digits

To show more significant digits, set the `"pillar.sigfig"` option to a larger value:


```r
options(digits = 7)
options(pillar.sigfig = 7)
c(1.2345, 12.345, 123.45, 1234.5, 12345)
#> [1]     1.2345    12.3450   123.4500  1234.5000 12345.0000
tibble(x = c(1.2345, 12.345, 123.45, 1234.5, 12345))
#> # A tibble: 5 x 1
#>            x
#>        <dbl>
#> 1     1.2345
#> 2    12.345 
#> 3   123.45  
#> 4  1234.5   
#> 5 12345
```

Setting `"pillar.sigfig"` to a larger value will not enhance the display with digits deemed insignificant:


```r
options(digits = 7)
options(pillar.sigfig = 7)
c(1.2345, 12.3456, 123.4567, 1234.5678, 12345.6789)
#> [1]     1.2345    12.3456   123.4567  1234.5678 12345.6789
tibble(x = c(1.2345, 12.3456, 123.4567, 1234.5678, 12345.6789))
#> # A tibble: 5 x 1
#>            x
#>        <dbl>
#> 1     1.2345
#> 2    12.3456
#> 3   123.4567
#> 4  1234.568 
#> 5 12345.68
```

### Fixed number of digits

To show a fixed number of decimal digits, use `num()` with a `digits` argument:


```r
num(c(1.2345, 12.345, 123.45, 1234.5, 12345), digits = 2)
#> <pillar_num:.2![5]>
#> [1]     1.23    12.35   123.45  1234.50 12345.00
```

See `vignette("numbers")` for details.

## Scientific notation

### When is it used?

Both base R and pillar switch to scientific notation when the decimal representation becomes too wide.
The larger `getOption("scipen")`, the stronger the resistance to switching to scientific notation.
The default `0` seems to be anchored at 13 digits for the integer part.


```r
123456789012
#> [1] 123456789012
123456789012.3
#> [1] 123456789012
1234567890123
#> [1] 1.234568e+12
1234567890123.4
#> [1] 1.234568e+12
options(scipen = 1)
1234567890123
#> [1] 1234567890123
12345678901234
#> [1] 1.234568e+13
12345678901234.5
#> [1] 1.234568e+13
```

The `"pillar.max_dec_width"` option is similar, it indicates the width that must be exceeded for a switch to scientific notation to happen.
This width includes the decimal separator.


```r
tibble(x = 123456789012)
#> # A tibble: 1 x 1
#>              x
#>          <dbl>
#> 1 123456789012
tibble(x = 123456789012.3)
#> # A tibble: 1 x 1
#>               x
#>           <dbl>
#> 1 123456789012.
tibble(x = 1234567890123)
#> # A tibble: 1 x 1
#>               x
#>           <dbl>
#> 1 1234567890123
tibble(x = 1234567890123.4)
#> # A tibble: 1 x 1
#>             x
#>         <dbl>
#> 1 1.234568e12
options(pillar.max_dec_width = 14)
tibble(x = 1234567890123)
#> # A tibble: 1 x 1
#>               x
#>           <dbl>
#> 1 1234567890123
tibble(x = 12345678901234)
#> # A tibble: 1 x 1
#>                x
#>            <dbl>
#> 1 12345678901234
tibble(x = 12345678901234.5)
#> # A tibble: 1 x 1
#>             x
#>         <dbl>
#> 1 1.234568e13
```

### Enforce notation

To avoid switching to scientific notation, set the `"pillar.max_dec_width"` option to a large value.
Note that if the required width is not available to show the column, it will not be shown at all in this case.
The `notation` argument to `num()` offers more options:


```r
num(12345678901234567, notation = "dec")
#> <pillar_num(dec)[1]>
#> [1] 12345678901234568
num(12345678901234567, notation = "sci")
#> <pillar_num(sci)[1]>
#> [1] 1.234568e16
num(12345678901234567, notation = "eng")
#> <pillar_num(eng)[1]>
#> [1] 12.34568e15
num(12345678901234567, notation = "si")
#> <pillar_num(si)[1]>
#> [1] 12.34568P
```

---
title: "Column formats"
output: rmarkdown::html_vignette
always_allow_html: true
vignette: >
  %\VignetteIndexEntry{Column formats}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---




```r
library(tibble)
```

## Overview

This vignette shows how to decorate columns for custom formatting.
We use the formattable package for demonstration because it already contains useful vector classes that apply a custom formatting to numbers.


```r
library(formattable)

tbl <- tibble(x = digits(9:11, 3))
tbl
#> # A tibble: 3 x 1
#>   x         
#>   <formttbl>
#> 1 9.000     
#> 2 10.000    
#> 3 11.000
```



The `x` column in the tibble above is a regular number with a formatting method.
It always will be shown with three digits after the decimal point.
This also applies to columns derived from `x`.


```r
library(dplyr)
tbl2 <-
  tbl %>%
  mutate(
    y = x + 1,
    z = x * x,
    v = y + z,
    lag = lag(x, default = x[[1]]),
    sin = sin(x),
    mean = mean(v),
    var = var(x)
  )

tbl2
#> # A tibble: 3 x 8
#>   x          y          z          v          lag        sin       mean      var
#>   <formttbl> <formttbl> <formttbl> <formttbl> <formttbl> <formttb> <formt> <dbl>
#> 1 9.000      10.000     81.000     91.000     9.000      0.412     111.667     1
#> 2 10.000     11.000     100.000    111.000    9.000      -0.544    111.667     1
#> 3 11.000     12.000     121.000    133.000    10.000     -1.000    111.667     1
```

Summaries also maintain the formatting.


```r
tbl2 %>%
  group_by(lag) %>%
  summarize(z = mean(z)) %>%
  ungroup()
#> # A tibble: 2 x 2
#>   lag        z         
#>   <formttbl> <formttbl>
#> 1 9.000      90.500    
#> 2 10.000     121.000
```

Same for pivoting operations.



```r
library(tidyr)

stocks <-
  expand_grid(id = factor(1:4), year = 2018:2022) %>%
  mutate(stock = currency(runif(20) * 10000))

stocks %>%
  pivot_wider(id_cols = id, names_from = year, values_from = stock)
#> # A tibble: 4 x 6
#>   id    `2018`     `2019`     `2020`     `2021`     `2022`    
#>   <fct> <formttbl> <formttbl> <formttbl> <formttbl> <formttbl>
#> 1 1     $4,197.73  $7,414.63  $5,507.74  $9,256.42  $6,599.49 
#> 2 2     $4,393.70  $9,315.85  $1,076.86  $9,781.00  $8,681.69 
#> 3 3     $4,058.69  $460.28    $3,214.04  $927.91    $453.61   
#> 4 4     $6,247.89  $6,507.82  $3,578.43  $198.98    $4,518.97
```

For ggplot2 we need to do [some work](https://github.com/tidyverse/ggplot2/pull/4065) to show apply the formatting to the scales.


```r
library(ggplot2)

# Needs https://github.com/tidyverse/ggplot2/pull/4065 or similar
stocks %>%
  ggplot(aes(x = year, y = stock, color = id)) +
  geom_line()
```

It pays off to specify formatting very early in the process.
The diagram below shows the principal stages of data analysis and exploration from "R for data science".



The subsequent diagram adds data formats, communication options, and explicit data formatting.
The original r4ds transitions are highlighted in bold.
There are two principal options where to apply formatting for results: right before communicating them, or right after importing.



Applying formatting early in the process gives the added benefit of showing the data in a useful format during the "Tidy", "Transform", and "Visualize" stages.
For this to be useful, we need to ensure that the formatting options applied early:

- give a good user experience for analysis
    - are easy to set up
    - keep sticky in the process of data analysis and exploration
    - support the analyst in asking the right questions about the data
    - convey the critical information at a glance, with support to go into greater detail easier
- look good for communication
    - are applied in the various communication options
    - support everything necessary to present the data in the desired way

Ensuring stickiness is difficult, and is insufficient for a dbplyr workflow where parts of the "Tidy", "Transform" or even "Visualize" stages are run on the database.
Often it's possible to derive a rule-based approach for formatting.


```r
tbl3 <-
  tibble(id = letters[1:3], x = 9:11) %>%
  mutate(
    y = x + 1,
    z = x * x,
    v = y + z,
    lag = lag(x, default = x[[1]]),
    sin = sin(x),
    mean = mean(v),
    var = var(x)
  )

tbl3
#> # A tibble: 3 x 9
#>   id        x     y     z     v   lag    sin  mean   var
#>   <chr> <int> <dbl> <int> <dbl> <int>  <dbl> <dbl> <dbl>
#> 1 a         9    10    81    91     9  0.412  112.     1
#> 2 b        10    11   100   111     9 -0.544  112.     1
#> 3 c        11    12   121   133    10 -1.00   112.     1

tbl3 %>%
  mutate(
    across(where(is.numeric), ~ digits(.x, 3)),
    across(where(~ is.numeric(.x) && mean(.x) > 50), ~ digits(.x, 1))
  )
#> # A tibble: 3 x 9
#>   id    x          y          z          v          lag       sin    mean  var  
#>   <chr> <formttbl> <formttbl> <formttbl> <formttbl> <formttb> <form> <for> <for>
#> 1 a     9.000      10.000     81.0       91.0       9.000     0.412  111.7 1.000
#> 2 b     10.000     11.000     100.0      111.0      9.000     -0.544 111.7 1.000
#> 3 c     11.000     12.000     121.0      133.0      10.000    -1.000 111.7 1.000
```

These rules can be stored in `quos()`:


```r
rules <- quos(
  across(where(is.numeric), ~ digits(.x, 3)),
  across(where(~ is.numeric(.x) && mean(.x) > 50), ~ digits(.x, 1))
)

tbl3 %>%
  mutate(!!!rules)
#> # A tibble: 3 x 9
#>   id    x          y          z          v          lag       sin    mean  var  
#>   <chr> <formttbl> <formttbl> <formttbl> <formttbl> <formttb> <form> <for> <for>
#> 1 a     9.000      10.000     81.0       91.0       9.000     0.412  111.7 1.000
#> 2 b     10.000     11.000     100.0      111.0      9.000     -0.544 111.7 1.000
#> 3 c     11.000     12.000     121.0      133.0      10.000    -1.000 111.7 1.000
```

This poses a few drawbacks:

- The syntax is repetitive and not very intuitive
- Rules that match multiple columns must be given in reverse order due to the way `mutate()` works, and are executed multiple times

What would a good API for rule-based formatting look like?

# output test

    Code
      # # Problems
      writeLines(format_error_bullets(problems("header", c("item 1", "item 2"))))
    Output
      header.
      x item 1.
      x item 2.
    Code
      writeLines(format_error_bullets(problems("header", LETTERS)))
    Output
      header.
      x A.
      x B.
      x C.
      x D.
      x E.
      x ... and 21 more problems.
    Code
      writeLines(format_error_bullets(problems("header", as.character(1:6))))
    Output
      header.
      x 1.
      x 2.
      x 3.
      x 4.
      x 5.
      x ... and 1 more problem.
    Code
      # # Bullets
      writeLines(format_error_bullets(bullets("header", c("item 1", "item 2"))))
    Output
      header.
      * item 1.
      * item 2.
    Code
      writeLines(format_error_bullets(bullets("header", LETTERS)))
    Output
      header.
      * A.
      * B.
      * C.
      * D.
      * E.
      * F.
      * G.
      * H.
      * I.
      * J.
      * K.
      * L.
      * M.
      * N.
      * O.
      * P.
      * Q.
      * R.
      * S.
      * T.
      * U.
      * V.
      * W.
      * X.
      * Y.
      * Z.
    Code
      writeLines(format_error_bullets(bullets("header", as.character(1:6))))
    Output
      header.
      * 1.
      * 2.
      * 3.
      * 4.
      * 5.
      * 6.
    Code
      # # Commas
      commas("1")
    Output
      [1] "1"
    Code
      commas(letters[2:4])
    Output
      [1] "b, c, and d"
    Code
      commas(LETTERS)
    Output
      [1] "A, B, C, D, E, and 21 more"

# output test

    Code
      add_row(tibble(), a = 1)
    Condition
      Error in `add_row()`:
      ! New rows can't add columns.
      x Can't find column `a` in `.data`.
    Code
      add_row(tibble(), a = 1, b = 2)
    Condition
      Error in `add_row()`:
      ! New rows can't add columns.
      x Can't find columns `a` and `b` in `.data`.
    Code
      add_row(tibble(), !!!set_names(letters))
    Condition
      Error in `add_row()`:
      ! New rows can't add columns.
      x Can't find columns `a`, `b`, `c`, `d`, `e`, and 21 more in `.data`.
    Code
      add_row(dplyr::group_by(tibble(a = 1), a))
    Condition
      Error in `add_row()`:
      ! Can't add rows to grouped data frames.
    Code
      add_row(tibble(a = 1), a = 2, .before = 1, .after = 1)
    Condition
      Error in `add_row()`:
      ! Can't specify both `.before` and `.after`.
    Code
      add_column(tibble(a = 1), a = 1)
    Condition
      Error in `add_column()`:
      ! Column name `a` must not be duplicated.
      Use `.name_repair` to specify repair.
      Caused by error in `repaired_names()`:
      ! Names must be unique.
      x These names are duplicated:
        * "a" at locations 1 and 2.
    Code
      add_column(tibble(a = 1, b = 2), a = 1, b = 2)
    Condition
      Error in `add_column()`:
      ! Column names `a` and `b` must not be duplicated.
      Use `.name_repair` to specify repair.
      Caused by error in `repaired_names()`:
      ! Names must be unique.
      x These names are duplicated:
        * "a" at locations 1 and 3.
        * "b" at locations 2 and 4.
    Code
      add_column(tibble(!!!set_names(letters)), !!!set_names(letters))
    Condition
      Error in `add_column()`:
      ! Column names `a`, `b`, `c`, `d`, `e`, and 21 more must not be duplicated.
      Use `.name_repair` to specify repair.
      Caused by error in `repaired_names()`:
      ! Names must be unique.
      x These names are duplicated:
        * "a" at locations 1 and 27.
        * "b" at locations 2 and 28.
        * "c" at locations 3 and 29.
        * "d" at locations 4 and 30.
        * "e" at locations 5 and 31.
        * ...
    Code
      add_column(tibble(a = 2:3), b = 4:6)
    Condition
      Error in `add_column()`:
      ! New columns must be compatible with `.data`.
      x New column has 3 rows.
      i `.data` has 2 rows.
    Code
      add_column(tibble(a = 1), b = 1, .before = 1, .after = 1)
    Condition
      Error in `add_column()`:
      ! Can't specify both `.before` and `.after`.

---
title: "Extending tibble"
author: "Kirill Müller, Hadley Wickham"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Extending tibble}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

Tibbles are S3 objects of class `c("tbl_df", "tbl", "data.frame")`.
This means that they inherit their behavior from the `"data.frame"` class and add two subclasses `"tbl"` and `"tbl_df"`.
The pillar and tibble packages provide methods for `"tbl"` and `"tbl_df"`, respectively.
Package authors and programmers can implement subclasses that extend either `"tbl_df"` (and its subclasses) or only `"tbl"` to provide custom behavior for tibble-like objects.
In addition, vectors classes can be implemented from scratch or on top of existing classes, to be used as columns.
This article provides links to the various customization points that help you avoiding reimplement existing behavior, and describes the difference between `"tbl"` and `"tbl_df"`.

Read more in [the second edition of Advanced R](https://adv-r.hadley.nz/index.html):

- about the internal representation of data frames and tibbles in the [Lists](https://adv-r.hadley.nz/vectors-chap.html#lists) and [Data frames and tibbles](https://adv-r.hadley.nz/vectors-chap.html#tibble) sections of the [Vectors chapter](https://adv-r.hadley.nz/vectors-chap.html),
- about R's S3 object-oriented system in the [S3 chapter](https://adv-r.hadley.nz/s3.html).

## Topics documented elsewhere

- Change or tweak the way a tibble prints: `vignette("extending", package = "pillar")`

- Change or tweak the way a column type is printed in a tibble: `vignette("pillar", package = "vctrs")`

- Implement a new column data type: `vignette("s3-vector", package = "vctrs")`

- Making your tibble subclass work well with dplyr: `?dplyr::dplyr_extending`

## Data frame subclasses


```r
library(tibble)
```

For tibble >= 3.0.0, the `"tbl"` class is responsible for printing, while the `"tbl_df"` class adds tibble's sturdy subsetting and subset assignment behavior (see `vignette("invariants")` for details).
This means that a data frame class that would like to (mostly) print like a tibble but keep the behavior from base R's data frames can inherit from `c("tbl", "data.frame")` or just from `"tbl"`.
An example is the `"tbl_sql"` class in the dbplyr package that is used to implement lazy database tables.

### Tibble example


```r
my_tbl_df <- new_tibble(
  list(a = 1:3, b = 2:4),
  class = "my_tbl_df"
)

tbl_sum.my_tbl_df <- function(x, ...) {
  c(
    "My custom tibble" = "Some info about it",
    NextMethod()
  )
}

my_tbl_df
```

```
## # A tibble: 3 x 2
##       a     b
##   <int> <int>
## 1     1     2
## 2     2     3
## 3     3     4
```

```r
my_tbl_df[, "a"]
```

```
## # A tibble: 3 x 1
##       a
##   <int>
## 1     1
## 2     2
## 3     3
```

### Data frame example


```r
my_tbl <- vctrs::new_data_frame(
  list(a = 1:3, b = 2:4),
  class = c("my_tbl", "tbl")
)

tbl_sum.my_tbl <- function(x, ...) {
  c(
    "My custom data frame" = "Some info about it",
    NextMethod()
  )
}

my_tbl
```

```
## # A data frame: 3 x 2
##       a     b
##   <int> <int>
## 1     1     2
## 2     2     3
## 3     3     4
```

```r
my_tbl[, "a"]
```

```
## [1] 1 2 3
```

---
title: "Invariants: Comparing behavior with data frames"
#output: rmarkdown::word_document
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Invariants: Comparing behavior with data frames}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

<style type="text/css">
.dftbl {
    width: 100%;
    table-layout: fixed;
    display: inline-table;
}

.error pre code {
    color: red;
}

.warning pre code {
    color: violet;
}
</style>

```{r}
#| include: false
# To suppress messages
library(tibble)
library(vctrs)

knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  error = TRUE
)
tibble:::set_dftbl_hooks()

options(
  lifecycle_verbosity = "warning",
  lifecycle_disable_warnings = FALSE,
  lifecycle_verbose_soft_deprecation = TRUE,
  lifecycle_repeat_warnings = TRUE
)

# Set to FALSE for production
eval_details <- (Sys.getenv("IN_GALLEY") != "")
```

This vignette defines invariants for subsetting and subset-assignment for tibbles, and illustrates where their behaviour differs from data frames.
The goal is to define a small set of invariants that consistently define how behaviors interact.
Some behaviors are defined using functions of the vctrs package, e.g. `vec_slice()`, `vec_recycle()` and `vec_as_index()`.
Refer to their documentation for more details about the invariants that they follow.

The subsetting and subassignment operators for data frames and tibbles are particularly tricky, because they support both row and column indexes, both of which are optionally missing.
We resolve this by first defining column access with `[[` and `$`, then column-wise subsetting with `[`, then row-wise subsetting, then the composition of both.

## Conventions

In this article, all behaviors are demonstrated using one example data frame and its tibble equivalent:

```{r}
#| label: setup
library(tibble)
library(vctrs)

new_df <- function() {
  df <- data.frame(n = c(1L, NA, 3L, NA))
  df$c <- letters[5:8]
  df$li <- list(9, 10:11, 12:14, "text")
  df
}

new_tbl <- function() {
  as_tibble(new_df())
}
```

Results of the same code for data frames and tibbles are presented side by side:

```{r}
#| label: show
#| dftbl: true
#| dftbl_always: true
new_df()
```

If the results are identical (after converting to a data frame if necessary), only the tibble result is shown.

Subsetting operations are read-only.
The same objects are reused in all examples:

```{r}
#| label: ro
df <- new_df()
tbl <- new_tbl()
```

Where needed, we also show examples with hierarchical columns containing a data frame or a matrix:

```{r}
#| label: setup2
new_tbl2 <- function() {
  tibble(
    tb = tbl,
    m = diag(4)
  )
}

new_df2 <- function() {
  df2 <- new_tbl2()
  class(df2) <- "data.frame"
  class(df2$tb) <- "data.frame"
  df2
}

df2 <- new_df2()
tbl2 <- new_tbl2()
```

```{r}
#| label: show-compare-2
#| dftbl: true
new_df()
```

For subset assignment (subassignment, for short), we need a fresh copy of the data for each test.
The `with_*()` functions (omitted here for brevity) allow for a more concise notation.
These functions take an assignment expression, execute it on a fresh copy of the data, and return the data for printing.
The first example prints what's really executed, further examples omit this output.

```{r}
#| label: with-def
#| include: false
with_df <- function(code, verbose = FALSE) {
  code <- rlang::enexpr(code)

  full_code <- rlang::quo({
    df <- new_df()
    !!code
    df
  })
  if (verbose) rlang::expr_print(rlang::quo_get_expr(full_code))
  rlang::eval_tidy(full_code)
}

with_tbl <- function(code, verbose = FALSE) {
  code <- rlang::enexpr(code)

  full_code <- rlang::quo({
    tbl <- new_tbl()
    !!code
    tbl
  })
  if (verbose) rlang::expr_print(rlang::quo_get_expr(full_code))
  rlang::eval_tidy(full_code)
}

with_df2 <- function(code) {
  code <- rlang::enexpr(code)

  full_code <- rlang::quo({
    df2 <- new_df2()
    !!code
    df2
  })
  rlang::eval_tidy(full_code)
}

with_tbl2 <- function(code) {
  code <- rlang::enexpr(code)

  full_code <- rlang::quo({
    tbl2 <- new_tbl2()
    !!code
    tbl2
  })
  rlang::eval_tidy(full_code)
}
```

```{r}
#| label: with-demo
#| dftbl: true
with_df(df$n <- rev(df$n), verbose = TRUE)
```

## Column extraction

### Definition of `x[[j]]`

`x[[j]]` is equal to `.subset2(x, j)`.

```{r}
#| label: double-bracket-equivalent-to-subset2
#| dftbl: true
df[[1]]
.subset2(df, 1)
```


```{r}
#| label: double-bracket-equivalent-to-subset2-detail
#| dftbl: true
#| include: !expr eval_details
#| eval: !expr eval_details
identical(df[[3]], .subset2(df, 3))
identical(df2[["df"]], .subset2(df2, "df"))
```

NB: `x[[j]]` always returns an object of size `nrow(x)` if the column exists.

```{r}
#| label: double-bracket-retains-size
#| dftbl: true
#| include: !expr eval_details
#| eval: !expr eval_details
vec_size(df[[1]])
vec_size(df[[3]])
vec_size(df2[[1]])
vec_size(df2[[2]])
```

`j` must be a single number or a string, as enforced by `.subset2(x, j)`.

```{r}
#| label: double-bracket-requires-scalar-j-index
#| dftbl: true
df[[1:2]]
df[[c("n", "c")]]
df[[TRUE]]
df[[mean]]
```

`NA` indexes, numeric out-of-bounds (OOB) values, and non-integers throw an error:

```{r}
#| label: double-bracket-j-oob-numeric
#| dftbl: true
df[[NA]]
df[[NA_character_]]
df[[NA_integer_]]
df[[-1]]
df[[4]]
df[[1.5]]
df[[Inf]]
```

Character OOB access is silent because a common package idiom is to check for the absence of a column with `is.null(df[[var]])`.

```{r}
#| label: double-bracket-j-oob-character
#| dftbl: true
df[["x"]]
```

### Definition of `x$name`

`x$name` and `x$"name"` are equal to `x[["name"]]`.

```{r}
#| label: dollar-equivalent-to-subset
#| dftbl: true
df$n
df$"n"
df[["n"]]
```

```{r}
#| label: dollar-equivalent-to-subset-detail
#| dftbl: true
#| include: !expr eval_details
#| eval: !expr eval_details
identical(df$li, df[["li"]])
identical(df2$tb, df2[["tb"]])
identical(df2$m, df2[["m"]])
```

Unlike data frames, tibbles do not partially match names.
Because `df$x` is rarely used in packages, it can raise a warning:

```{r}
#| label: dollar-equivalent-to-subset-pmatch
#| dftbl: true
df$l
df$not_present
```

## Column subsetting

### Definition of `x[j]`

`j` is converted to an integer vector by `vec_as_index(j, ncol(x), names = names(x))`.
Then `x[c(j_1, j_2, ..., j_n)]` is equivalent to `tibble(x[[j_1]], x[[j_2]], ..., x[[j_n]])`, keeping the corresponding column names.
This implies that `j` must be a numeric or character vector, or a logical vector with length 1 or `ncol(x)`.[^subset-extract-commute]

[^subset-extract-commute]: `x[j][[jj]]` is equal to `x[[ j[[jj]] ]]`, in particular `x[j][[1]]` is equal to `x[[j]]` for scalar numeric or integer `j`.


```{r}
#| label: bracket-j-definition
#| dftbl: true
df[1:2]
```

When subsetting repeated indexes, the resulting column names are undefined, do not rely on them.

```{r}
#| label: bracket-j-duplication
#| dftbl: true
df[c(1, 1)]
```

For tibbles with repeated column names, subsetting by name uses the first matching column.

`nrow(df[j])` equals `nrow(df)`.

```{r}
#| label: bracket-j-empty
#| dftbl: true
df[integer()]
```

Tibbles support indexing by a logical matrix, but only if all values in the returned vector are compatible.

```{r}
#| label: bracket-j-logical-matrix
#| dftbl: true
df[is.na(df)]
df[!is.na(df)]
```

### Definition of `x[, j]`

`x[, j]` is equal to `x[j]`.
Tibbles do not perform column extraction if `x[j]` would yield a single column.

```{r}
#| label: bracket-missing-i
#| dftbl: true
df[, 1]
df[, 1:2]
```

```{r}
#| label: bracket-missing-i-detail
#| dftbl: true
#| include: !expr eval_details
#| eval: !expr eval_details
identical(df[, 2:3], df[2:3])
identical(df2[, 1:2], df2[1:2])
```

### Definition of `x[, j, drop = TRUE]`

For backward compatiblity, `x[, j, drop = TRUE]` performs column __extraction__, returning `x[j][[1]]` when `ncol(x[j])` is 1.

```{r}
#| label: bracket-always-returns-tibble-drop
#| dftbl: true
df[, 1, drop = TRUE]
```

```{r}
#| label: bracket-always-returns-tibble-drop-detail
#| dftbl: true
#| include: !expr eval_details
#| eval: !expr eval_details
identical(df[, 3, drop = TRUE], df[[3]])
identical(df2[, 1, drop = TRUE], df2[[1]])
identical(df2[, 2, drop = TRUE], df2[[2]])
```

## Row subsetting

### Definition of `x[i, ]`

`x[i, ]` is equal to `tibble(vec_slice(x[[1]], i), vec_slice(x[[2]], i), ...)`.[^row-subset-efficiency]

[^row-subset-efficiency]: Row subsetting `x[i, ]` is not defined in terms of `x[[j]][i]` because that definition does not generalise to matrix and data frame columns.
For efficiency and backward compatibility, `i` is converted to an integer vector by `vec_as_index(i, nrow(x))` first.

```{r}
#| label: bracket-i
#| dftbl: true
df[3, ]
```

This means that `i` must be a numeric vector, or a logical vector of length `nrow(x)` or 1.
For compatibility, `i` can also be a character vector containing positive numbers.

```{r}
#| label: bracket-i-wrong-type
#| dftbl: true
df[mean, ]
df[list(1), ]
df["1", ]
```

Exception: OOB values generate warnings instead of errors:

```{r}
#| label: bracket-i-oob
#| dftbl: true
df[10, ]
df["x", ]
```


Unlike data frames, only logical vectors of length 1 are recycled.
<!-- TODO: should this be an error? #648 -->

```{r}
#| label: bracket-i-recycle
#| dftbl: true
df[c(TRUE, FALSE), ]
```

NB: scalar logicals are recycled, but scalar numerics are not.
That makes the `x[NA, ]` and `x[NA_integer_, ]` return different results.

```{r}
#| label: bracket-i-na
#| dftbl: true
df[NA, ]
df[NA_integer_, ]
```

### Definition of `x[i, , drop = TRUE]`

`drop = TRUE` has no effect when not selecting a single row:

```{r}
#| label: bracket-i-drop
#| dftbl: true
df[1, , drop = TRUE]
```

<!-- TODO: soft-deprecate -->

## Row and column subsetting

### Definition of `x[]` and `x[,]`

`x[]` and `x[,]` are equivalent to `x`.[^bracket-comma]

[^bracket-comma]: `x[,]` is equivalent to `x[]` because `x[, j]` is equivalent to `x[j]`.

### Definition of `x[i, j]`

`x[i, j]` is equal to `x[i, ][j]`.[^bracket-flip]

[^bracket-flip]: A more efficient implementation of `x[i, j]` would forward to `x[j][i, ]`.

```{r}
#| label: bracket-i-j-equivalent-to-i-subset-then-j
#| dftbl: true
#| include: !expr eval_details
#| eval: !expr eval_details
df[1, 1]
df[1, ][1]
identical(df[1, 2:3], df[2:3][1, ])
identical(df[2:3, 1], df[1][2:3, ])
identical(df2[2:3, 1:2], df2[1:2][2:3, ])
```


### Definition of `x[[i, j]]`

`i` must be a numeric vector of length 1.
`x[[i, j]]` is equal to `x[i, ][[j]]`, or `vctrs::vec_slice(x[[j]], i)`.[^bracket2-flip]

[^bracket2-flip]: Cell subsetting `x[[i, j]]` is not defined in terms of `x[[j]][[i]]` because that definition does not generalise to list, matrix and data frame columns.
A more efficient implementation of `x[[i, j]]` would check that `j` is a scalar and forward to `x[i, j][[1]]`.

```{r}
#| label: bracket-bracket-i-j-equivalent-to-i-subset-then-j
df[[1, 1]]
df[[1, 3]]
```

This implies that `j` must be a numeric or character vector of length 1.

NB: `vec_size(x[[i, j]])` always equals 1.
Unlike `x[i, ]`, `x[[i, ]]` is not valid.

## Column update

### Definition of `x[[j]] <- a`

If `a` is a vector then `x[[j]] <- a` replaces the `j`th column with value `a`.

```{r}
#| label: double-bracket-assign-definition
#| dftbl: true
#| include: !expr eval_details
#| eval: !expr eval_details
with_df(df[[1]] <- 0)
with_df(df[[3]] <- 4:1)
with_df2(df2[[1]] <- 0)
with_df2(df2[[2]] <- 4:1)
```

```{r}
#| label: double-bracket-assign-requires-scalar-j-index
#| dftbl: true
#| include: !expr eval_details
#| eval: !expr eval_details
with_df(df[[1]] <- 0)
with_df(df[["c"]] <- 0)
```

```{r}
#| label: double-bracket-assign-requires-scalar-j-index-error
#| dftbl: true
#| include: !expr eval_details
#| eval: !expr eval_details
with_df(df[[TRUE]] <- 0)
with_df(df[[1:3]] <- 0)
with_df(df[[c("n", "c")]] <- 0)
with_df(df[[FALSE]] <- 0)
with_df(df[[1:2]] <- 0)
with_df(df[[NA_integer_]] <- 0)
with_df(df[[NA]] <- 0)
with_df(df[[NA_character_]] <- 0)
```

`a` is recycled to the same size as `x` so must have size `nrow(x)` or 1.
(The only exception is when `a` is `NULL`, as described below.)
Recycling also works for list, data frame, and matrix columns.

```{r}
#| label: double-bracket-assign-recycle
#| dftbl: true
with_df(df[["li"]] <- list(0))
with_df2(df2[["tb"]] <- df[1, ])
with_df2(df2[["m"]] <- df2[["m"]][1, , drop = FALSE])
```

```{r}
#| label: double-bracket-requires-size
#| dftbl: true
#| include: !expr eval_details
#| eval: !expr eval_details
with_df(df[[1]] <- 1)
with_df(df[[1]] <- 4:1)
with_df(df[[1]] <- 3:1)
with_df(df[[1]] <- 2:1)
```

`j` must be a scalar numeric or a string, and cannot be `NA`.
If `j` is OOB, a new column is added on the right hand side, with name repair if needed.

```{r}
#| label: double-bracket-assign-supports-new
#| dftbl: true
with_df(df[["x"]] <- 0)
with_df(df[[4]] <- 0)
with_df(df[[5]] <- 0)
```

<!-- HW: should we permitted oob assignment with numeric j? It's a bit weird to create a column with unknonw column -->

`df[[j]] <- a` replaces the complete column so can change the type.

```{r}
#| label: double-bracket-assign-supports-type-change
#| dftbl: true
#| include: !expr eval_details
#| eval: !expr eval_details
with_df(df[[1]] <- df[[2]])
with_df(df[[2]] <- df[[3]])
with_df(df[[3]] <- df2[[1]])
with_df2(df2[[1]] <- df2[[2]])
with_df2(df2[[2]] <- df[[1]])
```

`[[<-` supports removing a column by assigning `NULL` to it.

```{r}
#| label: double-bracket-assign-supports-null
#| dftbl: true
#| include: !expr eval_details
#| eval: !expr eval_details
with_df(df[[1]] <- NULL)
with_df2(df2[[2]] <- NULL)
```

Removing a nonexistent column is a no-op.

```{r}
#| label: double-bracket-assign-supports-null-unknown
#| dftbl: true
#| include: !expr eval_details
#| eval: !expr eval_details
with_df(df[["q"]] <- NULL)
```

### Definition of `x$name <- a`

`x$name <- a` and `x$"name" <- a` are equivalent to `x[["name"]] <- a`.[^column-assign-symmetry]

[^column-assign-symmetry]: `$` behaves almost completely symmetrically to `[[` when comparing subsetting and subassignment.

```{r}
#| label: dollar-equivalent-to-subset-assign
#| dftbl: true
with_df(df$n <- 0)
with_df(df[["n"]] <- 0)
```

```{r}
#| label: dollar-equivalent-to-subset-assign-detail
#| dftbl: true
#| include: !expr eval_details
#| eval: !expr eval_details
with_df(df$"n" <- 0)
```

`$<-` does not perform partial matching.

```{r}
#| label: dollar-equivalent-to-subset-assign-pmatch
#| dftbl: true
with_df(df$l <- 0)
with_df(df[["l"]] <- 0)
```

## Column subassignment: `x[j] <- a`

* If `j` is missing, it's replaced with `seq_along(x)`
* If `j` is logical vector, it's converted to numeric with `seq_along(x)[j]`.

### `a` is a list or data frame

If `inherits(a, "list")` or `inherits(a, "data.frame")` is `TRUE`, then `x[j] <- a` is equivalent to `x[[j[[1]]] <- a[[1]]`, `x[[j[[2]]]] <- a[[2]]`, ...

```{r}
#| label: bracket-assign-def
#| dftbl: true
with_df(df[1:2] <- list("x", 4:1))
with_df(df[c("li", "x", "c")] <- list("x", 4:1, NULL))
```

If `length(a)` equals 1, then it is recycled to the same length as `j`.

```{r}
#| label: bracket-assign-recycles
#| dftbl: true
with_df(df[1:2] <- list(1))
with_df(df[1:2] <- list(0, 0, 0))
with_df(df[1:3] <- list(0, 0))
```

An attempt to update the same column twice gives an error.

```{r}
#| label: bracket-assign-multiple
#| dftbl: true
with_df(df[c(1, 1)] <- list(1, 2))
```

If `a` contains `NULL` values, the corresponding columns are removed *after* updating (i.e. position indexes refer to columns before any modifications).

```{r}
#| label: bracket-assign-remove
#| dftbl: true
with_df(df[1:2] <- list(NULL, 4:1))
```

`NA` indexes are not supported.

```{r}
#| label: bracket-assign-na
#| dftbl: true
#| include: !expr eval_details
#| eval: !expr eval_details
with_df(df[NA] <- list("x"))
with_df(df[NA_integer_] <- list("x"))
with_df(df[NA_character_] <- list("x"))
```

Just like column updates, `[<-` supports changing the type of an existing column.

```{r}
#| label: bracket-assign-supports-type-change
#| dftbl: true
#| include: !expr eval_details
#| eval: !expr eval_details
with_df(df[1] <- df[2])
with_df(df[2] <- df[3])
with_df(df[3] <- df2[1])
with_df2(df2[1] <- df2[2])
with_df2(df2[2] <- df[1])
```

Appending columns at the end (without gaps) is supported.
The name of new columns is determined by the LHS, the RHS, or by name repair (in that order of precedence).

```{r}
#| label: bracket-assign-names
#| dftbl: true
with_df(df[c("x", "y")] <- tibble("x", x = 4:1))
with_df(df[3:4] <- list("x", x = 4:1))
with_df(df[4] <- list(4:1))
with_df(df[5] <- list(4:1))
```

Tibbles support indexing by a logical matrix, but only for a scalar RHS, and if all columns updated are compatible with the value assigned.

```{r}
#| label: bracket-j-assign-logical-matrix
#| dftbl: true
with_df(df[is.na(df)] <- 4)
with_df(df[is.na(df)] <- 1:2)
with_df(df[matrix(c(rep(TRUE, 5), rep(FALSE, 7)), ncol = 3)] <- 4)
```

### `a` is a matrix or array

If `is.matrix(a)`, then `a` is coerced to a data frame with `as.data.frame()` before assigning.
If rows are assigned, the matrix type must be compatible with all columns.
If `is.array(a)` and `any(dim(a)[-1:-2] != 1)`, an error is thrown.

```{r}
#| label: bracket-assign-array
#| dftbl: true
with_df(df[1:2] <- matrix(8:1, ncol = 2))
with_df(df[1:3, 1:2] <- matrix(6:1, ncol = 2))
with_df(df[1:2] <- array(4:1, dim = c(4, 1, 1)))
with_df(df[1:2] <- array(8:1, dim = c(4, 2, 1)))
with_df(df[1:2] <- array(8:1, dim = c(2, 1, 4)))
with_df(df[1:2] <- array(8:1, dim = c(4, 1, 2)))
```

### `a` is another type of vector

If `vec_is(a)`, then `x[j] <- a` is equivalent to `x[j] <- list(a)`.
This is primarily provided for backward compatibility.

```{r}
#| label: bracket-assign-wraps
#| dftbl: true
with_df(df[1] <- 0)
with_df(df[1] <- list(0))
```

Matrices must be wrapped in `list()` before assignment to create a matrix column.

```{r}
#| label: bracket-assign-matrix
#| dftbl: true
with_df(df[1] <- list(matrix(1:8, ncol = 2)))

with_df(df[1:2] <- list(matrix(1:8, ncol = 2)))
```

### `a` is `NULL`

Entire columns can be removed.
Specifying `i` is an error.

```{r}
#| label: bracket-assign-null
#| dftbl: true
with_df(df[1] <- NULL)
with_df(df[, 2:3] <- NULL)
with_df(df[1, 2:3] <- NULL)
```

### `a` is not a vector

Any other type for `a` is an error.
Note that if `is.list(a)` is `TRUE`, but `inherits(a, "list")` is `FALSE`, then `a` is considered to be a scalar.
See `?vec_is` and `?vec_proxy` for details.

```{r}
#| label: bracket-assign-non-vector
#| dftbl: true
with_df(df[1] <- mean)
with_df(df[1] <- lm(mpg ~ wt, data = mtcars))
```

<!-- HW: we need better error messages for these cases -->

## Row subassignment: `x[i, ] <- list(...)`

`x[i, ] <- a` is the same as `vec_slice(x[[j_1]], i) <- a[[1]]`,  `vec_slice(x[[j_2]], i) <- a[[2]]`, ... .[^row-assign-symmetry]

[^row-assign-symmetry]: `x[i, ]` is symmetrical for subset and subassignment.

```{r}
#| label: bracket-i-assign
#| dftbl: true
with_df(df[2:3, ] <- df[1, ])
with_df(df[c(FALSE, TRUE, TRUE, FALSE), ] <- df[1, ])
```

```{r}
#| label: bracket-i-assign-detail
#| dftbl: true
#| include: !expr eval_details
#| eval: !expr eval_details
with_df(df[0:2, ] <- df[1, ])
with_df(df[0, ] <- df[1, ])
with_df(df[-2, ] <- df[1, ])
with_df(df[-1:2, ] <- df[1, ])
with_df(df[NA_integer_, ] <- df[1, ])
with_df2(df2[NA_integer_, ] <- df2[1, ])
with_df(df[TRUE, ] <- df[1, ])
with_df(df[FALSE, ] <- df[1, ])
with_df(df[NA, ] <- df[1, ])
```

Only values of size one can be recycled.

```{r}
#| label: bracket-i-recycle-assign
#| dftbl: true
with_df(df[2:3, ] <- df[1, ])
with_df(df[2:3, ] <- list(df$n[1], df$c[1:2], df$li[1]))
with_df(df[2:4, ] <- df[1:2, ])
```

```{r}
#| label: bracket-i-recycle-assign-detail
#| dftbl: true
#| include: !expr eval_details
#| eval: !expr eval_details
with_df2(df2[2:4, ] <- df2[1, ])
with_df2(df2[2:4, ] <- df2[2:3, ])
```

For compatibility, only a warning is issued for indexing beyond the number of rows.
Appending rows right at the end of the existing data is supported, without warning.

```{r}
#| label: bracket-i-oob-num
#| dftbl: true
with_df(df[5, ] <- df[1, ])
with_df(df[5:7, ] <- df[1, ])
with_df(df[6, ] <- df[1, ])
with_df(df[-5, ] <- df[1, ])
with_df(df[-(5:7), ] <- df[1, ])
with_df(df[-6, ] <- df[1, ])
```

For compatibility, `i` can also be a character vector containing positive numbers.

```{r}
#| label: bracket-i-character
#| dftbl: true
with_df(df[as.character(1:3), ] <- df[1, ])
```

```{r}
#| label: bracket-i-character-detail
#| dftbl: true
#| include: !expr eval_details
#| eval: !expr eval_details
with_df(df[as.character(-(1:3)), ] <- df[1, ])
with_df(df[as.character(3:5), ] <- df[1, ])
with_df(df[as.character(-(3:5)), ] <- df[1, ])
with_df(df[NA_character_, ] <- df[1, ])
```

## Row and column subassignment

### Definition of `x[i, j] <- a`

`x[i, j] <- a` is equivalent to `x[i, ][j] <- a`.[^bracket-assign-flip]

[^bracket-assign-flip]: `x[i, j]` is symmetrical for subsetting and subassignment.
A more efficient implementation of `x[i, j] <- a` would forward to `x[j][i, ] <- a`.

Subassignment to `x[i, j]` is stricter for tibbles than for data frames.
`x[i, j] <- a` can't change the data type of existing columns.

```{r}
#| label: bracket-i-data-type
#| dftbl: true
with_df(df[2:3, 1] <- df[1:2, 2])
with_df(df[2:3, 2] <- df[1:2, 3])
with_df(df[2:3, 3] <- df2[1:2, 1])
with_df2(df2[2:3, 1] <- df2[1:2, 2])
with_df2(df2[2:3, 2] <- df[1:2, 1])
```

A notable exception is the population of a column full of `NA` (which is of type `logical`), or the use of `NA` on the right-hand side of the assignment.

```{r}
#| label: bracket-i-j-na-init
#| dftbl: true
with_df({df$x <- NA; df[2:3, "x"] <- 3:2})
with_df({df[2:3, 2:3] <- NA})
```

For programming, it is always safer (and faster) to use the correct type of `NA` to initialize columns.

```{r}
#| label: bracket-i-j-typed-na-init
#| dftbl: true
with_df({df$x <- NA_integer_; df[2:3, "x"] <- 3:2})
```


For new columns, `x[i, j] <- a` fills the unassigned rows with `NA`.

```{r}
#| label: subassign-ij-new-column
#| dftbl: true
with_df(df[2:3, "n"] <- 1)
with_df(df[2:3, "x"] <- 1)
with_df(df[2:3, "n"] <- NULL)
```

Likewise, for new rows, `x[i, j] <- a` fills the unassigned columns with `NA`.

```{r}
#| label: append-rows-only-all-columns
#| dftbl: true
with_df(df[5, "n"] <- list(0L))
```


### Definition of `x[[i, j]] <- a`

`i` must be a numeric vector of length 1.
`x[[i, j]] <- a` is equivalent to `x[i, ][[j]] <- a`.[^double-bracket-ij-symmetry]

[^double-bracket-ij-symmetry]: `x[[i, j]]` is symmetrical for subsetting and subassignment.
An efficient implementation would check that `i` and `j` are scalar and forward to `x[i, j][[1]] <- a`.


```{r}
#| label: double-bracket-i-j-equivalent-to-row-subset-then-j
#| dftbl: true
#| include: !expr eval_details
#| eval: !expr eval_details
with_df(df[[1, 1]] <- 0)
with_df(df[1, ][[1]] <- 0)
with_df(df[[1, 3]] <- list(NULL))
with_df(df[1, ][[3]] <- list(NULL))
with_df2(df2[[1, 1]] <- df[1, ])
with_df2(df2[1, ][[1]] <- df[1, ])
with_df2(df2[[1, 2]] <- t(1:4))
with_df2(df2[1, ][[2]] <- t(1:4))
df[[1:2, 1]]
with_df(df[[1:2, 1]] <- 0)
```

NB: `vec_size(a)` must equal 1.
Unlike `x[i, ] <-`, `x[[i, ]] <-` is not valid.

```{r}
#| label: check
#| dftbl: true
#| include: false
stopifnot(identical(df, new_df()))
```

# output test

    Code
      new_tibble(1:3, nrow = 1)
    Condition
      Error in `new_tibble()`:
      ! `x` must be a list.
    Code
      new_tibble(as.list(1:3))
    Condition
      Error in `new_tibble()`:
      ! `names` must not be `NULL`.

# output test

    Code
      df <- tibble(a = 1, b = 2)
      names(df) <- NULL
    Condition
      Warning:
      The `value` argument of `names<-` can't be NULL as of tibble 3.0.0.
      Warning:
      The `value` argument of `names<-` must have the same length as `x` as of tibble 3.0.0.
      Warning:
      The `value` argument of `names<-` can't be empty as of tibble 3.0.0.
    Code
      names(df) <- "c"
    Condition
      Warning:
      The `value` argument of `names<-` must have the same length as `x` as of tibble 3.0.0.
      Warning:
      The `value` argument of `names<-` can't be empty as of tibble 3.0.0.
    Code
      names(df) <- c("..1", "..2")

# unnamed input gives uniquely named output

    Code
      out <- set_tidy_names(1:3)
    Message
      New names:
       -> ..1
       -> ..2
       -> ..3

# messages by default

    Code
      set_tidy_names(set_names(1, ""))
    Message
      New names:
       -> ..1
    Output
      ..1 
        1 

# dupes

    Code
      names <- tidy_names(c("a", "b", "a", "c", "b"))
    Message
      New names:
      a -> a..1
      b -> b..2
      a -> a..3
      b -> b..5

# empty

    Code
      names <- tidy_names("")
    Message
      New names:
       -> ..1

# NA

    Code
      names <- tidy_names(NA_character_)
    Message
      New names:
       -> ..1

# corner case

    Code
      expect_equal(tidy_names(c("a..2", "a")), c("a..2", "a"))
      expect_equal(tidy_names(c("a..3", "a", "a")), c("a..1", "a..2", "a..3"))
    Message
      New names:
      a..3 -> a..1
      a -> a..2
      a -> a..3
    Code
      expect_equal(tidy_names(c("a..2", "a", "a")), c("a..1", "a..2", "a..3"))
    Message
      New names:
      a..2 -> a..1
      a -> a..2
      a -> a..3
    Code
      expect_equal(tidy_names(c("a..2", "a..2", "a..2")), c("a..1", "a..2", "a..3"))
    Message
      New names:
      a..2 -> a..1
      a..2 -> a..3

# syntactic

    Code
      names <- tidy_names(c("a b"), syntactic = TRUE)
    Message
      New names:
      a b -> a.b

# some syntactic + message (#260)

    Code
      names <- tidy_names(c("a b", "c"), syntactic = TRUE)
    Message
      New names:
      a b -> a.b

# output test

    Code
      tibble(a = 1, a = 1)
    Condition
      Error in `tibble()`:
      ! Column name `a` must not be duplicated.
      Use `.name_repair` to specify repair.
      Caused by error in `repaired_names()`:
      ! Names must be unique.
      x These names are duplicated:
        * "a" at locations 1 and 2.
    Code
      tibble(a = new_environment())
    Condition
      Error in `tibble()`:
      ! All columns in a tibble must be vectors.
      x Column `a` is an environment.
    Code
      tibble(a = 1, b = 2:3, c = 4:6, d = 7:10)
    Condition
      Error in `tibble()`:
      ! Tibble columns must have compatible sizes.
      * Size 2: Existing data.
      * Size 3: Column `c`.
      i Only values of size one are recycled.

---
title: "Subsetting"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{subset}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

<style type="text/css">
.dftbl {
    width: 100%;
    table-layout: fixed;
    display: inline-table;
}

.error pre code {
    color: red;
}

.warning pre code {
    color: violet;
}
</style>

```{r, include = FALSE}
knitr::opts_chunk$set(
  error = TRUE,
  collapse = TRUE,
  comment = "#>"
)

tibble:::set_dftbl_hooks()

options(
  lifecycle_disable_warnings = FALSE,
  lifecycle_verbose_soft_deprecation = TRUE,
  lifecycle_repeat_warnings = TRUE
)
```

There are many, many ways to subset data frames and tibbles.

<figure>
[![Alignment](alignment.png "Alignment chart"){ width=100% }](https://twitter.com/skyetetra/status/1160012260456062977)
<figcaption>
Drawing by [Jacqueline Nolis](https://twitter.com/skyetetra/status/1160012260456062977)
</figcaption>
</figure>

This vignette is an attempt to provide a comprehensive overview over the behavior of the subsetting operators `$`, `[[` and `[`, highlighting where the tibble implementation differs from the data frame implementation.

```{r setup}
library(tibble)

new_df <- function() {
  df <- data.frame(a = 1:4)
  df$b <- letters[5:8]
  df$cd <- list(9, 10:11, 12:14, "text")
  df
}

new_tbl <- function() {
  as_tibble(new_df())
}
```

Results of the same code for data frames and tibbles are presented side by side:

```{r show, dftbl = TRUE, dftbl_always = TRUE}
new_df()
```

In the following, if the results are identical (after converting to a data frame if necessary), only the tibble result is shown, as in the example below.
This allows to spot differences easier.

```{r show-compare, dftbl = TRUE}
new_df()
```

Subsetting operations are read-only.
The same objects are reused in all examples:

```{r ro}
df <- new_df()
tbl <- new_tbl()
```

## $

With `$` subsetting, accessing a missing column gives a warning.
Inexact matching is not supported:

```{r dollar, dftbl = TRUE}
df$a
df$"a"
df$a[2:3]
df$cd
df$c
df$d
```

## [[

The `exact` argument is not supported by tibbles.

```{r double-bracket-col, dftbl = TRUE}
df[["a"]]
df[["cd", exact = TRUE]]
df[["cd", exact = FALSE]]
df[["c", exact = TRUE]]
df[["c", exact = FALSE]]
```

With two indexes, a single element is returned.
List columns are not unpacked by tibbles, the `[[` only unpacks columns.

```{r double-bracket-both, dftbl = TRUE}
df[[2, "a"]]
df[[2, "cd"]]
df[[1:2, "cd"]]
df[[2, "c"]]
df[[1:2, "c"]]
```

Exotic variants like recursive indexing are deprecated for tibbles.

```{r double-bracket-exotic, dftbl = TRUE}
df[[c(1, 2)]]
```

## [

With `[` subsetting, tibbles always return a tibble.
The `drop` argument is supported but has different defaults:

```{r bracket-col-one, dftbl = TRUE}
df["a"]
df["a", drop = FALSE]
df["a", drop = TRUE]
df[1]
df[0]
df[4]
df[NA]
df[NA_character_]
df[NA_integer_]
```

The same examples are repeated for two-dimensional indexing when omitting the row index:


```{r bracket-col-two, dftbl = TRUE}
df[, "a"]
df[, "a", drop = FALSE]
df[, "a", drop = TRUE]
df[, 1]
df[, 0]
df[, 4]
df[, NA]
df[, NA_character_]
df[, NA_integer_]
```

Multiple columns can be queried by passing a vector of column indexes (names, positions, or even a logical vector).
With the latter option, tibbles are a tad stricter:

```{r bracket-col-multi-one, dftbl = TRUE}
df[c("a", "b")]
df[character()]
df[1:2]
df[1:3]
df[1:4]
df[0:2]
df[-1:2]
df[-1]
df[-(1:2)]
df[integer()]
df[TRUE]
df[FALSE]
df[c(TRUE, TRUE, FALSE)]
df[c(FALSE, TRUE, FALSE)]
df[c(FALSE, TRUE)]
df[c(FALSE, TRUE, FALSE, TRUE)]
```

The same examples are repeated for two-dimensional indexing when omitting the row index:

```{r bracket-col-multi-two, dftbl = TRUE}
df[, c("a", "b")]
df[, character()]
df[, 1:2]
df[, 1:3]
df[, 1:4]
df[, 0:2]
df[, -1:2]
df[, -1]
df[, -(1:2)]
df[, integer()]
df[, TRUE]
df[, FALSE]
df[, c(TRUE, TRUE, FALSE)]
df[, c(FALSE, TRUE, FALSE)]
df[, c(FALSE, TRUE)]
df[, c(FALSE, TRUE, FALSE, TRUE)]
```

Row subsetting with integer indexes works almost identical.
Out-of-bounds subsetting is not recommended and may lead to an error in future versions.
Another special case is subsetting with `[1, , drop = TRUE]` where the data frame implementation returns a list.

```{r bracket-row-integer, dftbl = TRUE}
df[1, ]
df[1, , drop = TRUE]
df[1:2, ]
df[0, ]
df[integer(), ]
df[5, ]
df[4:5, ]
df[-1, ]
df[-1:2, ]
df[NA, ]
df[NA_integer_, ]
df[c(NA, 1), ]
```

Row subsetting with logical indexes also works almost identical, the index vector must have length one or the number of rows with tibbles.

```{r bracket-row-logical, dftbl = TRUE}
df[TRUE, ]
df[FALSE, ]
df[c(TRUE, FALSE), ]
df[c(TRUE, FALSE, TRUE), ]
df[c(TRUE, FALSE, TRUE, FALSE), ]
df[c(TRUE, FALSE, TRUE, FALSE, TRUE), ]
```

Indexing both row and column works more or less the same, except for `drop`:

```{r bracket-both, dftbl = TRUE}
df[1, "a"]
df[1, "a", drop = FALSE]
df[1, "a", drop = TRUE]
df[1:2, "a"]
df[1:2, "a", drop = FALSE]
df[1:2, "a", drop = TRUE]
df[1, c("a", "b")]
df[1, c("a", "b"), drop = FALSE]
df[1, c("a", "b"), drop = TRUE]
df[1:2, c("a", "b")]
```

Indexes can be omitted altogether, no differences here:

```{r bracket-empty, dftbl = TRUE}
df[]
df[,]
```

---
title: "Tibbles"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Tibbles}
  %\VignetteEngine{knitr::rmarkdown}
  \usepackage[utf8]{inputenc}
---



Tibbles are a modern take on data frames.
They keep the features that have stood the test of time, and drop the features that used to be convenient but are now frustrating.


```r
library(tibble)
```


## Creating

`tibble()` is a nice way to create data frames.
It encapsulates best practices for data frames:

  * It never changes an input's type (i.e., no more `stringsAsFactors = FALSE`!).

    
    ```r
    tibble(x = letters)
    #> # A tibble: 26 x 1
    #>    x    
    #>    <chr>
    #>  1 a    
    #>  2 b    
    #>  3 c    
    #>  4 d    
    #>  5 e    
    #>  6 f    
    #>  7 g    
    #>  8 h    
    #>  9 i    
    #> 10 j    
    #> # ... with 16 more rows
    ```

    This makes it easier to use with list-columns:

    
    ```r
    tibble(x = 1:3, y = list(1:5, 1:10, 1:20))
    #> # A tibble: 3 x 2
    #>       x y         
    #>   <int> <list>    
    #> 1     1 <int [5]> 
    #> 2     2 <int [10]>
    #> 3     3 <int [20]>
    ```

    List-columns are often created by `tidyr::nest()`, but they can be useful to
    create by hand.

  * It never adjusts the names of variables:

    
    ```r
    names(data.frame(`crazy name` = 1))
    #> [1] "crazy.name"
    names(tibble(`crazy name` = 1))
    #> [1] "crazy name"
    ```

  * It evaluates its arguments lazily and sequentially:

    
    ```r
    tibble(x = 1:5, y = x ^ 2)
    #> # A tibble: 5 x 2
    #>       x     y
    #>   <int> <dbl>
    #> 1     1     1
    #> 2     2     4
    #> 3     3     9
    #> 4     4    16
    #> 5     5    25
    ```

  * It never uses `row.names()`.
    The whole point of tidy data is to store variables in a consistent way.
    So it never stores a variable as special attribute.

  * It only recycles vectors of length 1.
    This is because recycling vectors of greater lengths is a frequent source of bugs.

## Coercion

To complement `tibble()`, tibble provides `as_tibble()` to coerce objects into tibbles.
Generally, `as_tibble()` methods are much simpler than `as.data.frame()` methods.
The method for lists has been written with an eye for performance:


```r
l <- replicate(26, sample(100), simplify = FALSE)
names(l) <- letters

timing <- bench::mark(
  as_tibble(l),
  as.data.frame(l),
  check = FALSE
)

timing
```



The speed of `as.data.frame()` is not usually a bottleneck when used interactively, but can be a problem when combining thousands of messy inputs into one tidy data frame.

## Tibbles vs data frames

There are three key differences between tibbles and data frames: printing, subsetting, and recycling rules.

### Printing

When you print a tibble, it only shows the first ten rows and all the columns that fit on one screen.
It also prints an abbreviated description of the column type, and uses font styles and color for highlighting:


```r
tibble(x = -5:100, y = 123.456 * (3^x))
#> # A tibble: 106 x 2
#>        x         y
#>    <int>     <dbl>
#>  1    -5     0.508
#>  2    -4     1.52 
#>  3    -3     4.57 
#>  4    -2    13.7  
#>  5    -1    41.2  
#>  6     0   123.   
#>  7     1   370.   
#>  8     2  1111.   
#>  9     3  3333.   
#> 10     4 10000.   
#> # ... with 96 more rows
```

Numbers are displayed with three significant figures by default, and a trailing dot that indicates the existence of a fractional component.

You can control the default appearance with options:

* `options(pillar.print_max = n, pillar.print_min = m)`: if there are more than `n` rows, print only the first `m` rows.
  Use `options(pillar.print_max = Inf)` to always show all rows.

* `options(pillar.width = n)`: use `n` character slots horizontally to show the data. If `n > getOption("width")`, this will result in multiple tiers. Use `options(pillar.width = Inf)` to always print all columns, regardless of the width of the screen.

See `?pillar::pillar_options` and `?tibble_options` for the available options, `vignette("types")` for an overview of the type abbreviations, `vignette("numbers")` for details on the formatting of numbers, and `vignette("digits")` for a comparison with data frame printing.

### Subsetting

Tibbles are quite strict about subsetting.
`[` always returns another tibble.
Contrast this with a data frame: sometimes `[` returns a data frame and sometimes it just returns a vector:


```r
df1 <- data.frame(x = 1:3, y = 3:1)
class(df1[, 1:2])
#> [1] "data.frame"
class(df1[, 1])
#> [1] "integer"

df2 <- tibble(x = 1:3, y = 3:1)
class(df2[, 1:2])
#> [1] "tbl_df"     "tbl"        "data.frame"
class(df2[, 1])
#> [1] "tbl_df"     "tbl"        "data.frame"
```

To extract a single column use `[[` or `$`:


```r
class(df2[[1]])
#> [1] "integer"
class(df2$x)
#> [1] "integer"
```

Tibbles are also stricter with `$`.
Tibbles never do partial matching, and will throw a warning and return `NULL` if the column does not exist:


```r
df <- data.frame(abc = 1)
df$a
#> [1] 1

df2 <- tibble(abc = 1)
df2$a
#> Warning: Unknown or uninitialised column: `a`.
#> NULL
```

However, tibbles respect the `drop` argument if it is provided:


```r
data.frame(a = 1:3)[, "a", drop = TRUE]
#> [1] 1 2 3
tibble(a = 1:3)[, "a", drop = TRUE]
#> [1] 1 2 3
```

Tibbles do not support row names.
They are removed when converting to a tibble or when subsetting:


```r
df <- data.frame(a = 1:3, row.names = letters[1:3])
rownames(df)
#> [1] "a" "b" "c"
rownames(as_tibble(df))
#> [1] "1" "2" "3"

tbl <- tibble(a = 1:3)
rownames(tbl) <- letters[1:3]
#> Warning: Setting row names on a tibble is deprecated.
rownames(tbl)
#> [1] "a" "b" "c"
rownames(tbl[1, ])
#> [1] "1"
```

See `vignette("invariants")` for a detailed comparison between tibbles and data frames.


### Recycling

When constructing a tibble, only values of length 1 are recycled.
The first column with length different to one determines the number of rows in the tibble, conflicts lead to an error:


```r
tibble(a = 1, b = 1:3)
#> # A tibble: 3 x 2
#>       a     b
#>   <dbl> <int>
#> 1     1     1
#> 2     1     2
#> 3     1     3
tibble(a = 1:3, b = 1)
#> # A tibble: 3 x 2
#>       a     b
#>   <int> <dbl>
#> 1     1     1
#> 2     2     1
#> 3     3     1
tibble(a = 1:3, c = 1:2)
#> Error in `tibble()`:
#> ! Tibble columns must have compatible sizes.
#> * Size 3: Existing data.
#> * Size 2: Column `c`.
#> i Only values of size one are recycled.
```

This also extends to tibbles with *zero* rows, which is sometimes important for programming:


```r
tibble(a = 1, b = integer())
#> # A tibble: 0 x 2
#> # ... with 2 variables: a <dbl>, b <int>
tibble(a = integer(), b = 1)
#> # A tibble: 0 x 2
#> # ... with 2 variables: a <int>, b <dbl>
```


### Arithmetic operations

Unlike data frames, tibbles don't support arithmetic operations on all columns.
The result is silently coerced to a data frame.
Do not rely on this behavior, it may become an error in a forthcoming version.


```r
tbl <- tibble(a = 1:3, b = 4:6)
tbl * 2
#>   a  b
#> 1 2  8
#> 2 4 10
#> 3 6 12
```

---
title: "Invariants: Comparing behavior with data frames"
#output: rmarkdown::word_document
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Invariants: Comparing behavior with data frames}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

<style type="text/css">
.dftbl {
    width: 100%;
    table-layout: fixed;
    display: inline-table;
}

.error pre code {
    color: red;
}

.warning pre code {
    color: violet;
}
</style>



This vignette defines invariants for subsetting and subset-assignment for tibbles, and illustrates where their behaviour differs from data frames.
The goal is to define a small set of invariants that consistently define how behaviors interact.
Some behaviors are defined using functions of the vctrs package, e.g. `vec_slice()`, `vec_recycle()` and `vec_as_index()`.
Refer to their documentation for more details about the invariants that they follow.

The subsetting and subassignment operators for data frames and tibbles are particularly tricky, because they support both row and column indexes, both of which are optionally missing.
We resolve this by first defining column access with `[[` and `$`, then column-wise subsetting with `[`, then row-wise subsetting, then the composition of both.

## Conventions

In this article, all behaviors are demonstrated using one example data frame and its tibble equivalent:


```r
library(tibble)
library(vctrs)
new_df <- function() {
  df <- data.frame(n = c(1L, NA, 3L, NA))
  df$c <- letters[5:8]
  df$li <- list(9, 10:11, 12:14, "text")
  df
}
new_tbl <- function() {
  as_tibble(new_df())
}
```

Results of the same code for data frames and tibbles are presented side by side:

<table class="dftbl"><tbody><tr style="vertical-align:top"><td>

```r
new_df()
#>    n c         li
#> 1  1 e          9
#> 2 NA f     10, 11
#> 3  3 g 12, 13, 14
#> 4 NA h       text
```

</td><td>

```r
new_tbl()
#> # A tibble: 4 x 3
#>       n c     li       
#>   <int> <chr> <list>   
#> 1     1 e     <dbl [1]>
#> 2    NA f     <int [2]>
#> 3     3 g     <int [3]>
#> 4    NA h     <chr [1]>
```

</td></tr></tbody></table>

If the results are identical (after converting to a data frame if necessary), only the tibble result is shown.

Subsetting operations are read-only.
The same objects are reused in all examples:


```r
df <- new_df()
tbl <- new_tbl()
```

Where needed, we also show examples with hierarchical columns containing a data frame or a matrix:


```r
new_tbl2 <- function() {
  tibble(
    tb = tbl,
    m = diag(4)
  )
}
new_df2 <- function() {
  df2 <- new_tbl2()
  class(df2) <- "data.frame"
  class(df2$tb) <- "data.frame"
  df2
}
df2 <- new_df2()
tbl2 <- new_tbl2()
```

<table class="dftbl"><tbody><tr style="vertical-align:top"><td>

```r
new_df()
#>    n c         li
#> 1  1 e          9
#> 2 NA f     10, 11
#> 3  3 g 12, 13, 14
#> 4 NA h       text
```

</td><td>

```r
new_tbl()
#> # A tibble: 4 x 3
#>       n c     li       
#>   <int> <chr> <list>   
#> 1     1 e     <dbl [1]>
#> 2    NA f     <int [2]>
#> 3     3 g     <int [3]>
#> 4    NA h     <chr [1]>
```

</td></tr></tbody></table>

For subset assignment (subassignment, for short), we need a fresh copy of the data for each test.
The `with_*()` functions (omitted here for brevity) allow for a more concise notation.
These functions take an assignment expression, execute it on a fresh copy of the data, and return the data for printing.
The first example prints what's really executed, further examples omit this output.



<table class="dftbl"><tbody><tr style="vertical-align:top"><td>

```r
with_df(df$n <- rev(df$n), verbose = TRUE)
#> {
#>   df <- new_df()
#>   df$n <- rev(df$n)
#>   df
#> }
#>    n c         li
#> 1 NA e          9
#> 2  3 f     10, 11
#> 3 NA g 12, 13, 14
#> 4  1 h       text
```

</td><td>

```r
with_tbl(tbl$n <- rev(tbl$n), verbose = TRUE)
#> {
#>   tbl <- new_tbl()
#>   tbl$n <- rev(tbl$n)
#>   tbl
#> }
#> # A tibble: 4 x 3
#>       n c     li       
#>   <int> <chr> <list>   
#> 1    NA e     <dbl [1]>
#> 2     3 f     <int [2]>
#> 3    NA g     <int [3]>
#> 4     1 h     <chr [1]>
```

</td></tr></tbody></table>

## Column extraction

### Definition of `x[[j]]`

`x[[j]]` is equal to `.subset2(x, j)`.

<table class="dftbl"><tbody><tr style="vertical-align:top"><td>

```r
df[[1]]
#> [1]  1 NA  3 NA
```

</td><td>

```r
tbl[[1]]
#> [1]  1 NA  3 NA
```

</td></tr><tr style="vertical-align:top"><td>

```r
.subset2(df, 1)
#> [1]  1 NA  3 NA
```

</td><td>

```r
.subset2(tbl, 1)
#> [1]  1 NA  3 NA
```

</td></tr></tbody></table>


<table class="dftbl"><tbody><tr style="vertical-align:top"><td>

```r
identical(df[[3]], .subset2(df, 3))
#> [1] TRUE
```

</td><td>

```r
identical(tbl[[3]], .subset2(tbl, 3))
#> [1] TRUE
```

</td></tr><tr style="vertical-align:top"><td>

```r
identical(df2[["df"]], .subset2(df2, "df"))
#> [1] TRUE
```

</td><td>

```r
identical(tbl2[["tbl"]], .subset2(tbl2, "tbl"))
#> [1] TRUE
```

</td></tr></tbody></table>

NB: `x[[j]]` always returns an object of size `nrow(x)` if the column exists.

<table class="dftbl"><tbody><tr style="vertical-align:top"><td>

```r
vec_size(df[[1]])
#> [1] 4
```

</td><td>

```r
vec_size(tbl[[1]])
#> [1] 4
```

</td></tr><tr style="vertical-align:top"><td>

```r
vec_size(df[[3]])
#> [1] 4
```

</td><td>

```r
vec_size(tbl[[3]])
#> [1] 4
```

</td></tr><tr style="vertical-align:top"><td>

```r
vec_size(df2[[1]])
#> [1] 4
```

</td><td>

```r
vec_size(tbl2[[1]])
#> [1] 4
```

</td></tr><tr style="vertical-align:top"><td>

```r
vec_size(df2[[2]])
#> [1] 4
```

</td><td>

```r
vec_size(tbl2[[2]])
#> [1] 4
```

</td></tr></tbody></table>

`j` must be a single number or a string, as enforced by `.subset2(x, j)`.

<table class="dftbl"><tbody><tr style="vertical-align:top"><td>

```r
df[[1:2]]
#> [1] NA
```

</td><td>

```r
tbl[[1:2]]
#> Error:
#> ! The `j` argument of `[[.tbl_df` can't be a vector of length 2 as of tibble 3.0.0.
#> i Recursive subsetting is deprecated for tibbles.
```

</td></tr><tr style="vertical-align:top"><td>

```r
df[[c("n", "c")]]
#> Error in .subset2(x, i, exact = exact): subscript out of bounds
```

</td><td>

```r
tbl[[c("n", "c")]]
#> Error in `tbl[[c("n", "c")]]`:
#> ! Can't extract column with `c("n", "c")`.
#> x Subscript `c("n", "c")` must be size 1, not 2.
```

</td></tr><tr style="vertical-align:top"><td>

```r
df[[TRUE]]
#> [1]  1 NA  3 NA
```

</td><td>

```r
tbl[[TRUE]]
#> Error in `tbl[[TRUE]]`:
#> ! Can't extract column with `TRUE`.
#> x `TRUE` must be numeric or character, not `TRUE`.
```

</td></tr><tr style="vertical-align:top"><td>

```r
df[[mean]]
#> Error in .subset2(x, i, exact = exact): invalid subscript type 'closure'
```

</td><td>

```r
tbl[[mean]]
#> Error in `tbl[[mean]]`:
#> ! Can't extract column with `mean`.
#> x `mean` must be numeric or character, not a function.
```

</td></tr></tbody></table>

`NA` indexes, numeric out-of-bounds (OOB) values, and non-integers throw an error:

<table class="dftbl"><tbody><tr style="vertical-align:top"><td>

```r
df[[NA]]
#> NULL
```

</td><td>

```r
tbl[[NA]]
#> Error in `tbl[[NA]]`:
#> ! Can't extract column with `NA`.
#> x Subscript `NA` must be a location, not an integer `NA`.
```

</td></tr><tr style="vertical-align:top"><td>

```r
df[[NA_character_]]
#> NULL
```

</td><td>

```r
tbl[[NA_character_]]
#> Error in `tbl[[NA_character_]]`:
#> ! Can't extract column with `NA_character_`.
#> x Subscript `NA_character_` must be a location, not a character `NA`.
```

</td></tr><tr style="vertical-align:top"><td>

```r
df[[NA_integer_]]
#> NULL
```

</td><td>

```r
tbl[[NA_integer_]]
#> Error in `tbl[[NA_integer_]]`:
#> ! Can't extract column with `NA_integer_`.
#> x Subscript `NA_integer_` must be a location, not an integer `NA`.
```

</td></tr><tr style="vertical-align:top"><td>

```r
df[[-1]]
#> Error in .subset2(x, i, exact = exact): invalid negative subscript in get1index <real>
```

</td><td>

```r
tbl[[-1]]
#> Error in `tbl[[-1]]`:
#> ! Can't extract column with `-1`.
#> x Subscript `-1` must be a positive location, not -1.
```

</td></tr><tr style="vertical-align:top"><td>

```r
df[[4]]
#> Error in .subset2(x, i, exact = exact): subscript out of bounds
```

</td><td>

```r
tbl[[4]]
#> Error in `tbl[[4]]`:
#> ! Can't extract columns past the end.
#> i Location 4 doesn't exist.
#> i There are only 3 columns.
```

</td></tr><tr style="vertical-align:top"><td>

```r
df[[1.5]]
#> [1]  1 NA  3 NA
```

</td><td>

```r
tbl[[1.5]]
#> Error in `tbl[[1.5]]`:
#> ! Can't extract column with `1.5`.
#> x Can't convert from `j` <double> to <integer> due to loss of precision.
```

</td></tr><tr style="vertical-align:top"><td>

```r
df[[Inf]]
#> NULL
```

</td><td>

```r
tbl[[Inf]]
#> Error in `tbl[[Inf]]`:
#> ! Can't extract column with `Inf`.
#> x Can't convert from `j` <double> to <integer> due to loss of precision.
```

</td></tr></tbody></table>

Character OOB access is silent because a common package idiom is to check for the absence of a column with `is.null(df[[var]])`.

<table class="dftbl"><tbody><tr style="vertical-align:top"><td>

```r
df[["x"]]
#> NULL
```

</td><td>

```r
tbl[["x"]]
#> NULL
```

</td></tr></tbody></table>

### Definition of `x$name`

`x$name` and `x$"name"` are equal to `x[["name"]]`.

<table class="dftbl"><tbody><tr style="vertical-align:top"><td>

```r
df$n
#> [1]  1 NA  3 NA
```

</td><td>

```r
tbl$n
#> [1]  1 NA  3 NA
```

</td></tr><tr style="vertical-align:top"><td>

```r
df$"n"
#> [1]  1 NA  3 NA
```

</td><td>

```r
tbl$"n"
#> [1]  1 NA  3 NA
```

</td></tr><tr style="vertical-align:top"><td>

```r
df[["n"]]
#> [1]  1 NA  3 NA
```

</td><td>

```r
tbl[["n"]]
#> [1]  1 NA  3 NA
```

</td></tr></tbody></table>

<table class="dftbl"><tbody><tr style="vertical-align:top"><td>

```r
identical(df$li, df[["li"]])
#> [1] TRUE
```

</td><td>

```r
identical(tbl$li, tbl[["li"]])
#> [1] TRUE
```

</td></tr><tr style="vertical-align:top"><td>

```r
identical(df2$tb, df2[["tb"]])
#> [1] TRUE
```

</td><td>

```r
identical(tbl2$tb, tbl2[["tb"]])
#> [1] TRUE
```

</td></tr><tr style="vertical-align:top"><td>

```r
identical(df2$m, df2[["m"]])
#> [1] TRUE
```

</td><td>

```r
identical(tbl2$m, tbl2[["m"]])
#> [1] TRUE
```

</td></tr></tbody></table>

Unlike data frames, tibbles do not partially match names.
Because `df$x` is rarely used in packages, it can raise a warning:

<table class="dftbl"><tbody><tr style="vertical-align:top"><td>

```r
df$l
#> [[1]]
#> [1] 9
#> 
#> [[2]]
#> [1] 10 11
#> 
#> [[3]]
#> [1] 12 13 14
#> 
#> [[4]]
#> [1] "text"
```

</td><td>

```r
tbl$l
#> Warning: Unknown or uninitialised
#> column: `l`.
#> NULL
```

</td></tr><tr style="vertical-align:top"><td>

```r
df$not_present
#> NULL
```

</td><td>

```r
tbl$not_present
#> Warning: Unknown or uninitialised
#> column: `not_present`.
#> NULL
```

</td></tr></tbody></table>

## Column subsetting

### Definition of `x[j]`

`j` is converted to an integer vector by `vec_as_index(j, ncol(x), names = names(x))`.
Then `x[c(j_1, j_2, ..., j_n)]` is equivalent to `tibble(x[[j_1]], x[[j_2]], ..., x[[j_n]])`, keeping the corresponding column names.
This implies that `j` must be a numeric or character vector, or a logical vector with length 1 or `ncol(x)`.[^subset-extract-commute]

[^subset-extract-commute]: `x[j][[jj]]` is equal to `x[[ j[[jj]] ]]`, in particular `x[j][[1]]` is equal to `x[[j]]` for scalar numeric or integer `j`.


<table class="dftbl"><tbody><tr style="vertical-align:top"><td>

```r
df[1:2]
#>    n c
#> 1  1 e
#> 2 NA f
#> 3  3 g
#> 4 NA h
```

</td><td>

```r
tbl[1:2]
#> # A tibble: 4 x 2
#>       n c    
#>   <int> <chr>
#> 1     1 e    
#> 2    NA f    
#> 3     3 g    
#> 4    NA h
```

</td></tr></tbody></table>

When subsetting repeated indexes, the resulting column names are undefined, do not rely on them.

<table class="dftbl"><tbody><tr style="vertical-align:top"><td>

```r
df[c(1, 1)]
#>    n n.1
#> 1  1   1
#> 2 NA  NA
#> 3  3   3
#> 4 NA  NA
```

</td><td>

```r
tbl[c(1, 1)]
#> # A tibble: 4 x 2
#>       n     n
#>   <int> <int>
#> 1     1     1
#> 2    NA    NA
#> 3     3     3
#> 4    NA    NA
```

</td></tr></tbody></table>

For tibbles with repeated column names, subsetting by name uses the first matching column.

`nrow(df[j])` equals `nrow(df)`.

<table class="dftbl"><tbody><tr style="vertical-align:top"><td>

```r
df[integer()]
#> data frame with 0 columns and 4 rows
```

</td><td>

```r
tbl[integer()]
#> # A tibble: 4 x 0
```

</td></tr></tbody></table>

Tibbles support indexing by a logical matrix, but only if all values in the returned vector are compatible.

<table class="dftbl"><tbody><tr style="vertical-align:top"><td>

```r
df[is.na(df)]
#> [[1]]
#> [1] NA
#> 
#> [[2]]
#> [1] NA
```

</td><td>

```r
tbl[is.na(tbl)]
#> [1] NA NA
```

</td></tr><tr style="vertical-align:top"><td>

```r
df[!is.na(df)]
#> [[1]]
#> [1] 1
#> 
#> [[2]]
#> [1] 3
#> 
#> [[3]]
#> [1] "e"
#> 
#> [[4]]
#> [1] "f"
#> 
#> [[5]]
#> [1] "g"
#> 
#> [[6]]
#> [1] "h"
#> 
#> [[7]]
#> [1] 9
#> 
#> [[8]]
#> [1] 10 11
#> 
#> [[9]]
#> [1] 12 13 14
#> 
#> [[10]]
#> [1] "text"
```

</td><td>

```r
tbl[!is.na(tbl)]
#> Error in `vec_c()`:
#> ! Can't combine `n` <integer> and `c` <character>.
```

</td></tr></tbody></table>

### Definition of `x[, j]`

`x[, j]` is equal to `x[j]`.
Tibbles do not perform column extraction if `x[j]` would yield a single column.

<table class="dftbl"><tbody><tr style="vertical-align:top"><td>

```r
df[, 1]
#> [1]  1 NA  3 NA
```

</td><td>

```r
tbl[, 1]
#> # A tibble: 4 x 1
#>       n
#>   <int>
#> 1     1
#> 2    NA
#> 3     3
#> 4    NA
```

</td></tr><tr style="vertical-align:top"><td>

```r
df[, 1:2]
#>    n c
#> 1  1 e
#> 2 NA f
#> 3  3 g
#> 4 NA h
```

</td><td>

```r
tbl[, 1:2]
#> # A tibble: 4 x 2
#>       n c    
#>   <int> <chr>
#> 1     1 e    
#> 2    NA f    
#> 3     3 g    
#> 4    NA h
```

</td></tr></tbody></table>

<table class="dftbl"><tbody><tr style="vertical-align:top"><td>

```r
identical(df[, 2:3], df[2:3])
#> [1] TRUE
```

</td><td>

```r
identical(tbl[, 2:3], tbl[2:3])
#> [1] TRUE
```

</td></tr><tr style="vertical-align:top"><td>

```r
identical(df2[, 1:2], df2[1:2])
#> [1] TRUE
```

</td><td>

```r
identical(tbl2[, 1:2], tbl2[1:2])
#> [1] TRUE
```

</td></tr></tbody></table>

### Definition of `x[, j, drop = TRUE]`

For backward compatiblity, `x[, j, drop = TRUE]` performs column __extraction__, returning `x[j][[1]]` when `ncol(x[j])` is 1.

<table class="dftbl"><tbody><tr style="vertical-align:top"><td>

```r
df[, 1, drop = TRUE]
#> [1]  1 NA  3 NA
```

</td><td>

```r
tbl[, 1, drop = TRUE]
#> [1]  1 NA  3 NA
```

</td></tr></tbody></table>

<table class="dftbl"><tbody><tr style="vertical-align:top"><td>

```r
identical(df[, 3, drop = TRUE], df[[3]])
#> [1] TRUE
```

</td><td>

```r
identical(tbl[, 3, drop = TRUE], tbl[[3]])
#> [1] TRUE
```

</td></tr><tr style="vertical-align:top"><td>

```r
identical(df2[, 1, drop = TRUE], df2[[1]])
#> [1] TRUE
```

</td><td>

```r
identical(tbl2[, 1, drop = TRUE], tbl2[[1]])
#> [1] TRUE
```

</td></tr><tr style="vertical-align:top"><td>

```r
identical(df2[, 2, drop = TRUE], df2[[2]])
#> [1] TRUE
```

</td><td>

```r
identical(tbl2[, 2, drop = TRUE], tbl2[[2]])
#> [1] TRUE
```

</td></tr></tbody></table>

## Row subsetting

### Definition of `x[i, ]`

`x[i, ]` is equal to `tibble(vec_slice(x[[1]], i), vec_slice(x[[2]], i), ...)`.[^row-subset-efficiency]

[^row-subset-efficiency]: Row subsetting `x[i, ]` is not defined in terms of `x[[j]][i]` because that definition does not generalise to matrix and data frame columns.
For efficiency and backward compatibility, `i` is converted to an integer vector by `vec_as_index(i, nrow(x))` first.

<table class="dftbl"><tbody><tr style="vertical-align:top"><td>

```r
df[3, ]
#>   n c         li
#> 3 3 g 12, 13, 14
```

</td><td>

```r
tbl[3, ]
#> # A tibble: 1 x 3
#>       n c     li       
#>   <int> <chr> <list>   
#> 1     3 g     <int [3]>
```

</td></tr></tbody></table>

This means that `i` must be a numeric vector, or a logical vector of length `nrow(x)` or 1.
For compatibility, `i` can also be a character vector containing positive numbers.

<table class="dftbl"><tbody><tr style="vertical-align:top"><td>

```r
df[mean, ]
#> Error in xj[i]: invalid subscript type 'closure'
```

</td><td>

```r
tbl[mean, ]
#> Error in `tbl[mean, ]`:
#> ! Can't subset rows with `mean`.
#> x `mean` must be logical, numeric, or character, not a function.
```

</td></tr><tr style="vertical-align:top"><td>

```r
df[list(1), ]
#> Error in xj[i]: invalid subscript type 'list'
```

</td><td>

```r
tbl[list(1), ]
#> Error in `tbl[list(1), ]`:
#> ! Can't subset rows with `list(1)`.
#> x `list(1)` must be logical, numeric, or character, not a list.
```

</td></tr><tr style="vertical-align:top"><td>

```r
df["1", ]
#>   n c li
#> 1 1 e  9
```

</td><td>

```r
tbl["1", ]
#> # A tibble: 1 x 3
#>       n c     li       
#>   <int> <chr> <list>   
#> 1     1 e     <dbl [1]>
```

</td></tr></tbody></table>

Exception: OOB values generate warnings instead of errors:

<table class="dftbl"><tbody><tr style="vertical-align:top"><td>

```r
df[10, ]
#>     n    c   li
#> NA NA <NA> NULL
```

</td><td>

```r
tbl[10, ]
#> Warning: The `i` argument of `[.tbl_df` must lie in [0, rows] if positive, as of tibble 3.0.0.
#> i Use `NA_integer_` as row index to obtain a row full of `NA` values.
#> # A tibble: 1 x 3
#>       n c     li    
#>   <int> <chr> <list>
#> 1    NA <NA>  <NULL>
```

</td></tr><tr style="vertical-align:top"><td>

```r
df["x", ]
#>     n    c   li
#> NA NA <NA> NULL
```

</td><td>

```r
tbl["x", ]
#> Warning: The `i` argument of `[.tbl_df` must use valid row names as of tibble 3.0.0.
#> i Use `NA_integer_` as row index to obtain a row full of `NA` values.
#> # A tibble: 1 x 3
#>       n c     li    
#>   <int> <chr> <list>
#> 1    NA <NA>  <NULL>
```

</td></tr></tbody></table>


Unlike data frames, only logical vectors of length 1 are recycled.
<!-- TODO: should this be an error? #648 -->

<table class="dftbl"><tbody><tr style="vertical-align:top"><td>

```r
df[c(TRUE, FALSE), ]
#>   n c         li
#> 1 1 e          9
#> 3 3 g 12, 13, 14
```

</td><td>

```r
tbl[c(TRUE, FALSE), ]
#> Error in `tbl[c(TRUE, FALSE), ]`:
#> ! Can't subset rows with `c(TRUE, FALSE)`.
#> x Logical subscript `c(TRUE, FALSE)` must be size 1 or 4, not 2.
```

</td></tr></tbody></table>

NB: scalar logicals are recycled, but scalar numerics are not.
That makes the `x[NA, ]` and `x[NA_integer_, ]` return different results.

<table class="dftbl"><tbody><tr style="vertical-align:top"><td>

```r
df[NA, ]
#>       n    c   li
#> NA   NA <NA> NULL
#> NA.1 NA <NA> NULL
#> NA.2 NA <NA> NULL
#> NA.3 NA <NA> NULL
```

</td><td>

```r
tbl[NA, ]
#> # A tibble: 4 x 3
#>       n c     li    
#>   <int> <chr> <list>
#> 1    NA <NA>  <NULL>
#> 2    NA <NA>  <NULL>
#> 3    NA <NA>  <NULL>
#> 4    NA <NA>  <NULL>
```

</td></tr><tr style="vertical-align:top"><td>

```r
df[NA_integer_, ]
#>     n    c   li
#> NA NA <NA> NULL
```

</td><td>

```r
tbl[NA_integer_, ]
#> # A tibble: 1 x 3
#>       n c     li    
#>   <int> <chr> <list>
#> 1    NA <NA>  <NULL>
```

</td></tr></tbody></table>

### Definition of `x[i, , drop = TRUE]`

`drop = TRUE` has no effect when not selecting a single row:

<table class="dftbl"><tbody><tr style="vertical-align:top"><td>

```r
df[1, , drop = TRUE]
#> $n
#> [1] 1
#> 
#> $c
#> [1] "e"
#> 
#> $li
#> $li[[1]]
#> [1] 9
```

</td><td>

```r
tbl[1, , drop = TRUE]
#> # A tibble: 1 x 3
#>       n c     li       
#>   <int> <chr> <list>   
#> 1     1 e     <dbl [1]>
```

</td></tr></tbody></table>

<!-- TODO: soft-deprecate -->

## Row and column subsetting

### Definition of `x[]` and `x[,]`

`x[]` and `x[,]` are equivalent to `x`.[^bracket-comma]

[^bracket-comma]: `x[,]` is equivalent to `x[]` because `x[, j]` is equivalent to `x[j]`.

### Definition of `x[i, j]`

`x[i, j]` is equal to `x[i, ][j]`.[^bracket-flip]

[^bracket-flip]: A more efficient implementation of `x[i, j]` would forward to `x[j][i, ]`.

<table class="dftbl"><tbody><tr style="vertical-align:top"><td>

```r
df[1, 1]
#> [1] 1
```

</td><td>

```r
tbl[1, 1]
#> # A tibble: 1 x 1
#>       n
#>   <int>
#> 1     1
```

</td></tr><tr style="vertical-align:top"><td>

```r
df[1, ][1]
#>   n
#> 1 1
```

</td><td>

```r
tbl[1, ][1]
#> # A tibble: 1 x 1
#>       n
#>   <int>
#> 1     1
```

</td></tr><tr style="vertical-align:top"><td>

```r
identical(df[1, 2:3], df[2:3][1, ])
#> [1] TRUE
```

</td><td>

```r
identical(tbl[1, 2:3], tbl[2:3][1, ])
#> [1] TRUE
```

</td></tr><tr style="vertical-align:top"><td>

```r
identical(df[2:3, 1], df[1][2:3, ])
#> [1] TRUE
```

</td><td>

```r
identical(tbl[2:3, 1], tbl[1][2:3, ])
#> [1] TRUE
```

</td></tr><tr style="vertical-align:top"><td>

```r
identical(df2[2:3, 1:2], df2[1:2][2:3, ])
#> [1] TRUE
```

</td><td>

```r
identical(tbl2[2:3, 1:2], tbl2[1:2][2:3, ])
#> [1] TRUE
```

</td></tr></tbody></table>


### Definition of `x[[i, j]]`

`i` must be a numeric vector of length 1.
`x[[i, j]]` is equal to `x[i, ][[j]]`, or `vctrs::vec_slice(x[[j]], i)`.[^bracket2-flip]

[^bracket2-flip]: Cell subsetting `x[[i, j]]` is not defined in terms of `x[[j]][[i]]` because that definition does not generalise to list, matrix and data frame columns.
A more efficient implementation of `x[[i, j]]` would check that `j` is a scalar and forward to `x[i, j][[1]]`.


```r
df[[1, 1]]
#> [1] 1
df[[1, 3]]
#> [1] 9
```

This implies that `j` must be a numeric or character vector of length 1.

NB: `vec_size(x[[i, j]])` always equals 1.
Unlike `x[i, ]`, `x[[i, ]]` is not valid.

## Column update

### Definition of `x[[j]] <- a`

If `a` is a vector then `x[[j]] <- a` replaces the `j`th column with value `a`.

<table class="dftbl"><tbody><tr style="vertical-align:top"><td>

```r
with_df(df[[1]] <- 0)
#>   n c         li
#> 1 0 e          9
#> 2 0 f     10, 11
#> 3 0 g 12, 13, 14
#> 4 0 h       text
```

</td><td>

```r
with_tbl(tbl[[1]] <- 0)
#> # A tibble: 4 x 3
#>       n c     li       
#>   <dbl> <chr> <list>   
#> 1     0 e     <dbl [1]>
#> 2     0 f     <int [2]>
#> 3     0 g     <int [3]>
#> 4     0 h     <chr [1]>
```

</td></tr><tr style="vertical-align:top"><td>

```r
with_df(df[[3]] <- 4:1)
#>    n c li
#> 1  1 e  4
#> 2 NA f  3
#> 3  3 g  2
#> 4 NA h  1
```

</td><td>

```r
with_tbl(tbl[[3]] <- 4:1)
#> # A tibble: 4 x 3
#>       n c        li
#>   <int> <chr> <int>
#> 1     1 e         4
#> 2    NA f         3
#> 3     3 g         2
#> 4    NA h         1
```

</td></tr><tr style="vertical-align:top"><td>

```r
with_df2(df2[[1]] <- 0)
#>   tb m.1 m.2 m.3 m.4
#> 1  0   1   0   0   0
#> 2  0   0   1   0   0
#> 3  0   0   0   1   0
#> 4  0   0   0   0   1
```

</td><td>

```r
with_tbl2(tbl2[[1]] <- 0)
#> # A tibble: 4 x 2
#>      tb m[,1]  [,2]  [,3]  [,4]
#>   <dbl> <dbl> <dbl> <dbl> <dbl>
#> 1     0     1     0     0     0
#> 2     0     0     1     0     0
#> 3     0     0     0     1     0
#> 4     0     0     0     0     1
```

</td></tr><tr style="vertical-align:top"><td>

```r
with_df2(df2[[2]] <- 4:1)
#>   tb.n tb.c      tb.li m
#> 1    1    e          9 4
#> 2   NA    f     10, 11 3
#> 3    3    g 12, 13, 14 2
#> 4   NA    h       text 1
```

</td><td>

```r
with_tbl2(tbl2[[2]] <- 4:1)
#> # A tibble: 4 x 2
#>    tb$n $c    $li           m
#>   <int> <chr> <list>    <int>
#> 1     1 e     <dbl [1]>     4
#> 2    NA f     <int [2]>     3
#> 3     3 g     <int [3]>     2
#> 4    NA h     <chr [1]>     1
```

</td></tr></tbody></table>

<table class="dftbl"><tbody><tr style="vertical-align:top"><td>

```r
with_df(df[[1]] <- 0)
#>   n c         li
#> 1 0 e          9
#> 2 0 f     10, 11
#> 3 0 g 12, 13, 14
#> 4 0 h       text
```

</td><td>

```r
with_tbl(tbl[[1]] <- 0)
#> # A tibble: 4 x 3
#>       n c     li       
#>   <dbl> <chr> <list>   
#> 1     0 e     <dbl [1]>
#> 2     0 f     <int [2]>
#> 3     0 g     <int [3]>
#> 4     0 h     <chr [1]>
```

</td></tr><tr style="vertical-align:top"><td>

```r
with_df(df[["c"]] <- 0)
#>    n c         li
#> 1  1 0          9
#> 2 NA 0     10, 11
#> 3  3 0 12, 13, 14
#> 4 NA 0       text
```

</td><td>

```r
with_tbl(tbl[["c"]] <- 0)
#> # A tibble: 4 x 3
#>       n     c li       
#>   <int> <dbl> <list>   
#> 1     1     0 <dbl [1]>
#> 2    NA     0 <int [2]>
#> 3     3     0 <int [3]>
#> 4    NA     0 <chr [1]>
```

</td></tr></tbody></table>

<table class="dftbl"><tbody><tr style="vertical-align:top"><td>

```r
with_df(df[[TRUE]] <- 0)
#>   n c         li
#> 1 0 e          9
#> 2 0 f     10, 11
#> 3 0 g 12, 13, 14
#> 4 0 h       text
```

</td><td>

```r
with_tbl(tbl[[TRUE]] <- 0)
#> Error in `[[<-`:
#> ! Can't assign column with `TRUE`.
#> x `TRUE` must be numeric or character, not `TRUE`.
```

</td></tr><tr style="vertical-align:top"><td>

```r
with_df(df[[1:3]] <- 0)
#> Error in `[[<-`(`*tmp*`, i, value = value): recursive indexing failed at level 2
```

</td><td>

```r
with_tbl(tbl[[1:3]] <- 0)
#> Error in `[[<-`:
#> ! Can't assign column with `1:3`.
#> x Subscript `1:3` must be size 1, not 3.
```

</td></tr><tr style="vertical-align:top"><td>

```r
with_df(df[[c("n", "c")]] <- 0)
#> Error in x[[i]] <- value: more elements supplied than there are to replace
```

</td><td>

```r
with_tbl(tbl[[c("n", "c")]] <- 0)
#> Error in `[[<-`:
#> ! Can't assign column with `c("n", "c")`.
#> x Subscript `c("n", "c")` must be size 1, not 2.
```

</td></tr><tr style="vertical-align:top"><td>

```r
with_df(df[[FALSE]] <- 0)
#> Error in x[[i]] <- value: attempt to select less than one element in integerOneIndex
```

</td><td>

```r
with_tbl(tbl[[FALSE]] <- 0)
#> Error in `[[<-`:
#> ! Can't assign column with `FALSE`.
#> x `FALSE` must be numeric or character, not `FALSE`.
```

</td></tr><tr style="vertical-align:top"><td>

```r
with_df(df[[1:2]] <- 0)
#> Error in x[[i]] <- value: more elements supplied than there are to replace
```

</td><td>

```r
with_tbl(tbl[[1:2]] <- 0)
#> Error in `[[<-`:
#> ! Can't assign column with `1:2`.
#> x Subscript `1:2` must be size 1, not 2.
```

</td></tr><tr style="vertical-align:top"><td>

```r
with_df(df[[NA_integer_]] <- 0)
#> Error in x[[i]] <- value: attempt to select more than one element in integerOneIndex
```

</td><td>

```r
with_tbl(tbl[[NA_integer_]] <- 0)
#> Error in `[[<-`:
#> ! Can't assign column with `NA_integer_`.
#> x Subscript `NA_integer_` must be a location, not an integer `NA`.
```

</td></tr><tr style="vertical-align:top"><td>

```r
with_df(df[[NA]] <- 0)
#> Error in x[[i]] <- value: attempt to select more than one element in integerOneIndex
```

</td><td>

```r
with_tbl(tbl[[NA]] <- 0)
#> Error in `[[<-`:
#> ! Can't assign column with `NA`.
#> x Subscript `NA` must be a location, not an integer `NA`.
```

</td></tr><tr style="vertical-align:top"><td>

```r
with_df(df[[NA_character_]] <- 0)
#> Error in if (names(x)[nc] == "") names(x)[nc] <- paste0("V", nc): missing value where TRUE/FALSE needed
```

</td><td>

```r
with_tbl(tbl[[NA_character_]] <- 0)
#> Error in `[[<-`:
#> ! Can't assign column with `NA_character_`.
#> x Subscript `NA_character_` must be a location, not a character `NA`.
```

</td></tr></tbody></table>

`a` is recycled to the same size as `x` so must have size `nrow(x)` or 1.
(The only exception is when `a` is `NULL`, as described below.)
Recycling also works for list, data frame, and matrix columns.

<table class="dftbl"><tbody><tr style="vertical-align:top"><td>

```r
with_df(df[["li"]] <- list(0))
#>    n c li
#> 1  1 e  0
#> 2 NA f  0
#> 3  3 g  0
#> 4 NA h  0
```

</td><td>

```r
with_tbl(tbl[["li"]] <- list(0))
#> # A tibble: 4 x 3
#>       n c     li       
#>   <int> <chr> <list>   
#> 1     1 e     <dbl [1]>
#> 2    NA f     <dbl [1]>
#> 3     3 g     <dbl [1]>
#> 4    NA h     <dbl [1]>
```

</td></tr><tr style="vertical-align:top"><td>

```r
with_df2(df2[["tb"]] <- df[1, ])
#> Error in `[[<-.data.frame`(`*tmp*`, "tb", value = structure(list(n = 1L, : replacement has 1 row, data has 4
```

</td><td>

```r
with_tbl2(tbl2[["tb"]] <- tbl[1, ])
#> # A tibble: 4 x 2
#>    tb$n $c    $li       m[,1]  [,2]
#>   <int> <chr> <list>    <dbl> <dbl>
#> 1     1 e     <dbl [1]>     1     0
#> 2     1 e     <dbl [1]>     0     1
#> 3     1 e     <dbl [1]>     0     0
#> 4     1 e     <dbl [1]>     0     0
#> # ... with 1 more variable:
#> #   m[3:4] <dbl>
```

</td></tr><tr style="vertical-align:top"><td>

```r
with_df2(df2[["m"]] <- df2[["m"]][1, , drop = FALSE])
#> Error in `[[<-.data.frame`(`*tmp*`, "m", value = structure(c(1, 0, 0, : replacement has 1 row, data has 4
```

</td><td>

```r
with_tbl2(tbl2[["m"]] <- tbl2[["m"]][1, , drop = FALSE])
#> # A tibble: 4 x 2
#>    tb$n $c    $li       m[,1]  [,2]
#>   <int> <chr> <list>    <dbl> <dbl>
#> 1     1 e     <dbl [1]>     1     0
#> 2    NA f     <int [2]>     1     0
#> 3     3 g     <int [3]>     1     0
#> 4    NA h     <chr [1]>     1     0
#> # ... with 1 more variable:
#> #   m[3:4] <dbl>
```

</td></tr></tbody></table>

<table class="dftbl"><tbody><tr style="vertical-align:top"><td>

```r
with_df(df[[1]] <- 1)
#>   n c         li
#> 1 1 e          9
#> 2 1 f     10, 11
#> 3 1 g 12, 13, 14
#> 4 1 h       text
```

</td><td>

```r
with_tbl(tbl[[1]] <- 1)
#> # A tibble: 4 x 3
#>       n c     li       
#>   <dbl> <chr> <list>   
#> 1     1 e     <dbl [1]>
#> 2     1 f     <int [2]>
#> 3     1 g     <int [3]>
#> 4     1 h     <chr [1]>
```

</td></tr><tr style="vertical-align:top"><td>

```r
with_df(df[[1]] <- 4:1)
#>   n c         li
#> 1 4 e          9
#> 2 3 f     10, 11
#> 3 2 g 12, 13, 14
#> 4 1 h       text
```

</td><td>

```r
with_tbl(tbl[[1]] <- 4:1)
#> # A tibble: 4 x 3
#>       n c     li       
#>   <int> <chr> <list>   
#> 1     4 e     <dbl [1]>
#> 2     3 f     <int [2]>
#> 3     2 g     <int [3]>
#> 4     1 h     <chr [1]>
```

</td></tr><tr style="vertical-align:top"><td>

```r
with_df(df[[1]] <- 3:1)
#> Error in `[[<-.data.frame`(`*tmp*`, 1, value = 3:1): replacement has 3 rows, data has 4
```

</td><td>

```r
with_tbl(tbl[[1]] <- 3:1)
#> Error in `[[<-`:
#> ! Assigned data `3:1` must be compatible with existing data.
#> x Existing data has 4 rows.
#> x Assigned data has 3 rows.
#> i Only vectors of size 1 are recycled.
#> Caused by error in `vectbl_recycle_rhs_rows()`:
#> ! Can't recycle input of size 3 to size 4.
```

</td></tr><tr style="vertical-align:top"><td>

```r
with_df(df[[1]] <- 2:1)
#>   n c         li
#> 1 2 e          9
#> 2 1 f     10, 11
#> 3 2 g 12, 13, 14
#> 4 1 h       text
```

</td><td>

```r
with_tbl(tbl[[1]] <- 2:1)
#> Error in `[[<-`:
#> ! Assigned data `2:1` must be compatible with existing data.
#> x Existing data has 4 rows.
#> x Assigned data has 2 rows.
#> i Only vectors of size 1 are recycled.
#> Caused by error in `vectbl_recycle_rhs_rows()`:
#> ! Can't recycle input of size 2 to size 4.
```

</td></tr></tbody></table>

`j` must be a scalar numeric or a string, and cannot be `NA`.
If `j` is OOB, a new column is added on the right hand side, with name repair if needed.

<table class="dftbl"><tbody><tr style="vertical-align:top"><td>

```r
with_df(df[["x"]] <- 0)
#>    n c         li x
#> 1  1 e          9 0
#> 2 NA f     10, 11 0
#> 3  3 g 12, 13, 14 0
#> 4 NA h       text 0
```

</td><td>

```r
with_tbl(tbl[["x"]] <- 0)
#> # A tibble: 4 x 4
#>       n c     li            x
#>   <int> <chr> <list>    <dbl>
#> 1     1 e     <dbl [1]>     0
#> 2    NA f     <int [2]>     0
#> 3     3 g     <int [3]>     0
#> 4    NA h     <chr [1]>     0
```

</td></tr><tr style="vertical-align:top"><td>

```r
with_df(df[[4]] <- 0)
#>    n c         li V4
#> 1  1 e          9  0
#> 2 NA f     10, 11  0
#> 3  3 g 12, 13, 14  0
#> 4 NA h       text  0
```

</td><td>

```r
with_tbl(tbl[[4]] <- 0)
#> # A tibble: 4 x 4
#>       n c     li         ...4
#>   <int> <chr> <list>    <dbl>
#> 1     1 e     <dbl [1]>     0
#> 2    NA f     <int [2]>     0
#> 3     3 g     <int [3]>     0
#> 4    NA h     <chr [1]>     0
```

</td></tr><tr style="vertical-align:top"><td>

```r
with_df(df[[5]] <- 0)
#> Warning in format.data.frame(if
#> (omit) x[seq_len(n0), , drop =
#> FALSE] else x, : corrupt data
#> frame: columns will be truncated or
#> padded with NAs
#>    n c         li      V5
#> 1  1 e          9 NULL  0
#> 2 NA f     10, 11 <NA>  0
#> 3  3 g 12, 13, 14 <NA>  0
#> 4 NA h       text <NA>  0
```

</td><td>

```r
with_tbl(tbl[[5]] <- 0)
#> Error in `[[<-`:
#> ! Can't assign to columns beyond the end with non-consecutive locations.
#> i Input has size 3.
#> x Subscript `5` contains non-consecutive location 5.
```

</td></tr></tbody></table>

<!-- HW: should we permitted oob assignment with numeric j? It's a bit weird to create a column with unknonw column -->

`df[[j]] <- a` replaces the complete column so can change the type.

<table class="dftbl"><tbody><tr style="vertical-align:top"><td>

```r
with_df(df[[1]] <- df[[2]])
#>   n c         li
#> 1 e e          9
#> 2 f f     10, 11
#> 3 g g 12, 13, 14
#> 4 h h       text
```

</td><td>

```r
with_tbl(tbl[[1]] <- tbl[[2]])
#> # A tibble: 4 x 3
#>   n     c     li       
#>   <chr> <chr> <list>   
#> 1 e     e     <dbl [1]>
#> 2 f     f     <int [2]>
#> 3 g     g     <int [3]>
#> 4 h     h     <chr [1]>
```

</td></tr><tr style="vertical-align:top"><td>

```r
with_df(df[[2]] <- df[[3]])
#>    n          c         li
#> 1  1          9          9
#> 2 NA     10, 11     10, 11
#> 3  3 12, 13, 14 12, 13, 14
#> 4 NA       text       text
```

</td><td>

```r
with_tbl(tbl[[2]] <- tbl[[3]])
#> # A tibble: 4 x 3
#>       n c         li       
#>   <int> <list>    <list>   
#> 1     1 <dbl [1]> <dbl [1]>
#> 2    NA <int [2]> <int [2]>
#> 3     3 <int [3]> <int [3]>
#> 4    NA <chr [1]> <chr [1]>
```

</td></tr><tr style="vertical-align:top"><td>

```r
with_df(df[[3]] <- df2[[1]])
#>    n c li.n li.c      li.li
#> 1  1 e    1    e          9
#> 2 NA f   NA    f     10, 11
#> 3  3 g    3    g 12, 13, 14
#> 4 NA h   NA    h       text
```

</td><td>

```r
with_tbl(tbl[[3]] <- tbl2[[1]])
#> # A tibble: 4 x 3
#>       n c      li$n $c    $li      
#>   <int> <chr> <int> <chr> <list>   
#> 1     1 e         1 e     <dbl [1]>
#> 2    NA f        NA f     <int [2]>
#> 3     3 g         3 g     <int [3]>
#> 4    NA h        NA h     <chr [1]>
```

</td></tr><tr style="vertical-align:top"><td>

```r
with_df2(df2[[1]] <- df2[[2]])
#>   tb.1 tb.2 tb.3 tb.4 m.1 m.2 m.3
#> 1    1    0    0    0   1   0   0
#> 2    0    1    0    0   0   1   0
#> 3    0    0    1    0   0   0   1
#> 4    0    0    0    1   0   0   0
#>   m.4
#> 1   0
#> 2   0
#> 3   0
#> 4   1
```

</td><td>

```r
with_tbl2(tbl2[[1]] <- tbl2[[2]])
#> # A tibble: 4 x 2
#>   tb[,1]  [,2]  [,3]  [,4] m[,1]
#>    <dbl> <dbl> <dbl> <dbl> <dbl>
#> 1      1     0     0     0     1
#> 2      0     1     0     0     0
#> 3      0     0     1     0     0
#> 4      0     0     0     1     0
#> # ... with 1 more variable:
#> #   m[2:4] <dbl>
```

</td></tr><tr style="vertical-align:top"><td>

```r
with_df2(df2[[2]] <- df[[1]])
#>   tb.n tb.c      tb.li  m
#> 1    1    e          9  1
#> 2   NA    f     10, 11 NA
#> 3    3    g 12, 13, 14  3
#> 4   NA    h       text NA
```

</td><td>

```r
with_tbl2(tbl2[[2]] <- tbl[[1]])
#> # A tibble: 4 x 2
#>    tb$n $c    $li           m
#>   <int> <chr> <list>    <int>
#> 1     1 e     <dbl [1]>     1
#> 2    NA f     <int [2]>    NA
#> 3     3 g     <int [3]>     3
#> 4    NA h     <chr [1]>    NA
```

</td></tr></tbody></table>

`[[<-` supports removing a column by assigning `NULL` to it.

<table class="dftbl"><tbody><tr style="vertical-align:top"><td>

```r
with_df(df[[1]] <- NULL)
#>   c         li
#> 1 e          9
#> 2 f     10, 11
#> 3 g 12, 13, 14
#> 4 h       text
```

</td><td>

```r
with_tbl(tbl[[1]] <- NULL)
#> # A tibble: 4 x 2
#>   c     li       
#>   <chr> <list>   
#> 1 e     <dbl [1]>
#> 2 f     <int [2]>
#> 3 g     <int [3]>
#> 4 h     <chr [1]>
```

</td></tr><tr style="vertical-align:top"><td>

```r
with_df2(df2[[2]] <- NULL)
#>   tb.n tb.c      tb.li
#> 1    1    e          9
#> 2   NA    f     10, 11
#> 3    3    g 12, 13, 14
#> 4   NA    h       text
```

</td><td>

```r
with_tbl2(tbl2[[2]] <- NULL)
#> # A tibble: 4 x 1
#>    tb$n $c    $li      
#>   <int> <chr> <list>   
#> 1     1 e     <dbl [1]>
#> 2    NA f     <int [2]>
#> 3     3 g     <int [3]>
#> 4    NA h     <chr [1]>
```

</td></tr></tbody></table>

Removing a nonexistent column is a no-op.

<table class="dftbl"><tbody><tr style="vertical-align:top"><td>

```r
with_df(df[["q"]] <- NULL)
#>    n c         li
#> 1  1 e          9
#> 2 NA f     10, 11
#> 3  3 g 12, 13, 14
#> 4 NA h       text
```

</td><td>

```r
with_tbl(tbl[["q"]] <- NULL)
#> # A tibble: 4 x 3
#>       n c     li       
#>   <int> <chr> <list>   
#> 1     1 e     <dbl [1]>
#> 2    NA f     <int [2]>
#> 3     3 g     <int [3]>
#> 4    NA h     <chr [1]>
```

</td></tr></tbody></table>

### Definition of `x$name <- a`

`x$name <- a` and `x$"name" <- a` are equivalent to `x[["name"]] <- a`.[^column-assign-symmetry]

[^column-assign-symmetry]: `$` behaves almost completely symmetrically to `[[` when comparing subsetting and subassignment.

<table class="dftbl"><tbody><tr style="vertical-align:top"><td>

```r
with_df(df$n <- 0)
#>   n c         li
#> 1 0 e          9
#> 2 0 f     10, 11
#> 3 0 g 12, 13, 14
#> 4 0 h       text
```

</td><td>

```r
with_tbl(tbl$n <- 0)
#> # A tibble: 4 x 3
#>       n c     li       
#>   <dbl> <chr> <list>   
#> 1     0 e     <dbl [1]>
#> 2     0 f     <int [2]>
#> 3     0 g     <int [3]>
#> 4     0 h     <chr [1]>
```

</td></tr><tr style="vertical-align:top"><td>

```r
with_df(df[["n"]] <- 0)
#>   n c         li
#> 1 0 e          9
#> 2 0 f     10, 11
#> 3 0 g 12, 13, 14
#> 4 0 h       text
```

</td><td>

```r
with_tbl(tbl[["n"]] <- 0)
#> # A tibble: 4 x 3
#>       n c     li       
#>   <dbl> <chr> <list>   
#> 1     0 e     <dbl [1]>
#> 2     0 f     <int [2]>
#> 3     0 g     <int [3]>
#> 4     0 h     <chr [1]>
```

</td></tr></tbody></table>

<table class="dftbl"><tbody><tr style="vertical-align:top"><td>

```r
with_df(df$"n" <- 0)
#>   n c         li
#> 1 0 e          9
#> 2 0 f     10, 11
#> 3 0 g 12, 13, 14
#> 4 0 h       text
```

</td><td>

```r
with_tbl(tbl$"n" <- 0)
#> # A tibble: 4 x 3
#>       n c     li       
#>   <dbl> <chr> <list>   
#> 1     0 e     <dbl [1]>
#> 2     0 f     <int [2]>
#> 3     0 g     <int [3]>
#> 4     0 h     <chr [1]>
```

</td></tr></tbody></table>

`$<-` does not perform partial matching.

<table class="dftbl"><tbody><tr style="vertical-align:top"><td>

```r
with_df(df$l <- 0)
#>    n c         li l
#> 1  1 e          9 0
#> 2 NA f     10, 11 0
#> 3  3 g 12, 13, 14 0
#> 4 NA h       text 0
```

</td><td>

```r
with_tbl(tbl$l <- 0)
#> # A tibble: 4 x 4
#>       n c     li            l
#>   <int> <chr> <list>    <dbl>
#> 1     1 e     <dbl [1]>     0
#> 2    NA f     <int [2]>     0
#> 3     3 g     <int [3]>     0
#> 4    NA h     <chr [1]>     0
```

</td></tr><tr style="vertical-align:top"><td>

```r
with_df(df[["l"]] <- 0)
#>    n c         li l
#> 1  1 e          9 0
#> 2 NA f     10, 11 0
#> 3  3 g 12, 13, 14 0
#> 4 NA h       text 0
```

</td><td>

```r
with_tbl(tbl[["l"]] <- 0)
#> # A tibble: 4 x 4
#>       n c     li            l
#>   <int> <chr> <list>    <dbl>
#> 1     1 e     <dbl [1]>     0
#> 2    NA f     <int [2]>     0
#> 3     3 g     <int [3]>     0
#> 4    NA h     <chr [1]>     0
```

</td></tr></tbody></table>

## Column subassignment: `x[j] <- a`

* If `j` is missing, it's replaced with `seq_along(x)`
* If `j` is logical vector, it's converted to numeric with `seq_along(x)[j]`.

### `a` is a list or data frame

If `inherits(a, "list")` or `inherits(a, "data.frame")` is `TRUE`, then `x[j] <- a` is equivalent to `x[[j[[1]]] <- a[[1]]`, `x[[j[[2]]]] <- a[[2]]`, ...

<table class="dftbl"><tbody><tr style="vertical-align:top"><td>

```r
with_df(df[1:2] <- list("x", 4:1))
#>   n c         li
#> 1 x 4          9
#> 2 x 3     10, 11
#> 3 x 2 12, 13, 14
#> 4 x 1       text
```

</td><td>

```r
with_tbl(tbl[1:2] <- list("x", 4:1))
#> # A tibble: 4 x 3
#>   n         c li       
#>   <chr> <int> <list>   
#> 1 x         4 <dbl [1]>
#> 2 x         3 <int [2]>
#> 3 x         2 <int [3]>
#> 4 x         1 <chr [1]>
```

</td></tr><tr style="vertical-align:top"><td>

```r
with_df(df[c("li", "x", "c")] <- list("x", 4:1, NULL))
#>    n li x
#> 1  1  x 4
#> 2 NA  x 3
#> 3  3  x 2
#> 4 NA  x 1
```

</td><td>

```r
with_tbl(tbl[c("li", "x", "c")] <- list("x", 4:1, NULL))
#> # A tibble: 4 x 3
#>       n li        x
#>   <int> <chr> <int>
#> 1     1 x         4
#> 2    NA x         3
#> 3     3 x         2
#> 4    NA x         1
```

</td></tr></tbody></table>

If `length(a)` equals 1, then it is recycled to the same length as `j`.

<table class="dftbl"><tbody><tr style="vertical-align:top"><td>

```r
with_df(df[1:2] <- list(1))
#>   n c         li
#> 1 1 1          9
#> 2 1 1     10, 11
#> 3 1 1 12, 13, 14
#> 4 1 1       text
```

</td><td>

```r
with_tbl(tbl[1:2] <- list(1))
#> # A tibble: 4 x 3
#>       n     c li       
#>   <dbl> <dbl> <list>   
#> 1     1     1 <dbl [1]>
#> 2     1     1 <int [2]>
#> 3     1     1 <int [3]>
#> 4     1     1 <chr [1]>
```

</td></tr><tr style="vertical-align:top"><td>

```r
with_df(df[1:2] <- list(0, 0, 0))
#> Warning in
#> `[<-.data.frame`(`*tmp*`, 1:2,
#> value = list(0, 0, 0)): provided 3
#> variables to replace 2 variables
#>   n c         li
#> 1 0 0          9
#> 2 0 0     10, 11
#> 3 0 0 12, 13, 14
#> 4 0 0       text
```

</td><td>

```r
with_tbl(tbl[1:2] <- list(0, 0, 0))
#> Error in `[<-`:
#> ! Can't recycle `list(0, 0, 0)` (size 3) to size 2.
```

</td></tr><tr style="vertical-align:top"><td>

```r
with_df(df[1:3] <- list(0, 0))
#>   n c li
#> 1 0 0  0
#> 2 0 0  0
#> 3 0 0  0
#> 4 0 0  0
```

</td><td>

```r
with_tbl(tbl[1:3] <- list(0, 0))
#> Error in `[<-`:
#> ! Can't recycle `list(0, 0)` (size 2) to size 3.
```

</td></tr></tbody></table>

An attempt to update the same column twice gives an error.

<table class="dftbl"><tbody><tr style="vertical-align:top"><td>

```r
with_df(df[c(1, 1)] <- list(1, 2))
#> Error in `[<-.data.frame`(`*tmp*`, c(1, 1), value = list(1, 2)): duplicate subscripts for columns
```

</td><td>

```r
with_tbl(tbl[c(1, 1)] <- list(1, 2))
#> Error in `[<-`:
#> ! Column index 1 is used more than once for assignment.
```

</td></tr></tbody></table>

If `a` contains `NULL` values, the corresponding columns are removed *after* updating (i.e. position indexes refer to columns before any modifications).

<table class="dftbl"><tbody><tr style="vertical-align:top"><td>

```r
with_df(df[1:2] <- list(NULL, 4:1))
#>   c         li
#> 1 4          9
#> 2 3     10, 11
#> 3 2 12, 13, 14
#> 4 1       text
```

</td><td>

```r
with_tbl(tbl[1:2] <- list(NULL, 4:1))
#> # A tibble: 4 x 2
#>       c li       
#>   <int> <list>   
#> 1     4 <dbl [1]>
#> 2     3 <int [2]>
#> 3     2 <int [3]>
#> 4     1 <chr [1]>
```

</td></tr></tbody></table>

`NA` indexes are not supported.

<table class="dftbl"><tbody><tr style="vertical-align:top"><td>

```r
with_df(df[NA] <- list("x"))
#> Error in `[<-.data.frame`(`*tmp*`, NA, value = list("x")): missing values are not allowed in subscripted assignments of data frames
```

</td><td>

```r
with_tbl(tbl[NA] <- list("x"))
#> Error in `[<-`:
#> ! Can't assign columns with `NA`.
#> x Subscript `NA` can't contain missing values.
#> x It has a missing value at location 1.
```

</td></tr><tr style="vertical-align:top"><td>

```r
with_df(df[NA_integer_] <- list("x"))
#> Error in `[<-.data.frame`(`*tmp*`, NA_integer_, value = list("x")): missing values are not allowed in subscripted assignments of data frames
```

</td><td>

```r
with_tbl(tbl[NA_integer_] <- list("x"))
#> Error in `[<-`:
#> ! Can't use NA as column index in a tibble for assignment.
```

</td></tr><tr style="vertical-align:top"><td>

```r
with_df(df[NA_character_] <- list("x"))
#> Error in `[<-.data.frame`(`*tmp*`, NA_character_, value = list("x")): missing values are not allowed in subscripted assignments of data frames
```

</td><td>

```r
with_tbl(tbl[NA_character_] <- list("x"))
#> Error in `[<-`:
#> ! Can't use NA as column index in a tibble for assignment.
```

</td></tr></tbody></table>

Just like column updates, `[<-` supports changing the type of an existing column.

<table class="dftbl"><tbody><tr style="vertical-align:top"><td>

```r
with_df(df[1] <- df[2])
#>   n c         li
#> 1 e e          9
#> 2 f f     10, 11
#> 3 g g 12, 13, 14
#> 4 h h       text
```

</td><td>

```r
with_tbl(tbl[1] <- tbl[2])
#> # A tibble: 4 x 3
#>   n     c     li       
#>   <chr> <chr> <list>   
#> 1 e     e     <dbl [1]>
#> 2 f     f     <int [2]>
#> 3 g     g     <int [3]>
#> 4 h     h     <chr [1]>
```

</td></tr><tr style="vertical-align:top"><td>

```r
with_df(df[2] <- df[3])
#>    n          c         li
#> 1  1          9          9
#> 2 NA     10, 11     10, 11
#> 3  3 12, 13, 14 12, 13, 14
#> 4 NA       text       text
```

</td><td>

```r
with_tbl(tbl[2] <- tbl[3])
#> # A tibble: 4 x 3
#>       n c         li       
#>   <int> <list>    <list>   
#> 1     1 <dbl [1]> <dbl [1]>
#> 2    NA <int [2]> <int [2]>
#> 3     3 <int [3]> <int [3]>
#> 4    NA <chr [1]> <chr [1]>
```

</td></tr><tr style="vertical-align:top"><td>

```r
with_df(df[3] <- df2[1])
#>    n c li.n li.c      li.li
#> 1  1 e    1    e          9
#> 2 NA f   NA    f     10, 11
#> 3  3 g    3    g 12, 13, 14
#> 4 NA h   NA    h       text
```

</td><td>

```r
with_tbl(tbl[3] <- tbl2[1])
#> # A tibble: 4 x 3
#>       n c      li$n $c    $li      
#>   <int> <chr> <int> <chr> <list>   
#> 1     1 e         1 e     <dbl [1]>
#> 2    NA f        NA f     <int [2]>
#> 3     3 g         3 g     <int [3]>
#> 4    NA h        NA h     <chr [1]>
```

</td></tr><tr style="vertical-align:top"><td>

```r
with_df2(df2[1] <- df2[2])
#>   tb.1 tb.2 tb.3 tb.4 m.1 m.2 m.3
#> 1    1    0    0    0   1   0   0
#> 2    0    1    0    0   0   1   0
#> 3    0    0    1    0   0   0   1
#> 4    0    0    0    1   0   0   0
#>   m.4
#> 1   0
#> 2   0
#> 3   0
#> 4   1
```

</td><td>

```r
with_tbl2(tbl2[1] <- tbl2[2])
#> # A tibble: 4 x 2
#>   tb[,1]  [,2]  [,3]  [,4] m[,1]
#>    <dbl> <dbl> <dbl> <dbl> <dbl>
#> 1      1     0     0     0     1
#> 2      0     1     0     0     0
#> 3      0     0     1     0     0
#> 4      0     0     0     1     0
#> # ... with 1 more variable:
#> #   m[2:4] <dbl>
```

</td></tr><tr style="vertical-align:top"><td>

```r
with_df2(df2[2] <- df[1])
#>   tb.n tb.c      tb.li  m
#> 1    1    e          9  1
#> 2   NA    f     10, 11 NA
#> 3    3    g 12, 13, 14  3
#> 4   NA    h       text NA
```

</td><td>

```r
with_tbl2(tbl2[2] <- tbl[1])
#> # A tibble: 4 x 2
#>    tb$n $c    $li           m
#>   <int> <chr> <list>    <int>
#> 1     1 e     <dbl [1]>     1
#> 2    NA f     <int [2]>    NA
#> 3     3 g     <int [3]>     3
#> 4    NA h     <chr [1]>    NA
```

</td></tr></tbody></table>

Appending columns at the end (without gaps) is supported.
The name of new columns is determined by the LHS, the RHS, or by name repair (in that order of precedence).

<table class="dftbl"><tbody><tr style="vertical-align:top"><td>

```r
with_df(df[c("x", "y")] <- tibble("x", x = 4:1))
#>    n c         li x y
#> 1  1 e          9 x 4
#> 2 NA f     10, 11 x 3
#> 3  3 g 12, 13, 14 x 2
#> 4 NA h       text x 1
```

</td><td>

```r
with_tbl(tbl[c("x", "y")] <- tibble("x", x = 4:1))
#> # A tibble: 4 x 5
#>       n c     li        x         y
#>   <int> <chr> <list>    <chr> <int>
#> 1     1 e     <dbl [1]> x         4
#> 2    NA f     <int [2]> x         3
#> 3     3 g     <int [3]> x         2
#> 4    NA h     <chr [1]> x         1
```

</td></tr><tr style="vertical-align:top"><td>

```r
with_df(df[3:4] <- list("x", x = 4:1))
#>    n c li x
#> 1  1 e  x 4
#> 2 NA f  x 3
#> 3  3 g  x 2
#> 4 NA h  x 1
```

</td><td>

```r
with_tbl(tbl[3:4] <- list("x", x = 4:1))
#> # A tibble: 4 x 4
#>       n c     li        x
#>   <int> <chr> <chr> <int>
#> 1     1 e     x         4
#> 2    NA f     x         3
#> 3     3 g     x         2
#> 4    NA h     x         1
```

</td></tr><tr style="vertical-align:top"><td>

```r
with_df(df[4] <- list(4:1))
#>    n c         li V4
#> 1  1 e          9  4
#> 2 NA f     10, 11  3
#> 3  3 g 12, 13, 14  2
#> 4 NA h       text  1
```

</td><td>

```r
with_tbl(tbl[4] <- list(4:1))
#> # A tibble: 4 x 4
#>       n c     li         ...4
#>   <int> <chr> <list>    <int>
#> 1     1 e     <dbl [1]>     4
#> 2    NA f     <int [2]>     3
#> 3     3 g     <int [3]>     2
#> 4    NA h     <chr [1]>     1
```

</td></tr><tr style="vertical-align:top"><td>

```r
with_df(df[5] <- list(4:1))
#> Error in `[<-.data.frame`(`*tmp*`, 5, value = list(4:1)): new columns would leave holes after existing columns
```

</td><td>

```r
with_tbl(tbl[5] <- list(4:1))
#> Error in `[<-`:
#> ! Can't assign to columns beyond the end with non-consecutive locations.
#> i Input has size 3.
#> x Subscript `5` contains non-consecutive location 5.
```

</td></tr></tbody></table>

Tibbles support indexing by a logical matrix, but only for a scalar RHS, and if all columns updated are compatible with the value assigned.

<table class="dftbl"><tbody><tr style="vertical-align:top"><td>

```r
with_df(df[is.na(df)] <- 4)
#>   n c         li
#> 1 1 e          9
#> 2 4 f     10, 11
#> 3 3 g 12, 13, 14
#> 4 4 h       text
```

</td><td>

```r
with_tbl(tbl[is.na(tbl)] <- 4)
#> # A tibble: 4 x 3
#>       n c     li       
#>   <int> <chr> <list>   
#> 1     1 e     <dbl [1]>
#> 2     4 f     <int [2]>
#> 3     3 g     <int [3]>
#> 4     4 h     <chr [1]>
```

</td></tr><tr style="vertical-align:top"><td>

```r
with_df(df[is.na(df)] <- 1:2)
#>   n c         li
#> 1 1 e          9
#> 2 1 f     10, 11
#> 3 3 g 12, 13, 14
#> 4 2 h       text
```

</td><td>

```r
with_tbl(tbl[is.na(tbl)] <- 1:2)
#> Error in `[<-`:
#> ! Subscript `is.na(tbl)` is a matrix, the data `1:2` must have size 1.
```

</td></tr><tr style="vertical-align:top"><td>

```r
with_df(df[matrix(c(rep(TRUE, 5), rep(FALSE, 7)), ncol = 3)] <- 4)
#>   n c         li
#> 1 4 4          9
#> 2 4 f     10, 11
#> 3 4 g 12, 13, 14
#> 4 4 h       text
```

</td><td>

```r
with_tbl(tbl[matrix(c(rep(TRUE, 5), rep(FALSE, 7)), ncol = 3)] <- 4)
#> Error in `[<-`:
#> ! Assigned data `4` must be compatible with existing data.
#> i Error occurred for column `c`.
#> Caused by error in `vec_assign()`:
#> ! Can't convert <double> to <character>.
```

</td></tr></tbody></table>

### `a` is a matrix or array

If `is.matrix(a)`, then `a` is coerced to a data frame with `as.data.frame()` before assigning.
If rows are assigned, the matrix type must be compatible with all columns.
If `is.array(a)` and `any(dim(a)[-1:-2] != 1)`, an error is thrown.

<table class="dftbl"><tbody><tr style="vertical-align:top"><td>

```r
with_df(df[1:2] <- matrix(8:1, ncol = 2))
#>   n c         li
#> 1 8 4          9
#> 2 7 3     10, 11
#> 3 6 2 12, 13, 14
#> 4 5 1       text
```

</td><td>

```r
with_tbl(tbl[1:2] <- matrix(8:1, ncol = 2))
#> # A tibble: 4 x 3
#>       n     c li       
#>   <int> <int> <list>   
#> 1     8     4 <dbl [1]>
#> 2     7     3 <int [2]>
#> 3     6     2 <int [3]>
#> 4     5     1 <chr [1]>
```

</td></tr><tr style="vertical-align:top"><td>

```r
with_df(df[1:3, 1:2] <- matrix(6:1, ncol = 2))
#>    n c         li
#> 1  6 3          9
#> 2  5 2     10, 11
#> 3  4 1 12, 13, 14
#> 4 NA h       text
```

</td><td>

```r
with_tbl(tbl[1:3, 1:2] <- matrix(6:1, ncol = 2))
#> Error in `[<-`:
#> ! Assigned data `matrix(6:1, ncol = 2)` must be compatible with existing data.
#> i Error occurred for column `c`.
#> Caused by error in `vec_assign()`:
#> ! Can't convert <integer> to <character>.
```

</td></tr><tr style="vertical-align:top"><td>

```r
with_df(df[1:2] <- array(4:1, dim = c(4, 1, 1)))
#>   n c         li
#> 1 4 4          9
#> 2 3 3     10, 11
#> 3 2 2 12, 13, 14
#> 4 1 1       text
```

</td><td>

```r
with_tbl(tbl[1:2] <- array(4:1, dim = c(4, 1, 1)))
#> # A tibble: 4 x 3
#>       n     c li       
#>   <int> <int> <list>   
#> 1     4     4 <dbl [1]>
#> 2     3     3 <int [2]>
#> 3     2     2 <int [3]>
#> 4     1     1 <chr [1]>
```

</td></tr><tr style="vertical-align:top"><td>

```r
with_df(df[1:2] <- array(8:1, dim = c(4, 2, 1)))
#>   n c         li
#> 1 8 4          9
#> 2 7 3     10, 11
#> 3 6 2 12, 13, 14
#> 4 5 1       text
```

</td><td>

```r
with_tbl(tbl[1:2] <- array(8:1, dim = c(4, 2, 1)))
#> # A tibble: 4 x 3
#>       n     c li       
#>   <int> <int> <list>   
#> 1     8     4 <dbl [1]>
#> 2     7     3 <int [2]>
#> 3     6     2 <int [3]>
#> 4     5     1 <chr [1]>
```

</td></tr><tr style="vertical-align:top"><td>

```r
with_df(df[1:2] <- array(8:1, dim = c(2, 1, 4)))
#>   n c         li
#> 1 8 4          9
#> 2 7 3     10, 11
#> 3 6 2 12, 13, 14
#> 4 5 1       text
```

</td><td>

```r
with_tbl(tbl[1:2] <- array(8:1, dim = c(2, 1, 4)))
#> Error in `[<-`:
#> ! `array(8:1, dim = c(2, 1, 4))` must be a vector, a bare list, a data frame, a matrix, or NULL.
```

</td></tr><tr style="vertical-align:top"><td>

```r
with_df(df[1:2] <- array(8:1, dim = c(4, 1, 2)))
#>   n c         li
#> 1 8 4          9
#> 2 7 3     10, 11
#> 3 6 2 12, 13, 14
#> 4 5 1       text
```

</td><td>

```r
with_tbl(tbl[1:2] <- array(8:1, dim = c(4, 1, 2)))
#> Error in `[<-`:
#> ! `array(8:1, dim = c(4, 1, 2))` must be a vector, a bare list, a data frame, a matrix, or NULL.
```

</td></tr></tbody></table>

### `a` is another type of vector

If `vec_is(a)`, then `x[j] <- a` is equivalent to `x[j] <- list(a)`.
This is primarily provided for backward compatibility.

<table class="dftbl"><tbody><tr style="vertical-align:top"><td>

```r
with_df(df[1] <- 0)
#>   n c         li
#> 1 0 e          9
#> 2 0 f     10, 11
#> 3 0 g 12, 13, 14
#> 4 0 h       text
```

</td><td>

```r
with_tbl(tbl[1] <- 0)
#> # A tibble: 4 x 3
#>       n c     li       
#>   <dbl> <chr> <list>   
#> 1     0 e     <dbl [1]>
#> 2     0 f     <int [2]>
#> 3     0 g     <int [3]>
#> 4     0 h     <chr [1]>
```

</td></tr><tr style="vertical-align:top"><td>

```r
with_df(df[1] <- list(0))
#>   n c         li
#> 1 0 e          9
#> 2 0 f     10, 11
#> 3 0 g 12, 13, 14
#> 4 0 h       text
```

</td><td>

```r
with_tbl(tbl[1] <- list(0))
#> # A tibble: 4 x 3
#>       n c     li       
#>   <dbl> <chr> <list>   
#> 1     0 e     <dbl [1]>
#> 2     0 f     <int [2]>
#> 3     0 g     <int [3]>
#> 4     0 h     <chr [1]>
```

</td></tr></tbody></table>

Matrices must be wrapped in `list()` before assignment to create a matrix column.

<table class="dftbl"><tbody><tr style="vertical-align:top"><td>

```r
with_df(df[1] <- list(matrix(1:8, ncol = 2)))
#>   n.1 n.2 c         li
#> 1   1   5 e          9
#> 2   2   6 f     10, 11
#> 3   3   7 g 12, 13, 14
#> 4   4   8 h       text
```

</td><td>

```r
with_tbl(tbl[1] <- list(matrix(1:8, ncol = 2)))
#> # A tibble: 4 x 3
#>   n[,1]  [,2] c     li       
#>   <int> <int> <chr> <list>   
#> 1     1     5 e     <dbl [1]>
#> 2     2     6 f     <int [2]>
#> 3     3     7 g     <int [3]>
#> 4     4     8 h     <chr [1]>
```

</td></tr><tr style="vertical-align:top"><td>


</td><td>


</td></tr><tr style="vertical-align:top"><td>

```r
with_df(df[1:2] <- list(matrix(1:8, ncol = 2)))
#>   n.1 n.2 c.1 c.2         li
#> 1   1   5   1   5          9
#> 2   2   6   2   6     10, 11
#> 3   3   7   3   7 12, 13, 14
#> 4   4   8   4   8       text
```

</td><td>

```r
with_tbl(tbl[1:2] <- list(matrix(1:8, ncol = 2)))
#> # A tibble: 4 x 3
#>   n[,1]  [,2] c[,1]  [,2] li       
#>   <int> <int> <int> <int> <list>   
#> 1     1     5     1     5 <dbl [1]>
#> 2     2     6     2     6 <int [2]>
#> 3     3     7     3     7 <int [3]>
#> 4     4     8     4     8 <chr [1]>
```

</td></tr></tbody></table>

### `a` is `NULL`

Entire columns can be removed.
Specifying `i` is an error.

<table class="dftbl"><tbody><tr style="vertical-align:top"><td>

```r
with_df(df[1] <- NULL)
#>   c         li
#> 1 e          9
#> 2 f     10, 11
#> 3 g 12, 13, 14
#> 4 h       text
```

</td><td>

```r
with_tbl(tbl[1] <- NULL)
#> # A tibble: 4 x 2
#>   c     li       
#>   <chr> <list>   
#> 1 e     <dbl [1]>
#> 2 f     <int [2]>
#> 3 g     <int [3]>
#> 4 h     <chr [1]>
```

</td></tr><tr style="vertical-align:top"><td>

```r
with_df(df[, 2:3] <- NULL)
#>    n
#> 1  1
#> 2 NA
#> 3  3
#> 4 NA
```

</td><td>

```r
with_tbl(tbl[, 2:3] <- NULL)
#> # A tibble: 4 x 1
#>       n
#>   <int>
#> 1     1
#> 2    NA
#> 3     3
#> 4    NA
```

</td></tr><tr style="vertical-align:top"><td>

```r
with_df(df[1, 2:3] <- NULL)
#> Error in x[[jj]][iseq] <- vjj: replacement has length zero
```

</td><td>

```r
with_tbl(tbl[1, 2:3] <- NULL)
#> Error in `[<-`:
#> ! `NULL` must be a vector, a bare list, a data frame or a matrix.
```

</td></tr></tbody></table>

### `a` is not a vector

Any other type for `a` is an error.
Note that if `is.list(a)` is `TRUE`, but `inherits(a, "list")` is `FALSE`, then `a` is considered to be a scalar.
See `?vec_is` and `?vec_proxy` for details.

<table class="dftbl"><tbody><tr style="vertical-align:top"><td>

```r
with_df(df[1] <- mean)
#> Error in rep(value, length.out = n): attempt to replicate an object of type 'closure'
```

</td><td>

```r
with_tbl(tbl[1] <- mean)
#> Error in `[<-`:
#> ! `mean` must be a vector, a bare list, a data frame, a matrix, or NULL.
```

</td></tr><tr style="vertical-align:top"><td>

```r
with_df(df[1] <- lm(mpg ~ wt, data = mtcars))
#> Warning in
#> `[<-.data.frame`(`*tmp*`, 1, value
#> = structure(list(coefficients =
#> c(`(Intercept)` = 37.285126167342,
#> : replacement element 2 has 32 rows
#> to replace 4 rows
#> Warning in
#> `[<-.data.frame`(`*tmp*`, 1, value
#> = structure(list(coefficients =
#> c(`(Intercept)` = 37.285126167342,
#> : replacement element 3 has 32 rows
#> to replace 4 rows
#> Warning in
#> `[<-.data.frame`(`*tmp*`, 1, value
#> = structure(list(coefficients =
#> c(`(Intercept)` = 37.285126167342,
#> : replacement element 5 has 32 rows
#> to replace 4 rows
#> Warning in
#> `[<-.data.frame`(`*tmp*`, 1, value
#> = structure(list(coefficients =
#> c(`(Intercept)` = 37.285126167342,
#> : replacement element 7 has 5 rows
#> to replace 4 rows
#> Error in `[<-.data.frame`(`*tmp*`, 1, value = structure(list(coefficients = c(`(Intercept)` = 37.285126167342, : replacement element 10 has 3 rows, need 4
```

</td><td>

```r
with_tbl(tbl[1] <- lm(mpg ~ wt, data = mtcars))
#> Error in `[<-`:
#> ! `lm(mpg ~ wt, data = mtcars)` must be a vector, a bare list, a data frame, a matrix, or NULL.
```

</td></tr></tbody></table>

<!-- HW: we need better error messages for these cases -->

## Row subassignment: `x[i, ] <- list(...)`

`x[i, ] <- a` is the same as `vec_slice(x[[j_1]], i) <- a[[1]]`,  `vec_slice(x[[j_2]], i) <- a[[2]]`, ... .[^row-assign-symmetry]

[^row-assign-symmetry]: `x[i, ]` is symmetrical for subset and subassignment.

<table class="dftbl"><tbody><tr style="vertical-align:top"><td>

```r
with_df(df[2:3, ] <- df[1, ])
#>    n c   li
#> 1  1 e    9
#> 2  1 e    9
#> 3  1 e    9
#> 4 NA h text
```

</td><td>

```r
with_tbl(tbl[2:3, ] <- tbl[1, ])
#> # A tibble: 4 x 3
#>       n c     li       
#>   <int> <chr> <list>   
#> 1     1 e     <dbl [1]>
#> 2     1 e     <dbl [1]>
#> 3     1 e     <dbl [1]>
#> 4    NA h     <chr [1]>
```

</td></tr><tr style="vertical-align:top"><td>

```r
with_df(df[c(FALSE, TRUE, TRUE, FALSE), ] <- df[1, ])
#>    n c   li
#> 1  1 e    9
#> 2  1 e    9
#> 3  1 e    9
#> 4 NA h text
```

</td><td>

```r
with_tbl(tbl[c(FALSE, TRUE, TRUE, FALSE), ] <- tbl[1, ])
#> # A tibble: 4 x 3
#>       n c     li       
#>   <int> <chr> <list>   
#> 1     1 e     <dbl [1]>
#> 2     1 e     <dbl [1]>
#> 3     1 e     <dbl [1]>
#> 4    NA h     <chr [1]>
```

</td></tr></tbody></table>

<table class="dftbl"><tbody><tr style="vertical-align:top"><td>

```r
with_df(df[0:2, ] <- df[1, ])
#>    n c         li
#> 1  1 e          9
#> 2  1 e          9
#> 3  3 g 12, 13, 14
#> 4 NA h       text
```

</td><td>

```r
with_tbl(tbl[0:2, ] <- tbl[1, ])
#> Error in `[<-`:
#> ! Can't assign rows with `0:2`.
#> x Subscript `0:2` can't contain `0` values.
#> i It has a `0` value at location 1.
```

</td></tr><tr style="vertical-align:top"><td>

```r
with_df(df[0, ] <- df[1, ])
#>    n c         li
#> 1  1 e          9
#> 2 NA f     10, 11
#> 3  3 g 12, 13, 14
#> 4 NA h       text
```

</td><td>

```r
with_tbl(tbl[0, ] <- tbl[1, ])
#> Error in `[<-`:
#> ! Can't assign rows with `0`.
#> x Subscript `0` can't contain `0` values.
#> i It has a `0` value at location 1.
```

</td></tr><tr style="vertical-align:top"><td>

```r
with_df(df[-2, ] <- df[1, ])
#>    n c     li
#> 1  1 e      9
#> 2 NA f 10, 11
#> 3  1 e      9
#> 4  1 e      9
```

</td><td>

```r
with_tbl(tbl[-2, ] <- tbl[1, ])
#> # A tibble: 4 x 3
#>       n c     li       
#>   <int> <chr> <list>   
#> 1     1 e     <dbl [1]>
#> 2    NA f     <int [2]>
#> 3     1 e     <dbl [1]>
#> 4     1 e     <dbl [1]>
```

</td></tr><tr style="vertical-align:top"><td>

```r
with_df(df[-1:2, ] <- df[1, ])
#> Error in seq_len(nrows)[i]: only 0's may be mixed with negative subscripts
```

</td><td>

```r
with_tbl(tbl[-1:2, ] <- tbl[1, ])
#> Error in `[<-`:
#> ! Can't assign rows with `-1:2`.
#> x Subscript `-1:2` can't contain `0` values.
#> i It has a `0` value at location 2.
```

</td></tr><tr style="vertical-align:top"><td>

```r
with_df(df[NA_integer_, ] <- df[1, ])
#> Error in `[<-.data.frame`(`*tmp*`, NA_integer_, , value = structure(list(: missing values are not allowed in subscripted assignments of data frames
```

</td><td>

```r
with_tbl(tbl[NA_integer_, ] <- tbl[1, ])
#> Error in `[<-`:
#> ! Can't assign rows with `NA_integer_`.
#> x Subscript `NA_integer_` can't contain missing values.
#> x It has a missing value at location 1.
```

</td></tr><tr style="vertical-align:top"><td>

```r
with_df2(df2[NA_integer_, ] <- df2[1, ])
#> Error in `[<-.data.frame`(`*tmp*`, NA_integer_, , value = structure(list(: missing values are not allowed in subscripted assignments of data frames
```

</td><td>

```r
with_tbl2(tbl2[NA_integer_, ] <- tbl2[1, ])
#> Error in `[<-`:
#> ! Can't assign rows with `NA_integer_`.
#> x Subscript `NA_integer_` can't contain missing values.
#> x It has a missing value at location 1.
```

</td></tr><tr style="vertical-align:top"><td>

```r
with_df(df[TRUE, ] <- df[1, ])
#>   n c li
#> 1 1 e  9
#> 2 1 e  9
#> 3 1 e  9
#> 4 1 e  9
```

</td><td>

```r
with_tbl(tbl[TRUE, ] <- tbl[1, ])
#> # A tibble: 4 x 3
#>       n c     li       
#>   <int> <chr> <list>   
#> 1     1 e     <dbl [1]>
#> 2     1 e     <dbl [1]>
#> 3     1 e     <dbl [1]>
#> 4     1 e     <dbl [1]>
```

</td></tr><tr style="vertical-align:top"><td>

```r
with_df(df[FALSE, ] <- df[1, ])
#>    n c         li
#> 1  1 e          9
#> 2 NA f     10, 11
#> 3  3 g 12, 13, 14
#> 4 NA h       text
```

</td><td>

```r
with_tbl(tbl[FALSE, ] <- tbl[1, ])
#> # A tibble: 4 x 3
#>       n c     li       
#>   <int> <chr> <list>   
#> 1     1 e     <dbl [1]>
#> 2    NA f     <int [2]>
#> 3     3 g     <int [3]>
#> 4    NA h     <chr [1]>
```

</td></tr><tr style="vertical-align:top"><td>

```r
with_df(df[NA, ] <- df[1, ])
#> Error in `[<-.data.frame`(`*tmp*`, NA, , value = structure(list(n = 1L, : missing values are not allowed in subscripted assignments of data frames
```

</td><td>

```r
with_tbl(tbl[NA, ] <- tbl[1, ])
#> Error in `[<-`:
#> ! Can't assign rows with `NA`.
#> x Subscript `NA` can't contain missing values.
#> x It has a missing value at location 1.
```

</td></tr></tbody></table>

Only values of size one can be recycled.

<table class="dftbl"><tbody><tr style="vertical-align:top"><td>

```r
with_df(df[2:3, ] <- df[1, ])
#>    n c   li
#> 1  1 e    9
#> 2  1 e    9
#> 3  1 e    9
#> 4 NA h text
```

</td><td>

```r
with_tbl(tbl[2:3, ] <- tbl[1, ])
#> # A tibble: 4 x 3
#>       n c     li       
#>   <int> <chr> <list>   
#> 1     1 e     <dbl [1]>
#> 2     1 e     <dbl [1]>
#> 3     1 e     <dbl [1]>
#> 4    NA h     <chr [1]>
```

</td></tr><tr style="vertical-align:top"><td>

```r
with_df(df[2:3, ] <- list(df$n[1], df$c[1:2], df$li[1]))
#>    n c   li
#> 1  1 e    9
#> 2  1 e    9
#> 3  1 f    9
#> 4 NA h text
```

</td><td>

```r
with_tbl(tbl[2:3, ] <- list(tbl$n[1], tbl$c[1:2], tbl$li[1]))
#> # A tibble: 4 x 3
#>       n c     li       
#>   <int> <chr> <list>   
#> 1     1 e     <dbl [1]>
#> 2     1 e     <dbl [1]>
#> 3     1 f     <dbl [1]>
#> 4    NA h     <chr [1]>
```

</td></tr><tr style="vertical-align:top"><td>

```r
with_df(df[2:4, ] <- df[1:2, ])
#> Error in `[<-.data.frame`(`*tmp*`, 2:4, , value = structure(list(n = c(1L, : replacement element 1 has 2 rows, need 3
```

</td><td>

```r
with_tbl(tbl[2:4, ] <- tbl[1:2, ])
#> Error in `[<-`:
#> ! Assigned data `tbl[1:2, ]` must be compatible with row subscript `2:4`.
#> x 3 rows must be assigned.
#> x Element 1 of assigned data has 2 rows.
#> i Only vectors of size 1 are recycled.
#> Caused by error in `vectbl_recycle_rhs_rows()`:
#> ! Can't recycle input of size 2 to size 3.
```

</td></tr></tbody></table>

<table class="dftbl"><tbody><tr style="vertical-align:top"><td>

```r
with_df2(df2[2:4, ] <- df2[1, ])
#> Error in `[<-.data.frame`(`*tmp*`, 2:4, , value = structure(list(tb = structure(list(: replacement element 1 is a matrix/data frame of 1 row, need 3
```

</td><td>

```r
with_tbl2(tbl2[2:4, ] <- tbl2[1, ])
#> # A tibble: 4 x 2
#>    tb$n $c    $li       m[,1]  [,2]
#>   <int> <chr> <list>    <dbl> <dbl>
#> 1     1 e     <dbl [1]>     1     0
#> 2     1 e     <dbl [1]>     1     0
#> 3     1 e     <dbl [1]>     1     0
#> 4     1 e     <dbl [1]>     1     0
#> # ... with 1 more variable:
#> #   m[3:4] <dbl>
```

</td></tr><tr style="vertical-align:top"><td>

```r
with_df2(df2[2:4, ] <- df2[2:3, ])
#> Error in `[<-.data.frame`(`*tmp*`, 2:4, , value = structure(list(tb = structure(list(: replacement element 1 is a matrix/data frame of 2 rows, need 3
```

</td><td>

```r
with_tbl2(tbl2[2:4, ] <- tbl2[2:3, ])
#> Error in `[<-`:
#> ! Assigned data `tbl2[2:3, ]` must be compatible with row subscript `2:4`.
#> x 3 rows must be assigned.
#> x Element 1 of assigned data has 2 rows.
#> i Only vectors of size 1 are recycled.
#> Caused by error in `vectbl_recycle_rhs_rows()`:
#> ! Can't recycle input of size 2 to size 3.
```

</td></tr></tbody></table>

For compatibility, only a warning is issued for indexing beyond the number of rows.
Appending rows right at the end of the existing data is supported, without warning.

<table class="dftbl"><tbody><tr style="vertical-align:top"><td>

```r
with_df(df[5, ] <- df[1, ])
#>    n c         li
#> 1  1 e          9
#> 2 NA f     10, 11
#> 3  3 g 12, 13, 14
#> 4 NA h       text
#> 5  1 e          9
```

</td><td>

```r
with_tbl(tbl[5, ] <- tbl[1, ])
#> # A tibble: 5 x 3
#>       n c     li       
#>   <int> <chr> <list>   
#> 1     1 e     <dbl [1]>
#> 2    NA f     <int [2]>
#> 3     3 g     <int [3]>
#> 4    NA h     <chr [1]>
#> 5     1 e     <dbl [1]>
```

</td></tr><tr style="vertical-align:top"><td>

```r
with_df(df[5:7, ] <- df[1, ])
#>    n c         li
#> 1  1 e          9
#> 2 NA f     10, 11
#> 3  3 g 12, 13, 14
#> 4 NA h       text
#> 5  1 e          9
#> 6  1 e          9
#> 7  1 e          9
```

</td><td>

```r
with_tbl(tbl[5:7, ] <- tbl[1, ])
#> # A tibble: 7 x 3
#>       n c     li       
#>   <int> <chr> <list>   
#> 1     1 e     <dbl [1]>
#> 2    NA f     <int [2]>
#> 3     3 g     <int [3]>
#> 4    NA h     <chr [1]>
#> 5     1 e     <dbl [1]>
#> 6     1 e     <dbl [1]>
#> 7     1 e     <dbl [1]>
```

</td></tr><tr style="vertical-align:top"><td>

```r
with_df(df[6, ] <- df[1, ])
#>    n    c         li
#> 1  1    e          9
#> 2 NA    f     10, 11
#> 3  3    g 12, 13, 14
#> 4 NA    h       text
#> 5 NA <NA>       NULL
#> 6  1    e          9
```

</td><td>

```r
with_tbl(tbl[6, ] <- tbl[1, ])
#> Error in `[<-`:
#> ! Can't assign to rows beyond the end with non-consecutive locations.
#> i Input has size 4.
#> x Subscript `6` contains non-consecutive location 6.
```

</td></tr><tr style="vertical-align:top"><td>

```r
with_df(df[-5, ] <- df[1, ])
#>   n c li
#> 1 1 e  9
#> 2 1 e  9
#> 3 1 e  9
#> 4 1 e  9
```

</td><td>

```r
with_tbl(tbl[-5, ] <- tbl[1, ])
#> Error in `[<-`:
#> ! Can't negate rows past the end.
#> i Location 5 doesn't exist.
#> i There are only 4 rows.
```

</td></tr><tr style="vertical-align:top"><td>

```r
with_df(df[-(5:7), ] <- df[1, ])
#>   n c li
#> 1 1 e  9
#> 2 1 e  9
#> 3 1 e  9
#> 4 1 e  9
```

</td><td>

```r
with_tbl(tbl[-(5:7), ] <- tbl[1, ])
#> Error in `[<-`:
#> ! Can't negate rows past the end.
#> i Locations 5, 6, and 7 don't exist.
#> i There are only 4 rows.
```

</td></tr><tr style="vertical-align:top"><td>

```r
with_df(df[-6, ] <- df[1, ])
#>   n c li
#> 1 1 e  9
#> 2 1 e  9
#> 3 1 e  9
#> 4 1 e  9
```

</td><td>

```r
with_tbl(tbl[-6, ] <- tbl[1, ])
#> Error in `[<-`:
#> ! Can't negate rows past the end.
#> i Location 6 doesn't exist.
#> i There are only 4 rows.
```

</td></tr></tbody></table>

For compatibility, `i` can also be a character vector containing positive numbers.

<table class="dftbl"><tbody><tr style="vertical-align:top"><td>

```r
with_df(df[as.character(1:3), ] <- df[1, ])
#>    n c   li
#> 1  1 e    9
#> 2  1 e    9
#> 3  1 e    9
#> 4 NA h text
```

</td><td>

```r
with_tbl(tbl[as.character(1:3), ] <- tbl[1, ])
#> # A tibble: 4 x 3
#>       n c     li       
#>   <int> <chr> <list>   
#> 1     1 e     <dbl [1]>
#> 2     1 e     <dbl [1]>
#> 3     1 e     <dbl [1]>
#> 4    NA h     <chr [1]>
```

</td></tr></tbody></table>

<table class="dftbl"><tbody><tr style="vertical-align:top"><td>

```r
with_df(df[as.character(-(1:3)), ] <- df[1, ])
#>     n c         li
#> 1   1 e          9
#> 2  NA f     10, 11
#> 3   3 g 12, 13, 14
#> 4  NA h       text
#> -1  1 e          9
#> -2  1 e          9
#> -3  1 e          9
```

</td><td>

```r
with_tbl(tbl[as.character(-(1:3)), ] <- tbl[1, ])
#> Warning: The `i` argument of `[.tbl_df` must use valid row names as of tibble 3.0.0.
#> i Use `NA_integer_` as row index to obtain a row full of `NA` values.
#> Error in `[<-`:
#> ! Can't use NA as row index in a tibble for assignment.
```

</td></tr><tr style="vertical-align:top"><td>

```r
with_df(df[as.character(3:5), ] <- df[1, ])
#>    n c     li
#> 1  1 e      9
#> 2 NA f 10, 11
#> 3  1 e      9
#> 4  1 e      9
#> 5  1 e      9
```

</td><td>

```r
with_tbl(tbl[as.character(3:5), ] <- tbl[1, ])
#> Warning: The `i` argument of `[.tbl_df` must use valid row names as of tibble 3.0.0.
#> i Use `NA_integer_` as row index to obtain a row full of `NA` values.
#> Error in `[<-`:
#> ! Can't use NA as row index in a tibble for assignment.
```

</td></tr><tr style="vertical-align:top"><td>

```r
with_df(df[as.character(-(3:5)), ] <- df[1, ])
#>     n c         li
#> 1   1 e          9
#> 2  NA f     10, 11
#> 3   3 g 12, 13, 14
#> 4  NA h       text
#> -3  1 e          9
#> -4  1 e          9
#> -5  1 e          9
```

</td><td>

```r
with_tbl(tbl[as.character(-(3:5)), ] <- tbl[1, ])
#> Warning: The `i` argument of `[.tbl_df` must use valid row names as of tibble 3.0.0.
#> i Use `NA_integer_` as row index to obtain a row full of `NA` values.
#> Error in `[<-`:
#> ! Can't use NA as row index in a tibble for assignment.
```

</td></tr><tr style="vertical-align:top"><td>

```r
with_df(df[NA_character_, ] <- df[1, ])
#> Error in `[<-.data.frame`(`*tmp*`, NA_character_, , value = structure(list(: missing values are not allowed in subscripted assignments of data frames
```

</td><td>

```r
with_tbl(tbl[NA_character_, ] <- tbl[1, ])
#> Error in `[<-`:
#> ! Can't use NA as row index in a tibble for assignment.
```

</td></tr></tbody></table>

## Row and column subassignment

### Definition of `x[i, j] <- a`

`x[i, j] <- a` is equivalent to `x[i, ][j] <- a`.[^bracket-assign-flip]

[^bracket-assign-flip]: `x[i, j]` is symmetrical for subsetting and subassignment.
A more efficient implementation of `x[i, j] <- a` would forward to `x[j][i, ] <- a`.

Subassignment to `x[i, j]` is stricter for tibbles than for data frames.
`x[i, j] <- a` can't change the data type of existing columns.

<table class="dftbl"><tbody><tr style="vertical-align:top"><td>

```r
with_df(df[2:3, 1] <- df[1:2, 2])
#>      n c         li
#> 1    1 e          9
#> 2    e f     10, 11
#> 3    f g 12, 13, 14
#> 4 <NA> h       text
```

</td><td>

```r
with_tbl(tbl[2:3, 1] <- tbl[1:2, 2])
#> Error in `[<-`:
#> ! Assigned data `tbl[1:2, 2]` must be compatible with existing data.
#> i Error occurred for column `n`.
#> Caused by error in `vec_assign()`:
#> ! Can't convert <character> to <integer>.
```

</td></tr><tr style="vertical-align:top"><td>

```r
with_df(df[2:3, 2] <- df[1:2, 3])
#> Warning in
#> `[<-.data.frame`(`*tmp*`, 2:3, 2,
#> value = list(9, 10:11)): provided 2
#> variables to replace 1 variables
#>    n c         li
#> 1  1 e          9
#> 2 NA 9     10, 11
#> 3  3 9 12, 13, 14
#> 4 NA h       text
```

</td><td>

```r
with_tbl(tbl[2:3, 2] <- tbl[1:2, 3])
#> Error in `[<-`:
#> ! Assigned data `tbl[1:2, 3]` must be compatible with existing data.
#> i Error occurred for column `c`.
#> Caused by error in `vec_assign()`:
#> ! Can't convert <list> to <character>.
```

</td></tr><tr style="vertical-align:top"><td>

```r
with_df(df[2:3, 3] <- df2[1:2, 1])
#> Warning in
#> `[<-.data.frame`(`*tmp*`, 2:3, 3,
#> value = structure(list(n = c(1L, :
#> provided 3 variables to replace 1
#> variables
#>    n c   li
#> 1  1 e    9
#> 2 NA f    1
#> 3  3 g   NA
#> 4 NA h text
```

</td><td>

```r
with_tbl(tbl[2:3, 3] <- tbl2[1:2, 1])
#> Error in `[<-`:
#> ! Assigned data `tbl2[1:2, 1]` must be compatible with existing data.
#> i Error occurred for column `li`.
#> Caused by error in `vec_assign()`:
#> ! Can't convert <tbl_df> to <list>.
```

</td></tr><tr style="vertical-align:top"><td>

```r
with_df2(df2[2:3, 1] <- df2[1:2, 2])
#> Warning in matrix(value, n, p):
#> data length [8] is not a
#> sub-multiple or multiple of the
#> number of columns [3]
#>   tb.n tb.c tb.li m.1 m.2 m.3 m.4
#> 1    1    e     9   1   0   0   0
#> 2    1    0     0   0   1   0   0
#> 3    0    1     0   0   0   1   0
#> 4   NA    h  text   0   0   0   1
```

</td><td>

```r
with_tbl2(tbl2[2:3, 1] <- tbl2[1:2, 2])
#> Error in `[<-`:
#> ! Assigned data `tbl2[1:2, 2]` must be compatible with existing data.
#> i Error occurred for column `tb`.
#> Caused by error in `vec_assign()`:
#> ! Can't convert <double[,4]> to <tbl_df>.
```

</td></tr><tr style="vertical-align:top"><td>

```r
with_df2(df2[2:3, 2] <- df[1:2, 1])
#>   tb.n tb.c      tb.li m.1 m.2 m.3
#> 1    1    e          9   1   0   0
#> 2   NA    f     10, 11   1   1   1
#> 3    3    g 12, 13, 14  NA  NA  NA
#> 4   NA    h       text   0   0   0
#>   m.4
#> 1   0
#> 2   1
#> 3  NA
#> 4   1
```

</td><td>

```r
with_tbl2(tbl2[2:3, 2] <- tbl[1:2, 1])
#> # A tibble: 4 x 2
#>    tb$n $c    $li       m[,1]  [,2]
#>   <int> <chr> <list>    <dbl> <dbl>
#> 1     1 e     <dbl [1]>     1     0
#> 2    NA f     <int [2]>     1     1
#> 3     3 g     <int [3]>    NA    NA
#> 4    NA h     <chr [1]>     0     0
#> # ... with 1 more variable:
#> #   m[3:4] <dbl>
```

</td></tr></tbody></table>

A notable exception is the population of a column full of `NA` (which is of type `logical`), or the use of `NA` on the right-hand side of the assignment.

<table class="dftbl"><tbody><tr style="vertical-align:top"><td>

```r
with_df({df$x <- NA; df[2:3, "x"] <- 3:2})
#>    n c         li  x
#> 1  1 e          9 NA
#> 2 NA f     10, 11  3
#> 3  3 g 12, 13, 14  2
#> 4 NA h       text NA
```

</td><td>

```r
with_tbl({tbl$x <- NA; tbl[2:3, "x"] <- 3:2})
#> # A tibble: 4 x 4
#>       n c     li            x
#>   <int> <chr> <list>    <int>
#> 1     1 e     <dbl [1]>    NA
#> 2    NA f     <int [2]>     3
#> 3     3 g     <int [3]>     2
#> 4    NA h     <chr [1]>    NA
```

</td></tr><tr style="vertical-align:top"><td>

```r
with_df({df[2:3, 2:3] <- NA})
#>    n    c   li
#> 1  1    e    9
#> 2 NA <NA>   NA
#> 3  3 <NA>   NA
#> 4 NA    h text
```

</td><td>

```r
with_tbl({tbl[2:3, 2:3] <- NA})
#> # A tibble: 4 x 3
#>       n c     li       
#>   <int> <chr> <list>   
#> 1     1 e     <dbl [1]>
#> 2    NA <NA>  <NULL>   
#> 3     3 <NA>  <NULL>   
#> 4    NA h     <chr [1]>
```

</td></tr></tbody></table>

For programming, it is always safer (and faster) to use the correct type of `NA` to initialize columns.

<table class="dftbl"><tbody><tr style="vertical-align:top"><td>

```r
with_df({df$x <- NA_integer_; df[2:3, "x"] <- 3:2})
#>    n c         li  x
#> 1  1 e          9 NA
#> 2 NA f     10, 11  3
#> 3  3 g 12, 13, 14  2
#> 4 NA h       text NA
```

</td><td>

```r
with_tbl({tbl$x <- NA_integer_; tbl[2:3, "x"] <- 3:2})
#> # A tibble: 4 x 4
#>       n c     li            x
#>   <int> <chr> <list>    <int>
#> 1     1 e     <dbl [1]>    NA
#> 2    NA f     <int [2]>     3
#> 3     3 g     <int [3]>     2
#> 4    NA h     <chr [1]>    NA
```

</td></tr></tbody></table>


For new columns, `x[i, j] <- a` fills the unassigned rows with `NA`.

<table class="dftbl"><tbody><tr style="vertical-align:top"><td>

```r
with_df(df[2:3, "n"] <- 1)
#>    n c         li
#> 1  1 e          9
#> 2  1 f     10, 11
#> 3  1 g 12, 13, 14
#> 4 NA h       text
```

</td><td>

```r
with_tbl(tbl[2:3, "n"] <- 1)
#> # A tibble: 4 x 3
#>       n c     li       
#>   <int> <chr> <list>   
#> 1     1 e     <dbl [1]>
#> 2     1 f     <int [2]>
#> 3     1 g     <int [3]>
#> 4    NA h     <chr [1]>
```

</td></tr><tr style="vertical-align:top"><td>

```r
with_df(df[2:3, "x"] <- 1)
#>    n c         li  x
#> 1  1 e          9 NA
#> 2 NA f     10, 11  1
#> 3  3 g 12, 13, 14  1
#> 4 NA h       text NA
```

</td><td>

```r
with_tbl(tbl[2:3, "x"] <- 1)
#> # A tibble: 4 x 4
#>       n c     li            x
#>   <int> <chr> <list>    <dbl>
#> 1     1 e     <dbl [1]>    NA
#> 2    NA f     <int [2]>     1
#> 3     3 g     <int [3]>     1
#> 4    NA h     <chr [1]>    NA
```

</td></tr><tr style="vertical-align:top"><td>

```r
with_df(df[2:3, "n"] <- NULL)
#> Error in x[[jj]][iseq] <- vjj: replacement has length zero
```

</td><td>

```r
with_tbl(tbl[2:3, "n"] <- NULL)
#> Error in `[<-`:
#> ! `NULL` must be a vector, a bare list, a data frame or a matrix.
```

</td></tr></tbody></table>

Likewise, for new rows, `x[i, j] <- a` fills the unassigned columns with `NA`.

<table class="dftbl"><tbody><tr style="vertical-align:top"><td>

```r
with_df(df[5, "n"] <- list(0L))
#>    n    c         li
#> 1  1    e          9
#> 2 NA    f     10, 11
#> 3  3    g 12, 13, 14
#> 4 NA    h       text
#> 5  0 <NA>       NULL
```

</td><td>

```r
with_tbl(tbl[5, "n"] <- list(0L))
#> # A tibble: 5 x 3
#>       n c     li       
#>   <int> <chr> <list>   
#> 1     1 e     <dbl [1]>
#> 2    NA f     <int [2]>
#> 3     3 g     <int [3]>
#> 4    NA h     <chr [1]>
#> 5     0 <NA>  <NULL>
```

</td></tr></tbody></table>


### Definition of `x[[i, j]] <- a`

`i` must be a numeric vector of length 1.
`x[[i, j]] <- a` is equivalent to `x[i, ][[j]] <- a`.[^double-bracket-ij-symmetry]

[^double-bracket-ij-symmetry]: `x[[i, j]]` is symmetrical for subsetting and subassignment.
An efficient implementation would check that `i` and `j` are scalar and forward to `x[i, j][[1]] <- a`.


<table class="dftbl"><tbody><tr style="vertical-align:top"><td>

```r
with_df(df[[1, 1]] <- 0)
#>    n c         li
#> 1  0 e          9
#> 2 NA f     10, 11
#> 3  3 g 12, 13, 14
#> 4 NA h       text
```

</td><td>

```r
with_tbl(tbl[[1, 1]] <- 0)
#> # A tibble: 4 x 3
#>       n c     li       
#>   <int> <chr> <list>   
#> 1     0 e     <dbl [1]>
#> 2    NA f     <int [2]>
#> 3     3 g     <int [3]>
#> 4    NA h     <chr [1]>
```

</td></tr><tr style="vertical-align:top"><td>

```r
with_df(df[1, ][[1]] <- 0)
#>    n c         li
#> 1  0 e          9
#> 2 NA f     10, 11
#> 3  3 g 12, 13, 14
#> 4 NA h       text
```

</td><td>

```r
with_tbl(tbl[1, ][[1]] <- 0)
#> # A tibble: 4 x 3
#>       n c     li       
#>   <int> <chr> <list>   
#> 1     0 e     <dbl [1]>
#> 2    NA f     <int [2]>
#> 3     3 g     <int [3]>
#> 4    NA h     <chr [1]>
```

</td></tr><tr style="vertical-align:top"><td>

```r
with_df(df[[1, 3]] <- list(NULL))
#>    n c         li
#> 1  1 e       NULL
#> 2 NA f     10, 11
#> 3  3 g 12, 13, 14
#> 4 NA h       text
```

</td><td>

```r
with_tbl(tbl[[1, 3]] <- list(NULL))
#> # A tibble: 4 x 3
#>       n c     li       
#>   <int> <chr> <list>   
#> 1     1 e     <NULL>   
#> 2    NA f     <int [2]>
#> 3     3 g     <int [3]>
#> 4    NA h     <chr [1]>
```

</td></tr><tr style="vertical-align:top"><td>

```r
with_df(df[1, ][[3]] <- list(NULL))
#>    n c         li
#> 1  1 e       NULL
#> 2 NA f     10, 11
#> 3  3 g 12, 13, 14
#> 4 NA h       text
```

</td><td>

```r
with_tbl(tbl[1, ][[3]] <- list(NULL))
#> # A tibble: 4 x 3
#>       n c     li       
#>   <int> <chr> <list>   
#> 1     1 e     <NULL>   
#> 2    NA f     <int [2]>
#> 3     3 g     <int [3]>
#> 4    NA h     <chr [1]>
```

</td></tr><tr style="vertical-align:top"><td>

```r
with_df2(df2[[1, 1]] <- df[1, ])
#> Error in `[[<-.data.frame`(`*tmp*`, iseq, value = structure(list(n = 1L, : replacement has 1 row, data has 4
```

</td><td>

```r
with_tbl2(tbl2[[1, 1]] <- tbl[1, ])
#> # A tibble: 4 x 2
#>    tb$n $c    $li       m[,1]  [,2]
#>   <int> <chr> <list>    <dbl> <dbl>
#> 1     1 e     <dbl [1]>     1     0
#> 2    NA f     <int [2]>     0     1
#> 3     3 g     <int [3]>     0     0
#> 4    NA h     <chr [1]>     0     0
#> # ... with 1 more variable:
#> #   m[3:4] <dbl>
```

</td></tr><tr style="vertical-align:top"><td>

```r
with_df2(df2[1, ][[1]] <- df[1, ])
#>   tb.n tb.c      tb.li m.1 m.2 m.3
#> 1    1    e          9   1   0   0
#> 2   NA    f     10, 11   0   1   0
#> 3    3    g 12, 13, 14   0   0   1
#> 4   NA    h       text   0   0   0
#>   m.4
#> 1   0
#> 2   0
#> 3   0
#> 4   1
```

</td><td>

```r
with_tbl2(tbl2[1, ][[1]] <- tbl[1, ])
#> # A tibble: 4 x 2
#>    tb$n $c    $li       m[,1]  [,2]
#>   <int> <chr> <list>    <dbl> <dbl>
#> 1     1 e     <dbl [1]>     1     0
#> 2    NA f     <int [2]>     0     1
#> 3     3 g     <int [3]>     0     0
#> 4    NA h     <chr [1]>     0     0
#> # ... with 1 more variable:
#> #   m[3:4] <dbl>
```

</td></tr><tr style="vertical-align:top"><td>

```r
with_df2(df2[[1, 2]] <- t(1:4))
#> Error in x[[jseq]][[iseq]] <- value: more elements supplied than there are to replace
```

</td><td>

```r
with_tbl2(tbl2[[1, 2]] <- t(1:4))
#> # A tibble: 4 x 2
#>    tb$n $c    $li       m[,1]  [,2]
#>   <int> <chr> <list>    <dbl> <dbl>
#> 1     1 e     <dbl [1]>     1     2
#> 2    NA f     <int [2]>     0     1
#> 3     3 g     <int [3]>     0     0
#> 4    NA h     <chr [1]>     0     0
#> # ... with 1 more variable:
#> #   m[3:4] <dbl>
```

</td></tr><tr style="vertical-align:top"><td>

```r
with_df2(df2[1, ][[2]] <- t(1:4))
#>   tb.n tb.c      tb.li m.1 m.2 m.3
#> 1    1    e          9   1   2   3
#> 2   NA    f     10, 11   0   1   0
#> 3    3    g 12, 13, 14   0   0   1
#> 4   NA    h       text   0   0   0
#>   m.4
#> 1   4
#> 2   0
#> 3   0
#> 4   1
```

</td><td>

```r
with_tbl2(tbl2[1, ][[2]] <- t(1:4))
#> # A tibble: 4 x 2
#>    tb$n $c    $li       m[,1]  [,2]
#>   <int> <chr> <list>    <dbl> <dbl>
#> 1     1 e     <dbl [1]>     1     2
#> 2    NA f     <int [2]>     0     1
#> 3     3 g     <int [3]>     0     0
#> 4    NA h     <chr [1]>     0     0
#> # ... with 1 more variable:
#> #   m[3:4] <dbl>
```

</td></tr><tr style="vertical-align:top"><td>

```r
df[[1:2, 1]]
#> Error in col[[i, exact = exact]]: attempt to select more than one element in vectorIndex
```

</td><td>

```r
tbl[[1:2, 1]]
#> Error in `tbl[[1:2, 1]]`:
#> ! Can't extract row with `1:2`.
#> x Subscript `1:2` must be size 1, not 2.
```

</td></tr><tr style="vertical-align:top"><td>

```r
with_df(df[[1:2, 1]] <- 0)
#> Error in `[[<-.data.frame`(`*tmp*`, 1:2, 1, value = 0): only a single element should be replaced
```

</td><td>

```r
with_tbl(tbl[[1:2, 1]] <- 0)
#> Error in `[[<-`:
#> ! Can't assign row with `1:2`.
#> x Subscript `1:2` must be size 1, not 2.
```

</td></tr></tbody></table>

NB: `vec_size(a)` must equal 1.
Unlike `x[i, ] <-`, `x[[i, ]] <-` is not valid.

# output test

    Code
      # # [.tbl_df is careful about names (#1245)
      foo <- tibble(x = 1:10, y = 1:10)
      foo[c("x", "y", "z")]
    Condition
      Error in `foo[c("x", "y", "z")]`:
      ! Can't subset columns that don't exist.
      x Column `z` doesn't exist.
    Code
      foo[c("w", "x", "y", "z")]
    Condition
      Error in `foo[c("w", "x", "y", "z")]`:
      ! Can't subset columns that don't exist.
      x Columns `w` and `z` don't exist.
    Code
      foo[as.matrix("x")]
    Condition
      Error in `foo[as.matrix("x")]`:
      ! Subscript `as.matrix("x")` is a matrix, it must be of type logical.
    Code
      foo[array("x", dim = c(1, 1, 1))]
    Condition
      Error in `foo[array("x", dim = c(1, 1, 1))]`:
      ! Can't subset columns with `array("x", dim = c(1, 1, 1))`.
      x Subscript `array("x", dim = c(1, 1, 1))` must be a simple vector, not an array.
    Code
      # # [.tbl_df is careful about column indexes (#83)
      foo <- tibble(x = 1:10, y = 1:10, z = 1:10)
      foo[0.5]
    Condition
      Error in `foo[0.5]`:
      ! Can't subset columns with `0.5`.
      x Can't convert from `j` <double> to <integer> due to loss of precision.
    Code
      foo[1:5]
    Condition
      Error in `foo[1:5]`:
      ! Can't subset columns past the end.
      i Locations 4 and 5 don't exist.
      i There are only 3 columns.
    Code
      foo[-1:1]
    Condition
      Error in `foo[-1:1]`:
      ! Can't subset columns with `-1:1`.
      x Negative and positive locations can't be mixed.
      i Subscript `-1:1` has a positive value at location 3.
    Code
      foo[c(-1, 1)]
    Condition
      Error in `foo[c(-1, 1)]`:
      ! Can't subset columns with `c(-1, 1)`.
      x Negative and positive locations can't be mixed.
      i Subscript `c(-1, 1)` has a positive value at location 2.
    Code
      foo[c(-1, NA)]
    Condition
      Error in `foo[c(-1, NA)]`:
      ! Can't subset columns with `c(-1, NA)`.
      x Negative locations can't have missing values.
      i Subscript `c(-1, NA)` has a missing value at location 2.
    Code
      foo[-4]
    Condition
      Error in `foo[-4]`:
      ! Can't negate columns past the end.
      i Location 4 doesn't exist.
      i There are only 3 columns.
    Code
      foo[c(1:3, NA)]
    Condition
      Error in `foo[c(1:3, NA)]`:
      ! Can't subset columns with `c(1:3, NA)`.
      x Subscript `c(1:3, NA)` can't contain missing values.
      x It has a missing value at location 4.
    Code
      foo[as.matrix(1)]
    Condition
      Error in `foo[as.matrix(1)]`:
      ! Subscript `as.matrix(1)` is a matrix, it must be of type logical.
    Code
      foo[array(1, dim = c(1, 1, 1))]
    Condition
      Error in `foo[array(1, dim = c(1, 1, 1))]`:
      ! Can't subset columns with `array(1, dim = c(1, 1, 1))`.
      x Subscript `array(1, dim = c(1, 1, 1))` must be a simple vector, not an array.
    Code
      foo[mean]
    Condition
      Error in `foo[mean]`:
      ! Can't subset columns with `mean`.
      x `mean` must be logical, numeric, or character, not a function.
    Code
      foo[foo]
    Condition
      Error in `foo[foo]`:
      ! Can't subset columns with `foo`.
      x `foo` must be logical, numeric, or character, not a <tbl_df/tbl/data.frame> object.
    Code
      # # [.tbl_df is careful about row indexes
      foo <- tibble(x = 1:3, y = 1:3, z = 1:3)
      foo[0.5, ]
    Condition
      Error in `foo[0.5, ]`:
      ! Can't subset rows with `0.5`.
      x Can't convert from `i` <double> to <integer> due to loss of precision.
    Code
      invisible(foo[1:5, ])
    Condition
      Warning:
      The `i` argument of `[.tbl_df` must lie in [0, rows] if positive, as of tibble 3.0.0.
      i Use `NA_integer_` as row index to obtain a row full of `NA` values.
    Code
      foo[-1:1, ]
    Condition
      Error in `foo[-1:1, ]`:
      ! Can't subset rows with `-1:1`.
      x Negative and positive locations can't be mixed.
      i Subscript `-1:1` has a positive value at location 3.
    Code
      foo[c(-1, 1), ]
    Condition
      Error in `foo[c(-1, 1), ]`:
      ! Can't subset rows with `c(-1, 1)`.
      x Negative and positive locations can't be mixed.
      i Subscript `c(-1, 1)` has a positive value at location 2.
    Code
      foo[c(-1, NA), ]
    Condition
      Error in `foo[c(-1, NA), ]`:
      ! Can't subset rows with `c(-1, NA)`.
      x Negative locations can't have missing values.
      i Subscript `c(-1, NA)` has a missing value at location 2.
    Code
      invisible(foo[-4, ])
    Condition
      Warning:
      The `i` argument of `[.tbl_df` must lie in [-rows, 0] if negative, as of tibble 3.0.0.
      i Use `NA_integer_` as row index to obtain a row full of `NA` values.
    Code
      foo[array(1, dim = c(1, 1, 1)), ]
    Condition
      Error in `foo[array(1, dim = c(1, 1, 1)), ]`:
      ! Can't subset rows with `array(1, dim = c(1, 1, 1))`.
      x Subscript `array(1, dim = c(1, 1, 1))` must be a simple vector, not an array.
    Code
      foo[mean, ]
    Condition
      Error in `foo[mean, ]`:
      ! Can't subset rows with `mean`.
      x `mean` must be logical, numeric, or character, not a function.
    Code
      foo[foo, ]
    Condition
      Error in `foo[foo, ]`:
      ! Can't subset rows with `foo`.
      x `foo` must be logical, numeric, or character, not a <tbl_df/tbl/data.frame> object.
    Code
      # # [.tbl_df is careful about column flags (#83)
      foo <- tibble(x = 1:10, y = 1:10, z = 1:10)
      foo[c(TRUE, TRUE)]
    Condition
      Error in `foo[c(TRUE, TRUE)]`:
      ! Can't subset columns with `c(TRUE, TRUE)`.
      x Logical subscript `c(TRUE, TRUE)` must be size 1 or 3, not 2.
    Code
      foo[c(TRUE, TRUE, FALSE, FALSE)]
    Condition
      Error in `foo[c(TRUE, TRUE, FALSE, FALSE)]`:
      ! Can't subset columns with `c(TRUE, TRUE, FALSE, FALSE)`.
      x Logical subscript `c(TRUE, TRUE, FALSE, FALSE)` must be size 1 or 3, not 4.
    Code
      foo[c(TRUE, TRUE, NA)]
    Condition
      Error in `foo[c(TRUE, TRUE, NA)]`:
      ! Can't subset columns with `c(TRUE, TRUE, NA)`.
      x Subscript `c(TRUE, TRUE, NA)` can't contain missing values.
      x It has a missing value at location 3.
    Code
      foo[as.matrix(TRUE)]
    Condition
      Error in `foo[as.matrix(TRUE)]`:
      ! Subscript `as.matrix(TRUE)` is a matrix, it must have the same dimensions as the input.
    Code
      foo[array(TRUE, dim = c(1, 1, 1))]
    Condition
      Error in `foo[array(TRUE, dim = c(1, 1, 1))]`:
      ! Can't subset columns with `array(TRUE, dim = c(1, 1, 1))`.
      x Subscript `array(TRUE, dim = c(1, 1, 1))` must be a simple vector, not an array.
    Code
      # # [.tbl_df is careful about row flags
      foo <- tibble(x = 1:3, y = 1:3, z = 1:3)
      foo[c(TRUE, TRUE), ]
    Condition
      Error in `foo[c(TRUE, TRUE), ]`:
      ! Can't subset rows with `c(TRUE, TRUE)`.
      x Logical subscript `c(TRUE, TRUE)` must be size 1 or 3, not 2.
    Code
      foo[c(TRUE, TRUE, FALSE, FALSE), ]
    Condition
      Error in `foo[c(TRUE, TRUE, FALSE, FALSE), ]`:
      ! Can't subset rows with `c(TRUE, TRUE, FALSE, FALSE)`.
      x Logical subscript `c(TRUE, TRUE, FALSE, FALSE)` must be size 1 or 3, not 4.
    Code
      foo[array(TRUE, dim = c(1, 1, 1)), ]
    Condition
      Error in `foo[array(TRUE, dim = c(1, 1, 1)), ]`:
      ! Can't subset rows with `array(TRUE, dim = c(1, 1, 1))`.
      x Subscript `array(TRUE, dim = c(1, 1, 1))` must be a simple vector, not an array.
    Code
      # # [.tbl_df rejects unknown column indexes (#83)
      foo <- tibble(x = 1:10, y = 1:10, z = 1:10)
      foo[list(1:3)]
    Condition
      Error in `foo[list(1:3)]`:
      ! Can't subset columns with `list(1:3)`.
      x `list(1:3)` must be logical, numeric, or character, not a list.
    Code
      foo[as.list(1:3)]
    Condition
      Error in `foo[as.list(1:3)]`:
      ! Can't subset columns with `as.list(1:3)`.
      x `as.list(1:3)` must be logical, numeric, or character, not a list.
    Code
      foo[factor(1:3)]
    Condition
      Error in `foo[factor(1:3)]`:
      ! Can't subset columns that don't exist.
      x Columns `1`, `2`, and `3` don't exist.
    Code
      foo[Sys.Date()]
    Condition
      Error in `foo[Sys.Date()]`:
      ! Can't subset columns with `Sys.Date()`.
      x `Sys.Date()` must be logical, numeric, or character, not a <Date> object.
    Code
      # # [.tbl_df rejects unknown row indexes
      foo <- tibble(x = 1:10, y = 1:10, z = 1:10)
      foo[list(1:3), ]
    Condition
      Error in `foo[list(1:3), ]`:
      ! Can't subset rows with `list(1:3)`.
      x `list(1:3)` must be logical, numeric, or character, not a list.
    Code
      foo[as.list(1:3), ]
    Condition
      Error in `foo[as.list(1:3), ]`:
      ! Can't subset rows with `as.list(1:3)`.
      x `as.list(1:3)` must be logical, numeric, or character, not a list.
    Code
      foo[factor(1:3), ]
    Condition
      Error in `vec_as_location()`:
      ! Can't use character names to index an unnamed vector.
    Code
      foo[Sys.Date(), ]
    Condition
      Error in `foo[Sys.Date(), ]`:
      ! Can't subset rows with `Sys.Date()`.
      x `Sys.Date()` must be logical, numeric, or character, not a <Date> object.
    Code
      # # [.tbl_df and matrix subsetting
      foo <- tibble(a = 1:3, b = letters[1:3])
      foo[is.na(foo)]
    Output
      <unspecified> [0]
    Code
      foo[!is.na(foo)]
    Condition
      Error in `vec_c()`:
      ! Can't combine `a` <integer> and `b` <character>.
    Code
      foo[as.matrix("x")]
    Condition
      Error in `foo[as.matrix("x")]`:
      ! Subscript `as.matrix("x")` is a matrix, it must be of type logical.
    Code
      foo[array("x", dim = c(1, 1, 1))]
    Condition
      Error in `foo[array("x", dim = c(1, 1, 1))]`:
      ! Can't subset columns with `array("x", dim = c(1, 1, 1))`.
      x Subscript `array("x", dim = c(1, 1, 1))` must be a simple vector, not an array.
    Code
      # # [.tbl_df and OOB indexing
      foo <- tibble(a = 1:3, b = letters[1:3])
      invisible(foo[3:5, ])
    Condition
      Warning:
      The `i` argument of `[.tbl_df` must lie in [0, rows] if positive, as of tibble 3.0.0.
      i Use `NA_integer_` as row index to obtain a row full of `NA` values.
    Code
      invisible(foo[-(3:5), ])
    Condition
      Warning:
      The `i` argument of `[.tbl_df` must lie in [-rows, 0] if negative, as of tibble 3.0.0.
      i Use `NA_integer_` as row index to obtain a row full of `NA` values.
    Code
      invisible(foo["x", ])
    Condition
      Warning:
      The `i` argument of `[.tbl_df` must use valid row names as of tibble 3.0.0.
      i Use `NA_integer_` as row index to obtain a row full of `NA` values.
    Code
      # # [.tbl_df and logical recycling
      foo <- tibble(a = 1:4, b = a)
      foo[c(TRUE, FALSE), ]
    Condition
      Error in `foo[c(TRUE, FALSE), ]`:
      ! Can't subset rows with `c(TRUE, FALSE)`.
      x Logical subscript `c(TRUE, FALSE)` must be size 1 or 4, not 2.
    Code
      # # [[.tbl_df rejects invalid column indexes
      foo <- tibble(x = 1:10, y = 1:10)
      foo[[]]
    Condition
      Error in `foo[[]]`:
      ! Subscript can't be missing for tibbles in `[[`.
    Code
      foo[[, 1]]
    Condition
      Error in `foo[[, 1]]`:
      ! Subscript can't be missing for tibbles in `[[`.
    Code
      foo[[1, ]]
    Condition
      Error in `foo[[1, ]]`:
      ! Subscript can't be missing for tibbles in `[[`.
    Code
      foo[[, ]]
    Condition
      Error in `foo[[, ]]`:
      ! Subscript can't be missing for tibbles in `[[`.
    Code
      foo[[1:3]]
    Condition
      Error in `foo[[1:3]]`:
      ! Can't extract column with `1:3`.
      x Subscript `1:3` must be size 1, not 3.
    Code
      foo[[letters[1:3]]]
    Condition
      Error in `foo[[letters[1:3]]]`:
      ! Can't extract column with `letters[1:3]`.
      x Subscript `letters[1:3]` must be size 1, not 3.
    Code
      foo[[TRUE]]
    Condition
      Error in `foo[[TRUE]]`:
      ! Can't extract column with `TRUE`.
      x `TRUE` must be numeric or character, not `TRUE`.
    Code
      foo[[-1]]
    Condition
      Error in `foo[[-1]]`:
      ! Can't extract column with `-1`.
      x Subscript `-1` must be a positive location, not -1.
    Code
      foo[[1.5]]
    Condition
      Error in `foo[[1.5]]`:
      ! Can't extract column with `1.5`.
      x Can't convert from `j` <double> to <integer> due to loss of precision.
    Code
      foo[[3]]
    Condition
      Error in `foo[[3]]`:
      ! Can't extract columns past the end.
      i Location 3 doesn't exist.
      i There are only 2 columns.
    Code
      foo[[Inf]]
    Condition
      Error in `foo[[Inf]]`:
      ! Can't extract column with `Inf`.
      x Can't convert from `j` <double> to <integer> due to loss of precision.
    Code
      foo[[mean]]
    Condition
      Error in `foo[[mean]]`:
      ! Can't extract column with `mean`.
      x `mean` must be numeric or character, not a function.
    Code
      foo[[foo]]
    Condition
      Error in `foo[[foo]]`:
      ! Can't extract column with `foo`.
      x `foo` must be numeric or character, not a <tbl_df/tbl/data.frame> object.
    Code
      # # [[.tbl_df throws error with NA index
      foo <- tibble(x = 1:10, y = 1:10)
      foo[[NA]]
    Condition
      Error in `foo[[NA]]`:
      ! Can't extract column with `NA`.
      x Subscript `NA` must be a location, not an integer `NA`.
    Code
      # # $.tbl_df and partial matching/invalid columns
      foo <- tibble(data = 1:10)
      foo$d
    Condition
      Warning:
      Unknown or uninitialised column: `d`.
    Output
      NULL
    Code
      foo$e
    Condition
      Warning:
      Unknown or uninitialised column: `e`.
    Output
      NULL
    Code
      # # [<-.tbl_df rejects unknown column indexes (#83)
      foo <- tibble(x = 1:10, y = 1:10, z = 1:10)
      foo[list(1:3)] <- 1
    Condition
      Error in `[<-`:
      ! Can't assign columns with `list(1:3)`.
      x `list(1:3)` must be logical, numeric, or character, not a list.
    Code
      foo[as.list(1:3)] <- 1
    Condition
      Error in `[<-`:
      ! Can't assign columns with `as.list(1:3)`.
      x `as.list(1:3)` must be logical, numeric, or character, not a list.
    Code
      foo[factor(1:3)] <- 1
      foo[Sys.Date()] <- 1
    Condition
      Error in `[<-`:
      ! Can't assign columns with `Sys.Date()`.
      x `Sys.Date()` must be logical, numeric, or character, not a <Date> object.
    Code
      # # [.tbl_df emits lifecycle warnings with one-column matrix indexes (#760)
      foo <- tibble(x = 1:10, y = 1:10, z = 1:10)
      invisible(foo[matrix(1:2, ncol = 1), ])
    Condition
      Warning:
      The `i` argument of `[` can't be a matrix as of tibble 3.0.0.
      i Convert to a vector.
    Code
      invisible(foo[matrix(rep(TRUE, 10), ncol = 1), ])
    Condition
      Warning:
      The `i` argument of `[` can't be a matrix as of tibble 3.0.0.
      i Convert to a vector.
    Code
      # # [<-.tbl_df rejects unknown row indexes
      foo <- tibble(x = 1:10, y = 1:10, z = 1:10)
      foo[list(1:3), ] <- 1
    Condition
      Error in `[<-`:
      ! Can't assign rows with `list(1:3)`.
      x `list(1:3)` must be logical, numeric, or character, not a list.
    Code
      foo[as.list(1:3), ] <- 1
    Condition
      Error in `[<-`:
      ! Can't assign rows with `as.list(1:3)`.
      x `as.list(1:3)` must be logical, numeric, or character, not a list.
    Code
      foo[factor(1:3), ] <- 1
    Condition
      Error in `vec_as_location()`:
      ! Can't use character names to index an unnamed vector.
    Code
      foo[Sys.Date(), ] <- 1
    Condition
      Error in `[<-`:
      ! Can't assign rows with `Sys.Date()`.
      x `Sys.Date()` must be logical, numeric, or character, not a <Date> object.
    Code
      # # [<-.tbl_df throws an error with duplicate indexes (#658)
      df <- tibble(x = 1:2, y = x)
      df[c(1, 1)] <- 3
    Condition
      Error in `[<-`:
      ! Column index 1 is used more than once for assignment.
    Code
      df[, c(1, 1)] <- 3
    Condition
      Error in `[<-`:
      ! Column index 1 is used more than once for assignment.
    Code
      df[c(1, 1), ] <- 3
    Condition
      Error in `[<-`:
      ! Row index 1 is used more than once for assignment.
    Code
      # # [<-.tbl_df throws an error with NA indexes
      df <- tibble(x = 1:2, y = x)
      df[NA] <- 3
    Condition
      Error in `[<-`:
      ! Can't assign columns with `NA`.
      x Subscript `NA` can't contain missing values.
      x It has a missing value at location 1.
    Code
      df[NA, ] <- 3
    Condition
      Error in `[<-`:
      ! Can't assign rows with `NA`.
      x Subscript `NA` can't contain missing values.
      x It has a missing value at location 1.
    Code
      # # [<-.tbl_df and logical indexes
      df <- tibble(x = 1:2, y = x)
      df[FALSE] <- 1
      df
    Output
      # A tibble: 2 x 2
            x     y
        <int> <int>
      1     1     1
      2     2     2
    Code
      df[, TRUE] <- 2
      df
    Output
      # A tibble: 2 x 2
            x     y
        <dbl> <dbl>
      1     2     2
      2     2     2
    Code
      # # [<-.tbl_df throws an error with bad RHS
      df <- tibble(x = 1:2, y = x)
      df[] <- mean
    Condition
      Error in `[<-`:
      ! `mean` must be a vector, a bare list, a data frame, a matrix, or NULL.
    Code
      df[] <- lm(y ~ x, df)
    Condition
      Error in `[<-`:
      ! `lm(y ~ x, df)` must be a vector, a bare list, a data frame, a matrix, or NULL.
    Code
      # # [<-.tbl_df throws an error with OOB assignment
      df <- tibble(x = 1:2, y = x)
      df[4:5] <- 3
    Condition
      Error in `[<-`:
      ! Can't assign to columns beyond the end with non-consecutive locations.
      i Input has size 2.
      x Subscript `4:5` contains non-consecutive location 4.
    Code
      df[4:5, ] <- 3
    Condition
      Error in `[<-`:
      ! Can't assign to rows beyond the end with non-consecutive locations.
      i Input has size 2.
      x Subscript `4:5` contains non-consecutive location 4.
    Code
      df[-4, ] <- 3
    Condition
      Error in `[<-`:
      ! Can't negate rows past the end.
      i Location 4 doesn't exist.
      i There are only 2 rows.
    Code
      df[-(4:5), ] <- 3
    Condition
      Error in `[<-`:
      ! Can't negate rows past the end.
      i Locations 4 and 5 don't exist.
      i There are only 2 rows.
    Code
      # # [<-.tbl_df and recycling
      df <- tibble(x = 1:3, y = x, z = y)
      df[1:2] <- list(0, 0, 0)
    Condition
      Error in `[<-`:
      ! Can't recycle `list(0, 0, 0)` (size 3) to size 2.
    Code
      df[] <- list(0, 0)
    Condition
      Error in `[<-`:
      ! Can't recycle input of size 2 to size 3.
    Code
      df[1, ] <- 1:3
    Condition
      Error in `[<-`:
      ! Assigned data `1:3` must be compatible with row subscript `1`.
      x 1 row must be assigned.
      x Assigned data has 3 rows.
      i Row updates require a list value. Do you need `list()` or `as.list()`?
      Caused by error in `vectbl_recycle_rhs_rows()`:
      ! Can't recycle input of size 3 to size 1.
    Code
      df[1:2, ] <- 1:3
    Condition
      Error in `[<-`:
      ! Assigned data `1:3` must be compatible with row subscript `1:2`.
      x 2 rows must be assigned.
      x Assigned data has 3 rows.
      i Only vectors of size 1 are recycled.
      Caused by error in `vectbl_recycle_rhs_rows()`:
      ! Can't recycle input of size 3 to size 2.
    Code
      df[, ] <- 1:2
    Condition
      Error in `[<-`:
      ! Assigned data `1:2` must be compatible with existing data.
      x Existing data has 3 rows.
      x Assigned data has 2 rows.
      i Only vectors of size 1 are recycled.
      Caused by error in `vectbl_recycle_rhs_rows()`:
      ! Can't recycle input of size 2 to size 3.
    Code
      df[1, ] <- list(a = 1:3, b = 1)
    Condition
      Error in `[<-`:
      ! Can't recycle input of size 2 to size 3.
    Code
      df[1, ] <- list(a = 1, b = 1:3)
    Condition
      Error in `[<-`:
      ! Can't recycle input of size 2 to size 3.
    Code
      df[1:2, ] <- list(a = 1:3, b = 1)
    Condition
      Error in `[<-`:
      ! Can't recycle input of size 2 to size 3.
    Code
      df[1:2, ] <- list(a = 1, b = 1:3)
    Condition
      Error in `[<-`:
      ! Can't recycle input of size 2 to size 3.
    Code
      df[1, 1:2] <- list(a = 1:3, b = 1)
    Condition
      Error in `[<-`:
      ! Assigned data `list(a = 1:3, b = 1)` must be compatible with row subscript `1`.
      x 1 row must be assigned.
      x Element 1 of assigned data has 3 rows.
      i Row updates require a list value. Do you need `list()` or `as.list()`?
      Caused by error in `vectbl_recycle_rhs_rows()`:
      ! Can't recycle input of size 3 to size 1.
    Code
      df[1, 1:2] <- list(a = 1, b = 1:3)
    Condition
      Error in `[<-`:
      ! Assigned data `list(a = 1, b = 1:3)` must be compatible with row subscript `1`.
      x 1 row must be assigned.
      x Element 2 of assigned data has 3 rows.
      i Row updates require a list value. Do you need `list()` or `as.list()`?
      Caused by error in `vectbl_recycle_rhs_rows()`:
      ! Can't recycle input of size 3 to size 1.
    Code
      df[1:2, 1:2] <- list(a = 1:3, b = 1)
    Condition
      Error in `[<-`:
      ! Assigned data `list(a = 1:3, b = 1)` must be compatible with row subscript `1:2`.
      x 2 rows must be assigned.
      x Element 1 of assigned data has 3 rows.
      i Only vectors of size 1 are recycled.
      Caused by error in `vectbl_recycle_rhs_rows()`:
      ! Can't recycle input of size 3 to size 2.
    Code
      df[1:2, 1:2] <- list(a = 1, b = 1:3)
    Condition
      Error in `[<-`:
      ! Assigned data `list(a = 1, b = 1:3)` must be compatible with row subscript `1:2`.
      x 2 rows must be assigned.
      x Element 2 of assigned data has 3 rows.
      i Only vectors of size 1 are recycled.
      Caused by error in `vectbl_recycle_rhs_rows()`:
      ! Can't recycle input of size 3 to size 2.
    Code
      df[1, ] <- list(a = 1:3, b = 1, c = 1:3)
    Condition
      Error in `[<-`:
      ! Assigned data `list(a = 1:3, b = 1, c = 1:3)` must be compatible with row subscript `1`.
      x 1 row must be assigned.
      x Element 1 of assigned data has 3 rows.
      i Row updates require a list value. Do you need `list()` or `as.list()`?
      Caused by error in `vectbl_recycle_rhs_rows()`:
      ! Can't recycle input of size 3 to size 1.
    Code
      df[1, ] <- list(a = 1, b = 1:3, c = 1:3)
    Condition
      Error in `[<-`:
      ! Assigned data `list(a = 1, b = 1:3, c = 1:3)` must be compatible with row subscript `1`.
      x 1 row must be assigned.
      x Element 2 of assigned data has 3 rows.
      i Row updates require a list value. Do you need `list()` or `as.list()`?
      Caused by error in `vectbl_recycle_rhs_rows()`:
      ! Can't recycle input of size 3 to size 1.
    Code
      df[1:2, ] <- list(a = 1:3, b = 1, c = 1:3)
    Condition
      Error in `[<-`:
      ! Assigned data `list(a = 1:3, b = 1, c = 1:3)` must be compatible with row subscript `1:2`.
      x 2 rows must be assigned.
      x Element 1 of assigned data has 3 rows.
      i Only vectors of size 1 are recycled.
      Caused by error in `vectbl_recycle_rhs_rows()`:
      ! Can't recycle input of size 3 to size 2.
    Code
      df[1:2, ] <- list(a = 1, b = 1:3, c = 1:3)
    Condition
      Error in `[<-`:
      ! Assigned data `list(a = 1, b = 1:3, c = 1:3)` must be compatible with row subscript `1:2`.
      x 2 rows must be assigned.
      x Element 2 of assigned data has 3 rows.
      i Only vectors of size 1 are recycled.
      Caused by error in `vectbl_recycle_rhs_rows()`:
      ! Can't recycle input of size 3 to size 2.
    Code
      # # [<-.tbl_df and coercion
      df <- tibble(x = 1:3, y = letters[1:3], z = as.list(1:3))
      df[1:3, 1:2] <- df[2:3]
    Condition
      Error in `[<-`:
      ! Assigned data `df[2:3]` must be compatible with existing data.
      i Error occurred for column `x`.
      Caused by error in `vec_assign()`:
      ! Can't convert <character> to <integer>.
    Code
      df[1:3, 1:2] <- df[1]
    Condition
      Error in `[<-`:
      ! Assigned data `df[1]` must be compatible with existing data.
      i Error occurred for column `y`.
      Caused by error in `vec_assign()`:
      ! Can't convert <integer> to <character>.
    Code
      df[1:3, 1:2] <- df[[1]]
    Condition
      Error in `[<-`:
      ! Assigned data `df[[1]]` must be compatible with existing data.
      i Error occurred for column `y`.
      Caused by error in `vec_assign()`:
      ! Can't convert <integer> to <character>.
    Code
      df[1:3, 1:3] <- df[3:1]
    Condition
      Error in `[<-`:
      ! Assigned data `df[3:1]` must be compatible with existing data.
      i Error occurred for column `x`.
      Caused by error in `vec_assign()`:
      ! Can't convert <list> to <integer>.
    Code
      df[1:3, 1:3] <- NULL
    Condition
      Error in `[<-`:
      ! `NULL` must be a vector, a bare list, a data frame or a matrix.
    Code
      # # [<-.tbl_df and overwriting NA
      df <- tibble(x = rep(NA, 3), z = matrix(NA, ncol = 2, dimnames = list(NULL, c(
        "a", "b"))))
      df[1, "x"] <- 5
      df[1, "z"] <- 5
      df
    Output
      # A tibble: 3 x 2
            x z[,"a"] [,"b"]
        <dbl>   <dbl>  <dbl>
      1     5       5      5
      2    NA      NA     NA
      3    NA      NA     NA
    Code
      # # [<-.tbl_df and overwriting with NA
      df <- tibble(a = TRUE, b = 1L, c = sqrt(2), d = 0+3i + 1, e = "e", f = raw(1),
      g = tibble(x = 1, y = 1), h = matrix(1:3, nrow = 1))
      df[FALSE, "a"] <- NA
      df[FALSE, "b"] <- NA
      df[FALSE, "c"] <- NA
      df[FALSE, "d"] <- NA
      df[FALSE, "e"] <- NA
      df[FALSE, "f"] <- NA
      df[FALSE, "g"] <- NA
      df[FALSE, "h"] <- NA
      df
    Output
      # A tibble: 1 x 8
        a         b     c d     e     f       g$x    $y h[,1]  [,2]  [,3]
        <lgl> <int> <dbl> <cpl> <chr> <raw> <dbl> <dbl> <int> <int> <int>
      1 TRUE      1  1.41 1+3i  e     00        1     1     1     2     3
    Code
      df[integer(), "a"] <- NA
      df[integer(), "b"] <- NA
      df[integer(), "c"] <- NA
      df[integer(), "d"] <- NA
      df[integer(), "e"] <- NA
      df[integer(), "f"] <- NA
      df[integer(), "g"] <- NA
      df[integer(), "h"] <- NA
      df
    Output
      # A tibble: 1 x 8
        a         b     c d     e     f       g$x    $y h[,1]  [,2]  [,3]
        <lgl> <int> <dbl> <cpl> <chr> <raw> <dbl> <dbl> <int> <int> <int>
      1 TRUE      1  1.41 1+3i  e     00        1     1     1     2     3
    Code
      df[1, "a"] <- NA
      df[1, "b"] <- NA
      df[1, "c"] <- NA
      df[1, "d"] <- NA
      df[1, "e"] <- NA
      df[1, "f"] <- NA
      df[1, "g"] <- NA
      df[1, "h"] <- NA
      df
    Output
      # A tibble: 1 x 8
        a         b     c d     e     f       g$x    $y h[,1]  [,2]  [,3]
        <lgl> <int> <dbl> <cpl> <chr> <raw> <dbl> <dbl> <int> <int> <int>
      1 NA       NA    NA NA    <NA>  00       NA    NA    NA    NA    NA
    Code
      # # [<-.tbl_df and matrix subsetting
      foo <- tibble(a = 1:3, b = letters[1:3])
      foo[!is.na(foo)] <- "bogus"
    Condition
      Error in `[<-`:
      ! Assigned data `"bogus"` must be compatible with existing data.
      i Error occurred for column `a`.
      Caused by error in `vec_assign()`:
      ! Can't convert <character> to <integer>.
    Code
      foo[as.matrix("x")] <- NA
    Condition
      Error in `[<-`:
      ! Subscript `as.matrix("x")` is a matrix, it must be of type logical.
    Code
      foo[array("x", dim = c(1, 1, 1))] <- NA
      foo[is.na(foo)] <- 1:3
    Condition
      Error in `[<-`:
      ! Subscript `is.na(foo)` is a matrix, the data `1:3` must have size 1.
    Code
      foo[is.na(foo)] <- lm(a ~ b, foo)
    Condition
      Error in `[<-`:
      ! Subscript `is.na(foo)` is a matrix, the data `lm(a ~ b, foo)` must be a vector of size 1.
    Code
      # # [[<-.tbl_df rejects invalid column indexes
      foo <- tibble(x = 1:10, y = 1:10)
      foo[[]] <- 1
    Condition
      Error in `[[<-`:
      ! Subscript can't be missing for tibbles in `[[<-`.
    Code
      foo[[, 1]] <- 1
    Condition
      Error in `[[<-`:
      ! Subscript can't be missing for tibbles in `[[<-`.
    Code
      foo[[1, ]] <- 1
    Condition
      Error in `[[<-`:
      ! Subscript can't be missing for tibbles in `[[<-`.
    Code
      foo[[, ]] <- 1
    Condition
      Error in `[[<-`:
      ! Subscript can't be missing for tibbles in `[[<-`.
    Code
      foo[[1:3]] <- 1
    Condition
      Error in `[[<-`:
      ! Can't assign column with `1:3`.
      x Subscript `1:3` must be size 1, not 3.
    Code
      foo[[letters[1:3]]] <- 1
    Condition
      Error in `[[<-`:
      ! Can't assign column with `letters[1:3]`.
      x Subscript `letters[1:3]` must be size 1, not 3.
    Code
      foo[[TRUE]] <- 1
    Condition
      Error in `[[<-`:
      ! Can't assign column with `TRUE`.
      x `TRUE` must be numeric or character, not `TRUE`.
    Code
      foo[[NA_integer_]] <- 1
    Condition
      Error in `[[<-`:
      ! Can't assign column with `NA_integer_`.
      x Subscript `NA_integer_` must be a location, not an integer `NA`.
    Code
      foo[[mean]] <- 1
    Condition
      Error in `[[<-`:
      ! Can't assign column with `mean`.
      x `mean` must be numeric or character, not a function.
    Code
      foo[[foo]] <- 1
    Condition
      Error in `[[<-`:
      ! Can't assign column with `foo`.
      x `foo` must be numeric or character, not a <tbl_df/tbl/data.frame> object.
    Code
      foo[[1:3, 1]] <- 1
    Condition
      Error in `[[<-`:
      ! Can't assign row with `1:3`.
      x Subscript `1:3` must be size 1, not 3.
    Code
      foo[[TRUE, 1]] <- 1
    Condition
      Error in `[[<-`:
      ! Can't assign row with `TRUE`.
      x `TRUE` must be numeric or character, not `TRUE`.
    Code
      foo[[mean, 1]] <- 1
    Condition
      Error in `[[<-`:
      ! Can't assign row with `mean`.
      x `mean` must be numeric or character, not a function.
    Code
      foo[[foo, 1]] <- 1
    Condition
      Error in `[[<-`:
      ! Can't assign row with `foo`.
      x `foo` must be numeric or character, not a <tbl_df/tbl/data.frame> object.
    Code
      # # [[<-.tbl_df throws an error with OOB assignment
      df <- tibble(x = 1:2, y = x)
      df[[4]] <- 3
    Condition
      Error in `[[<-`:
      ! Can't assign to columns beyond the end with non-consecutive locations.
      i Input has size 2.
      x Subscript `4` contains non-consecutive location 4.
    Code
      # # [[<-.tbl_df throws an error with bad RHS
      df <- tibble(x = 1:2, y = x)
      df[[1]] <- mean
    Condition
      Error in `[[<-`:
      ! Assigned data `mean` must be a vector.
      Caused by error in `vec_size()`:
      ! `x` must be a vector, not a function.
    Code
      df[[1]] <- lm(y ~ x, df)
    Condition
      Error in `[[<-`:
      ! Assigned data `lm(y ~ x, df)` must be a vector.
      Caused by error in `vec_size()`:
      ! `x` must be a vector, not a <lm> object.
    Code
      # # [[<-.tbl_df recycles only values of length one
      df <- tibble(x = 1:3)
      df[["x"]] <- 8:9
    Condition
      Error in `[[<-`:
      ! Assigned data `8:9` must be compatible with existing data.
      x Existing data has 3 rows.
      x Assigned data has 2 rows.
      i Only vectors of size 1 are recycled.
      Caused by error in `vectbl_recycle_rhs_rows()`:
      ! Can't recycle input of size 2 to size 3.
    Code
      df[["w"]] <- 8:9
    Condition
      Error in `[[<-`:
      ! Assigned data `8:9` must be compatible with existing data.
      x Existing data has 3 rows.
      x Assigned data has 2 rows.
      i Only vectors of size 1 are recycled.
      Caused by error in `vectbl_recycle_rhs_rows()`:
      ! Can't recycle input of size 2 to size 3.
    Code
      df[["a"]] <- character()
    Condition
      Error in `[[<-`:
      ! Assigned data `character()` must be compatible with existing data.
      x Existing data has 3 rows.
      x Assigned data has 0 rows.
      i Only vectors of size 1 are recycled.
      Caused by error in `vectbl_recycle_rhs_rows()`:
      ! Can't recycle input of size 0 to size 3.
    Code
      # # [<-.tbl_df throws an error with invalid values
      df <- tibble(x = 1:2, y = x)
      df[1] <- lm(y ~ x, df)
    Condition
      Error in `[<-`:
      ! `lm(y ~ x, df)` must be a vector, a bare list, a data frame, a matrix, or NULL.
    Code
      df[1:2, 1] <- NULL
    Condition
      Error in `[<-`:
      ! `NULL` must be a vector, a bare list, a data frame or a matrix.
    Code
      # # $<- recycles only values of length one
      df <- tibble(x = 1:3)
      df$x <- 8:9
    Condition
      Error in `$<-`:
      ! Assigned data `8:9` must be compatible with existing data.
      x Existing data has 3 rows.
      x Assigned data has 2 rows.
      i Only vectors of size 1 are recycled.
      Caused by error in `vectbl_recycle_rhs_rows()`:
      ! Can't recycle input of size 2 to size 3.
    Code
      df$w <- 8:9
    Condition
      Error in `$<-`:
      ! Assigned data `8:9` must be compatible with existing data.
      x Existing data has 3 rows.
      x Assigned data has 2 rows.
      i Only vectors of size 1 are recycled.
      Caused by error in `vectbl_recycle_rhs_rows()`:
      ! Can't recycle input of size 2 to size 3.
    Code
      df$a <- character()
    Condition
      Error in `$<-`:
      ! Assigned data `character()` must be compatible with existing data.
      x Existing data has 3 rows.
      x Assigned data has 0 rows.
      i Only vectors of size 1 are recycled.
      Caused by error in `vectbl_recycle_rhs_rows()`:
      ! Can't recycle input of size 0 to size 3.

# as_tibble() implements unique names

    Code
      invalid_df <- as_tibble(list(3, 4, 5), .name_repair = "unique")
    Message <simpleMessage>
      New names:
      * `` -> ...1
      * `` -> ...2
      * `` -> ...3

# as_tibble() implements universal names

    Code
      invalid_df <- as_tibble(list(3, 4, 5), .name_repair = "universal")
    Message <simpleMessage>
      New names:
      * `` -> ...1
      * `` -> ...2
      * `` -> ...3

# as_tibble.matrix() supports .name_repair

    Code
      universal <- as_tibble(x, .name_repair = "universal")
    Message <simpleMessage>
      New names:
      * `` -> ...1
      * `` -> ...2

---

    Code
      universal <- as_tibble(x, .name_repair = "universal")
    Message <simpleMessage>
      New names:
      * `if` -> .if

# as_tibble.poly() supports .name_repair

    Code
      universal <- as_tibble(x, .name_repair = "universal")
    Message <simpleMessage>
      New names:
      * `1` -> ...1
      * `2` -> ...2
      * `3` -> ...3

# as_tibble.table() supports .name_repair

    Code
      universal <- as_tibble(x, .name_repair = "universal")
    Message <simpleMessage>
      New names:
      * a -> a...1
      * a -> a...2

---

    Code
      universal <- as_tibble(x, .name_repair = "universal")
    Message <simpleMessage>
      New names:
      * `if` -> .if

---

    Code
      universal <- as_tibble(x, .name_repair = "universal")
    Message <simpleMessage>
      New names:
      * n -> n...2
      * n -> n...3

# as_tibble.ts() supports .name_repair, minimal by default (#537)

    Code
      universal <- as_tibble(x, .name_repair = "universal")
    Message <simpleMessage>
      New names:
      * `` -> ...1
      * `` -> ...2

---

    Code
      universal <- as_tibble(x, .name_repair = "universal")
    Message <simpleMessage>
      New names:
      * `if` -> .if

# creates col names with name repair

    Code
      out <- as_tibble(x, .name_repair = "unique")
    Message <simpleMessage>
      New names:
      * `` -> ...1
      * `` -> ...2

---

    Code
      out <- as_tibble(x, .name_repair = "universal")
    Message <simpleMessage>
      New names:
      * `` -> ...1
      * `` -> ...2

# output test

    Code
      as_tibble(list(1))
    Error <tibble_error_column_names_cannot_be_empty>
      Column 1 must be named.
    Code
      as_tibble(list(1, 2))
    Error <tibble_error_column_names_cannot_be_empty>
      Columns 1 and 2 must be named.
    Code
      as_tibble(list(a = 1, 2))
    Error <tibble_error_column_names_cannot_be_empty>
      Column 2 must be named.
    Code
      as_tibble(as.list(1:26))
    Error <tibble_error_column_names_cannot_be_empty>
      Columns 1, 2, 3, 4, 5, and 21 more must be named.
    Code
      as_tibble(set_names(list(1), "..1"))
    Error <tibble_error_column_names_cannot_be_dot_dot>
      Column 1 must not have names of the form ... or ..j.
    Code
      as_tibble(set_names(list(1:26), paste0("..", 1:26)))
    Error <rlang_error>
      `nm` must be `NULL` or a character vector the same length as `x`
    Code
      as_tibble(list(a = 1, a = 1))
    Error <tibble_error_column_names_must_be_unique>
      Column name `a` must not be duplicated.
    Code
      as_tibble(list(a = 1, a = 1, b = 1, b = 1))
    Error <tibble_error_column_names_must_be_unique>
      Column names `a` and `b` must not be duplicated.
    Code
      as_tibble(list(a = new_environment()))
    Error <tibble_error_column_scalar_type>
      All columns in a tibble must be vectors.
      x Column `a` is an environment.
    Code
      as_tibble_row(list(1))
    Error <tibble_error_column_names_cannot_be_empty>
      Column 1 must be named.
    Code
      as_tibble_row(list(1, 2))
    Error <tibble_error_column_names_cannot_be_empty>
      Columns 1 and 2 must be named.
    Code
      as_tibble_row(list(a = 1, 2))
    Error <tibble_error_column_names_cannot_be_empty>
      Column 2 must be named.
    Code
      as_tibble_row(as.list(1:26))
    Error <tibble_error_column_names_cannot_be_empty>
      Columns 1, 2, 3, 4, 5, and 21 more must be named.
    Code
      as_tibble_row(set_names(list(1), "..1"))
    Error <tibble_error_column_names_cannot_be_dot_dot>
      Column 1 must not have names of the form ... or ..j.
    Code
      as_tibble_row(set_names(list(1:26), paste0("..", 1:26)))
    Error <rlang_error>
      `nm` must be `NULL` or a character vector the same length as `x`
    Code
      as_tibble_row(list(a = 1, a = 1))
    Error <tibble_error_column_names_must_be_unique>
      Column name `a` must not be duplicated.
    Code
      as_tibble_row(list(a = 1, a = 1, b = 1, b = 1))
    Error <tibble_error_column_names_must_be_unique>
      Column names `a` and `b` must not be duplicated.
    Code
      as_tibble_row(list(a = new_environment()))
    Error <vctrs_error_scalar_type>
      Input must be a vector, not an environment.
    Code
      as_tibble_row(list(a = 1:3))
    Error <tibble_error_as_tibble_row_size_one>
      All elements must be size one, use `list()` to wrap.
      x Element `a` is of size 3.
    Code
      as_tibble_row(list(a = 1:3, b = 1:3))
    Error <tibble_error_as_tibble_row_size_one>
      All elements must be size one, use `list()` to wrap.
      x Element `a` is of size 3.
      x Element `b` is of size 3.

# output test

    Code
      enframe(1:3, value = NULL)
    Condition
      Error in `enframe()`:
      ! `value` can't be NULL.
    Code
      nrow(enframe(Titanic))
    Output
      [1] 4
    Code
      vec_names(enframe(Titanic)$value)
    Output
      NULL
    Code
      enframe(Titanic)$value
    Output
      , , Age = Child, Survived = No
      
            Sex
      Class  Male Female
        [1,]    0      0
        [2,]    0      0
        [3,]   35     17
        [4,]    0      0
      
      , , Age = Adult, Survived = No
      
            Sex
      Class  Male Female
        [1,]  118      4
        [2,]  154     13
        [3,]  387     89
        [4,]  670      3
      
      , , Age = Child, Survived = Yes
      
            Sex
      Class  Male Female
        [1,]    5      1
        [2,]   11     13
        [3,]   13     14
        [4,]    0      0
      
      , , Age = Adult, Survived = Yes
      
            Sex
      Class  Male Female
        [1,]   57    140
        [2,]   14     80
        [3,]   75     76
        [4,]  192     20
      

# output test

    Code
      enframe(1:3, value = NULL)
    Error <tibble_error_enframe_value_null>
      `value` can't be NULL.
    Code
      enframe(Titanic)
    Error <tibble_error_enframe_has_dim>
      `x` must not have more than one dimension. `length(dim(x))` must be zero or one, not 4.

# output test

    Code
      # # Bullets
    Code
      bullets("header", c("item 1", "item 2"))
    Output
      [1] "header.\nx item 1.\nx item 2."
    Code
      bullets("header", LETTERS)
    Output
      [1] "header.\nx A.\nx B.\nx C.\nx D.\nx E.\nx F.\nx G.\nx H.\nx I.\nx J.\nx K.\nx L.\nx M.\nx N.\nx O.\nx P.\nx Q.\nx R.\nx S.\nx T.\nx U.\nx V.\nx W.\nx X.\nx Y.\nx Z."
    Code
      bullets("header", 1:6)
    Output
      [1] "header.\n* 1.\n* 2.\n* 3.\n* 4.\n* 5.\n* 6."
    Code
      # # Commas
    Code
      commas("1")
    Output
      [1] "1"
    Code
      commas(letters[2:4])
    Output
      [1] "b, c, and d"
    Code
      commas(LETTERS)
    Output
      [1] "A, B, C, D, E, and 21 more"

# output test

    Code
      # # [.tbl_df is careful about names (#1245)
    Code
      foo <- tibble(x = 1:10, y = 1:10)
    Code
      foo[c("x", "y", "z")]
    Error <vctrs_error_subscript_oob>
      Can't subset columns that don't exist.
      x Column `z` doesn't exist.
    Code
      foo[c("w", "x", "y", "z")]
    Error <vctrs_error_subscript_oob>
      Can't subset columns that don't exist.
      x Columns `w` and `z` don't exist.
    Code
      foo[as.matrix("x")]
    Error <tibble_error_subset_matrix_must_be_logical>
      Subscript `as.matrix("x")` is a matrix, it must be of type logical.
    Code
      foo[array("x", dim = c(1, 1, 1))]
    Error <vctrs_error_subscript_type>
      Must subset columns with a valid subscript vector.
      x Subscript `array("x", dim = c(1, 1, 1))` must be a simple vector, not an array.
    Code
      # # [.tbl_df is careful about column indexes (#83)
    Code
      foo <- tibble(x = 1:10, y = 1:10, z = 1:10)
    Code
      foo[0.5]
    Error <vctrs_error_subscript_type>
      Must subset columns with a valid subscript vector.
      x Can't convert from <double> to <integer> due to loss of precision.
    Code
      foo[1:5]
    Error <vctrs_error_subscript_oob>
      Can't subset columns that don't exist.
      x Locations 4 and 5 don't exist.
      i There are only 3 columns.
    Code
      foo[-1:1]
    Error <vctrs_error_subscript_type>
      Must subset columns with a valid subscript vector.
      x Negative and positive locations can't be mixed.
      i Subscript `-1:1` has a positive value at location 3.
    Code
      foo[c(-1, 1)]
    Error <vctrs_error_subscript_type>
      Must subset columns with a valid subscript vector.
      x Negative and positive locations can't be mixed.
      i Subscript `c(-1, 1)` has a positive value at location 2.
    Code
      foo[c(-1, NA)]
    Error <vctrs_error_subscript_type>
      Must subset columns with a valid subscript vector.
      x Negative locations can't have missing values.
      i Subscript `c(-1, NA)` has a missing value at location 2.
    Code
      foo[-4]
    Error <vctrs_error_subscript_oob>
      Can't negate columns that don't exist.
      x Location 4 doesn't exist.
      i There are only 3 columns.
    Code
      foo[c(1:3, NA)]
    Error <tibble_error_na_column_index>
      Can't use NA as column index with `[` at position 4.
    Code
      foo[as.matrix(1)]
    Error <tibble_error_subset_matrix_must_be_logical>
      Subscript `as.matrix(1)` is a matrix, it must be of type logical.
    Code
      foo[array(1, dim = c(1, 1, 1))]
    Error <vctrs_error_subscript_type>
      Must subset columns with a valid subscript vector.
      x Subscript `array(1, dim = c(1, 1, 1))` must be a simple vector, not an array.
    Code
      foo[mean]
    Error <vctrs_error_subscript_type>
      Must subset columns with a valid subscript vector.
      x Subscript `mean` has the wrong type `function`.
      i It must be logical, numeric, or character.
    Code
      foo[foo]
    Error <vctrs_error_subscript_type>
      Must subset columns with a valid subscript vector.
      x Subscript `foo` has the wrong type `tbl_df<
        x: integer
        y: integer
        z: integer
      >`.
      i It must be logical, numeric, or character.
    Code
      # # [.tbl_df is careful about row indexes
    Code
      foo <- tibble(x = 1:3, y = 1:3, z = 1:3)
    Code
      foo[0.5, ]
    Error <vctrs_error_subscript_type>
      Must subset rows with a valid subscript vector.
      x Can't convert from <double> to <integer> due to loss of precision.
    Code
      invisible(foo[1:5, ])
    Warning <lifecycle_warning_deprecated>
      The `i` argument of ``[.tbl_df`()` must lie in [0, rows] if positive, as of tibble 3.0.0.
      Use `NA_integer_` as row index to obtain a row full of `NA` values.
    Code
      foo[-1:1, ]
    Error <vctrs_error_subscript_type>
      Must subset rows with a valid subscript vector.
      x Negative and positive locations can't be mixed.
      i Subscript `-1:1` has a positive value at location 3.
    Code
      foo[c(-1, 1), ]
    Error <vctrs_error_subscript_type>
      Must subset rows with a valid subscript vector.
      x Negative and positive locations can't be mixed.
      i Subscript `c(-1, 1)` has a positive value at location 2.
    Code
      foo[c(-1, NA), ]
    Error <vctrs_error_subscript_type>
      Must subset rows with a valid subscript vector.
      x Negative locations can't have missing values.
      i Subscript `c(-1, NA)` has a missing value at location 2.
    Code
      invisible(foo[-4, ])
    Warning <lifecycle_warning_deprecated>
      The `i` argument of ``[.tbl_df`()` must lie in [-rows, 0] if negative, as of tibble 3.0.0.
      Use `NA_integer_` as row index to obtain a row full of `NA` values.
    Code
      foo[array(1, dim = c(1, 1, 1)), ]
    Error <vctrs_error_subscript_type>
      Must subset rows with a valid subscript vector.
      x Subscript `array(1, dim = c(1, 1, 1))` must be a simple vector, not an array.
    Code
      foo[mean, ]
    Error <vctrs_error_subscript_type>
      Must subset rows with a valid subscript vector.
      x Subscript `mean` has the wrong type `function`.
      i It must be logical, numeric, or character.
    Code
      foo[foo, ]
    Error <vctrs_error_subscript_type>
      Must subset rows with a valid subscript vector.
      x Subscript `foo` has the wrong type `tbl_df<
        x: integer
        y: integer
        z: integer
      >`.
      i It must be logical, numeric, or character.
    Code
      # # [.tbl_df is careful about column flags (#83)
    Code
      foo <- tibble(x = 1:10, y = 1:10, z = 1:10)
    Code
      foo[c(TRUE, TRUE)]
    Error <vctrs_error_subscript_size>
      Must subset columns with a valid subscript vector.
      i Logical subscripts must match the size of the indexed input.
      x Input has size 3 but subscript `c(TRUE, TRUE)` has size 2.
    Code
      foo[c(TRUE, TRUE, FALSE, FALSE)]
    Error <vctrs_error_subscript_size>
      Must subset columns with a valid subscript vector.
      i Logical subscripts must match the size of the indexed input.
      x Input has size 3 but subscript `c(TRUE, TRUE, FALSE, FALSE)` has size 4.
    Code
      foo[c(TRUE, TRUE, NA)]
    Error <tibble_error_na_column_index>
      Can't use NA as column index with `[` at position 3.
    Code
      foo[as.matrix(TRUE)]
    Error <tibble_error_subset_matrix_must_have_same_dimensions>
      Subscript `as.matrix(TRUE)` is a matrix, it must have the same dimensions as the input.
    Code
      foo[array(TRUE, dim = c(1, 1, 1))]
    Error <vctrs_error_subscript_type>
      Must subset columns with a valid subscript vector.
      x Subscript `array(TRUE, dim = c(1, 1, 1))` must be a simple vector, not an array.
    Code
      # # [.tbl_df is careful about row flags
    Code
      foo <- tibble(x = 1:3, y = 1:3, z = 1:3)
    Code
      foo[c(TRUE, TRUE), ]
    Error <vctrs_error_subscript_size>
      Must subset rows with a valid subscript vector.
      i Logical subscripts must match the size of the indexed input.
      x Input has size 3 but subscript `c(TRUE, TRUE)` has size 2.
    Code
      foo[c(TRUE, TRUE, FALSE, FALSE), ]
    Error <vctrs_error_subscript_size>
      Must subset rows with a valid subscript vector.
      i Logical subscripts must match the size of the indexed input.
      x Input has size 3 but subscript `c(TRUE, TRUE, FALSE, FALSE)` has size 4.
    Code
      foo[array(TRUE, dim = c(1, 1, 1)), ]
    Error <vctrs_error_subscript_type>
      Must subset rows with a valid subscript vector.
      x Subscript `array(TRUE, dim = c(1, 1, 1))` must be a simple vector, not an array.
    Code
      # # [.tbl_df rejects unknown column indexes (#83)
    Code
      foo <- tibble(x = 1:10, y = 1:10, z = 1:10)
    Code
      foo[list(1:3)]
    Error <vctrs_error_subscript_type>
      Must subset columns with a valid subscript vector.
      x Subscript `list(1:3)` has the wrong type `list`.
      i It must be logical, numeric, or character.
    Code
      foo[as.list(1:3)]
    Error <vctrs_error_subscript_type>
      Must subset columns with a valid subscript vector.
      x Subscript `as.list(1:3)` has the wrong type `list`.
      i It must be logical, numeric, or character.
    Code
      foo[factor(1:3)]
    Error <vctrs_error_subscript_oob>
      Can't subset columns that don't exist.
      x Columns `1`, `2`, and `3` don't exist.
    Code
      foo[Sys.Date()]
    Error <vctrs_error_subscript_type>
      Must subset columns with a valid subscript vector.
      x Subscript `Sys.Date()` has the wrong type `date`.
      i It must be logical, numeric, or character.
    Code
      # # [.tbl_df rejects unknown row indexes
    Code
      foo <- tibble(x = 1:10, y = 1:10, z = 1:10)
    Code
      foo[list(1:3), ]
    Error <vctrs_error_subscript_type>
      Must subset rows with a valid subscript vector.
      x Subscript `list(1:3)` has the wrong type `list`.
      i It must be logical, numeric, or character.
    Code
      foo[as.list(1:3), ]
    Error <vctrs_error_subscript_type>
      Must subset rows with a valid subscript vector.
      x Subscript `as.list(1:3)` has the wrong type `list`.
      i It must be logical, numeric, or character.
    Code
      foo[factor(1:3), ]
    Error <simpleError>
      Can't use character names to index an unnamed vector.
    Code
      foo[Sys.Date(), ]
    Error <vctrs_error_subscript_type>
      Must subset rows with a valid subscript vector.
      x Subscript `Sys.Date()` has the wrong type `date`.
      i It must be logical, numeric, or character.
    Code
      # # [.tbl_df and matrix subsetting
    Code
      foo <- tibble(a = 1:3, b = letters[1:3])
    Code
      foo[is.na(foo)]
    Output
      <unspecified> [0]
    Code
      foo[!is.na(foo)]
    Error <vctrs_error_incompatible_type>
      Can't combine `a` <integer> and `b` <character>.
    Code
      foo[as.matrix("x")]
    Error <tibble_error_subset_matrix_must_be_logical>
      Subscript `as.matrix("x")` is a matrix, it must be of type logical.
    Code
      foo[array("x", dim = c(1, 1, 1))]
    Error <vctrs_error_subscript_type>
      Must subset columns with a valid subscript vector.
      x Subscript `array("x", dim = c(1, 1, 1))` must be a simple vector, not an array.
    Code
      # # [.tbl_df and OOB indexing
    Code
      foo <- tibble(a = 1:3, b = letters[1:3])
    Code
      invisible(foo[3:5, ])
    Warning <lifecycle_warning_deprecated>
      The `i` argument of ``[.tbl_df`()` must lie in [0, rows] if positive, as of tibble 3.0.0.
      Use `NA_integer_` as row index to obtain a row full of `NA` values.
    Code
      invisible(foo[-(3:5), ])
    Warning <lifecycle_warning_deprecated>
      The `i` argument of ``[.tbl_df`()` must lie in [-rows, 0] if negative, as of tibble 3.0.0.
      Use `NA_integer_` as row index to obtain a row full of `NA` values.
    Code
      invisible(foo["x", ])
    Warning <lifecycle_warning_deprecated>
      The `i` argument of ``[.tbl_df`()` must use valid row names as of tibble 3.0.0.
      Use `NA_integer_` as row index to obtain a row full of `NA` values.
    Code
      # # [.tbl_df and logical recycling
    Code
      foo <- tibble(a = 1:4, b = a)
    Code
      foo[c(TRUE, FALSE), ]
    Error <vctrs_error_subscript_size>
      Must subset rows with a valid subscript vector.
      i Logical subscripts must match the size of the indexed input.
      x Input has size 4 but subscript `c(TRUE, FALSE)` has size 2.
    Code
      # # [[.tbl_df rejects invalid column indexes
    Code
      foo <- tibble(x = 1:10, y = 1:10)
    Code
      foo[[]]
    Error <tibble_error_subset_columns_non_missing_only>
      Subscript can't be missing for tibbles in `[[`.
    Code
      foo[[, 1]]
    Error <tibble_error_subset_columns_non_missing_only>
      Subscript can't be missing for tibbles in `[[`.
    Code
      foo[[1, ]]
    Error <tibble_error_subset_columns_non_missing_only>
      Subscript can't be missing for tibbles in `[[`.
    Code
      foo[[, ]]
    Error <tibble_error_subset_columns_non_missing_only>
      Subscript can't be missing for tibbles in `[[`.
    Code
      foo[[1:3]]
    Error <vctrs_error_subscript_type>
      Must extract column with a single valid subscript.
      x Subscript `1:3` has size 3 but must be size 1.
    Code
      foo[[letters[1:3]]]
    Error <vctrs_error_subscript_type>
      Must extract column with a single valid subscript.
      x Subscript `letters[1:3]` has size 3 but must be size 1.
    Code
      foo[[TRUE]]
    Error <vctrs_error_subscript_type>
      Must extract column with a single valid subscript.
      x Subscript `TRUE` has the wrong type `logical`.
      i It must be numeric or character.
    Code
      foo[[-1]]
    Error <vctrs_error_subscript_type>
      Must extract column with a single valid subscript.
      x Subscript `-1` has value -1 but must be a positive location.
    Code
      foo[[1.5]]
    Error <vctrs_error_subscript_type>
      Must extract column with a single valid subscript.
      x Can't convert from <double> to <integer> due to loss of precision.
    Code
      foo[[3]]
    Error <vctrs_error_subscript_oob>
      Can't subset columns that don't exist.
      x Location 3 doesn't exist.
      i There are only 2 columns.
    Code
      foo[[Inf]]
    Error <vctrs_error_subscript_type>
      Must extract column with a single valid subscript.
      x Can't convert from <double> to <integer> due to loss of precision.
    Code
      foo[[mean]]
    Error <vctrs_error_subscript_type>
      Must extract column with a single valid subscript.
      x Subscript `mean` has the wrong type `function`.
      i It must be numeric or character.
    Code
      foo[[foo]]
    Error <vctrs_error_subscript_type>
      Must extract column with a single valid subscript.
      x Subscript `foo` has the wrong type `tbl_df<
        x: integer
        y: integer
      >`.
      i It must be numeric or character.
    Code
      # # [[.tbl_df throws error with NA index
    Code
      foo <- tibble(x = 1:10, y = 1:10)
    Code
      foo[[NA]]
    Error <vctrs_error_subscript_type>
      Must extract column with a single valid subscript.
      x Subscript `NA` can't be `NA`.
    Code
      # # $.tbl_df and partial matching/invalid columns
    Code
      foo <- tibble(data = 1:10)
    Code
      foo$d
    Warning <warning>
      Unknown or uninitialised column: `d`.
    Output
      NULL
    Code
      foo$e
    Warning <warning>
      Unknown or uninitialised column: `e`.
    Output
      NULL
    Code
      # # [<-.tbl_df rejects unknown column indexes (#83)
    Code
      foo <- tibble(x = 1:10, y = 1:10, z = 1:10)
    Code
      foo[list(1:3)] <- 1
    Error <vctrs_error_subscript_type>
      Must assign to columns with a valid subscript vector.
      x Subscript `list(1:3)` has the wrong type `list`.
      i It must be logical, numeric, or character.
    Code
      foo[as.list(1:3)] <- 1
    Error <vctrs_error_subscript_type>
      Must assign to columns with a valid subscript vector.
      x Subscript `as.list(1:3)` has the wrong type `list`.
      i It must be logical, numeric, or character.
    Code
      foo[factor(1:3)] <- 1
    Error <vctrs_error_subscript_oob>
      Can't assign to columns that don't exist.
      x Columns `1`, `2`, and `3` don't exist.
    Code
      foo[Sys.Date()] <- 1
    Error <vctrs_error_subscript_type>
      Must assign to columns with a valid subscript vector.
      x Subscript `Sys.Date()` has the wrong type `date`.
      i It must be logical, numeric, or character.
    Code
      # # [.tbl_df emits lifecycle warnings with one-column matrix indexes (#760)
    Code
      foo <- tibble(x = 1:10, y = 1:10, z = 1:10)
    Code
      invisible(foo[matrix(1:2, ncol = 1), ])
    Warning <lifecycle_warning_deprecated>
      The `i` argument of ``[`()` can't be a matrix as of tibble 3.0.0.
      Convert to a vector.
    Code
      invisible(foo[matrix(rep(TRUE, 10), ncol = 1), ])
    Warning <lifecycle_warning_deprecated>
      The `i` argument of ``[`()` can't be a matrix as of tibble 3.0.0.
      Convert to a vector.
    Code
      # # [<-.tbl_df rejects unknown row indexes
    Code
      foo <- tibble(x = 1:10, y = 1:10, z = 1:10)
    Code
      foo[list(1:3), ] <- 1
    Error <vctrs_error_subscript_type>
      Must assign to rows with a valid subscript vector.
      x Subscript `list(1:3)` has the wrong type `list`.
      i It must be logical, numeric, or character.
    Code
      foo[as.list(1:3), ] <- 1
    Error <vctrs_error_subscript_type>
      Must assign to rows with a valid subscript vector.
      x Subscript `as.list(1:3)` has the wrong type `list`.
      i It must be logical, numeric, or character.
    Code
      foo[factor(1:3), ] <- 1
    Error <simpleError>
      Can't use character names to index an unnamed vector.
    Code
      foo[Sys.Date(), ] <- 1
    Error <vctrs_error_subscript_type>
      Must assign to rows with a valid subscript vector.
      x Subscript `Sys.Date()` has the wrong type `date`.
      i It must be logical, numeric, or character.
    Code
      # # [<-.tbl_df throws an error with duplicate indexes (#658)
    Code
      df <- tibble(x = 1:2, y = x)
    Code
      df[c(1, 1)] <- 3
    Error <tibble_error_duplicate_column_subscript_for_assignment>
      Column index 1 is used more than once for assignment.
    Code
      df[, c(1, 1)] <- 3
    Error <tibble_error_duplicate_column_subscript_for_assignment>
      Column index 1 is used more than once for assignment.
    Code
      df[c(1, 1), ] <- 3
    Error <tibble_error_duplicate_row_subscript_for_assignment>
      Row index 1 is used more than once for assignment.
    Code
      # # [<-.tbl_df throws an error with NA indexes
    Code
      df <- tibble(x = 1:2, y = x)
    Code
      df[NA] <- 3
    Error <tibble_error_na_column_index>
      Can't use NA as column index with `[` at positions 1 and 2.
    Code
      df[NA, ] <- 3
    Error <tibble_error_assign_rows_non_na_only>
      Can't use NA as row index in a tibble for assignment.
    Code
      # # [<-.tbl_df throws an error with bad RHS
    Code
      df <- tibble(x = 1:2, y = x)
    Code
      df[] <- mean
    Error <tibble_error_need_rhs_vector_or_null>
      `mean` must be a vector, a bare list, a data frame, a matrix, or NULL.
    Code
      df[] <- lm(y ~ x, df)
    Error <tibble_error_need_rhs_vector_or_null>
      `lm(y ~ x, df)` must be a vector, a bare list, a data frame, a matrix, or NULL.
    Code
      # # [<-.tbl_df throws an error with OOB assignment
    Code
      df <- tibble(x = 1:2, y = x)
    Code
      df[4:5] <- 3
    Error <vctrs_error_subscript_oob>
      Can't assign to columns beyond the end with non-consecutive locations.
      i Input has size 2.
      x Subscript `4:5` contains non-consecutive location 4.
    Code
      df[4:5, ] <- 3
    Error <vctrs_error_subscript_oob>
      Can't assign to rows beyond the end with non-consecutive locations.
      i Input has size 2.
      x Subscript `4:5` contains non-consecutive location 4.
    Code
      df[-4, ] <- 3
    Error <vctrs_error_subscript_oob>
      Can't negate rows that don't exist.
      x Location 4 doesn't exist.
      i There are only 2 rows.
    Code
      df[-(4:5), ] <- 3
    Error <vctrs_error_subscript_oob>
      Can't negate rows that don't exist.
      x Locations 4 and 5 don't exist.
      i There are only 2 rows.
    Code
      # # [<-.tbl_df and recycling
    Code
      df <- tibble(x = 1:3, y = x, z = y)
    Code
      df[1:2] <- list(0, 0, 0)
    Error <vctrs_error_incompatible_size>
      Can't recycle `list(0, 0, 0)` (size 3) to size 2.
    Code
      df[] <- list(0, 0)
    Error <vctrs_error_incompatible_size>
      Can't recycle input of size 2 to size 3.
    Code
      df[1, ] <- 1:3
    Error <tibble_error_assign_incompatible_size>
      Assigned data `1:3` must be compatible with row subscript `1`.
      x 1 row must be assigned.
      x Assigned data has 3 rows.
      i Row updates require a list value. Do you need `list()` or `as.list()`?
    Code
      df[1:2, ] <- 1:3
    Error <tibble_error_assign_incompatible_size>
      Assigned data `1:3` must be compatible with row subscript `1:2`.
      x 2 rows must be assigned.
      x Assigned data has 3 rows.
      i Only vectors of size 1 are recycled.
    Code
      df[, ] <- 1:2
    Error <tibble_error_assign_incompatible_size>
      Assigned data `1:2` must be compatible with existing data.
      x Existing data has 3 rows.
      x Assigned data has 2 rows.
      i Only vectors of size 1 are recycled.
    Code
      # # [<-.tbl_df and coercion
    Code
      df <- tibble(x = 1:3, y = letters[1:3], z = as.list(1:3))
    Code
      df[1:3, 1:2] <- df[2:3]
    Error <tibble_error_assign_incompatible_type>
      Assigned data `df[2:3]` must be compatible with existing data.
      i Error occurred for column `x`.
      x Can't convert <character> to <integer>.
    Code
      df[1:3, 1:2] <- df[1]
    Error <tibble_error_assign_incompatible_type>
      Assigned data `df[1]` must be compatible with existing data.
      i Error occurred for column `y`.
      x Can't convert <integer> to <character>.
    Code
      df[1:3, 1:2] <- df[[1]]
    Error <tibble_error_assign_incompatible_type>
      Assigned data `df[[1]]` must be compatible with existing data.
      i Error occurred for column `y`.
      x Can't convert <integer> to <character>.
    Code
      df[1:3, 1:3] <- df[3:1]
    Error <tibble_error_assign_incompatible_type>
      Assigned data `df[3:1]` must be compatible with existing data.
      i Error occurred for column `x`.
      x Can't convert <list> to <integer>.
    Code
      df[1:3, 1:3] <- NULL
    Error <tibble_error_need_rhs_vector>
      `NULL` must be a vector, a bare list, a data frame or a matrix.
    Code
      # # [<-.tbl_df and overwriting NA
    Code
      df <- tibble(x = rep(NA, 3))
    Code
      df[1, "x"] <- 5
    Error <tibble_error_assign_incompatible_type>
      Assigned data `5` must be compatible with existing data.
      i Error occurred for column `x`.
      x Can't convert from <double> to <logical> due to loss of precision.
      * Locations: 1.
    Code
      # # [<-.tbl_df and matrix subsetting
    Code
      foo <- tibble(a = 1:3, b = letters[1:3])
    Code
      foo[!is.na(foo)] <- "bogus"
    Error <tibble_error_assign_incompatible_type>
      Assigned data `"bogus"` must be compatible with existing data.
      i Error occurred for column `a`.
      x Can't convert <character> to <integer>.
    Code
      foo[as.matrix("x")] <- NA
    Error <tibble_error_subset_matrix_must_be_logical>
      Subscript `as.matrix("x")` is a matrix, it must be of type logical.
    Code
      foo[array("x", dim = c(1, 1, 1))] <- NA
    Code
      foo[is.na(foo)] <- 1:3
    Error <tibble_error_subset_matrix_must_be_scalar>
      Subscript `is.na(foo)` is a matrix, the data `1:3` must have size 1.
    Code
      foo[is.na(foo)] <- lm(a ~ b, foo)
    Error <tibble_error_subset_matrix_scalar_type>
      Subscript `is.na(foo)` is a matrix, the data `lm(a ~ b, foo)` must be a vector of size 1.
    Code
      # # [[.tbl_df rejects invalid column indexes
    Code
      foo <- tibble(x = 1:10, y = 1:10)
    Code
      foo[[]] <- 1
    Error <tibble_error_assign_columns_non_missing_only>
      Subscript can't be missing for tibbles in `[[<-`.
    Code
      foo[[, 1]] <- 1
    Error <tibble_error_assign_columns_non_missing_only>
      Subscript can't be missing for tibbles in `[[<-`.
    Code
      foo[[1, ]] <- 1
    Error <tibble_error_assign_columns_non_missing_only>
      Subscript can't be missing for tibbles in `[[<-`.
    Code
      foo[[, ]] <- 1
    Error <tibble_error_assign_columns_non_missing_only>
      Subscript can't be missing for tibbles in `[[<-`.
    Code
      foo[[1:3]] <- 1
    Error <vctrs_error_subscript_type>
      Must assign to column with a single valid subscript.
      x Subscript `1:3` has size 3 but must be size 1.
    Code
      foo[[letters[1:3]]] <- 1
    Error <vctrs_error_subscript_type>
      Must assign to column with a single valid subscript.
      x Subscript `letters[1:3]` has size 3 but must be size 1.
    Code
      foo[[TRUE]] <- 1
    Error <vctrs_error_subscript_type>
      Must assign to column with a single valid subscript.
      x Subscript `TRUE` has the wrong type `logical`.
      i It must be numeric or character.
    Code
      foo[[mean]] <- 1
    Error <vctrs_error_subscript_type>
      Must assign to columns with a valid subscript vector.
      x Subscript `mean` has the wrong type `function`.
      i It must be logical, numeric, or character.
    Code
      foo[[foo]] <- 1
    Error <vctrs_error_subscript_type>
      Must assign to column with a single valid subscript.
      x Subscript `foo` has the wrong type `tbl_df<
        x: integer
        y: integer
      >`.
      i It must be numeric or character.
    Code
      foo[[1:3, 1]] <- 1
    Error <vctrs_error_subscript_type>
      Must assign to row with a single valid subscript.
      x Subscript `1:3` has size 3 but must be size 1.
    Code
      foo[[TRUE, 1]] <- 1
    Error <vctrs_error_subscript_type>
      Must assign to row with a single valid subscript.
      x Subscript `TRUE` has the wrong type `logical`.
      i It must be numeric or character.
    Code
      foo[[mean, 1]] <- 1
    Error <vctrs_error_subscript_type>
      Must assign to row with a single valid subscript.
      x Subscript `mean` has the wrong type `function`.
      i It must be numeric or character.
    Code
      foo[[foo, 1]] <- 1
    Error <vctrs_error_subscript_type>
      Must assign to row with a single valid subscript.
      x Subscript `foo` has the wrong type `tbl_df<
        x: integer
        y: integer
      >`.
      i It must be numeric or character.
    Code
      # # [[<-.tbl_df throws an error with OOB assignment
    Code
      df <- tibble(x = 1:2, y = x)
    Code
      df[[4]] <- 3
    Error <vctrs_error_subscript_oob>
      Can't assign to columns beyond the end with non-consecutive locations.
      i Input has size 2.
      x Subscript `4` contains non-consecutive location 4.
    Code
      # # [[<-.tbl_df throws an error with bad RHS
    Code
      df <- tibble(x = 1:2, y = x)
    Code
      df[[1]] <- mean
    Error <vctrs_error_scalar_type>
      Input must be a vector, not a function.
    Code
      df[[1]] <- lm(y ~ x, df)
    Error <vctrs_error_scalar_type>
      Input must be a vector, not a `lm` object.
    Code
      # # [[<-.tbl_df recycles only values of length one
    Code
      df <- tibble(x = 1:3)
    Code
      df[["x"]] <- 8:9
    Error <tibble_error_assign_incompatible_size>
      Assigned data `8:9` must be compatible with existing data.
      x Existing data has 3 rows.
      x Assigned data has 2 rows.
      i Only vectors of size 1 are recycled.
    Code
      df[["w"]] <- 8:9
    Error <tibble_error_assign_incompatible_size>
      Assigned data `8:9` must be compatible with existing data.
      x Existing data has 3 rows.
      x Assigned data has 2 rows.
      i Only vectors of size 1 are recycled.
    Code
      df[["a"]] <- character()
    Error <tibble_error_assign_incompatible_size>
      Assigned data `character()` must be compatible with existing data.
      x Existing data has 3 rows.
      x Assigned data has 0 rows.
      i Only vectors of size 1 are recycled.
    Code
      # # [<-.tbl_df throws an error with invalid values
    Code
      df <- tibble(x = 1:2, y = x)
    Code
      df[1] <- lm(y ~ x, df)
    Error <tibble_error_need_rhs_vector_or_null>
      `lm(y ~ x, df)` must be a vector, a bare list, a data frame, a matrix, or NULL.
    Code
      df[1:2, 1] <- NULL
    Error <tibble_error_need_rhs_vector>
      `NULL` must be a vector, a bare list, a data frame or a matrix.
    Code
      # # $<- recycles only values of length one
    Code
      df <- tibble(x = 1:3)
    Code
      df$x <- 8:9
    Error <tibble_error_assign_incompatible_size>
      Assigned data `8:9` must be compatible with existing data.
      x Existing data has 3 rows.
      x Assigned data has 2 rows.
      i Only vectors of size 1 are recycled.
    Code
      df$w <- 8:9
    Error <tibble_error_assign_incompatible_size>
      Assigned data `8:9` must be compatible with existing data.
      x Existing data has 3 rows.
      x Assigned data has 2 rows.
      i Only vectors of size 1 are recycled.
    Code
      df$a <- character()
    Error <tibble_error_assign_incompatible_size>
      Assigned data `character()` must be compatible with existing data.
      x Existing data has 3 rows.
      x Assigned data has 0 rows.
      i Only vectors of size 1 are recycled.

# output test

    Code
      df <- tibble(a = 1, b = 2)
    Code
      names(df) <- NULL
    Warning <lifecycle_warning_deprecated>
      The `value` argument of ``names<-`()` can't be NULL as of tibble 3.0.0.
      `names` must not be `NULL`.
    Warning <lifecycle_warning_deprecated>
      The `value` argument of ``names<-`()` must have the same length as `x` as of tibble 3.0.0.
      `names` must have length 2, not 0.
    Warning <lifecycle_warning_deprecated>
      The `value` argument of ``names<-`()` can't be empty as of tibble 3.0.0.
      Columns 1 and 2 must be named.
    Code
      names(df) <- "c"
    Warning <lifecycle_warning_deprecated>
      The `value` argument of ``names<-`()` must have the same length as `x` as of tibble 3.0.0.
      `names` must have length 2, not 1.
    Warning <lifecycle_warning_deprecated>
      The `value` argument of ``names<-`()` can't be empty as of tibble 3.0.0.
      Column 2 must be named.
    Code
      names(df) <- c("..1", "..2")

# output test

    Code
      tibble(a = 1, a = 1)
    Error <tibble_error_column_names_must_be_unique>
      Column name `a` must not be duplicated.
      Use .name_repair to specify repair.
    Code
      tibble(a = new_environment())
    Error <tibble_error_column_scalar_type>
      All columns in a tibble must be vectors.
      x Column `a` is an environment.
    Code
      tibble(a = 1, b = 2:3, c = 4:6, d = 7:10)
    Error <tibble_error_incompatible_size>
      Tibble columns must have compatible sizes.
      * Size 2: Existing data.
      * Size 3: Column `c`.
      i Only values of size one are recycled.

# output test

    Code
      repaired_names(letters[1:3], repair_hint = FALSE)
    Output
      [1] "a" "b" "c"
    Code
      repaired_names("", repair_hint = FALSE)
    Error <tibble_error_column_names_cannot_be_empty>
      Column 1 must be named.
    Code
      repaired_names("", repair_hint = TRUE)
    Error <tibble_error_column_names_cannot_be_empty>
      Column 1 must be named.
      Use .name_repair to specify repair.
    Code
      repaired_names(c("a", "a"), repair_hint = FALSE)
    Error <tibble_error_column_names_must_be_unique>
      Column name `a` must not be duplicated.
    Code
      repaired_names("..1", repair_hint = FALSE)
    Error <tibble_error_column_names_cannot_be_dot_dot>
      Column 1 must not have names of the form ... or ..j.
    Code
      repaired_names(c("a", "a"), repair_hint = FALSE, .name_repair = "universal")
    Message <rlib_message_name_repair>
      New names:
      * `a` -> `a...1`
      * `a` -> `a...2`
    Output
      [1] "a...1" "a...2"
    Code
      repaired_names(c("a", "a"), repair_hint = FALSE, .name_repair = "universal",
      quiet = TRUE)
    Output
      [1] "a...1" "a...2"
    Code
      repaired_names(c("if"), repair_hint = FALSE, .name_repair = "universal")
    Message <rlib_message_name_repair>
      New names:
      * `if` -> `.if`
    Output
      [1] ".if"

# output test

    Code
      add_row(tibble(), a = 1)
    Error <tibble_error_incompatible_new_rows>
      New rows can't add columns.
      x Can't find column `a` in `.data`.
    Code
      add_row(tibble(), a = 1, b = 2)
    Error <tibble_error_incompatible_new_rows>
      New rows can't add columns.
      x Can't find columns `a` and `b` in `.data`.
    Code
      add_row(tibble(), a = "a", b = "b", c = "c", d = "d", e = "e", f = "f", g = "g",
      h = "h", i = "i", j = "j", k = "k", l = "l", m = "m", n = "n", o = "o", p = "p",
      q = "q", r = "r", s = "s", t = "t", u = "u", v = "v", w = "w", x = "x", y = "y",
      z = "z")
    Error <tibble_error_incompatible_new_rows>
      New rows can't add columns.
      x Can't find columns `a`, `b`, `c`, `d`, `e`, and 21 more in `.data`.
    Code
      add_row(dplyr::group_by(tibble(a = 1), a))
    Error <tibble_error_add_rows_to_grouped_df>
      Can't add rows to grouped data frames.
    Code
      add_row(tibble(a = 1), a = 2, .before = 1, .after = 1)
    Error <tibble_error_both_before_after>
      Can't specify both `.before` and `.after`.
    Code
      add_column(tibble(a = 1), a = 1)
    Error <tibble_error_column_names_must_be_unique>
      Column name `a` must not be duplicated.
      Use .name_repair to specify repair.
      Caused by error in `stop_vctrs()`:
      ! Names must be unique.
      x These names are duplicated:
        * "a" at locations 1 and 2.
    Code
      add_column(tibble(a = 1, b = 2), a = 1, b = 2)
    Error <tibble_error_column_names_must_be_unique>
      Column names `a` and `b` must not be duplicated.
      Use .name_repair to specify repair.
      Caused by error in `stop_vctrs()`:
      ! Names must be unique.
      x These names are duplicated:
        * "a" at locations 1 and 3.
        * "b" at locations 2 and 4.
    Code
      add_column(tibble(a = "a", b = "b", c = "c", d = "d", e = "e", f = "f", g = "g",
        h = "h", i = "i", j = "j", k = "k", l = "l", m = "m", n = "n", o = "o", p = "p",
        q = "q", r = "r", s = "s", t = "t", u = "u", v = "v", w = "w", x = "x", y = "y",
        z = "z"), a = "a", b = "b", c = "c", d = "d", e = "e", f = "f", g = "g", h = "h",
      i = "i", j = "j", k = "k", l = "l", m = "m", n = "n", o = "o", p = "p", q = "q",
      r = "r", s = "s", t = "t", u = "u", v = "v", w = "w", x = "x", y = "y", z = "z")
    Error <tibble_error_column_names_must_be_unique>
      Column names `a`, `b`, `c`, `d`, `e`, and 21 more must not be duplicated.
      Use .name_repair to specify repair.
      Caused by error in `stop_vctrs()`:
      ! Names must be unique.
      x These names are duplicated:
        * "a" at locations 1 and 27.
        * "b" at locations 2 and 28.
        * "c" at locations 3 and 29.
        * "d" at locations 4 and 30.
        * "e" at locations 5 and 31.
        * ...
    Code
      add_column(tibble(a = 2:3), b = 4:6)
    Error <tibble_error_incompatible_new_cols>
      New columns must be compatible with `.data`.
      x New column has 3 rows.
      i `.data` has 2 rows.
    Code
      add_column(tibble(a = 1), b = 1, .before = 1, .after = 1)
    Error <tibble_error_both_before_after>
      Can't specify both `.before` and `.after`.

---
title: "Column types"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Column types}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---




```r
library(tibble)
```

## Overview

This vignette shows an overview of known data types and their abbreviations, and their origin.
For example, `<int>` in the header of a column indicates an integer column, and `<chr>` denotes a character column.










|Class                       |Data type            |Example                          |Column header |
|:---------------------------|:--------------------|:--------------------------------|:-------------|
|Atomic                      |logical              |`TRUE`                           |lgl           |
|                            |integer              |`1L`                             |int           |
|                            |double               |`1.5`                            |dbl           |
|                            |character            |`"A"`                            |chr           |
|                            |complex              |`0+1i`                           |cpl           |
|                            |raw                  |`as.raw(1)`                      |raw           |
|                            |list                 |`list(1)`                        |list          |
|                            |named list           |`list(a = 1)`                    |named list    |
|Built-in objects            |factor               |`factor("A")`                    |fct           |
|                            |ordered              |`ordered("a")`                   |ord           |
|                            |Date                 |`Sys.Date()`                     |date          |
|                            |POSIXt               |`Sys.time()`                     |dttm          |
|                            |difftime             |`vctrs::new_duration(1)`         |drtn          |
|Objects from other packages |hms                  |`hms::hms(1)`                    |time          |
|                            |integer64            |`bit64::as.integer64(1e+10)`     |int64         |
|                            |blob                 |`blob::blob(raw(1))`             |blob          |
|Data frames                 |data.frame           |`data.frame(a = 1)`              |df[,1]        |
|                            |tbl_df               |`tibble(a = 1)`                  |tibble[,1]    |
|Unchanged                   |AsIs                 |`I(1L)`                          |I<int>        |
|vctrs types                 |unspecified          |`vctrs::unspecified(1)`          |???           |
|                            |vctrs_list_of        |`vctrs::list_of(c(1L))`          |list<int>     |
|                            |vctrs_vctr           |`vctrs::new_vctr(1L)`            |vctrs_vc      |
|                            |vctrs_partial_factor |`vctrs::partial_factor(letters)` |prtl_fctr     |
|                            |vctrs_partial_frame  |`vctrs::partial_frame(a = 1)`    |prtl          |
|Language objects            |function             |`function() NULL`                |fn            |
|                            |symbol               |`quote(a)`                       |sym           |
|                            |expression           |`parse(text = "a <- 1\nb<- 2")`  |expression    |
|                            |quosures             |`rlang::quos(a = 1)`             |quos          |

## Example values

The following overview contains example values for each type:


```
#> Rows: 1
#> Columns: 23
#> $ logical       <lgl> TRUE
#> $ integer       <int> 1
#> $ double        <dbl> 1.5
#> $ character     <chr> "A"
#> $ complex       <cpl> 0+1i
#> $ raw           <raw> 01
#> $ list          <list> 1
#> $ `named list`  <named list> 1
#> $ factor        <fct> A
#> $ ordered       <ord> a
#> $ Date          <date> 2021-07-30
#> $ POSIXt        <dttm> 2021-07-30 04:11:25
#> $ difftime      <drtn> 1 secs
#> $ hms           <time> 00:00:01
#> $ integer64     <int64> 10000000000
#> $ blob          <blob> 00
#> $ data.frame    <df[,1]> <data.frame[1 x 1]>
#> $ tbl_df        <tibble[,1]> <tbl_df[1 x 1]>
#> $ AsIs          <I<int>> 1
#> $ unspecified   <???> NA
#> $ vctrs_list_of <list<int>> 1
#> $ vctrs_vctr    <vctrs_vc> 1
#> $ quosures      <quos> <~1>
```


## Implementation

The column header is obtained by calling `pillar::type_sum()` on the column.
This is an S3 method that can be overridden,
but most of the time it is more useful to override `vctrs::vec_ptype_abbr()`:


```r
pillar:::type_sum.default
```

# output test

    Code
      invalid_df("must be integer", "col", "\nFix this.")
    Output
      [1] "Column `col` must be integer.\nFix this."
    Code
      invalid_df("must be numeric", c("col1", "col2"))
    Output
      [1] "Columns `col1` and `col2` must be numeric."
    Code
      use_repair(TRUE)
    Output
      [1] "\nUse .name_repair to specify repair."
    Code
      use_repair(FALSE)

# output test

    Code
      add_row(tibble(), a = 1)
    Condition
      Error:
      ! New rows can't add columns.
      x Can't find column `a` in `.data`.
    Code
      add_row(tibble(), a = 1, b = 2)
    Condition
      Error:
      ! New rows can't add columns.
      x Can't find columns `a` and `b` in `.data`.
    Code
      add_row(tibble(), a = "a", b = "b", c = "c", d = "d", e = "e", f = "f", g = "g",
      h = "h", i = "i", j = "j", k = "k", l = "l", m = "m", n = "n", o = "o", p = "p",
      q = "q", r = "r", s = "s", t = "t", u = "u", v = "v", w = "w", x = "x", y = "y",
      z = "z")
    Condition
      Error:
      ! New rows can't add columns.
      x Can't find columns `a`, `b`, `c`, `d`, `e`, and 21 more in `.data`.
    Code
      add_row(dplyr::group_by(tibble(a = 1), a))
    Condition
      Error:
      ! Can't add rows to grouped data frames.
    Code
      add_row(tibble(a = 1), a = 2, .before = 1, .after = 1)
    Condition
      Error:
      ! Can't specify both `.before` and `.after`.
    Code
      add_column(tibble(a = 1), a = 1)
    Condition
      Error:
      ! Column name `a` must not be duplicated.
      Use .name_repair to specify repair.
      Caused by error in `repaired_names()`:
      ! Names must be unique.
      x These names are duplicated:
        * "a" at locations 1 and 2.
    Code
      add_column(tibble(a = 1, b = 2), a = 1, b = 2)
    Condition
      Error:
      ! Column names `a` and `b` must not be duplicated.
      Use .name_repair to specify repair.
      Caused by error in `repaired_names()`:
      ! Names must be unique.
      x These names are duplicated:
        * "a" at locations 1 and 3.
        * "b" at locations 2 and 4.
    Code
      add_column(tibble(a = "a", b = "b", c = "c", d = "d", e = "e", f = "f", g = "g",
        h = "h", i = "i", j = "j", k = "k", l = "l", m = "m", n = "n", o = "o", p = "p",
        q = "q", r = "r", s = "s", t = "t", u = "u", v = "v", w = "w", x = "x", y = "y",
        z = "z"), a = "a", b = "b", c = "c", d = "d", e = "e", f = "f", g = "g", h = "h",
      i = "i", j = "j", k = "k", l = "l", m = "m", n = "n", o = "o", p = "p", q = "q",
      r = "r", s = "s", t = "t", u = "u", v = "v", w = "w", x = "x", y = "y", z = "z")
    Condition
      Error:
      ! Column names `a`, `b`, `c`, `d`, `e`, and 21 more must not be duplicated.
      Use .name_repair to specify repair.
      Caused by error in `repaired_names()`:
      ! Names must be unique.
      x These names are duplicated:
        * "a" at locations 1 and 27.
        * "b" at locations 2 and 28.
        * "c" at locations 3 and 29.
        * "d" at locations 4 and 30.
        * "e" at locations 5 and 31.
        * ...
    Code
      add_column(tibble(a = 2:3), b = 4:6)
    Condition
      Error:
      ! New columns must be compatible with `.data`.
      x New column has 3 rows.
      i `.data` has 2 rows.
    Code
      add_column(tibble(a = 1), b = 1, .before = 1, .after = 1)
    Condition
      Error:
      ! Can't specify both `.before` and `.after`.

# output test

    Code
      mtcars2 <- as_tibble(mtcars, rownames = NA)
      print_without_body(mtcars2, n = 8L, width = 30L)
    Output
      # A tibble: 32 x 11
      <body created by pillar>
      # ... with 24 more rows, and
      #   7 more variables:
      #   drat <dbl>, wt <dbl>,
      #   qsec <dbl>, vs <dbl>,
      #   am <dbl>, gear <dbl>,
      #   carb <dbl>
    Code
      print_without_body(as_tibble(trees), n = 5L, width = 30L)
    Output
      # A tibble: 31 x 3
      <body created by pillar>
      # ... with 26 more rows
    Code
      print_without_body(as_tibble(trees), n = -1L, width = 30L)
    Output
      # A tibble: 31 x 3
      <body created by pillar>
      # ... with 21 more rows
    Code
      print_without_body(as_tibble(trees), n = Inf, width = 15L)
    Output
      # A tibble: 31
      #   x 3
      <body created by pillar>
      # ... with 1
      #   more
      #   variable:
      #   Volume <dbl>
    Code
      print_without_body(as_tibble(trees), n = NULL, width = 70L)
    Output
      # A tibble: 31 x 3
      <body created by pillar>
      # ... with 21 more rows
    Code
      print_without_body(as_unknown_rows(trees), n = 10, width = 70L)
    Output
      # A tibble: ?? x 3
      <body created by pillar>
      # ... with more rows
    Code
      print_without_body(as_unknown_rows(trees[1:9, ]), n = 10, width = 70L)
    Output
      # A tibble: ?? x 3
      <body created by pillar>
    Code
      print_without_body(as_unknown_rows(trees[1:10, ]), n = 10, width = 70L)
    Output
      # A tibble: ?? x 3
      <body created by pillar>
    Code
      print_without_body(as_unknown_rows(trees[1:11, ]), n = 10, width = 70L)
    Output
      # A tibble: ?? x 3
      <body created by pillar>
      # ... with more rows
    Code
      print_without_body(df_all, n = NULL, width = 30L)
    Output
      # A tibble: 3 x 9
      <body created by pillar>
      # ... with 5 more variables:
      #   e <fct>, f <date>,
      #   g <dttm>, h <list>,
      #   i <list>
    Code
      print_without_body(df_all, n = NULL, width = 300L)
    Output
      # A tibble: 3 x 9
      <body created by pillar>
    Code
      print_without_body(tibble(a = seq.int(10000)), n = 5L, width = 30L)
    Output
      # A tibble: 10,000 x 1
      <body created by pillar>
      # ... with 9,995 more rows
    Code
      print_without_body(tibble(a = character(), b = logical()), width = 30L)
    Output
      # A tibble: 0 x 2
      <body created by pillar>
      # ... with 2 variables:
      #   a <chr>, b <lgl>
    Code
      print_without_body(as_tibble(trees)[character()], n = 5L, width = 30L)
    Output
      # A tibble: 31 x 0
      <body created by pillar>
    Code
      print_without_body(as_unknown_rows(trees[integer(), ]), n = 5L, width = 30L)
    Output
      # A tibble: ?? x 3
      <body created by pillar>
      # ... with 3 variables:
      #   Girth <dbl>,
      #   Height <dbl>,
      #   Volume <dbl>
    Code
      print_without_body(as_unknown_rows(trees[, character()]), n = 5L, width = 30L)
    Output
      # A tibble: ?? x 0
      <body created by pillar>
      # ... with at least 5 rows
      #   total
    Code
      print_without_body(as_unknown_rows(tibble(a = seq.int(10000))), n = 5L, width = 30L)
    Output
      # A tibble: ?? x 1
      <body created by pillar>
      # ... with more rows

# full output test

    Code
      df <- tibble(x = as.POSIXct("2016-01-01 12:34:56 GMT") + 1:12)
      df$y <- as.POSIXlt(df$x)
      print(df, n = 8L, width = 60L)
    Output
      # A tibble: 12 x 2
        x                   y                  
        <dttm>              <dttm>             
      1 2016-01-01 12:34:57 2016-01-01 12:34:57
      2 2016-01-01 12:34:58 2016-01-01 12:34:58
      3 2016-01-01 12:34:59 2016-01-01 12:34:59
      4 2016-01-01 12:35:00 2016-01-01 12:35:00
      5 2016-01-01 12:35:01 2016-01-01 12:35:01
      6 2016-01-01 12:35:02 2016-01-01 12:35:02
      7 2016-01-01 12:35:03 2016-01-01 12:35:03
      8 2016-01-01 12:35:04 2016-01-01 12:35:04
      # ... with 4 more rows
    Code
      x <- c("成交日期", "合同录入日期")
      df <- setNames(tibble(1:3, 4:6), x)
      print(df, n = 8L, width = 60L)
    Output
      # A tibble: 3 x 2
        成交日期 合同录入日期
           <int>        <int>
      1        1            4
      2        2            5
      3        3            6

# output test

    Code
      rownames_to_column(mtcars, "cyl")
    Error <tibble_error_column_names_must_be_unique>
      Column name `cyl` must not be duplicated.
      Caused by error in `stop_vctrs()`:
      ! Names must be unique.
      x These names are duplicated:
        * "cyl" at locations 2 and 12.
    Code
      rowid_to_column(trees, "Volume")
    Error <tibble_error_column_names_must_be_unique>
      Column name `Volume` must not be duplicated.
      Caused by error in `stop_vctrs()`:
      ! Names must be unique.
      x These names are duplicated:
        * "Volume" at locations 3 and 4.
    Code
      column_to_rownames(mtcars, "cyl")
    Error <tibble_error_already_has_rownames>
      `.data` must be a data frame without row names.
    Code
      column_to_rownames(trees, "foo")
    Error <tibble_error_unknown_column_names>
      Can't find column `foo` in `.data`.

# output test

    Code
      rownames_to_column(mtcars, "cyl")
    Error <tibble_error_column_names_must_be_unique>
      Column name `cyl` must not be duplicated.
    Code
      rowid_to_column(trees, "Volume")
    Error <tibble_error_column_names_must_be_unique>
      Column name `Volume` must not be duplicated.
    Code
      column_to_rownames(mtcars, "cyl")
    Error <tibble_error_already_has_rownames>
      `.data` must be a data frame without row names.
    Code
      column_to_rownames(trees, "foo")
    Error <tibble_error_unknown_column_names>
      Can't find column `foo` in `.data`.

# output test

    Code
      new_tibble(1:3, nrow = 1)
    Error <tibble_error_new_tibble_must_be_list>
      `x` must be a list.
    Code
      new_tibble(as.list(1:3))
    Error <tibble_error_names_must_be_non_null>
      `names` must not be `NULL`.

# output test

    Code
      as_tibble(list(1))
    Error <tibble_error_column_names_cannot_be_empty>
      Column 1 must be named.
      Use .name_repair to specify repair.
    Code
      as_tibble(list(1, 2))
    Error <tibble_error_column_names_cannot_be_empty>
      Columns 1 and 2 must be named.
      Use .name_repair to specify repair.
    Code
      as_tibble(list(a = 1, 2))
    Error <tibble_error_column_names_cannot_be_empty>
      Column 2 must be named.
      Use .name_repair to specify repair.
    Code
      as_tibble(as.list(1:26))
    Error <tibble_error_column_names_cannot_be_empty>
      Columns 1, 2, 3, 4, 5, and 21 more must be named.
      Use .name_repair to specify repair.
    Code
      as_tibble(set_names(list(1), "..1"))
    Error <tibble_error_column_names_cannot_be_dot_dot>
      Column 1 must not have names of the form ... or ..j.
      Use .name_repair to specify repair.
    Code
      as_tibble(set_names(list(1:26), paste0("..", 1:26)))
    Error <rlang_error>
      `nm` must be `NULL` or a character vector the same length as `x`
    Code
      as_tibble(list(a = 1, a = 1))
    Error <tibble_error_column_names_must_be_unique>
      Column name `a` must not be duplicated.
      Use .name_repair to specify repair.
    Code
      as_tibble(list(a = 1, a = 1, b = 1, b = 1))
    Error <tibble_error_column_names_must_be_unique>
      Column names `a` and `b` must not be duplicated.
      Use .name_repair to specify repair.
    Code
      as_tibble(list(a = new_environment()))
    Error <tibble_error_column_scalar_type>
      All columns in a tibble must be vectors.
      x Column `a` is an environment.
    Code
      as_tibble_row(list(1))
    Error <tibble_error_column_names_cannot_be_empty>
      Column 1 must be named.
      Use .name_repair to specify repair.
    Code
      as_tibble_row(list(1, 2))
    Error <tibble_error_column_names_cannot_be_empty>
      Columns 1 and 2 must be named.
      Use .name_repair to specify repair.
    Code
      as_tibble_row(list(a = 1, 2))
    Error <tibble_error_column_names_cannot_be_empty>
      Column 2 must be named.
      Use .name_repair to specify repair.
    Code
      as_tibble_row(as.list(1:26))
    Error <tibble_error_column_names_cannot_be_empty>
      Columns 1, 2, 3, 4, 5, and 21 more must be named.
      Use .name_repair to specify repair.
    Code
      as_tibble_row(set_names(list(1), "..1"))
    Error <tibble_error_column_names_cannot_be_dot_dot>
      Column 1 must not have names of the form ... or ..j.
      Use .name_repair to specify repair.
    Code
      as_tibble_row(set_names(list(1:26), paste0("..", 1:26)))
    Error <rlang_error>
      `nm` must be `NULL` or a character vector the same length as `x`
    Code
      as_tibble_row(list(a = 1, a = 1))
    Error <tibble_error_column_names_must_be_unique>
      Column name `a` must not be duplicated.
      Use .name_repair to specify repair.
    Code
      as_tibble_row(list(a = 1, a = 1, b = 1, b = 1))
    Error <tibble_error_column_names_must_be_unique>
      Column names `a` and `b` must not be duplicated.
      Use .name_repair to specify repair.
    Code
      as_tibble_row(list(a = new_environment()))
    Error <vctrs_error_scalar_type>
      Input must be a vector, not an environment.
    Code
      as_tibble_row(list(a = 1:3))
    Error <tibble_error_as_tibble_row_size_one>
      All elements must be size one, use `list()` to wrap.
      x Element `a` is of size 3.
    Code
      as_tibble_row(list(a = 1:3, b = 1:3))
    Error <tibble_error_as_tibble_row_size_one>
      All elements must be size one, use `list()` to wrap.
      x Element `a` is of size 3.
      x Element `b` is of size 3.

# output test

    Code
      as_tibble(list(1))
    Error <tibble_error_column_names_cannot_be_empty>
      Column 1 must be named.
      Use .name_repair to specify repair.
      Caused by error in `stop_vctrs()`:
      ! Names can't be empty.
      x Empty name found at location 1.
    Code
      as_tibble(list(1, 2))
    Error <tibble_error_column_names_cannot_be_empty>
      Columns 1 and 2 must be named.
      Use .name_repair to specify repair.
      Caused by error in `stop_vctrs()`:
      ! Names can't be empty.
      x Empty names found at locations 1 and 2.
    Code
      as_tibble(list(a = 1, 2))
    Error <tibble_error_column_names_cannot_be_empty>
      Column 2 must be named.
      Use .name_repair to specify repair.
      Caused by error in `stop_vctrs()`:
      ! Names can't be empty.
      x Empty name found at location 2.
    Code
      as_tibble(as.list(1:26))
    Error <tibble_error_column_names_cannot_be_empty>
      Columns 1, 2, 3, 4, 5, and 21 more must be named.
      Use .name_repair to specify repair.
      Caused by error in `stop_vctrs()`:
      ! Names can't be empty.
      x Empty names found at locations 1, 2, 3, 4, 5, etc.
    Code
      as_tibble(set_names(list(1), "..1"))
    Error <tibble_error_column_names_cannot_be_dot_dot>
      Column 1 must not have names of the form ... or ..j.
      Use .name_repair to specify repair.
      Caused by error in `stop_vctrs()`:
      ! Names can't be of the form `...` or `..j`.
      x These names are invalid:
        * "..1" at location 1.
    Code
      as_tibble(set_names(list(1:26), paste0("..", 1:26)))
    Error <rlang_error>
      The size of `nm` (26) must be compatible with the size of `x` (1).
    Code
      as_tibble(list(a = 1, a = 1))
    Error <tibble_error_column_names_must_be_unique>
      Column name `a` must not be duplicated.
      Use .name_repair to specify repair.
      Caused by error in `stop_vctrs()`:
      ! Names must be unique.
      x These names are duplicated:
        * "a" at locations 1 and 2.
    Code
      as_tibble(list(a = 1, a = 1, b = 1, b = 1))
    Error <tibble_error_column_names_must_be_unique>
      Column names `a` and `b` must not be duplicated.
      Use .name_repair to specify repair.
      Caused by error in `stop_vctrs()`:
      ! Names must be unique.
      x These names are duplicated:
        * "a" at locations 1 and 2.
        * "b" at locations 3 and 4.
    Code
      as_tibble(list(a = new_environment()))
    Error <tibble_error_column_scalar_type>
      All columns in a tibble must be vectors.
      x Column `a` is an environment.
    Code
      as_tibble_row(list(1))
    Error <tibble_error_column_names_cannot_be_empty>
      Column 1 must be named.
      Use .name_repair to specify repair.
      Caused by error in `stop_vctrs()`:
      ! Names can't be empty.
      x Empty name found at location 1.
    Code
      as_tibble_row(list(1, 2))
    Error <tibble_error_column_names_cannot_be_empty>
      Columns 1 and 2 must be named.
      Use .name_repair to specify repair.
      Caused by error in `stop_vctrs()`:
      ! Names can't be empty.
      x Empty names found at locations 1 and 2.
    Code
      as_tibble_row(list(a = 1, 2))
    Error <tibble_error_column_names_cannot_be_empty>
      Column 2 must be named.
      Use .name_repair to specify repair.
      Caused by error in `stop_vctrs()`:
      ! Names can't be empty.
      x Empty name found at location 2.
    Code
      as_tibble_row(as.list(1:26))
    Error <tibble_error_column_names_cannot_be_empty>
      Columns 1, 2, 3, 4, 5, and 21 more must be named.
      Use .name_repair to specify repair.
      Caused by error in `stop_vctrs()`:
      ! Names can't be empty.
      x Empty names found at locations 1, 2, 3, 4, 5, etc.
    Code
      as_tibble_row(set_names(list(1), "..1"))
    Error <tibble_error_column_names_cannot_be_dot_dot>
      Column 1 must not have names of the form ... or ..j.
      Use .name_repair to specify repair.
      Caused by error in `stop_vctrs()`:
      ! Names can't be of the form `...` or `..j`.
      x These names are invalid:
        * "..1" at location 1.
    Code
      as_tibble_row(set_names(list(1:26), paste0("..", 1:26)))
    Error <rlang_error>
      The size of `nm` (26) must be compatible with the size of `x` (1).
    Code
      as_tibble_row(list(a = 1, a = 1))
    Error <tibble_error_column_names_must_be_unique>
      Column name `a` must not be duplicated.
      Use .name_repair to specify repair.
      Caused by error in `stop_vctrs()`:
      ! Names must be unique.
      x These names are duplicated:
        * "a" at locations 1 and 2.
    Code
      as_tibble_row(list(a = 1, a = 1, b = 1, b = 1))
    Error <tibble_error_column_names_must_be_unique>
      Column names `a` and `b` must not be duplicated.
      Use .name_repair to specify repair.
      Caused by error in `stop_vctrs()`:
      ! Names must be unique.
      x These names are duplicated:
        * "a" at locations 1 and 2.
        * "b" at locations 3 and 4.
    Code
      as_tibble_row(list(a = new_environment()))
    Error <vctrs_error_scalar_type>
      Input must be a vector, not an environment.
    Code
      as_tibble_row(list(a = 1:3))
    Error <tibble_error_as_tibble_row_size_one>
      All elements must be size one, use `list()` to wrap.
      x Element `a` is of size 3.
    Code
      as_tibble_row(list(a = 1:3, b = 1:3))
    Error <tibble_error_as_tibble_row_size_one>
      All elements must be size one, use `list()` to wrap.
      x Element `a` is of size 3.
      x Element `b` is of size 3.

# output test

    Code
      tibble(a = 1, a = 1)
    Error <tibble_error_column_names_must_be_unique>
      Column name `a` must not be duplicated.
      Use .name_repair to specify repair.
      Caused by error in `stop_vctrs()`:
      ! Names must be unique.
      x These names are duplicated:
        * "a" at locations 1 and 2.
    Code
      tibble(a = new_environment())
    Error <tibble_error_column_scalar_type>
      All columns in a tibble must be vectors.
      x Column `a` is an environment.
    Code
      tibble(a = 1, b = 2:3, c = 4:6, d = 7:10)
    Error <tibble_error_incompatible_size>
      Tibble columns must have compatible sizes.
      * Size 2: Existing data.
      * Size 3: Column `c`.
      i Only values of size one are recycled.

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/subsetting.R
\name{subsetting}
\alias{subsetting}
\alias{$.tbl_df}
\alias{$<-.tbl_df}
\alias{[[.tbl_df}
\alias{[[<-.tbl_df}
\alias{[.tbl_df}
\alias{[<-.tbl_df}
\title{Subsetting tibbles}
\usage{
\method{$}{tbl_df}(x, name)

\method{$}{tbl_df}(x, name) <- value

\method{[[}{tbl_df}(x, i, j, ..., exact = TRUE)

\method{[[}{tbl_df}(x, i, j, ...) <- value

\method{[}{tbl_df}(x, i, j, drop = FALSE, ...)

\method{[}{tbl_df}(x, i, j, ...) <- value
}
\arguments{
\item{x}{A tibble.}

\item{name}{A \link{name} or a string.}

\item{value}{A value to store in a row, column, range or cell.
Tibbles are stricter than data frames in what is accepted here.}

\item{i, j}{Row and column indices. If \code{j} is omitted, \code{i} is used as column index.}

\item{...}{Ignored.}

\item{exact}{Ignored, with a warning.}

\item{drop}{Coerce to a vector if fetching one column via \code{tbl[, j]} .
Default \code{FALSE}, ignored when accessing a column via \code{tbl[j]} .}
}
\description{
Accessing columns, rows, or cells via \code{$}, \code{[[}, or \code{[} is mostly similar to
\link[base:Extract]{regular data frames}. However, the
behavior is different for tibbles and data frames in some cases:
\itemize{
\item \code{[} always returns a tibble by default, even if
only one column is accessed.
\item Partial matching of column names with \code{$} and \code{[[} is not supported, and
\code{NULL} is returned.
For \code{$}, a warning is given.
\item Only scalars (vectors of length one) or vectors with the
same length as the number of rows can be used for assignment.
\item Rows outside of the tibble's boundaries cannot be accessed.
\item When updating with \verb{[[<-} and \verb{[<-}, type changes of entire columns are
supported, but updating a part of a column requires that the new value is
coercible to the existing type.
See \code{\link[=vec_slice]{vec_slice()}} for the underlying implementation.
}

Unstable return type and implicit partial matching can lead to surprises and
bugs that are hard to catch. If you rely on code that requires the original
data frame behavior, coerce to a data frame via \code{\link[=as.data.frame]{as.data.frame()}}.
}
\details{
For better compatibility with older code written for regular data frames,
\code{[} supports a \code{drop} argument which defaults to \code{FALSE}.
New code should use \code{[[} to turn a column into a vector.
}
\examples{
df <- data.frame(a = 1:3, bc = 4:6)
tbl <- tibble(a = 1:3, bc = 4:6)

# Subsetting single columns:
df[, "a"]
tbl[, "a"]
tbl[, "a", drop = TRUE]
as.data.frame(tbl)[, "a"]

# Subsetting single rows with the drop argument:
df[1, , drop = TRUE]
tbl[1, , drop = TRUE]
as.list(tbl[1, ])

\dontshow{if ((Sys.getenv("NOT_CRAN") != "true" || Sys.getenv("IN_PKGDOWN") == "true")) (if (getRversion() >= "3.4") withAutoprint else force)(\{ # examplesIf}
# Accessing non-existent columns:
df$b
tbl$b

df[["b", exact = FALSE]]
tbl[["b", exact = FALSE]]

df$bd <- c("n", "e", "w")
tbl$bd <- c("n", "e", "w")
df$b
tbl$b
\dontshow{\}) # examplesIf}

df$b <- 7:9
tbl$b <- 7:9
df$b
tbl$b

# Identical behavior:
tbl[1, ]
tbl[1, c("bc", "a")]
tbl[, c("bc", "a")]
tbl[c("bc", "a")]
tbl["a"]
tbl$a
tbl[["a"]]
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sub.R, R/subassign.R
\name{subsetting}
\alias{subsetting}
\alias{$.tbl_df}
\alias{[[.tbl_df}
\alias{[.tbl_df}
\alias{$<-.tbl_df}
\alias{[[<-.tbl_df}
\alias{[<-.tbl_df}
\title{Subsetting tibbles}
\usage{
\method{$}{tbl_df}(x, name)

\method{[[}{tbl_df}(x, i, j, ..., exact = TRUE)

\method{[}{tbl_df}(x, i, j, drop = FALSE, ...)

\method{$}{tbl_df}(x, name) <- value

\method{[[}{tbl_df}(x, i, j, ...) <- value

\method{[}{tbl_df}(x, i, j, ...) <- value
}
\arguments{
\item{x}{A tibble.}

\item{name}{A \link{name} or a string.}

\item{i, j}{Row and column indices. If \code{j} is omitted, \code{i} is used as column index.}

\item{...}{Ignored.}

\item{exact}{Ignored, with a warning.}

\item{drop}{Coerce to a vector if fetching one column via \code{tbl[, j]} .
Default \code{FALSE}, ignored when accessing a column via \code{tbl[j]} .}

\item{value}{A value to store in a row, column, range or cell.
Tibbles are stricter than data frames in what is accepted here.}
}
\description{
Accessing columns, rows, or cells via \code{$}, \code{[[}, or \code{[} is mostly similar to
\link[base:Extract]{regular data frames}. However, the
behavior is different for tibbles and data frames in some cases:
\itemize{
\item \code{[} always returns a tibble by default, even if
only one column is accessed.
\item Partial matching of column names with \code{$} and \code{[[} is not supported, and
\code{NULL} is returned.
For \code{$}, a warning is given.
\item Only scalars (vectors of length one) or vectors with the
same length as the number of rows can be used for assignment.
\item Rows outside of the tibble's boundaries cannot be accessed.
\item When updating with \verb{[[<-} and \verb{[<-}, type changes of entire columns are
supported, but updating a part of a column requires that the new value is
coercible to the existing type.
See \code{\link[=vec_slice]{vec_slice()}} for the underlying implementation.
}

Unstable return type and implicit partial matching can lead to surprises and
bugs that are hard to catch. If you rely on code that requires the original
data frame behavior, coerce to a data frame via \code{\link[=as.data.frame]{as.data.frame()}}.
}
\details{
For better compatibility with older code written for regular data frames,
\code{[} supports a \code{drop} argument which defaults to \code{FALSE}.
New code should use \code{[[} to turn a column into a vector.
}
\examples{
df <- data.frame(a = 1:3, bc = 4:6)
tbl <- tibble(a = 1:3, bc = 4:6)

# Subsetting single columns:
df[, "a"]
tbl[, "a"]
tbl[, "a", drop = TRUE]
as.data.frame(tbl)[, "a"]

# Subsetting single rows with the drop argument:
df[1, , drop = TRUE]
tbl[1, , drop = TRUE]
as.list(tbl[1, ])

\dontshow{if ((Sys.getenv("NOT_CRAN") != "true" || Sys.getenv("IN_PKGDOWN") == "true")) (if (getRversion() >= "3.4") withAutoprint else force)(\{ # examplesIf}
# Accessing non-existent columns:
df$b
tbl$b

df[["b", exact = FALSE]]
tbl[["b", exact = FALSE]]

df$bd <- c("n", "e", "w")
tbl$bd <- c("n", "e", "w")
df$b
tbl$b
\dontshow{\}) # examplesIf}

df$b <- 7:9
tbl$b <- 7:9
df$b
tbl$b

# Identical behavior:
tbl[1, ]
tbl[1, c("bc", "a")]
tbl[, c("bc", "a")]
tbl[c("bc", "a")]
tbl["a"]
tbl$a
tbl[["a"]]
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/options.R
\docType{data}
\name{tibble_options}
\alias{tibble_options}
\title{Package options}
\usage{
tibble_options
}
\description{
Options that affect interactive display.
See \link[pillar:pillar_options]{pillar::pillar_options} for options that affect display on the console,
and \code{\link[cli:num_ansi_colors]{cli::num_ansi_colors()}} for enabling and disabling colored output
via ANSI sequences like \verb{[3m[38;5;246m[39m[23m}.
}
\details{
These options can be set via \code{\link[=options]{options()}} and queried via \code{\link[=getOption]{getOption()}}.
For this, add a \code{tibble.} prefix (the package name and a dot) to the option name.
Example: for an option \code{foo}, use \code{options(tibble.foo = value)} to set it
and \code{getOption("tibble.foo")} to retrieve the current value.
An option value of \code{NULL} means that the default is used.
}
\section{Options for the tibble package}{

\itemize{
\item \code{view_max}: Maximum number of rows shown by \code{\link[=view]{view()}}
if the input is not a data frame, passed on to \code{\link[=head]{head()}}. Default: \code{1000}.
}
}

\examples{
# Default setting:
getOption("tibble.view_max")

# Change for the duration of the session:
old <- options(tibble.view_max = 100)

# view() would show only 100 rows e.g. for a lazy data frame

# Change back to the original value:
options(old)

# Local scope:
local({
  rlang::local_options(tibble.view_max = 100)
  # view() would show only 100 rows e.g. for a lazy data frame
})
# view() would show the default 1000 rows e.g. for a lazy data frame
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tibble-package.R
\docType{package}
\name{tibble-package}
\alias{tibble-package}
\title{tibble: Simple Data Frames}
\description{
\if{html}{\figure{logo.png}{options: align='right' alt='logo' width='120'}}

Provides a 'tbl_df' class (the 'tibble') with stricter checking and better formatting than the traditional data frame.
}
\details{
\ifelse{html}{\href{https://lifecycle.r-lib.org/articles/stages.html#stable}{\figure{lifecycle-stable.svg}{options: alt='[Stable]'}}}{\strong{[Stable]}}

The tibble package provides utilities for handling \strong{tibbles}, where
"tibble" is a colloquial term for the S3 \code{\link{tbl_df}} class. The \code{\link{tbl_df}}
class is a special case of the base \code{\link[base:data.frame]{data.frame}}.
class, developed in response to lessons learned over many years of data
analysis with data frames.

Tibble is the central data structure for the set of packages known as the
\href{https://www.tidyverse.org/packages/}{tidyverse}, including
\href{https://dplyr.tidyverse.org/}{dplyr},
\href{https://ggplot2.tidyverse.org/}{ggplot2},
\href{https://tidyr.tidyverse.org/}{tidyr}, and
\href{https://readr.tidyverse.org/}{readr}.

General resources:
\itemize{
\item Website for the tibble package: \url{https://tibble.tidyverse.org}
\item \href{https://r4ds.had.co.nz/tibbles.html}{Tibbles chapter} in \emph{R for Data
Science}
}

Resources on specific topics:
\itemize{
\item Create a tibble: \code{\link[=tibble]{tibble()}}, \code{\link[=as_tibble]{as_tibble()}}, \code{\link[=tribble]{tribble()}}, \code{\link[=enframe]{enframe()}}
\item Inspect a tibble: \code{\link[=print.tbl]{print.tbl()}}, \code{\link[=glimpse]{glimpse()}}
\item Details on the S3 \code{tbl_df} class: \code{\linkS4class{tbl_df}}
\item Package options: \link{tibble_options}
}
}
\seealso{
Useful links:
\itemize{
  \item \url{https://tibble.tidyverse.org/}
  \item \url{https://github.com/tidyverse/tibble}
  \item Report bugs at \url{https://github.com/tidyverse/tibble/issues}
}

}
\author{
\strong{Maintainer}: Kirill Müller \email{krlmlr+r@mailbox.org}

Authors:
\itemize{
  \item Hadley Wickham \email{hadley@rstudio.com}
}

Other contributors:
\itemize{
  \item Romain Francois \email{romain@r-enthusiasts.com} [contributor]
  \item Jennifer Bryan \email{jenny@rstudio.com} [contributor]
  \item RStudio [copyright holder, funder]
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tibble-package.R
\docType{package}
\name{tibble-package}
\alias{tibble-package}
\title{tibble: Simple Data Frames}
\description{
\if{html}{\figure{logo.png}{options: style='float: right' alt='logo' width='120'}}

Provides a 'tbl_df' class (the 'tibble') with stricter checking and better formatting than the traditional data frame.
}
\details{
\ifelse{html}{\href{https://lifecycle.r-lib.org/articles/stages.html#stable}{\figure{lifecycle-stable.svg}{options: alt='[Stable]'}}}{\strong{[Stable]}}

The tibble package provides utilities for handling \strong{tibbles}, where
"tibble" is a colloquial term for the S3 \code{\link{tbl_df}} class. The \code{\link{tbl_df}}
class is a special case of the base \code{\link[base:data.frame]{data.frame}}
class, developed in response to lessons learned over many years of data
analysis with data frames.

Tibble is the central data structure for the set of packages known as the
\href{https://www.tidyverse.org/packages/}{tidyverse}, including
\href{https://dplyr.tidyverse.org/}{dplyr},
\href{https://ggplot2.tidyverse.org/}{ggplot2},
\href{https://tidyr.tidyverse.org/}{tidyr}, and
\href{https://readr.tidyverse.org/}{readr}.

General resources:
\itemize{
\item Website for the tibble package: \url{https://tibble.tidyverse.org}
\item \href{https://adv-r.hadley.nz/vectors-chap.html}{Vectors chapter} in \emph{Advanced R}
(2nd edition), specifically the
\href{https://adv-r.hadley.nz/vectors-chap.html#tibble}{Data frames and tibbles section}
}

Resources on specific topics:
\itemize{
\item Create a tibble: \code{\link[=tibble]{tibble()}}, \code{\link[=as_tibble]{as_tibble()}}, \code{\link[=tribble]{tribble()}}, \code{\link[=enframe]{enframe()}}
\item Inspect a tibble: \code{\link[=print.tbl]{print.tbl()}}, \code{\link[=glimpse]{glimpse()}}
\item Details on the S3 \code{tbl_df} class: \code{\linkS4class{tbl_df}}
\item Package options: \link{tibble_options}
}
}
\seealso{
Useful links:
\itemize{
  \item \url{https://tibble.tidyverse.org/}
  \item \url{https://github.com/tidyverse/tibble}
  \item Report bugs at \url{https://github.com/tidyverse/tibble/issues}
}

}
\author{
\strong{Maintainer}: Kirill Müller \email{kirill@cynkra.com} (\href{https://orcid.org/0000-0002-1416-3412}{ORCID})

Authors:
\itemize{
  \item Hadley Wickham \email{hadley@rstudio.com}
}

Other contributors:
\itemize{
  \item Romain Francois \email{romain@r-enthusiasts.com} [contributor]
  \item Jennifer Bryan \email{jenny@rstudio.com} [contributor]
  \item RStudio [copyright holder, funder]
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tibble.R
\name{tibble}
\alias{tibble}
\alias{tibble_row}
\title{Build a data frame}
\usage{
tibble(
  ...,
  .rows = NULL,
  .name_repair = c("check_unique", "unique", "universal", "minimal")
)

tibble_row(
  ...,
  .name_repair = c("check_unique", "unique", "universal", "minimal")
)
}
\arguments{
\item{...}{<\code{\link[rlang:dyn-dots]{dynamic-dots}}>
A set of name-value pairs. These arguments are
processed with \code{\link[rlang:defusing-advanced]{rlang::quos()}} and support unquote via \code{\link{!!}} and
unquote-splice via \code{\link{!!!}}. Use \verb{:=} to create columns that start with a dot.

Arguments are evaluated sequentially.
You can refer to previously created elements directly or using the \link{.data}
pronoun.
To refer explicitly to objects in the calling environment, use \code{\link{!!}} or
\link{.env}, e.g. \code{!!.data} or \code{.env$.data} for the special case of an object
named \code{.data}.}

\item{.rows}{The number of rows, useful to create a 0-column tibble or
just as an additional check.}

\item{.name_repair}{Treatment of problematic column names:
\itemize{
\item \code{"minimal"}: No name repair or checks, beyond basic existence,
\item \code{"unique"}: Make sure names are unique and not empty,
\item \code{"check_unique"}: (default value), no name repair, but check they are
\code{unique},
\item \code{"universal"}: Make the names \code{unique} and syntactic
\item a function: apply custom name repair (e.g., \code{.name_repair = make.names}
for names in the style of base R).
\item A purrr-style anonymous function, see \code{\link[rlang:as_function]{rlang::as_function()}}
}

This argument is passed on as \code{repair} to \code{\link[vctrs:vec_as_names]{vctrs::vec_as_names()}}.
See there for more details on these terms and the strategies used
to enforce them.}
}
\value{
A tibble, which is a colloquial term for an object of class
\code{\link[=tbl_df-class]{tbl_df}}. A \code{\link[=tbl_df-class]{tbl_df}} object is also a data
frame, i.e. it has class \code{data.frame}.
}
\description{
\code{tibble()} constructs a data frame. It is used like \code{\link[base:data.frame]{base::data.frame()}}, but
with a couple notable differences:
\itemize{
\item The returned data frame has the class \code{\link[=tbl_df-class]{tbl_df}}, in
addition to \code{data.frame}. This allows so-called "tibbles" to exhibit some
special behaviour, such as \link[=formatting]{enhanced printing}. Tibbles are
fully described in \code{\link[=tbl_df-class]{tbl_df}}.
\item \code{tibble()} is much lazier than \code{\link[base:data.frame]{base::data.frame()}} in terms of
transforming the user's input.
\itemize{
\item Character vectors are not coerced to factor.
\item List-columns are expressly anticipated and do not require special tricks.
\item Column names are not modified.
\item Inner names in columns are left unchanged.
}
\item \code{tibble()} builds columns sequentially. When defining a column, you can
refer to columns created earlier in the call. Only columns of length one
are recycled.
\item If a column evaluates to a data frame or tibble, it is nested or spliced.
If it evaluates to a matrix or a array, it remains a matrix or array,
respectively.
See examples.
}

\code{tibble_row()} constructs a data frame that is guaranteed to occupy one row.
Vector columns are required to have size one, non-vector columns are wrapped
in a list.
}
\examples{
# Unnamed arguments are named with their expression:
a <- 1:5
tibble(a, a * 2)

# Scalars (vectors of length one) are recycled:
tibble(a, b = a * 2, c = 1)

# Columns are available in subsequent expressions:
tibble(x = runif(10), y = x * 2)

# tibble() never coerces its inputs,
str(tibble(letters))
str(tibble(x = list(diag(1), diag(2))))

# or munges column names (unless requested),
tibble(`a + b` = 1:5)

# but it forces you to take charge of names, if they need repair:
try(tibble(x = 1, x = 2))
tibble(x = 1, x = 2, .name_repair = "unique")
tibble(x = 1, x = 2, .name_repair = "minimal")

## By default, non-syntactic names are allowed,
df <- tibble(`a 1` = 1, `a 2` = 2)
## because you can still index by name:
df[["a 1"]]
df$`a 1`
with(df, `a 1`)

## Syntactic names are easier to work with, though, and you can request them:
df <- tibble(`a 1` = 1, `a 2` = 2, .name_repair = "universal")
df$a.1

## You can specify your own name repair function:
tibble(x = 1, x = 2, .name_repair = make.unique)

fix_names <- function(x) gsub("\\\\s+", "_", x)
tibble(`year 1` = 1, `year 2` = 2, .name_repair = fix_names)

## purrr-style anonymous functions and constants
## are also supported
tibble(x = 1, x = 2, .name_repair = ~ make.names(., unique = TRUE))

tibble(x = 1, x = 2, .name_repair = ~ c("a", "b"))

# Tibbles can contain columns that are tibbles or matrices
# if the number of rows is compatible. Unnamed tibbled are
# spliced, i.e. the inner columns are inserted into the
# tibble under construction.
tibble(
  a = 1:3,
  tibble(
    b = 4:6,
    c = 7:9
  ),
  d = tibble(
    e = tibble(
      f = b
    )
  )
)
tibble(
  a = 1:3,
  b = diag(3),
  c = cor(trees),
  d = Titanic[1:3, , , ]
)

# Data can not contain tibbles or matrices with incompatible number of rows:
try(tibble(a = 1:3, b = tibble(c = 4:7)))

# Use := to create columns with names that start with a dot:
tibble(.dotted := 3)

# This also works, but might break in the future:
tibble(.dotted = 3)

# You can unquote an expression:
x <- 3
tibble(x = 1, y = x)
tibble(x = 1, y = !!x)

# You can splice-unquote a list of quosures and expressions:
tibble(!!!list(x = rlang::quo(1:10), y = quote(x * 2)))

# Use .data, .env and !! to refer explicitly to columns or outside objects
a <- 1
tibble(a = 2, b = a)
tibble(a = 2, b = .data$a)
tibble(a = 2, b = .env$a)
tibble(a = 2, b = !!a)
try(tibble(a = 2, b = .env$bogus))
try(tibble(a = 2, b = !!bogus))

# Use tibble_row() to construct a one-row tibble:
tibble_row(a = 1, lm = lm(Height ~ Girth + Volume, data = trees))
}
\seealso{
Use \code{\link[=as_tibble]{as_tibble()}} to turn an existing object into a tibble. Use
\code{enframe()} to convert a named vector into a tibble. Name repair is
detailed in \code{\link[vctrs:vec_as_names]{vctrs::vec_as_names()}}.
See \link{quasiquotation} for more details on tidy dots semantics,
i.e. exactly how  the \code{...} argument is processed.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pillar.R
\name{char}
\alias{char}
\alias{set_char_opts}
\title{Format a character vector}
\usage{
char(
  x,
  ...,
  min_chars = NULL,
  shorten = c("back", "front", "mid", "abbreviate")
)

set_char_opts(
  x,
  ...,
  min_chars = NULL,
  shorten = c("back", "front", "mid", "abbreviate")
)
}
\arguments{
\item{x}{A character vector.}

\item{...}{These dots are for future extensions and must be empty.}

\item{min_chars}{The minimum width to allocate to this column, defaults to 15.
The \code{"pillar.min_chars"} \link[pillar:pillar_options]{option} is not consulted.}

\item{shorten}{How to abbreviate the data if necessary:
\itemize{
\item \code{"back"} (default): add an ellipsis at the end
\item \code{"front"}: add an ellipsis at the front
\item \code{"mid"}: add an ellipsis in the middle
\item \code{"abbreviate"}: use \code{\link[=abbreviate]{abbreviate()}}
}}
}
\description{
\ifelse{html}{\href{https://lifecycle.r-lib.org/articles/stages.html#experimental}{\figure{lifecycle-experimental.svg}{options: alt='[Experimental]'}}}{\strong{[Experimental]}}

Constructs a character vector that can be formatted with predefined minimum width
or without width restrictions, and where the abbreviation style can be configured.

The formatting is applied when the vector is printed or formatted,
and also in a tibble column.

\code{set_char_opts()} adds formatting options to an arbitrary character vector,
useful for composing with other types.
}
\examples{
# Display as a vector:
char(letters[1:3])
\dontshow{if ({ set.seed(20210331); rlang::is_installed("stringi") }) (if (getRversion() >= "3.4") withAutoprint else force)(\{ # examplesIf}
# Space constraints:
rand_strings <- stringi::stri_rand_strings(10, seq(40, 22, by = -2))

# Plain character vectors get truncated if space is limited:
data_with_id <- function(id) {
  tibble(
    id,
    some_number_1 = 1, some_number_2 = 2, some_number_3 = 3,
    some_number_4 = 4, some_number_5 = 5, some_number_6 = 6,
    some_number_7 = 7, some_number_8 = 8, some_number_9 = 9
  )
}
data_with_id(rand_strings)

# Use char() to avoid or control truncation
data_with_id(char(rand_strings, min_chars = 24))
data_with_id(char(rand_strings, min_chars = Inf))
data_with_id(char(rand_strings, min_chars = 24, shorten = "mid"))

# Lorem Ipsum, one sentence per row.
lipsum <- unlist(strsplit(stringi::stri_rand_lipsum(1), "(?<=[.]) +", perl = TRUE))
tibble(
  back = char(lipsum, shorten = "back"),
  front = char(lipsum, shorten = "front"),
  mid = char(lipsum, shorten = "mid")
)
tibble(abbr = char(lipsum, shorten = "abbreviate"))
\dontshow{\}) # examplesIf}
}
\seealso{
Other vector classes: 
\code{\link{num}()}
}
\concept{vector classes}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/add.R
\name{add_row}
\alias{add_row}
\alias{add_case}
\title{Add rows to a data frame}
\usage{
add_row(.data, ..., .before = NULL, .after = NULL)
}
\arguments{
\item{.data}{Data frame to append to.}

\item{...}{<\code{\link[rlang:dyn-dots]{dynamic-dots}}>
Name-value pairs, passed on to \code{\link[=tibble]{tibble()}}. Values can be defined
only for columns that already exist in \code{.data} and unset columns will get an
\code{NA} value.}

\item{.before, .after}{One-based row index where to add the new rows,
default: after last row.}
}
\description{
This is a convenient way to add one or more rows of data to an existing data
frame. See \code{\link[=tribble]{tribble()}} for an easy way to create an complete
data frame row-by-row. Use \code{\link[=tibble_row]{tibble_row()}} to ensure that the new data
has only one row.

\code{add_case()} is an alias of \code{add_row()}.
}
\examples{
# add_row ---------------------------------
df <- tibble(x = 1:3, y = 3:1)

df \%>\% add_row(x = 4, y = 0)

# You can specify where to add the new rows
df \%>\% add_row(x = 4, y = 0, .before = 2)

# You can supply vectors, to add multiple rows (this isn't
# recommended because it's a bit hard to read)
df \%>\% add_row(x = 4:5, y = 0:-1)

# Use tibble_row() to add one row only
df \%>\% add_row(tibble_row(x = 4, y = 0))
try(df \%>\% add_row(tibble_row(x = 4:5, y = 0:-1)))

# Absent variables get missing values
df \%>\% add_row(x = 4)

# You can't create new variables
try(df \%>\% add_row(z = 10))
}
\seealso{
Other addition: 
\code{\link{add_column}()}
}
\concept{addition}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pillar.R
\name{num}
\alias{num}
\alias{set_num_opts}
\title{Format a numeric vector}
\usage{
num(
  x,
  ...,
  sigfig = NULL,
  digits = NULL,
  label = NULL,
  scale = NULL,
  notation = c("fit", "dec", "sci", "eng", "si"),
  fixed_exponent = NULL,
  extra_sigfig = NULL
)

set_num_opts(
  x,
  ...,
  sigfig = NULL,
  digits = NULL,
  label = NULL,
  scale = NULL,
  notation = c("fit", "dec", "sci", "eng", "si"),
  fixed_exponent = NULL,
  extra_sigfig = NULL
)
}
\arguments{
\item{x}{A numeric vector.}

\item{...}{These dots are for future extensions and must be empty.}

\item{sigfig}{Define the number of significant digits to show. Must be one or greater.
The \code{"pillar.sigfig"} \link[pillar:pillar_options]{option} is not consulted.
Can't be combined with \code{digits}.}

\item{digits}{Number of digits after the decimal points to show.
Positive numbers specify the exact number of digits to show.
Negative numbers specify (after negation) the maximum number of digits to show.
With \code{digits = 2}, the numbers 1.2 and 1.234 are printed as 1.20 and 1.23,
with \code{digits = -2} as 1.2 and 1.23, respectively.
Can't be combined with \code{sigfig}.}

\item{label}{A label to show instead of the type description.}

\item{scale}{Multiplier to apply to the data before showing.
Useful for displaying e.g. percentages.
Must be combined with \code{label}.}

\item{notation}{One of \code{"fit"}, \code{"dec"}, \code{"sci"}, \code{"eng"}, or \code{"si"}.
\itemize{
\item \code{"fit"}: Use decimal notation if it fits and if it consumes 13 digits or less,
otherwise use scientific notation. (The default for numeric pillars.)
\item \code{"dec"}: Use decimal notation, regardless of width.
\item \code{"sci"}: Use scientific notation.
\item \code{"eng"}: Use engineering notation, i.e. scientific notation
using exponents that are a multiple of three.
\item \code{"si"}: Use SI notation, prefixes between \code{1e-24} and \code{1e24} are supported.
}}

\item{fixed_exponent}{Use the same exponent for all numbers in scientific, engineering or SI notation.
\code{-Inf} uses the smallest, \code{+Inf} the largest fixed_exponent present in the data.
The default is to use varying exponents.}

\item{extra_sigfig}{If \code{TRUE}, increase the number of significant digits if the data consists of
numbers of the same magnitude with subtle differences.}
}
\description{
\ifelse{html}{\href{https://lifecycle.r-lib.org/articles/stages.html#experimental}{\figure{lifecycle-experimental.svg}{options: alt='[Experimental]'}}}{\strong{[Experimental]}}

Constructs a numeric vector that can be formatted with predefined
significant digits, or with a maximum or fixed number of digits
after the decimal point.
Scaling is supported, as well as forcing a decimal, scientific
or engineering notation.
If a label is given, it is shown in the header of a column.

The formatting is applied when the vector is printed or formatted,
and also in a tibble column.
The formatting annotation and the class survives most arithmetic transformations,
the most notable exceptions are \code{\link[=var]{var()}} and \code{\link[=sd]{sd()}}.

\code{set_num_opts()} adds formatting options to an arbitrary numeric vector,
useful for composing with other types.
}
\examples{
# Display as a vector
num(9:11 * 100 + 0.5)

# Significant figures
tibble(
  x3 = num(9:11 * 100 + 0.5, sigfig = 3),
  x4 = num(9:11 * 100 + 0.5, sigfig = 4),
  x5 = num(9:11 * 100 + 0.5, sigfig = 5),
)

# Maximum digits after the decimal points
tibble(
  x0 = num(9:11 * 100 + 0.5, digits = 0),
  x1 = num(9:11 * 100 + 0.5, digits = -1),
  x2 = num(9:11 * 100 + 0.5, digits = -2),
)

# Use fixed digits and a currency label
tibble(
  usd = num(9:11 * 100 + 0.5, digits = 2, label = "USD"),
  gbp = num(9:11 * 100 + 0.5, digits = 2, label = "£"),
  chf = num(9:11 * 100 + 0.5, digits = 2, label = "SFr")
)

# Scale
tibble(
  small  = num(9:11 / 1000 + 0.00005, label = "\%", scale = 100),
  medium = num(9:11 / 100 + 0.0005, label = "\%", scale = 100),
  large  = num(9:11 / 10 + 0.005, label = "\%", scale = 100)
)

# Notation
tibble(
  sci = num(10^(-13:6), notation = "sci"),
  eng = num(10^(-13:6), notation = "eng"),
  si  = num(10^(-13:6), notation = "si"),
  dec = num(10^(-13:6), notation = "dec")
)

# Fixed exponent
tibble(
  scimin = num(10^(-7:6) * 123, notation = "sci", fixed_exponent = -Inf),
  engmin = num(10^(-7:6) * 123, notation = "eng", fixed_exponent = -Inf),
  simin  = num(10^(-7:6) * 123, notation = "si", fixed_exponent = -Inf)
)

tibble(
  scismall = num(10^(-7:6) * 123, notation = "sci", fixed_exponent = -3),
  scilarge = num(10^(-7:6) * 123, notation = "sci", fixed_exponent = 3),
  scimax   = num(10^(-7:6) * 123, notation = "sci", fixed_exponent = Inf)
)

#' Extra significant digits
tibble(
  default = num(100 + 1:3 * 0.001),
  extra1 = num(100 + 1:3 * 0.001, extra_sigfig = TRUE),
  extra2 = num(100 + 1:3 * 0.0001, extra_sigfig = TRUE),
  extra3 = num(10000 + 1:3 * 0.00001, extra_sigfig = TRUE)
)
}
\seealso{
Other vector classes: 
\code{\link{char}()}
}
\concept{vector classes}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/enframe.R
\name{enframe}
\alias{enframe}
\alias{deframe}
\title{Converting vectors to data frames, and vice versa}
\usage{
enframe(x, name = "name", value = "value")

deframe(x)
}
\arguments{
\item{x}{A vector (for \code{enframe()}) or a data frame with one or two columns
(for \code{deframe()}).}

\item{name, value}{Names of the columns that store the names and values.
If \code{name} is \code{NULL}, a one-column tibble is returned; \code{value} cannot be \code{NULL}.}
}
\value{
For \code{enframe()}, a \link{tibble} with two columns (if \code{name} is not \code{NULL}, the default)
or one column (otherwise).

For \code{deframe()}, a vector (named or unnamed).
}
\description{
\code{enframe()} converts named atomic vectors or lists to one- or two-column
data frames.
For a list, the result will be a nested tibble with a column of type \code{list}.
For unnamed vectors, the natural sequence is used as name column.

\code{deframe()} converts two-column data frames to a named vector or list,
using the first column as name and the second column as value.
If the input has only one column, an unnamed vector is returned.
}
\examples{
enframe(1:3)
enframe(c(a = 5, b = 7))
enframe(list(one = 1, two = 2:3, three = 4:6))
deframe(enframe(3:1))
deframe(tibble(a = 1:3))
deframe(tibble(a = as.list(1:3)))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lst.R
\name{lst}
\alias{lst}
\title{Build a list}
\usage{
lst(...)
}
\arguments{
\item{...}{<\code{\link[rlang:dyn-dots]{dynamic-dots}}>
A set of name-value pairs. These arguments are
processed with \code{\link[rlang:defusing-advanced]{rlang::quos()}} and support unquote via \code{\link{!!}} and
unquote-splice via \code{\link{!!!}}. Use \verb{:=} to create columns that start with a dot.

Arguments are evaluated sequentially.
You can refer to previously created elements directly or using the \link{.data}
pronoun.
To refer explicitly to objects in the calling environment, use \code{\link{!!}} or
\link{.env}, e.g. \code{!!.data} or \code{.env$.data} for the special case of an object
named \code{.data}.}
}
\value{
A named list.
}
\description{
\code{lst()} constructs a list, similar to \code{\link[base:list]{base::list()}}, but with some of the
same features as \code{\link[=tibble]{tibble()}}. \code{lst()} builds components sequentially. When
defining a component, you can refer to components created earlier in the
call. \code{lst()} also generates missing names automatically.

See \code{\link[rlang:list2]{rlang::list2()}} for a simpler and faster alternative without tibble's
evaluation and auto-name semantics.
}
\examples{
# the value of n can be used immediately in the definition of x
lst(n = 5, x = runif(n))

# missing names are constructed from user's input
lst(1:3, z = letters[4:6], runif(3))

a <- 1:3
b <- letters[4:6]
lst(a, b)

# pre-formed quoted expressions can be used with lst() and then
# unquoted (with !!) or unquoted and spliced (with !!!)
n1 <- 2
n2 <- 3
n_stuff <- quote(n1 + n2)
x_stuff <- quote(seq_len(n))
lst(!!!list(n = n_stuff, x = x_stuff))
lst(n = !!n_stuff, x = !!x_stuff)
lst(n = 4, x = !!x_stuff)
lst(!!!list(n = 2, x = x_stuff))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tribble.R
\name{tribble}
\alias{tribble}
\title{Row-wise tibble creation}
\usage{
tribble(...)
}
\arguments{
\item{...}{<\code{\link[rlang:dyn-dots]{dynamic-dots}}>
Arguments specifying the structure of a \code{tibble}.
Variable names should be formulas, and may only appear before the data.
These arguments are processed with \code{\link[rlang:list2]{rlang::list2()}}
and support unquote via \code{\link{!!}} and unquote-splice via \code{\link{!!!}}.}
}
\value{
A \link{tibble}.
}
\description{
\ifelse{html}{\out{<a href='https://lifecycle.r-lib.org/articles/stages.html'><img src='figures/lifecycle-maturing.svg' alt='Maturing lifecycle'></a>}}{\strong{Maturing}}

Create \link{tibble}s using an easier to read row-by-row layout.
This is useful for small tables of data where readability is
important.  Please see \link{tibble-package} for a general introduction.
}
\examples{
tribble(
  ~colA, ~colB,
  "a",   1,
  "b",   2,
  "c",   3
)

# tribble will create a list column if the value in any cell is
# not a scalar
tribble(
  ~x,  ~y,
  "a", 1:3,
  "b", 4:6
)
}
\seealso{
See \link{quasiquotation} for more details on tidy dots semantics,
i.e. exactly how  the \code{...} argument is processed.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/deprecated.R
\name{name-repair}
\alias{name-repair}
\title{Name repair}
\description{
Please review \code{\link[vctrs:vec_as_names]{vctrs::vec_as_names()}}.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tibble.R
\name{is_tibble}
\alias{is_tibble}
\title{Test if the object is a tibble}
\usage{
is_tibble(x)
}
\arguments{
\item{x}{An object}
}
\value{
\code{TRUE} if the object inherits from the \code{tbl_df} class.
}
\description{
This function returns \code{TRUE} for tibbles or subclasses thereof,
and \code{FALSE} for all other objects, including regular data frames.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tribble.R
\name{tribble}
\alias{tribble}
\title{Row-wise tibble creation}
\usage{
tribble(...)
}
\arguments{
\item{...}{<\code{\link[rlang:dyn-dots]{dynamic-dots}}>
Arguments specifying the structure of a \code{tibble}.
Variable names should be formulas, and may only appear before the data.
These arguments are processed with \code{\link[rlang:list2]{rlang::list2()}}
and support unquote via \code{\link{!!}} and unquote-splice via \code{\link{!!!}}.}
}
\value{
A \link{tibble}.
}
\description{
Create \link{tibble}s using an easier to read row-by-row layout.
This is useful for small tables of data where readability is
important.  Please see \link{tibble-package} for a general introduction.
}
\examples{
tribble(
  ~colA, ~colB,
  "a",   1,
  "b",   2,
  "c",   3
)

# tribble will create a list column if the value in any cell is
# not a scalar
tribble(
  ~x,  ~y,
  "a", 1:3,
  "b", 4:6
)
\dontshow{if (rlang::is_installed("dplyr") && packageVersion("dplyr") >= "1.0.5") (if (getRversion() >= "3.4") withAutoprint else force)(\{ # examplesIf}

# Use dplyr::mutate(dplyr::across(...)) to assign an explicit type
tribble(
  ~a, ~b, ~c,
  1, "2000-01-01", "1.5"
) \%>\%
  dplyr::mutate(
    dplyr::across(a, as.integer),
    dplyr::across(b, as.Date)
  )
\dontshow{\}) # examplesIf}
}
\seealso{
See \link{quasiquotation} for more details on tidy dots semantics,
i.e. exactly how  the \code{...} argument is processed.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tidy_names.R
\name{name-repair-superseded}
\alias{name-repair-superseded}
\alias{tidy_names}
\alias{name-repair-retired}
\alias{set_tidy_names}
\alias{repair_names}
\title{Superseded functions for name repair}
\usage{
tidy_names(name, syntactic = FALSE, quiet = FALSE)

set_tidy_names(x, syntactic = FALSE, quiet = FALSE)

repair_names(x, prefix = "V", sep = "")
}
\arguments{
\item{name}{A \code{names} attribute, usually a character vector.}

\item{syntactic}{Should names be made syntactically valid? If \code{FALSE}, uses
same logic as \code{.name_repair = "unique"}. If \code{TRUE}, uses same logic as
\code{.name_repair = "universal"}.}

\item{quiet}{Whether to suppress messages about name repair.}

\item{x}{A vector.}

\item{prefix}{A string, the prefix to use for new column names.}

\item{sep}{A string inserted between the column name and de-duplicating
number.}
}
\value{
\code{x} with repaired names or a repaired version of \code{name}.
}
\description{
\ifelse{html}{\href{https://lifecycle.r-lib.org/articles/stages.html#superseded}{\figure{lifecycle-superseded.svg}{options: alt='[Superseded]'}}}{\strong{[Superseded]}}

\code{tidy_names()}, \code{set_tidy_names()}, and \code{repair_names()} were early efforts
to facilitate \emph{post hoc} name repair in tibble, given that \code{\link[=tibble]{tibble()}} and
\code{\link[=as_tibble]{as_tibble()}} did not do this.

From tibble v2.0.0, the \code{.name_repair} argument gives direct access to three
specific levels of name repair: \code{minimal}, \code{unique}, and \code{universal}.
See \code{\link[vctrs:vec_as_names]{vctrs::vec_as_names()}} for the implementation of the underlying logic.
}
\section{Life cycle}{


These functions are superseded. The \code{repair_names()} logic
will also remain available in \code{\link[vctrs:vec_as_names_legacy]{vctrs::vec_as_names_legacy()}}.

\if{html}{\out{<div class="sourceCode">}}\preformatted{tibble(..., `.name_repair = "unique"`)
## is preferred to
df <- tibble(...)
set_tidy_names(df, syntactic = FALSE)

tibble(..., `.name_repair = "universal"`)
## is preferred to
df <- tibble(...)
set_tidy_names(df, syntactic = TRUE)
}\if{html}{\out{</div>}}
}

\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tidy_names.R
\name{name-repair-superseded}
\alias{name-repair-superseded}
\alias{tidy_names}
\alias{name-repair-retired}
\alias{set_tidy_names}
\alias{repair_names}
\title{Superseded functions for name repair}
\usage{
tidy_names(name, syntactic = FALSE, quiet = FALSE)

set_tidy_names(x, syntactic = FALSE, quiet = FALSE)

repair_names(x, prefix = "V", sep = "")
}
\arguments{
\item{name}{A \code{names} attribute, usually a character vector.}

\item{syntactic}{Should names be made syntactically valid? If \code{FALSE}, uses
same logic as \code{.name_repair = "unique"}. If \code{TRUE}, uses same logic as
\code{.name_repair = "universal"}.}

\item{quiet}{Whether to suppress messages about name repair.}

\item{x}{A vector.}

\item{prefix}{A string, the prefix to use for new column names.}

\item{sep}{A string inserted between the column name and de-duplicating
number.}
}
\value{
\code{x} with repaired names or a repaired version of \code{name}.
}
\description{
\ifelse{html}{\href{https://lifecycle.r-lib.org/articles/stages.html#superseded}{\figure{lifecycle-superseded.svg}{options: alt='[Superseded]'}}}{\strong{[Superseded]}}

\code{tidy_names()}, \code{set_tidy_names()}, and \code{repair_names()} were early efforts
to facilitate \emph{post hoc} name repair in tibble, given that \code{\link[=tibble]{tibble()}} and
\code{\link[=as_tibble]{as_tibble()}} did not do this.

From tibble v2.0.0, the \code{.name_repair} argument gives direct access to three
specific levels of name repair: \code{minimal}, \code{unique}, and \code{universal}.
See \code{\link[vctrs:vec_as_names]{vctrs::vec_as_names()}} for the implementation of the underlying logic.
}
\section{Life cycle}{


These functions are superseded. The \code{repair_names()} logic
will also remain available in \code{\link[vctrs:vec_as_names_legacy]{vctrs::vec_as_names_legacy()}}.\preformatted{tibble(..., `.name_repair = "unique"`)
## is preferred to
df <- tibble(...)
set_tidy_names(df, syntactic = FALSE)

tibble(..., `.name_repair = "universal"`)
## is preferred to
df <- tibble(...)
set_tidy_names(df, syntactic = TRUE)
}
}

\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/as_tibble.R
\name{as_tibble}
\alias{as_tibble}
\alias{as_tibble.data.frame}
\alias{as_tibble.list}
\alias{as_tibble.matrix}
\alias{as_tibble.table}
\alias{as_tibble.NULL}
\alias{as_tibble.default}
\alias{as_tibble_row}
\alias{as_tibble_col}
\title{Coerce lists, matrices, and more to data frames}
\usage{
as_tibble(
  x,
  ...,
  .rows = NULL,
  .name_repair = c("check_unique", "unique", "universal", "minimal"),
  rownames = pkgconfig::get_config("tibble::rownames", NULL)
)

\method{as_tibble}{data.frame}(
  x,
  validate = NULL,
  ...,
  .rows = NULL,
  .name_repair = c("check_unique", "unique", "universal", "minimal"),
  rownames = pkgconfig::get_config("tibble::rownames", NULL)
)

\method{as_tibble}{list}(
  x,
  validate = NULL,
  ...,
  .rows = NULL,
  .name_repair = c("check_unique", "unique", "universal", "minimal")
)

\method{as_tibble}{matrix}(x, ..., validate = NULL, .name_repair = NULL)

\method{as_tibble}{table}(x, `_n` = "n", ..., n = `_n`, .name_repair = "check_unique")

\method{as_tibble}{NULL}(x, ...)

\method{as_tibble}{default}(x, ...)

as_tibble_row(
  x,
  .name_repair = c("check_unique", "unique", "universal", "minimal")
)

as_tibble_col(x, column_name = "value")
}
\arguments{
\item{x}{A data frame, list, matrix, or other object that could reasonably be
coerced to a tibble.}

\item{...}{Unused, for extensibility.}

\item{.rows}{The number of rows, useful to create a 0-column tibble or
just as an additional check.}

\item{.name_repair}{Treatment of problematic column names:
\itemize{
\item \code{"minimal"}: No name repair or checks, beyond basic existence,
\item \code{"unique"}: Make sure names are unique and not empty,
\item \code{"check_unique"}: (default value), no name repair, but check they are
\code{unique},
\item \code{"universal"}: Make the names \code{unique} and syntactic
\item a function: apply custom name repair (e.g., \code{.name_repair = make.names}
for names in the style of base R).
\item A purrr-style anonymous function, see \code{\link[rlang:as_function]{rlang::as_function()}}
}

This argument is passed on as \code{repair} to \code{\link[vctrs:vec_as_names]{vctrs::vec_as_names()}}.
See there for more details on these terms and the strategies used
to enforce them.}

\item{rownames}{How to treat existing row names of a data frame or matrix:
\itemize{
\item \code{NULL}: remove row names. This is the default.
\item \code{NA}: keep row names.
\item A string: the name of a new column. Existing rownames are transferred
into this column and the \code{row.names} attribute is deleted.
No name repair is applied to the new column name, even if \code{x} already contains
a column of that name.
Use \code{as_tibble(rownames_to_column(...))} to safeguard against this case.
}

Read more in \link{rownames}.}

\item{_n, validate}{\ifelse{html}{\href{https://lifecycle.r-lib.org/articles/stages.html#soft-deprecated}{\figure{lifecycle-soft-deprecated.svg}{options: alt='[Soft-deprecated]'}}}{\strong{[Soft-deprecated]}}

For compatibility only, do not use for new code.}

\item{n}{Name for count column, default: \code{"n"}.}

\item{column_name}{Name of the column.}
}
\description{
\code{as_tibble()} turns an existing object, such as a data frame or
matrix, into a so-called tibble, a data frame with class \code{\link{tbl_df}}. This is
in contrast with \code{\link[=tibble]{tibble()}}, which builds a tibble from individual columns.
\code{as_tibble()} is to \code{\link[=tibble]{tibble()}} as \code{\link[base:as.data.frame]{base::as.data.frame()}} is to
\code{\link[base:data.frame]{base::data.frame()}}.

\code{as_tibble()} is an S3 generic, with methods for:
\itemize{
\item \code{\link[base:data.frame]{data.frame}}: Thin wrapper around the \code{list} method
that implements tibble's treatment of \link{rownames}.
\item \code{\link[base:matrix]{matrix}}, \code{\link[stats:poly]{poly}},
\code{\link[stats:ts]{ts}}, \code{\link[base:table]{table}}
\item Default: Other inputs are first coerced with \code{\link[base:as.data.frame]{base::as.data.frame()}}.
}

\code{as_tibble_row()} converts a vector to a tibble with one row.
If the input is a list, all elements must have size one.

\code{as_tibble_col()} converts a vector to a tibble with one column.
}
\section{Row names}{

The default behavior is to silently remove row names.

New code should explicitly convert row names to a new column using the
\code{rownames} argument.

For existing code that relies on the retention of row names, call
\code{pkgconfig::set_config("tibble::rownames" = NA)} in your script or in your
package's \code{\link[=.onLoad]{.onLoad()}}  function.
}

\section{Life cycle}{

Using \code{as_tibble()} for vectors is superseded as of version 3.0.0,
prefer the more expressive \code{as_tibble_row()} and
\code{as_tibble_col()} variants for new code.
}

\examples{
m <- matrix(rnorm(50), ncol = 5)
colnames(m) <- c("a", "b", "c", "d", "e")
df <- as_tibble(m)

as_tibble_row(c(a = 1, b = 2))
as_tibble_row(list(c = "three", d = list(4:5)))
as_tibble_row(1:3, .name_repair = "unique")

as_tibble_col(1:3)
as_tibble_col(
  list(c = "three", d = list(4:5)),
  column_name = "data"
)
}
\seealso{
\code{\link[=tibble]{tibble()}} constructs a tibble from individual columns. \code{\link[=enframe]{enframe()}}
converts a named vector to a tibble with a column of names and column of
values. Name repair is implemented using \code{\link[vctrs:vec_as_names]{vctrs::vec_as_names()}}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/as_tibble.R
\name{as_tibble}
\alias{as_tibble}
\alias{as_tibble.data.frame}
\alias{as_tibble.list}
\alias{as_tibble.matrix}
\alias{as_tibble.table}
\alias{as_tibble.NULL}
\alias{as_tibble.default}
\alias{as_tibble_row}
\alias{as_tibble_col}
\title{Coerce lists, matrices, and more to data frames}
\usage{
as_tibble(
  x,
  ...,
  .rows = NULL,
  .name_repair = c("check_unique", "unique", "universal", "minimal"),
  rownames = pkgconfig::get_config("tibble::rownames", NULL)
)

\method{as_tibble}{data.frame}(
  x,
  validate = NULL,
  ...,
  .rows = NULL,
  .name_repair = c("check_unique", "unique", "universal", "minimal"),
  rownames = pkgconfig::get_config("tibble::rownames", NULL)
)

\method{as_tibble}{list}(
  x,
  validate = NULL,
  ...,
  .rows = NULL,
  .name_repair = c("check_unique", "unique", "universal", "minimal")
)

\method{as_tibble}{matrix}(x, ..., validate = NULL, .name_repair = NULL)

\method{as_tibble}{table}(x, `_n` = "n", ..., n = `_n`, .name_repair = "check_unique")

\method{as_tibble}{`NULL`}(x, ...)

\method{as_tibble}{default}(x, ...)

as_tibble_row(
  x,
  .name_repair = c("check_unique", "unique", "universal", "minimal")
)

as_tibble_col(x, column_name = "value")
}
\arguments{
\item{x}{A data frame, list, matrix, or other object that could reasonably be
coerced to a tibble.}

\item{...}{Unused, for extensibility.}

\item{.rows}{The number of rows, useful to create a 0-column tibble or
just as an additional check.}

\item{.name_repair}{Treatment of problematic column names:
\itemize{
\item \code{"minimal"}: No name repair or checks, beyond basic existence,
\item \code{"unique"}: Make sure names are unique and not empty,
\item \code{"check_unique"}: (default value), no name repair, but check they are
\code{unique},
\item \code{"universal"}: Make the names \code{unique} and syntactic
\item a function: apply custom name repair (e.g., \code{.name_repair = make.names}
for names in the style of base R).
\item A purrr-style anonymous function, see \code{\link[rlang:as_function]{rlang::as_function()}}
}

This argument is passed on as \code{repair} to \code{\link[vctrs:vec_as_names]{vctrs::vec_as_names()}}.
See there for more details on these terms and the strategies used
to enforce them.}

\item{rownames}{How to treat existing row names of a data frame or matrix:
\itemize{
\item \code{NULL}: remove row names. This is the default.
\item \code{NA}: keep row names.
\item A string: the name of a new column. Existing rownames are transferred
into this column and the \code{row.names} attribute is deleted.
Read more in \link{rownames}.
}}

\item{_n, validate}{\ifelse{html}{\href{https://lifecycle.r-lib.org/articles/stages.html#soft-deprecated}{\figure{lifecycle-soft-deprecated.svg}{options: alt='[Soft-deprecated]'}}}{\strong{[Soft-deprecated]}}

For compatibility only, do not use for new code.}

\item{n}{Name for count column, default: \code{"n"}.}

\item{column_name}{Name of the column.}
}
\description{
\code{as_tibble()} turns an existing object, such as a data frame or
matrix, into a so-called tibble, a data frame with class \code{\link{tbl_df}}. This is
in contrast with \code{\link[=tibble]{tibble()}}, which builds a tibble from individual columns.
\code{as_tibble()} is to \code{\link[=tibble]{tibble()}} as \code{\link[base:as.data.frame]{base::as.data.frame()}} is to
\code{\link[base:data.frame]{base::data.frame()}}.

\code{as_tibble()} is an S3 generic, with methods for:
\itemize{
\item \code{\link[base:data.frame]{data.frame}}: Thin wrapper around the \code{list} method
that implements tibble's treatment of \link{rownames}.
\item \code{\link[base:matrix]{matrix}}, \code{\link[stats:poly]{poly}},
\code{\link[stats:ts]{ts}}, \code{\link[base:table]{table}}
\item Default: Other inputs are first coerced with \code{\link[base:as.data.frame]{base::as.data.frame()}}.
}

\code{as_tibble_row()} converts a vector to a tibble with one row.
If the input is a list, all elements must have size one.

\code{as_tibble_col()} converts a vector to a tibble with one column.
}
\section{Row names}{

The default behavior is to silently remove row names.

New code should explicitly convert row names to a new column using the
\code{rownames} argument.

For existing code that relies on the retention of row names, call
\code{pkgconfig::set_config("tibble::rownames" = NA)} in your script or in your
package's \code{\link[=.onLoad]{.onLoad()}}  function.
}

\section{Life cycle}{

Using \code{as_tibble()} for vectors is superseded as of version 3.0.0,
prefer the more expressive \code{as_tibble_row()} and
\code{as_tibble_col()} variants for new code.
}

\examples{
m <- matrix(rnorm(50), ncol = 5)
colnames(m) <- c("a", "b", "c", "d", "e")
df <- as_tibble(m)

as_tibble_row(c(a = 1, b = 2))
as_tibble_row(list(c = "three", d = list(4:5)))
as_tibble_row(1:3, .name_repair = "unique")

as_tibble_col(1:3)
as_tibble_col(
  list(c = "three", d = list(4:5)),
  column_name = "data"
)
}
\seealso{
\code{\link[=tibble]{tibble()}} constructs a tibble from individual columns. \code{\link[=enframe]{enframe()}}
converts a named vector to a tibble with a column of names and column of
values. Name repair is implemented using \code{\link[vctrs:vec_as_names]{vctrs::vec_as_names()}}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/add.R
\name{add_column}
\alias{add_column}
\title{Add columns to a data frame}
\usage{
add_column(
  .data,
  ...,
  .before = NULL,
  .after = NULL,
  .name_repair = c("check_unique", "unique", "universal", "minimal")
)
}
\arguments{
\item{.data}{Data frame to append to.}

\item{...}{<\code{\link[rlang:dyn-dots]{dynamic-dots}}>
Name-value pairs, passed on to \code{\link[=tibble]{tibble()}}. All values must have
the same size of \code{.data} or size 1.}

\item{.before, .after}{One-based column index or column name where to add the
new columns, default: after last column.}

\item{.name_repair}{Treatment of problematic column names:
\itemize{
\item \code{"minimal"}: No name repair or checks, beyond basic existence,
\item \code{"unique"}: Make sure names are unique and not empty,
\item \code{"check_unique"}: (default value), no name repair, but check they are
\code{unique},
\item \code{"universal"}: Make the names \code{unique} and syntactic
\item a function: apply custom name repair (e.g., \code{.name_repair = make.names}
for names in the style of base R).
\item A purrr-style anonymous function, see \code{\link[rlang:as_function]{rlang::as_function()}}
}

This argument is passed on as \code{repair} to \code{\link[vctrs:vec_as_names]{vctrs::vec_as_names()}}.
See there for more details on these terms and the strategies used
to enforce them.}
}
\description{
This is a convenient way to add one or more columns to an existing data
frame.
}
\examples{
# add_column ---------------------------------
df <- tibble(x = 1:3, y = 3:1)

df \%>\% add_column(z = -1:1, w = 0)
df \%>\% add_column(z = -1:1, .before = "y")

# You can't overwrite existing columns
try(df \%>\% add_column(x = 4:6))

# You can't create new observations
try(df \%>\% add_column(z = 1:5))

}
\seealso{
Other addition: 
\code{\link{add_row}()}
}
\concept{addition}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/view.R
\name{view}
\alias{view}
\title{View an object}
\usage{
view(x, title = NULL, ..., n = NULL)
}
\arguments{
\item{x}{The object to display.}

\item{title}{The title to use for the display, by default
the deparsed expression is used.}

\item{...}{Unused, must be empty.}

\item{n}{Maximum number of rows to display. Only used if \code{x} is not a
data frame. Uses the \code{view_max} \link[=tibble_options]{option} by default.}
}
\description{
\ifelse{html}{\href{https://lifecycle.r-lib.org/articles/stages.html#experimental}{\figure{lifecycle-experimental.svg}{options: alt='[Experimental]'}}}{\strong{[Experimental]}}

Calls \code{\link[utils:View]{utils::View()}} on the input and returns it, invisibly.
If the input is not a data frame, it is processed using a variant of
\code{as.data.frame(head(x, n))}.
A message is printed if the number of rows exceeds \code{n}.
This function has no effect in non\link{interactive} sessions.
}
\details{
The RStudio IDE overrides \code{utils::View()}, this is picked up
correctly.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/print.R
\name{knit_print.trunc_mat}
\alias{knit_print.trunc_mat}
\title{knit_print method for trunc mat}
\usage{
knit_print.trunc_mat(x, options)
}
\description{
knit_print method for trunc mat
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tribble.R
\name{frame_matrix}
\alias{frame_matrix}
\title{Row-wise matrix creation}
\usage{
frame_matrix(...)
}
\arguments{
\item{...}{<\code{\link[rlang:dyn-dots]{dynamic-dots}}>
Arguments specifying the structure of a \code{frame_matrix}.
Column names should be formulas, and may only appear before the data.
These arguments are processed with \code{\link[rlang:list2]{rlang::list2()}}
and support unquote via \code{\link{!!}} and unquote-splice via \code{\link{!!!}}.}
}
\value{
A \link{matrix}.
}
\description{
Create matrices laying out the data in rows, similar to
\code{matrix(..., byrow = TRUE)}, with a nicer-to-read syntax.
This is useful for small matrices, e.g. covariance matrices, where readability
is important. The syntax is inspired by \code{\link[=tribble]{tribble()}}.
}
\examples{
frame_matrix(
  ~col1, ~col2,
  1,     3,
  5,     2
)
}
\seealso{
See \link{quasiquotation} for more details on tidy dots semantics,
i.e. exactly how  the \code{...} argument is processed.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/print.R
\name{formatting}
\alias{formatting}
\alias{print.tbl}
\alias{format.tbl}
\alias{print.tbl_df}
\alias{format.tbl_df}
\alias{trunc_mat}
\title{Printing tibbles}
\usage{
\method{print}{tbl_df}(x, ..., n = NULL, width = NULL, n_extra = NULL)

\method{format}{tbl_df}(x, ..., n = NULL, width = NULL, n_extra = NULL)

trunc_mat(x, n = NULL, width = NULL, n_extra = NULL)
}
\arguments{
\item{x}{Object to format or print.}

\item{...}{Other arguments passed on to individual methods.}

\item{n}{Number of rows to show. If \code{NULL}, the default, will print all rows
if less than option \code{tibble.print_max}. Otherwise, will print
\code{tibble.print_min} rows.}

\item{width}{Width of text output to generate. This defaults to \code{NULL}, which
means use \code{getOption("tibble.width")} or (if also \code{NULL})
\code{getOption("width")}; the latter displays only the columns that fit on one
screen. You can also set \code{options(tibble.width = Inf)} to override this
default and always print all columns.}

\item{n_extra}{Number of extra columns to print abbreviated information for,
if the width is too small for the entire tibble. If \code{NULL}, the default,
will print information about at most \code{tibble.max_extra_cols} extra columns.}
}
\description{
\ifelse{html}{\out{<a href='https://lifecycle.r-lib.org/articles/stages.html'><img src='figures/lifecycle-maturing.svg' alt='Maturing lifecycle'></a>}}{\strong{Maturing}}

One of the main features of the \code{tbl_df} class is the printing:
\itemize{
\item Tibbles only print as many rows and columns as fit on one screen,
supplemented by a summary of the remaining rows and columns.
\item Tibble reveals the type of each column, which keeps the user informed about
whether a variable is, e.g., \verb{<chr>} or \verb{<fct>} (character versus factor).
}

Printing can be tweaked for a one-off call by calling \code{print()} explicitly
and setting arguments like \code{n} and \code{width}. More persistent control is
available by setting the options described below.
}
\section{Package options}{


The following options are used by the tibble and pillar packages
to format and print \code{tbl_df} objects.
Used by the formatting workhorse \code{trunc_mat()} and, therefore,
indirectly, by \code{print.tbl()}.
\itemize{
\item \code{tibble.print_max}: Row number threshold: Maximum number of rows printed.
Set to \code{Inf} to always print all rows.  Default: 20.
\item \code{tibble.print_min}: Number of rows printed if row number threshold is
exceeded. Default: 10.
\item \code{tibble.width}: Output width. Default: \code{NULL} (use \code{width} option).
\item \code{tibble.max_extra_cols}: Number of extra columns printed in reduced form.
Default: 100.
}



\itemize{
\item \code{pillar.bold}: Use bold font, e.g. for column headers? This currently
defaults to \code{FALSE}, because many terminal fonts have poor support for
bold fonts.
\item \code{pillar.subtle}: Use subtle style, e.g. for row numbers and data types?
Default: \code{TRUE}.
\item \code{pillar.subtle_num}: Use subtle style for insignificant digits? Default:
\code{FALSE}, is also affected by the \code{pillar.subtle} option.
\item \code{pillar.neg}: Highlight negative numbers? Default: \code{TRUE}.
\item \code{pillar.sigfig}: The number of significant digits that will be printed and
highlighted, default: \code{3}. Set the \code{pillar.subtle} option to \code{FALSE} to
turn off highlighting of significant digits.
\item \code{pillar.min_title_chars}: The minimum number of characters for the column
title, default: \code{15}.  Column titles may be truncated up to that width to
save horizontal space. Set to \code{Inf} to turn off truncation of column
titles.
\item \code{pillar.min_chars}: The minimum number of characters wide to
display character columns, default: \code{0}.  Character columns may be
truncated up to that width to save horizontal space. Set to \code{Inf} to
turn off truncation of character columns.
}

}

\examples{
print(as_tibble(mtcars))
print(as_tibble(mtcars), n = 1)
print(as_tibble(mtcars), n = 3)

print(as_tibble(iris), n = 100)

print(mtcars, width = 10)

mtcars2 <- as_tibble(cbind(mtcars, mtcars), .name_repair = "unique")
print(mtcars2, n = 25, n_extra = 3)

trunc_mat(mtcars)

\dontshow{if (requireNamespace("nycflights13", quietly = TRUE)) (if (getRversion() >= "3.4") withAutoprint else force)(\{ # examplesIf}
print(nycflights13::flights, n_extra = 2)
print(nycflights13::flights, width = Inf)
\dontshow{\}) # examplesIf}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rownames.R
\name{rownames}
\alias{rownames}
\alias{has_rownames}
\alias{remove_rownames}
\alias{rownames_to_column}
\alias{rowid_to_column}
\alias{column_to_rownames}
\title{Tools for working with row names}
\usage{
has_rownames(.data)

remove_rownames(.data)

rownames_to_column(.data, var = "rowname")

rowid_to_column(.data, var = "rowid")

column_to_rownames(.data, var = "rowname")
}
\arguments{
\item{.data}{A data frame.}

\item{var}{Name of column to use for rownames.}
}
\value{
\code{column_to_rownames()} always returns a data frame.
\code{has_rownames()} returns a scalar logical.
All other functions return an object of the same class as the input.
}
\description{
While a tibble can have row names (e.g., when converting from a regular data
frame), they are removed when subsetting with the \code{[} operator.
A warning will be raised when attempting to assign non-\code{NULL} row names
to a tibble.
Generally, it is best to avoid row names, because they are basically a
character column with different semantics than every other column.

These functions allow to you detect if a data frame has row names
(\code{has_rownames()}), remove them (\code{remove_rownames()}), or convert
them back-and-forth between an explicit column (\code{rownames_to_column()}
and \code{column_to_rownames()}).
Also included is \code{rowid_to_column()}, which adds a column at the start of the
dataframe of ascending sequential row ids starting at 1. Note that this will
remove any existing row names.
}
\examples{
# Detect row names ----------------------------------------------------
has_rownames(mtcars)
has_rownames(trees)

# Remove row names ----------------------------------------------------
remove_rownames(mtcars) \%>\% has_rownames()

# Convert between row names and column --------------------------------
mtcars_tbl <- rownames_to_column(mtcars, var = "car") \%>\% as_tibble()
mtcars_tbl
column_to_rownames(mtcars_tbl, var = "car") \%>\% head()

# Adding rowid as a column --------------------------------------------
rowid_to_column(trees) \%>\% head()

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/new.R
\name{new_tibble}
\alias{new_tibble}
\alias{validate_tibble}
\title{Tibble constructor and validator}
\usage{
new_tibble(x, ..., nrow = NULL, class = NULL, subclass = NULL)

validate_tibble(x)
}
\arguments{
\item{x}{A tibble-like object.}

\item{...}{Name-value pairs of additional attributes.}

\item{nrow}{The number of rows, inferred from \code{x} if omitted.}

\item{class}{Subclasses to assign to the new object, default: none.}

\item{subclass}{Deprecated, retained for compatibility. Please use the \code{class} argument.}
}
\description{
Creates or validates a subclass of a tibble.
These function is mostly useful for package authors that implement subclasses
of a tibble, like \pkg{sf} or \pkg{tsibble}.

\code{new_tibble()} creates a new object as a subclass of \code{tbl_df}, \code{tbl} and \code{data.frame}.
This function is optimized for performance, checks are reduced to a minimum.
See \code{\link[vctrs:new_data_frame]{vctrs::new_data_frame()}} for details.

\code{validate_tibble()} checks a tibble for internal consistency.
Correct behavior can be guaranteed only if this function
runs without raising an error.
}
\section{Construction}{


For \code{new_tibble()}, \code{x} must be a list.
The \code{nrow} argument may be omitted as of tibble 3.1.4.
If present, every element of the list \code{x} should have \code{\link[vctrs:vec_size]{vctrs::vec_size()}}
equal to this value.
(But this is not checked by the constructor).
This takes the place of the "row.names" attribute in a data frame.
\code{x} must have names (or be empty),
but the names are not checked for correctness.
}

\section{Validation}{

\code{validate_tibble()} checks for "minimal" names
and that all columns are vectors, data frames or matrices.
It also makes sure that all columns have the same length,
and that \code{\link[vctrs:vec_size]{vctrs::vec_size()}} is consistent with the data.
}

\examples{
# The nrow argument can be omitted:
new_tibble(list(a = 1:3, b = 4:6))

# Existing row.names attributes are ignored:
try(validate_tibble(new_tibble(trees, nrow = 3)))

# The length of all columns must be compatible with the nrow argument:
try(validate_tibble(new_tibble(list(a = 1:3, b = 4:6), nrow = 2)))
}
\seealso{
\code{\link[=tibble]{tibble()}} and \code{\link[=as_tibble]{as_tibble()}} for ways to construct a tibble
with recycling of scalars and automatic name repair,
and \code{\link[vctrs:df_list]{vctrs::df_list()}} and \code{\link[vctrs:new_data_frame]{vctrs::new_data_frame()}}
for lower-level implementations.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/new.R
\name{new_tibble}
\alias{new_tibble}
\alias{validate_tibble}
\title{Tibble constructor and validator}
\usage{
new_tibble(x, ..., nrow = NULL, class = NULL, subclass = NULL)

validate_tibble(x)
}
\arguments{
\item{x}{A tibble-like object.}

\item{...}{Name-value pairs of additional attributes.}

\item{nrow}{The number of rows, inferred from \code{x} if omitted.}

\item{class}{Subclasses to assign to the new object, default: none.}

\item{subclass}{Deprecated, retained for compatibility. Please use the \code{class} argument.}
}
\description{
Creates or validates a subclass of a tibble.
These function is mostly useful for package authors that implement subclasses
of a tibble, like \pkg{sf} or \pkg{tsibble}.

\code{new_tibble()} creates a new object as a subclass of \code{tbl_df}, \code{tbl} and \code{data.frame}.
This function is optimized for performance, checks are reduced to a minimum.
See \code{\link[vctrs:new_data_frame]{vctrs::new_data_frame()}} for details.

\code{validate_tibble()} checks a tibble for internal consistency.
Correct behavior can be guaranteed only if this function
runs without raising an error.
}
\section{Construction}{


For \code{new_tibble()}, \code{x} must be a list.
The \code{nrow} argument may be omitted as of tibble 3.1.4.
If present, every element of the list \code{x} should have \code{\link[vctrs:vec_size]{vctrs::vec_size()}}
equal to this value.
(But this is not checked by the constructor).
This takes the place of the "row.names" attribute in a data frame.
\code{x} must have names (or be empty),
but the names are not checked for correctness.
}

\section{Validation}{

\code{validate_tibble()} checks for "minimal" names
and that all columns are vectors, data frames or matrices.
It also makes sure that all columns have the same length,
and that \code{\link[vctrs:vec_size]{vctrs::vec_size()}} is consistent with the data.
}

\examples{
# The nrow argument is always required:
new_tibble(list(a = 1:3, b = 4:6), nrow = 3)

# Existing row.names attributes are ignored:
try(validate_tibble(new_tibble(trees, nrow = 3)))

# The length of all columns must be compatible with the nrow argument:
try(validate_tibble(new_tibble(list(a = 1:3, b = 4:6), nrow = 2)))
}
\seealso{
\code{\link[=tibble]{tibble()}} and \code{\link[=as_tibble]{as_tibble()}} for ways to construct a tibble
with recycling of scalars and automatic name repair,
and \code{\link[vctrs:df_list]{vctrs::df_list()}} and \code{\link[vctrs:new_data_frame]{vctrs::new_data_frame()}}
for lower-level implementations.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/print.R
\name{formatting}
\alias{formatting}
\alias{print.tbl}
\alias{format.tbl}
\alias{print.tbl_df}
\alias{format.tbl_df}
\title{Printing tibbles}
\usage{
\method{print}{tbl_df}(
  x,
  width = NULL,
  ...,
  n = NULL,
  max_extra_cols = NULL,
  max_footer_lines = NULL
)

\method{format}{tbl_df}(
  x,
  width = NULL,
  ...,
  n = NULL,
  max_extra_cols = NULL,
  max_footer_lines = NULL
)
}
\arguments{
\item{x}{Object to format or print.}

\item{width}{Width of text output to generate. This defaults to \code{NULL}, which
means use the \code{width} \link[pillar:pillar_options]{option}.}

\item{...}{Passed on to \code{\link[=tbl_format_setup]{tbl_format_setup()}}.}

\item{n}{Number of rows to show. If \code{NULL}, the default, will print all rows
if less than the \code{print_max} \link[pillar:pillar_options]{option}.
Otherwise, will print as many rows as specified by the
\code{print_min} \link[pillar:pillar_options]{option}.}

\item{max_extra_cols}{Number of extra columns to print abbreviated information for,
if the width is too small for the entire tibble. If \code{NULL},
the \code{max_extra_cols} \link[pillar:pillar_options]{option} is used.
The previously defined \code{n_extra} argument is soft-deprecated.}

\item{max_footer_lines}{Maximum number of footer lines. If \code{NULL},
the \code{max_footer_lines} \link[pillar:pillar_options]{option} is used.}
}
\description{
One of the main features of the \code{tbl_df} class is the printing:
\itemize{
\item Tibbles only print as many rows and columns as fit on one screen,
supplemented by a summary of the remaining rows and columns.
\item Tibble reveals the type of each column, which keeps the user informed about
whether a variable is, e.g., \verb{<chr>} or \verb{<fct>} (character versus factor).
See \code{vignette("types")} for an overview of common
type abbreviations.
}

Printing can be tweaked for a one-off call by calling \code{print()} explicitly
and setting arguments like \code{n} and \code{width}. More persistent control is
available by setting the options described in \link[pillar:pillar_options]{pillar::pillar_options}.
See also \code{vignette("digits")} for a comparison to base options,
and \code{vignette("numbers")} that showcases \code{\link[=num]{num()}} and \code{\link[=char]{char()}}
for creating columns with custom formatting options.

As of tibble 3.1.0, printing is handled entirely by the \pkg{pillar} package.
If you implement a package that extends tibble,
the printed output can be customized in various ways.
See \code{vignette("extending", package = "pillar")} for details,
and \link[pillar:pillar_options]{pillar::pillar_options} for options that control the display in the console.
}
\examples{
print(as_tibble(mtcars))
print(as_tibble(mtcars), n = 1)
print(as_tibble(mtcars), n = 3)

print(as_tibble(trees), n = 100)

print(mtcars, width = 10)

mtcars2 <- as_tibble(cbind(mtcars, mtcars), .name_repair = "unique")
print(mtcars2, n = 25, max_extra_cols = 3)

\dontshow{if (requireNamespace("nycflights13", quietly = TRUE)) (if (getRversion() >= "3.4") withAutoprint else force)(\{ # examplesIf}
print(nycflights13::flights, max_footer_lines = 1)
print(nycflights13::flights, width = Inf)
\dontshow{\}) # examplesIf}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/deprecated.R
\name{deprecated}
\alias{deprecated}
\alias{data_frame}
\alias{tibble_}
\alias{data_frame_}
\alias{lst_}
\alias{as_data_frame}
\alias{as.tibble}
\alias{frame_data}
\title{Deprecated functions}
\usage{
data_frame(...)

tibble_(xs)

data_frame_(xs)

lst_(xs)

as_data_frame(x, ...)

as.tibble(x, ...)

frame_data(...)
}
\description{
\ifelse{html}{\href{https://lifecycle.r-lib.org/articles/stages.html#deprecated}{\figure{lifecycle-deprecated.svg}{options: alt='[Deprecated]'}}}{\strong{[Deprecated]}}

Use \code{\link[=tibble]{tibble()}} instead of \code{data_frame()}.

Use \link{quasiquotation} instead of \code{tibble_()}, \code{data_frame_()}, and \code{lst_()}.

Use \code{\link[=as_tibble]{as_tibble()}} instead of \code{as_data_frame()} or \code{as.tibble()}, but mind the
new signature and semantics.

Use \code{\link[=tribble]{tribble()}} instead of \code{frame_data()}.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/glimpse.R, R/has-name.R, R/pipe.R, R/tbl_sum.R
\docType{import}
\name{reexports}
\alias{reexports}
\alias{glimpse}
\alias{has_name}
\alias{\%>\%}
\alias{tbl_sum}
\alias{obj_sum}
\alias{type_sum}
\alias{size_sum}
\title{Objects exported from other packages}
\keyword{internal}
\description{
These objects are imported from other packages. Follow the links
below to see their documentation.

\describe{
  \item{magrittr}{\code{\link[magrittr:pipe]{\%>\%}}}

  \item{pillar}{\code{\link[pillar]{glimpse}}, \code{\link[pillar:type_sum]{obj_sum}}, \code{\link[pillar:type_sum]{size_sum}}, \code{\link[pillar]{tbl_sum}}, \code{\link[pillar]{type_sum}}}

  \item{rlang}{\code{\link[rlang]{has_name}}}
}}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/print.R
\name{trunc_mat}
\alias{trunc_mat}
\title{Legacy printing}
\usage{
trunc_mat(x, n = NULL, width = NULL, n_extra = NULL)
}
\arguments{
\item{x}{Object to format or print.}

\item{n}{Number of rows to show. If \code{NULL}, the default, will print all rows
if less than option \code{tibble.print_max}. Otherwise, will print
\code{tibble.print_min} rows.}

\item{width}{Width of text output to generate. This defaults to \code{NULL}, which
means use \code{getOption("tibble.width")} or (if also \code{NULL})
\code{getOption("width")}; the latter displays only the columns that fit on one
screen. You can also set \code{options(tibble.width = Inf)} to override this
default and always print all columns, this may be slow for very wide tibbles.}

\item{n_extra}{Number of extra columns to print abbreviated information for,
if the width is too small for the entire tibble. If \code{NULL}, the default,
will print information about at most \code{tibble.max_extra_cols} extra columns.}
}
\value{
An object with a \code{print()} method that will print the input
similarly to a tibble.
The internal data format is an implementation detail, do not rely on it.
}
\description{
\ifelse{html}{\href{https://lifecycle.r-lib.org/articles/stages.html#deprecated}{\figure{lifecycle-deprecated.svg}{options: alt='[Deprecated]'}}}{\strong{[Deprecated]}}
As of tibble 3.1.0, printing is handled entirely by the \pkg{pillar} package.
Do not use this function.
If you implement a package that extend tibble,
the printed output can be customized in various ways.
See \code{vignette("extending", package = "pillar")} for details.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tbl_df.R
\name{tbl_df-class}
\alias{tbl_df-class}
\alias{tbl_df}
\title{\code{tbl_df} class}
\description{
The \code{tbl_df} class is a subclass of \code{\link[base:data.frame]{data.frame}},
created in order to have different default behaviour. The colloquial term
"tibble" refers to a data frame that has the \code{tbl_df} class. Tibble is the
central data structure for the set of packages known as the
\href{https://www.tidyverse.org/packages/}{tidyverse}, including
\href{https://dplyr.tidyverse.org/}{dplyr},
\href{https://ggplot2.tidyverse.org/}{ggplot2},
\href{https://tidyr.tidyverse.org/}{tidyr}, and
\href{https://readr.tidyverse.org/}{readr}.

The general ethos is that tibbles are lazy and surly: they do less and
complain more than base \link{data.frame}s. This forces
problems to be tackled earlier and more explicitly, typically leading to code
that is more expressive and robust.
}
\section{Properties of \code{tbl_df}}{


Objects of class \code{tbl_df} have:
\itemize{
\item A \code{class} attribute of \code{c("tbl_df", "tbl", "data.frame")}.
\item A base type of \code{"list"}, where each element of the list has the same
\code{\link[vctrs:vec_size]{vctrs::vec_size()}}.
\item A \code{names} attribute that is a character vector the same length as the
underlying list.
\item A \code{row.names} attribute, included for compatibility with \link{data.frame}.
This attribute is only consulted to query the number of rows,
any row names that might be stored there are ignored
by most tibble methods.
}
}

\section{Behavior of \code{tbl_df}}{


How default behaviour of tibbles differs from that of
\link{data.frame}s, during creation and access:
\itemize{
\item Column data is not coerced. A character vector is not turned into a factor.
List-columns are expressly anticipated and do not require special tricks.
Internal names are never stripped from column data.
Read more in \code{\link[=tibble]{tibble()}}.
\item Recycling only happens for a length 1 input.
Read more in \code{\link[vctrs:vec_recycle]{vctrs::vec_recycle()}}.
\item Column names are not munged, although missing names are auto-populated.
Empty and duplicated column names are strongly discouraged, but the user
must indicate how to resolve. Read more in \code{\link[vctrs:vec_as_names]{vctrs::vec_as_names()}}.
\item Row names are not added and are strongly discouraged, in favor of storing
that info as a column. Read about in \link{rownames}.
\item \code{df[, j]} returns a tibble; it does not automatically extract the column
inside. \code{df[, j, drop = FALSE]} is the default. Read more in \link{subsetting}.
\item There is no partial matching when \code{$} is used to index by name. \code{df$name}
for a nonexistent name generates a warning. Read more in \link{subsetting}.
}

See \code{vignette("invariants")} for a detailed description of the behavior.

Furthermore, printing and inspection are a very high priority.
The goal is to convey as much information as possible, in a concise way,
even for large and complex tibbles. Read more in \link{formatting}.
}

\seealso{
\code{\link[=tibble]{tibble()}}, \code{\link[=as_tibble]{as_tibble()}}, \code{\link[=tribble]{tribble()}}, \code{\link[=print.tbl]{print.tbl()}},
\code{\link[=glimpse]{glimpse()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tibble.R
\name{is.tibble}
\alias{is.tibble}
\title{Deprecated test for tibble-ness}
\usage{
is.tibble(x)
}
\arguments{
\item{x}{An object}
}
\description{
\ifelse{html}{\href{https://lifecycle.r-lib.org/articles/stages.html#soft-deprecated}{\figure{lifecycle-soft-deprecated.svg}{options: alt='[Soft-deprecated]'}}}{\strong{[Soft-deprecated]}}

Please use \code{\link[=is_tibble]{is_tibble()}} instead.
}
\keyword{internal}
